import os.path
import sys

sys.path.append(os.path.dirname(__file__))  # let code under test load stubs
from device_test import DeviceTestCase

class TestAdtranModels(DeviceTestCase):
    def test_adtran_sdx6000_SDX_6320_16_sdx6320_RL_23_5_33_10_59_5_254(self):
        if DeviceTestCase.TestDisabled:
                DeviceTestCase.skipTest(self, "Python 3.7 doesn't support this library")
        obj = self.run_data("adtran.sdx6000.SDX 6320-16.sdx6320-RL-23.5-33.10_59_5_254")
        exp_info = {
                'model': 'SDX 6320-16',
                'sta_count': 338,
                'sw_version': 'sdx6320-RL-23.5-33',
                'vendor': 'Adtran'
        }
        exp_stats = {
         
        }
        exp_sta_stats = {
                'ADTN23123E9C': {       'dev_macs': ['38:f8:f6:52:b1:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU2',
                                        'serial': 'ADTN23123E9C'},
                'ADTN23123E9D': {       'dev_macs': ['cc:66:18:dd:8f:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU16',
                                        'serial': 'ADTN23123E9D'},
                'ADTN23123E9E': {       'dev_macs': ['cc:66:18:50:b7:70'],
                                        'ip_address': None,
                                        'name': 'ADTN23123E9E',
                                        'serial': 'ADTN23123E9E'},
                'ADTN23123EA1': {       'dev_macs': ['cc:66:18:db:77:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU17',
                                        'serial': 'ADTN23123EA1'},
                'ADTN23123EA8': {       'dev_macs': ['cc:66:18:dd:88:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU29',
                                        'serial': 'ADTN23123EA8'},
                'ADTN23123EB7': {       'dev_macs': ['38:f8:f6:52:ac:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU10',
                                        'serial': 'ADTN23123EB7'},
                'ADTN23123EBC': {       'dev_macs': ['cc:66:18:dc:77:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU13',
                                        'serial': 'ADTN23123EBC'},
                'ADTN23123EC0': {       'dev_macs': ['cc:66:18:db:65:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU11',
                                        'serial': 'ADTN23123EC0'},
                'ADTN23123F18': {       'dev_macs': ['cc:66:18:db:7d:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU4',
                                        'serial': 'ADTN23123F18'},
                'ADTN23123F2B': {       'dev_macs': ['cc:66:18:50:5c:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU11',
                                        'serial': 'ADTN23123F2B'},
                'ADTN23123F57': {       'dev_macs': ['cc:66:18:dc:68:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU2',
                                        'serial': 'ADTN23123F57'},
                'ADTN23123FAD': {       'dev_macs': ['cc:66:18:dc:76:60'],
                                        'ip_address': None,
                                        'name': 'ADTN23123FAD',
                                        'serial': 'ADTN23123FAD'},
                'ADTN23123FC4': {       'dev_macs': ['cc:66:18:dd:89:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU22',
                                        'serial': 'ADTN23123FC4'},
                'ADTN23123FDC': {       'dev_macs': ['cc:66:18:d6:37:81'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU23',
                                        'serial': 'ADTN23123FDC'},
                'ADTN23123FE9': {       'dev_macs': ['cc:66:18:50:b1:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU13',
                                        'serial': 'ADTN23123FE9'},
                'ADTN23123FEC': {       'dev_macs': ['cc:66:18:4c:03:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU28',
                                        'serial': 'ADTN23123FEC'},
                'ADTN23123FEE': {       'dev_macs': ['cc:66:18:db:65:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU12',
                                        'serial': 'ADTN23123FEE'},
                'ADTN23123FF5': {       'dev_macs': ['cc:66:18:d6:0c:10'],
                                        'ip_address': None,
                                        'name': 'ADTN23123FF5',
                                        'serial': 'ADTN23123FF5'},
                'ADTN23124008': {       'dev_macs': ['cc:66:18:d6:3b:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU33',
                                        'serial': 'ADTN23124008'},
                'ADTN2312408C': {       'dev_macs': ['38:f8:f6:52:ad:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU17',
                                        'serial': 'ADTN2312408C'},
                'ADTN23124097': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU17',
                                        'serial': 'ADTN23124097'},
                'ADTN23124099': {       'dev_macs': ['cc:66:18:51:50:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU19',
                                        'serial': 'ADTN23124099'},
                'ADTN231240A6': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU13',
                                        'serial': 'ADTN231240A6'},
                'ADTN231240B0': {       'dev_macs': ['cc:66:18:dc:5d:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU1',
                                        'serial': 'ADTN231240B0'},
                'ADTN23124128': {       'dev_macs': ['cc:66:18:dc:82:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU8',
                                        'serial': 'ADTN23124128'},
                'ADTN231241A2': {       'dev_macs': ['cc:66:18:50:ae:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU21',
                                        'serial': 'ADTN231241A2'},
                'ADTN231241BF': {       'dev_macs': ['cc:66:18:50:50:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU22',
                                        'serial': 'ADTN231241BF'},
                'ADTN231241C0': {       'dev_macs': ['cc:66:18:dd:8f:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU10',
                                        'serial': 'ADTN231241C0'},
                'ADTN231241D0': {       'dev_macs': ['cc:66:18:dd:7c:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU8',
                                        'serial': 'ADTN231241D0'},
                'ADTN231241D1': {       'dev_macs': ['38:f8:f6:52:ad:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU3',
                                        'serial': 'ADTN231241D1'},
                'ADTN23124214': {       'dev_macs': ['cc:66:18:50:f1:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU20',
                                        'serial': 'ADTN23124214'},
                'ADTN23124304': {       'dev_macs': ['cc:66:18:d6:0b:60'],
                                        'ip_address': None,
                                        'name': 'ADTN23124304',
                                        'serial': 'ADTN23124304'},
                'ADTN2312432E': {       'dev_macs': ['cc:66:18:d6:0a:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU0',
                                        'serial': 'ADTN2312432E'},
                'ADTN2312445C': {       'dev_macs': ['cc:66:18:50:d8:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU7',
                                        'serial': 'ADTN2312445C'},
                'ADTN23124491': {       'dev_macs': ['cc:66:18:d5:fd:61'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU19',
                                        'serial': 'ADTN23124491'},
                'ADTN231244BA': {       'dev_macs': ['38:f8:f6:52:aa:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU0',
                                        'serial': 'ADTN231244BA'},
                'ADTN231244BC': {       'dev_macs': ['38:f8:f6:52:a7:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU8',
                                        'serial': 'ADTN231244BC'},
                'ADTN231244EB': {       'dev_macs': ['cc:66:18:d6:3a:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU17',
                                        'serial': 'ADTN231244EB'},
                'ADTN23124510': {       'dev_macs': ['cc:66:18:50:3d:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU26',
                                        'serial': 'ADTN23124510'},
                'ADTN231245C6': {       'dev_macs': ['cc:66:18:4b:fa:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU6',
                                        'serial': 'ADTN231245C6'},
                'ADTN2312460A': {       'dev_macs': ['38:f8:f6:52:a3:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU2',
                                        'serial': 'ADTN2312460A'},
                'ADTN23124650': {       'dev_macs': ['cc:66:18:dc:81:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU3',
                                        'serial': 'ADTN23124650'},
                'ADTN23124713': {       'dev_macs': ['cc:66:18:dc:88:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU20',
                                        'serial': 'ADTN23124713'},
                'ADTN231247A1': {       'dev_macs': ['cc:66:18:50:51:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU26',
                                        'serial': 'ADTN231247A1'},
                'ADTN23124849': {       'dev_macs': ['cc:66:18:d6:07:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU1',
                                        'serial': 'ADTN23124849'},
                'ADTN231248A8': {       'dev_macs': ['cc:66:18:50:6c:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU9',
                                        'serial': 'ADTN231248A8'},
                'ADTN23124900': {       'dev_macs': ['cc:66:18:dc:72:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU7',
                                        'serial': 'ADTN23124900'},
                'ADTN2312491E': {       'dev_macs': ['cc:66:18:dd:88:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU6',
                                        'serial': 'ADTN2312491E'},
                'ADTN23124966': {       'dev_macs': ['cc:66:18:50:37:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU15',
                                        'serial': 'ADTN23124966'},
                'ADTN23124A18': {       'dev_macs': ['cc:66:18:d6:38:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU21',
                                        'serial': 'ADTN23124A18'},
                'ADTN23124A23': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU3',
                                        'serial': 'ADTN23124A23'},
                'ADTN23124A76': {       'dev_macs': [       '36:3a:c5:c7:0e:34',
                                                            'cc:66:18:50:b4:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU19',
                                        'serial': 'ADTN23124A76'},
                'ADTN23124B3F': {       'dev_macs': ['cc:66:18:50:43:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU22',
                                        'serial': 'ADTN23124B3F'},
                'ADTN23124B7E': {       'dev_macs': ['cc:66:18:50:ef:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU14',
                                        'serial': 'ADTN23124B7E'},
                'ADTN23124BA5': {       'dev_macs': ['cc:66:18:d6:38:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU23',
                                        'serial': 'ADTN23124BA5'},
                'ADTN23124C55': {       'dev_macs': ['cc:66:18:50:81:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU27',
                                        'serial': 'ADTN23124C55'},
                'ADTN23124C6A': {       'dev_macs': ['cc:66:18:51:26:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU27',
                                        'serial': 'ADTN23124C6A'},
                'ADTN23124C73': {       'dev_macs': ['cc:66:18:d6:3c:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU18',
                                        'serial': 'ADTN23124C73'},
                'ADTN23124D38': {       'dev_macs': ['cc:66:18:dc:77:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU6',
                                        'serial': 'ADTN23124D38'},
                'ADTN23124D41': {       'dev_macs': ['cc:66:18:dc:71:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU0',
                                        'serial': 'ADTN23124D41'},
                'ADTN23124D43': {       'dev_macs': ['cc:66:18:dc:56:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU1',
                                        'serial': 'ADTN23124D43'},
                'ADTN23124D46': {       'dev_macs': ['38:f8:f6:52:b4:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU5',
                                        'serial': 'ADTN23124D46'},
                'ADTN23124D50': {       'dev_macs': ['cc:66:18:dc:78:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU5',
                                        'serial': 'ADTN23124D50'},
                'ADTN23124D68': {       'dev_macs': ['cc:66:18:dc:88:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU0',
                                        'serial': 'ADTN23124D68'},
                'ADTN23124D7D': {       'dev_macs': ['38:f8:f6:52:7a:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU7',
                                        'serial': 'ADTN23124D7D'},
                'ADTN23124D7E': {       'dev_macs': ['cc:66:18:dc:74:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU5',
                                        'serial': 'ADTN23124D7E'},
                'ADTN23124D91': {       'dev_macs': ['38:f8:f6:52:a9:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU0',
                                        'serial': 'ADTN23124D91'},
                'ADTN23124D93': {       'dev_macs': ['cc:66:18:dc:87:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU4',
                                        'serial': 'ADTN23124D93'},
                'ADTN23124D98': {       'dev_macs': ['cc:66:18:dd:92:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU4',
                                        'serial': 'ADTN23124D98'},
                'ADTN23124D9E': {       'dev_macs': ['cc:66:18:db:66:60'],
                                        'ip_address': None,
                                        'name': 'ADTN23124D9E',
                                        'serial': 'ADTN23124D9E'},
                'ADTN23124DA0': {       'dev_macs': ['cc:66:18:50:48:80'],
                                        'ip_address': None,
                                        'name': 'ADTN23124DA0',
                                        'serial': 'ADTN23124DA0'},
                'ADTN23124DA6': {       'dev_macs': ['cc:66:18:dc:77:f0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124DA6',
                                        'serial': 'ADTN23124DA6'},
                'ADTN23124DB0': {       'dev_macs': ['cc:66:18:dd:7e:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU7',
                                        'serial': 'ADTN23124DB0'},
                'ADTN23124DB2': {       'dev_macs': ['cc:66:18:dc:80:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU6',
                                        'serial': 'ADTN23124DB2'},
                'ADTN23124DB7': {       'dev_macs': ['cc:66:18:4b:fa:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU8',
                                        'serial': 'ADTN23124DB7'},
                'ADTN23124DBC': {       'dev_macs': ['38:f8:f6:52:ac:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU9',
                                        'serial': 'ADTN23124DBC'},
                'ADTN23124DC7': {       'dev_macs': ['cc:66:18:dd:75:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU1',
                                        'serial': 'ADTN23124DC7'},
                'ADTN23124DCE': {       'dev_macs': ['cc:66:18:dd:2a:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU11',
                                        'serial': 'ADTN23124DCE'},
                'ADTN23124DD9': {       'dev_macs': ['cc:66:18:dd:7e:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU2',
                                        'serial': 'ADTN23124DD9'},
                'ADTN23124DDA': {       'dev_macs': ['cc:66:18:dc:6e:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU1',
                                        'serial': 'ADTN23124DDA'},
                'ADTN23124DDD': {       'dev_macs': ['cc:66:18:dc:57:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU5',
                                        'serial': 'ADTN23124DDD'},
                'ADTN23124DDF': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU16',
                                        'serial': 'ADTN23124DDF'},
                'ADTN23124DE0': {       'dev_macs': ['cc:66:18:dd:8e:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU2',
                                        'serial': 'ADTN23124DE0'},
                'ADTN23124DE4': {       'dev_macs': ['cc:66:18:dc:76:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU6',
                                        'serial': 'ADTN23124DE4'},
                'ADTN23124DE8': {       'dev_macs': ['cc:66:18:50:b5:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU12',
                                        'serial': 'ADTN23124DE8'},
                'ADTN23124DEB': {       'dev_macs': ['38:f8:f6:52:9c:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU3',
                                        'serial': 'ADTN23124DEB'},
                'ADTN23124DF1': {       'dev_macs': ['cc:66:18:dc:7f:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU0',
                                        'serial': 'ADTN23124DF1'},
                'ADTN23124DF3': {       'dev_macs': ['cc:66:18:dc:7d:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU9',
                                        'serial': 'ADTN23124DF3'},
                'ADTN23124DF5': {       'dev_macs': ['cc:66:18:dc:6e:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU19',
                                        'serial': 'ADTN23124DF5'},
                'ADTN23124DF6': {       'dev_macs': ['38:f8:f6:52:ac:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU3',
                                        'serial': 'ADTN23124DF6'},
                'ADTN23124DF7': {       'dev_macs': ['cc:66:18:dd:7f:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU14',
                                        'serial': 'ADTN23124DF7'},
                'ADTN23124DFB': {       'dev_macs': ['cc:66:18:dd:73:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU0',
                                        'serial': 'ADTN23124DFB'},
                'ADTN23124DFC': {       'dev_macs': ['cc:66:18:dc:77:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU5',
                                        'serial': 'ADTN23124DFC'},
                'ADTN23124E04': {       'dev_macs': ['cc:66:18:4c:01:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU4',
                                        'serial': 'ADTN23124E04'},
                'ADTN23124E07': {       'dev_macs': ['cc:66:18:dc:56:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU0',
                                        'serial': 'ADTN23124E07'},
                'ADTN23124E0D': {       'dev_macs': ['cc:66:18:dc:75:a0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124E0D',
                                        'serial': 'ADTN23124E0D'},
                'ADTN23124E14': {       'dev_macs': ['cc:66:18:dc:77:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU1',
                                        'serial': 'ADTN23124E14'},
                'ADTN23124E16': {       'dev_macs': ['cc:66:18:dc:73:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU9',
                                        'serial': 'ADTN23124E16'},
                'ADTN23124E17': {       'dev_macs': ['cc:66:18:dc:7d:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU15',
                                        'serial': 'ADTN23124E17'},
                'ADTN23124E18': {       'dev_macs': ['cc:66:18:dc:88:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU1',
                                        'serial': 'ADTN23124E18'},
                'ADTN23124E19': {       'dev_macs': ['cc:66:18:dd:7b:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU21',
                                        'serial': 'ADTN23124E19'},
                'ADTN23124E1F': {       'dev_macs': ['cc:66:18:dd:8b:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU30',
                                        'serial': 'ADTN23124E1F'},
                'ADTN23124E22': {       'dev_macs': ['cc:66:18:dd:8d:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU12',
                                        'serial': 'ADTN23124E22'},
                'ADTN23124E23': {       'dev_macs': ['cc:66:18:4c:03:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU24',
                                        'serial': 'ADTN23124E23'},
                'ADTN23124E25': {       'dev_macs': ['38:f8:f6:52:af:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU13',
                                        'serial': 'ADTN23124E25'},
                'ADTN23124E26': {       'dev_macs': ['cc:66:18:dd:93:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU3',
                                        'serial': 'ADTN23124E26'},
                'ADTN23124E27': {       'dev_macs': ['cc:66:18:dc:70:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU25',
                                        'serial': 'ADTN23124E27'},
                'ADTN23124E29': {       'dev_macs': ['cc:66:18:dd:68:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU7',
                                        'serial': 'ADTN23124E29'},
                'ADTN23124E2A': {       'dev_macs': ['cc:66:18:d6:08:e0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124E2A',
                                        'serial': 'ADTN23124E2A'},
                'ADTN23124E2B': {       'dev_macs': ['38:f8:f6:52:b2:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU10',
                                        'serial': 'ADTN23124E2B'},
                'ADTN23124E2D': {       'dev_macs': ['cc:66:18:dd:8a:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU15',
                                        'serial': 'ADTN23124E2D'},
                'ADTN23124E30': {       'dev_macs': ['cc:66:18:dc:79:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU5',
                                        'serial': 'ADTN23124E30'},
                'ADTN23124E33': {       'dev_macs': ['cc:66:18:d6:08:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU17',
                                        'serial': 'ADTN23124E33'},
                'ADTN23124E35': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU3',
                                        'serial': 'ADTN23124E35'},
                'ADTN23124E37': {       'dev_macs': ['cc:66:18:dc:6e:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU11',
                                        'serial': 'ADTN23124E37'},
                'ADTN23124E3A': {       'dev_macs': ['cc:66:18:dd:8a:b1'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU7',
                                        'serial': 'ADTN23124E3A'},
                'ADTN23124E3E': {       'dev_macs': ['cc:66:18:dc:70:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU4',
                                        'serial': 'ADTN23124E3E'},
                'ADTN23124E3F': {       'dev_macs': ['cc:66:18:dd:83:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU23',
                                        'serial': 'ADTN23124E3F'},
                'ADTN23124E40': {       'dev_macs': ['cc:66:18:dd:8a:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU0',
                                        'serial': 'ADTN23124E40'},
                'ADTN23124E43': {       'dev_macs': ['cc:66:18:d6:03:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU18',
                                        'serial': 'ADTN23124E43'},
                'ADTN23124E45': {       'dev_macs': ['cc:66:18:dc:83:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU0',
                                        'serial': 'ADTN23124E45'},
                'ADTN23124E4B': {       'dev_macs': ['cc:66:18:dd:8f:b0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124E4B',
                                        'serial': 'ADTN23124E4B'},
                'ADTN23124E4D': {       'dev_macs': ['cc:66:18:d6:08:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU4',
                                        'serial': 'ADTN23124E4D'},
                'ADTN23124E50': {       'dev_macs': ['cc:66:18:d6:00:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU11',
                                        'serial': 'ADTN23124E50'},
                'ADTN23124E52': {       'dev_macs': ['cc:66:18:dd:82:c0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124E52',
                                        'serial': 'ADTN23124E52'},
                'ADTN23124E57': {       'dev_macs': ['38:f8:f6:52:ab:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU9',
                                        'serial': 'ADTN23124E57'},
                'ADTN23124E5B': {       'dev_macs': ['cc:66:18:4c:02:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU17',
                                        'serial': 'ADTN23124E5B'},
                'ADTN23124E64': {       'dev_macs': ['cc:66:18:4b:fa:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU26',
                                        'serial': 'ADTN23124E64'},
                'ADTN23124E67': {       'dev_macs': ['cc:66:18:dd:82:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU22',
                                        'serial': 'ADTN23124E67'},
                'ADTN23124E6A': {       'dev_macs': ['cc:66:18:d6:3a:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU5',
                                        'serial': 'ADTN23124E6A'},
                'ADTN23124E6B': {       'dev_macs': ['cc:66:18:dc:87:c0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124E6B',
                                        'serial': 'ADTN23124E6B'},
                'ADTN23124E71': {       'dev_macs': ['cc:66:18:dd:84:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU0',
                                        'serial': 'ADTN23124E71'},
                'ADTN23124E73': {       'dev_macs': ['cc:66:18:50:52:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU13',
                                        'serial': 'ADTN23124E73'},
                'ADTN23124E7A': {       'dev_macs': ['cc:66:18:dd:7e:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU1',
                                        'serial': 'ADTN23124E7A'},
                'ADTN23124E84': {       'dev_macs': ['cc:66:18:dc:71:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU5',
                                        'serial': 'ADTN23124E84'},
                'ADTN23124E86': {       'dev_macs': ['cc:66:18:dc:6d:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU8',
                                        'serial': 'ADTN23124E86'},
                'ADTN23124E8A': {       'dev_macs': ['cc:66:18:51:30:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU14',
                                        'serial': 'ADTN23124E8A'},
                'ADTN23124E92': {       'dev_macs': ['38:f8:f6:52:a6:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU6',
                                        'serial': 'ADTN23124E92'},
                'ADTN23124EA4': {       'dev_macs': ['cc:66:18:4c:8d:d0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124EA4',
                                        'serial': 'ADTN23124EA4'},
                'ADTN23124EA5': {       'dev_macs': ['cc:66:18:dc:8a:10'],
                                        'ip_address': None,
                                        'name': 'ADTN23124EA5',
                                        'serial': 'ADTN23124EA5'},
                'ADTN23124ED3': {       'dev_macs': ['cc:66:18:d6:32:11'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU7',
                                        'serial': 'ADTN23124ED3'},
                'ADTN23124ED9': {       'dev_macs': ['cc:66:18:dd:7c:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU3',
                                        'serial': 'ADTN23124ED9'},
                'ADTN23124EDF': {       'dev_macs': ['cc:66:18:dc:81:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU3',
                                        'serial': 'ADTN23124EDF'},
                'ADTN23124EFA': {       'dev_macs': ['cc:66:18:db:76:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU2',
                                        'serial': 'ADTN23124EFA'},
                'ADTN23124F06': {       'dev_macs': ['38:f8:f6:52:ab:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU10',
                                        'serial': 'ADTN23124F06'},
                'ADTN23124F14': {       'dev_macs': ['cc:66:18:dc:6b:c0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124F14',
                                        'serial': 'ADTN23124F14'},
                'ADTN23124F2B': {       'dev_macs': ['cc:66:18:dc:81:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU10',
                                        'serial': 'ADTN23124F2B'},
                'ADTN23124F2D': {       'dev_macs': ['38:f8:f6:52:af:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU0',
                                        'serial': 'ADTN23124F2D'},
                'ADTN23124F30': {       'dev_macs': ['cc:66:18:dc:75:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU6',
                                        'serial': 'ADTN23124F30'},
                'ADTN23124F34': {       'dev_macs': ['38:f8:f6:52:ac:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU8',
                                        'serial': 'ADTN23124F34'},
                'ADTN23124F37': {       'dev_macs': ['cc:66:18:dd:89:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU5',
                                        'serial': 'ADTN23124F37'},
                'ADTN23124F3C': {       'dev_macs': ['cc:66:18:dc:78:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU4',
                                        'serial': 'ADTN23124F3C'},
                'ADTN23124F74': {       'dev_macs': ['cc:66:18:dd:83:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU16',
                                        'serial': 'ADTN23124F74'},
                'ADTN23124F99': {       'dev_macs': ['cc:66:18:dc:75:51'],
                                        'ip_address': None,
                                        'name': 'ADTN23124F99',
                                        'serial': 'ADTN23124F99'},
                'ADTN2312501D': {       'dev_macs': ['cc:66:18:dc:73:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU18',
                                        'serial': 'ADTN2312501D'},
                'ADTN23125020': {       'dev_macs': ['cc:66:18:dd:7d:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU4',
                                        'serial': 'ADTN23125020'},
                'ADTN23125026': {       'dev_macs': ['cc:66:18:dd:79:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU1',
                                        'serial': 'ADTN23125026'},
                'ADTN23125037': {       'dev_macs': ['38:f8:f6:52:ac:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU13',
                                        'serial': 'ADTN23125037'},
                'ADTN23125055': {       'dev_macs': ['cc:66:18:dc:81:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU2',
                                        'serial': 'ADTN23125055'},
                'ADTN23125061': {       'dev_macs': ['cc:66:18:4c:01:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU0',
                                        'serial': 'ADTN23125061'},
                'ADTN23125065': {       'dev_macs': ['cc:66:18:50:4f:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU15',
                                        'serial': 'ADTN23125065'},
                'ADTN2312506C': {       'dev_macs': ['cc:66:18:51:42:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU19',
                                        'serial': 'ADTN2312506C'},
                'ADTN23125088': {       'dev_macs': ['cc:66:18:dd:8b:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU3',
                                        'serial': 'ADTN23125088'},
                'ADTN2312509E': {       'dev_macs': ['cc:66:18:d6:08:f0'],
                                        'ip_address': None,
                                        'name': 'ADTN2312509E',
                                        'serial': 'ADTN2312509E'},
                'ADTN231250BB': {       'dev_macs': ['cc:66:18:db:65:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU13',
                                        'serial': 'ADTN231250BB'},
                'ADTN231250BF': {       'dev_macs': ['cc:66:18:d6:08:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU10',
                                        'serial': 'ADTN231250BF'},
                'ADTN231250C0': {       'dev_macs': ['cc:66:18:d6:09:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU9',
                                        'serial': 'ADTN231250C0'},
                'ADTN231250C3': {       'dev_macs': ['cc:66:18:dc:77:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU23',
                                        'serial': 'ADTN231250C3'},
                'ADTN231250C6': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU2',
                                        'serial': 'ADTN231250C6'},
                'ADTN231250C8': {       'dev_macs': ['cc:66:18:51:50:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU16',
                                        'serial': 'ADTN231250C8'},
                'ADTN231250CC': {       'dev_macs': ['cc:66:18:dd:7b:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU6',
                                        'serial': 'ADTN231250CC'},
                'ADTN231250CD': {       'dev_macs': ['cc:66:18:50:63:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU11',
                                        'serial': 'ADTN231250CD'},
                'ADTN231250D1': {       'dev_macs': ['cc:66:18:50:f5:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU14',
                                        'serial': 'ADTN231250D1'},
                'ADTN231250DD': {       'dev_macs': ['cc:66:18:4c:02:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU12',
                                        'serial': 'ADTN231250DD'},
                'ADTN231250FE': {       'dev_macs': ['cc:66:18:d6:02:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU8',
                                        'serial': 'ADTN231250FE'},
                'ADTN23125103': {       'dev_macs': ['cc:66:18:d6:04:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU9',
                                        'serial': 'ADTN23125103'},
                'ADTN2312510B': {       'dev_macs': ['cc:66:18:dc:88:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU20',
                                        'serial': 'ADTN2312510B'},
                'ADTN2312510C': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU11',
                                        'serial': 'ADTN2312510C'},
                'ADTN23125111': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU7',
                                        'serial': 'ADTN23125111'},
                'ADTN23125116': {       'dev_macs': ['cc:66:18:dc:79:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU14',
                                        'serial': 'ADTN23125116'},
                'ADTN2312511D': {       'dev_macs': ['cc:66:18:d6:0b:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU12',
                                        'serial': 'ADTN2312511D'},
                'ADTN23125126': {       'dev_macs': ['cc:66:18:dc:6a:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU18',
                                        'serial': 'ADTN23125126'},
                'ADTN23125129': {       'dev_macs': ['cc:66:18:d6:09:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU16',
                                        'serial': 'ADTN23125129'},
                'ADTN2312512C': {       'dev_macs': ['cc:66:18:dc:6c:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU16',
                                        'serial': 'ADTN2312512C'},
                'ADTN2312512E': {       'dev_macs': ['cc:66:18:dd:8e:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU10',
                                        'serial': 'ADTN2312512E'},
                'ADTN2312512F': {       'dev_macs': ['cc:66:18:4b:fa:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU15',
                                        'serial': 'ADTN2312512F'},
                'ADTN23125133': {       'dev_macs': ['cc:66:18:db:76:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU18',
                                        'serial': 'ADTN23125133'},
                'ADTN23125135': {       'dev_macs': ['cc:66:18:50:3e:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU26',
                                        'serial': 'ADTN23125135'},
                'ADTN2312513A': {       'dev_macs': ['cc:66:18:50:50:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU9',
                                        'serial': 'ADTN2312513A'},
                'ADTN23125143': {       'dev_macs': ['cc:66:18:d6:07:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU8',
                                        'serial': 'ADTN23125143'},
                'ADTN23125146': {       'dev_macs': ['cc:66:18:dd:91:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU7',
                                        'serial': 'ADTN23125146'},
                'ADTN23125148': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU12',
                                        'serial': 'ADTN23125148'},
                'ADTN23125149': {       'dev_macs': ['cc:66:18:d6:36:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU1',
                                        'serial': 'ADTN23125149'},
                'ADTN2312514A': {       'dev_macs': ['cc:66:18:dc:83:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU18',
                                        'serial': 'ADTN2312514A'},
                'ADTN2312514B': {       'dev_macs': ['cc:66:18:d6:35:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU12',
                                        'serial': 'ADTN2312514B'},
                'ADTN2312514C': {       'dev_macs': ['cc:66:18:50:87:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU21',
                                        'serial': 'ADTN2312514C'},
                'ADTN2312514D': {       'dev_macs': ['38:f8:f6:52:b3:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU9',
                                        'serial': 'ADTN2312514D'},
                'ADTN2312514E': {       'dev_macs': ['cc:66:18:dd:8a:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU11',
                                        'serial': 'ADTN2312514E'},
                'ADTN2312514F': {       'dev_macs': ['cc:66:18:51:24:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU21',
                                        'serial': 'ADTN2312514F'},
                'ADTN23125150': {       'dev_macs': ['cc:66:18:d6:36:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x14-ONU0',
                                        'serial': 'ADTN23125150'},
                'ADTN23125153': {       'dev_macs': ['38:f8:f6:52:a8:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU19',
                                        'serial': 'ADTN23125153'},
                'ADTN23125158': {       'dev_macs': ['cc:66:18:dd:92:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU11',
                                        'serial': 'ADTN23125158'},
                'ADTN2312515B': {       'dev_macs': ['cc:66:18:dc:7f:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU12',
                                        'serial': 'ADTN2312515B'},
                'ADTN23125160': {       'dev_macs': ['cc:66:18:dd:69:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU4',
                                        'serial': 'ADTN23125160'},
                'ADTN23125161': {       'dev_macs': ['cc:66:18:dc:65:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU6',
                                        'serial': 'ADTN23125161'},
                'ADTN23125166': {       'dev_macs': ['cc:66:18:dd:78:e0'],
                                        'ip_address': None,
                                        'name': 'ADTN23125166',
                                        'serial': 'ADTN23125166'},
                'ADTN23125171': {       'dev_macs': ['cc:66:18:4c:04:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU2',
                                        'serial': 'ADTN23125171'},
                'ADTN2312517D': {       'dev_macs': ['cc:66:18:dd:78:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU11',
                                        'serial': 'ADTN2312517D'},
                'ADTN2312517F': {       'dev_macs': ['cc:66:18:dd:23:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU11',
                                        'serial': 'ADTN2312517F'},
                'ADTN2312518D': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU23',
                                        'serial': 'ADTN2312518D'},
                'ADTN2312518E': {       'dev_macs': ['cc:66:18:db:74:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU10',
                                        'serial': 'ADTN2312518E'},
                'ADTN23125190': {       'dev_macs': ['cc:66:18:50:75:70'],
                                        'ip_address': None,
                                        'name': 'ADTN23125190',
                                        'serial': 'ADTN23125190'},
                'ADTN23125194': {       'dev_macs': ['cc:66:18:50:81:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU10',
                                        'serial': 'ADTN23125194'},
                'ADTN23125195': {       'dev_macs': ['cc:66:18:50:b8:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU5',
                                        'serial': 'ADTN23125195'},
                'ADTN231251AA': {       'dev_macs': ['cc:66:18:d6:37:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU20',
                                        'serial': 'ADTN231251AA'},
                'ADTN231251AB': {       'dev_macs': ['cc:66:18:4c:a3:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU20',
                                        'serial': 'ADTN231251AB'},
                'ADTN231251B0': {       'dev_macs': ['cc:66:18:dd:8d:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU15',
                                        'serial': 'ADTN231251B0'},
                'ADTN231251B1': {       'dev_macs': ['cc:66:18:dd:6a:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU13',
                                        'serial': 'ADTN231251B1'},
                'ADTN231251E2': {       'dev_macs': ['38:f8:f6:52:71:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU2',
                                        'serial': 'ADTN231251E2'},
                'ADTN231251EB': {       'dev_macs': ['cc:66:18:dc:56:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU5',
                                        'serial': 'ADTN231251EB'},
                'ADTN231251EE': {       'dev_macs': ['cc:66:18:dd:93:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU13',
                                        'serial': 'ADTN231251EE'},
                'ADTN231251F6': {       'dev_macs': ['cc:66:18:d6:08:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU6',
                                        'serial': 'ADTN231251F6'},
                'ADTN231251F7': {       'dev_macs': ['cc:66:18:dc:79:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU4',
                                        'serial': 'ADTN231251F7'},
                'ADTN23125206': {       'dev_macs': ['cc:66:18:50:6e:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU8',
                                        'serial': 'ADTN23125206'},
                'ADTN2314428A': {       'dev_macs': ['cc:66:18:dc:97:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU20',
                                        'serial': 'ADTN2314428A'},
                'ADTN23144666': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU25',
                                        'serial': 'ADTN23144666'},
                'ADTN23144681': {       'dev_macs': ['cc:66:18:d6:3b:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU5',
                                        'serial': 'ADTN23144681'},
                'ADTN2314468C': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU17',
                                        'serial': 'ADTN2314468C'},
                'ADTN23144699': {       'dev_macs': ['cc:66:18:dc:78:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU18',
                                        'serial': 'ADTN23144699'},
                'ADTN2314469E': {       'dev_macs': ['cc:66:18:dd:87:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU24',
                                        'serial': 'ADTN2314469E'},
                'ADTN231446A3': {       'dev_macs': ['38:f8:f6:52:a9:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU13',
                                        'serial': 'ADTN231446A3'},
                'ADTN231446A5': {       'dev_macs': ['cc:66:18:d6:3a:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU17',
                                        'serial': 'ADTN231446A5'},
                'ADTN231446B4': {       'dev_macs': ['cc:66:18:50:23:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU14',
                                        'serial': 'ADTN231446B4'},
                'ADTN231446B8': {       'dev_macs': ['38:f8:f6:52:a8:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU13',
                                        'serial': 'ADTN231446B8'},
                'ADTN231446BC': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU12',
                                        'serial': 'ADTN231446BC'},
                'ADTN231446C0': {       'dev_macs': ['cc:66:18:50:5b:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU18',
                                        'serial': 'ADTN231446C0'},
                'ADTN231446CB': {       'dev_macs': ['cc:66:18:dd:6a:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU14',
                                        'serial': 'ADTN231446CB'},
                'ADTN231446CC': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU14',
                                        'serial': 'ADTN231446CC'},
                'ADTN231446CE': {       'dev_macs': ['cc:66:18:dc:89:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU15',
                                        'serial': 'ADTN231446CE'},
                'ADTN23144702': {       'dev_macs': ['cc:66:18:d6:08:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU11',
                                        'serial': 'ADTN23144702'},
                'ADTN23144707': {       'dev_macs': ['cc:66:18:dd:52:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU10',
                                        'serial': 'ADTN23144707'},
                'ADTN23144714': {       'dev_macs': ['cc:66:18:dd:8d:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x14-ONU1',
                                        'serial': 'ADTN23144714'},
                'ADTN23144735': {       'dev_macs': ['cc:66:18:dc:84:51'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU25',
                                        'serial': 'ADTN23144735'},
                'ADTN23144748': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU14',
                                        'serial': 'ADTN23144748'},
                'ADTN23144759': {       'dev_macs': ['cc:66:18:4b:fc:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU15',
                                        'serial': 'ADTN23144759'},
                'ADTN23144773': {       'dev_macs': ['cc:66:18:dd:29:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU17',
                                        'serial': 'ADTN23144773'},
                'ADTN2314477A': {       'dev_macs': ['cc:66:18:dd:8a:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU26',
                                        'serial': 'ADTN2314477A'},
                'ADTN23144796': {       'dev_macs': ['cc:66:18:50:ce:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU14',
                                        'serial': 'ADTN23144796'},
                'ADTN231447A4': {       'dev_macs': ['cc:66:18:dc:86:d1'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU31',
                                        'serial': 'ADTN231447A4'},
                'ADTN231447A5': {       'dev_macs': ['cc:66:18:50:58:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU24',
                                        'serial': 'ADTN231447A5'},
                'ADTN231447AB': {       'dev_macs': ['cc:66:18:dc:b3:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU28',
                                        'serial': 'ADTN231447AB'},
                'ADTN231447AC': {       'dev_macs': ['cc:66:18:dc:b3:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU23',
                                        'serial': 'ADTN231447AC'},
                'ADTN231447AE': {       'dev_macs': ['cc:66:18:d6:03:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU15',
                                        'serial': 'ADTN231447AE'},
                'ADTN231447BD': {       'dev_macs': ['38:f8:f6:52:a8:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU12',
                                        'serial': 'ADTN231447BD'},
                'ADTN231447DA': {       'dev_macs': ['38:f8:f6:52:ab:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU24',
                                        'serial': 'ADTN231447DA'},
                'ADTN231447E3': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU15',
                                        'serial': 'ADTN231447E3'},
                'ADTN231447F3': {       'dev_macs': ['cc:66:18:dd:89:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU29',
                                        'serial': 'ADTN231447F3'},
                'ADTN231447F9': {       'dev_macs': ['38:f8:f6:52:a8:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU4',
                                        'serial': 'ADTN231447F9'},
                'ADTN2314480D': {       'dev_macs': ['cc:66:18:d6:0a:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU27',
                                        'serial': 'ADTN2314480D'},
                'ADTN2314483B': {       'dev_macs': ['cc:66:18:d6:07:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU13',
                                        'serial': 'ADTN2314483B'},
                'ADTN23144861': {       'dev_macs': ['cc:66:18:50:9a:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU15',
                                        'serial': 'ADTN23144861'},
                'ADTN2314486C': {       'dev_macs': ['cc:66:18:dc:84:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU16',
                                        'serial': 'ADTN2314486C'},
                'ADTN23144883': {       'dev_macs': ['cc:66:18:d6:0a:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU9',
                                        'serial': 'ADTN23144883'},
                'ADTN2314489C': {       'dev_macs': ['cc:66:18:d6:0c:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU6',
                                        'serial': 'ADTN2314489C'},
                'ADTN2314489D': {       'dev_macs': ['cc:66:18:50:6e:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU8',
                                        'serial': 'ADTN2314489D'},
                'ADTN231448B4': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU22',
                                        'serial': 'ADTN231448B4'},
                'ADTN231448B8': {       'dev_macs': ['cc:66:18:dd:8c:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU3',
                                        'serial': 'ADTN231448B8'},
                'ADTN231448BA': {       'dev_macs': ['cc:66:18:d6:38:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU7',
                                        'serial': 'ADTN231448BA'},
                'ADTN231448C9': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU18',
                                        'serial': 'ADTN231448C9'},
                'ADTN231448D1': {       'dev_macs': ['cc:66:18:50:b3:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU15',
                                        'serial': 'ADTN231448D1'},
                'ADTN231448D5': {       'dev_macs': ['cc:66:18:d6:0c:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU21',
                                        'serial': 'ADTN231448D5'},
                'ADTN231448FC': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU2',
                                        'serial': 'ADTN231448FC'},
                'ADTN231448FE': {       'dev_macs': ['cc:66:18:d6:0b:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU16',
                                        'serial': 'ADTN231448FE'},
                'ADTN23144916': {       'dev_macs': ['38:f8:f6:52:a8:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU19',
                                        'serial': 'ADTN23144916'},
                'ADTN23144A15': {       'dev_macs': ['cc:66:18:50:6e:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU7',
                                        'serial': 'ADTN23144A15'},
                'ADTN23144A2B': {       'dev_macs': ['cc:66:18:dc:89:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU22',
                                        'serial': 'ADTN23144A2B'},
                'ADTN23144A53': {       'dev_macs': ['cc:66:18:d6:38:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU4',
                                        'serial': 'ADTN23144A53'},
                'ADTN23144A5A': {       'dev_macs': ['cc:66:18:4c:03:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU3',
                                        'serial': 'ADTN23144A5A'},
                'ADTN23144A5C': {       'dev_macs': ['cc:66:18:4c:00:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU6',
                                        'serial': 'ADTN23144A5C'},
                'ADTN23144A69': {       'dev_macs': ['cc:66:18:51:04:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU32',
                                        'serial': 'ADTN23144A69'},
                'ADTN23144A70': {       'dev_macs': ['cc:66:18:50:d0:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU24',
                                        'serial': 'ADTN23144A70'},
                'ADTN23144A84': {       'dev_macs': ['cc:66:18:d6:34:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU5',
                                        'serial': 'ADTN23144A84'},
                'ADTN23144A86': {       'dev_macs': [       '08:55:31:e1:ea:4b',
                                                            '18:e8:29:34:ed:33',
                                                            '18:e8:29:36:d4:71',
                                                            '18:e8:29:3a:1a:3d',
                                                            '18:e8:29:8e:ed:6f',
                                                            '18:e8:29:8e:ed:78',
                                                            '18:e8:29:8e:f1:06',
                                                            '24:5a:4c:e4:c8:a1',
                                                            '74:83:c2:a6:5a:93',
                                                            '74:83:c2:a6:7f:9f',
                                                            '74:83:c2:a6:86:4e',
                                                            '74:83:c2:e8:a2:61',
                                                            '74:83:c2:ea:a6:01',
                                                            '74:83:c2:ea:a6:28',
                                                            '74:83:c2:ec:11:8b',
                                                            '74:ac:b9:80:d4:53',
                                                            '94:a6:7e:dd:ad:81',
                                                            'b4:fb:e4:5a:4f:8e',
                                                            'b8:94:70:f8:2b:2a',
                                                            'dc:2c:6e:dd:2e:26',
                                                            'f4:92:bf:38:a5:74',
                                                            'f4:92:bf:38:a7:25',
                                                            'f4:92:bf:38:a7:c2',
                                                            'f4:92:bf:d6:a8:ac'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU25',
                                        'serial': 'ADTN23144A86'},
                'ADTN23144A97': {       'dev_macs': ['cc:66:18:d6:37:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU16',
                                        'serial': 'ADTN23144A97'},
                'ADTN23144A9B': {       'dev_macs': ['cc:66:18:51:21:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU11',
                                        'serial': 'ADTN23144A9B'},
                'ADTN23144A9C': {       'dev_macs': ['cc:66:18:dd:8a:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU16',
                                        'serial': 'ADTN23144A9C'},
                'ADTN23144AB0': {       'dev_macs': ['cc:66:18:4c:01:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU26',
                                        'serial': 'ADTN23144AB0'},
                'ADTN23144ACF': {       'dev_macs': ['38:f8:f6:52:a1:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU12',
                                        'serial': 'ADTN23144ACF'},
                'ADTN23144B20': {       'dev_macs': ['cc:66:18:dc:aa:e1'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU31',
                                        'serial': 'ADTN23144B20'},
                'ADTN23144B24': {       'dev_macs': ['cc:66:18:dc:8b:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU18',
                                        'serial': 'ADTN23144B24'},
                'ADTN23144B2C': {       'dev_macs': ['cc:66:18:d6:3c:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU17',
                                        'serial': 'ADTN23144B2C'},
                'ADTN23144B3A': {       'dev_macs': ['cc:66:18:db:7e:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU20',
                                        'serial': 'ADTN23144B3A'},
                'ADTN23144B3F': {       'dev_macs': ['cc:66:18:50:63:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU8',
                                        'serial': 'ADTN23144B3F'},
                'ADTN23144B4D': {       'dev_macs': ['cc:66:18:50:9c:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU21',
                                        'serial': 'ADTN23144B4D'},
                'ADTN23144B58': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU19',
                                        'serial': 'ADTN23144B58'},
                'ADTN23144B5E': {       'dev_macs': ['cc:66:18:51:31:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU10',
                                        'serial': 'ADTN23144B5E'},
                'ADTN23144BF7': {       'dev_macs': ['cc:66:18:4c:55:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU16',
                                        'serial': 'ADTN23144BF7'},
                'ADTN23144C10': {       'dev_macs': ['cc:66:18:d6:36:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU21',
                                        'serial': 'ADTN23144C10'},
                'ADTN23144C17': {       'dev_macs': ['cc:66:18:51:04:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU25',
                                        'serial': 'ADTN23144C17'},
                'ADTN23144C33': {       'dev_macs': ['cc:66:18:dd:91:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x14-ONU2',
                                        'serial': 'ADTN23144C33'},
                'ADTN23144C46': {       'dev_macs': ['cc:66:18:dc:75:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU20',
                                        'serial': 'ADTN23144C46'},
                'ADTN23144C4E': {       'dev_macs': ['cc:66:18:50:a9:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU21',
                                        'serial': 'ADTN23144C4E'},
                'ADTN23144C8B': {       'dev_macs': ['cc:66:18:50:62:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU9',
                                        'serial': 'ADTN23144C8B'},
                'ADTN23144CC3': {       'dev_macs': ['cc:66:18:dd:8a:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU19',
                                        'serial': 'ADTN23144CC3'},
                'ADTN23144CD5': {       'dev_macs': ['cc:66:18:dd:93:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU18',
                                        'serial': 'ADTN23144CD5'},
                'ADTN23144D50': {       'dev_macs': ['cc:66:18:d6:0c:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU17',
                                        'serial': 'ADTN23144D50'},
                'ADTN23144D99': {       'dev_macs': ['cc:66:18:50:3f:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU7',
                                        'serial': 'ADTN23144D99'},
                'ADTN23144DDF': {       'dev_macs': ['cc:66:18:d6:37:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU16',
                                        'serial': 'ADTN23144DDF'},
                'ADTN23144EC8': {       'dev_macs': ['cc:66:18:50:ad:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU24',
                                        'serial': 'ADTN23144EC8'},
                'ADTN23144ED2': {       'dev_macs': ['cc:66:18:50:7b:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU30',
                                        'serial': 'ADTN23144ED2'},
                'ADTN23144ED3': {       'dev_macs': ['cc:66:18:50:b7:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU34',
                                        'serial': 'ADTN23144ED3'},
                'ADTN23144EDF': {       'dev_macs': ['cc:66:18:50:39:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU19',
                                        'serial': 'ADTN23144EDF'},
                'ADTN23144EE0': {       'dev_macs': ['cc:66:18:50:b6:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU23',
                                        'serial': 'ADTN23144EE0'},
                'ADTN23144EE3': {       'dev_macs': ['cc:66:18:dc:a8:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU28',
                                        'serial': 'ADTN23144EE3'},
                'ADTN23144EFB': {       'dev_macs': ['cc:66:18:dc:b4:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU17',
                                        'serial': 'ADTN23144EFB'},
                'ADTN23144EFD': {       'dev_macs': ['cc:66:18:dc:71:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU20',
                                        'serial': 'ADTN23144EFD'},
                'ADTN23144EFF': {       'dev_macs': ['cc:66:18:50:4a:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU20',
                                        'serial': 'ADTN23144EFF'},
                'ADTN23144F00': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU25',
                                        'serial': 'ADTN23144F00'},
                'ADTN23145120': {       'dev_macs': ['cc:66:18:4b:fe:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU20',
                                        'serial': 'ADTN23145120'},
                'ADTN24314195': {       'dev_macs': ['cc:66:18:d6:39:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU10',
                                        'serial': 'ADTN24314195'},
                'ADTN243141A3': {       'dev_macs': ['cc:66:18:50:e6:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU22',
                                        'serial': 'ADTN243141A3'},
                'ADTN243141AC': {       'dev_macs': ['cc:66:18:dd:81:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU19',
                                        'serial': 'ADTN243141AC'},
                'ADTN24314314': {       'dev_macs': ['cc:66:18:51:27:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU22',
                                        'serial': 'ADTN24314314'},
                'ADTN243144FB': {       'dev_macs': ['cc:66:18:50:fd:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU6',
                                        'serial': 'ADTN243144FB'},
                'ADTN24314766': {       'dev_macs': ['cc:66:18:50:3e:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU28',
                                        'serial': 'ADTN24314766'},
                'ADTN24363AE7': {       'dev_macs': ['38:f8:f6:52:ab:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU29',
                                        'serial': 'ADTN24363AE7'},
                'ADTN24363B6B': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU27',
                                        'serial': 'ADTN24363B6B'},
                'ADTN24363BBA': {       'dev_macs': ['cc:66:18:50:b6:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU18',
                                        'serial': 'ADTN24363BBA'},
                'ADTN24363C49': {       'dev_macs': ['cc:66:18:51:2e:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU31',
                                        'serial': 'ADTN24363C49'},
                'ADTN24363CC6': {       'dev_macs': ['cc:66:18:50:62:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU13',
                                        'serial': 'ADTN24363CC6'},
                'ADTN24363D3A': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU21',
                                        'serial': 'ADTN24363D3A'},
                'ADTN24363D4D': {       'dev_macs': ['cc:66:18:50:6a:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU32',
                                        'serial': 'ADTN24363D4D'},
                'ADTN24363DB9': {       'dev_macs': ['cc:66:18:50:b1:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU16',
                                        'serial': 'ADTN24363DB9'},
                'ADTN24363DC9': {       'dev_macs': ['cc:66:18:50:ac:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU28',
                                        'serial': 'ADTN24363DC9'},
                'ADTN243896F9': {       'dev_macs': ['cc:66:18:51:25:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU30',
                                        'serial': 'ADTN243896F9'},
                'ADTN244917A4': {       'dev_macs': ['cc:66:18:50:9b:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU22',
                                        'serial': 'ADTN244917A4'},
                'ADTN251521A2': {       'dev_macs': ['cc:66:18:50:ba:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU17',
                                        'serial': 'ADTN251521A2'},
                'CXNK00A4305B': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU19',
                                        'serial': 'CXNK00A4305B'}
        }
        exp_ne_msg = {
                'data': {       'cpu': 53.35,
                                'interfaces': [       {       'admin_up': True,
                                                              'id': '0/1',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/1',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8x',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/1',
                                                                                      'id': 'xgspon '
                                                                                            '0/1',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/1',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8x',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/10',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dc:88:30'],
                                                                                       'in_nucast_pkts': '17350',
                                                                                       'in_octets': '13335759005',
                                                                                       'in_ucast_pkts': '104288639',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '481481868213',
                                                                                       'out_ucast_pkts': '328067017',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDY4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:88:a0'],
                                                                                       'in_nucast_pkts': '17353',
                                                                                       'in_octets': '3375284024',
                                                                                       'in_ucast_pkts': '7643960',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '38970248215',
                                                                                       'out_ucast_pkts': '31960265',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTE4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:81:d0'],
                                                                                       'in_nucast_pkts': '116896',
                                                                                       'in_octets': '36055555296',
                                                                                       'in_ucast_pkts': '133646574',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9898',
                                                                                       'out_octets': '481248857766',
                                                                                       'out_ucast_pkts': '378880965',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDU1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:81:e0'],
                                                                                       'in_nucast_pkts': '116891',
                                                                                       'in_octets': '49636365763',
                                                                                       'in_ucast_pkts': '281729576',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9894',
                                                                                       'out_octets': '1329234239498',
                                                                                       'out_ucast_pkts': '918260033',
                                                                                       'poller_hash': 'QURUTjIzMTI0NjUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7d:00'],
                                                                                       'in_nucast_pkts': '24484',
                                                                                       'in_octets': '6556027515',
                                                                                       'in_ucast_pkts': '13343348',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2072',
                                                                                       'out_octets': '40619971557',
                                                                                       'out_ucast_pkts': '34613725',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDIwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:74:90'],
                                                                                       'in_nucast_pkts': '185353',
                                                                                       'in_octets': '62164914093',
                                                                                       'in_ucast_pkts': '236837758',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15625',
                                                                                       'out_octets': '875539994170',
                                                                                       'out_ucast_pkts': '653727994',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDdFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:80:50'],
                                                                                       'in_nucast_pkts': '17337',
                                                                                       'in_octets': '11797895551',
                                                                                       'in_ucast_pkts': '77635048',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1465',
                                                                                       'out_octets': '301282269730',
                                                                                       'out_ucast_pkts': '209671611',
                                                                                       'poller_hash': 'QURUTjIzMTI0REIyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:68:b0'],
                                                                                       'in_nucast_pkts': '17355',
                                                                                       'in_octets': '3607712444',
                                                                                       'in_ucast_pkts': '16831967',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1468',
                                                                                       'out_octets': '60418812521',
                                                                                       'out_ucast_pkts': '45761191',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTI5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7c:d0'],
                                                                                       'in_nucast_pkts': '17359',
                                                                                       'in_octets': '7555932399',
                                                                                       'in_ucast_pkts': '48517272',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1468',
                                                                                       'out_octets': '272090504758',
                                                                                       'out_ucast_pkts': '194566979',
                                                                                       'poller_hash': 'QURUTjIzMTI0MUQwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:d0'],
                                                                                       'in_nucast_pkts': '17497',
                                                                                       'in_octets': '12357854316',
                                                                                       'in_ucast_pkts': '39057101',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '195856588153',
                                                                                       'out_ucast_pkts': '146883781',
                                                                                       'poller_hash': 'QURUTjIzMTI0REJDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:b2:80'],
                                                                                       'in_nucast_pkts': '17074',
                                                                                       'in_octets': '5123053333',
                                                                                       'in_ucast_pkts': '12730687',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '39204583525',
                                                                                       'out_ucast_pkts': '35900007',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTJCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:23:20'],
                                                                                       'in_nucast_pkts': '56280',
                                                                                       'in_octets': '75997970204',
                                                                                       'in_ucast_pkts': '230241542',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4649',
                                                                                       'out_octets': '983572185597',
                                                                                       'out_ucast_pkts': '706426217',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTdGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:7f:60'],
                                                                                       'in_nucast_pkts': '17350',
                                                                                       'in_octets': '24490568755',
                                                                                       'in_ucast_pkts': '74536339',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '219456576798',
                                                                                       'out_ucast_pkts': '166369684',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTVCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:6a:40'],
                                                                                       'in_nucast_pkts': '17357',
                                                                                       'in_octets': '11117991299',
                                                                                       'in_ucast_pkts': '50364145',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '137294957416',
                                                                                       'out_ucast_pkts': '117912283',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUIxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:30:c0'],
                                                                                       'in_nucast_pkts': '114985',
                                                                                       'in_octets': '21659055433',
                                                                                       'in_ucast_pkts': '102931082',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3834',
                                                                                       'out_octets': '455566541671',
                                                                                       'out_ucast_pkts': '320085741',
                                                                                       'poller_hash': 'QURUTjIzMTI0RThBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:4f:90'],
                                                                                       'in_nucast_pkts': '114988',
                                                                                       'in_octets': '30698021276',
                                                                                       'in_ucast_pkts': '152237846',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3834',
                                                                                       'out_octets': '493904889370',
                                                                                       'out_ucast_pkts': '362374584',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDY1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8f:50'],
                                                                                       'in_nucast_pkts': '17354',
                                                                                       'in_octets': '29097681192',
                                                                                       'in_ucast_pkts': '95864727',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '281619296041',
                                                                                       'out_ucast_pkts': '225231276',
                                                                                       'poller_hash': 'QURUTjIzMTIzRTlEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:30'],
                                                                                       'in_nucast_pkts': '17776',
                                                                                       'in_octets': '95455458929',
                                                                                       'in_ucast_pkts': '224634758',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '598351349113',
                                                                                       'out_ucast_pkts': '459796937',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTMzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:03:40'],
                                                                                       'in_nucast_pkts': '116887',
                                                                                       'in_octets': '138058217301',
                                                                                       'in_ucast_pkts': '365230395',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9896',
                                                                                       'out_octets': '1101010958219',
                                                                                       'out_ucast_pkts': '807821198',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTQzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d5:fd:61'],
                                                                                       'in_nucast_pkts': '309',
                                                                                       'in_octets': '6398862455',
                                                                                       'in_ucast_pkts': '46841830',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '218148219816',
                                                                                       'out_ucast_pkts': '148815866',
                                                                                       'poller_hash': 'QURUTjIzMTI0NDkxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:88:70'],
                                                                                       'in_nucast_pkts': '116887',
                                                                                       'in_octets': '174393990995',
                                                                                       'in_ucast_pkts': '494997024',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9894',
                                                                                       'out_octets': '1118290591416',
                                                                                       'out_ucast_pkts': '932843019',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTBCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0c:90'],
                                                                                       'in_nucast_pkts': '116924',
                                                                                       'in_octets': '90453348503',
                                                                                       'in_ucast_pkts': '160959414',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9897',
                                                                                       'out_octets': '416534696335',
                                                                                       'out_ucast_pkts': '325483464',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OEQ1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '08:55:31:e1:ea:4b',
                                                                                                                        '18:e8:29:34:ed:33',
                                                                                                                        '18:e8:29:36:d4:71',
                                                                                                                        '18:e8:29:3a:1a:3d',
                                                                                                                        '18:e8:29:8e:ed:6f',
                                                                                                                        '18:e8:29:8e:ed:78',
                                                                                                                        '18:e8:29:8e:f1:06',
                                                                                                                        '24:5a:4c:e4:c8:a1',
                                                                                                                        '74:83:c2:a6:5a:93',
                                                                                                                        '74:83:c2:a6:7f:9f',
                                                                                                                        '74:83:c2:a6:86:4e',
                                                                                                                        '74:83:c2:e8:a2:61',
                                                                                                                        '74:83:c2:ea:a6:01',
                                                                                                                        '74:83:c2:ea:a6:28',
                                                                                                                        '74:83:c2:ec:11:8b',
                                                                                                                        '74:ac:b9:80:d4:53',
                                                                                                                        '94:a6:7e:dd:ad:81',
                                                                                                                        'b4:fb:e4:5a:4f:8e',
                                                                                                                        'b8:94:70:f8:2b:2a',
                                                                                                                        'dc:2c:6e:dd:2e:26',
                                                                                                                        'f4:92:bf:38:a5:74',
                                                                                                                        'f4:92:bf:38:a7:25',
                                                                                                                        'f4:92:bf:38:a7:c2',
                                                                                                                        'f4:92:bf:d6:a8:ac'],
                                                                                       'in_nucast_pkts': '8119817',
                                                                                       'in_octets': '1884647079326',
                                                                                       'in_ucast_pkts': '9369806084',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '155642',
                                                                                       'out_octets': '28843170418904',
                                                                                       'out_ucast_pkts': '21400391889',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTg2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:b3:c0'],
                                                                                       'in_nucast_pkts': '40455',
                                                                                       'in_octets': '13102866261',
                                                                                       'in_ucast_pkts': '36130618',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3423',
                                                                                       'out_octets': '106071734827',
                                                                                       'out_ucast_pkts': '90461016',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0FDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:87:20'],
                                                                                       'in_nucast_pkts': '17355',
                                                                                       'in_octets': '13123606756',
                                                                                       'in_ucast_pkts': '69752375',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '290532092288',
                                                                                       'out_ucast_pkts': '204848809',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NjlFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:3e:d0'],
                                                                                       'in_nucast_pkts': '87861',
                                                                                       'in_octets': '47614043797',
                                                                                       'in_ucast_pkts': '200344149',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2878',
                                                                                       'out_octets': '609293328531',
                                                                                       'out_ucast_pkts': '439693210',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTM1IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/10',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/10',
                                                                                      'id': 'xgspon '
                                                                                            '0/10',
                                                                                      'links': [       {       'connected_time': '511515',
                                                                                                               'fiber_length': 5677,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDY4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511514',
                                                                                                               'fiber_length': 6613,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTE4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '3449060',
                                                                                                               'fiber_length': 6229,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDU1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '3449056',
                                                                                                               'fiber_length': 5972,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NjUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '721987',
                                                                                                               'fiber_length': 5554,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '5440106',
                                                                                                               'fiber_length': 6288,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDdFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511514',
                                                                                                               'fiber_length': 5688,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REIyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511514',
                                                                                                               'fiber_length': 5757,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTI5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511510',
                                                                                                               'fiber_length': 5845,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MUQwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511514',
                                                                                                               'fiber_length': 6895,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REJDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511513',
                                                                                                               'fiber_length': 5938,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTJCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '1620457',
                                                                                                               'fiber_length': 7112,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTdGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511513',
                                                                                                               'fiber_length': 6916,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTVCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511511',
                                                                                                               'fiber_length': 6439,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUIxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '3449058',
                                                                                                               'fiber_length': 6277,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RThBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '3449060',
                                                                                                               'fiber_length': 6171,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDY1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511511',
                                                                                                               'fiber_length': 6505,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRTlEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511510',
                                                                                                               'fiber_length': 6490,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTMzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '3449061',
                                                                                                               'fiber_length': 6584,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTQzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511515',
                                                                                                               'fiber_length': 6819,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NDkxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '3449060',
                                                                                                               'fiber_length': 6776,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTBCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '3449059',
                                                                                                               'fiber_length': 5964,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OEQ1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '5440103',
                                                                                                               'fiber_length': 7783,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTg2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '1193288',
                                                                                                               'fiber_length': 6748,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '511513',
                                                                                                               'fiber_length': 6818,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NjlFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659},
                                                                                                       {       'connected_time': '2588442',
                                                                                                               'fiber_length': 6346,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTM1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.659}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/10',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/11',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:af:10'],
                                                                                       'in_nucast_pkts': '17492',
                                                                                       'in_octets': '5143788698',
                                                                                       'in_ucast_pkts': '12155920',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '52724693825',
                                                                                       'out_ucast_pkts': '41538448',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjJEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:79:10'],
                                                                                       'in_nucast_pkts': '17767',
                                                                                       'in_octets': '19444386289',
                                                                                       'in_ucast_pkts': '36348646',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '144731181340',
                                                                                       'out_ucast_pkts': '110500290',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDI2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:68:90'],
                                                                                       'in_nucast_pkts': '17343',
                                                                                       'in_octets': '6881121985',
                                                                                       'in_ucast_pkts': '10791436',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '27921309796',
                                                                                       'out_ucast_pkts': '20288820',
                                                                                       'poller_hash': 'QURUTjIzMTIzRjU3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8b:b0'],
                                                                                       'in_nucast_pkts': '17777',
                                                                                       'in_octets': '19955393626',
                                                                                       'in_ucast_pkts': '80533969',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '296743546858',
                                                                                       'out_ucast_pkts': '220008288',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDg4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6b:c0'],
                                                                                       'in_nucast_pkts': '17343',
                                                                                       'in_octets': '7546768448',
                                                                                       'in_ucast_pkts': '13490279',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '32999075131',
                                                                                       'out_ucast_pkts': '28090751',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjE0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:89:00'],
                                                                                       'in_nucast_pkts': '17771',
                                                                                       'in_octets': '14552608567',
                                                                                       'in_ucast_pkts': '55071587',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '118612809491',
                                                                                       'out_ucast_pkts': '93178460',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjM3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:65:60'],
                                                                                       'in_nucast_pkts': '17346',
                                                                                       'in_octets': '37249983009',
                                                                                       'in_ucast_pkts': '59033750',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '215519017972',
                                                                                       'out_ucast_pkts': '168055395',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTYxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:48:80'],
                                                                                       'in_nucast_pkts': '31563',
                                                                                       'in_octets': '33193181638',
                                                                                       'in_ucast_pkts': '112223303',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1053',
                                                                                       'out_octets': '458897956956',
                                                                                       'out_ucast_pkts': '325661781',
                                                                                       'poller_hash': 'QURUTjIzMTI0REEwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:02:b0'],
                                                                                       'in_nucast_pkts': '17346',
                                                                                       'in_octets': '2257714498',
                                                                                       'in_ucast_pkts': '10560616',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '36643183297',
                                                                                       'out_ucast_pkts': '27915703',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEZFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:09:70'],
                                                                                       'in_nucast_pkts': '32302',
                                                                                       'in_octets': '15455041408',
                                                                                       'in_ucast_pkts': '68701404',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2719',
                                                                                       'out_octets': '311664606418',
                                                                                       'out_ucast_pkts': '223820942',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:40'],
                                                                                       'in_nucast_pkts': '32074',
                                                                                       'in_octets': '15119379583',
                                                                                       'in_ucast_pkts': '98696275',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2713',
                                                                                       'out_octets': '354591641228',
                                                                                       'out_ucast_pkts': '250169385',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEJGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:65:70'],
                                                                                       'in_nucast_pkts': '17349',
                                                                                       'in_octets': '6949689103',
                                                                                       'in_ucast_pkts': '30596842',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '154299720484',
                                                                                       'out_ucast_pkts': '111934773',
                                                                                       'poller_hash': 'QURUTjIzMTIzRUMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:65:d0'],
                                                                                       'in_nucast_pkts': '32073',
                                                                                       'in_octets': '9337586634',
                                                                                       'in_ucast_pkts': '22417412',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2713',
                                                                                       'out_octets': '47856079004',
                                                                                       'out_ucast_pkts': '45368420',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkVFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b1:50'],
                                                                                       'in_nucast_pkts': '115000',
                                                                                       'in_octets': '66411983841',
                                                                                       'in_ucast_pkts': '250512278',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3835',
                                                                                       'out_octets': '1423739286835',
                                                                                       'out_ucast_pkts': '1044417982',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkU5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:f5:40'],
                                                                                       'in_nucast_pkts': '114799',
                                                                                       'in_octets': '57045873988',
                                                                                       'in_ucast_pkts': '337607038',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3828',
                                                                                       'out_octets': '1524879113596',
                                                                                       'out_ucast_pkts': '1180391737',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEQxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:89:30'],
                                                                                       'in_nucast_pkts': '17378',
                                                                                       'in_octets': '53122213483',
                                                                                       'in_ucast_pkts': '167810645',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1469',
                                                                                       'out_octets': '420548488076',
                                                                                       'out_ucast_pkts': '318233832',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkNFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:37:10'],
                                                                                       'in_nucast_pkts': '32089',
                                                                                       'in_octets': '11857620441',
                                                                                       'in_ucast_pkts': '43266679',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2715',
                                                                                       'out_octets': '364213727415',
                                                                                       'out_ucast_pkts': '253032203',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RERGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0c:40'],
                                                                                       'in_nucast_pkts': '17772',
                                                                                       'in_octets': '17672935333',
                                                                                       'in_ucast_pkts': '33715104',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '74712486667',
                                                                                       'out_ucast_pkts': '61735837',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RDUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3c:20'],
                                                                                       'in_nucast_pkts': '17356',
                                                                                       'in_octets': '7171049590',
                                                                                       'in_ucast_pkts': '46866609',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '260566772494',
                                                                                       'out_ucast_pkts': '186045618',
                                                                                       'poller_hash': 'QURUTjIzMTI0QzczIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:39:50'],
                                                                                       'in_nucast_pkts': '25782',
                                                                                       'in_octets': '37542868176',
                                                                                       'in_ucast_pkts': '76101459',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '849',
                                                                                       'out_octets': '149492399337',
                                                                                       'out_ucast_pkts': '149742341',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RURGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:97:70'],
                                                                                       'in_nucast_pkts': '17346',
                                                                                       'in_octets': '6349422540',
                                                                                       'in_ucast_pkts': '27790797',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '121891202937',
                                                                                       'out_ucast_pkts': '95008852',
                                                                                       'poller_hash': 'QURUTjIzMTQ0MjhBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ae:70'],
                                                                                       'in_nucast_pkts': '116608',
                                                                                       'in_octets': '51659450960',
                                                                                       'in_ucast_pkts': '382203301',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3831',
                                                                                       'out_octets': '1410544228418',
                                                                                       'out_ucast_pkts': '992370722',
                                                                                       'poller_hash': 'QURUTjIzMTI0MUEyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:9b:50'],
                                                                                       'in_nucast_pkts': '17496',
                                                                                       'in_octets': '57766222085',
                                                                                       'in_ucast_pkts': '69006479',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '153363791304',
                                                                                       'out_ucast_pkts': '125730792',
                                                                                       'poller_hash': 'QURUTjI0NDkxN0E0IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/11',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/11',
                                                                                      'id': 'xgspon '
                                                                                            '0/11',
                                                                                      'links': [       {       'connected_time': '511502',
                                                                                                               'fiber_length': 4645,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjJEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511502',
                                                                                                               'fiber_length': 4747,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDI2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511503',
                                                                                                               'fiber_length': 4661,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRjU3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511502',
                                                                                                               'fiber_length': 4852,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDg4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511503',
                                                                                                               'fiber_length': 7344,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511499',
                                                                                                               'fiber_length': 7364,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjM3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511503',
                                                                                                               'fiber_length': 5086,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTYxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '946254',
                                                                                                               'fiber_length': 8008,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REEwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511500',
                                                                                                               'fiber_length': 4938,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEZFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '946258',
                                                                                                               'fiber_length': 8251,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '946255',
                                                                                                               'fiber_length': 8000,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEJGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511499',
                                                                                                               'fiber_length': 4770,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRUMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '946255',
                                                                                                               'fiber_length': 8343,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkVFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '3449049',
                                                                                                               'fiber_length': 8068,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkU5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '3449048',
                                                                                                               'fiber_length': 8127,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEQxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511501',
                                                                                                               'fiber_length': 7294,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '946257',
                                                                                                               'fiber_length': 8094,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RERGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511501',
                                                                                                               'fiber_length': 7706,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RDUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511499',
                                                                                                               'fiber_length': 4666,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QzczIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '762405',
                                                                                                               'fiber_length': 8136,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RURGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511501',
                                                                                                               'fiber_length': 7421,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0MjhBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '3449046',
                                                                                                               'fiber_length': 8116,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MUEyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913},
                                                                                                       {       'connected_time': '511506',
                                                                                                               'fiber_length': 4703,
                                                                                                               'poller_hash': 'QURUTjI0NDkxN0E0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4913}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/11',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/12',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dd:78:e0'],
                                                                                       'in_nucast_pkts': '17345',
                                                                                       'in_octets': '13570085705',
                                                                                       'in_ucast_pkts': '46186668',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '166640051211',
                                                                                       'out_ucast_pkts': '149119825',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTY2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:75:70'],
                                                                                       'in_nucast_pkts': '13736',
                                                                                       'in_octets': '6225522635',
                                                                                       'in_ucast_pkts': '24138193',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '84342536497',
                                                                                       'out_ucast_pkts': '63254582',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTkwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:40'],
                                                                                       'in_nucast_pkts': '17347',
                                                                                       'in_octets': '5943079236',
                                                                                       'in_ucast_pkts': '24328987',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '170208238795',
                                                                                       'out_ucast_pkts': '125604721',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTQwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7c:f0'],
                                                                                       'in_nucast_pkts': '17774',
                                                                                       'in_octets': '2445008553',
                                                                                       'in_ucast_pkts': '17998019',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '87124072293',
                                                                                       'out_ucast_pkts': '62663530',
                                                                                       'poller_hash': 'QURUTjIzMTI0RUQ5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:70:a0'],
                                                                                       'in_nucast_pkts': '184357',
                                                                                       'in_octets': '261894506093',
                                                                                       'in_ucast_pkts': '730736385',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15616',
                                                                                       'out_octets': '2036903540767',
                                                                                       'out_ucast_pkts': '1780740679',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTNFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:56:40'],
                                                                                       'in_nucast_pkts': '13959',
                                                                                       'in_octets': '21097991933',
                                                                                       'in_ucast_pkts': '51157611',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '245583176080',
                                                                                       'out_ucast_pkts': '187777485',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUVCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:76:40'],
                                                                                       'in_nucast_pkts': '13960',
                                                                                       'in_octets': '4994513603',
                                                                                       'in_ucast_pkts': '13310420',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '44519148743',
                                                                                       'out_ucast_pkts': '38135069',
                                                                                       'poller_hash': 'QURUTjIzMTI0REU0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7e:00'],
                                                                                       'in_nucast_pkts': '17344',
                                                                                       'in_octets': '71894337987',
                                                                                       'in_ucast_pkts': '116007144',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '261076709092',
                                                                                       'out_ucast_pkts': '208646296',
                                                                                       'poller_hash': 'QURUTjIzMTI0REIwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:20'],
                                                                                       'in_nucast_pkts': '17071',
                                                                                       'in_octets': '6657479764',
                                                                                       'in_ucast_pkts': '31241128',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '275313555690',
                                                                                       'out_ucast_pkts': '201361428',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjM0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ab:30'],
                                                                                       'in_nucast_pkts': '14078',
                                                                                       'in_octets': '11207439649',
                                                                                       'in_ucast_pkts': '31634032',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '156037674829',
                                                                                       'out_ucast_pkts': '115883166',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTU3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:e0'],
                                                                                       'in_nucast_pkts': '17068',
                                                                                       'in_octets': '23508181967',
                                                                                       'in_ucast_pkts': '34230325',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '117692129423',
                                                                                       'out_ucast_pkts': '93226952',
                                                                                       'poller_hash': 'QURUTjIzMTIzRUI3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6e:40'],
                                                                                       'in_nucast_pkts': '17767',
                                                                                       'in_octets': '5155013968',
                                                                                       'in_ucast_pkts': '34663408',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '210018721979',
                                                                                       'out_ucast_pkts': '145411454',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTM3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8f:b0'],
                                                                                       'in_nucast_pkts': '17662',
                                                                                       'in_octets': '27987474147',
                                                                                       'in_ucast_pkts': '114204034',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1465',
                                                                                       'out_octets': '429942845377',
                                                                                       'out_ucast_pkts': '308714554',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTRCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:52:10'],
                                                                                       'in_nucast_pkts': '13738',
                                                                                       'in_octets': '8150070916',
                                                                                       'in_ucast_pkts': '24703195',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '154412345995',
                                                                                       'out_ucast_pkts': '110311064',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTczIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:7d:70'],
                                                                                       'in_nucast_pkts': '17347',
                                                                                       'in_octets': '7152861603',
                                                                                       'in_ucast_pkts': '27555356',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '108732896514',
                                                                                       'out_ucast_pkts': '79082535',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTE3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6c:b0'],
                                                                                       'in_nucast_pkts': '17771',
                                                                                       'in_octets': '9693079177',
                                                                                       'in_ucast_pkts': '67618894',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '287715030573',
                                                                                       'out_ucast_pkts': '201871393',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTJDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:77:30'],
                                                                                       'in_nucast_pkts': '17343',
                                                                                       'in_octets': '4519727774',
                                                                                       'in_ucast_pkts': '20018006',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '92169783415',
                                                                                       'out_ucast_pkts': '67778587',
                                                                                       'poller_hash': 'QURUTjIzMTIzRUExIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:76:40'],
                                                                                       'in_nucast_pkts': '17344',
                                                                                       'in_octets': '33003402586',
                                                                                       'in_ucast_pkts': '86853826',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '322535900258',
                                                                                       'out_ucast_pkts': '233738809',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTMzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:a3:c0'],
                                                                                       'in_nucast_pkts': '17075',
                                                                                       'in_octets': '38171933287',
                                                                                       'in_ucast_pkts': '91083116',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '333219784794',
                                                                                       'out_ucast_pkts': '239709601',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUFCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0b:60'],
                                                                                       'in_nucast_pkts': '14266',
                                                                                       'in_octets': '60905332',
                                                                                       'in_ucast_pkts': '220207',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '186254722',
                                                                                       'out_ucast_pkts': '254635',
                                                                                       'poller_hash': 'QURUTjIzMTI0MzA0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:87:b0'],
                                                                                       'in_nucast_pkts': '17069',
                                                                                       'in_octets': '5704883251',
                                                                                       'in_ucast_pkts': '7721731',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '11207396635',
                                                                                       'out_ucast_pkts': '10837380',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTRDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:89:60'],
                                                                                       'in_nucast_pkts': '17347',
                                                                                       'in_octets': '4868512344',
                                                                                       'in_ucast_pkts': '51003832',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '154865189855',
                                                                                       'out_ucast_pkts': '104664563',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkM0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:10'],
                                                                                       'in_nucast_pkts': '17355',
                                                                                       'in_octets': '3985209650',
                                                                                       'in_ucast_pkts': '3739148',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '3947279049',
                                                                                       'out_ucast_pkts': '3942003',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEMzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:58:40'],
                                                                                       'in_nucast_pkts': '182787',
                                                                                       'in_octets': '122285030297',
                                                                                       'in_ucast_pkts': '286802952',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6048',
                                                                                       'out_octets': '958305135954',
                                                                                       'out_ucast_pkts': '731293813',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0E1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:80'],
                                                                                       'in_nucast_pkts': '17346',
                                                                                       'in_octets': '2305401881',
                                                                                       'in_ucast_pkts': '8993831',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '26549022939',
                                                                                       'out_ucast_pkts': '21176507',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzdBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0a:00'],
                                                                                       'in_nucast_pkts': '33987',
                                                                                       'in_octets': '35230512504',
                                                                                       'in_ucast_pkts': '35656386',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2877',
                                                                                       'out_octets': '15607170836',
                                                                                       'out_ucast_pkts': '18811292',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODBEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:b3:a0'],
                                                                                       'in_nucast_pkts': '17347',
                                                                                       'in_octets': '18278042675',
                                                                                       'in_ucast_pkts': '39931214',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '187321586366',
                                                                                       'out_ucast_pkts': '136018147',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0FCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:89:30'],
                                                                                       'in_nucast_pkts': '17676',
                                                                                       'in_octets': '174845768581',
                                                                                       'in_ucast_pkts': '251770340',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '606185853600',
                                                                                       'out_ucast_pkts': '484055879',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0YzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:7b:50'],
                                                                                       'in_nucast_pkts': '17435',
                                                                                       'in_octets': '18736611599',
                                                                                       'in_ucast_pkts': '29611412',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '569',
                                                                                       'out_octets': '70520406471',
                                                                                       'out_ucast_pkts': '60883818',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUQyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:42:00'],
                                                                                       'in_nucast_pkts': '13735',
                                                                                       'in_octets': '5787604886',
                                                                                       'in_ucast_pkts': '50188010',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '140051627721',
                                                                                       'out_ucast_pkts': '98055630',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDZDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6a:d0'],
                                                                                       'in_nucast_pkts': '17073',
                                                                                       'in_octets': '5040846635',
                                                                                       'in_ucast_pkts': '18204867',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '29200136912',
                                                                                       'out_ucast_pkts': '21341048',
                                                                                       'poller_hash': 'QURUTjI0MzYzRDREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:aa:e1'],
                                                                                       'in_nucast_pkts': '307',
                                                                                       'in_octets': '12297137773',
                                                                                       'in_ucast_pkts': '74863842',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '254976806134',
                                                                                       'out_ucast_pkts': '187481803',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjIwIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/12',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/12',
                                                                                      'id': 'xgspon '
                                                                                            '0/12',
                                                                                      'links': [       {       'connected_time': '511499',
                                                                                                               'fiber_length': 5195,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '411498',
                                                                                                               'fiber_length': 6895,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTkwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511500',
                                                                                                               'fiber_length': 5726,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTQwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511500',
                                                                                                               'fiber_length': 5679,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RUQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '5440063',
                                                                                                               'fiber_length': 5805,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTNFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '411498',
                                                                                                               'fiber_length': 6685,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUVCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '411500',
                                                                                                               'fiber_length': 6822,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REU0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511498',
                                                                                                               'fiber_length': 5405,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511501',
                                                                                                               'fiber_length': 5229,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjM0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '411499',
                                                                                                               'fiber_length': 6531,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTU3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511504',
                                                                                                               'fiber_length': 3552,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRUI3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511500',
                                                                                                               'fiber_length': 5276,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTM3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511495',
                                                                                                               'fiber_length': 5187,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTRCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '411498',
                                                                                                               'fiber_length': 4886,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTczIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511501',
                                                                                                               'fiber_length': 4322,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTE3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511500',
                                                                                                               'fiber_length': 5460,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTJDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511502',
                                                                                                               'fiber_length': 3586,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRUExIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511499',
                                                                                                               'fiber_length': 3613,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTMzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511499',
                                                                                                               'fiber_length': 5452,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUFCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '411499',
                                                                                                               'fiber_length': 4895,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MzA0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511499',
                                                                                                               'fiber_length': 5052,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTRDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511501',
                                                                                                               'fiber_length': 3515,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkM0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511504',
                                                                                                               'fiber_length': 3450,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEMzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '5440060',
                                                                                                               'fiber_length': 4061,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0E1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511499',
                                                                                                               'fiber_length': 3993,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzdBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '1002979',
                                                                                                               'fiber_length': 4007,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODBEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511498',
                                                                                                               'fiber_length': 4238,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511504',
                                                                                                               'fiber_length': 4128,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0YzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '508250',
                                                                                                               'fiber_length': 4439,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUQyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '411498',
                                                                                                               'fiber_length': 6899,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDZDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511510',
                                                                                                               'fiber_length': 3928,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRDREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135},
                                                                                                       {       'connected_time': '511504',
                                                                                                               'fiber_length': 5562,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4135}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/12',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/13',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dd:84:00'],
                                                                                       'in_nucast_pkts': '14301',
                                                                                       'in_octets': '23521366774',
                                                                                       'in_ucast_pkts': '40895445',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '91736619845',
                                                                                       'out_ucast_pkts': '75806891',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTcxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6e:f0'],
                                                                                       'in_nucast_pkts': '14301',
                                                                                       'in_octets': '4925425766',
                                                                                       'in_ucast_pkts': '26949352',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '166267130463',
                                                                                       'out_ucast_pkts': '116451702',
                                                                                       'poller_hash': 'QURUTjIzMTI0RERBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8e:50'],
                                                                                       'in_nucast_pkts': '14301',
                                                                                       'in_octets': '5965607609',
                                                                                       'in_ucast_pkts': '18743666',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '83538905826',
                                                                                       'out_ucast_pkts': '65251732',
                                                                                       'poller_hash': 'QURUTjIzMTI0REUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:b0'],
                                                                                       'in_nucast_pkts': '13739',
                                                                                       'in_octets': '15227797031',
                                                                                       'in_ucast_pkts': '76341628',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '336060770122',
                                                                                       'out_ucast_pkts': '237694633',
                                                                                       'poller_hash': 'QURUTjIzMTI0REY2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:01:50'],
                                                                                       'in_nucast_pkts': '13739',
                                                                                       'in_octets': '545655781',
                                                                                       'in_ucast_pkts': '1123435',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '391776268',
                                                                                       'out_ucast_pkts': '722858',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTA0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:79:70'],
                                                                                       'in_nucast_pkts': '13956',
                                                                                       'in_octets': '8809687008',
                                                                                       'in_ucast_pkts': '24162714',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '57389462190',
                                                                                       'out_ucast_pkts': '46911227',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a6:80'],
                                                                                       'in_nucast_pkts': '13879',
                                                                                       'in_octets': '13903633169',
                                                                                       'in_ucast_pkts': '78829773',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '399712886520',
                                                                                       'out_ucast_pkts': '286849218',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTkyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6d:50'],
                                                                                       'in_nucast_pkts': '13955',
                                                                                       'in_octets': '2548805306',
                                                                                       'in_ucast_pkts': '16585756',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '67292117686',
                                                                                       'out_ucast_pkts': '49198748',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTg2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:81:b0'],
                                                                                       'in_nucast_pkts': '13739',
                                                                                       'in_octets': '42012549892',
                                                                                       'in_ucast_pkts': '61902384',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '291478069020',
                                                                                       'out_ucast_pkts': '212080804',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTk0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:e0'],
                                                                                       'in_nucast_pkts': '14289',
                                                                                       'in_octets': '2854488104',
                                                                                       'in_ucast_pkts': '9495644',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '40971690775',
                                                                                       'out_ucast_pkts': '29089697',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTRFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:00'],
                                                                                       'in_nucast_pkts': '14078',
                                                                                       'in_octets': '19069866624',
                                                                                       'in_ucast_pkts': '46572371',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '267524211604',
                                                                                       'out_ucast_pkts': '205567921',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkI4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:6a:80'],
                                                                                       'in_nucast_pkts': '184398',
                                                                                       'in_octets': '152252613088',
                                                                                       'in_ucast_pkts': '614225179',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15623',
                                                                                       'out_octets': '2157383541720',
                                                                                       'out_ucast_pkts': '1604677636',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkNCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:9a:d0'],
                                                                                       'in_nucast_pkts': '181358',
                                                                                       'in_octets': '144437797488',
                                                                                       'in_ucast_pkts': '1022860980',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6045',
                                                                                       'out_octets': '4872132831389',
                                                                                       'out_ucast_pkts': '3478110433',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODYxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6e:b0'],
                                                                                       'in_nucast_pkts': '14079',
                                                                                       'in_octets': '3578998606',
                                                                                       'in_ucast_pkts': '25157851',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '92781727659',
                                                                                       'out_ucast_pkts': '65792280',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTE1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:55:80'],
                                                                                       'in_nucast_pkts': '13740',
                                                                                       'in_octets': '7157217416',
                                                                                       'in_ucast_pkts': '14080599',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '62900310056',
                                                                                       'out_ucast_pkts': '46212700',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QkY3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:02:90'],
                                                                                       'in_nucast_pkts': '14078',
                                                                                       'in_octets': '1317198415',
                                                                                       'in_ucast_pkts': '5401632',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '14367286856',
                                                                                       'out_ucast_pkts': '11234680',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTVCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:62:10'],
                                                                                       'in_nucast_pkts': '17070',
                                                                                       'in_octets': '6446512919',
                                                                                       'in_ucast_pkts': '9277120',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '25774365898',
                                                                                       'out_ucast_pkts': '22077595',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzhCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b6:c0'],
                                                                                       'in_nucast_pkts': '13739',
                                                                                       'in_octets': '3138746474',
                                                                                       'in_ucast_pkts': '11181202',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '56765714029',
                                                                                       'out_ucast_pkts': '41327644',
                                                                                       'poller_hash': 'QURUTjI0MzYzQkJBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:81:80'],
                                                                                       'in_nucast_pkts': '13960',
                                                                                       'in_octets': '2291465599',
                                                                                       'in_ucast_pkts': '10180171',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '27268442391',
                                                                                       'out_ucast_pkts': '21783386',
                                                                                       'poller_hash': 'QURUTjI0MzE0MUFDIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/13',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/13',
                                                                                      'id': 'xgspon '
                                                                                            '0/13',
                                                                                      'links': [       {       'connected_time': '411496',
                                                                                                               'fiber_length': 8742,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTcxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411500',
                                                                                                               'fiber_length': 8706,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RERBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411497',
                                                                                                               'fiber_length': 8348,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411500',
                                                                                                               'fiber_length': 8325,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411499',
                                                                                                               'fiber_length': 8705,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTA0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411496',
                                                                                                               'fiber_length': 8651,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411499',
                                                                                                               'fiber_length': 8747,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTkyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411499',
                                                                                                               'fiber_length': 8737,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTg2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411493',
                                                                                                               'fiber_length': 9481,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTk0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411499',
                                                                                                               'fiber_length': 8614,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTRFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411498',
                                                                                                               'fiber_length': 9457,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkI4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '5440093',
                                                                                                               'fiber_length': 8828,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '5440092',
                                                                                                               'fiber_length': 8370,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODYxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411495',
                                                                                                               'fiber_length': 7986,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTE1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411498',
                                                                                                               'fiber_length': 8538,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QkY3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411499',
                                                                                                               'fiber_length': 8775,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTVCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '511505',
                                                                                                               'fiber_length': 8911,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzhCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411503',
                                                                                                               'fiber_length': 8469,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQkJBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691},
                                                                                                       {       'connected_time': '411503',
                                                                                                               'fiber_length': 9546,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MUFDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5691}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/13',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/14',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:d6:0a:70'],
                                                                                       'in_nucast_pkts': '13959',
                                                                                       'in_octets': '13239955667',
                                                                                       'in_ucast_pkts': '90565957',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '454831495229',
                                                                                       'out_ucast_pkts': '310897979',
                                                                                       'poller_hash': 'QURUTjIzMTI0MzJFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:07:e0'],
                                                                                       'in_nucast_pkts': '17345',
                                                                                       'in_octets': '7099641883',
                                                                                       'in_ucast_pkts': '60506241',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '149997063454',
                                                                                       'out_ucast_pkts': '111038622',
                                                                                       'poller_hash': 'QURUTjIzMTI0ODQ5IHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '380493',
                                                                                       'in_octets': '117104594',
                                                                                       'in_ucast_pkts': '0',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEM2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:03:f0'],
                                                                                       'in_nucast_pkts': '13741',
                                                                                       'in_octets': '14008554476',
                                                                                       'in_ucast_pkts': '40711560',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '109204659528',
                                                                                       'out_ucast_pkts': '84841729',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTVBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:38:f0'],
                                                                                       'in_nucast_pkts': '13433',
                                                                                       'in_octets': '7408149367',
                                                                                       'in_ucast_pkts': '37311232',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1108',
                                                                                       'out_octets': '127573221452',
                                                                                       'out_ucast_pkts': '92544293',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTUzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:34:50'],
                                                                                       'in_nucast_pkts': '13963',
                                                                                       'in_octets': '8590335792',
                                                                                       'in_ucast_pkts': '49772757',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '279308620557',
                                                                                       'out_ucast_pkts': '191253374',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTg0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:00:f0'],
                                                                                       'in_nucast_pkts': '13738',
                                                                                       'in_octets': '4460347522',
                                                                                       'in_ucast_pkts': '17508885',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '79367287959',
                                                                                       'out_ucast_pkts': '57326821',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTVDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:3f:10'],
                                                                                       'in_nucast_pkts': '17069',
                                                                                       'in_octets': '5898394823',
                                                                                       'in_ucast_pkts': '22128245',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '92783601530',
                                                                                       'out_ucast_pkts': '65414083',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RDk5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:63:f0'],
                                                                                       'in_nucast_pkts': '13737',
                                                                                       'in_octets': '8181387735',
                                                                                       'in_ucast_pkts': '17123621',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '105546938363',
                                                                                       'out_ucast_pkts': '77495536',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjNGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:31:10'],
                                                                                       'in_nucast_pkts': '13740',
                                                                                       'in_octets': '14872244755',
                                                                                       'in_ucast_pkts': '66466167',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '278553882577',
                                                                                       'out_ucast_pkts': '195340506',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjVFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:21:80'],
                                                                                       'in_nucast_pkts': '13738',
                                                                                       'in_octets': '5679942087',
                                                                                       'in_ucast_pkts': '27282218',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '160675401445',
                                                                                       'out_ucast_pkts': '123185468',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTlCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a1:70'],
                                                                                       'in_nucast_pkts': '13771',
                                                                                       'in_octets': '9428401483',
                                                                                       'in_ucast_pkts': '20053837',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '460',
                                                                                       'out_octets': '84235522516',
                                                                                       'out_ucast_pkts': '66454311',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QUNGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:30'],
                                                                                       'in_nucast_pkts': '17095',
                                                                                       'in_octets': '70748521978',
                                                                                       'in_ucast_pkts': '73321371',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '571',
                                                                                       'out_octets': '77868860229',
                                                                                       'out_ucast_pkts': '64782879',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDM3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ef:c0'],
                                                                                       'in_nucast_pkts': '13737',
                                                                                       'in_octets': '4536473801',
                                                                                       'in_ucast_pkts': '13816992',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '72110420027',
                                                                                       'out_ucast_pkts': '51855448',
                                                                                       'poller_hash': 'QURUTjIzMTI0QjdFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:37:30'],
                                                                                       'in_nucast_pkts': '13737',
                                                                                       'in_octets': '13904986517',
                                                                                       'in_ucast_pkts': '44885617',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '182674651329',
                                                                                       'out_ucast_pkts': '136382090',
                                                                                       'poller_hash': 'QURUTjIzMTI0OTY2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b1:30'],
                                                                                       'in_nucast_pkts': '13738',
                                                                                       'in_octets': '6350521169',
                                                                                       'in_ucast_pkts': '9334062',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '459',
                                                                                       'out_octets': '36665106028',
                                                                                       'out_ucast_pkts': '28912537',
                                                                                       'poller_hash': 'QURUTjI0MzYzREI5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ba:f0'],
                                                                                       'in_nucast_pkts': '8180',
                                                                                       'in_octets': '18576547566',
                                                                                       'in_ucast_pkts': '55485289',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '268',
                                                                                       'out_octets': '163409177184',
                                                                                       'out_ucast_pkts': '128401759',
                                                                                       'poller_hash': 'QURUTjI1MTUyMUEyIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/14',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/14',
                                                                                      'id': 'xgspon '
                                                                                            '0/14',
                                                                                      'links': [       {       'connected_time': '411500',
                                                                                                               'fiber_length': 9068,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MzJFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '511506',
                                                                                                               'fiber_length': 9154,
                                                                                                               'poller_hash': 'QURUTjIzMTI0ODQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '411500',
                                                                                                               'fiber_length': 9038,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEM2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '411498',
                                                                                                               'fiber_length': 9303,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTVBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '386516',
                                                                                                               'fiber_length': 9292,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTUzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '411499',
                                                                                                               'fiber_length': 8903,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTg0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '411500',
                                                                                                               'fiber_length': 9318,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTVDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '511505',
                                                                                                               'fiber_length': 9196,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RDk5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '411497',
                                                                                                               'fiber_length': 9156,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjNGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '411497',
                                                                                                               'fiber_length': 9074,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjVFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '411499',
                                                                                                               'fiber_length': 9481,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTlCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '411497',
                                                                                                               'fiber_length': 9284,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QUNGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '511504',
                                                                                                               'fiber_length': 9249,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDM3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '411500',
                                                                                                               'fiber_length': 9207,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QjdFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '411500',
                                                                                                               'fiber_length': 9245,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OTY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '411504',
                                                                                                               'fiber_length': 9079,
                                                                                                               'poller_hash': 'QURUTjI0MzYzREI5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496},
                                                                                                       {       'connected_time': '238122',
                                                                                                               'fiber_length': 9416,
                                                                                                               'poller_hash': 'QURUTjI1MTUyMUEyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4496}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/14',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/15',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:d6:36:60'],
                                                                                       'in_nucast_pkts': '17345',
                                                                                       'in_octets': '27657546519',
                                                                                       'in_ucast_pkts': '50955624',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '136880871237',
                                                                                       'out_ucast_pkts': '103266280',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8d:b0'],
                                                                                       'in_nucast_pkts': '17781',
                                                                                       'in_octets': '1786217507',
                                                                                       'in_ucast_pkts': '3854069',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '28407662258',
                                                                                       'out_ucast_pkts': '19842664',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzE0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:91:80'],
                                                                                       'in_nucast_pkts': '17349',
                                                                                       'in_octets': '505527167',
                                                                                       'in_ucast_pkts': '748493',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '1389952277',
                                                                                       'out_ucast_pkts': '1338405',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzMzIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/15',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/15',
                                                                                      'id': 'xgspon '
                                                                                            '0/15',
                                                                                      'links': [       {       'connected_time': '511506',
                                                                                                               'fiber_length': 1195,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.354},
                                                                                                       {       'connected_time': '511505',
                                                                                                               'fiber_length': 780,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.354},
                                                                                                       {       'connected_time': '511505',
                                                                                                               'fiber_length': 539,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzMzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.354}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/15',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/16',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/16',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNg==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/16',
                                                                                      'id': 'xgspon '
                                                                                            '0/16',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/16',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/2',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dc:71:b0'],
                                                                                       'in_nucast_pkts': '17468',
                                                                                       'in_octets': '1991337600',
                                                                                       'in_ucast_pkts': '19911217',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '59145047964',
                                                                                       'out_ucast_pkts': '40137087',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDQxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:5d:e0'],
                                                                                       'in_nucast_pkts': '17340',
                                                                                       'in_octets': '1758382852',
                                                                                       'in_ucast_pkts': '6919707',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '31194916904',
                                                                                       'out_ucast_pkts': '22510192',
                                                                                       'poller_hash': 'QURUTjIzMTI0MEIwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7e:90'],
                                                                                       'in_nucast_pkts': '17754',
                                                                                       'in_octets': '11748288735',
                                                                                       'in_ucast_pkts': '59006709',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '158629248942',
                                                                                       'out_ucast_pkts': '130598930',
                                                                                       'poller_hash': 'QURUTjIzMTI0REQ5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ad:c0'],
                                                                                       'in_nucast_pkts': '17128',
                                                                                       'in_octets': '12164965242',
                                                                                       'in_ucast_pkts': '54352699',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '173894020013',
                                                                                       'out_ucast_pkts': '132678844',
                                                                                       'poller_hash': 'QURUTjIzMTI0MUQxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:78:c0'],
                                                                                       'in_nucast_pkts': '17346',
                                                                                       'in_octets': '8328880127',
                                                                                       'in_ucast_pkts': '43387796',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '148958989222',
                                                                                       'out_ucast_pkts': '105207087',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjNDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:70'],
                                                                                       'in_nucast_pkts': '17492',
                                                                                       'in_octets': '9017069532',
                                                                                       'in_ucast_pkts': '23865459',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '73342838858',
                                                                                       'out_ucast_pkts': '61095120',
                                                                                       'poller_hash': 'QURUTjIzMTI0REZDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:75:f0'],
                                                                                       'in_nucast_pkts': '17785',
                                                                                       'in_octets': '5986842931',
                                                                                       'in_ucast_pkts': '23664046',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1466',
                                                                                       'out_octets': '91673181754',
                                                                                       'out_ucast_pkts': '66103257',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:91:b0'],
                                                                                       'in_nucast_pkts': '17348',
                                                                                       'in_octets': '15904022055',
                                                                                       'in_ucast_pkts': '42189853',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '114430149310',
                                                                                       'out_ucast_pkts': '95211145',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTQ2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fa:80'],
                                                                                       'in_nucast_pkts': '77249',
                                                                                       'in_octets': '50957718414',
                                                                                       'in_ucast_pkts': '319700124',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '1204346775941',
                                                                                       'out_ucast_pkts': '845022783',
                                                                                       'poller_hash': 'QURUTjIzMTI0REI3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6c:f0'],
                                                                                       'in_nucast_pkts': '77324',
                                                                                       'in_octets': '195543612937',
                                                                                       'in_ucast_pkts': '871541381',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '3617072709960',
                                                                                       'out_ucast_pkts': '2675239709',
                                                                                       'poller_hash': 'QURUTjIzMTI0OEE4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:f0'],
                                                                                       'in_nucast_pkts': '78522',
                                                                                       'in_octets': '29883348983',
                                                                                       'in_ucast_pkts': '165609852',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6648',
                                                                                       'out_octets': '593970879853',
                                                                                       'out_ucast_pkts': '422689492',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDlFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:00:60'],
                                                                                       'in_nucast_pkts': '80518',
                                                                                       'in_octets': '21490240212',
                                                                                       'in_ucast_pkts': '132756443',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6655',
                                                                                       'out_octets': '502414832140',
                                                                                       'out_ucast_pkts': '358664556',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0b:a0'],
                                                                                       'in_nucast_pkts': '17335',
                                                                                       'in_octets': '5937602802',
                                                                                       'in_ucast_pkts': '26500273',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '98450751115',
                                                                                       'out_ucast_pkts': '73726980',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTFEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:60'],
                                                                                       'in_nucast_pkts': '79948',
                                                                                       'in_octets': '44008802832',
                                                                                       'in_ucast_pkts': '316733739',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6646',
                                                                                       'out_octets': '1441341370416',
                                                                                       'out_ucast_pkts': '988837617',
                                                                                       'poller_hash': 'QURUTjIzMTIzRUJDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:79:f0'],
                                                                                       'in_nucast_pkts': '17423',
                                                                                       'in_octets': '6462624002',
                                                                                       'in_ucast_pkts': '42174636',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '172648586399',
                                                                                       'out_ucast_pkts': '122316571',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTE2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fa:60'],
                                                                                       'in_nucast_pkts': '78016',
                                                                                       'in_octets': '64001662950',
                                                                                       'in_ucast_pkts': '217080517',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '624482317804',
                                                                                       'out_ucast_pkts': '459967946',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTJGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:50:60'],
                                                                                       'in_nucast_pkts': '17490',
                                                                                       'in_octets': '9334372387',
                                                                                       'in_ucast_pkts': '31816728',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '184950235686',
                                                                                       'out_ucast_pkts': '139574894',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEM4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:29:50'],
                                                                                       'in_nucast_pkts': '17342',
                                                                                       'in_octets': '12752149376',
                                                                                       'in_ucast_pkts': '123564297',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '378037658066',
                                                                                       'out_ucast_pkts': '258052921',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzczIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:5b:70'],
                                                                                       'in_nucast_pkts': '17495',
                                                                                       'in_octets': '11301586522',
                                                                                       'in_ucast_pkts': '47241964',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '159092114439',
                                                                                       'out_ucast_pkts': '116924650',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:40'],
                                                                                       'in_nucast_pkts': '17065',
                                                                                       'in_octets': '9987180858',
                                                                                       'in_ucast_pkts': '48716005',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '162591640679',
                                                                                       'out_ucast_pkts': '118790606',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OTE2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:7e:b0'],
                                                                                       'in_nucast_pkts': '17771',
                                                                                       'in_octets': '3369267906',
                                                                                       'in_ucast_pkts': '19106450',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '93231884104',
                                                                                       'out_ucast_pkts': '66152081',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjNBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:36:70'],
                                                                                       'in_nucast_pkts': '17345',
                                                                                       'in_octets': '5263934024',
                                                                                       'in_ucast_pkts': '12460366',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '31009469267',
                                                                                       'out_ucast_pkts': '25092088',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzEwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:89:50'],
                                                                                       'in_nucast_pkts': '17786',
                                                                                       'in_octets': '15221587024',
                                                                                       'in_ucast_pkts': '84353268',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1469',
                                                                                       'out_octets': '395982070421',
                                                                                       'out_ucast_pkts': '292015605',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTJCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b6:30'],
                                                                                       'in_nucast_pkts': '5130',
                                                                                       'in_octets': '831887605',
                                                                                       'in_ucast_pkts': '2077424',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '172',
                                                                                       'out_octets': '7914458678',
                                                                                       'out_ucast_pkts': '6076230',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ad:d0'],
                                                                                       'in_nucast_pkts': '17074',
                                                                                       'in_octets': '41487522417',
                                                                                       'in_ucast_pkts': '46058779',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '86523696291',
                                                                                       'out_ucast_pkts': '70721780',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUM4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:04:00'],
                                                                                       'in_nucast_pkts': '17067',
                                                                                       'in_octets': '5557607180',
                                                                                       'in_ucast_pkts': '33598451',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '102148766689',
                                                                                       'out_ucast_pkts': '72254854',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzE3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:3d:30'],
                                                                                       'in_nucast_pkts': '17069',
                                                                                       'in_octets': '17043278935',
                                                                                       'in_ucast_pkts': '35921917',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '275488666428',
                                                                                       'out_ucast_pkts': '194127485',
                                                                                       'poller_hash': 'QURUTjIzMTI0NTEwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:3e:40'],
                                                                                       'in_nucast_pkts': '17070',
                                                                                       'in_octets': '7687648925',
                                                                                       'in_ucast_pkts': '67999725',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '327100804142',
                                                                                       'out_ucast_pkts': '223208202',
                                                                                       'poller_hash': 'QURUTjI0MzE0NzY2IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/2',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/2',
                                                                                      'id': 'xgspon '
                                                                                            '0/2',
                                                                                      'links': [       {       'connected_time': '511495',
                                                                                                               'fiber_length': 1926,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDQxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511496',
                                                                                                               'fiber_length': 1969,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MEIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511496',
                                                                                                               'fiber_length': 1780,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511490',
                                                                                                               'fiber_length': 2471,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MUQxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511495',
                                                                                                               'fiber_length': 1807,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjNDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511501',
                                                                                                               'fiber_length': 1986,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REZDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -12.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511497',
                                                                                                               'fiber_length': 2396,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511496',
                                                                                                               'fiber_length': 2088,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '2317051',
                                                                                                               'fiber_length': 2058,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REI3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '2317053',
                                                                                                               'fiber_length': 2069,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OEE4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '2317051',
                                                                                                               'fiber_length': 2313,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDlFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '2317052',
                                                                                                               'fiber_length': 2008,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511495',
                                                                                                               'fiber_length': 540,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTFEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '2317053',
                                                                                                               'fiber_length': 2230,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRUJDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511490',
                                                                                                               'fiber_length': 1746,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTE2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '2317051',
                                                                                                               'fiber_length': 1857,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTJGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511500',
                                                                                                               'fiber_length': 1996,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEM4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511494',
                                                                                                               'fiber_length': 233,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzczIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511502',
                                                                                                               'fiber_length': 1780,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -12.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511499',
                                                                                                               'fiber_length': 2012,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OTE2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511502',
                                                                                                               'fiber_length': 545,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjNBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511498',
                                                                                                               'fiber_length': 1734,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzEwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511493',
                                                                                                               'fiber_length': 324,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTJCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '153366',
                                                                                                               'fiber_length': 700,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511498',
                                                                                                               'fiber_length': 489,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUM4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511498',
                                                                                                               'fiber_length': 716,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzE3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511500',
                                                                                                               'fiber_length': 313,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NTEwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -12.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737},
                                                                                                       {       'connected_time': '511505',
                                                                                                               'fiber_length': 859,
                                                                                                               'poller_hash': 'QURUTjI0MzE0NzY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2737}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/2',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/3',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dc:83:00'],
                                                                                       'in_nucast_pkts': '185143',
                                                                                       'in_octets': '114224840443',
                                                                                       'in_ucast_pkts': '284025037',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15627',
                                                                                       'out_octets': '1198673274406',
                                                                                       'out_ucast_pkts': '937808721',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTQ1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:36:e0'],
                                                                                       'in_nucast_pkts': '134175',
                                                                                       'in_octets': '41028323456',
                                                                                       'in_ucast_pkts': '85142587',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '11217',
                                                                                       'out_octets': '362600897682',
                                                                                       'out_ucast_pkts': '269818780',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTQ5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:a0'],
                                                                                       'in_nucast_pkts': '64552',
                                                                                       'in_octets': '100324784568',
                                                                                       'in_ucast_pkts': '420812486',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2100',
                                                                                       'out_octets': '1288077963545',
                                                                                       'out_ucast_pkts': '932948319',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0Y5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3b:b0'],
                                                                                       'in_nucast_pkts': '184373',
                                                                                       'in_octets': '23546687807',
                                                                                       'in_ucast_pkts': '147610870',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15621',
                                                                                       'out_octets': '718037747535',
                                                                                       'out_ucast_pkts': '499348896',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NjgxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0c:50'],
                                                                                       'in_nucast_pkts': '26506',
                                                                                       'in_octets': '17636558073',
                                                                                       'in_ucast_pkts': '103017701',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2241',
                                                                                       'out_octets': '413594199234',
                                                                                       'out_ucast_pkts': '300633066',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODlDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:38:50'],
                                                                                       'in_nucast_pkts': '184380',
                                                                                       'in_octets': '106130690715',
                                                                                       'in_ucast_pkts': '366861213',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15628',
                                                                                       'out_octets': '1545613386456',
                                                                                       'out_ucast_pkts': '1120007761',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OEJBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6e:e0'],
                                                                                       'in_nucast_pkts': '184524',
                                                                                       'in_octets': '760563575203',
                                                                                       'in_ucast_pkts': '1084810059',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6045',
                                                                                       'out_octets': '2715739030109',
                                                                                       'out_ucast_pkts': '2051082644',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODlEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0a:f0'],
                                                                                       'in_nucast_pkts': '184391',
                                                                                       'in_octets': '120634866501',
                                                                                       'in_ucast_pkts': '753224717',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15614',
                                                                                       'out_octets': '1762704597089',
                                                                                       'out_ucast_pkts': '1355324479',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODgzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:52:20'],
                                                                                       'in_nucast_pkts': '184406',
                                                                                       'in_octets': '29353683826',
                                                                                       'in_ucast_pkts': '228444142',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15623',
                                                                                       'out_octets': '706729102780',
                                                                                       'out_ucast_pkts': '500738346',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzA3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:c0'],
                                                                                       'in_nucast_pkts': '184449',
                                                                                       'in_octets': '40055826188',
                                                                                       'in_ucast_pkts': '113244758',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15617',
                                                                                       'out_octets': '301307270087',
                                                                                       'out_ucast_pkts': '261636539',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzAyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:10'],
                                                                                       'in_nucast_pkts': '181365',
                                                                                       'in_octets': '59332714181',
                                                                                       'in_ucast_pkts': '103988355',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6046',
                                                                                       'out_octets': '288012885834',
                                                                                       'out_ucast_pkts': '237523808',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0JEIHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '0',
                                                                                       'in_octets': '0',
                                                                                       'in_ucast_pkts': '0',
                                                                                       'oper_up': False,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkNDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:03:a0'],
                                                                                       'in_nucast_pkts': '184400',
                                                                                       'in_octets': '40388633921',
                                                                                       'in_ucast_pkts': '169099413',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15615',
                                                                                       'out_octets': '901890785599',
                                                                                       'out_ucast_pkts': '640995591',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0FFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0b:70'],
                                                                                       'in_nucast_pkts': '52015',
                                                                                       'in_octets': '1710802939',
                                                                                       'in_ucast_pkts': '2856140',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4401',
                                                                                       'out_octets': '3567786663',
                                                                                       'out_ucast_pkts': '3617595',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OEZFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3a:70'],
                                                                                       'in_nucast_pkts': '34680',
                                                                                       'in_octets': '10840280335',
                                                                                       'in_ucast_pkts': '42618547',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2932',
                                                                                       'out_octets': '137901116493',
                                                                                       'out_ucast_pkts': '106451556',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkE1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:83:50'],
                                                                                       'in_nucast_pkts': '165877',
                                                                                       'in_octets': '94609140575',
                                                                                       'in_ucast_pkts': '332441703',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '13938',
                                                                                       'out_octets': '1245110213602',
                                                                                       'out_ucast_pkts': '928888614',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTRBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:4a:f0'],
                                                                                       'in_nucast_pkts': '89585',
                                                                                       'in_octets': '44680359865',
                                                                                       'in_ucast_pkts': '158497249',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2988',
                                                                                       'out_octets': '563961505377',
                                                                                       'out_ucast_pkts': '406261121',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUZGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:38:00'],
                                                                                       'in_nucast_pkts': '78181',
                                                                                       'in_octets': '27899552121',
                                                                                       'in_ucast_pkts': '54651869',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6617',
                                                                                       'out_octets': '90806962833',
                                                                                       'out_ucast_pkts': '86369262',
                                                                                       'poller_hash': 'QURUTjIzMTI0QTE4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '36:3a:c5:c7:0e:34',
                                                                                                                        'cc:66:18:50:b4:20'],
                                                                                       'in_nucast_pkts': '51748',
                                                                                       'in_octets': '449522113442',
                                                                                       'in_ucast_pkts': '526457874',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1725',
                                                                                       'out_octets': '848622428896',
                                                                                       'out_ucast_pkts': '737635649',
                                                                                       'poller_hash': 'QURUTjIzMTI0QTc2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:43:e0'],
                                                                                       'in_nucast_pkts': '181510',
                                                                                       'in_octets': '112941005951',
                                                                                       'in_ucast_pkts': '351102468',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6045',
                                                                                       'out_octets': '946005969035',
                                                                                       'out_ucast_pkts': '699659776',
                                                                                       'poller_hash': 'QURUTjIzMTI0QjNGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:38:30'],
                                                                                       'in_nucast_pkts': '29660',
                                                                                       'in_octets': '92826400199',
                                                                                       'in_ucast_pkts': '188162824',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2513',
                                                                                       'out_octets': '453129660973',
                                                                                       'out_ucast_pkts': '337137185',
                                                                                       'poller_hash': 'QURUTjIzMTI0QkE1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:62:90'],
                                                                                       'in_nucast_pkts': '182366',
                                                                                       'in_octets': '200222667955',
                                                                                       'in_ucast_pkts': '1068086035',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6047',
                                                                                       'out_octets': '5909796819926',
                                                                                       'out_ucast_pkts': '4494340833',
                                                                                       'poller_hash': 'QURUTjI0MzYzQ0M2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8c:80'],
                                                                                       'in_nucast_pkts': '53768',
                                                                                       'in_octets': '90783772980',
                                                                                       'in_ucast_pkts': '337362943',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4457',
                                                                                       'out_octets': '888976983721',
                                                                                       'out_ucast_pkts': '643600902',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OEI4IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/3',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/3',
                                                                                      'id': 'xgspon '
                                                                                            '0/3',
                                                                                      'links': [       {       'connected_time': '5440096',
                                                                                                               'fiber_length': 13527,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTQ1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '3910768',
                                                                                                               'fiber_length': 13074,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '1889034',
                                                                                                               'fiber_length': 14757,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0Y5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '5440063',
                                                                                                               'fiber_length': 14954,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NjgxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '781465',
                                                                                                               'fiber_length': 14487,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODlDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '5440092',
                                                                                                               'fiber_length': 14745,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OEJBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '5440093',
                                                                                                               'fiber_length': 14466,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODlEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '5440093',
                                                                                                               'fiber_length': 14826,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODgzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '5440095',
                                                                                                               'fiber_length': 14521,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzA3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '5440096',
                                                                                                               'fiber_length': 14662,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzAyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '5440094',
                                                                                                               'fiber_length': 13218,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0JEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '5440065',
                                                                                                               'fiber_length': 12154,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '5440096',
                                                                                                               'fiber_length': 13016,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '1534721',
                                                                                                               'fiber_length': 14823,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OEZFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '1022696',
                                                                                                               'fiber_length': 14178,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkE1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '4857791',
                                                                                                               'fiber_length': 14481,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTRBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '2684908',
                                                                                                               'fiber_length': 14568,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -26.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '2312767',
                                                                                                               'fiber_length': 14439,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QTE4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '1551603',
                                                                                                               'fiber_length': 12765,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QTc2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '5440065',
                                                                                                               'fiber_length': 13633,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QjNGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '873712',
                                                                                                               'fiber_length': 14906,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QkE1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '5440065',
                                                                                                               'fiber_length': 13230,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQ0M2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471},
                                                                                                       {       'connected_time': '1553615',
                                                                                                               'fiber_length': 13206,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OEI4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5471}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/3',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/4',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dc:56:70'],
                                                                                       'in_nucast_pkts': '80464',
                                                                                       'in_octets': '64191871858',
                                                                                       'in_ucast_pkts': '294720133',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6629',
                                                                                       'out_octets': '1307641974656',
                                                                                       'out_ucast_pkts': '937844307',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTA3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:20'],
                                                                                       'in_nucast_pkts': '14661',
                                                                                       'in_octets': '6991765314',
                                                                                       'in_ucast_pkts': '30718708',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1210',
                                                                                       'out_octets': '110390447582',
                                                                                       'out_ucast_pkts': '78915022',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTE0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:69:60'],
                                                                                       'in_nucast_pkts': '66683',
                                                                                       'in_octets': '31690461601',
                                                                                       'in_ucast_pkts': '130691656',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5640',
                                                                                       'out_octets': '376612482296',
                                                                                       'out_ucast_pkts': '336192464',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTYwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:57:60'],
                                                                                       'in_nucast_pkts': '8874',
                                                                                       'in_octets': '1105351558',
                                                                                       'in_ucast_pkts': '5692128',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '750',
                                                                                       'out_octets': '33797115840',
                                                                                       'out_ucast_pkts': '24223033',
                                                                                       'poller_hash': 'QURUTjIzMTI0REREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fa:30'],
                                                                                       'in_nucast_pkts': '77250',
                                                                                       'in_octets': '31411288346',
                                                                                       'in_ucast_pkts': '99595628',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2567',
                                                                                       'out_octets': '402318506803',
                                                                                       'out_ucast_pkts': '294620515',
                                                                                       'poller_hash': 'QURUTjIzMTI0NUM2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:7a:e0'],
                                                                                       'in_nucast_pkts': '112203',
                                                                                       'in_octets': '178265876806',
                                                                                       'in_ucast_pkts': '377896222',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3734',
                                                                                       'out_octets': '974919567981',
                                                                                       'out_ucast_pkts': '749790248',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDdEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:76:60'],
                                                                                       'in_nucast_pkts': '80462',
                                                                                       'in_octets': '34208278773',
                                                                                       'in_ucast_pkts': '133402787',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6631',
                                                                                       'out_octets': '379173521004',
                                                                                       'out_ucast_pkts': '272048012',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkFEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:75:51'],
                                                                                       'in_nucast_pkts': '1343',
                                                                                       'in_octets': '30099881929',
                                                                                       'in_ucast_pkts': '135547910',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6629',
                                                                                       'out_octets': '689845211523',
                                                                                       'out_ucast_pkts': '493316076',
                                                                                       'poller_hash': 'QURUTjIzMTI0Rjk5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ab:20'],
                                                                                       'in_nucast_pkts': '62088',
                                                                                       'in_octets': '69782859508',
                                                                                       'in_ucast_pkts': '222531899',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2020',
                                                                                       'out_octets': '560038723744',
                                                                                       'out_ucast_pkts': '407371035',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjA2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:78:a0'],
                                                                                       'in_nucast_pkts': '78543',
                                                                                       'in_octets': '16544493187',
                                                                                       'in_ucast_pkts': '103247590',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6628',
                                                                                       'out_octets': '277598045489',
                                                                                       'out_ucast_pkts': '202126901',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTdEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b5:50'],
                                                                                       'in_nucast_pkts': '77259',
                                                                                       'in_octets': '30947372870',
                                                                                       'in_ucast_pkts': '161571631',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2570',
                                                                                       'out_octets': '942520427841',
                                                                                       'out_ucast_pkts': '685844825',
                                                                                       'poller_hash': 'QURUTjIzMTI0REU4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:93:80'],
                                                                                       'in_nucast_pkts': '78132',
                                                                                       'in_octets': '37564494279',
                                                                                       'in_ucast_pkts': '212070619',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6587',
                                                                                       'out_octets': '705153503921',
                                                                                       'out_ucast_pkts': '503870012',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUVFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:8a:10'],
                                                                                       'in_nucast_pkts': '23934',
                                                                                       'in_octets': '12259358305',
                                                                                       'in_ucast_pkts': '65309438',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1979',
                                                                                       'out_octets': '238371002118',
                                                                                       'out_ucast_pkts': '171853941',
                                                                                       'poller_hash': 'QURUTjIzMTI0RUE1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b7:70'],
                                                                                       'in_nucast_pkts': '77254',
                                                                                       'in_octets': '9552622113',
                                                                                       'in_ucast_pkts': '30258842',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2574',
                                                                                       'out_octets': '85272096163',
                                                                                       'out_ucast_pkts': '69784441',
                                                                                       'poller_hash': 'QURUTjIzMTIzRTlFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:09:30'],
                                                                                       'in_nucast_pkts': '17430',
                                                                                       'in_octets': '19898956699',
                                                                                       'in_ucast_pkts': '131222900',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '357137099261',
                                                                                       'out_ucast_pkts': '244740097',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTI5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6a:e0'],
                                                                                       'in_nucast_pkts': '79067',
                                                                                       'in_octets': '26256197341',
                                                                                       'in_ucast_pkts': '85186117',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6630',
                                                                                       'out_octets': '417679382577',
                                                                                       'out_ucast_pkts': '303843932',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTI2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:50'],
                                                                                       'in_nucast_pkts': '77253',
                                                                                       'in_octets': '28178292115',
                                                                                       'in_ucast_pkts': '152473006',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2570',
                                                                                       'out_octets': '693591396875',
                                                                                       'out_ucast_pkts': '490410131',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTUzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:37:60'],
                                                                                       'in_nucast_pkts': '78525',
                                                                                       'in_octets': '25125301558',
                                                                                       'in_ucast_pkts': '150490446',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6626',
                                                                                       'out_octets': '539450720535',
                                                                                       'out_ucast_pkts': '375595976',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUFBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:24:e0'],
                                                                                       'in_nucast_pkts': '43114',
                                                                                       'in_octets': '4458164244',
                                                                                       'in_ucast_pkts': '13682224',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1417',
                                                                                       'out_octets': '39806039770',
                                                                                       'out_ucast_pkts': '28911659',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTRGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:50:b0'],
                                                                                       'in_nucast_pkts': '77255',
                                                                                       'in_octets': '19655585764',
                                                                                       'in_ucast_pkts': '71781369',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2572',
                                                                                       'out_octets': '275176434769',
                                                                                       'out_ucast_pkts': '202804538',
                                                                                       'poller_hash': 'QURUTjIzMTI0MUJGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:37:81'],
                                                                                       'in_nucast_pkts': '308',
                                                                                       'in_octets': '6353151082',
                                                                                       'in_ucast_pkts': '36858175',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1466',
                                                                                       'out_octets': '300504350152',
                                                                                       'out_ucast_pkts': '215317430',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkRDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ab:00'],
                                                                                       'in_nucast_pkts': '43675',
                                                                                       'in_octets': '35810310413',
                                                                                       'in_ucast_pkts': '103057074',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1455',
                                                                                       'out_octets': '440061119703',
                                                                                       'out_ucast_pkts': '316118515',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0RBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:84:51'],
                                                                                       'in_nucast_pkts': '640',
                                                                                       'in_octets': '20128057580',
                                                                                       'in_ucast_pkts': '58996755',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3021',
                                                                                       'out_octets': '223867537482',
                                                                                       'out_ucast_pkts': '167630986',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzM1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:01:10'],
                                                                                       'in_nucast_pkts': '77257',
                                                                                       'in_octets': '37842338820',
                                                                                       'in_ucast_pkts': '103822789',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2573',
                                                                                       'out_octets': '392270827975',
                                                                                       'out_ucast_pkts': '296845783',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QUIwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:81:70'],
                                                                                       'in_nucast_pkts': '77255',
                                                                                       'in_octets': '80112491001',
                                                                                       'in_ucast_pkts': '170775637',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2573',
                                                                                       'out_octets': '569526737669',
                                                                                       'out_ucast_pkts': '427832231',
                                                                                       'poller_hash': 'QURUTjIzMTI0QzU1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:a8:60'],
                                                                                       'in_nucast_pkts': '78535',
                                                                                       'in_octets': '48505866974',
                                                                                       'in_ucast_pkts': '136961601',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6626',
                                                                                       'out_octets': '478541428408',
                                                                                       'out_ucast_pkts': '341019414',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUUzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a3:a0'],
                                                                                       'in_nucast_pkts': '69483',
                                                                                       'in_octets': '9445157701',
                                                                                       'in_ucast_pkts': '80859241',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2316',
                                                                                       'out_octets': '255804624224',
                                                                                       'out_ucast_pkts': '177152668',
                                                                                       'poller_hash': 'QURUTjIzMTI0NjBBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ab:a0'],
                                                                                       'in_nucast_pkts': '77256',
                                                                                       'in_octets': '8706303577',
                                                                                       'in_ucast_pkts': '42490684',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2571',
                                                                                       'out_octets': '215562903845',
                                                                                       'out_ucast_pkts': '162529006',
                                                                                       'poller_hash': 'QURUTjI0MzYzQUU3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:25:f0'],
                                                                                       'in_nucast_pkts': '77256',
                                                                                       'in_octets': '17543890243',
                                                                                       'in_ucast_pkts': '16181738',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2570',
                                                                                       'out_octets': '20443373744',
                                                                                       'out_ucast_pkts': '19280419',
                                                                                       'poller_hash': 'QURUTjI0Mzg5NkY5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:2e:00'],
                                                                                       'in_nucast_pkts': '56791',
                                                                                       'in_octets': '36629619784',
                                                                                       'in_ucast_pkts': '86480359',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1897',
                                                                                       'out_octets': '570731716285',
                                                                                       'out_ucast_pkts': '410052208',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzQ5IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/4',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/4',
                                                                                      'id': 'xgspon '
                                                                                            '0/4',
                                                                                      'links': [       {       'connected_time': '2317053',
                                                                                                               'fiber_length': 3992,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTA3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '421798',
                                                                                                               'fiber_length': 4123,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '1967097',
                                                                                                               'fiber_length': 4527,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTYwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '261424',
                                                                                                               'fiber_length': 3424,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317059',
                                                                                                               'fiber_length': 3878,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NUM2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '3363454',
                                                                                                               'fiber_length': 4613,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDdEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317055',
                                                                                                               'fiber_length': 3684,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkFEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317057',
                                                                                                               'fiber_length': 3674,
                                                                                                               'poller_hash': 'QURUTjIzMTI0Rjk5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '1816875',
                                                                                                               'fiber_length': 4267,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjA2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317056',
                                                                                                               'fiber_length': 3861,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTdEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317057',
                                                                                                               'fiber_length': 3367,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REU4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317050',
                                                                                                               'fiber_length': 4569,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUVFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '688772',
                                                                                                               'fiber_length': 3857,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RUE1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317054',
                                                                                                               'fiber_length': 4488,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRTlFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '511507',
                                                                                                               'fiber_length': 4696,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTI5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317056',
                                                                                                               'fiber_length': 4079,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTI2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317051',
                                                                                                               'fiber_length': 3719,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTUzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317055',
                                                                                                               'fiber_length': 4022,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUFBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '1293152',
                                                                                                               'fiber_length': 3775,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTRGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -24.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317058',
                                                                                                               'fiber_length': 4059,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MUJGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 4481,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkRDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '1309470',
                                                                                                               'fiber_length': 3596,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0RBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '1053537',
                                                                                                               'fiber_length': 4136,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzM1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317047',
                                                                                                               'fiber_length': 4574,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QUIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317058',
                                                                                                               'fiber_length': 4209,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QzU1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317048',
                                                                                                               'fiber_length': 4287,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUUzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2083695',
                                                                                                               'fiber_length': 4437,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NjBBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317066',
                                                                                                               'fiber_length': 3543,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQUU3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '2317062',
                                                                                                               'fiber_length': 3918,
                                                                                                               'poller_hash': 'QURUTjI0Mzg5NkY5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769},
                                                                                                       {       'connected_time': '1702391',
                                                                                                               'fiber_length': 4116,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3769}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/4',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/5',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dc:7f:40'],
                                                                                       'in_nucast_pkts': '78535',
                                                                                       'in_octets': '32074553609',
                                                                                       'in_ucast_pkts': '247138061',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6651',
                                                                                       'out_octets': '851716345889',
                                                                                       'out_ucast_pkts': '606755361',
                                                                                       'poller_hash': 'QURUTjIzMTI0REYxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:56:90'],
                                                                                       'in_nucast_pkts': '78530',
                                                                                       'in_octets': '9573246216',
                                                                                       'in_ucast_pkts': '28968827',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6646',
                                                                                       'out_octets': '94670181667',
                                                                                       'out_ucast_pkts': '79312527',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDQzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:71:00'],
                                                                                       'in_nucast_pkts': '79111',
                                                                                       'in_octets': '35790774369',
                                                                                       'in_ucast_pkts': '168984150',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '621131891083',
                                                                                       'out_ucast_pkts': '448566543',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUUyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:93:30'],
                                                                                       'in_nucast_pkts': '37776',
                                                                                       'in_octets': '17380293068',
                                                                                       'in_ucast_pkts': '88982916',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3197',
                                                                                       'out_octets': '493923471987',
                                                                                       'out_ucast_pkts': '350348488',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTI2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:92:20'],
                                                                                       'in_nucast_pkts': '78535',
                                                                                       'in_octets': '88158106536',
                                                                                       'in_ucast_pkts': '371249138',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6646',
                                                                                       'out_octets': '1100356693456',
                                                                                       'out_ucast_pkts': '806203412',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDk4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:78:60'],
                                                                                       'in_nucast_pkts': '78529',
                                                                                       'in_octets': '28476675251',
                                                                                       'in_ucast_pkts': '142287961',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6647',
                                                                                       'out_octets': '953590409355',
                                                                                       'out_ucast_pkts': '703346325',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:c0'],
                                                                                       'in_nucast_pkts': '78632',
                                                                                       'in_octets': '2863545110',
                                                                                       'in_ucast_pkts': '10210777',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6649',
                                                                                       'out_octets': '36091222902',
                                                                                       'out_ucast_pkts': '26579330',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDM4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:72:20'],
                                                                                       'in_nucast_pkts': '80130',
                                                                                       'in_octets': '42976698193',
                                                                                       'in_ucast_pkts': '299332771',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6651',
                                                                                       'out_octets': '1255152815589',
                                                                                       'out_ucast_pkts': '901031913',
                                                                                       'poller_hash': 'QURUTjIzMTI0OTAwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:f0'],
                                                                                       'in_nucast_pkts': '29041',
                                                                                       'in_octets': '88986655438',
                                                                                       'in_ucast_pkts': '119673652',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2456',
                                                                                       'out_octets': '271763618943',
                                                                                       'out_ucast_pkts': '249838064',
                                                                                       'poller_hash': 'QURUTjIzMTI0REE2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:73:e0'],
                                                                                       'in_nucast_pkts': '37777',
                                                                                       'in_octets': '2961602648',
                                                                                       'in_ucast_pkts': '26561230',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3197',
                                                                                       'out_octets': '82802554675',
                                                                                       'out_ucast_pkts': '56265867',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTE2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:81:10'],
                                                                                       'in_nucast_pkts': '78535',
                                                                                       'in_octets': '125019801865',
                                                                                       'in_ucast_pkts': '590023136',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6649',
                                                                                       'out_octets': '1927420214171',
                                                                                       'out_ucast_pkts': '1436718789',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjJCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:5c:00'],
                                                                                       'in_nucast_pkts': '51160',
                                                                                       'in_octets': '54907922403',
                                                                                       'in_ucast_pkts': '175360064',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1706',
                                                                                       'out_octets': '621698507690',
                                                                                       'out_ucast_pkts': '454730480',
                                                                                       'poller_hash': 'QURUTjIzMTIzRjJCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:75:a0'],
                                                                                       'in_nucast_pkts': '37809',
                                                                                       'in_octets': '8907565649',
                                                                                       'in_ucast_pkts': '69442996',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3197',
                                                                                       'out_octets': '174818309953',
                                                                                       'out_ucast_pkts': '127345107',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTBEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:af:00'],
                                                                                       'in_nucast_pkts': '77271',
                                                                                       'in_octets': '29472585908',
                                                                                       'in_ucast_pkts': '59851576',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '187663777776',
                                                                                       'out_ucast_pkts': '152376578',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTI1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7f:80'],
                                                                                       'in_nucast_pkts': '61715',
                                                                                       'in_octets': '94627388922',
                                                                                       'in_ucast_pkts': '167676867',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5228',
                                                                                       'out_octets': '406432771728',
                                                                                       'out_ucast_pkts': '346990952',
                                                                                       'poller_hash': 'QURUTjIzMTI0REY3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8d:c0'],
                                                                                       'in_nucast_pkts': '78533',
                                                                                       'in_octets': '25244124422',
                                                                                       'in_ucast_pkts': '77906098',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6647',
                                                                                       'out_octets': '201047220883',
                                                                                       'out_ucast_pkts': '154873044',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUIwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:83:00'],
                                                                                       'in_nucast_pkts': '80396',
                                                                                       'in_octets': '90723615998',
                                                                                       'in_ucast_pkts': '231116455',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6647',
                                                                                       'out_octets': '770911853601',
                                                                                       'out_ucast_pkts': '568332465',
                                                                                       'poller_hash': 'QURUTjIzMTI0Rjc0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3a:f0'],
                                                                                       'in_nucast_pkts': '78533',
                                                                                       'in_octets': '66612886069',
                                                                                       'in_ucast_pkts': '366803880',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6651',
                                                                                       'out_octets': '1174174610445',
                                                                                       'out_ucast_pkts': '859824473',
                                                                                       'poller_hash': 'QURUTjIzMTI0NEVCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:50:e0'],
                                                                                       'in_nucast_pkts': '77274',
                                                                                       'in_octets': '19847184105',
                                                                                       'in_ucast_pkts': '60122034',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '187773959886',
                                                                                       'out_ucast_pkts': '141393438',
                                                                                       'poller_hash': 'QURUTjIzMTI0MDk5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:71:10'],
                                                                                       'in_nucast_pkts': '78536',
                                                                                       'in_octets': '30883850198',
                                                                                       'in_ucast_pkts': '148123087',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6651',
                                                                                       'out_octets': '425517749953',
                                                                                       'out_ucast_pkts': '389834749',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUZEIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/5',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/5',
                                                                                      'id': 'xgspon '
                                                                                            '0/5',
                                                                                      'links': [       {       'connected_time': '2317053',
                                                                                                               'fiber_length': 5944,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REYxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317055',
                                                                                                               'fiber_length': 6063,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDQzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317054',
                                                                                                               'fiber_length': 5212,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUUyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '1114341',
                                                                                                               'fiber_length': 7531,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTI2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317056',
                                                                                                               'fiber_length': 5967,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDk4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317060',
                                                                                                               'fiber_length': 5915,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317054',
                                                                                                               'fiber_length': 6210,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDM4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317056',
                                                                                                               'fiber_length': 6197,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OTAwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '856063',
                                                                                                               'fiber_length': 5510,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REE2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '1114341',
                                                                                                               'fiber_length': 7481,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTE2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317057',
                                                                                                               'fiber_length': 6120,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjJCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '1534011',
                                                                                                               'fiber_length': 5584,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRjJCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '1114341',
                                                                                                               'fiber_length': 7351,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTBEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317058',
                                                                                                               'fiber_length': 5209,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTI1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '1820396',
                                                                                                               'fiber_length': 8168,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REY3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317055',
                                                                                                               'fiber_length': 5245,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317057',
                                                                                                               'fiber_length': 6325,
                                                                                                               'poller_hash': 'QURUTjIzMTI0Rjc0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317059',
                                                                                                               'fiber_length': 8586,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NEVCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317059',
                                                                                                               'fiber_length': 6279,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MDk5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097},
                                                                                                       {       'connected_time': '2317058',
                                                                                                               'fiber_length': 6232,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1097}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/5',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/6',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:d6:08:e0'],
                                                                                       'in_nucast_pkts': '78538',
                                                                                       'in_octets': '16840309678',
                                                                                       'in_ucast_pkts': '96638856',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6651',
                                                                                       'out_octets': '427069282997',
                                                                                       'out_ucast_pkts': '315618342',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTJBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:8d:d0'],
                                                                                       'in_nucast_pkts': '77453',
                                                                                       'in_octets': '178991664099',
                                                                                       'in_ucast_pkts': '614932667',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2574',
                                                                                       'out_octets': '1876167151735',
                                                                                       'out_ucast_pkts': '1340522307',
                                                                                       'poller_hash': 'QURUTjIzMTI0RUE0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0c:10'],
                                                                                       'in_nucast_pkts': '168735',
                                                                                       'in_octets': '100638156252',
                                                                                       'in_ucast_pkts': '616564644',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '14159',
                                                                                       'out_octets': '973818902633',
                                                                                       'out_ucast_pkts': '937248545',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkY1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:7d:e0'],
                                                                                       'in_nucast_pkts': '78531',
                                                                                       'in_octets': '11540874941',
                                                                                       'in_ucast_pkts': '73015080',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6647',
                                                                                       'out_octets': '296399143311',
                                                                                       'out_ucast_pkts': '208464895',
                                                                                       'poller_hash': 'QURUTjIzMTIzRjE4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b8:50'],
                                                                                       'in_nucast_pkts': '79176',
                                                                                       'in_octets': '37525762453',
                                                                                       'in_ucast_pkts': '286489395',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '897851171839',
                                                                                       'out_ucast_pkts': '626176397',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTk1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7b:30'],
                                                                                       'in_nucast_pkts': '80463',
                                                                                       'in_octets': '123766625329',
                                                                                       'in_ucast_pkts': '306487604',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6653',
                                                                                       'out_octets': '1029083777933',
                                                                                       'out_ucast_pkts': '824202792',
                                                                                       'poller_hash': 'QURUTjIzMTI1MENDIHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '4149318',
                                                                                       'in_octets': '1218507587',
                                                                                       'in_ucast_pkts': '1097786',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTExIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6e:30'],
                                                                                       'in_nucast_pkts': '181362',
                                                                                       'in_octets': '117476837393',
                                                                                       'in_ucast_pkts': '782362731',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6046',
                                                                                       'out_octets': '2690641132374',
                                                                                       'out_ucast_pkts': '1905365208',
                                                                                       'poller_hash': 'QURUTjIzMTI1MjA2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:b3:60'],
                                                                                       'in_nucast_pkts': '77257',
                                                                                       'in_octets': '25913101840',
                                                                                       'in_ucast_pkts': '195827682',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '538979312501',
                                                                                       'out_ucast_pkts': '388999274',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:63:40'],
                                                                                       'in_nucast_pkts': '77257',
                                                                                       'in_octets': '19421389746',
                                                                                       'in_ucast_pkts': '134022591',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '396004731028',
                                                                                       'out_ucast_pkts': '273905695',
                                                                                       'poller_hash': 'QURUTjIzMTI1MENEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:35:20'],
                                                                                       'in_nucast_pkts': '78535',
                                                                                       'in_octets': '22993023654',
                                                                                       'in_ucast_pkts': '119087151',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6649',
                                                                                       'out_octets': '578989661367',
                                                                                       'out_ucast_pkts': '410335584',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTRCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a9:70'],
                                                                                       'in_nucast_pkts': '77273',
                                                                                       'in_octets': '20153608281',
                                                                                       'in_ucast_pkts': '54448909',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '171013626473',
                                                                                       'out_ucast_pkts': '133959810',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkEzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:23:70'],
                                                                                       'in_nucast_pkts': '77267',
                                                                                       'in_octets': '140871742311',
                                                                                       'in_ucast_pkts': '650119495',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '2152820974527',
                                                                                       'out_ucast_pkts': '1546960421',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkI0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fc:e0'],
                                                                                       'in_nucast_pkts': '6380',
                                                                                       'in_octets': '10392725188',
                                                                                       'in_ucast_pkts': '13276517',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '214',
                                                                                       'out_octets': '42341526171',
                                                                                       'out_ucast_pkts': '33214746',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzU5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:37:40'],
                                                                                       'in_nucast_pkts': '78633',
                                                                                       'in_octets': '41130133183',
                                                                                       'in_ucast_pkts': '267578468',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6644',
                                                                                       'out_octets': '983360137787',
                                                                                       'out_ucast_pkts': '737983039',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTk3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:b4:b0'],
                                                                                       'in_nucast_pkts': '71032',
                                                                                       'in_octets': '91458886374',
                                                                                       'in_ucast_pkts': '156074061',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6014',
                                                                                       'out_octets': '492238996224',
                                                                                       'out_ucast_pkts': '357362480',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUZCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:aa:e0'],
                                                                                       'in_nucast_pkts': '151308',
                                                                                       'in_octets': '8129995719',
                                                                                       'in_ucast_pkts': '14979672',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4924',
                                                                                       'out_octets': '4163271384',
                                                                                       'out_ucast_pkts': '8737748',
                                                                                       'poller_hash': 'QURUTjIzMTI0NEJBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:8b:90'],
                                                                                       'in_nucast_pkts': '168',
                                                                                       'in_octets': '24287069',
                                                                                       'in_ucast_pkts': '268180',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '13',
                                                                                       'out_octets': '723445891',
                                                                                       'out_ucast_pkts': '488924',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjI0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fe:d0'],
                                                                                       'in_nucast_pkts': '2481',
                                                                                       'in_octets': '671175529',
                                                                                       'in_ucast_pkts': '4503121',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '84',
                                                                                       'out_octets': '23274656091',
                                                                                       'out_ucast_pkts': '16045359',
                                                                                       'poller_hash': 'QURUTjIzMTQ1MTIwIHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '0',
                                                                                       'in_octets': '0',
                                                                                       'in_ucast_pkts': '0',
                                                                                       'oper_up': False,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjI0MzYzRDNBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:39:90'],
                                                                                       'in_nucast_pkts': '80304',
                                                                                       'in_octets': '183035148948',
                                                                                       'in_ucast_pkts': '1236860971',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6637',
                                                                                       'out_octets': '4528321524493',
                                                                                       'out_ucast_pkts': '3140107222',
                                                                                       'poller_hash': 'QURUTjI0MzE0MTk1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:e6:40'],
                                                                                       'in_nucast_pkts': '77272',
                                                                                       'in_octets': '33000539155',
                                                                                       'in_ucast_pkts': '261722329',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '754928010546',
                                                                                       'out_ucast_pkts': '529398495',
                                                                                       'poller_hash': 'QURUTjI0MzE0MUEzIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/6',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/6',
                                                                                      'id': 'xgspon '
                                                                                            '0/6',
                                                                                      'links': [       {       'connected_time': '2317060',
                                                                                                               'fiber_length': 5521,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTJBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317056',
                                                                                                               'fiber_length': 5919,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RUE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '4936369',
                                                                                                               'fiber_length': 5995,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkY1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317061',
                                                                                                               'fiber_length': 5797,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRjE4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317059',
                                                                                                               'fiber_length': 6115,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTk1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317061',
                                                                                                               'fiber_length': 6181,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MENDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317060',
                                                                                                               'fiber_length': 6734,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTExIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '5440097',
                                                                                                               'fiber_length': 7635,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MjA2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317060',
                                                                                                               'fiber_length': 5523,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317056',
                                                                                                               'fiber_length': 5847,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MENEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317059',
                                                                                                               'fiber_length': 6444,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTRCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317052',
                                                                                                               'fiber_length': 6030,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkEzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317058',
                                                                                                               'fiber_length': 6378,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkI0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '190728',
                                                                                                               'fiber_length': 6744,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzU5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317055',
                                                                                                               'fiber_length': 6051,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTk3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2096136',
                                                                                                               'fiber_length': 6816,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '5440101',
                                                                                                               'fiber_length': 7484,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NEJBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '4440',
                                                                                                               'fiber_length': 6580,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjI0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '96470',
                                                                                                               'fiber_length': 5781,
                                                                                                               'poller_hash': 'QURUTjIzMTQ1MTIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '1634735',
                                                                                                               'fiber_length': 6741,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRDNBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317064',
                                                                                                               'fiber_length': 6741,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MTk1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595},
                                                                                                       {       'connected_time': '2317064',
                                                                                                               'fiber_length': 6541,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MUEzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4595}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/6',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/7',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:4c:01:c0'],
                                                                                       'in_nucast_pkts': '161659',
                                                                                       'in_octets': '50049796547',
                                                                                       'in_ucast_pkts': '152471599',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5396',
                                                                                       'out_octets': '508085431347',
                                                                                       'out_ucast_pkts': '392883353',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDYxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:04:00'],
                                                                                       'in_nucast_pkts': '79177',
                                                                                       'in_octets': '52585387879',
                                                                                       'in_ucast_pkts': '290160443',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2576',
                                                                                       'out_octets': '988651401643',
                                                                                       'out_ucast_pkts': '738980630',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTcxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:82:c0'],
                                                                                       'in_nucast_pkts': '186255',
                                                                                       'in_octets': '241400071505',
                                                                                       'in_ucast_pkts': '1428422001',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15626',
                                                                                       'out_octets': '4760659326578',
                                                                                       'out_ucast_pkts': '3461599307',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTUyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:70'],
                                                                                       'in_nucast_pkts': '93222',
                                                                                       'in_octets': '47749529318',
                                                                                       'in_ucast_pkts': '316280876',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '7869',
                                                                                       'out_octets': '1402728868805',
                                                                                       'out_ucast_pkts': '1073548328',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3a:e0'],
                                                                                       'in_nucast_pkts': '15047',
                                                                                       'in_octets': '8726665362',
                                                                                       'in_ucast_pkts': '29108039',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1273',
                                                                                       'out_octets': '76215660012',
                                                                                       'out_ucast_pkts': '56408305',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTZBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:d8:90'],
                                                                                       'in_nucast_pkts': '162985',
                                                                                       'in_octets': '23530899217',
                                                                                       'in_ucast_pkts': '109177020',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5398',
                                                                                       'out_octets': '480313137336',
                                                                                       'out_ucast_pkts': '355957819',
                                                                                       'poller_hash': 'QURUTjIzMTI0NDVDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a7:30'],
                                                                                       'in_nucast_pkts': '28072',
                                                                                       'in_octets': '24195404784',
                                                                                       'in_ucast_pkts': '129400634',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '936',
                                                                                       'out_octets': '612910262602',
                                                                                       'out_ucast_pkts': '440182029',
                                                                                       'poller_hash': 'QURUTjIzMTI0NEJDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:50:10'],
                                                                                       'in_nucast_pkts': '181362',
                                                                                       'in_octets': '156763924400',
                                                                                       'in_ucast_pkts': '451405583',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6046',
                                                                                       'out_octets': '1183497788545',
                                                                                       'out_ucast_pkts': '929911749',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTNBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8f:d0'],
                                                                                       'in_nucast_pkts': '186588',
                                                                                       'in_octets': '11791523102',
                                                                                       'in_ucast_pkts': '32348353',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15625',
                                                                                       'out_octets': '45114517815',
                                                                                       'out_ucast_pkts': '40397613',
                                                                                       'poller_hash': 'QURUTjIzMTI0MUMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:92:e0'],
                                                                                       'in_nucast_pkts': '117189',
                                                                                       'in_octets': '120100217958',
                                                                                       'in_ucast_pkts': '671422934',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9922',
                                                                                       'out_octets': '2910180516762',
                                                                                       'out_ucast_pkts': '2077796789',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTU4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:02:f0'],
                                                                                       'in_nucast_pkts': '181344',
                                                                                       'in_octets': '19922779626',
                                                                                       'in_ucast_pkts': '37878045',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6046',
                                                                                       'out_octets': '73347970096',
                                                                                       'out_ucast_pkts': '65446477',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:65:a0'],
                                                                                       'in_nucast_pkts': '184824',
                                                                                       'in_octets': '120709171346',
                                                                                       'in_ucast_pkts': '222440403',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15627',
                                                                                       'out_octets': '704134560785',
                                                                                       'out_ucast_pkts': '529806623',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEJCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ce:70'],
                                                                                       'in_nucast_pkts': '181594',
                                                                                       'in_octets': '79498161718',
                                                                                       'in_ucast_pkts': '544856175',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6046',
                                                                                       'out_octets': '2679365284691',
                                                                                       'out_ucast_pkts': '1839981061',
                                                                                       'poller_hash': 'QURUTjIzMTQ0Nzk2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:84:90'],
                                                                                       'in_nucast_pkts': '70409',
                                                                                       'in_octets': '31887830912',
                                                                                       'in_ucast_pkts': '152296330',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5871',
                                                                                       'out_octets': '756680875145',
                                                                                       'out_ucast_pkts': '540611654',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODZDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:78:80'],
                                                                                       'in_nucast_pkts': '186260',
                                                                                       'in_octets': '32708626057',
                                                                                       'in_ucast_pkts': '75413117',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15613',
                                                                                       'out_octets': '151746807797',
                                                                                       'out_ucast_pkts': '132901226',
                                                                                       'poller_hash': 'QURUTjIzMTQ0Njk5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:50'],
                                                                                       'in_nucast_pkts': '56138',
                                                                                       'in_octets': '70658652100',
                                                                                       'in_ucast_pkts': '260887099',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4750',
                                                                                       'out_octets': '1013212977198',
                                                                                       'out_ucast_pkts': '735130401',
                                                                                       'poller_hash': 'QURUTjIzMTQ0Q0MzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:f1:00'],
                                                                                       'in_nucast_pkts': '108558',
                                                                                       'in_octets': '128047818368',
                                                                                       'in_ucast_pkts': '389186495',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3756',
                                                                                       'out_octets': '1605817453286',
                                                                                       'out_ucast_pkts': '1183465374',
                                                                                       'poller_hash': 'QURUTjIzMTI0MjE0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:a9:00'],
                                                                                       'in_nucast_pkts': '19235',
                                                                                       'in_octets': '9067388101',
                                                                                       'in_ucast_pkts': '11640227',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '642',
                                                                                       'out_octets': '32123065177',
                                                                                       'out_ucast_pkts': '25321888',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzRFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:fd:b0'],
                                                                                       'in_nucast_pkts': '19253',
                                                                                       'in_octets': '8221915797',
                                                                                       'in_ucast_pkts': '19612117',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '643',
                                                                                       'out_octets': '68669345709',
                                                                                       'out_ucast_pkts': '50805687',
                                                                                       'poller_hash': 'QURUTjI0MzE0NEZCIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/7',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/7',
                                                                                      'id': 'xgspon '
                                                                                            '0/7',
                                                                                      'links': [       {       'connected_time': '4857795',
                                                                                                               'fiber_length': 10820,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDYxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '2317062',
                                                                                                               'fiber_length': 10756,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTcxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '5440101',
                                                                                                               'fiber_length': 9016,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTUyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '2742692',
                                                                                                               'fiber_length': 10425,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '443591',
                                                                                                               'fiber_length': 10462,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTZBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '4857797',
                                                                                                               'fiber_length': 10872,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NDVDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '841331',
                                                                                                               'fiber_length': 9328,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NEJDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '5440102',
                                                                                                               'fiber_length': 9589,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTNBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '5440102',
                                                                                                               'fiber_length': 10748,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MUMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '3456381',
                                                                                                               'fiber_length': 10811,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTU4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '5440069',
                                                                                                               'fiber_length': 8976,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '5440069',
                                                                                                               'fiber_length': 8993,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEJCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '5440100',
                                                                                                               'fiber_length': 9375,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0Nzk2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '2045293',
                                                                                                               'fiber_length': 10744,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODZDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '5440101',
                                                                                                               'fiber_length': 8892,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0Njk5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '1656089',
                                                                                                               'fiber_length': 10521,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0MzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '3232209',
                                                                                                               'fiber_length': 9856,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MjE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '576233',
                                                                                                               'fiber_length': 10106,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzRFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431},
                                                                                                       {       'connected_time': '576799',
                                                                                                               'fiber_length': 10046,
                                                                                                               'poller_hash': 'QURUTjI0MzE0NEZCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.431}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/7',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/8',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dd:73:50'],
                                                                                       'in_nucast_pkts': '17352',
                                                                                       'in_octets': '12583730852',
                                                                                       'in_ucast_pkts': '15238765',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '32349249168',
                                                                                       'out_ucast_pkts': '27675715',
                                                                                       'poller_hash': 'QURUTjIzMTI0REZCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:75:a0'],
                                                                                       'in_nucast_pkts': '17350',
                                                                                       'in_octets': '4068733367',
                                                                                       'in_ucast_pkts': '27154906',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '75599876134',
                                                                                       'out_ucast_pkts': '52488869',
                                                                                       'poller_hash': 'QURUTjIzMTI0REM3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:76:a0'],
                                                                                       'in_nucast_pkts': '13965',
                                                                                       'in_octets': '8718124297',
                                                                                       'in_ucast_pkts': '31366044',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1182',
                                                                                       'out_octets': '87654360048',
                                                                                       'out_ucast_pkts': '62827471',
                                                                                       'poller_hash': 'QURUTjIzMTI0RUZBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:81:80'],
                                                                                       'in_nucast_pkts': '17645',
                                                                                       'in_octets': '7043293710',
                                                                                       'in_ucast_pkts': '15487796',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '58159716504',
                                                                                       'out_ucast_pkts': '44857988',
                                                                                       'poller_hash': 'QURUTjIzMTI0RURGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:79:10'],
                                                                                       'in_nucast_pkts': '17350',
                                                                                       'in_octets': '1766869586',
                                                                                       'in_ucast_pkts': '9286544',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '22473306375',
                                                                                       'out_ucast_pkts': '16882303',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUY3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:71:e0'],
                                                                                       'in_nucast_pkts': '17352',
                                                                                       'in_octets': '19201215989',
                                                                                       'in_ucast_pkts': '43126746',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '94247613907',
                                                                                       'out_ucast_pkts': '74374409',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTg0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:10'],
                                                                                       'in_nucast_pkts': '17354',
                                                                                       'in_octets': '63139814593',
                                                                                       'in_ucast_pkts': '195740707',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '575283213404',
                                                                                       'out_ucast_pkts': '422753217',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUY2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:32:11'],
                                                                                       'in_nucast_pkts': '312',
                                                                                       'in_octets': '19165886336',
                                                                                       'in_ucast_pkts': '38938258',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '135508186721',
                                                                                       'out_ucast_pkts': '106776301',
                                                                                       'poller_hash': 'QURUTjIzMTI0RUQzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:07:b0'],
                                                                                       'in_nucast_pkts': '17349',
                                                                                       'in_octets': '535160215',
                                                                                       'in_ucast_pkts': '1205277',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '2244417648',
                                                                                       'out_ucast_pkts': '1800523',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTQzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:04:10'],
                                                                                       'in_nucast_pkts': '17349',
                                                                                       'in_octets': '57260055059',
                                                                                       'in_ucast_pkts': '185213121',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '715177338108',
                                                                                       'out_ucast_pkts': '502819501',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTAzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8e:b0'],
                                                                                       'in_nucast_pkts': '17357',
                                                                                       'in_octets': '8271351533',
                                                                                       'in_ucast_pkts': '41007607',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '239064115958',
                                                                                       'out_ucast_pkts': '176634277',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTJFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:07:60'],
                                                                                       'in_nucast_pkts': '17359',
                                                                                       'in_octets': '15589904464',
                                                                                       'in_ucast_pkts': '36724183',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '214990073767',
                                                                                       'out_ucast_pkts': '150125402',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODNCIHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '92656',
                                                                                       'in_octets': '1243896716',
                                                                                       'in_ucast_pkts': '1237098',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '451',
                                                                                       'out_octets': '1213350778',
                                                                                       'out_ucast_pkts': '1231323',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzQ4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b3:a0'],
                                                                                       'in_nucast_pkts': '17077',
                                                                                       'in_octets': '5288053961',
                                                                                       'in_ucast_pkts': '18085701',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '63945371536',
                                                                                       'out_ucast_pkts': '50263467',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OEQxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:10'],
                                                                                       'in_nucast_pkts': '17358',
                                                                                       'in_octets': '5720460979',
                                                                                       'in_ucast_pkts': '54734286',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '170594327927',
                                                                                       'out_ucast_pkts': '116816421',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTlDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3c:30'],
                                                                                       'in_nucast_pkts': '11978',
                                                                                       'in_octets': '3250210157',
                                                                                       'in_ucast_pkts': '24466505',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1013',
                                                                                       'out_octets': '75017766361',
                                                                                       'out_ucast_pkts': '51821106',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjJDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:93:c0'],
                                                                                       'in_nucast_pkts': '17353',
                                                                                       'in_octets': '429884396',
                                                                                       'in_ucast_pkts': '648423',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '463891147',
                                                                                       'out_ucast_pkts': '612318',
                                                                                       'poller_hash': 'QURUTjIzMTQ0Q0Q1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:75:00'],
                                                                                       'in_nucast_pkts': '17344',
                                                                                       'in_octets': '4382109537',
                                                                                       'in_ucast_pkts': '9379472',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1465',
                                                                                       'out_octets': '26920753660',
                                                                                       'out_ucast_pkts': '21845818',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzQ2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:9c:b0'],
                                                                                       'in_nucast_pkts': '17077',
                                                                                       'in_octets': '21087915910',
                                                                                       'in_ucast_pkts': '52964384',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '151984788371',
                                                                                       'out_ucast_pkts': '113414646',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:d0:70'],
                                                                                       'in_nucast_pkts': '11789',
                                                                                       'in_octets': '1576302843',
                                                                                       'in_ucast_pkts': '7857963',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '394',
                                                                                       'out_octets': '29168052431',
                                                                                       'out_ucast_pkts': '21875490',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTcwIHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '0',
                                                                                       'in_octets': '0',
                                                                                       'in_ucast_pkts': '0',
                                                                                       'oper_up': False,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RjAwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:51:30'],
                                                                                       'in_nucast_pkts': '17074',
                                                                                       'in_octets': '8007726891',
                                                                                       'in_ucast_pkts': '57195326',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '170355661509',
                                                                                       'out_ucast_pkts': '126103840',
                                                                                       'poller_hash': 'QURUTjIzMTI0N0ExIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:26:a0'],
                                                                                       'in_nucast_pkts': '17076',
                                                                                       'in_octets': '5211792737',
                                                                                       'in_ucast_pkts': '32682412',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '104083040915',
                                                                                       'out_ucast_pkts': '72720282',
                                                                                       'poller_hash': 'QURUTjIzMTI0QzZBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ac:60'],
                                                                                       'in_nucast_pkts': '17229',
                                                                                       'in_octets': '4548076901',
                                                                                       'in_ucast_pkts': '56225134',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '178557922115',
                                                                                       'out_ucast_pkts': '119393201',
                                                                                       'poller_hash': 'QURUTjI0MzYzREM5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:27:10'],
                                                                                       'in_nucast_pkts': '15132',
                                                                                       'in_octets': '30461726021',
                                                                                       'in_ucast_pkts': '153669174',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '505',
                                                                                       'out_octets': '472024713335',
                                                                                       'out_ucast_pkts': '353532698',
                                                                                       'poller_hash': 'QURUTjI0MzE0MzE0IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/8',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/8',
                                                                                      'id': 'xgspon '
                                                                                            '0/8',
                                                                                      'links': [       {       'connected_time': '511510',
                                                                                                               'fiber_length': 2183,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REZCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511510',
                                                                                                               'fiber_length': 2394,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REM3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '411662',
                                                                                                               'fiber_length': 2496,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RUZBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511510',
                                                                                                               'fiber_length': 3018,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RURGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 2254,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUY3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511511',
                                                                                                               'fiber_length': 3256,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTg0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511509',
                                                                                                               'fiber_length': 2382,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511506',
                                                                                                               'fiber_length': 2629,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RUQzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511506',
                                                                                                               'fiber_length': 3430,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTQzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511509',
                                                                                                               'fiber_length': 3301,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTAzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 3383,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTJFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511506',
                                                                                                               'fiber_length': 2182,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODNCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 3353,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzQ4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511504',
                                                                                                               'fiber_length': 2661,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OEQxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 3282,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTlDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '352937',
                                                                                                               'fiber_length': 4011,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjJDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 3249,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0Q1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511503',
                                                                                                               'fiber_length': 2665,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzQ2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 3298,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '352937',
                                                                                                               'fiber_length': 3865,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTcwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511502',
                                                                                                               'fiber_length': 3323,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RjAwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511512',
                                                                                                               'fiber_length': 3094,
                                                                                                               'poller_hash': 'QURUTjIzMTI0N0ExIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511512',
                                                                                                               'fiber_length': 3062,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QzZBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '511516',
                                                                                                               'fiber_length': 3347,
                                                                                                               'poller_hash': 'QURUTjI0MzYzREM5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577},
                                                                                                       {       'connected_time': '453158',
                                                                                                               'fiber_length': 3270,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MzE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2577}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/8',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/9',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:a9:50'],
                                                                                       'in_nucast_pkts': '17678',
                                                                                       'in_octets': '31181646609',
                                                                                       'in_ucast_pkts': '145785231',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1469',
                                                                                       'out_octets': '521978203351',
                                                                                       'out_ucast_pkts': '375429160',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDkxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7e:50'],
                                                                                       'in_nucast_pkts': '17350',
                                                                                       'in_octets': '6424793475',
                                                                                       'in_ucast_pkts': '15583782',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '66052251294',
                                                                                       'out_ucast_pkts': '52733391',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTdBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:b1:c0'],
                                                                                       'in_nucast_pkts': '17800',
                                                                                       'in_octets': '90235861365',
                                                                                       'in_ucast_pkts': '202145141',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1469',
                                                                                       'out_octets': '549655463279',
                                                                                       'out_ucast_pkts': '408842240',
                                                                                       'poller_hash': 'QURUTjIzMTIzRTlDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:9c:b0'],
                                                                                       'in_nucast_pkts': '17500',
                                                                                       'in_octets': '43499037247',
                                                                                       'in_ucast_pkts': '94176808',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '298195340695',
                                                                                       'out_ucast_pkts': '223126720',
                                                                                       'poller_hash': 'QURUTjIzMTI0REVCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:87:80'],
                                                                                       'in_nucast_pkts': '17350',
                                                                                       'in_octets': '8638783925',
                                                                                       'in_ucast_pkts': '46431856',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '133805388064',
                                                                                       'out_ucast_pkts': '99643458',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDkzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:b4:c0'],
                                                                                       'in_nucast_pkts': '17073',
                                                                                       'in_octets': '11314547205',
                                                                                       'in_ucast_pkts': '22941611',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '98372197237',
                                                                                       'out_ucast_pkts': '76185217',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDQ2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:88:60'],
                                                                                       'in_nucast_pkts': '17350',
                                                                                       'in_octets': '9058510653',
                                                                                       'in_ucast_pkts': '39985645',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '241684711586',
                                                                                       'out_ucast_pkts': '170715081',
                                                                                       'poller_hash': 'QURUTjIzMTI0OTFFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:b1'],
                                                                                       'in_nucast_pkts': '390',
                                                                                       'in_octets': '11166614608',
                                                                                       'in_ucast_pkts': '24360922',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '84529303524',
                                                                                       'out_ucast_pkts': '65666060',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTNBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:7d:c0'],
                                                                                       'in_nucast_pkts': '17779',
                                                                                       'in_octets': '18985244898',
                                                                                       'in_ucast_pkts': '106949594',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '438385612413',
                                                                                       'out_ucast_pkts': '315523487',
                                                                                       'poller_hash': 'QURUTjIzMTI0REYzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:74:b0'],
                                                                                       'in_nucast_pkts': '17349',
                                                                                       'in_octets': '930032427',
                                                                                       'in_ucast_pkts': '3534867',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '8545155679',
                                                                                       'out_ucast_pkts': '6754964',
                                                                                       'poller_hash': 'QURUTjIzMTI1MThFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:2a:40'],
                                                                                       'in_nucast_pkts': '17788',
                                                                                       'in_octets': '14946109629',
                                                                                       'in_ucast_pkts': '58363627',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1468',
                                                                                       'out_octets': '294012971396',
                                                                                       'out_ucast_pkts': '213405469',
                                                                                       'poller_hash': 'QURUTjIzMTI0RENFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8d:d0'],
                                                                                       'in_nucast_pkts': '17350',
                                                                                       'in_octets': '2315752953',
                                                                                       'in_ucast_pkts': '7552312',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '20525064080',
                                                                                       'out_ucast_pkts': '17015732',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTIyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:66:60'],
                                                                                       'in_nucast_pkts': '17777',
                                                                                       'in_octets': '13446246092',
                                                                                       'in_ucast_pkts': '88985714',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '326420716091',
                                                                                       'out_ucast_pkts': '238645457',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDlFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:c0'],
                                                                                       'in_nucast_pkts': '17378',
                                                                                       'in_octets': '4864823600',
                                                                                       'in_ucast_pkts': '9100075',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '32633620755',
                                                                                       'out_ucast_pkts': '26720543',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTJEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ad:50'],
                                                                                       'in_nucast_pkts': '17075',
                                                                                       'in_octets': '11346630986',
                                                                                       'in_ucast_pkts': '57003429',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '229678363616',
                                                                                       'out_ucast_pkts': '168885489',
                                                                                       'poller_hash': 'QURUTjIzMTI0MDhDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:73:90'],
                                                                                       'in_nucast_pkts': '17349',
                                                                                       'in_octets': '44042595027',
                                                                                       'in_ucast_pkts': '73274929',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '235157386943',
                                                                                       'out_ucast_pkts': '174144687',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDFEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6e:20'],
                                                                                       'in_nucast_pkts': '17358',
                                                                                       'in_octets': '8390216571',
                                                                                       'in_ucast_pkts': '66250060',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '324869683562',
                                                                                       'out_ucast_pkts': '223917456',
                                                                                       'poller_hash': 'QURUTjIzMTI0REY1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:88:c0'],
                                                                                       'in_nucast_pkts': '17349',
                                                                                       'in_octets': '13287612317',
                                                                                       'in_ucast_pkts': '83805288',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '266246543460',
                                                                                       'out_ucast_pkts': '186300386',
                                                                                       'poller_hash': 'QURUTjIzMTI0NzEzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7b:70'],
                                                                                       'in_nucast_pkts': '17642',
                                                                                       'in_octets': '13216747722',
                                                                                       'in_ucast_pkts': '77811734',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '213176396900',
                                                                                       'out_ucast_pkts': '148724509',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTE5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:82:00'],
                                                                                       'in_nucast_pkts': '17351',
                                                                                       'in_octets': '4500406894',
                                                                                       'in_ucast_pkts': '13799152',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '64753526744',
                                                                                       'out_ucast_pkts': '51310902',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTY3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:83:40'],
                                                                                       'in_nucast_pkts': '17413',
                                                                                       'in_octets': '21108613447',
                                                                                       'in_ucast_pkts': '66454572',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1472',
                                                                                       'out_octets': '265124843284',
                                                                                       'out_ucast_pkts': '197979817',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTNGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:03:e0'],
                                                                                       'in_nucast_pkts': '17573',
                                                                                       'in_octets': '16510429418',
                                                                                       'in_ucast_pkts': '47316879',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '581',
                                                                                       'out_octets': '132080811021',
                                                                                       'out_ucast_pkts': '102434920',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTIzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:70:f0'],
                                                                                       'in_nucast_pkts': '17356',
                                                                                       'in_octets': '12201351423',
                                                                                       'in_ucast_pkts': '96462911',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '280128997438',
                                                                                       'out_ucast_pkts': '210615612',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTI3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fa:00'],
                                                                                       'in_nucast_pkts': '17077',
                                                                                       'in_octets': '8081348185',
                                                                                       'in_ucast_pkts': '45173344',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '191072021092',
                                                                                       'out_ucast_pkts': '133766604',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTY0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:87:c0'],
                                                                                       'in_nucast_pkts': '17353',
                                                                                       'in_octets': '4808821172',
                                                                                       'in_ucast_pkts': '26666839',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '90330747117',
                                                                                       'out_ucast_pkts': '65389355',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTZCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:03:d0'],
                                                                                       'in_nucast_pkts': '2084',
                                                                                       'in_octets': '274980763',
                                                                                       'in_ucast_pkts': '775381',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '70',
                                                                                       'out_octets': '1458085763',
                                                                                       'out_ucast_pkts': '1423806',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkVDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:88:b0'],
                                                                                       'in_nucast_pkts': '17354',
                                                                                       'in_octets': '1412860165',
                                                                                       'in_ucast_pkts': '5199266',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '18726906322',
                                                                                       'out_ucast_pkts': '14643689',
                                                                                       'poller_hash': 'QURUTjIzMTIzRUE4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8b:c0'],
                                                                                       'in_nucast_pkts': '17350',
                                                                                       'in_octets': '3183811034',
                                                                                       'in_ucast_pkts': '28605061',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '110348349871',
                                                                                       'out_ucast_pkts': '77108246',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTFGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:82:30'],
                                                                                       'in_nucast_pkts': '17351',
                                                                                       'in_octets': '5581427159',
                                                                                       'in_ucast_pkts': '22229830',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '61468584165',
                                                                                       'out_ucast_pkts': '53311404',
                                                                                       'poller_hash': 'QURUTjIzMTI0MTI4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:86:d1'],
                                                                                       'in_nucast_pkts': '314',
                                                                                       'in_octets': '9818933607',
                                                                                       'in_ucast_pkts': '26316509',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '108363902187',
                                                                                       'out_ucast_pkts': '77185946',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0E0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:04:e0'],
                                                                                       'in_nucast_pkts': '17075',
                                                                                       'in_octets': '11614398483',
                                                                                       'in_ucast_pkts': '26579067',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '72537821329',
                                                                                       'out_ucast_pkts': '60433602',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTY5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3b:c0'],
                                                                                       'in_nucast_pkts': '17349',
                                                                                       'in_octets': '32971990275',
                                                                                       'in_ucast_pkts': '47577357',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1467',
                                                                                       'out_octets': '89779815388',
                                                                                       'out_ucast_pkts': '72520182',
                                                                                       'poller_hash': 'QURUTjIzMTI0MDA4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b7:d0'],
                                                                                       'in_nucast_pkts': '17079',
                                                                                       'in_octets': '21339841748',
                                                                                       'in_ucast_pkts': '84681430',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '570',
                                                                                       'out_octets': '343899216165',
                                                                                       'out_ucast_pkts': '254910822',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUQzIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/9',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/9',
                                                                                      'id': 'xgspon '
                                                                                            '0/9',
                                                                                      'links': [       {       'connected_time': '511494',
                                                                                                               'fiber_length': 5783,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDkxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511506',
                                                                                                               'fiber_length': 5853,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTdBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511514',
                                                                                                               'fiber_length': 5669,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRTlDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 3861,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REVCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511497',
                                                                                                               'fiber_length': 4090,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDkzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511509',
                                                                                                               'fiber_length': 4612,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDQ2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511495',
                                                                                                               'fiber_length': 5476,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OTFFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511498',
                                                                                                               'fiber_length': 5824,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTNBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511514',
                                                                                                               'fiber_length': 3643,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REYzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511497',
                                                                                                               'fiber_length': 3909,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MThFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 5338,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RENFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 3743,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTIyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511509',
                                                                                                               'fiber_length': 4273,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDlFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 4867,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTJEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511501',
                                                                                                               'fiber_length': 5423,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MDhDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511506',
                                                                                                               'fiber_length': 4938,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDFEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 5198,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REY1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511499',
                                                                                                               'fiber_length': 5184,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NzEzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511513',
                                                                                                               'fiber_length': 5202,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTE5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511511',
                                                                                                               'fiber_length': 5100,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTY3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511512',
                                                                                                               'fiber_length': 5421,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTNGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511512',
                                                                                                               'fiber_length': 4748,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTIzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511508',
                                                                                                               'fiber_length': 4716,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTI3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511507',
                                                                                                               'fiber_length': 4657,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTY0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511507',
                                                                                                               'fiber_length': 5879,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTZCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '61774',
                                                                                                               'fiber_length': 5819,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkVDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511509',
                                                                                                               'fiber_length': 4449,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRUE4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511512',
                                                                                                               'fiber_length': 4391,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTFGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511513',
                                                                                                               'fiber_length': 5794,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MTI4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511505',
                                                                                                               'fiber_length': 5397,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0E0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511511',
                                                                                                               'fiber_length': 4383,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTY5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511513',
                                                                                                               'fiber_length': 5786,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MDA4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975},
                                                                                                       {       'connected_time': '511498',
                                                                                                               'fiber_length': 5736,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUQzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3975}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/9',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/1',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:ce',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/1',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMQ==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/2',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:cf',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/2',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMg==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/3',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:d0',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/3',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMw==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/4',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:d1',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/4',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvNA==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'management 0/1',
                                                              'mac_address': '38:f8:f6:ac:65:c8',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'management 0/1',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgbWFuYWdlbWVudCAwLzE=',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/1',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '58275531',
                                                              'in_octets': '49816948633458',
                                                              'in_ucast_pkts': '450096209404',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:ca',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/1',
                                                              'oper_up': True,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '61592239',
                                                              'out_octets': '48973188331903',
                                                              'out_ucast_pkts': '176504962513',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC8x',
                                                              'speed': '10000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/2',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:cb',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/2',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC8y',
                                                              'speed': '10000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/3',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:cc',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/3',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC8z',
                                                              'speed': '10000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/4',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:cd',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/4',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC80',
                                                              'speed': '10000000000',
                                                              'type': 6}],
                                'manufacturer': 'Adtran',
                                'mem': 55.04,
                                'model': 'SDX 6320-16',
                                'name': 'IndianBayou-x1',
                                'poller_hash': 'b2x0IExCQURUTjIzMjJBQTIzNA==',
                                'serial_number': 'LBADTN2322AA234',
                                'sw_version': 'sdx6320-RL-23.5-33',
                                'temperature': 68.0,
                                'uptime': '5440305'},
                'peers': [       {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17678',
                                                                       'in_octets': '31181646609',
                                                                       'in_ucast_pkts': '145785231',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1469',
                                                                       'out_octets': '521978203351',
                                                                       'out_ucast_pkts': '375429160',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1469',
                                                                       'in_octets': '521978203351',
                                                                       'in_ucast_pkts': '375429160',
                                                                       'links': [       {       'in_nucast_pkts': '1469',
                                                                                                'in_octets': '521978203351',
                                                                                                'in_ucast_pkts': '375429160',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17678',
                                                                                                'out_octets': '31181646609',
                                                                                                'out_ucast_pkts': '145785231',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17678',
                                                                       'out_octets': '31181646609',
                                                                       'out_ucast_pkts': '145785231',
                                                                       'poller_hash': 'QURUTjIzMTI0RDkxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511494',
                                                                                                                        'fiber_length': 5783,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDkxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ5MQ==',
                                         'serial_number': 'ADTN23124D91',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511494'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17350',
                                                                       'in_octets': '13335759005',
                                                                       'in_ucast_pkts': '104288639',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '481481868213',
                                                                       'out_ucast_pkts': '328067017',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '481481868213',
                                                                       'in_ucast_pkts': '328067017',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '481481868213',
                                                                                                'in_ucast_pkts': '328067017',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17350',
                                                                                                'out_octets': '13335759005',
                                                                                                'out_ucast_pkts': '104288639',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17350',
                                                                       'out_octets': '13335759005',
                                                                       'out_ucast_pkts': '104288639',
                                                                       'poller_hash': 'QURUTjIzMTI0RDY4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511515',
                                                                                                                        'fiber_length': 5677,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDY4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ2OA==',
                                         'serial_number': 'ADTN23124D68',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511515'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17350',
                                                                       'in_octets': '6424793475',
                                                                       'in_ucast_pkts': '15583782',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '66052251294',
                                                                       'out_ucast_pkts': '52733391',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '66052251294',
                                                                       'in_ucast_pkts': '52733391',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '66052251294',
                                                                                                'in_ucast_pkts': '52733391',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17350',
                                                                                                'out_octets': '6424793475',
                                                                                                'out_ucast_pkts': '15583782',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17350',
                                                                       'out_octets': '6424793475',
                                                                       'out_ucast_pkts': '15583782',
                                                                       'poller_hash': 'QURUTjIzMTI0RTdBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511506',
                                                                                                                        'fiber_length': 5853,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTdBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU3QQ==',
                                         'serial_number': 'ADTN23124E7A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511506'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17353',
                                                                       'in_octets': '3375284024',
                                                                       'in_ucast_pkts': '7643960',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '38970248215',
                                                                       'out_ucast_pkts': '31960265',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '38970248215',
                                                                       'in_ucast_pkts': '31960265',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '38970248215',
                                                                                                'in_ucast_pkts': '31960265',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17353',
                                                                                                'out_octets': '3375284024',
                                                                                                'out_ucast_pkts': '7643960',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17353',
                                                                       'out_octets': '3375284024',
                                                                       'out_ucast_pkts': '7643960',
                                                                       'poller_hash': 'QURUTjIzMTI0RTE4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511514',
                                                                                                                        'fiber_length': 6613,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTE4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxOA==',
                                         'serial_number': 'ADTN23124E18',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511514'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17800',
                                                                       'in_octets': '90235861365',
                                                                       'in_ucast_pkts': '202145141',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1469',
                                                                       'out_octets': '549655463279',
                                                                       'out_ucast_pkts': '408842240',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1469',
                                                                       'in_octets': '549655463279',
                                                                       'in_ucast_pkts': '408842240',
                                                                       'links': [       {       'in_nucast_pkts': '1469',
                                                                                                'in_octets': '549655463279',
                                                                                                'in_ucast_pkts': '408842240',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17800',
                                                                                                'out_octets': '90235861365',
                                                                                                'out_ucast_pkts': '202145141',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17800',
                                                                       'out_octets': '90235861365',
                                                                       'out_ucast_pkts': '202145141',
                                                                       'poller_hash': 'QURUTjIzMTIzRTlDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511514',
                                                                                                                        'fiber_length': 5669,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRTlDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0U5Qw==',
                                         'serial_number': 'ADTN23123E9C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511514'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17500',
                                                                       'in_octets': '43499037247',
                                                                       'in_ucast_pkts': '94176808',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '298195340695',
                                                                       'out_ucast_pkts': '223126720',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '298195340695',
                                                                       'in_ucast_pkts': '223126720',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '298195340695',
                                                                                                'in_ucast_pkts': '223126720',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17500',
                                                                                                'out_octets': '43499037247',
                                                                                                'out_ucast_pkts': '94176808',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17500',
                                                                       'out_octets': '43499037247',
                                                                       'out_ucast_pkts': '94176808',
                                                                       'poller_hash': 'QURUTjIzMTI0REVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 3861,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERFQg==',
                                         'serial_number': 'ADTN23124DEB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17350',
                                                                       'in_octets': '8638783925',
                                                                       'in_ucast_pkts': '46431856',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '133805388064',
                                                                       'out_ucast_pkts': '99643458',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '133805388064',
                                                                       'in_ucast_pkts': '99643458',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '133805388064',
                                                                                                'in_ucast_pkts': '99643458',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17350',
                                                                                                'out_octets': '8638783925',
                                                                                                'out_ucast_pkts': '46431856',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17350',
                                                                       'out_octets': '8638783925',
                                                                       'out_ucast_pkts': '46431856',
                                                                       'poller_hash': 'QURUTjIzMTI0RDkzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511497',
                                                                                                                        'fiber_length': 4090,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDkzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ5Mw==',
                                         'serial_number': 'ADTN23124D93',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511497'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17073',
                                                                       'in_octets': '11314547205',
                                                                       'in_ucast_pkts': '22941611',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '98372197237',
                                                                       'out_ucast_pkts': '76185217',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VNV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '98372197237',
                                                                       'in_ucast_pkts': '76185217',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '98372197237',
                                                                                                'in_ucast_pkts': '76185217',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17073',
                                                                                                'out_octets': '11314547205',
                                                                                                'out_ucast_pkts': '22941611',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17073',
                                                                       'out_octets': '11314547205',
                                                                       'out_ucast_pkts': '22941611',
                                                                       'poller_hash': 'QURUTjIzMTI0RDQ2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511509',
                                                                                                                        'fiber_length': 4612,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDQ2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ0Ng==',
                                         'serial_number': 'ADTN23124D46',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511509'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17350',
                                                                       'in_octets': '9058510653',
                                                                       'in_ucast_pkts': '39985645',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '241684711586',
                                                                       'out_ucast_pkts': '170715081',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '241684711586',
                                                                       'in_ucast_pkts': '170715081',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '241684711586',
                                                                                                'in_ucast_pkts': '170715081',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17350',
                                                                                                'out_octets': '9058510653',
                                                                                                'out_ucast_pkts': '39985645',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17350',
                                                                       'out_octets': '9058510653',
                                                                       'out_ucast_pkts': '39985645',
                                                                       'poller_hash': 'QURUTjIzMTI0OTFFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511495',
                                                                                                                        'fiber_length': 5476,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OTFFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDkxRQ==',
                                         'serial_number': 'ADTN2312491E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511495'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '390',
                                                                       'in_octets': '11166614608',
                                                                       'in_ucast_pkts': '24360922',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '84529303524',
                                                                       'out_ucast_pkts': '65666060',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '84529303524',
                                                                       'in_ucast_pkts': '65666060',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '84529303524',
                                                                                                'in_ucast_pkts': '65666060',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '390',
                                                                                                'out_octets': '11166614608',
                                                                                                'out_ucast_pkts': '24360922',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '390',
                                                                       'out_octets': '11166614608',
                                                                       'out_ucast_pkts': '24360922',
                                                                       'poller_hash': 'QURUTjIzMTI0RTNBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511498',
                                                                                                                        'fiber_length': 5824,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTNBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzQQ==',
                                         'serial_number': 'ADTN23124E3A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17779',
                                                                       'in_octets': '18985244898',
                                                                       'in_ucast_pkts': '106949594',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '438385612413',
                                                                       'out_ucast_pkts': '315523487',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VOV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '438385612413',
                                                                       'in_ucast_pkts': '315523487',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '438385612413',
                                                                                                'in_ucast_pkts': '315523487',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17779',
                                                                                                'out_octets': '18985244898',
                                                                                                'out_ucast_pkts': '106949594',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17779',
                                                                       'out_octets': '18985244898',
                                                                       'out_ucast_pkts': '106949594',
                                                                       'poller_hash': 'QURUTjIzMTI0REYzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511514',
                                                                                                                        'fiber_length': 3643,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REYzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGMw==',
                                         'serial_number': 'ADTN23124DF3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5-5-b',
                                         'uptime': '511514'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17349',
                                                                       'in_octets': '930032427',
                                                                       'in_ucast_pkts': '3534867',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '8545155679',
                                                                       'out_ucast_pkts': '6754964',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '8545155679',
                                                                       'in_ucast_pkts': '6754964',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '8545155679',
                                                                                                'in_ucast_pkts': '6754964',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17349',
                                                                                                'out_octets': '930032427',
                                                                                                'out_ucast_pkts': '3534867',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17349',
                                                                       'out_octets': '930032427',
                                                                       'out_ucast_pkts': '3534867',
                                                                       'poller_hash': 'QURUTjIzMTI1MThFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511497',
                                                                                                                        'fiber_length': 3909,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MThFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE4RQ==',
                                         'serial_number': 'ADTN2312518E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511497'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17788',
                                                                       'in_octets': '14946109629',
                                                                       'in_ucast_pkts': '58363627',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1468',
                                                                       'out_octets': '294012971396',
                                                                       'out_ucast_pkts': '213405469',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1468',
                                                                       'in_octets': '294012971396',
                                                                       'in_ucast_pkts': '213405469',
                                                                       'links': [       {       'in_nucast_pkts': '1468',
                                                                                                'in_octets': '294012971396',
                                                                                                'in_ucast_pkts': '213405469',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17788',
                                                                                                'out_octets': '14946109629',
                                                                                                'out_ucast_pkts': '58363627',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17788',
                                                                       'out_octets': '14946109629',
                                                                       'out_ucast_pkts': '58363627',
                                                                       'poller_hash': 'QURUTjIzMTI0RENFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 5338,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RENFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERDRQ==',
                                         'serial_number': 'ADTN23124DCE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17350',
                                                                       'in_octets': '2315752953',
                                                                       'in_ucast_pkts': '7552312',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '20525064080',
                                                                       'out_ucast_pkts': '17015732',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '20525064080',
                                                                       'in_ucast_pkts': '17015732',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '20525064080',
                                                                                                'in_ucast_pkts': '17015732',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17350',
                                                                                                'out_octets': '2315752953',
                                                                                                'out_ucast_pkts': '7552312',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17350',
                                                                       'out_octets': '2315752953',
                                                                       'out_ucast_pkts': '7552312',
                                                                       'poller_hash': 'QURUTjIzMTI0RTIyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 3743,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTIyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyMg==',
                                         'serial_number': 'ADTN23124E22',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '116896',
                                                                       'in_octets': '36055555296',
                                                                       'in_ucast_pkts': '133646574',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9898',
                                                                       'out_octets': '481248857766',
                                                                       'out_ucast_pkts': '378880965',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9898',
                                                                       'in_octets': '481248857766',
                                                                       'in_ucast_pkts': '378880965',
                                                                       'links': [       {       'in_nucast_pkts': '9898',
                                                                                                'in_octets': '481248857766',
                                                                                                'in_ucast_pkts': '378880965',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '116896',
                                                                                                'out_octets': '36055555296',
                                                                                                'out_ucast_pkts': '133646574',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '116896',
                                                                       'out_octets': '36055555296',
                                                                       'out_ucast_pkts': '133646574',
                                                                       'poller_hash': 'QURUTjIzMTI1MDU1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3449060',
                                                                                                                        'fiber_length': 6229,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDU1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA1NQ==',
                                         'serial_number': 'ADTN23125055',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3449060'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17468',
                                                                       'in_octets': '1991337600',
                                                                       'in_ucast_pkts': '19911217',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '59145047964',
                                                                       'out_ucast_pkts': '40137087',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '59145047964',
                                                                       'in_ucast_pkts': '40137087',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '59145047964',
                                                                                                'in_ucast_pkts': '40137087',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17468',
                                                                                                'out_octets': '1991337600',
                                                                                                'out_ucast_pkts': '19911217',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17468',
                                                                       'out_octets': '1991337600',
                                                                       'out_ucast_pkts': '19911217',
                                                                       'poller_hash': 'QURUTjIzMTI0RDQxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511495',
                                                                                                                        'fiber_length': 1926,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDQxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ0MQ==',
                                         'serial_number': 'ADTN23124D41',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511495'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17340',
                                                                       'in_octets': '1758382852',
                                                                       'in_ucast_pkts': '6919707',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '31194916904',
                                                                       'out_ucast_pkts': '22510192',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '31194916904',
                                                                       'in_ucast_pkts': '22510192',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '31194916904',
                                                                                                'in_ucast_pkts': '22510192',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17340',
                                                                                                'out_octets': '1758382852',
                                                                                                'out_ucast_pkts': '6919707',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17340',
                                                                       'out_octets': '1758382852',
                                                                       'out_ucast_pkts': '6919707',
                                                                       'poller_hash': 'QURUTjIzMTI0MEIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511496',
                                                                                                                        'fiber_length': 1969,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MEIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDBCMA==',
                                         'serial_number': 'ADTN231240B0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511496'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17754',
                                                                       'in_octets': '11748288735',
                                                                       'in_ucast_pkts': '59006709',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '158629248942',
                                                                       'out_ucast_pkts': '130598930',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '158629248942',
                                                                       'in_ucast_pkts': '130598930',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '158629248942',
                                                                                                'in_ucast_pkts': '130598930',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17754',
                                                                                                'out_octets': '11748288735',
                                                                                                'out_ucast_pkts': '59006709',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17754',
                                                                       'out_octets': '11748288735',
                                                                       'out_ucast_pkts': '59006709',
                                                                       'poller_hash': 'QURUTjIzMTI0REQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511496',
                                                                                                                        'fiber_length': 1780,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEREOQ==',
                                         'serial_number': 'ADTN23124DD9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511496'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-8.1.13.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x8-ONU13_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTNfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0MEE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MEE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x8-ONU13',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDBBNg==',
                                         'serial_number': 'ADTN231240A6',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17777',
                                                                       'in_octets': '13446246092',
                                                                       'in_ucast_pkts': '88985714',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124D9E_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '326420716091',
                                                                       'out_ucast_pkts': '238645457',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RDlFX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '326420716091',
                                                                       'in_ucast_pkts': '238645457',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '326420716091',
                                                                                                'in_ucast_pkts': '238645457',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17777',
                                                                                                'out_octets': '13446246092',
                                                                                                'out_ucast_pkts': '88985714',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17777',
                                                                       'out_octets': '13446246092',
                                                                       'out_ucast_pkts': '88985714',
                                                                       'poller_hash': 'QURUTjIzMTI0RDlFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511509',
                                                                                                                        'fiber_length': 4273,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDlFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124D9E',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ5RQ==',
                                         'serial_number': 'ADTN23124D9E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511509'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78535',
                                                                       'in_octets': '32074553609',
                                                                       'in_ucast_pkts': '247138061',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6651',
                                                                       'out_octets': '851716345889',
                                                                       'out_ucast_pkts': '606755361',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6651',
                                                                       'in_octets': '851716345889',
                                                                       'in_ucast_pkts': '606755361',
                                                                       'links': [       {       'in_nucast_pkts': '6651',
                                                                                                'in_octets': '851716345889',
                                                                                                'in_ucast_pkts': '606755361',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78535',
                                                                                                'out_octets': '32074553609',
                                                                                                'out_ucast_pkts': '247138061',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78535',
                                                                       'out_octets': '32074553609',
                                                                       'out_ucast_pkts': '247138061',
                                                                       'poller_hash': 'QURUTjIzMTI0REYxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317053',
                                                                                                                        'fiber_length': 5944,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REYxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGMQ==',
                                         'serial_number': 'ADTN23124DF1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317053'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '80464',
                                                                       'in_octets': '64191871858',
                                                                       'in_ucast_pkts': '294720133',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6629',
                                                                       'out_octets': '1307641974656',
                                                                       'out_ucast_pkts': '937844307',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6629',
                                                                       'in_octets': '1307641974656',
                                                                       'in_ucast_pkts': '937844307',
                                                                       'links': [       {       'in_nucast_pkts': '6629',
                                                                                                'in_octets': '1307641974656',
                                                                                                'in_ucast_pkts': '937844307',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '80464',
                                                                                                'out_octets': '64191871858',
                                                                                                'out_ucast_pkts': '294720133',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '80464',
                                                                       'out_octets': '64191871858',
                                                                       'out_ucast_pkts': '294720133',
                                                                       'poller_hash': 'QURUTjIzMTI0RTA3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317053',
                                                                                                                        'fiber_length': 3992,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTA3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUwNw==',
                                         'serial_number': 'ADTN23124E07',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317053'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14661',
                                                                       'in_octets': '6991765314',
                                                                       'in_ucast_pkts': '30718708',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1210',
                                                                       'out_octets': '110390447582',
                                                                       'out_ucast_pkts': '78915022',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1210',
                                                                       'in_octets': '110390447582',
                                                                       'in_ucast_pkts': '78915022',
                                                                       'links': [       {       'in_nucast_pkts': '1210',
                                                                                                'in_octets': '110390447582',
                                                                                                'in_ucast_pkts': '78915022',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14661',
                                                                                                'out_octets': '6991765314',
                                                                                                'out_ucast_pkts': '30718708',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14661',
                                                                       'out_octets': '6991765314',
                                                                       'out_ucast_pkts': '30718708',
                                                                       'poller_hash': 'QURUTjIzMTI0RTE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '421798',
                                                                                                                        'fiber_length': 4123,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxNA==',
                                         'serial_number': 'ADTN23124E14',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '421798'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78530',
                                                                       'in_octets': '9573246216',
                                                                       'in_ucast_pkts': '28968827',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6646',
                                                                       'out_octets': '94670181667',
                                                                       'out_ucast_pkts': '79312527',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6646',
                                                                       'in_octets': '94670181667',
                                                                       'in_ucast_pkts': '79312527',
                                                                       'links': [       {       'in_nucast_pkts': '6646',
                                                                                                'in_octets': '94670181667',
                                                                                                'in_ucast_pkts': '79312527',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78530',
                                                                                                'out_octets': '9573246216',
                                                                                                'out_ucast_pkts': '28968827',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78530',
                                                                       'out_octets': '9573246216',
                                                                       'out_ucast_pkts': '28968827',
                                                                       'poller_hash': 'QURUTjIzMTI0RDQzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317055',
                                                                                                                        'fiber_length': 6063,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDQzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ0Mw==',
                                         'serial_number': 'ADTN23124D43',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317055'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '79111',
                                                                       'in_octets': '35790774369',
                                                                       'in_ucast_pkts': '168984150',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '621131891083',
                                                                       'out_ucast_pkts': '448566543',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '621131891083',
                                                                       'in_ucast_pkts': '448566543',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '621131891083',
                                                                                                'in_ucast_pkts': '448566543',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '79111',
                                                                                                'out_octets': '35790774369',
                                                                                                'out_ucast_pkts': '168984150',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '79111',
                                                                       'out_octets': '35790774369',
                                                                       'out_ucast_pkts': '168984150',
                                                                       'poller_hash': 'QURUTjIzMTI1MUUyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317054',
                                                                                                                        'fiber_length': 5212,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUUyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFFMg==',
                                         'serial_number': 'ADTN231251E2',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317054'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '116891',
                                                                       'in_octets': '49636365763',
                                                                       'in_ucast_pkts': '281729576',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9894',
                                                                       'out_octets': '1329234239498',
                                                                       'out_ucast_pkts': '918260033',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9894',
                                                                       'in_octets': '1329234239498',
                                                                       'in_ucast_pkts': '918260033',
                                                                       'links': [       {       'in_nucast_pkts': '9894',
                                                                                                'in_octets': '1329234239498',
                                                                                                'in_ucast_pkts': '918260033',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '116891',
                                                                                                'out_octets': '49636365763',
                                                                                                'out_ucast_pkts': '281729576',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '116891',
                                                                       'out_octets': '49636365763',
                                                                       'out_ucast_pkts': '281729576',
                                                                       'poller_hash': 'QURUTjIzMTI0NjUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3449056',
                                                                                                                        'fiber_length': 5972,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NjUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDY1MA==',
                                         'serial_number': 'ADTN23124650',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3449056'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '37776',
                                                                       'in_octets': '17380293068',
                                                                       'in_ucast_pkts': '88982916',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3197',
                                                                       'out_octets': '493923471987',
                                                                       'out_ucast_pkts': '350348488',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3197',
                                                                       'in_octets': '493923471987',
                                                                       'in_ucast_pkts': '350348488',
                                                                       'links': [       {       'in_nucast_pkts': '3197',
                                                                                                'in_octets': '493923471987',
                                                                                                'in_ucast_pkts': '350348488',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '37776',
                                                                                                'out_octets': '17380293068',
                                                                                                'out_ucast_pkts': '88982916',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '37776',
                                                                       'out_octets': '17380293068',
                                                                       'out_ucast_pkts': '88982916',
                                                                       'poller_hash': 'QURUTjIzMTI0RTI2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1114341',
                                                                                                                        'fiber_length': 7531,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTI2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyNg==',
                                         'serial_number': 'ADTN23124E26',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1114341'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78535',
                                                                       'in_octets': '88158106536',
                                                                       'in_ucast_pkts': '371249138',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6646',
                                                                       'out_octets': '1100356693456',
                                                                       'out_ucast_pkts': '806203412',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6646',
                                                                       'in_octets': '1100356693456',
                                                                       'in_ucast_pkts': '806203412',
                                                                       'links': [       {       'in_nucast_pkts': '6646',
                                                                                                'in_octets': '1100356693456',
                                                                                                'in_ucast_pkts': '806203412',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78535',
                                                                                                'out_octets': '88158106536',
                                                                                                'out_ucast_pkts': '371249138',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78535',
                                                                       'out_octets': '88158106536',
                                                                       'out_ucast_pkts': '371249138',
                                                                       'poller_hash': 'QURUTjIzMTI0RDk4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317056',
                                                                                                                        'fiber_length': 5967,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDk4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ5OA==',
                                         'serial_number': 'ADTN23124D98',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317056'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78529',
                                                                       'in_octets': '28476675251',
                                                                       'in_ucast_pkts': '142287961',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6647',
                                                                       'out_octets': '953590409355',
                                                                       'out_ucast_pkts': '703346325',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6647',
                                                                       'in_octets': '953590409355',
                                                                       'in_ucast_pkts': '703346325',
                                                                       'links': [       {       'in_nucast_pkts': '6647',
                                                                                                'in_octets': '953590409355',
                                                                                                'in_ucast_pkts': '703346325',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78529',
                                                                                                'out_octets': '28476675251',
                                                                                                'out_ucast_pkts': '142287961',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78529',
                                                                       'out_octets': '28476675251',
                                                                       'out_ucast_pkts': '142287961',
                                                                       'poller_hash': 'QURUTjIzMTI0RDUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317060',
                                                                                                                        'fiber_length': 5915,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ1MA==',
                                         'serial_number': 'ADTN23124D50',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317060'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78632',
                                                                       'in_octets': '2863545110',
                                                                       'in_ucast_pkts': '10210777',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6649',
                                                                       'out_octets': '36091222902',
                                                                       'out_ucast_pkts': '26579330',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6649',
                                                                       'in_octets': '36091222902',
                                                                       'in_ucast_pkts': '26579330',
                                                                       'links': [       {       'in_nucast_pkts': '6649',
                                                                                                'in_octets': '36091222902',
                                                                                                'in_ucast_pkts': '26579330',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78632',
                                                                                                'out_octets': '2863545110',
                                                                                                'out_ucast_pkts': '10210777',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78632',
                                                                       'out_octets': '2863545110',
                                                                       'out_ucast_pkts': '10210777',
                                                                       'poller_hash': 'QURUTjIzMTI0RDM4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317054',
                                                                                                                        'fiber_length': 6210,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDM4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQzOA==',
                                         'serial_number': 'ADTN23124D38',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317054'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-3.1.3.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x3-ONU3_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VM19FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0RTM1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTM1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x3-ONU3',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzNQ==',
                                         'serial_number': 'ADTN23124E35',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '80130',
                                                                       'in_octets': '42976698193',
                                                                       'in_ucast_pkts': '299332771',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6651',
                                                                       'out_octets': '1255152815589',
                                                                       'out_ucast_pkts': '901031913',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6651',
                                                                       'in_octets': '1255152815589',
                                                                       'in_ucast_pkts': '901031913',
                                                                       'links': [       {       'in_nucast_pkts': '6651',
                                                                                                'in_octets': '1255152815589',
                                                                                                'in_ucast_pkts': '901031913',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '80130',
                                                                                                'out_octets': '42976698193',
                                                                                                'out_ucast_pkts': '299332771',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '80130',
                                                                       'out_octets': '42976698193',
                                                                       'out_ucast_pkts': '299332771',
                                                                       'poller_hash': 'QURUTjIzMTI0OTAwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317056',
                                                                                                                        'fiber_length': 6197,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OTAwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDkwMA==',
                                         'serial_number': 'ADTN23124900',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317056'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '66683',
                                                                       'in_octets': '31690461601',
                                                                       'in_ucast_pkts': '130691656',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5640',
                                                                       'out_octets': '376612482296',
                                                                       'out_ucast_pkts': '336192464',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5640',
                                                                       'in_octets': '376612482296',
                                                                       'in_ucast_pkts': '336192464',
                                                                       'links': [       {       'in_nucast_pkts': '5640',
                                                                                                'in_octets': '376612482296',
                                                                                                'in_ucast_pkts': '336192464',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '66683',
                                                                                                'out_octets': '31690461601',
                                                                                                'out_ucast_pkts': '130691656',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '66683',
                                                                       'out_octets': '31690461601',
                                                                       'out_ucast_pkts': '130691656',
                                                                       'poller_hash': 'QURUTjIzMTI1MTYwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1967097',
                                                                                                                        'fiber_length': 4527,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTYwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE2MA==',
                                         'serial_number': 'ADTN23125160',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1967097'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '8874',
                                                                       'in_octets': '1105351558',
                                                                       'in_ucast_pkts': '5692128',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '750',
                                                                       'out_octets': '33797115840',
                                                                       'out_ucast_pkts': '24223033',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '750',
                                                                       'in_octets': '33797115840',
                                                                       'in_ucast_pkts': '24223033',
                                                                       'links': [       {       'in_nucast_pkts': '750',
                                                                                                'in_octets': '33797115840',
                                                                                                'in_ucast_pkts': '24223033',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '8874',
                                                                                                'out_octets': '1105351558',
                                                                                                'out_ucast_pkts': '5692128',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '8874',
                                                                       'out_octets': '1105351558',
                                                                       'out_ucast_pkts': '5692128',
                                                                       'poller_hash': 'QURUTjIzMTI0REREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '261424',
                                                                                                                        'fiber_length': 3424,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERERA==',
                                         'serial_number': 'ADTN23124DDD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '261424'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17378',
                                                                       'in_octets': '4864823600',
                                                                       'in_ucast_pkts': '9100075',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '32633620755',
                                                                       'out_ucast_pkts': '26720543',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '32633620755',
                                                                       'in_ucast_pkts': '26720543',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '32633620755',
                                                                                                'in_ucast_pkts': '26720543',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17378',
                                                                                                'out_octets': '4864823600',
                                                                                                'out_ucast_pkts': '9100075',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17378',
                                                                       'out_octets': '4864823600',
                                                                       'out_ucast_pkts': '9100075',
                                                                       'poller_hash': 'QURUTjIzMTI0RTJEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 4867,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTJEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyRA==',
                                         'serial_number': 'ADTN23124E2D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77250',
                                                                       'in_octets': '31411288346',
                                                                       'in_ucast_pkts': '99595628',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2567',
                                                                       'out_octets': '402318506803',
                                                                       'out_ucast_pkts': '294620515',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VNl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2567',
                                                                       'in_octets': '402318506803',
                                                                       'in_ucast_pkts': '294620515',
                                                                       'links': [       {       'in_nucast_pkts': '2567',
                                                                                                'in_octets': '402318506803',
                                                                                                'in_ucast_pkts': '294620515',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77250',
                                                                                                'out_octets': '31411288346',
                                                                                                'out_ucast_pkts': '99595628',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77250',
                                                                       'out_octets': '31411288346',
                                                                       'out_ucast_pkts': '99595628',
                                                                       'poller_hash': 'QURUTjIzMTI0NUM2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317059',
                                                                                                                        'fiber_length': 3878,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NUM2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDVDNg==',
                                         'serial_number': 'ADTN231245C6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317059'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17128',
                                                                       'in_octets': '12164965242',
                                                                       'in_ucast_pkts': '54352699',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '173894020013',
                                                                       'out_ucast_pkts': '132678844',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '173894020013',
                                                                       'in_ucast_pkts': '132678844',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '173894020013',
                                                                                                'in_ucast_pkts': '132678844',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17128',
                                                                                                'out_octets': '12164965242',
                                                                                                'out_ucast_pkts': '54352699',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17128',
                                                                       'out_octets': '12164965242',
                                                                       'out_ucast_pkts': '54352699',
                                                                       'poller_hash': 'QURUTjIzMTI0MUQxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511490',
                                                                                                                        'fiber_length': 2471,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MUQxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDFEMQ==',
                                         'serial_number': 'ADTN231241D1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511490'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-8.1.16.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x8-ONU16_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTZfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0RERGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RERGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x8-ONU16',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '8.1.16"(ADTN23124DDF) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/8.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-08-26T02:42:02Z',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERERg==',
                                         'serial_number': 'ADTN23124DDF',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '29041',
                                                                       'in_octets': '88986655438',
                                                                       'in_ucast_pkts': '119673652',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124DA6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2456',
                                                                       'out_octets': '271763618943',
                                                                       'out_ucast_pkts': '249838064',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0REE2X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2456',
                                                                       'in_octets': '271763618943',
                                                                       'in_ucast_pkts': '249838064',
                                                                       'links': [       {       'in_nucast_pkts': '2456',
                                                                                                'in_octets': '271763618943',
                                                                                                'in_ucast_pkts': '249838064',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '29041',
                                                                                                'out_octets': '88986655438',
                                                                                                'out_ucast_pkts': '119673652',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '29041',
                                                                       'out_octets': '88986655438',
                                                                       'out_ucast_pkts': '119673652',
                                                                       'poller_hash': 'QURUTjIzMTI0REE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '856063',
                                                                                                                        'fiber_length': 5510,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124DA6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERBNg==',
                                         'serial_number': 'ADTN23124DA6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '856063'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '24484',
                                                                       'in_octets': '6556027515',
                                                                       'in_ucast_pkts': '13343348',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2072',
                                                                       'out_octets': '40619971557',
                                                                       'out_ucast_pkts': '34613725',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2072',
                                                                       'in_octets': '40619971557',
                                                                       'in_ucast_pkts': '34613725',
                                                                       'links': [       {       'in_nucast_pkts': '2072',
                                                                                                'in_octets': '40619971557',
                                                                                                'in_ucast_pkts': '34613725',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '24484',
                                                                                                'out_octets': '6556027515',
                                                                                                'out_ucast_pkts': '13343348',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '24484',
                                                                       'out_octets': '6556027515',
                                                                       'out_ucast_pkts': '13343348',
                                                                       'poller_hash': 'QURUTjIzMTI1MDIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '721987',
                                                                                                                        'fiber_length': 5554,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTAyMA==',
                                         'serial_number': 'ADTN23125020',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '721987'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '37777',
                                                                       'in_octets': '2961602648',
                                                                       'in_ucast_pkts': '26561230',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3197',
                                                                       'out_octets': '82802554675',
                                                                       'out_ucast_pkts': '56265867',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VOV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3197',
                                                                       'in_octets': '82802554675',
                                                                       'in_ucast_pkts': '56265867',
                                                                       'links': [       {       'in_nucast_pkts': '3197',
                                                                                                'in_octets': '82802554675',
                                                                                                'in_ucast_pkts': '56265867',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '37777',
                                                                                                'out_octets': '2961602648',
                                                                                                'out_ucast_pkts': '26561230',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '37777',
                                                                       'out_octets': '2961602648',
                                                                       'out_ucast_pkts': '26561230',
                                                                       'poller_hash': 'QURUTjIzMTI0RTE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1114341',
                                                                                                                        'fiber_length': 7481,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxNg==',
                                         'serial_number': 'ADTN23124E16',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1114341'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17346',
                                                                       'in_octets': '8328880127',
                                                                       'in_ucast_pkts': '43387796',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '148958989222',
                                                                       'out_ucast_pkts': '105207087',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '148958989222',
                                                                       'in_ucast_pkts': '105207087',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '148958989222',
                                                                                                'in_ucast_pkts': '105207087',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17346',
                                                                                                'out_octets': '8328880127',
                                                                                                'out_ucast_pkts': '43387796',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17346',
                                                                       'out_octets': '8328880127',
                                                                       'out_ucast_pkts': '43387796',
                                                                       'poller_hash': 'QURUTjIzMTI0RjNDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511495',
                                                                                                                        'fiber_length': 1807,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjNDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYzQw==',
                                         'serial_number': 'ADTN23124F3C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511495'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17492',
                                                                       'in_octets': '5143788698',
                                                                       'in_ucast_pkts': '12155920',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '52724693825',
                                                                       'out_ucast_pkts': '41538448',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '52724693825',
                                                                       'in_ucast_pkts': '41538448',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '52724693825',
                                                                                                'in_ucast_pkts': '41538448',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17492',
                                                                                                'out_octets': '5143788698',
                                                                                                'out_ucast_pkts': '12155920',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17492',
                                                                       'out_octets': '5143788698',
                                                                       'out_ucast_pkts': '12155920',
                                                                       'poller_hash': 'QURUTjIzMTI0RjJEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511502',
                                                                                                                        'fiber_length': 4645,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjJEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYyRA==',
                                         'serial_number': 'ADTN23124F2D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511502'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17075',
                                                                       'in_octets': '11346630986',
                                                                       'in_ucast_pkts': '57003429',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '229678363616',
                                                                       'out_ucast_pkts': '168885489',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTdfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '229678363616',
                                                                       'in_ucast_pkts': '168885489',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '229678363616',
                                                                                                'in_ucast_pkts': '168885489',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17075',
                                                                                                'out_octets': '11346630986',
                                                                                                'out_ucast_pkts': '57003429',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17075',
                                                                       'out_octets': '11346630986',
                                                                       'out_ucast_pkts': '57003429',
                                                                       'poller_hash': 'QURUTjIzMTI0MDhDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511501',
                                                                                                                        'fiber_length': 5423,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MDhDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDA4Qw==',
                                         'serial_number': 'ADTN2312408C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511501'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '185353',
                                                                       'in_octets': '62164914093',
                                                                       'in_ucast_pkts': '236837758',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15625',
                                                                       'out_octets': '875539994170',
                                                                       'out_ucast_pkts': '653727994',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15625',
                                                                       'in_octets': '875539994170',
                                                                       'in_ucast_pkts': '653727994',
                                                                       'links': [       {       'in_nucast_pkts': '15625',
                                                                                                'in_octets': '875539994170',
                                                                                                'in_ucast_pkts': '653727994',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '185353',
                                                                                                'out_octets': '62164914093',
                                                                                                'out_ucast_pkts': '236837758',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '185353',
                                                                       'out_octets': '62164914093',
                                                                       'out_ucast_pkts': '236837758',
                                                                       'poller_hash': 'QURUTjIzMTI0RDdFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440106',
                                                                                                                        'fiber_length': 6288,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDdFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ3RQ==',
                                         'serial_number': 'ADTN23124D7E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440106'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '112203',
                                                                       'in_octets': '178265876806',
                                                                       'in_ucast_pkts': '377896222',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3734',
                                                                       'out_octets': '974919567981',
                                                                       'out_ucast_pkts': '749790248',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3734',
                                                                       'in_octets': '974919567981',
                                                                       'in_ucast_pkts': '749790248',
                                                                       'links': [       {       'in_nucast_pkts': '3734',
                                                                                                'in_octets': '974919567981',
                                                                                                'in_ucast_pkts': '749790248',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '112203',
                                                                                                'out_octets': '178265876806',
                                                                                                'out_ucast_pkts': '377896222',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '112203',
                                                                       'out_octets': '178265876806',
                                                                       'out_ucast_pkts': '377896222',
                                                                       'poller_hash': 'QURUTjIzMTI0RDdEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3363454',
                                                                                                                        'fiber_length': 4613,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDdEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ3RA==',
                                         'serial_number': 'ADTN23124D7D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3363454'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17349',
                                                                       'in_octets': '44042595027',
                                                                       'in_ucast_pkts': '73274929',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '235157386943',
                                                                       'out_ucast_pkts': '174144687',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '235157386943',
                                                                       'in_ucast_pkts': '174144687',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '235157386943',
                                                                                                'in_ucast_pkts': '174144687',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17349',
                                                                                                'out_octets': '44042595027',
                                                                                                'out_ucast_pkts': '73274929',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17349',
                                                                       'out_octets': '44042595027',
                                                                       'out_ucast_pkts': '73274929',
                                                                       'poller_hash': 'QURUTjIzMTI1MDFEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511506',
                                                                                                                        'fiber_length': 4938,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDFEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTAxRA==',
                                         'serial_number': 'ADTN2312501D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511506'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17492',
                                                                       'in_octets': '9017069532',
                                                                       'in_ucast_pkts': '23865459',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '73342838858',
                                                                       'out_ucast_pkts': '61095120',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '73342838858',
                                                                       'in_ucast_pkts': '61095120',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '73342838858',
                                                                                                'in_ucast_pkts': '61095120',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17492',
                                                                                                'out_octets': '9017069532',
                                                                                                'out_ucast_pkts': '23865459',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17492',
                                                                       'out_octets': '9017069532',
                                                                       'out_ucast_pkts': '23865459',
                                                                       'poller_hash': 'QURUTjIzMTI0REZDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511501',
                                                                                                                        'fiber_length': 1986,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REZDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGQw==',
                                         'serial_number': 'ADTN23124DFC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511501'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17358',
                                                                       'in_octets': '8390216571',
                                                                       'in_ucast_pkts': '66250060',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '324869683562',
                                                                       'out_ucast_pkts': '223917456',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTlfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '324869683562',
                                                                       'in_ucast_pkts': '223917456',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '324869683562',
                                                                                                'in_ucast_pkts': '223917456',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17358',
                                                                                                'out_octets': '8390216571',
                                                                                                'out_ucast_pkts': '66250060',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17358',
                                                                       'out_octets': '8390216571',
                                                                       'out_ucast_pkts': '66250060',
                                                                       'poller_hash': 'QURUTjIzMTI0REY1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 5198,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REY1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGNQ==',
                                         'serial_number': 'ADTN23124DF5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17785',
                                                                       'in_octets': '5986842931',
                                                                       'in_ucast_pkts': '23664046',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1466',
                                                                       'out_octets': '91673181754',
                                                                       'out_ucast_pkts': '66103257',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1466',
                                                                       'in_octets': '91673181754',
                                                                       'in_ucast_pkts': '66103257',
                                                                       'links': [       {       'in_nucast_pkts': '1466',
                                                                                                'in_octets': '91673181754',
                                                                                                'in_ucast_pkts': '66103257',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17785',
                                                                                                'out_octets': '5986842931',
                                                                                                'out_ucast_pkts': '23664046',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17785',
                                                                       'out_octets': '5986842931',
                                                                       'out_ucast_pkts': '23664046',
                                                                       'poller_hash': 'QURUTjIzMTI0RjMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511497',
                                                                                                                        'fiber_length': 2396,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYzMA==',
                                         'serial_number': 'ADTN23124F30',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511497'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78535',
                                                                       'in_octets': '125019801865',
                                                                       'in_ucast_pkts': '590023136',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6649',
                                                                       'out_octets': '1927420214171',
                                                                       'out_ucast_pkts': '1436718789',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6649',
                                                                       'in_octets': '1927420214171',
                                                                       'in_ucast_pkts': '1436718789',
                                                                       'links': [       {       'in_nucast_pkts': '6649',
                                                                                                'in_octets': '1927420214171',
                                                                                                'in_ucast_pkts': '1436718789',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78535',
                                                                                                'out_octets': '125019801865',
                                                                                                'out_ucast_pkts': '590023136',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78535',
                                                                       'out_octets': '125019801865',
                                                                       'out_ucast_pkts': '590023136',
                                                                       'poller_hash': 'QURUTjIzMTI0RjJCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317057',
                                                                                                                        'fiber_length': 6120,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjJCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYyQg==',
                                         'serial_number': 'ADTN23124F2B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317057'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '80462',
                                                                       'in_octets': '34208278773',
                                                                       'in_ucast_pkts': '133402787',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23123FAD_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6631',
                                                                       'out_octets': '379173521004',
                                                                       'out_ucast_pkts': '272048012',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTIzRkFEX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6631',
                                                                       'in_octets': '379173521004',
                                                                       'in_ucast_pkts': '272048012',
                                                                       'links': [       {       'in_nucast_pkts': '6631',
                                                                                                'in_octets': '379173521004',
                                                                                                'in_ucast_pkts': '272048012',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '80462',
                                                                                                'out_octets': '34208278773',
                                                                                                'out_ucast_pkts': '133402787',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '80462',
                                                                       'out_octets': '34208278773',
                                                                       'out_ucast_pkts': '133402787',
                                                                       'poller_hash': 'QURUTjIzMTIzRkFEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317055',
                                                                                                                        'fiber_length': 3684,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkFEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23123FAD',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZBRA==',
                                         'serial_number': 'ADTN23123FAD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317055'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17349',
                                                                       'in_octets': '13287612317',
                                                                       'in_ucast_pkts': '83805288',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '266246543460',
                                                                       'out_ucast_pkts': '186300386',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '266246543460',
                                                                       'in_ucast_pkts': '186300386',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '266246543460',
                                                                                                'in_ucast_pkts': '186300386',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17349',
                                                                                                'out_octets': '13287612317',
                                                                                                'out_ucast_pkts': '83805288',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17349',
                                                                       'out_octets': '13287612317',
                                                                       'out_ucast_pkts': '83805288',
                                                                       'poller_hash': 'QURUTjIzMTI0NzEzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511499',
                                                                                                                        'fiber_length': 5184,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NzEzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDcxMw==',
                                         'serial_number': 'ADTN23124713',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '1343',
                                                                       'in_octets': '30099881929',
                                                                       'in_ucast_pkts': '135547910',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124F99_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6629',
                                                                       'out_octets': '689845211523',
                                                                       'out_ucast_pkts': '493316076',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0Rjk5X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6629',
                                                                       'in_octets': '689845211523',
                                                                       'in_ucast_pkts': '493316076',
                                                                       'links': [       {       'in_nucast_pkts': '6629',
                                                                                                'in_octets': '689845211523',
                                                                                                'in_ucast_pkts': '493316076',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '1343',
                                                                                                'out_octets': '30099881929',
                                                                                                'out_ucast_pkts': '135547910',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1343',
                                                                       'out_octets': '30099881929',
                                                                       'out_ucast_pkts': '135547910',
                                                                       'poller_hash': 'QURUTjIzMTI0Rjk5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317057',
                                                                                                                        'fiber_length': 3674,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0Rjk5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124F99',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEY5OQ==',
                                         'serial_number': 'ADTN23124F99',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317057'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17642',
                                                                       'in_octets': '13216747722',
                                                                       'in_ucast_pkts': '77811734',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '213176396900',
                                                                       'out_ucast_pkts': '148724509',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '213176396900',
                                                                       'in_ucast_pkts': '148724509',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '213176396900',
                                                                                                'in_ucast_pkts': '148724509',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17642',
                                                                                                'out_octets': '13216747722',
                                                                                                'out_ucast_pkts': '77811734',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17642',
                                                                       'out_octets': '13216747722',
                                                                       'out_ucast_pkts': '77811734',
                                                                       'poller_hash': 'QURUTjIzMTI0RTE5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511513',
                                                                                                                        'fiber_length': 5202,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTE5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxOQ==',
                                         'serial_number': 'ADTN23124E19',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511513'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17767',
                                                                       'in_octets': '19444386289',
                                                                       'in_ucast_pkts': '36348646',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '144731181340',
                                                                       'out_ucast_pkts': '110500290',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '144731181340',
                                                                       'in_ucast_pkts': '110500290',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '144731181340',
                                                                                                'in_ucast_pkts': '110500290',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17767',
                                                                                                'out_octets': '19444386289',
                                                                                                'out_ucast_pkts': '36348646',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17767',
                                                                       'out_octets': '19444386289',
                                                                       'out_ucast_pkts': '36348646',
                                                                       'poller_hash': 'QURUTjIzMTI1MDI2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511502',
                                                                                                                        'fiber_length': 4747,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDI2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTAyNg==',
                                         'serial_number': 'ADTN23125026',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511502'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '62088',
                                                                       'in_octets': '69782859508',
                                                                       'in_ucast_pkts': '222531899',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2020',
                                                                       'out_octets': '560038723744',
                                                                       'out_ucast_pkts': '407371035',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2020',
                                                                       'in_octets': '560038723744',
                                                                       'in_ucast_pkts': '407371035',
                                                                       'links': [       {       'in_nucast_pkts': '2020',
                                                                                                'in_octets': '560038723744',
                                                                                                'in_ucast_pkts': '407371035',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '62088',
                                                                                                'out_octets': '69782859508',
                                                                                                'out_ucast_pkts': '222531899',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '62088',
                                                                       'out_octets': '69782859508',
                                                                       'out_ucast_pkts': '222531899',
                                                                       'poller_hash': 'QURUTjIzMTI0RjA2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1816875',
                                                                                                                        'fiber_length': 4267,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjA2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYwNg==',
                                         'serial_number': 'ADTN23124F06',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1816875'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17343',
                                                                       'in_octets': '6881121985',
                                                                       'in_ucast_pkts': '10791436',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '27921309796',
                                                                       'out_ucast_pkts': '20288820',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '27921309796',
                                                                       'in_ucast_pkts': '20288820',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '27921309796',
                                                                                                'in_ucast_pkts': '20288820',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17343',
                                                                                                'out_octets': '6881121985',
                                                                                                'out_ucast_pkts': '10791436',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17343',
                                                                       'out_octets': '6881121985',
                                                                       'out_ucast_pkts': '10791436',
                                                                       'poller_hash': 'QURUTjIzMTIzRjU3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511503',
                                                                                                                        'fiber_length': 4661,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRjU3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0Y1Nw==',
                                         'serial_number': 'ADTN23123F57',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511503'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17337',
                                                                       'in_octets': '11797895551',
                                                                       'in_ucast_pkts': '77635048',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1465',
                                                                       'out_octets': '301282269730',
                                                                       'out_ucast_pkts': '209671611',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1465',
                                                                       'in_octets': '301282269730',
                                                                       'in_ucast_pkts': '209671611',
                                                                       'links': [       {       'in_nucast_pkts': '1465',
                                                                                                'in_octets': '301282269730',
                                                                                                'in_ucast_pkts': '209671611',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17337',
                                                                                                'out_octets': '11797895551',
                                                                                                'out_ucast_pkts': '77635048',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17337',
                                                                       'out_octets': '11797895551',
                                                                       'out_ucast_pkts': '77635048',
                                                                       'poller_hash': 'QURUTjIzMTI0REIyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511514',
                                                                                                                        'fiber_length': 5688,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REIyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERCMg==',
                                         'serial_number': 'ADTN23124DB2',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511514'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '51160',
                                                                       'in_octets': '54907922403',
                                                                       'in_ucast_pkts': '175360064',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1706',
                                                                       'out_octets': '621698507690',
                                                                       'out_ucast_pkts': '454730480',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1706',
                                                                       'in_octets': '621698507690',
                                                                       'in_ucast_pkts': '454730480',
                                                                       'links': [       {       'in_nucast_pkts': '1706',
                                                                                                'in_octets': '621698507690',
                                                                                                'in_ucast_pkts': '454730480',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '51160',
                                                                                                'out_octets': '54907922403',
                                                                                                'out_ucast_pkts': '175360064',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '51160',
                                                                       'out_octets': '54907922403',
                                                                       'out_ucast_pkts': '175360064',
                                                                       'poller_hash': 'QURUTjIzMTIzRjJCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1534011',
                                                                                                                        'fiber_length': 5584,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRjJCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0YyQg==',
                                         'serial_number': 'ADTN23123F2B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1534011'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17355',
                                                                       'in_octets': '3607712444',
                                                                       'in_ucast_pkts': '16831967',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1468',
                                                                       'out_octets': '60418812521',
                                                                       'out_ucast_pkts': '45761191',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1468',
                                                                       'in_octets': '60418812521',
                                                                       'in_ucast_pkts': '45761191',
                                                                       'links': [       {       'in_nucast_pkts': '1468',
                                                                                                'in_octets': '60418812521',
                                                                                                'in_ucast_pkts': '45761191',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17355',
                                                                                                'out_octets': '3607712444',
                                                                                                'out_ucast_pkts': '16831967',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17355',
                                                                       'out_octets': '3607712444',
                                                                       'out_ucast_pkts': '16831967',
                                                                       'poller_hash': 'QURUTjIzMTI0RTI5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511514',
                                                                                                                        'fiber_length': 5757,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTI5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyOQ==',
                                         'serial_number': 'ADTN23124E29',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511514'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17777',
                                                                       'in_octets': '19955393626',
                                                                       'in_ucast_pkts': '80533969',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '296743546858',
                                                                       'out_ucast_pkts': '220008288',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '296743546858',
                                                                       'in_ucast_pkts': '220008288',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '296743546858',
                                                                                                'in_ucast_pkts': '220008288',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17777',
                                                                                                'out_octets': '19955393626',
                                                                                                'out_ucast_pkts': '80533969',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17777',
                                                                       'out_octets': '19955393626',
                                                                       'out_ucast_pkts': '80533969',
                                                                       'poller_hash': 'QURUTjIzMTI1MDg4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511502',
                                                                                                                        'fiber_length': 4852,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDg4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA4OA==',
                                         'serial_number': 'ADTN23125088',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511502'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17348',
                                                                       'in_octets': '15904022055',
                                                                       'in_ucast_pkts': '42189853',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '114430149310',
                                                                       'out_ucast_pkts': '95211145',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '114430149310',
                                                                       'in_ucast_pkts': '95211145',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '114430149310',
                                                                                                'in_ucast_pkts': '95211145',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17348',
                                                                                                'out_octets': '15904022055',
                                                                                                'out_ucast_pkts': '42189853',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17348',
                                                                       'out_octets': '15904022055',
                                                                       'out_ucast_pkts': '42189853',
                                                                       'poller_hash': 'QURUTjIzMTI1MTQ2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511496',
                                                                                                                        'fiber_length': 2088,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0Ng==',
                                         'serial_number': 'ADTN23125146',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511496'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78543',
                                                                       'in_octets': '16544493187',
                                                                       'in_ucast_pkts': '103247590',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6628',
                                                                       'out_octets': '277598045489',
                                                                       'out_ucast_pkts': '202126901',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6628',
                                                                       'in_octets': '277598045489',
                                                                       'in_ucast_pkts': '202126901',
                                                                       'links': [       {       'in_nucast_pkts': '6628',
                                                                                                'in_octets': '277598045489',
                                                                                                'in_ucast_pkts': '202126901',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78543',
                                                                                                'out_octets': '16544493187',
                                                                                                'out_ucast_pkts': '103247590',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78543',
                                                                       'out_octets': '16544493187',
                                                                       'out_ucast_pkts': '103247590',
                                                                       'poller_hash': 'QURUTjIzMTI1MTdEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317056',
                                                                                                                        'fiber_length': 3861,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTdEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE3RA==',
                                         'serial_number': 'ADTN2312517D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317056'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17345',
                                                                       'in_octets': '13570085705',
                                                                       'in_ucast_pkts': '46186668',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23125166_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '166640051211',
                                                                       'out_ucast_pkts': '149119825',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI1MTY2X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '166640051211',
                                                                       'in_ucast_pkts': '149119825',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '166640051211',
                                                                                                'in_ucast_pkts': '149119825',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17345',
                                                                                                'out_octets': '13570085705',
                                                                                                'out_ucast_pkts': '46186668',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17345',
                                                                       'out_octets': '13570085705',
                                                                       'out_ucast_pkts': '46186668',
                                                                       'poller_hash': 'QURUTjIzMTI1MTY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511499',
                                                                                                                        'fiber_length': 5195,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23125166',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE2Ng==',
                                         'serial_number': 'ADTN23125166',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17351',
                                                                       'in_octets': '4500406894',
                                                                       'in_ucast_pkts': '13799152',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '64753526744',
                                                                       'out_ucast_pkts': '51310902',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '64753526744',
                                                                       'in_ucast_pkts': '51310902',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '64753526744',
                                                                                                'in_ucast_pkts': '51310902',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17351',
                                                                                                'out_octets': '4500406894',
                                                                                                'out_ucast_pkts': '13799152',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17351',
                                                                       'out_octets': '4500406894',
                                                                       'out_ucast_pkts': '13799152',
                                                                       'poller_hash': 'QURUTjIzMTI0RTY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511511',
                                                                                                                        'fiber_length': 5100,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU22',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU2Nw==',
                                         'serial_number': 'ADTN23124E67',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511511'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13736',
                                                                       'in_octets': '6225522635',
                                                                       'in_ucast_pkts': '24138193',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23125190_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '84342536497',
                                                                       'out_ucast_pkts': '63254582',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI1MTkwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '84342536497',
                                                                       'in_ucast_pkts': '63254582',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '84342536497',
                                                                                                'in_ucast_pkts': '63254582',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13736',
                                                                                                'out_octets': '6225522635',
                                                                                                'out_ucast_pkts': '24138193',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13736',
                                                                       'out_octets': '6225522635',
                                                                       'out_ucast_pkts': '24138193',
                                                                       'poller_hash': 'QURUTjIzMTI1MTkwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411498',
                                                                                                                        'fiber_length': 6895,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTkwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23125190',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE5MA==',
                                         'serial_number': 'ADTN23125190',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17359',
                                                                       'in_octets': '7555932399',
                                                                       'in_ucast_pkts': '48517272',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1468',
                                                                       'out_octets': '272090504758',
                                                                       'out_ucast_pkts': '194566979',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VOF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1468',
                                                                       'in_octets': '272090504758',
                                                                       'in_ucast_pkts': '194566979',
                                                                       'links': [       {       'in_nucast_pkts': '1468',
                                                                                                'in_octets': '272090504758',
                                                                                                'in_ucast_pkts': '194566979',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17359',
                                                                                                'out_octets': '7555932399',
                                                                                                'out_ucast_pkts': '48517272',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17359',
                                                                       'out_octets': '7555932399',
                                                                       'out_ucast_pkts': '48517272',
                                                                       'poller_hash': 'QURUTjIzMTI0MUQwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511510',
                                                                                                                        'fiber_length': 5845,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MUQwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDFEMA==',
                                         'serial_number': 'ADTN231241D0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511510'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17347',
                                                                       'in_octets': '5943079236',
                                                                       'in_ucast_pkts': '24328987',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '170208238795',
                                                                       'out_ucast_pkts': '125604721',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '170208238795',
                                                                       'in_ucast_pkts': '125604721',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '170208238795',
                                                                                                'in_ucast_pkts': '125604721',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17347',
                                                                                                'out_octets': '5943079236',
                                                                                                'out_ucast_pkts': '24328987',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17347',
                                                                       'out_octets': '5943079236',
                                                                       'out_ucast_pkts': '24328987',
                                                                       'poller_hash': 'QURUTjIzMTI0RTQwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511500',
                                                                                                                        'fiber_length': 5726,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTQwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU0MA==',
                                         'serial_number': 'ADTN23124E40',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '37809',
                                                                       'in_octets': '8907565649',
                                                                       'in_ucast_pkts': '69442996',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124E0D_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3197',
                                                                       'out_octets': '174818309953',
                                                                       'out_ucast_pkts': '127345107',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RTBEX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3197',
                                                                       'in_octets': '174818309953',
                                                                       'in_ucast_pkts': '127345107',
                                                                       'links': [       {       'in_nucast_pkts': '3197',
                                                                                                'in_octets': '174818309953',
                                                                                                'in_ucast_pkts': '127345107',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '37809',
                                                                                                'out_octets': '8907565649',
                                                                                                'out_ucast_pkts': '69442996',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '37809',
                                                                       'out_octets': '8907565649',
                                                                       'out_ucast_pkts': '69442996',
                                                                       'poller_hash': 'QURUTjIzMTI0RTBEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1114341',
                                                                                                                        'fiber_length': 7351,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTBEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124E0D',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUwRA==',
                                         'serial_number': 'ADTN23124E0D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1114341'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14301',
                                                                       'in_octets': '23521366774',
                                                                       'in_ucast_pkts': '40895445',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '91736619845',
                                                                       'out_ucast_pkts': '75806891',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '91736619845',
                                                                       'in_ucast_pkts': '75806891',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '91736619845',
                                                                                                'in_ucast_pkts': '75806891',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14301',
                                                                                                'out_octets': '23521366774',
                                                                                                'out_ucast_pkts': '40895445',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14301',
                                                                       'out_octets': '23521366774',
                                                                       'out_ucast_pkts': '40895445',
                                                                       'poller_hash': 'QURUTjIzMTI0RTcxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411496',
                                                                                                                        'fiber_length': 8742,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTcxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU3MQ==',
                                         'serial_number': 'ADTN23124E71',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411496'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17413',
                                                                       'in_octets': '21108613447',
                                                                       'in_ucast_pkts': '66454572',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1472',
                                                                       'out_octets': '265124843284',
                                                                       'out_ucast_pkts': '197979817',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1472',
                                                                       'in_octets': '265124843284',
                                                                       'in_ucast_pkts': '197979817',
                                                                       'links': [       {       'in_nucast_pkts': '1472',
                                                                                                'in_octets': '265124843284',
                                                                                                'in_ucast_pkts': '197979817',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17413',
                                                                                                'out_octets': '21108613447',
                                                                                                'out_ucast_pkts': '66454572',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17413',
                                                                       'out_octets': '21108613447',
                                                                       'out_ucast_pkts': '66454572',
                                                                       'poller_hash': 'QURUTjIzMTI0RTNGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511512',
                                                                                                                        'fiber_length': 5421,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTNGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzRg==',
                                         'serial_number': 'ADTN23124E3F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511512'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17774',
                                                                       'in_octets': '2445008553',
                                                                       'in_ucast_pkts': '17998019',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '87124072293',
                                                                       'out_ucast_pkts': '62663530',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '87124072293',
                                                                       'in_ucast_pkts': '62663530',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '87124072293',
                                                                                                'in_ucast_pkts': '62663530',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17774',
                                                                                                'out_octets': '2445008553',
                                                                                                'out_ucast_pkts': '17998019',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17774',
                                                                       'out_octets': '2445008553',
                                                                       'out_ucast_pkts': '17998019',
                                                                       'poller_hash': 'QURUTjIzMTI0RUQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511500',
                                                                                                                        'fiber_length': 5679,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RUQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVEOQ==',
                                         'serial_number': 'ADTN23124ED9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77259',
                                                                       'in_octets': '30947372870',
                                                                       'in_ucast_pkts': '161571631',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2570',
                                                                       'out_octets': '942520427841',
                                                                       'out_ucast_pkts': '685844825',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2570',
                                                                       'in_octets': '942520427841',
                                                                       'in_ucast_pkts': '685844825',
                                                                       'links': [       {       'in_nucast_pkts': '2570',
                                                                                                'in_octets': '942520427841',
                                                                                                'in_ucast_pkts': '685844825',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77259',
                                                                                                'out_octets': '30947372870',
                                                                                                'out_ucast_pkts': '161571631',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77259',
                                                                       'out_octets': '30947372870',
                                                                       'out_ucast_pkts': '161571631',
                                                                       'poller_hash': 'QURUTjIzMTI0REU4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317057',
                                                                                                                        'fiber_length': 3367,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REU4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERFOA==',
                                         'serial_number': 'ADTN23124DE8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317057'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '184357',
                                                                       'in_octets': '261894506093',
                                                                       'in_ucast_pkts': '730736385',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15616',
                                                                       'out_octets': '2036903540767',
                                                                       'out_ucast_pkts': '1780740679',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTRfRVRIXzE=',
                                                                       'speed': '100000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15616',
                                                                       'in_octets': '2036903540767',
                                                                       'in_ucast_pkts': '1780740679',
                                                                       'links': [       {       'in_nucast_pkts': '15616',
                                                                                                'in_octets': '2036903540767',
                                                                                                'in_ucast_pkts': '1780740679',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '184357',
                                                                                                'out_octets': '261894506093',
                                                                                                'out_ucast_pkts': '730736385',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '184357',
                                                                       'out_octets': '261894506093',
                                                                       'out_ucast_pkts': '730736385',
                                                                       'poller_hash': 'QURUTjIzMTI0RTNFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440063',
                                                                                                                        'fiber_length': 5805,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTNFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzRQ==',
                                         'serial_number': 'ADTN23124E3E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5-5-b',
                                         'uptime': '5440063'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13959',
                                                                       'in_octets': '21097991933',
                                                                       'in_ucast_pkts': '51157611',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '245583176080',
                                                                       'out_ucast_pkts': '187777485',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '245583176080',
                                                                       'in_ucast_pkts': '187777485',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '245583176080',
                                                                                                'in_ucast_pkts': '187777485',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13959',
                                                                                                'out_octets': '21097991933',
                                                                                                'out_ucast_pkts': '51157611',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13959',
                                                                       'out_octets': '21097991933',
                                                                       'out_ucast_pkts': '51157611',
                                                                       'poller_hash': 'QURUTjIzMTI1MUVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411498',
                                                                                                                        'fiber_length': 6685,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFFQg==',
                                         'serial_number': 'ADTN231251EB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14301',
                                                                       'in_octets': '4925425766',
                                                                       'in_ucast_pkts': '26949352',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '166267130463',
                                                                       'out_ucast_pkts': '116451702',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '166267130463',
                                                                       'in_ucast_pkts': '116451702',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '166267130463',
                                                                                                'in_ucast_pkts': '116451702',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14301',
                                                                                                'out_octets': '4925425766',
                                                                                                'out_ucast_pkts': '26949352',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14301',
                                                                       'out_octets': '4925425766',
                                                                       'out_ucast_pkts': '26949352',
                                                                       'poller_hash': 'QURUTjIzMTI0RERBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411500',
                                                                                                                        'fiber_length': 8706,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RERBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEREQQ==',
                                         'serial_number': 'ADTN23124DDA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14301',
                                                                       'in_octets': '5965607609',
                                                                       'in_ucast_pkts': '18743666',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '83538905826',
                                                                       'out_ucast_pkts': '65251732',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '83538905826',
                                                                       'in_ucast_pkts': '65251732',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '83538905826',
                                                                                                'in_ucast_pkts': '65251732',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14301',
                                                                                                'out_octets': '5965607609',
                                                                                                'out_ucast_pkts': '18743666',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14301',
                                                                       'out_octets': '5965607609',
                                                                       'out_ucast_pkts': '18743666',
                                                                       'poller_hash': 'QURUTjIzMTI0REUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411497',
                                                                                                                        'fiber_length': 8348,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERFMA==',
                                         'serial_number': 'ADTN23124DE0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411497'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17352',
                                                                       'in_octets': '12583730852',
                                                                       'in_ucast_pkts': '15238765',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '32349249168',
                                                                       'out_ucast_pkts': '27675715',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '32349249168',
                                                                       'in_ucast_pkts': '27675715',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '32349249168',
                                                                                                'in_ucast_pkts': '27675715',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17352',
                                                                                                'out_octets': '12583730852',
                                                                                                'out_ucast_pkts': '15238765',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17352',
                                                                       'out_octets': '12583730852',
                                                                       'out_ucast_pkts': '15238765',
                                                                       'poller_hash': 'QURUTjIzMTI0REZCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511510',
                                                                                                                        'fiber_length': 2183,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REZCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGQg==',
                                         'serial_number': 'ADTN23124DFB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511510'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13960',
                                                                       'in_octets': '4994513603',
                                                                       'in_ucast_pkts': '13310420',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '44519148743',
                                                                       'out_ucast_pkts': '38135069',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '44519148743',
                                                                       'in_ucast_pkts': '38135069',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '44519148743',
                                                                                                'in_ucast_pkts': '38135069',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13960',
                                                                                                'out_octets': '4994513603',
                                                                                                'out_ucast_pkts': '13310420',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13960',
                                                                       'out_octets': '4994513603',
                                                                       'out_ucast_pkts': '13310420',
                                                                       'poller_hash': 'QURUTjIzMTI0REU0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411500',
                                                                                                                        'fiber_length': 6822,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REU0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERFNA==',
                                         'serial_number': 'ADTN23124DE4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17497',
                                                                       'in_octets': '12357854316',
                                                                       'in_ucast_pkts': '39057101',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '195856588153',
                                                                       'out_ucast_pkts': '146883781',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VOV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '195856588153',
                                                                       'in_ucast_pkts': '146883781',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '195856588153',
                                                                                                'in_ucast_pkts': '146883781',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17497',
                                                                                                'out_octets': '12357854316',
                                                                                                'out_ucast_pkts': '39057101',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17497',
                                                                       'out_octets': '12357854316',
                                                                       'out_ucast_pkts': '39057101',
                                                                       'poller_hash': 'QURUTjIzMTI0REJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511514',
                                                                                                                        'fiber_length': 6895,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERCQw==',
                                         'serial_number': 'ADTN23124DBC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511514'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77271',
                                                                       'in_octets': '29472585908',
                                                                       'in_ucast_pkts': '59851576',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '187663777776',
                                                                       'out_ucast_pkts': '152376578',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '187663777776',
                                                                       'in_ucast_pkts': '152376578',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '187663777776',
                                                                                                'in_ucast_pkts': '152376578',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77271',
                                                                                                'out_octets': '29472585908',
                                                                                                'out_ucast_pkts': '59851576',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77271',
                                                                       'out_octets': '29472585908',
                                                                       'out_ucast_pkts': '59851576',
                                                                       'poller_hash': 'QURUTjIzMTI0RTI1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317058',
                                                                                                                        'fiber_length': 5209,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTI1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyNQ==',
                                         'serial_number': 'ADTN23124E25',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317058'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17344',
                                                                       'in_octets': '71894337987',
                                                                       'in_ucast_pkts': '116007144',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '261076709092',
                                                                       'out_ucast_pkts': '208646296',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '261076709092',
                                                                       'in_ucast_pkts': '208646296',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '261076709092',
                                                                                                'in_ucast_pkts': '208646296',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17344',
                                                                                                'out_octets': '71894337987',
                                                                                                'out_ucast_pkts': '116007144',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17344',
                                                                       'out_octets': '71894337987',
                                                                       'out_ucast_pkts': '116007144',
                                                                       'poller_hash': 'QURUTjIzMTI0REIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511498',
                                                                                                                        'fiber_length': 5405,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERCMA==',
                                         'serial_number': 'ADTN23124DB0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17071',
                                                                       'in_octets': '6657479764',
                                                                       'in_ucast_pkts': '31241128',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '275313555690',
                                                                       'out_ucast_pkts': '201361428',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVThfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '275313555690',
                                                                       'in_ucast_pkts': '201361428',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '275313555690',
                                                                                                'in_ucast_pkts': '201361428',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17071',
                                                                                                'out_octets': '6657479764',
                                                                                                'out_ucast_pkts': '31241128',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17071',
                                                                       'out_octets': '6657479764',
                                                                       'out_ucast_pkts': '31241128',
                                                                       'poller_hash': 'QURUTjIzMTI0RjM0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511501',
                                                                                                                        'fiber_length': 5229,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjM0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYzNA==',
                                         'serial_number': 'ADTN23124F34',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511501'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13739',
                                                                       'in_octets': '15227797031',
                                                                       'in_ucast_pkts': '76341628',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '336060770122',
                                                                       'out_ucast_pkts': '237694633',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '336060770122',
                                                                       'in_ucast_pkts': '237694633',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '336060770122',
                                                                                                'in_ucast_pkts': '237694633',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13739',
                                                                                                'out_octets': '15227797031',
                                                                                                'out_ucast_pkts': '76341628',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13739',
                                                                       'out_octets': '15227797031',
                                                                       'out_ucast_pkts': '76341628',
                                                                       'poller_hash': 'QURUTjIzMTI0REY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411500',
                                                                                                                        'fiber_length': 8325,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGNg==',
                                         'serial_number': 'ADTN23124DF6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13739',
                                                                       'in_octets': '545655781',
                                                                       'in_ucast_pkts': '1123435',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '391776268',
                                                                       'out_ucast_pkts': '722858',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '391776268',
                                                                       'in_ucast_pkts': '722858',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '391776268',
                                                                                                'in_ucast_pkts': '722858',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13739',
                                                                                                'out_octets': '545655781',
                                                                                                'out_ucast_pkts': '1123435',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13739',
                                                                       'out_octets': '545655781',
                                                                       'out_ucast_pkts': '1123435',
                                                                       'poller_hash': 'QURUTjIzMTI0RTA0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411499',
                                                                                                                        'fiber_length': 8705,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTA0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUwNA==',
                                         'serial_number': 'ADTN23124E04',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13956',
                                                                       'in_octets': '8809687008',
                                                                       'in_ucast_pkts': '24162714',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '57389462190',
                                                                       'out_ucast_pkts': '46911227',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '57389462190',
                                                                       'in_ucast_pkts': '46911227',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '57389462190',
                                                                                                'in_ucast_pkts': '46911227',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13956',
                                                                                                'out_octets': '8809687008',
                                                                                                'out_ucast_pkts': '24162714',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13956',
                                                                       'out_octets': '8809687008',
                                                                       'out_ucast_pkts': '24162714',
                                                                       'poller_hash': 'QURUTjIzMTI0RTMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411496',
                                                                                                                        'fiber_length': 8651,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzMA==',
                                         'serial_number': 'ADTN23124E30',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411496'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14078',
                                                                       'in_octets': '11207439649',
                                                                       'in_ucast_pkts': '31634032',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '156037674829',
                                                                       'out_ucast_pkts': '115883166',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '156037674829',
                                                                       'in_ucast_pkts': '115883166',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '156037674829',
                                                                                                'in_ucast_pkts': '115883166',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14078',
                                                                                                'out_octets': '11207439649',
                                                                                                'out_ucast_pkts': '31634032',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14078',
                                                                       'out_octets': '11207439649',
                                                                       'out_ucast_pkts': '31634032',
                                                                       'poller_hash': 'QURUTjIzMTI0RTU3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411499',
                                                                                                                        'fiber_length': 6531,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTU3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU1Nw==',
                                         'serial_number': 'ADTN23124E57',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17068',
                                                                       'in_octets': '23508181967',
                                                                       'in_ucast_pkts': '34230325',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '117692129423',
                                                                       'out_ucast_pkts': '93226952',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTEwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '117692129423',
                                                                       'in_ucast_pkts': '93226952',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '117692129423',
                                                                                                'in_ucast_pkts': '93226952',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17068',
                                                                                                'out_octets': '23508181967',
                                                                                                'out_ucast_pkts': '34230325',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17068',
                                                                       'out_octets': '23508181967',
                                                                       'out_ucast_pkts': '34230325',
                                                                       'poller_hash': 'QURUTjIzMTIzRUI3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511504',
                                                                                                                        'fiber_length': 3552,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRUI3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0VCNw==',
                                         'serial_number': 'ADTN23123EB7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511504'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17767',
                                                                       'in_octets': '5155013968',
                                                                       'in_ucast_pkts': '34663408',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '210018721979',
                                                                       'out_ucast_pkts': '145411454',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTExX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '210018721979',
                                                                       'in_ucast_pkts': '145411454',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '210018721979',
                                                                                                'in_ucast_pkts': '145411454',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17767',
                                                                                                'out_octets': '5155013968',
                                                                                                'out_ucast_pkts': '34663408',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17767',
                                                                       'out_octets': '5155013968',
                                                                       'out_ucast_pkts': '34663408',
                                                                       'poller_hash': 'QURUTjIzMTI0RTM3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511500',
                                                                                                                        'fiber_length': 5276,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTM3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzNw==',
                                         'serial_number': 'ADTN23124E37',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17662',
                                                                       'in_octets': '27987474147',
                                                                       'in_ucast_pkts': '114204034',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124E4B_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1465',
                                                                       'out_octets': '429942845377',
                                                                       'out_ucast_pkts': '308714554',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RTRCX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1465',
                                                                       'in_octets': '429942845377',
                                                                       'in_ucast_pkts': '308714554',
                                                                       'links': [       {       'in_nucast_pkts': '1465',
                                                                                                'in_octets': '429942845377',
                                                                                                'in_ucast_pkts': '308714554',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17662',
                                                                                                'out_octets': '27987474147',
                                                                                                'out_ucast_pkts': '114204034',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17662',
                                                                       'out_octets': '27987474147',
                                                                       'out_ucast_pkts': '114204034',
                                                                       'poller_hash': 'QURUTjIzMTI0RTRCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511495',
                                                                                                                        'fiber_length': 5187,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTRCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124E4B',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU0Qg==',
                                         'serial_number': 'ADTN23124E4B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511495'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '61715',
                                                                       'in_octets': '94627388922',
                                                                       'in_ucast_pkts': '167676867',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5228',
                                                                       'out_octets': '406432771728',
                                                                       'out_ucast_pkts': '346990952',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTRfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5228',
                                                                       'in_octets': '406432771728',
                                                                       'in_ucast_pkts': '346990952',
                                                                       'links': [       {       'in_nucast_pkts': '5228',
                                                                                                'in_octets': '406432771728',
                                                                                                'in_ucast_pkts': '346990952',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '61715',
                                                                                                'out_octets': '94627388922',
                                                                                                'out_ucast_pkts': '167676867',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '61715',
                                                                       'out_octets': '94627388922',
                                                                       'out_ucast_pkts': '167676867',
                                                                       'poller_hash': 'QURUTjIzMTI0REY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1820396',
                                                                                                                        'fiber_length': 8168,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGNw==',
                                         'serial_number': 'ADTN23124DF7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1820396'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13738',
                                                                       'in_octets': '8150070916',
                                                                       'in_ucast_pkts': '24703195',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '154412345995',
                                                                       'out_ucast_pkts': '110311064',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTEzX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '154412345995',
                                                                       'in_ucast_pkts': '110311064',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '154412345995',
                                                                                                'in_ucast_pkts': '110311064',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13738',
                                                                                                'out_octets': '8150070916',
                                                                                                'out_ucast_pkts': '24703195',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13738',
                                                                       'out_octets': '8150070916',
                                                                       'out_ucast_pkts': '24703195',
                                                                       'poller_hash': 'QURUTjIzMTI0RTczIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411498',
                                                                                                                        'fiber_length': 4886,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTczIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU3Mw==',
                                         'serial_number': 'ADTN23124E73',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17347',
                                                                       'in_octets': '7152861603',
                                                                       'in_ucast_pkts': '27555356',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '108732896514',
                                                                       'out_ucast_pkts': '79082535',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTE1X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '108732896514',
                                                                       'in_ucast_pkts': '79082535',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '108732896514',
                                                                                                'in_ucast_pkts': '79082535',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17347',
                                                                                                'out_octets': '7152861603',
                                                                                                'out_ucast_pkts': '27555356',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17347',
                                                                       'out_octets': '7152861603',
                                                                       'out_ucast_pkts': '27555356',
                                                                       'poller_hash': 'QURUTjIzMTI0RTE3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511501',
                                                                                                                        'fiber_length': 4322,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTE3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxNw==',
                                         'serial_number': 'ADTN23124E17',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511501'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78533',
                                                                       'in_octets': '25244124422',
                                                                       'in_ucast_pkts': '77906098',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6647',
                                                                       'out_octets': '201047220883',
                                                                       'out_ucast_pkts': '154873044',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6647',
                                                                       'in_octets': '201047220883',
                                                                       'in_ucast_pkts': '154873044',
                                                                       'links': [       {       'in_nucast_pkts': '6647',
                                                                                                'in_octets': '201047220883',
                                                                                                'in_ucast_pkts': '154873044',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78533',
                                                                                                'out_octets': '25244124422',
                                                                                                'out_ucast_pkts': '77906098',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78533',
                                                                       'out_octets': '25244124422',
                                                                       'out_ucast_pkts': '77906098',
                                                                       'poller_hash': 'QURUTjIzMTI1MUIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317055',
                                                                                                                        'fiber_length': 5245,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFCMA==',
                                         'serial_number': 'ADTN231251B0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317055'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17074',
                                                                       'in_octets': '5123053333',
                                                                       'in_ucast_pkts': '12730687',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '39204583525',
                                                                       'out_ucast_pkts': '35900007',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '39204583525',
                                                                       'in_ucast_pkts': '35900007',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '39204583525',
                                                                                                'in_ucast_pkts': '35900007',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17074',
                                                                                                'out_octets': '5123053333',
                                                                                                'out_ucast_pkts': '12730687',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17074',
                                                                       'out_octets': '5123053333',
                                                                       'out_ucast_pkts': '12730687',
                                                                       'poller_hash': 'QURUTjIzMTI0RTJCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511513',
                                                                                                                        'fiber_length': 5938,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTJCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyQg==',
                                         'serial_number': 'ADTN23124E2B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511513'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78132',
                                                                       'in_octets': '37564494279',
                                                                       'in_ucast_pkts': '212070619',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6587',
                                                                       'out_octets': '705153503921',
                                                                       'out_ucast_pkts': '503870012',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6587',
                                                                       'in_octets': '705153503921',
                                                                       'in_ucast_pkts': '503870012',
                                                                       'links': [       {       'in_nucast_pkts': '6587',
                                                                                                'in_octets': '705153503921',
                                                                                                'in_ucast_pkts': '503870012',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78132',
                                                                                                'out_octets': '37564494279',
                                                                                                'out_ucast_pkts': '212070619',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78132',
                                                                       'out_octets': '37564494279',
                                                                       'out_ucast_pkts': '212070619',
                                                                       'poller_hash': 'QURUTjIzMTI1MUVFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317050',
                                                                                                                        'fiber_length': 4569,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUVFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFFRQ==',
                                         'serial_number': 'ADTN231251EE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317050'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13879',
                                                                       'in_octets': '13903633169',
                                                                       'in_ucast_pkts': '78829773',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '399712886520',
                                                                       'out_ucast_pkts': '286849218',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '399712886520',
                                                                       'in_ucast_pkts': '286849218',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '399712886520',
                                                                                                'in_ucast_pkts': '286849218',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13879',
                                                                                                'out_octets': '13903633169',
                                                                                                'out_ucast_pkts': '78829773',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13879',
                                                                       'out_octets': '13903633169',
                                                                       'out_ucast_pkts': '78829773',
                                                                       'poller_hash': 'QURUTjIzMTI0RTkyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411499',
                                                                                                                        'fiber_length': 8747,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTkyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU5Mg==',
                                         'serial_number': 'ADTN23124E92',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17573',
                                                                       'in_octets': '16510429418',
                                                                       'in_ucast_pkts': '47316879',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '581',
                                                                       'out_octets': '132080811021',
                                                                       'out_ucast_pkts': '102434920',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '581',
                                                                       'in_octets': '132080811021',
                                                                       'in_ucast_pkts': '102434920',
                                                                       'links': [       {       'in_nucast_pkts': '581',
                                                                                                'in_octets': '132080811021',
                                                                                                'in_ucast_pkts': '102434920',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17573',
                                                                                                'out_octets': '16510429418',
                                                                                                'out_ucast_pkts': '47316879',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17573',
                                                                       'out_octets': '16510429418',
                                                                       'out_ucast_pkts': '47316879',
                                                                       'poller_hash': 'QURUTjIzMTI0RTIzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511512',
                                                                                                                        'fiber_length': 4748,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTIzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyMw==',
                                         'serial_number': 'ADTN23124E23',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511512'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.25.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17356',
                                                                       'in_octets': '12201351423',
                                                                       'in_ucast_pkts': '96462911',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU25_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '280128997438',
                                                                       'out_ucast_pkts': '210615612',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '280128997438',
                                                                       'in_ucast_pkts': '210615612',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '280128997438',
                                                                                                'in_ucast_pkts': '210615612',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17356',
                                                                                                'out_octets': '12201351423',
                                                                                                'out_ucast_pkts': '96462911',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17356',
                                                                       'out_octets': '12201351423',
                                                                       'out_ucast_pkts': '96462911',
                                                                       'poller_hash': 'QURUTjIzMTI0RTI3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 4716,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTI3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU25',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyNw==',
                                         'serial_number': 'ADTN23124E27',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17077',
                                                                       'in_octets': '8081348185',
                                                                       'in_ucast_pkts': '45173344',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '191072021092',
                                                                       'out_ucast_pkts': '133766604',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '191072021092',
                                                                       'in_ucast_pkts': '133766604',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '191072021092',
                                                                                                'in_ucast_pkts': '133766604',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17077',
                                                                                                'out_octets': '8081348185',
                                                                                                'out_ucast_pkts': '45173344',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17077',
                                                                       'out_octets': '8081348185',
                                                                       'out_ucast_pkts': '45173344',
                                                                       'poller_hash': 'QURUTjIzMTI0RTY0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511507',
                                                                                                                        'fiber_length': 4657,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTY0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU2NA==',
                                         'serial_number': 'ADTN23124E64',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511507'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17343',
                                                                       'in_octets': '7546768448',
                                                                       'in_ucast_pkts': '13490279',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124F14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '32999075131',
                                                                       'out_ucast_pkts': '28090751',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RjE0X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '32999075131',
                                                                       'in_ucast_pkts': '28090751',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '32999075131',
                                                                                                'in_ucast_pkts': '28090751',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17343',
                                                                                                'out_octets': '7546768448',
                                                                                                'out_ucast_pkts': '13490279',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17343',
                                                                       'out_octets': '7546768448',
                                                                       'out_ucast_pkts': '13490279',
                                                                       'poller_hash': 'QURUTjIzMTI0RjE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511503',
                                                                                                                        'fiber_length': 7344,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124F14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYxNA==',
                                         'serial_number': 'ADTN23124F14',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511503'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.27.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17353',
                                                                       'in_octets': '4808821172',
                                                                       'in_ucast_pkts': '26666839',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124E6B_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '90330747117',
                                                                       'out_ucast_pkts': '65389355',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RTZCX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '90330747117',
                                                                       'in_ucast_pkts': '65389355',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '90330747117',
                                                                                                'in_ucast_pkts': '65389355',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17353',
                                                                                                'out_octets': '4808821172',
                                                                                                'out_ucast_pkts': '26666839',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17353',
                                                                       'out_octets': '4808821172',
                                                                       'out_ucast_pkts': '26666839',
                                                                       'poller_hash': 'QURUTjIzMTI0RTZCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511507',
                                                                                                                        'fiber_length': 5879,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTZCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124E6B',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU2Qg==',
                                         'serial_number': 'ADTN23124E6B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511507'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '56280',
                                                                       'in_octets': '75997970204',
                                                                       'in_ucast_pkts': '230241542',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4649',
                                                                       'out_octets': '983572185597',
                                                                       'out_ucast_pkts': '706426217',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4649',
                                                                       'in_octets': '983572185597',
                                                                       'in_ucast_pkts': '706426217',
                                                                       'links': [       {       'in_nucast_pkts': '4649',
                                                                                                'in_octets': '983572185597',
                                                                                                'in_ucast_pkts': '706426217',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '56280',
                                                                                                'out_octets': '75997970204',
                                                                                                'out_ucast_pkts': '230241542',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '56280',
                                                                       'out_octets': '75997970204',
                                                                       'out_ucast_pkts': '230241542',
                                                                       'poller_hash': 'QURUTjIzMTI1MTdGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1620457',
                                                                                                                        'fiber_length': 7112,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTdGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE3Rg==',
                                         'serial_number': 'ADTN2312517F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1620457'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '80396',
                                                                       'in_octets': '90723615998',
                                                                       'in_ucast_pkts': '231116455',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6647',
                                                                       'out_octets': '770911853601',
                                                                       'out_ucast_pkts': '568332465',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6647',
                                                                       'in_octets': '770911853601',
                                                                       'in_ucast_pkts': '568332465',
                                                                       'links': [       {       'in_nucast_pkts': '6647',
                                                                                                'in_octets': '770911853601',
                                                                                                'in_ucast_pkts': '568332465',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '80396',
                                                                                                'out_octets': '90723615998',
                                                                                                'out_ucast_pkts': '231116455',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '80396',
                                                                       'out_octets': '90723615998',
                                                                       'out_ucast_pkts': '231116455',
                                                                       'poller_hash': 'QURUTjIzMTI0Rjc0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317057',
                                                                                                                        'fiber_length': 6325,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0Rjc0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEY3NA==',
                                         'serial_number': 'ADTN23124F74',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317057'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.28.0.eth.1.phy',
                                                                       'in_nucast_pkts': '2084',
                                                                       'in_octets': '274980763',
                                                                       'in_ucast_pkts': '775381',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU28_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '70',
                                                                       'out_octets': '1458085763',
                                                                       'out_ucast_pkts': '1423806',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjhfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '70',
                                                                       'in_octets': '1458085763',
                                                                       'in_ucast_pkts': '1423806',
                                                                       'links': [       {       'in_nucast_pkts': '70',
                                                                                                'in_octets': '1458085763',
                                                                                                'in_ucast_pkts': '1423806',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '2084',
                                                                                                'out_octets': '274980763',
                                                                                                'out_ucast_pkts': '775381',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2084',
                                                                       'out_octets': '274980763',
                                                                       'out_ucast_pkts': '775381',
                                                                       'poller_hash': 'QURUTjIzMTIzRkVDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '61774',
                                                                                                                        'fiber_length': 5819,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkVDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU28',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZFQw==',
                                         'serial_number': 'ADTN23123FEC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '61774'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '161659',
                                                                       'in_octets': '50049796547',
                                                                       'in_ucast_pkts': '152471599',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5396',
                                                                       'out_octets': '508085431347',
                                                                       'out_ucast_pkts': '392883353',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5396',
                                                                       'in_octets': '508085431347',
                                                                       'in_ucast_pkts': '392883353',
                                                                       'links': [       {       'in_nucast_pkts': '5396',
                                                                                                'in_octets': '508085431347',
                                                                                                'in_ucast_pkts': '392883353',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '161659',
                                                                                                'out_octets': '50049796547',
                                                                                                'out_ucast_pkts': '152471599',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '161659',
                                                                       'out_octets': '50049796547',
                                                                       'out_ucast_pkts': '152471599',
                                                                       'poller_hash': 'QURUTjIzMTI1MDYxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4857795',
                                                                                                                        'fiber_length': 10820,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDYxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA2MQ==',
                                         'serial_number': 'ADTN23125061',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '4857795'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77249',
                                                                       'in_octets': '50957718414',
                                                                       'in_ucast_pkts': '319700124',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '1204346775941',
                                                                       'out_ucast_pkts': '845022783',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '1204346775941',
                                                                       'in_ucast_pkts': '845022783',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '1204346775941',
                                                                                                'in_ucast_pkts': '845022783',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77249',
                                                                                                'out_octets': '50957718414',
                                                                                                'out_ucast_pkts': '319700124',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77249',
                                                                       'out_octets': '50957718414',
                                                                       'out_ucast_pkts': '319700124',
                                                                       'poller_hash': 'QURUTjIzMTI0REI3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317051',
                                                                                                                        'fiber_length': 2058,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REI3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERCNw==',
                                         'serial_number': 'ADTN23124DB7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317051'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17771',
                                                                       'in_octets': '9693079177',
                                                                       'in_ucast_pkts': '67618894',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '287715030573',
                                                                       'out_ucast_pkts': '201871393',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTE2X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '287715030573',
                                                                       'in_ucast_pkts': '201871393',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '287715030573',
                                                                                                'in_ucast_pkts': '201871393',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17771',
                                                                                                'out_octets': '9693079177',
                                                                                                'out_ucast_pkts': '67618894',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17771',
                                                                       'out_octets': '9693079177',
                                                                       'out_ucast_pkts': '67618894',
                                                                       'poller_hash': 'QURUTjIzMTI1MTJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511500',
                                                                                                                        'fiber_length': 5460,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEyQw==',
                                         'serial_number': 'ADTN2312512C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17343',
                                                                       'in_octets': '4519727774',
                                                                       'in_ucast_pkts': '20018006',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '92169783415',
                                                                       'out_ucast_pkts': '67778587',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTE3X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '92169783415',
                                                                       'in_ucast_pkts': '67778587',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '92169783415',
                                                                                                'in_ucast_pkts': '67778587',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17343',
                                                                                                'out_octets': '4519727774',
                                                                                                'out_ucast_pkts': '20018006',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17343',
                                                                       'out_octets': '4519727774',
                                                                       'out_ucast_pkts': '20018006',
                                                                       'poller_hash': 'QURUTjIzMTIzRUExIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511502',
                                                                                                                        'fiber_length': 3586,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRUExIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0VBMQ==',
                                         'serial_number': 'ADTN23123EA1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511502'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17350',
                                                                       'in_octets': '4068733367',
                                                                       'in_ucast_pkts': '27154906',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '75599876134',
                                                                       'out_ucast_pkts': '52488869',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '75599876134',
                                                                       'in_ucast_pkts': '52488869',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '75599876134',
                                                                                                'in_ucast_pkts': '52488869',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17350',
                                                                                                'out_octets': '4068733367',
                                                                                                'out_ucast_pkts': '27154906',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17350',
                                                                       'out_octets': '4068733367',
                                                                       'out_ucast_pkts': '27154906',
                                                                       'poller_hash': 'QURUTjIzMTI0REM3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511510',
                                                                                                                        'fiber_length': 2394,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REM3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERDNw==',
                                         'serial_number': 'ADTN23124DC7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511510'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13965',
                                                                       'in_octets': '8718124297',
                                                                       'in_ucast_pkts': '31366044',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '87654360048',
                                                                       'out_ucast_pkts': '62827471',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '87654360048',
                                                                       'in_ucast_pkts': '62827471',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '87654360048',
                                                                                                'in_ucast_pkts': '62827471',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13965',
                                                                                                'out_octets': '8718124297',
                                                                                                'out_ucast_pkts': '31366044',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13965',
                                                                       'out_octets': '8718124297',
                                                                       'out_ucast_pkts': '31366044',
                                                                       'poller_hash': 'QURUTjIzMTI0RUZBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411662',
                                                                                                                        'fiber_length': 2496,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RUZBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVGQQ==',
                                         'serial_number': 'ADTN23124EFA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411662'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17344',
                                                                       'in_octets': '33003402586',
                                                                       'in_ucast_pkts': '86853826',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '322535900258',
                                                                       'out_ucast_pkts': '233738809',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTE4X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '322535900258',
                                                                       'in_ucast_pkts': '233738809',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '322535900258',
                                                                                                'in_ucast_pkts': '233738809',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17344',
                                                                                                'out_octets': '33003402586',
                                                                                                'out_ucast_pkts': '86853826',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17344',
                                                                       'out_octets': '33003402586',
                                                                       'out_ucast_pkts': '86853826',
                                                                       'poller_hash': 'QURUTjIzMTI1MTMzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511499',
                                                                                                                        'fiber_length': 3613,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTMzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEzMw==',
                                         'serial_number': 'ADTN23125133',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17645',
                                                                       'in_octets': '7043293710',
                                                                       'in_ucast_pkts': '15487796',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '58159716504',
                                                                       'out_ucast_pkts': '44857988',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '58159716504',
                                                                       'in_ucast_pkts': '44857988',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '58159716504',
                                                                                                'in_ucast_pkts': '44857988',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17645',
                                                                                                'out_octets': '7043293710',
                                                                                                'out_ucast_pkts': '15487796',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17645',
                                                                       'out_octets': '7043293710',
                                                                       'out_ucast_pkts': '15487796',
                                                                       'poller_hash': 'QURUTjIzMTI0RURGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511510',
                                                                                                                        'fiber_length': 3018,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RURGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVERg==',
                                         'serial_number': 'ADTN23124EDF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511510'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '79177',
                                                                       'in_octets': '52585387879',
                                                                       'in_ucast_pkts': '290160443',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '988651401643',
                                                                       'out_ucast_pkts': '738980630',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '988651401643',
                                                                       'in_ucast_pkts': '738980630',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '988651401643',
                                                                                                'in_ucast_pkts': '738980630',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '79177',
                                                                                                'out_octets': '52585387879',
                                                                                                'out_ucast_pkts': '290160443',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '79177',
                                                                       'out_octets': '52585387879',
                                                                       'out_ucast_pkts': '290160443',
                                                                       'poller_hash': 'QURUTjIzMTI1MTcxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317062',
                                                                                                                        'fiber_length': 10756,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTcxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE3MQ==',
                                         'serial_number': 'ADTN23125171',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317062'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '23934',
                                                                       'in_octets': '12259358305',
                                                                       'in_ucast_pkts': '65309438',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124EA5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1979',
                                                                       'out_octets': '238371002118',
                                                                       'out_ucast_pkts': '171853941',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RUE1X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1979',
                                                                       'in_octets': '238371002118',
                                                                       'in_ucast_pkts': '171853941',
                                                                       'links': [       {       'in_nucast_pkts': '1979',
                                                                                                'in_octets': '238371002118',
                                                                                                'in_ucast_pkts': '171853941',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '23934',
                                                                                                'out_octets': '12259358305',
                                                                                                'out_ucast_pkts': '65309438',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '23934',
                                                                       'out_octets': '12259358305',
                                                                       'out_ucast_pkts': '65309438',
                                                                       'poller_hash': 'QURUTjIzMTI0RUE1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '688772',
                                                                                                                        'fiber_length': 3857,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RUE1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124EA5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVBNQ==',
                                         'serial_number': 'ADTN23124EA5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '688772'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77324',
                                                                       'in_octets': '195543612937',
                                                                       'in_ucast_pkts': '871541381',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '3617072709960',
                                                                       'out_ucast_pkts': '2675239709',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VOV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '3617072709960',
                                                                       'in_ucast_pkts': '2675239709',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '3617072709960',
                                                                                                'in_ucast_pkts': '2675239709',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77324',
                                                                                                'out_octets': '195543612937',
                                                                                                'out_ucast_pkts': '871541381',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77324',
                                                                       'out_octets': '195543612937',
                                                                       'out_ucast_pkts': '871541381',
                                                                       'poller_hash': 'QURUTjIzMTI0OEE4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317053',
                                                                                                                        'fiber_length': 2069,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OEE4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDhBOA==',
                                         'serial_number': 'ADTN231248A8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317053'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.29.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17354',
                                                                       'in_octets': '1412860165',
                                                                       'in_ucast_pkts': '5199266',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU29_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '18726906322',
                                                                       'out_ucast_pkts': '14643689',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjlfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '18726906322',
                                                                       'in_ucast_pkts': '14643689',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '18726906322',
                                                                                                'in_ucast_pkts': '14643689',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17354',
                                                                                                'out_octets': '1412860165',
                                                                                                'out_ucast_pkts': '5199266',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17354',
                                                                       'out_octets': '1412860165',
                                                                       'out_ucast_pkts': '5199266',
                                                                       'poller_hash': 'QURUTjIzMTIzRUE4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511509',
                                                                                                                        'fiber_length': 4449,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRUE4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU29',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0VBOA==',
                                         'serial_number': 'ADTN23123EA8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511509'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '186255',
                                                                       'in_octets': '241400071505',
                                                                       'in_ucast_pkts': '1428422001',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124E52_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15626',
                                                                       'out_octets': '4760659326578',
                                                                       'out_ucast_pkts': '3461599307',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RTUyX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15626',
                                                                       'in_octets': '4760659326578',
                                                                       'in_ucast_pkts': '3461599307',
                                                                       'links': [       {       'in_nucast_pkts': '15626',
                                                                                                'in_octets': '4760659326578',
                                                                                                'in_ucast_pkts': '3461599307',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '186255',
                                                                                                'out_octets': '241400071505',
                                                                                                'out_ucast_pkts': '1428422001',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '186255',
                                                                       'out_octets': '241400071505',
                                                                       'out_ucast_pkts': '1428422001',
                                                                       'poller_hash': 'QURUTjIzMTI0RTUyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440101',
                                                                                                                        'fiber_length': 9016,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTUyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124E52',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU1Mg==',
                                         'serial_number': 'ADTN23124E52',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440101'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17350',
                                                                       'in_octets': '24490568755',
                                                                       'in_ucast_pkts': '74536339',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '219456576798',
                                                                       'out_ucast_pkts': '166369684',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '219456576798',
                                                                       'in_ucast_pkts': '166369684',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '219456576798',
                                                                                                'in_ucast_pkts': '166369684',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17350',
                                                                                                'out_octets': '24490568755',
                                                                                                'out_ucast_pkts': '74536339',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17350',
                                                                       'out_octets': '24490568755',
                                                                       'out_ucast_pkts': '74536339',
                                                                       'poller_hash': 'QURUTjIzMTI1MTVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511513',
                                                                                                                        'fiber_length': 6916,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE1Qg==',
                                         'serial_number': 'ADTN2312515B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511513'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17357',
                                                                       'in_octets': '11117991299',
                                                                       'in_ucast_pkts': '50364145',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '137294957416',
                                                                       'out_ucast_pkts': '117912283',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '137294957416',
                                                                       'in_ucast_pkts': '117912283',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '137294957416',
                                                                                                'in_ucast_pkts': '117912283',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17357',
                                                                                                'out_octets': '11117991299',
                                                                                                'out_ucast_pkts': '50364145',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17357',
                                                                       'out_octets': '11117991299',
                                                                       'out_ucast_pkts': '50364145',
                                                                       'poller_hash': 'QURUTjIzMTI1MUIxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511511',
                                                                                                                        'fiber_length': 6439,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUIxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFCMQ==',
                                         'serial_number': 'ADTN231251B1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511511'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17350',
                                                                       'in_octets': '1766869586',
                                                                       'in_ucast_pkts': '9286544',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '22473306375',
                                                                       'out_ucast_pkts': '16882303',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '22473306375',
                                                                       'in_ucast_pkts': '16882303',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '22473306375',
                                                                                                'in_ucast_pkts': '16882303',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17350',
                                                                                                'out_octets': '1766869586',
                                                                                                'out_ucast_pkts': '9286544',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17350',
                                                                       'out_octets': '1766869586',
                                                                       'out_ucast_pkts': '9286544',
                                                                       'poller_hash': 'QURUTjIzMTI1MUY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 2254,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFGNw==',
                                         'serial_number': 'ADTN231251F7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17352',
                                                                       'in_octets': '19201215989',
                                                                       'in_ucast_pkts': '43126746',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '94247613907',
                                                                       'out_ucast_pkts': '74374409',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '94247613907',
                                                                       'in_ucast_pkts': '74374409',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '94247613907',
                                                                                                'in_ucast_pkts': '74374409',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17352',
                                                                                                'out_octets': '19201215989',
                                                                                                'out_ucast_pkts': '43126746',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17352',
                                                                       'out_octets': '19201215989',
                                                                       'out_ucast_pkts': '43126746',
                                                                       'poller_hash': 'QURUTjIzMTI0RTg0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511511',
                                                                                                                        'fiber_length': 3256,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTg0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU4NA==',
                                         'serial_number': 'ADTN23124E84',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511511'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-6.1.3.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x6-ONU3_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VM19FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0QTIzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QTIzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x6-ONU3',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '6.1.3"(ADTN23124A23) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/6.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-08-14T13:51:35Z',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEEyMw==',
                                         'serial_number': 'ADTN23124A23',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.30.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17350',
                                                                       'in_octets': '3183811034',
                                                                       'in_ucast_pkts': '28605061',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU30_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '110348349871',
                                                                       'out_ucast_pkts': '77108246',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMzBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '110348349871',
                                                                       'in_ucast_pkts': '77108246',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '110348349871',
                                                                                                'in_ucast_pkts': '77108246',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17350',
                                                                                                'out_octets': '3183811034',
                                                                                                'out_ucast_pkts': '28605061',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17350',
                                                                       'out_octets': '3183811034',
                                                                       'out_ucast_pkts': '28605061',
                                                                       'poller_hash': 'QURUTjIzMTI0RTFGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511512',
                                                                                                                        'fiber_length': 4391,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTFGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU30',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxRg==',
                                         'serial_number': 'ADTN23124E1F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511512'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17771',
                                                                       'in_octets': '14552608567',
                                                                       'in_ucast_pkts': '55071587',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '118612809491',
                                                                       'out_ucast_pkts': '93178460',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '118612809491',
                                                                       'in_ucast_pkts': '93178460',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '118612809491',
                                                                                                'in_ucast_pkts': '93178460',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17771',
                                                                                                'out_octets': '14552608567',
                                                                                                'out_ucast_pkts': '55071587',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17771',
                                                                       'out_octets': '14552608567',
                                                                       'out_ucast_pkts': '55071587',
                                                                       'poller_hash': 'QURUTjIzMTI0RjM3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511499',
                                                                                                                        'fiber_length': 7364,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjM3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYzNw==',
                                         'serial_number': 'ADTN23124F37',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '114985',
                                                                       'in_octets': '21659055433',
                                                                       'in_ucast_pkts': '102931082',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3834',
                                                                       'out_octets': '455566541671',
                                                                       'out_ucast_pkts': '320085741',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3834',
                                                                       'in_octets': '455566541671',
                                                                       'in_ucast_pkts': '320085741',
                                                                       'links': [       {       'in_nucast_pkts': '3834',
                                                                                                'in_octets': '455566541671',
                                                                                                'in_ucast_pkts': '320085741',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '114985',
                                                                                                'out_octets': '21659055433',
                                                                                                'out_ucast_pkts': '102931082',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '114985',
                                                                       'out_octets': '21659055433',
                                                                       'out_ucast_pkts': '102931082',
                                                                       'poller_hash': 'QURUTjIzMTI0RThBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3449058',
                                                                                                                        'fiber_length': 6277,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RThBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU4QQ==',
                                         'serial_number': 'ADTN23124E8A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3449058'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '114988',
                                                                       'in_octets': '30698021276',
                                                                       'in_ucast_pkts': '152237846',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3834',
                                                                       'out_octets': '493904889370',
                                                                       'out_ucast_pkts': '362374584',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3834',
                                                                       'in_octets': '493904889370',
                                                                       'in_ucast_pkts': '362374584',
                                                                       'links': [       {       'in_nucast_pkts': '3834',
                                                                                                'in_octets': '493904889370',
                                                                                                'in_ucast_pkts': '362374584',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '114988',
                                                                                                'out_octets': '30698021276',
                                                                                                'out_ucast_pkts': '152237846',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '114988',
                                                                       'out_octets': '30698021276',
                                                                       'out_ucast_pkts': '152237846',
                                                                       'poller_hash': 'QURUTjIzMTI1MDY1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3449060',
                                                                                                                        'fiber_length': 6171,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDY1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA2NQ==',
                                         'serial_number': 'ADTN23125065',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3449060'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17354',
                                                                       'in_octets': '63139814593',
                                                                       'in_ucast_pkts': '195740707',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '575283213404',
                                                                       'out_ucast_pkts': '422753217',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '575283213404',
                                                                       'in_ucast_pkts': '422753217',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '575283213404',
                                                                                                'in_ucast_pkts': '422753217',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17354',
                                                                                                'out_octets': '63139814593',
                                                                                                'out_ucast_pkts': '195740707',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17354',
                                                                       'out_octets': '63139814593',
                                                                       'out_ucast_pkts': '195740707',
                                                                       'poller_hash': 'QURUTjIzMTI1MUY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511509',
                                                                                                                        'fiber_length': 2382,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFGNg==',
                                         'serial_number': 'ADTN231251F6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511509'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17354',
                                                                       'in_octets': '29097681192',
                                                                       'in_ucast_pkts': '95864727',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '281619296041',
                                                                       'out_ucast_pkts': '225231276',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '281619296041',
                                                                       'in_ucast_pkts': '225231276',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '281619296041',
                                                                                                'in_ucast_pkts': '225231276',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17354',
                                                                                                'out_octets': '29097681192',
                                                                                                'out_ucast_pkts': '95864727',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17354',
                                                                       'out_octets': '29097681192',
                                                                       'out_ucast_pkts': '95864727',
                                                                       'poller_hash': 'QURUTjIzMTIzRTlEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511511',
                                                                                                                        'fiber_length': 6505,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRTlEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0U5RA==',
                                         'serial_number': 'ADTN23123E9D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511511'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17075',
                                                                       'in_octets': '38171933287',
                                                                       'in_ucast_pkts': '91083116',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '333219784794',
                                                                       'out_ucast_pkts': '239709601',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTIwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '333219784794',
                                                                       'in_ucast_pkts': '239709601',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '333219784794',
                                                                                                'in_ucast_pkts': '239709601',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17075',
                                                                                                'out_octets': '38171933287',
                                                                                                'out_ucast_pkts': '91083116',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17075',
                                                                       'out_octets': '38171933287',
                                                                       'out_ucast_pkts': '91083116',
                                                                       'poller_hash': 'QURUTjIzMTI1MUFCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511499',
                                                                                                                        'fiber_length': 5452,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUFCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFBQg==',
                                         'serial_number': 'ADTN231251AB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '312',
                                                                       'in_octets': '19165886336',
                                                                       'in_ucast_pkts': '38938258',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '135508186721',
                                                                       'out_ucast_pkts': '106776301',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VN19FVEhfMQ==',
                                                                       'speed': '100000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '135508186721',
                                                                       'in_ucast_pkts': '106776301',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '135508186721',
                                                                                                'in_ucast_pkts': '106776301',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '312',
                                                                                                'out_octets': '19165886336',
                                                                                                'out_ucast_pkts': '38938258',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '312',
                                                                       'out_octets': '19165886336',
                                                                       'out_ucast_pkts': '38938258',
                                                                       'poller_hash': 'QURUTjIzMTI0RUQzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511506',
                                                                                                                        'fiber_length': 2629,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RUQzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVEMw==',
                                         'serial_number': 'ADTN23124ED3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511506'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17776',
                                                                       'in_octets': '95455458929',
                                                                       'in_ucast_pkts': '224634758',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '598351349113',
                                                                       'out_ucast_pkts': '459796937',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '598351349113',
                                                                       'in_ucast_pkts': '459796937',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '598351349113',
                                                                                                'in_ucast_pkts': '459796937',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17776',
                                                                                                'out_octets': '95455458929',
                                                                                                'out_ucast_pkts': '224634758',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17776',
                                                                       'out_octets': '95455458929',
                                                                       'out_ucast_pkts': '224634758',
                                                                       'poller_hash': 'QURUTjIzMTI0RTMzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511510',
                                                                                                                        'fiber_length': 6490,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTMzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzMw==',
                                         'serial_number': 'ADTN23124E33',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511510'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '93222',
                                                                       'in_octets': '47749529318',
                                                                       'in_ucast_pkts': '316280876',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '7869',
                                                                       'out_octets': '1402728868805',
                                                                       'out_ucast_pkts': '1073548328',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '7869',
                                                                       'in_octets': '1402728868805',
                                                                       'in_ucast_pkts': '1073548328',
                                                                       'links': [       {       'in_nucast_pkts': '7869',
                                                                                                'in_octets': '1402728868805',
                                                                                                'in_ucast_pkts': '1073548328',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '93222',
                                                                                                'out_octets': '47749529318',
                                                                                                'out_ucast_pkts': '316280876',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '93222',
                                                                       'out_octets': '47749529318',
                                                                       'out_ucast_pkts': '316280876',
                                                                       'poller_hash': 'QURUTjIzMTI0RTREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2742692',
                                                                                                                        'fiber_length': 10425,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU0RA==',
                                         'serial_number': 'ADTN23124E4D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2742692'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78522',
                                                                       'in_octets': '29883348983',
                                                                       'in_ucast_pkts': '165609852',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN2312509E_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6648',
                                                                       'out_octets': '593970879853',
                                                                       'out_ucast_pkts': '422689492',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI1MDlFX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6648',
                                                                       'in_octets': '593970879853',
                                                                       'in_ucast_pkts': '422689492',
                                                                       'links': [       {       'in_nucast_pkts': '6648',
                                                                                                'in_octets': '593970879853',
                                                                                                'in_ucast_pkts': '422689492',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78522',
                                                                                                'out_octets': '29883348983',
                                                                                                'out_ucast_pkts': '165609852',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78522',
                                                                       'out_octets': '29883348983',
                                                                       'out_ucast_pkts': '165609852',
                                                                       'poller_hash': 'QURUTjIzMTI1MDlFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317051',
                                                                                                                        'fiber_length': 2313,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDlFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN2312509E',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA5RQ==',
                                         'serial_number': 'ADTN2312509E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317051'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77254',
                                                                       'in_octets': '9552622113',
                                                                       'in_ucast_pkts': '30258842',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23123E9E_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2574',
                                                                       'out_octets': '85272096163',
                                                                       'out_ucast_pkts': '69784441',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTIzRTlFX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2574',
                                                                       'in_octets': '85272096163',
                                                                       'in_ucast_pkts': '69784441',
                                                                       'links': [       {       'in_nucast_pkts': '2574',
                                                                                                'in_octets': '85272096163',
                                                                                                'in_ucast_pkts': '69784441',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77254',
                                                                                                'out_octets': '9552622113',
                                                                                                'out_ucast_pkts': '30258842',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77254',
                                                                       'out_octets': '9552622113',
                                                                       'out_ucast_pkts': '30258842',
                                                                       'poller_hash': 'QURUTjIzMTIzRTlFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317054',
                                                                                                                        'fiber_length': 4488,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRTlFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23123E9E',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0U5RQ==',
                                         'serial_number': 'ADTN23123E9E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317054'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78538',
                                                                       'in_octets': '16840309678',
                                                                       'in_ucast_pkts': '96638856',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124E2A_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6651',
                                                                       'out_octets': '427069282997',
                                                                       'out_ucast_pkts': '315618342',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RTJBX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6651',
                                                                       'in_octets': '427069282997',
                                                                       'in_ucast_pkts': '315618342',
                                                                       'links': [       {       'in_nucast_pkts': '6651',
                                                                                                'in_octets': '427069282997',
                                                                                                'in_ucast_pkts': '315618342',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78538',
                                                                                                'out_octets': '16840309678',
                                                                                                'out_ucast_pkts': '96638856',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78538',
                                                                       'out_octets': '16840309678',
                                                                       'out_ucast_pkts': '96638856',
                                                                       'poller_hash': 'QURUTjIzMTI0RTJBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317060',
                                                                                                                        'fiber_length': 5521,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTJBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124E2A',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyQQ==',
                                         'serial_number': 'ADTN23124E2A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317060'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77453',
                                                                       'in_octets': '178991664099',
                                                                       'in_ucast_pkts': '614932667',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124EA4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2574',
                                                                       'out_octets': '1876167151735',
                                                                       'out_ucast_pkts': '1340522307',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RUE0X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2574',
                                                                       'in_octets': '1876167151735',
                                                                       'in_ucast_pkts': '1340522307',
                                                                       'links': [       {       'in_nucast_pkts': '2574',
                                                                                                'in_octets': '1876167151735',
                                                                                                'in_ucast_pkts': '1340522307',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77453',
                                                                                                'out_octets': '178991664099',
                                                                                                'out_ucast_pkts': '614932667',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77453',
                                                                       'out_octets': '178991664099',
                                                                       'out_ucast_pkts': '614932667',
                                                                       'poller_hash': 'QURUTjIzMTI0RUE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317056',
                                                                                                                        'fiber_length': 5919,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RUE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124EA4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVBNA==',
                                         'serial_number': 'ADTN23124EA4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317056'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '116887',
                                                                       'in_octets': '138058217301',
                                                                       'in_ucast_pkts': '365230395',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9896',
                                                                       'out_octets': '1101010958219',
                                                                       'out_ucast_pkts': '807821198',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9896',
                                                                       'in_octets': '1101010958219',
                                                                       'in_ucast_pkts': '807821198',
                                                                       'links': [       {       'in_nucast_pkts': '9896',
                                                                                                'in_octets': '1101010958219',
                                                                                                'in_ucast_pkts': '807821198',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '116887',
                                                                                                'out_octets': '138058217301',
                                                                                                'out_ucast_pkts': '365230395',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '116887',
                                                                       'out_octets': '138058217301',
                                                                       'out_ucast_pkts': '365230395',
                                                                       'poller_hash': 'QURUTjIzMTI0RTQzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3449061',
                                                                                                                        'fiber_length': 6584,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTQzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU0Mw==',
                                         'serial_number': 'ADTN23124E43',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3449061'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '15047',
                                                                       'in_octets': '8726665362',
                                                                       'in_ucast_pkts': '29108039',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1273',
                                                                       'out_octets': '76215660012',
                                                                       'out_ucast_pkts': '56408305',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1273',
                                                                       'in_octets': '76215660012',
                                                                       'in_ucast_pkts': '56408305',
                                                                       'links': [       {       'in_nucast_pkts': '1273',
                                                                                                'in_octets': '76215660012',
                                                                                                'in_ucast_pkts': '56408305',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '15047',
                                                                                                'out_octets': '8726665362',
                                                                                                'out_ucast_pkts': '29108039',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15047',
                                                                       'out_octets': '8726665362',
                                                                       'out_ucast_pkts': '29108039',
                                                                       'poller_hash': 'QURUTjIzMTI0RTZBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '443591',
                                                                                                                        'fiber_length': 10462,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTZBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU2QQ==',
                                         'serial_number': 'ADTN23124E6A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '443591'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17346',
                                                                       'in_octets': '37249983009',
                                                                       'in_ucast_pkts': '59033750',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '215519017972',
                                                                       'out_ucast_pkts': '168055395',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '215519017972',
                                                                       'in_ucast_pkts': '168055395',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '215519017972',
                                                                                                'in_ucast_pkts': '168055395',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17346',
                                                                                                'out_octets': '37249983009',
                                                                                                'out_ucast_pkts': '59033750',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17346',
                                                                       'out_octets': '37249983009',
                                                                       'out_ucast_pkts': '59033750',
                                                                       'poller_hash': 'QURUTjIzMTI1MTYxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511503',
                                                                                                                        'fiber_length': 5086,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTYxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE2MQ==',
                                         'serial_number': 'ADTN23125161',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511503'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13955',
                                                                       'in_octets': '2548805306',
                                                                       'in_ucast_pkts': '16585756',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '67292117686',
                                                                       'out_ucast_pkts': '49198748',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '67292117686',
                                                                       'in_ucast_pkts': '49198748',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '67292117686',
                                                                                                'in_ucast_pkts': '49198748',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13955',
                                                                                                'out_octets': '2548805306',
                                                                                                'out_ucast_pkts': '16585756',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13955',
                                                                       'out_octets': '2548805306',
                                                                       'out_ucast_pkts': '16585756',
                                                                       'poller_hash': 'QURUTjIzMTI0RTg2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411499',
                                                                                                                        'fiber_length': 8737,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTg2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU4Ng==',
                                         'serial_number': 'ADTN23124E86',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '80518',
                                                                       'in_octets': '21490240212',
                                                                       'in_ucast_pkts': '132756443',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6655',
                                                                       'out_octets': '502414832140',
                                                                       'out_ucast_pkts': '358664556',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6655',
                                                                       'in_octets': '502414832140',
                                                                       'in_ucast_pkts': '358664556',
                                                                       'links': [       {       'in_nucast_pkts': '6655',
                                                                                                'in_octets': '502414832140',
                                                                                                'in_ucast_pkts': '358664556',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '80518',
                                                                                                'out_octets': '21490240212',
                                                                                                'out_ucast_pkts': '132756443',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '80518',
                                                                       'out_octets': '21490240212',
                                                                       'out_ucast_pkts': '132756443',
                                                                       'poller_hash': 'QURUTjIzMTI0RTUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317052',
                                                                                                                        'fiber_length': 2008,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU1MA==',
                                         'serial_number': 'ADTN23124E50',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317052'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '31563',
                                                                       'in_octets': '33193181638',
                                                                       'in_ucast_pkts': '112223303',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124DA0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1053',
                                                                       'out_octets': '458897956956',
                                                                       'out_ucast_pkts': '325661781',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0REEwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1053',
                                                                       'in_octets': '458897956956',
                                                                       'in_ucast_pkts': '325661781',
                                                                       'links': [       {       'in_nucast_pkts': '1053',
                                                                                                'in_octets': '458897956956',
                                                                                                'in_ucast_pkts': '325661781',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '31563',
                                                                                                'out_octets': '33193181638',
                                                                                                'out_ucast_pkts': '112223303',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '31563',
                                                                       'out_octets': '33193181638',
                                                                       'out_ucast_pkts': '112223303',
                                                                       'poller_hash': 'QURUTjIzMTI0REEwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '946254',
                                                                                                                        'fiber_length': 8008,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REEwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124DA0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERBMA==',
                                         'serial_number': 'ADTN23124DA0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '946254'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17351',
                                                                       'in_octets': '5581427159',
                                                                       'in_ucast_pkts': '22229830',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '61468584165',
                                                                       'out_ucast_pkts': '53311404',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VOF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '61468584165',
                                                                       'in_ucast_pkts': '53311404',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '61468584165',
                                                                                                'in_ucast_pkts': '53311404',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17351',
                                                                                                'out_octets': '5581427159',
                                                                                                'out_ucast_pkts': '22229830',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17351',
                                                                       'out_octets': '5581427159',
                                                                       'out_ucast_pkts': '22229830',
                                                                       'poller_hash': 'QURUTjIzMTI0MTI4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511513',
                                                                                                                        'fiber_length': 5794,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MTI4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDEyOA==',
                                         'serial_number': 'ADTN23124128',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511513'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '162985',
                                                                       'in_octets': '23530899217',
                                                                       'in_ucast_pkts': '109177020',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5398',
                                                                       'out_octets': '480313137336',
                                                                       'out_ucast_pkts': '355957819',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5398',
                                                                       'in_octets': '480313137336',
                                                                       'in_ucast_pkts': '355957819',
                                                                       'links': [       {       'in_nucast_pkts': '5398',
                                                                                                'in_octets': '480313137336',
                                                                                                'in_ucast_pkts': '355957819',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '162985',
                                                                                                'out_octets': '23530899217',
                                                                                                'out_ucast_pkts': '109177020',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '162985',
                                                                       'out_octets': '23530899217',
                                                                       'out_ucast_pkts': '109177020',
                                                                       'poller_hash': 'QURUTjIzMTI0NDVDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4857797',
                                                                                                                        'fiber_length': 10872,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NDVDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDQ1Qw==',
                                         'serial_number': 'ADTN2312445C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '4857797'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14266',
                                                                       'in_octets': '60905332',
                                                                       'in_ucast_pkts': '220207',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124304_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '186254722',
                                                                       'out_ucast_pkts': '254635',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0MzA0X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '186254722',
                                                                       'in_ucast_pkts': '254635',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '186254722',
                                                                                                'in_ucast_pkts': '254635',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14266',
                                                                                                'out_octets': '60905332',
                                                                                                'out_ucast_pkts': '220207',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14266',
                                                                       'out_octets': '60905332',
                                                                       'out_ucast_pkts': '220207',
                                                                       'poller_hash': 'QURUTjIzMTI0MzA0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411499',
                                                                                                                        'fiber_length': 4895,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MzA0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124304',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDMwNA==',
                                         'serial_number': 'ADTN23124304',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '28072',
                                                                       'in_octets': '24195404784',
                                                                       'in_ucast_pkts': '129400634',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '936',
                                                                       'out_octets': '612910262602',
                                                                       'out_ucast_pkts': '440182029',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '936',
                                                                       'in_octets': '612910262602',
                                                                       'in_ucast_pkts': '440182029',
                                                                       'links': [       {       'in_nucast_pkts': '936',
                                                                                                'in_octets': '612910262602',
                                                                                                'in_ucast_pkts': '440182029',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '28072',
                                                                                                'out_octets': '24195404784',
                                                                                                'out_ucast_pkts': '129400634',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '28072',
                                                                       'out_octets': '24195404784',
                                                                       'out_ucast_pkts': '129400634',
                                                                       'poller_hash': 'QURUTjIzMTI0NEJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '841331',
                                                                                                                        'fiber_length': 9328,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NEJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDRCQw==',
                                         'serial_number': 'ADTN231244BC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '841331'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17335',
                                                                       'in_octets': '5937602802',
                                                                       'in_ucast_pkts': '26500273',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '98450751115',
                                                                       'out_ucast_pkts': '73726980',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '98450751115',
                                                                       'in_ucast_pkts': '73726980',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '98450751115',
                                                                                                'in_ucast_pkts': '73726980',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17335',
                                                                                                'out_octets': '5937602802',
                                                                                                'out_ucast_pkts': '26500273',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17335',
                                                                       'out_octets': '5937602802',
                                                                       'out_ucast_pkts': '26500273',
                                                                       'poller_hash': 'QURUTjIzMTI1MTFEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511495',
                                                                                                                        'fiber_length': 540,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTFEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTExRA==',
                                         'serial_number': 'ADTN2312511D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511495'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '309',
                                                                       'in_octets': '6398862455',
                                                                       'in_ucast_pkts': '46841830',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '218148219816',
                                                                       'out_ucast_pkts': '148815866',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '218148219816',
                                                                       'in_ucast_pkts': '148815866',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '218148219816',
                                                                                                'in_ucast_pkts': '148815866',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '309',
                                                                                                'out_octets': '6398862455',
                                                                                                'out_ucast_pkts': '46841830',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '309',
                                                                       'out_octets': '6398862455',
                                                                       'out_ucast_pkts': '46841830',
                                                                       'poller_hash': 'QURUTjIzMTI0NDkxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511515',
                                                                                                                        'fiber_length': 6819,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NDkxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDQ5MQ==',
                                         'serial_number': 'ADTN23124491',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511515'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17349',
                                                                       'in_octets': '535160215',
                                                                       'in_ucast_pkts': '1205277',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '2244417648',
                                                                       'out_ucast_pkts': '1800523',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VOF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '2244417648',
                                                                       'in_ucast_pkts': '1800523',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '2244417648',
                                                                                                'in_ucast_pkts': '1800523',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17349',
                                                                                                'out_octets': '535160215',
                                                                                                'out_ucast_pkts': '1205277',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17349',
                                                                       'out_octets': '535160215',
                                                                       'out_ucast_pkts': '1205277',
                                                                       'poller_hash': 'QURUTjIzMTI1MTQzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511506',
                                                                                                                        'fiber_length': 3430,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTQzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0Mw==',
                                         'serial_number': 'ADTN23125143',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511506'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17346',
                                                                       'in_octets': '2257714498',
                                                                       'in_ucast_pkts': '10560616',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '36643183297',
                                                                       'out_ucast_pkts': '27915703',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '36643183297',
                                                                       'in_ucast_pkts': '27915703',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '36643183297',
                                                                                                'in_ucast_pkts': '27915703',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17346',
                                                                                                'out_octets': '2257714498',
                                                                                                'out_ucast_pkts': '10560616',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17346',
                                                                       'out_octets': '2257714498',
                                                                       'out_ucast_pkts': '10560616',
                                                                       'poller_hash': 'QURUTjIzMTI1MEZFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511500',
                                                                                                                        'fiber_length': 4938,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEZFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBGRQ==',
                                         'serial_number': 'ADTN231250FE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '168735',
                                                                       'in_octets': '100638156252',
                                                                       'in_ucast_pkts': '616564644',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23123FF5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14159',
                                                                       'out_octets': '973818902633',
                                                                       'out_ucast_pkts': '937248545',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTIzRkY1X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '14159',
                                                                       'in_octets': '973818902633',
                                                                       'in_ucast_pkts': '937248545',
                                                                       'links': [       {       'in_nucast_pkts': '14159',
                                                                                                'in_octets': '973818902633',
                                                                                                'in_ucast_pkts': '937248545',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '168735',
                                                                                                'out_octets': '100638156252',
                                                                                                'out_ucast_pkts': '616564644',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '168735',
                                                                       'out_octets': '100638156252',
                                                                       'out_ucast_pkts': '616564644',
                                                                       'poller_hash': 'QURUTjIzMTIzRkY1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4936369',
                                                                                                                        'fiber_length': 5995,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkY1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23123FF5',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZGNQ==',
                                         'serial_number': 'ADTN23123FF5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '4936369'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17349',
                                                                       'in_octets': '57260055059',
                                                                       'in_ucast_pkts': '185213121',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '715177338108',
                                                                       'out_ucast_pkts': '502819501',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VOV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '715177338108',
                                                                       'in_ucast_pkts': '502819501',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '715177338108',
                                                                                                'in_ucast_pkts': '502819501',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17349',
                                                                                                'out_octets': '57260055059',
                                                                                                'out_ucast_pkts': '185213121',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17349',
                                                                       'out_octets': '57260055059',
                                                                       'out_ucast_pkts': '185213121',
                                                                       'poller_hash': 'QURUTjIzMTI1MTAzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511509',
                                                                                                                        'fiber_length': 3301,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTAzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEwMw==',
                                         'serial_number': 'ADTN23125103',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511509'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17430',
                                                                       'in_octets': '19898956699',
                                                                       'in_ucast_pkts': '131222900',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '357137099261',
                                                                       'out_ucast_pkts': '244740097',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '357137099261',
                                                                       'in_ucast_pkts': '244740097',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '357137099261',
                                                                                                'in_ucast_pkts': '244740097',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17430',
                                                                                                'out_octets': '19898956699',
                                                                                                'out_ucast_pkts': '131222900',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17430',
                                                                       'out_octets': '19898956699',
                                                                       'out_ucast_pkts': '131222900',
                                                                       'poller_hash': 'QURUTjIzMTI1MTI5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511507',
                                                                                                                        'fiber_length': 4696,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTI5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEyOQ==',
                                         'serial_number': 'ADTN23125129',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511507'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13959',
                                                                       'in_octets': '13239955667',
                                                                       'in_ucast_pkts': '90565957',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '454831495229',
                                                                       'out_ucast_pkts': '310897979',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '454831495229',
                                                                       'in_ucast_pkts': '310897979',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '454831495229',
                                                                                                'in_ucast_pkts': '310897979',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13959',
                                                                                                'out_octets': '13239955667',
                                                                                                'out_ucast_pkts': '90565957',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13959',
                                                                       'out_octets': '13239955667',
                                                                       'out_ucast_pkts': '90565957',
                                                                       'poller_hash': 'QURUTjIzMTI0MzJFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411500',
                                                                                                                        'fiber_length': 9068,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MzJFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDMyRQ==',
                                         'serial_number': 'ADTN2312432E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '32302',
                                                                       'in_octets': '15455041408',
                                                                       'in_ucast_pkts': '68701404',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2719',
                                                                       'out_octets': '311664606418',
                                                                       'out_ucast_pkts': '223820942',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTlfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2719',
                                                                       'in_octets': '311664606418',
                                                                       'in_ucast_pkts': '223820942',
                                                                       'links': [       {       'in_nucast_pkts': '2719',
                                                                                                'in_octets': '311664606418',
                                                                                                'in_ucast_pkts': '223820942',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '32302',
                                                                                                'out_octets': '15455041408',
                                                                                                'out_ucast_pkts': '68701404',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '32302',
                                                                       'out_octets': '15455041408',
                                                                       'out_ucast_pkts': '68701404',
                                                                       'poller_hash': 'QURUTjIzMTI1MEMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '946258',
                                                                                                                        'fiber_length': 8251,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDMA==',
                                         'serial_number': 'ADTN231250C0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '946258'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '32074',
                                                                       'in_octets': '15119379583',
                                                                       'in_ucast_pkts': '98696275',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2713',
                                                                       'out_octets': '354591641228',
                                                                       'out_ucast_pkts': '250169385',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTEwX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2713',
                                                                       'in_octets': '354591641228',
                                                                       'in_ucast_pkts': '250169385',
                                                                       'links': [       {       'in_nucast_pkts': '2713',
                                                                                                'in_octets': '354591641228',
                                                                                                'in_ucast_pkts': '250169385',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '32074',
                                                                                                'out_octets': '15119379583',
                                                                                                'out_ucast_pkts': '98696275',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '32074',
                                                                       'out_octets': '15119379583',
                                                                       'out_ucast_pkts': '98696275',
                                                                       'poller_hash': 'QURUTjIzMTI1MEJGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '946255',
                                                                                                                        'fiber_length': 8000,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEJGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBCRg==',
                                         'serial_number': 'ADTN231250BF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '946255'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17345',
                                                                       'in_octets': '7099641883',
                                                                       'in_ucast_pkts': '60506241',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '149997063454',
                                                                       'out_ucast_pkts': '111038622',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '149997063454',
                                                                       'in_ucast_pkts': '111038622',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '149997063454',
                                                                                                'in_ucast_pkts': '111038622',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17345',
                                                                                                'out_octets': '7099641883',
                                                                                                'out_ucast_pkts': '60506241',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17345',
                                                                       'out_octets': '7099641883',
                                                                       'out_ucast_pkts': '60506241',
                                                                       'poller_hash': 'QURUTjIzMTI0ODQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511506',
                                                                                                                        'fiber_length': 9154,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0ODQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDg0OQ==',
                                         'serial_number': 'ADTN23124849',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511506'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '79948',
                                                                       'in_octets': '44008802832',
                                                                       'in_ucast_pkts': '316733739',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6646',
                                                                       'out_octets': '1441341370416',
                                                                       'out_ucast_pkts': '988837617',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6646',
                                                                       'in_octets': '1441341370416',
                                                                       'in_ucast_pkts': '988837617',
                                                                       'links': [       {       'in_nucast_pkts': '6646',
                                                                                                'in_octets': '1441341370416',
                                                                                                'in_ucast_pkts': '988837617',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '79948',
                                                                                                'out_octets': '44008802832',
                                                                                                'out_ucast_pkts': '316733739',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '79948',
                                                                       'out_octets': '44008802832',
                                                                       'out_ucast_pkts': '316733739',
                                                                       'poller_hash': 'QURUTjIzMTIzRUJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317053',
                                                                                                                        'fiber_length': 2230,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRUJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0VCQw==',
                                         'serial_number': 'ADTN23123EBC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317053'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17423',
                                                                       'in_octets': '6462624002',
                                                                       'in_ucast_pkts': '42174636',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '172648586399',
                                                                       'out_ucast_pkts': '122316571',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTRfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '172648586399',
                                                                       'in_ucast_pkts': '122316571',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '172648586399',
                                                                                                'in_ucast_pkts': '122316571',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17423',
                                                                                                'out_octets': '6462624002',
                                                                                                'out_ucast_pkts': '42174636',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17423',
                                                                       'out_octets': '6462624002',
                                                                       'out_ucast_pkts': '42174636',
                                                                       'poller_hash': 'QURUTjIzMTI1MTE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511490',
                                                                                                                        'fiber_length': 1746,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTExNg==',
                                         'serial_number': 'ADTN23125116',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511490'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78531',
                                                                       'in_octets': '11540874941',
                                                                       'in_ucast_pkts': '73015080',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6647',
                                                                       'out_octets': '296399143311',
                                                                       'out_ucast_pkts': '208464895',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6647',
                                                                       'in_octets': '296399143311',
                                                                       'in_ucast_pkts': '208464895',
                                                                       'links': [       {       'in_nucast_pkts': '6647',
                                                                                                'in_octets': '296399143311',
                                                                                                'in_ucast_pkts': '208464895',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78531',
                                                                                                'out_octets': '11540874941',
                                                                                                'out_ucast_pkts': '73015080',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78531',
                                                                       'out_octets': '11540874941',
                                                                       'out_ucast_pkts': '73015080',
                                                                       'poller_hash': 'QURUTjIzMTIzRjE4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317061',
                                                                                                                        'fiber_length': 5797,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRjE4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0YxOA==',
                                         'serial_number': 'ADTN23123F18',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317061'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '79176',
                                                                       'in_octets': '37525762453',
                                                                       'in_ucast_pkts': '286489395',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '897851171839',
                                                                       'out_ucast_pkts': '626176397',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VNV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '897851171839',
                                                                       'in_ucast_pkts': '626176397',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '897851171839',
                                                                                                'in_ucast_pkts': '626176397',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '79176',
                                                                                                'out_octets': '37525762453',
                                                                                                'out_ucast_pkts': '286489395',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '79176',
                                                                       'out_octets': '37525762453',
                                                                       'out_ucast_pkts': '286489395',
                                                                       'poller_hash': 'QURUTjIzMTI1MTk1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317059',
                                                                                                                        'fiber_length': 6115,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTk1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE5NQ==',
                                         'serial_number': 'ADTN23125195',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317059'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17349',
                                                                       'in_octets': '6949689103',
                                                                       'in_ucast_pkts': '30596842',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '154299720484',
                                                                       'out_ucast_pkts': '111934773',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTExX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '154299720484',
                                                                       'in_ucast_pkts': '111934773',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '154299720484',
                                                                                                'in_ucast_pkts': '111934773',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17349',
                                                                                                'out_octets': '6949689103',
                                                                                                'out_ucast_pkts': '30596842',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17349',
                                                                       'out_octets': '6949689103',
                                                                       'out_ucast_pkts': '30596842',
                                                                       'poller_hash': 'QURUTjIzMTIzRUMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511499',
                                                                                                                        'fiber_length': 4770,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRUMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0VDMA==',
                                         'serial_number': 'ADTN23123EC0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78533',
                                                                       'in_octets': '66612886069',
                                                                       'in_ucast_pkts': '366803880',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6651',
                                                                       'out_octets': '1174174610445',
                                                                       'out_ucast_pkts': '859824473',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6651',
                                                                       'in_octets': '1174174610445',
                                                                       'in_ucast_pkts': '859824473',
                                                                       'links': [       {       'in_nucast_pkts': '6651',
                                                                                                'in_octets': '1174174610445',
                                                                                                'in_ucast_pkts': '859824473',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78533',
                                                                                                'out_octets': '66612886069',
                                                                                                'out_ucast_pkts': '366803880',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78533',
                                                                       'out_octets': '66612886069',
                                                                       'out_ucast_pkts': '366803880',
                                                                       'poller_hash': 'QURUTjIzMTI0NEVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317059',
                                                                                                                        'fiber_length': 8586,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NEVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDRFQg==',
                                         'serial_number': 'ADTN231244EB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317059'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '32073',
                                                                       'in_octets': '9337586634',
                                                                       'in_ucast_pkts': '22417412',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2713',
                                                                       'out_octets': '47856079004',
                                                                       'out_ucast_pkts': '45368420',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTEyX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2713',
                                                                       'in_octets': '47856079004',
                                                                       'in_ucast_pkts': '45368420',
                                                                       'links': [       {       'in_nucast_pkts': '2713',
                                                                                                'in_octets': '47856079004',
                                                                                                'in_ucast_pkts': '45368420',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '32073',
                                                                                                'out_octets': '9337586634',
                                                                                                'out_ucast_pkts': '22417412',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '32073',
                                                                       'out_octets': '9337586634',
                                                                       'out_ucast_pkts': '22417412',
                                                                       'poller_hash': 'QURUTjIzMTIzRkVFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '946255',
                                                                                                                        'fiber_length': 8343,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkVFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZFRQ==',
                                         'serial_number': 'ADTN23123FEE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '946255'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '380493',
                                                                       'in_octets': '117104594',
                                                                       'in_ucast_pkts': '0',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '380493',
                                                                                                'out_octets': '117104594',
                                                                                                'out_ucast_pkts': '0',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '380493',
                                                                       'out_octets': '117104594',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'QURUTjIzMTI1MEM2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411500',
                                                                                                                        'fiber_length': 9038,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEM2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDNg==',
                                         'serial_number': 'ADTN231250C6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181362',
                                                                       'in_octets': '156763924400',
                                                                       'in_ucast_pkts': '451405583',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6046',
                                                                       'out_octets': '1183497788545',
                                                                       'out_ucast_pkts': '929911749',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VOV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6046',
                                                                       'in_octets': '1183497788545',
                                                                       'in_ucast_pkts': '929911749',
                                                                       'links': [       {       'in_nucast_pkts': '6046',
                                                                                                'in_octets': '1183497788545',
                                                                                                'in_ucast_pkts': '929911749',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181362',
                                                                                                'out_octets': '156763924400',
                                                                                                'out_ucast_pkts': '451405583',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181362',
                                                                       'out_octets': '156763924400',
                                                                       'out_ucast_pkts': '451405583',
                                                                       'poller_hash': 'QURUTjIzMTI1MTNBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440102',
                                                                                                                        'fiber_length': 9589,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTNBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEzQQ==',
                                         'serial_number': 'ADTN2312513A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440102'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '186588',
                                                                       'in_octets': '11791523102',
                                                                       'in_ucast_pkts': '32348353',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15625',
                                                                       'out_octets': '45114517815',
                                                                       'out_ucast_pkts': '40397613',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15625',
                                                                       'in_octets': '45114517815',
                                                                       'in_ucast_pkts': '40397613',
                                                                       'links': [       {       'in_nucast_pkts': '15625',
                                                                                                'in_octets': '45114517815',
                                                                                                'in_ucast_pkts': '40397613',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '186588',
                                                                                                'out_octets': '11791523102',
                                                                                                'out_ucast_pkts': '32348353',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '186588',
                                                                       'out_octets': '11791523102',
                                                                       'out_ucast_pkts': '32348353',
                                                                       'poller_hash': 'QURUTjIzMTI0MUMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440102',
                                                                                                                        'fiber_length': 10748,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MUMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDFDMA==',
                                         'serial_number': 'ADTN231241C0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440102'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78016',
                                                                       'in_octets': '64001662950',
                                                                       'in_ucast_pkts': '217080517',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '624482317804',
                                                                       'out_ucast_pkts': '459967946',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '624482317804',
                                                                       'in_ucast_pkts': '459967946',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '624482317804',
                                                                                                'in_ucast_pkts': '459967946',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78016',
                                                                                                'out_octets': '64001662950',
                                                                                                'out_ucast_pkts': '217080517',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78016',
                                                                       'out_octets': '64001662950',
                                                                       'out_ucast_pkts': '217080517',
                                                                       'poller_hash': 'QURUTjIzMTI1MTJGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317051',
                                                                                                                        'fiber_length': 1857,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTJGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEyRg==',
                                         'serial_number': 'ADTN2312512F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317051'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '80463',
                                                                       'in_octets': '123766625329',
                                                                       'in_ucast_pkts': '306487604',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6653',
                                                                       'out_octets': '1029083777933',
                                                                       'out_ucast_pkts': '824202792',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6653',
                                                                       'in_octets': '1029083777933',
                                                                       'in_ucast_pkts': '824202792',
                                                                       'links': [       {       'in_nucast_pkts': '6653',
                                                                                                'in_octets': '1029083777933',
                                                                                                'in_ucast_pkts': '824202792',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '80463',
                                                                                                'out_octets': '123766625329',
                                                                                                'out_ucast_pkts': '306487604',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '80463',
                                                                       'out_octets': '123766625329',
                                                                       'out_ucast_pkts': '306487604',
                                                                       'poller_hash': 'QURUTjIzMTI1MENDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317061',
                                                                                                                        'fiber_length': 6181,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MENDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDQw==',
                                         'serial_number': 'ADTN231250CC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317061'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '117189',
                                                                       'in_octets': '120100217958',
                                                                       'in_ucast_pkts': '671422934',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9922',
                                                                       'out_octets': '2910180516762',
                                                                       'out_ucast_pkts': '2077796789',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9922',
                                                                       'in_octets': '2910180516762',
                                                                       'in_ucast_pkts': '2077796789',
                                                                       'links': [       {       'in_nucast_pkts': '9922',
                                                                                                'in_octets': '2910180516762',
                                                                                                'in_ucast_pkts': '2077796789',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '117189',
                                                                                                'out_octets': '120100217958',
                                                                                                'out_ucast_pkts': '671422934',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '117189',
                                                                       'out_octets': '120100217958',
                                                                       'out_ucast_pkts': '671422934',
                                                                       'poller_hash': 'QURUTjIzMTI1MTU4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3456381',
                                                                                                                        'fiber_length': 10811,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTU4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE1OA==',
                                         'serial_number': 'ADTN23125158',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3456381'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '115000',
                                                                       'in_octets': '66411983841',
                                                                       'in_ucast_pkts': '250512278',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3835',
                                                                       'out_octets': '1423739286835',
                                                                       'out_ucast_pkts': '1044417982',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTEzX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3835',
                                                                       'in_octets': '1423739286835',
                                                                       'in_ucast_pkts': '1044417982',
                                                                       'links': [       {       'in_nucast_pkts': '3835',
                                                                                                'in_octets': '1423739286835',
                                                                                                'in_ucast_pkts': '1044417982',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '115000',
                                                                                                'out_octets': '66411983841',
                                                                                                'out_ucast_pkts': '250512278',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '115000',
                                                                       'out_octets': '66411983841',
                                                                       'out_ucast_pkts': '250512278',
                                                                       'poller_hash': 'QURUTjIzMTIzRkU5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3449049',
                                                                                                                        'fiber_length': 8068,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkU5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZFOQ==',
                                         'serial_number': 'ADTN23123FE9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3449049'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-3.1.17.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x3-ONU17_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTdfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0MDk3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MDk3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x3-ONU17',
                                         'offline_reason': 'OFFLINE_REASON_POWER_OFF',
                                         'offline_reason_detail': 'A local power '
                                                                  'outage has occurred '
                                                                  'for ONU "onu '
                                                                  '3.1.17"(ADTN23124097).',
                                         'offline_time': '2025-07-31T22:19:54Z',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDA5Nw==',
                                         'serial_number': 'ADTN23124097',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '4149318',
                                                                       'in_octets': '1218507587',
                                                                       'in_ucast_pkts': '1097786',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '4149318',
                                                                                                'out_octets': '1218507587',
                                                                                                'out_ucast_pkts': '1097786',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4149318',
                                                                       'out_octets': '1218507587',
                                                                       'out_ucast_pkts': '1097786',
                                                                       'poller_hash': 'QURUTjIzMTI1MTExIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317060',
                                                                                                                        'fiber_length': 6734,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTExIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTExMQ==',
                                         'serial_number': 'ADTN23125111',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317060'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17357',
                                                                       'in_octets': '8271351533',
                                                                       'in_ucast_pkts': '41007607',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '239064115958',
                                                                       'out_ucast_pkts': '176634277',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '239064115958',
                                                                       'in_ucast_pkts': '176634277',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '239064115958',
                                                                                                'in_ucast_pkts': '176634277',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17357',
                                                                                                'out_octets': '8271351533',
                                                                                                'out_ucast_pkts': '41007607',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17357',
                                                                       'out_octets': '8271351533',
                                                                       'out_ucast_pkts': '41007607',
                                                                       'poller_hash': 'QURUTjIzMTI1MTJFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 3383,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTJFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEyRQ==',
                                         'serial_number': 'ADTN2312512E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '79067',
                                                                       'in_octets': '26256197341',
                                                                       'in_ucast_pkts': '85186117',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6630',
                                                                       'out_octets': '417679382577',
                                                                       'out_ucast_pkts': '303843932',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6630',
                                                                       'in_octets': '417679382577',
                                                                       'in_ucast_pkts': '303843932',
                                                                       'links': [       {       'in_nucast_pkts': '6630',
                                                                                                'in_octets': '417679382577',
                                                                                                'in_ucast_pkts': '303843932',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '79067',
                                                                                                'out_octets': '26256197341',
                                                                                                'out_ucast_pkts': '85186117',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '79067',
                                                                       'out_octets': '26256197341',
                                                                       'out_ucast_pkts': '85186117',
                                                                       'poller_hash': 'QURUTjIzMTI1MTI2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317056',
                                                                                                                        'fiber_length': 4079,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTI2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEyNg==',
                                         'serial_number': 'ADTN23125126',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317056'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17069',
                                                                       'in_octets': '5704883251',
                                                                       'in_ucast_pkts': '7721731',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '11207396635',
                                                                       'out_ucast_pkts': '10837380',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTIxX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '11207396635',
                                                                       'in_ucast_pkts': '10837380',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '11207396635',
                                                                                                'in_ucast_pkts': '10837380',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17069',
                                                                                                'out_octets': '5704883251',
                                                                                                'out_ucast_pkts': '7721731',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17069',
                                                                       'out_octets': '5704883251',
                                                                       'out_ucast_pkts': '7721731',
                                                                       'poller_hash': 'QURUTjIzMTI1MTRDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511499',
                                                                                                                        'fiber_length': 5052,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTRDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0Qw==',
                                         'serial_number': 'ADTN2312514C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511499'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-7.1.11.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x7-ONU11_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTFfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI1MTBDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTBDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x7-ONU11',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEwQw==',
                                         'serial_number': 'ADTN2312510C',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181362',
                                                                       'in_octets': '117476837393',
                                                                       'in_ucast_pkts': '782362731',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6046',
                                                                       'out_octets': '2690641132374',
                                                                       'out_ucast_pkts': '1905365208',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6046',
                                                                       'in_octets': '2690641132374',
                                                                       'in_ucast_pkts': '1905365208',
                                                                       'links': [       {       'in_nucast_pkts': '6046',
                                                                                                'in_octets': '2690641132374',
                                                                                                'in_ucast_pkts': '1905365208',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181362',
                                                                                                'out_octets': '117476837393',
                                                                                                'out_ucast_pkts': '782362731',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181362',
                                                                       'out_octets': '117476837393',
                                                                       'out_ucast_pkts': '782362731',
                                                                       'poller_hash': 'QURUTjIzMTI1MjA2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440097',
                                                                                                                        'fiber_length': 7635,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MjA2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTIwNg==',
                                         'serial_number': 'ADTN23125206',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440097'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181344',
                                                                       'in_octets': '19922779626',
                                                                       'in_ucast_pkts': '37878045',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6046',
                                                                       'out_octets': '73347970096',
                                                                       'out_ucast_pkts': '65446477',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6046',
                                                                       'in_octets': '73347970096',
                                                                       'in_ucast_pkts': '65446477',
                                                                       'links': [       {       'in_nucast_pkts': '6046',
                                                                                                'in_octets': '73347970096',
                                                                                                'in_ucast_pkts': '65446477',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181344',
                                                                                                'out_octets': '19922779626',
                                                                                                'out_ucast_pkts': '37878045',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181344',
                                                                       'out_octets': '19922779626',
                                                                       'out_ucast_pkts': '37878045',
                                                                       'poller_hash': 'QURUTjIzMTI1MEREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440069',
                                                                                                                        'fiber_length': 8976,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBERA==',
                                         'serial_number': 'ADTN231250DD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440069'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '184824',
                                                                       'in_octets': '120709171346',
                                                                       'in_ucast_pkts': '222440403',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15627',
                                                                       'out_octets': '704134560785',
                                                                       'out_ucast_pkts': '529806623',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15627',
                                                                       'in_octets': '704134560785',
                                                                       'in_ucast_pkts': '529806623',
                                                                       'links': [       {       'in_nucast_pkts': '15627',
                                                                                                'in_octets': '704134560785',
                                                                                                'in_ucast_pkts': '529806623',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '184824',
                                                                                                'out_octets': '120709171346',
                                                                                                'out_ucast_pkts': '222440403',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '184824',
                                                                       'out_octets': '120709171346',
                                                                       'out_ucast_pkts': '222440403',
                                                                       'poller_hash': 'QURUTjIzMTI1MEJCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440069',
                                                                                                                        'fiber_length': 8993,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEJCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBCQg==',
                                         'serial_number': 'ADTN231250BB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440069'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '116887',
                                                                       'in_octets': '174393990995',
                                                                       'in_ucast_pkts': '494997024',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9894',
                                                                       'out_octets': '1118290591416',
                                                                       'out_ucast_pkts': '932843019',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9894',
                                                                       'in_octets': '1118290591416',
                                                                       'in_ucast_pkts': '932843019',
                                                                       'links': [       {       'in_nucast_pkts': '9894',
                                                                                                'in_octets': '1118290591416',
                                                                                                'in_ucast_pkts': '932843019',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '116887',
                                                                                                'out_octets': '174393990995',
                                                                                                'out_ucast_pkts': '494997024',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '116887',
                                                                       'out_octets': '174393990995',
                                                                       'out_ucast_pkts': '494997024',
                                                                       'poller_hash': 'QURUTjIzMTI1MTBCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3449060',
                                                                                                                        'fiber_length': 6776,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTBCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEwQg==',
                                         'serial_number': 'ADTN2312510B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3449060'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13739',
                                                                       'in_octets': '42012549892',
                                                                       'in_ucast_pkts': '61902384',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '291478069020',
                                                                       'out_ucast_pkts': '212080804',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTEwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '291478069020',
                                                                       'in_ucast_pkts': '212080804',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '291478069020',
                                                                                                'in_ucast_pkts': '212080804',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13739',
                                                                                                'out_octets': '42012549892',
                                                                                                'out_ucast_pkts': '61902384',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13739',
                                                                       'out_octets': '42012549892',
                                                                       'out_ucast_pkts': '61902384',
                                                                       'poller_hash': 'QURUTjIzMTI1MTk0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411493',
                                                                                                                        'fiber_length': 9481,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTk0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE5NA==',
                                         'serial_number': 'ADTN23125194',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411493'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-7.1.12.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x7-ONU12_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTJfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI1MTQ4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x7-ONU12',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0OA==',
                                         'serial_number': 'ADTN23125148',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14289',
                                                                       'in_octets': '2854488104',
                                                                       'in_ucast_pkts': '9495644',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '40971690775',
                                                                       'out_ucast_pkts': '29089697',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTExX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '40971690775',
                                                                       'in_ucast_pkts': '29089697',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '40971690775',
                                                                                                'in_ucast_pkts': '29089697',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14289',
                                                                                                'out_octets': '2854488104',
                                                                                                'out_ucast_pkts': '9495644',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14289',
                                                                       'out_octets': '2854488104',
                                                                       'out_ucast_pkts': '9495644',
                                                                       'poller_hash': 'QURUTjIzMTI1MTRFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411499',
                                                                                                                        'fiber_length': 8614,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTRFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0RQ==',
                                         'serial_number': 'ADTN2312514E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77257',
                                                                       'in_octets': '25913101840',
                                                                       'in_ucast_pkts': '195827682',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '538979312501',
                                                                       'out_ucast_pkts': '388999274',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VOV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '538979312501',
                                                                       'in_ucast_pkts': '388999274',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '538979312501',
                                                                                                'in_ucast_pkts': '388999274',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77257',
                                                                                                'out_octets': '25913101840',
                                                                                                'out_ucast_pkts': '195827682',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77257',
                                                                       'out_octets': '25913101840',
                                                                       'out_ucast_pkts': '195827682',
                                                                       'poller_hash': 'QURUTjIzMTI1MTREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317060',
                                                                                                                        'fiber_length': 5523,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0RA==',
                                         'serial_number': 'ADTN2312514D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317060'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77257',
                                                                       'in_octets': '19421389746',
                                                                       'in_ucast_pkts': '134022591',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '396004731028',
                                                                       'out_ucast_pkts': '273905695',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '396004731028',
                                                                       'in_ucast_pkts': '273905695',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '396004731028',
                                                                                                'in_ucast_pkts': '273905695',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77257',
                                                                                                'out_octets': '19421389746',
                                                                                                'out_ucast_pkts': '134022591',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77257',
                                                                       'out_octets': '19421389746',
                                                                       'out_ucast_pkts': '134022591',
                                                                       'poller_hash': 'QURUTjIzMTI1MENEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317056',
                                                                                                                        'fiber_length': 5847,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MENEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDRA==',
                                         'serial_number': 'ADTN231250CD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317056'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77253',
                                                                       'in_octets': '28178292115',
                                                                       'in_ucast_pkts': '152473006',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2570',
                                                                       'out_octets': '693591396875',
                                                                       'out_ucast_pkts': '490410131',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2570',
                                                                       'in_octets': '693591396875',
                                                                       'in_ucast_pkts': '490410131',
                                                                       'links': [       {       'in_nucast_pkts': '2570',
                                                                                                'in_octets': '693591396875',
                                                                                                'in_ucast_pkts': '490410131',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77253',
                                                                                                'out_octets': '28178292115',
                                                                                                'out_ucast_pkts': '152473006',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77253',
                                                                       'out_octets': '28178292115',
                                                                       'out_ucast_pkts': '152473006',
                                                                       'poller_hash': 'QURUTjIzMTI1MTUzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317051',
                                                                                                                        'fiber_length': 3719,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTUzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE1Mw==',
                                         'serial_number': 'ADTN23125153',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317051'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17347',
                                                                       'in_octets': '4868512344',
                                                                       'in_ucast_pkts': '51003832',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '154865189855',
                                                                       'out_ucast_pkts': '104664563',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTIyX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '154865189855',
                                                                       'in_ucast_pkts': '104664563',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '154865189855',
                                                                                                'in_ucast_pkts': '104664563',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17347',
                                                                                                'out_octets': '4868512344',
                                                                                                'out_ucast_pkts': '51003832',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17347',
                                                                       'out_octets': '4868512344',
                                                                       'out_ucast_pkts': '51003832',
                                                                       'poller_hash': 'QURUTjIzMTIzRkM0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511501',
                                                                                                                        'fiber_length': 3515,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkM0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU22',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZDNA==',
                                         'serial_number': 'ADTN23123FC4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511501'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17345',
                                                                       'in_octets': '27657546519',
                                                                       'in_ucast_pkts': '50955624',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x14-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '136880871237',
                                                                       'out_ucast_pkts': '103266280',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDE0LU9OVTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '136880871237',
                                                                       'in_ucast_pkts': '103266280',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '136880871237',
                                                                                                'in_ucast_pkts': '103266280',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17345',
                                                                                                'out_octets': '27657546519',
                                                                                                'out_ucast_pkts': '50955624',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17345',
                                                                       'out_octets': '27657546519',
                                                                       'out_ucast_pkts': '50955624',
                                                                       'poller_hash': 'QURUTjIzMTI1MTUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511506',
                                                                                                                        'fiber_length': 1195,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x14-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE1MA==',
                                         'serial_number': 'ADTN23125150',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5-5-b',
                                         'uptime': '511506'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78525',
                                                                       'in_octets': '25125301558',
                                                                       'in_ucast_pkts': '150490446',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6626',
                                                                       'out_octets': '539450720535',
                                                                       'out_ucast_pkts': '375595976',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6626',
                                                                       'in_octets': '539450720535',
                                                                       'in_ucast_pkts': '375595976',
                                                                       'links': [       {       'in_nucast_pkts': '6626',
                                                                                                'in_octets': '539450720535',
                                                                                                'in_ucast_pkts': '375595976',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78525',
                                                                                                'out_octets': '25125301558',
                                                                                                'out_ucast_pkts': '150490446',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78525',
                                                                       'out_octets': '25125301558',
                                                                       'out_ucast_pkts': '150490446',
                                                                       'poller_hash': 'QURUTjIzMTI1MUFBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317055',
                                                                                                                        'fiber_length': 4022,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUFBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFBQQ==',
                                         'serial_number': 'ADTN231251AA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317055'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '43114',
                                                                       'in_octets': '4458164244',
                                                                       'in_ucast_pkts': '13682224',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1417',
                                                                       'out_octets': '39806039770',
                                                                       'out_ucast_pkts': '28911659',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1417',
                                                                       'in_octets': '39806039770',
                                                                       'in_ucast_pkts': '28911659',
                                                                       'links': [       {       'in_nucast_pkts': '1417',
                                                                                                'in_octets': '39806039770',
                                                                                                'in_ucast_pkts': '28911659',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '43114',
                                                                                                'out_octets': '4458164244',
                                                                                                'out_ucast_pkts': '13682224',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '43114',
                                                                       'out_octets': '4458164244',
                                                                       'out_ucast_pkts': '13682224',
                                                                       'poller_hash': 'QURUTjIzMTI1MTRGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1293152',
                                                                                                                        'fiber_length': 3775,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTRGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0Rg==',
                                         'serial_number': 'ADTN2312514F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1293152'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77255',
                                                                       'in_octets': '19655585764',
                                                                       'in_ucast_pkts': '71781369',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2572',
                                                                       'out_octets': '275176434769',
                                                                       'out_ucast_pkts': '202804538',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2572',
                                                                       'in_octets': '275176434769',
                                                                       'in_ucast_pkts': '202804538',
                                                                       'links': [       {       'in_nucast_pkts': '2572',
                                                                                                'in_octets': '275176434769',
                                                                                                'in_ucast_pkts': '202804538',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77255',
                                                                                                'out_octets': '19655585764',
                                                                                                'out_ucast_pkts': '71781369',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77255',
                                                                       'out_octets': '19655585764',
                                                                       'out_ucast_pkts': '71781369',
                                                                       'poller_hash': 'QURUTjIzMTI0MUJGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317058',
                                                                                                                        'fiber_length': 4059,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MUJGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU22',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDFCRg==',
                                         'serial_number': 'ADTN231241BF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317058'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '308',
                                                                       'in_octets': '6353151082',
                                                                       'in_ucast_pkts': '36858175',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1466',
                                                                       'out_octets': '300504350152',
                                                                       'out_ucast_pkts': '215317430',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1466',
                                                                       'in_octets': '300504350152',
                                                                       'in_ucast_pkts': '215317430',
                                                                       'links': [       {       'in_nucast_pkts': '1466',
                                                                                                'in_octets': '300504350152',
                                                                                                'in_ucast_pkts': '215317430',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '308',
                                                                                                'out_octets': '6353151082',
                                                                                                'out_ucast_pkts': '36858175',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '308',
                                                                       'out_octets': '6353151082',
                                                                       'out_ucast_pkts': '36858175',
                                                                       'poller_hash': 'QURUTjIzMTIzRkRDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 4481,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkRDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZEQw==',
                                         'serial_number': 'ADTN23123FDC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '185143',
                                                                       'in_octets': '114224840443',
                                                                       'in_ucast_pkts': '284025037',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15627',
                                                                       'out_octets': '1198673274406',
                                                                       'out_ucast_pkts': '937808721',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15627',
                                                                       'in_octets': '1198673274406',
                                                                       'in_ucast_pkts': '937808721',
                                                                       'links': [       {       'in_nucast_pkts': '15627',
                                                                                                'in_octets': '1198673274406',
                                                                                                'in_ucast_pkts': '937808721',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '185143',
                                                                                                'out_octets': '114224840443',
                                                                                                'out_ucast_pkts': '284025037',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '185143',
                                                                       'out_octets': '114224840443',
                                                                       'out_ucast_pkts': '284025037',
                                                                       'poller_hash': 'QURUTjIzMTI0RTQ1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440096',
                                                                                                                        'fiber_length': 13527,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTQ1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU0NQ==',
                                         'serial_number': 'ADTN23124E45',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440096'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78535',
                                                                       'in_octets': '22993023654',
                                                                       'in_ucast_pkts': '119087151',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6649',
                                                                       'out_octets': '578989661367',
                                                                       'out_ucast_pkts': '410335584',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6649',
                                                                       'in_octets': '578989661367',
                                                                       'in_ucast_pkts': '410335584',
                                                                       'links': [       {       'in_nucast_pkts': '6649',
                                                                                                'in_octets': '578989661367',
                                                                                                'in_ucast_pkts': '410335584',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78535',
                                                                                                'out_octets': '22993023654',
                                                                                                'out_ucast_pkts': '119087151',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78535',
                                                                       'out_octets': '22993023654',
                                                                       'out_ucast_pkts': '119087151',
                                                                       'poller_hash': 'QURUTjIzMTI1MTRCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317059',
                                                                                                                        'fiber_length': 6444,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTRCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0Qg==',
                                         'serial_number': 'ADTN2312514B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317059'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17355',
                                                                       'in_octets': '3985209650',
                                                                       'in_ucast_pkts': '3739148',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '3947279049',
                                                                       'out_ucast_pkts': '3942003',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTIzX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '3947279049',
                                                                       'in_ucast_pkts': '3942003',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '3947279049',
                                                                                                'in_ucast_pkts': '3942003',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17355',
                                                                                                'out_octets': '3985209650',
                                                                                                'out_ucast_pkts': '3739148',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17355',
                                                                       'out_octets': '3985209650',
                                                                       'out_ucast_pkts': '3739148',
                                                                       'poller_hash': 'QURUTjIzMTI1MEMzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511504',
                                                                                                                        'fiber_length': 3450,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEMzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDMw==',
                                         'serial_number': 'ADTN231250C3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511504'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '134175',
                                                                       'in_octets': '41028323456',
                                                                       'in_ucast_pkts': '85142587',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11217',
                                                                       'out_octets': '362600897682',
                                                                       'out_ucast_pkts': '269818780',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '11217',
                                                                       'in_octets': '362600897682',
                                                                       'in_ucast_pkts': '269818780',
                                                                       'links': [       {       'in_nucast_pkts': '11217',
                                                                                                'in_octets': '362600897682',
                                                                                                'in_ucast_pkts': '269818780',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '134175',
                                                                                                'out_octets': '41028323456',
                                                                                                'out_ucast_pkts': '85142587',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '134175',
                                                                       'out_octets': '41028323456',
                                                                       'out_ucast_pkts': '85142587',
                                                                       'poller_hash': 'QURUTjIzMTI1MTQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3910768',
                                                                                                                        'fiber_length': 13074,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0OQ==',
                                         'serial_number': 'ADTN23125149',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3910768'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17490',
                                                                       'in_octets': '9334372387',
                                                                       'in_ucast_pkts': '31816728',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '184950235686',
                                                                       'out_ucast_pkts': '139574894',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '184950235686',
                                                                       'in_ucast_pkts': '139574894',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '184950235686',
                                                                                                'in_ucast_pkts': '139574894',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17490',
                                                                                                'out_octets': '9334372387',
                                                                                                'out_ucast_pkts': '31816728',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17490',
                                                                       'out_octets': '9334372387',
                                                                       'out_ucast_pkts': '31816728',
                                                                       'poller_hash': 'QURUTjIzMTI1MEM4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511500',
                                                                                                                        'fiber_length': 1996,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEM4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDOA==',
                                         'serial_number': 'ADTN231250C8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '114799',
                                                                       'in_octets': '57045873988',
                                                                       'in_ucast_pkts': '337607038',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3828',
                                                                       'out_octets': '1524879113596',
                                                                       'out_ucast_pkts': '1180391737',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE0X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3828',
                                                                       'in_octets': '1524879113596',
                                                                       'in_ucast_pkts': '1180391737',
                                                                       'links': [       {       'in_nucast_pkts': '3828',
                                                                                                'in_octets': '1524879113596',
                                                                                                'in_ucast_pkts': '1180391737',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '114799',
                                                                                                'out_octets': '57045873988',
                                                                                                'out_ucast_pkts': '337607038',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '114799',
                                                                       'out_octets': '57045873988',
                                                                       'out_ucast_pkts': '337607038',
                                                                       'poller_hash': 'QURUTjIzMTI1MEQxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3449048',
                                                                                                                        'fiber_length': 8127,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEQxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBEMQ==',
                                         'serial_number': 'ADTN231250D1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3449048'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '182787',
                                                                       'in_octets': '122285030297',
                                                                       'in_ucast_pkts': '286802952',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6048',
                                                                       'out_octets': '958305135954',
                                                                       'out_ucast_pkts': '731293813',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI0X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6048',
                                                                       'in_octets': '958305135954',
                                                                       'in_ucast_pkts': '731293813',
                                                                       'links': [       {       'in_nucast_pkts': '6048',
                                                                                                'in_octets': '958305135954',
                                                                                                'in_ucast_pkts': '731293813',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '182787',
                                                                                                'out_octets': '122285030297',
                                                                                                'out_ucast_pkts': '286802952',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '182787',
                                                                       'out_octets': '122285030297',
                                                                       'out_ucast_pkts': '286802952',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0E1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440060',
                                                                                                                        'fiber_length': 4061,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0E1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdBNQ==',
                                         'serial_number': 'ADTN231447A5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440060'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-2.1.2.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x2-ONU2_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMl9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0OEZDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEZDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x2-ONU2',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhGQw==',
                                         'serial_number': 'ADTN231448FC',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '64552',
                                                                       'in_octets': '100324784568',
                                                                       'in_ucast_pkts': '420812486',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2100',
                                                                       'out_octets': '1288077963545',
                                                                       'out_ucast_pkts': '932948319',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VNF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2100',
                                                                       'in_octets': '1288077963545',
                                                                       'in_ucast_pkts': '932948319',
                                                                       'links': [       {       'in_nucast_pkts': '2100',
                                                                                                'in_octets': '1288077963545',
                                                                                                'in_ucast_pkts': '932948319',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '64552',
                                                                                                'out_octets': '100324784568',
                                                                                                'out_ucast_pkts': '420812486',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '64552',
                                                                       'out_octets': '100324784568',
                                                                       'out_ucast_pkts': '420812486',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0Y5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1889034',
                                                                                                                        'fiber_length': 14757,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0Y5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdGOQ==',
                                         'serial_number': 'ADTN231447F9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1889034'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '184373',
                                                                       'in_octets': '23546687807',
                                                                       'in_ucast_pkts': '147610870',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15621',
                                                                       'out_octets': '718037747535',
                                                                       'out_ucast_pkts': '499348896',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15621',
                                                                       'in_octets': '718037747535',
                                                                       'in_ucast_pkts': '499348896',
                                                                       'links': [       {       'in_nucast_pkts': '15621',
                                                                                                'in_octets': '718037747535',
                                                                                                'in_ucast_pkts': '499348896',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '184373',
                                                                                                'out_octets': '23546687807',
                                                                                                'out_ucast_pkts': '147610870',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '184373',
                                                                       'out_octets': '23546687807',
                                                                       'out_ucast_pkts': '147610870',
                                                                       'poller_hash': 'QURUTjIzMTQ0NjgxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440063',
                                                                                                                        'fiber_length': 14954,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NjgxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDY4MQ==',
                                         'serial_number': 'ADTN23144681',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440063'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-11.1.25.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x11-ONU25_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI1X0VUSF8x'},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0NjY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NjY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x11-ONU25',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDY2Ng==',
                                         'serial_number': 'ADTN23144666',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77273',
                                                                       'in_octets': '20153608281',
                                                                       'in_ucast_pkts': '54448909',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '171013626473',
                                                                       'out_ucast_pkts': '133959810',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '171013626473',
                                                                       'in_ucast_pkts': '133959810',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '171013626473',
                                                                                                'in_ucast_pkts': '133959810',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77273',
                                                                                                'out_octets': '20153608281',
                                                                                                'out_ucast_pkts': '54448909',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77273',
                                                                       'out_octets': '20153608281',
                                                                       'out_ucast_pkts': '54448909',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkEzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317052',
                                                                                                                        'fiber_length': 6030,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkEzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZBMw==',
                                         'serial_number': 'ADTN231446A3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317052'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77267',
                                                                       'in_octets': '140871742311',
                                                                       'in_ucast_pkts': '650119495',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '2152820974527',
                                                                       'out_ucast_pkts': '1546960421',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '2152820974527',
                                                                       'in_ucast_pkts': '1546960421',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '2152820974527',
                                                                                                'in_ucast_pkts': '1546960421',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77267',
                                                                                                'out_octets': '140871742311',
                                                                                                'out_ucast_pkts': '650119495',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77267',
                                                                       'out_octets': '140871742311',
                                                                       'out_ucast_pkts': '650119495',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkI0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317058',
                                                                                                                        'fiber_length': 6378,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkI0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZCNA==',
                                         'serial_number': 'ADTN231446B4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317058'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '26506',
                                                                       'in_octets': '17636558073',
                                                                       'in_ucast_pkts': '103017701',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2241',
                                                                       'out_octets': '413594199234',
                                                                       'out_ucast_pkts': '300633066',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2241',
                                                                       'in_octets': '413594199234',
                                                                       'in_ucast_pkts': '300633066',
                                                                       'links': [       {       'in_nucast_pkts': '2241',
                                                                                                'in_octets': '413594199234',
                                                                                                'in_ucast_pkts': '300633066',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '26506',
                                                                                                'out_octets': '17636558073',
                                                                                                'out_ucast_pkts': '103017701',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '26506',
                                                                       'out_octets': '17636558073',
                                                                       'out_ucast_pkts': '103017701',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODlDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '781465',
                                                                                                                        'fiber_length': 14487,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODlDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDg5Qw==',
                                         'serial_number': 'ADTN2314489C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '781465'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '184380',
                                                                       'in_octets': '106130690715',
                                                                       'in_ucast_pkts': '366861213',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15628',
                                                                       'out_octets': '1545613386456',
                                                                       'out_ucast_pkts': '1120007761',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15628',
                                                                       'in_octets': '1545613386456',
                                                                       'in_ucast_pkts': '1120007761',
                                                                       'links': [       {       'in_nucast_pkts': '15628',
                                                                                                'in_octets': '1545613386456',
                                                                                                'in_ucast_pkts': '1120007761',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '184380',
                                                                                                'out_octets': '106130690715',
                                                                                                'out_ucast_pkts': '366861213',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '184380',
                                                                       'out_octets': '106130690715',
                                                                       'out_ucast_pkts': '366861213',
                                                                       'poller_hash': 'QURUTjIzMTQ0OEJBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440092',
                                                                                                                        'fiber_length': 14745,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEJBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhCQQ==',
                                         'serial_number': 'ADTN231448BA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440092'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '184524',
                                                                       'in_octets': '760563575203',
                                                                       'in_ucast_pkts': '1084810059',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6045',
                                                                       'out_octets': '2715739030109',
                                                                       'out_ucast_pkts': '2051082644',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6045',
                                                                       'in_octets': '2715739030109',
                                                                       'in_ucast_pkts': '2051082644',
                                                                       'links': [       {       'in_nucast_pkts': '6045',
                                                                                                'in_octets': '2715739030109',
                                                                                                'in_ucast_pkts': '2051082644',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '184524',
                                                                                                'out_octets': '760563575203',
                                                                                                'out_ucast_pkts': '1084810059',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '184524',
                                                                       'out_octets': '760563575203',
                                                                       'out_ucast_pkts': '1084810059',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODlEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440093',
                                                                                                                        'fiber_length': 14466,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODlEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDg5RA==',
                                         'serial_number': 'ADTN2314489D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440093'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17359',
                                                                       'in_octets': '15589904464',
                                                                       'in_ucast_pkts': '36724183',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '214990073767',
                                                                       'out_ucast_pkts': '150125402',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '214990073767',
                                                                       'in_ucast_pkts': '150125402',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '214990073767',
                                                                                                'in_ucast_pkts': '150125402',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17359',
                                                                                                'out_octets': '15589904464',
                                                                                                'out_ucast_pkts': '36724183',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17359',
                                                                       'out_octets': '15589904464',
                                                                       'out_ucast_pkts': '36724183',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODNCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511506',
                                                                                                                        'fiber_length': 2182,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODNCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDgzQg==',
                                         'serial_number': 'ADTN2314483B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511506'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181594',
                                                                       'in_octets': '79498161718',
                                                                       'in_ucast_pkts': '544856175',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6046',
                                                                       'out_octets': '2679365284691',
                                                                       'out_ucast_pkts': '1839981061',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6046',
                                                                       'in_octets': '2679365284691',
                                                                       'in_ucast_pkts': '1839981061',
                                                                       'links': [       {       'in_nucast_pkts': '6046',
                                                                                                'in_octets': '2679365284691',
                                                                                                'in_ucast_pkts': '1839981061',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181594',
                                                                                                'out_octets': '79498161718',
                                                                                                'out_ucast_pkts': '544856175',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181594',
                                                                       'out_octets': '79498161718',
                                                                       'out_ucast_pkts': '544856175',
                                                                       'poller_hash': 'QURUTjIzMTQ0Nzk2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440100',
                                                                                                                        'fiber_length': 9375,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0Nzk2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDc5Ng==',
                                         'serial_number': 'ADTN23144796',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440100'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '184391',
                                                                       'in_octets': '120634866501',
                                                                       'in_ucast_pkts': '753224717',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15614',
                                                                       'out_octets': '1762704597089',
                                                                       'out_ucast_pkts': '1355324479',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VOV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15614',
                                                                       'in_octets': '1762704597089',
                                                                       'in_ucast_pkts': '1355324479',
                                                                       'links': [       {       'in_nucast_pkts': '15614',
                                                                                                'in_octets': '1762704597089',
                                                                                                'in_ucast_pkts': '1355324479',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '184391',
                                                                                                'out_octets': '120634866501',
                                                                                                'out_ucast_pkts': '753224717',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '184391',
                                                                       'out_octets': '120634866501',
                                                                       'out_ucast_pkts': '753224717',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODgzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440093',
                                                                                                                        'fiber_length': 14826,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODgzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDg4Mw==',
                                         'serial_number': 'ADTN23144883',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440093'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-4.1.18.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x4-ONU18_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMThfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0OEM5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEM5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x4-ONU18',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhDOQ==',
                                         'serial_number': 'ADTN231448C9',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '184406',
                                                                       'in_octets': '29353683826',
                                                                       'in_ucast_pkts': '228444142',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15623',
                                                                       'out_octets': '706729102780',
                                                                       'out_ucast_pkts': '500738346',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15623',
                                                                       'in_octets': '706729102780',
                                                                       'in_ucast_pkts': '500738346',
                                                                       'links': [       {       'in_nucast_pkts': '15623',
                                                                                                'in_octets': '706729102780',
                                                                                                'in_ucast_pkts': '500738346',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '184406',
                                                                                                'out_octets': '29353683826',
                                                                                                'out_ucast_pkts': '228444142',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '184406',
                                                                       'out_octets': '29353683826',
                                                                       'out_ucast_pkts': '228444142',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzA3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440095',
                                                                                                                        'fiber_length': 14521,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzA3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDcwNw==',
                                         'serial_number': 'ADTN23144707',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440095'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-12.1.12.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x12-ONU12_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTEyX0VUSF8x'},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0NkJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x12-ONU12',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZCQw==',
                                         'serial_number': 'ADTN231446BC',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17346',
                                                                       'in_octets': '2305401881',
                                                                       'in_ucast_pkts': '8993831',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '26549022939',
                                                                       'out_ucast_pkts': '21176507',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI2X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '26549022939',
                                                                       'in_ucast_pkts': '21176507',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '26549022939',
                                                                                                'in_ucast_pkts': '21176507',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17346',
                                                                                                'out_octets': '2305401881',
                                                                                                'out_ucast_pkts': '8993831',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17346',
                                                                       'out_octets': '2305401881',
                                                                       'out_ucast_pkts': '8993831',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzdBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511499',
                                                                                                                        'fiber_length': 3993,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzdBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDc3QQ==',
                                         'serial_number': 'ADTN2314477A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '184449',
                                                                       'in_octets': '40055826188',
                                                                       'in_ucast_pkts': '113244758',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15617',
                                                                       'out_octets': '301307270087',
                                                                       'out_ucast_pkts': '261636539',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15617',
                                                                       'in_octets': '301307270087',
                                                                       'in_ucast_pkts': '261636539',
                                                                       'links': [       {       'in_nucast_pkts': '15617',
                                                                                                'in_octets': '301307270087',
                                                                                                'in_ucast_pkts': '261636539',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '184449',
                                                                                                'out_octets': '40055826188',
                                                                                                'out_ucast_pkts': '113244758',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '184449',
                                                                       'out_octets': '40055826188',
                                                                       'out_ucast_pkts': '113244758',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzAyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440096',
                                                                                                                        'fiber_length': 14662,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzAyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDcwMg==',
                                         'serial_number': 'ADTN23144702',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440096'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181365',
                                                                       'in_octets': '59332714181',
                                                                       'in_ucast_pkts': '103988355',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6046',
                                                                       'out_octets': '288012885834',
                                                                       'out_ucast_pkts': '237523808',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6046',
                                                                       'in_octets': '288012885834',
                                                                       'in_ucast_pkts': '237523808',
                                                                       'links': [       {       'in_nucast_pkts': '6046',
                                                                                                'in_octets': '288012885834',
                                                                                                'in_ucast_pkts': '237523808',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181365',
                                                                                                'out_octets': '59332714181',
                                                                                                'out_ucast_pkts': '103988355',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181365',
                                                                       'out_octets': '59332714181',
                                                                       'out_ucast_pkts': '103988355',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0JEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440094',
                                                                                                                        'fiber_length': 13218,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0JEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdCRA==',
                                         'serial_number': 'ADTN231447BD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440094'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14078',
                                                                       'in_octets': '19069866624',
                                                                       'in_ucast_pkts': '46572371',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '267524211604',
                                                                       'out_ucast_pkts': '205567921',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTEzX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '267524211604',
                                                                       'in_ucast_pkts': '205567921',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '267524211604',
                                                                                                'in_ucast_pkts': '205567921',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14078',
                                                                                                'out_octets': '19069866624',
                                                                                                'out_ucast_pkts': '46572371',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14078',
                                                                       'out_octets': '19069866624',
                                                                       'out_ucast_pkts': '46572371',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkI4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411498',
                                                                                                                        'fiber_length': 9457,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkI4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZCOA==',
                                         'serial_number': 'ADTN231446B8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '116924',
                                                                       'in_octets': '90453348503',
                                                                       'in_ucast_pkts': '160959414',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9897',
                                                                       'out_octets': '416534696335',
                                                                       'out_ucast_pkts': '325483464',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9897',
                                                                       'in_octets': '416534696335',
                                                                       'in_ucast_pkts': '325483464',
                                                                       'links': [       {       'in_nucast_pkts': '9897',
                                                                                                'in_octets': '416534696335',
                                                                                                'in_ucast_pkts': '325483464',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '116924',
                                                                                                'out_octets': '90453348503',
                                                                                                'out_ucast_pkts': '160959414',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '116924',
                                                                       'out_octets': '90453348503',
                                                                       'out_ucast_pkts': '160959414',
                                                                       'poller_hash': 'QURUTjIzMTQ0OEQ1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3449059',
                                                                                                                        'fiber_length': 5964,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEQ1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhENQ==',
                                         'serial_number': 'ADTN231448D5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3449059'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '92656',
                                                                       'in_octets': '1243896716',
                                                                       'in_ucast_pkts': '1237098',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '451',
                                                                       'out_octets': '1213350778',
                                                                       'out_ucast_pkts': '1231323',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '451',
                                                                       'in_octets': '1213350778',
                                                                       'in_ucast_pkts': '1231323',
                                                                       'links': [       {       'in_nucast_pkts': '451',
                                                                                                'in_octets': '1213350778',
                                                                                                'in_ucast_pkts': '1231323',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '92656',
                                                                                                'out_octets': '1243896716',
                                                                                                'out_ucast_pkts': '1237098',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '92656',
                                                                       'out_octets': '1243896716',
                                                                       'out_ucast_pkts': '1237098',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzQ4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 3353,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzQ4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDc0OA==',
                                         'serial_number': 'ADTN23144748',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17781',
                                                                       'in_octets': '1786217507',
                                                                       'in_ucast_pkts': '3854069',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x14-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '28407662258',
                                                                       'out_ucast_pkts': '19842664',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDE0LU9OVTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '28407662258',
                                                                       'in_ucast_pkts': '19842664',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '28407662258',
                                                                                                'in_ucast_pkts': '19842664',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17781',
                                                                                                'out_octets': '1786217507',
                                                                                                'out_ucast_pkts': '3854069',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17781',
                                                                       'out_octets': '1786217507',
                                                                       'out_ucast_pkts': '3854069',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511505',
                                                                                                                        'fiber_length': 780,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x14-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDcxNA==',
                                         'serial_number': 'ADTN23144714',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511505'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '184398',
                                                                       'in_octets': '152252613088',
                                                                       'in_ucast_pkts': '614225179',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15623',
                                                                       'out_octets': '2157383541720',
                                                                       'out_ucast_pkts': '1604677636',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE0X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15623',
                                                                       'in_octets': '2157383541720',
                                                                       'in_ucast_pkts': '1604677636',
                                                                       'links': [       {       'in_nucast_pkts': '15623',
                                                                                                'in_octets': '2157383541720',
                                                                                                'in_ucast_pkts': '1604677636',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '184398',
                                                                                                'out_octets': '152252613088',
                                                                                                'out_ucast_pkts': '614225179',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '184398',
                                                                       'out_octets': '152252613088',
                                                                       'out_ucast_pkts': '614225179',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkNCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440093',
                                                                                                                        'fiber_length': 8828,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZDQg==',
                                         'serial_number': 'ADTN231446CB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440093'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-6.1.15.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x6-ONU15_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTVfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0N0UzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0UzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x6-ONU15',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdFMw==',
                                         'serial_number': 'ADTN231447E3',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.27.0.eth.1.phy',
                                                                       'in_nucast_pkts': '33987',
                                                                       'in_octets': '35230512504',
                                                                       'in_ucast_pkts': '35656386',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU27_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2877',
                                                                       'out_octets': '15607170836',
                                                                       'out_ucast_pkts': '18811292',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI3X0VUSF8x',
                                                                       'speed': '100000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2877',
                                                                       'in_octets': '15607170836',
                                                                       'in_ucast_pkts': '18811292',
                                                                       'links': [       {       'in_nucast_pkts': '2877',
                                                                                                'in_octets': '15607170836',
                                                                                                'in_ucast_pkts': '18811292',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '33987',
                                                                                                'out_octets': '35230512504',
                                                                                                'out_ucast_pkts': '35656386',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '33987',
                                                                       'out_octets': '35230512504',
                                                                       'out_ucast_pkts': '35656386',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODBEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1002979',
                                                                                                                        'fiber_length': 4007,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODBEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU27',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDgwRA==',
                                         'serial_number': 'ADTN2314480D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1002979'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181358',
                                                                       'in_octets': '144437797488',
                                                                       'in_ucast_pkts': '1022860980',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6045',
                                                                       'out_octets': '4872132831389',
                                                                       'out_ucast_pkts': '3478110433',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE1X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6045',
                                                                       'in_octets': '4872132831389',
                                                                       'in_ucast_pkts': '3478110433',
                                                                       'links': [       {       'in_nucast_pkts': '6045',
                                                                                                'in_octets': '4872132831389',
                                                                                                'in_ucast_pkts': '3478110433',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181358',
                                                                                                'out_octets': '144437797488',
                                                                                                'out_ucast_pkts': '1022860980',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181358',
                                                                       'out_octets': '144437797488',
                                                                       'out_ucast_pkts': '1022860980',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODYxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440092',
                                                                                                                        'fiber_length': 8370,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODYxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDg2MQ==',
                                         'serial_number': 'ADTN23144861',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440092'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17342',
                                                                       'in_octets': '12752149376',
                                                                       'in_ucast_pkts': '123564297',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '378037658066',
                                                                       'out_ucast_pkts': '258052921',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '378037658066',
                                                                       'in_ucast_pkts': '258052921',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '378037658066',
                                                                                                'in_ucast_pkts': '258052921',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17342',
                                                                                                'out_octets': '12752149376',
                                                                                                'out_ucast_pkts': '123564297',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17342',
                                                                       'out_octets': '12752149376',
                                                                       'out_ucast_pkts': '123564297',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzczIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511494',
                                                                                                                        'fiber_length': 233,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzczIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDc3Mw==',
                                         'serial_number': 'ADTN23144773',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511494'},
                                 {       'interfaces': [       {       'admin_up': False,
                                                                       'id': 'onu-subscr-if-2.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 0,
                                                                       'name': 'IndianBayou-x2-ONU14_ETH_1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTRfRVRIXzE=',
                                                                       'type': 6},
                                                               {       'admin_up': False,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': False,
                                                                                                'out_nucast_pkts': '0',
                                                                                                'out_octets': '0',
                                                                                                'out_ucast_pkts': '0',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkNDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440065',
                                                                                                                        'fiber_length': 12154,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZDQw==',
                                         'serial_number': 'ADTN231446CC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440065'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17495',
                                                                       'in_octets': '11301586522',
                                                                       'in_ucast_pkts': '47241964',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '159092114439',
                                                                       'out_ucast_pkts': '116924650',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMThfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '159092114439',
                                                                       'in_ucast_pkts': '116924650',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '159092114439',
                                                                                                'in_ucast_pkts': '116924650',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17495',
                                                                                                'out_octets': '11301586522',
                                                                                                'out_ucast_pkts': '47241964',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17495',
                                                                       'out_octets': '11301586522',
                                                                       'out_ucast_pkts': '47241964',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511502',
                                                                                                                        'fiber_length': 1780,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZDMA==',
                                         'serial_number': 'ADTN231446C0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511502'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '184400',
                                                                       'in_octets': '40388633921',
                                                                       'in_ucast_pkts': '169099413',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15615',
                                                                       'out_octets': '901890785599',
                                                                       'out_ucast_pkts': '640995591',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15615',
                                                                       'in_octets': '901890785599',
                                                                       'in_ucast_pkts': '640995591',
                                                                       'links': [       {       'in_nucast_pkts': '15615',
                                                                                                'in_octets': '901890785599',
                                                                                                'in_ucast_pkts': '640995591',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '184400',
                                                                                                'out_octets': '40388633921',
                                                                                                'out_ucast_pkts': '169099413',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '184400',
                                                                       'out_octets': '40388633921',
                                                                       'out_ucast_pkts': '169099413',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0FFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440096',
                                                                                                                        'fiber_length': 13016,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdBRQ==',
                                         'serial_number': 'ADTN231447AE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440096'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '52015',
                                                                       'in_octets': '1710802939',
                                                                       'in_ucast_pkts': '2856140',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4401',
                                                                       'out_octets': '3567786663',
                                                                       'out_ucast_pkts': '3617595',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4401',
                                                                       'in_octets': '3567786663',
                                                                       'in_ucast_pkts': '3617595',
                                                                       'links': [       {       'in_nucast_pkts': '4401',
                                                                                                'in_octets': '3567786663',
                                                                                                'in_ucast_pkts': '3617595',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '52015',
                                                                                                'out_octets': '1710802939',
                                                                                                'out_ucast_pkts': '2856140',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '52015',
                                                                       'out_octets': '1710802939',
                                                                       'out_ucast_pkts': '2856140',
                                                                       'poller_hash': 'QURUTjIzMTQ0OEZFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1534721',
                                                                                                                        'fiber_length': 14823,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEZFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhGRQ==',
                                         'serial_number': 'ADTN231448FE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1534721'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '34680',
                                                                       'in_octets': '10840280335',
                                                                       'in_ucast_pkts': '42618547',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2932',
                                                                       'out_octets': '137901116493',
                                                                       'out_ucast_pkts': '106451556',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2932',
                                                                       'in_octets': '137901116493',
                                                                       'in_ucast_pkts': '106451556',
                                                                       'links': [       {       'in_nucast_pkts': '2932',
                                                                                                'in_octets': '137901116493',
                                                                                                'in_ucast_pkts': '106451556',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '34680',
                                                                                                'out_octets': '10840280335',
                                                                                                'out_ucast_pkts': '42618547',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '34680',
                                                                       'out_octets': '10840280335',
                                                                       'out_ucast_pkts': '42618547',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkE1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1022696',
                                                                                                                        'fiber_length': 14178,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkE1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZBNQ==',
                                         'serial_number': 'ADTN231446A5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1022696'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17065',
                                                                       'in_octets': '9987180858',
                                                                       'in_ucast_pkts': '48716005',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '162591640679',
                                                                       'out_ucast_pkts': '118790606',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '162591640679',
                                                                       'in_ucast_pkts': '118790606',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '162591640679',
                                                                                                'in_ucast_pkts': '118790606',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17065',
                                                                                                'out_octets': '9987180858',
                                                                                                'out_ucast_pkts': '48716005',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17065',
                                                                       'out_octets': '9987180858',
                                                                       'out_ucast_pkts': '48716005',
                                                                       'poller_hash': 'QURUTjIzMTQ0OTE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511499',
                                                                                                                        'fiber_length': 2012,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OTE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDkxNg==',
                                         'serial_number': 'ADTN23144916',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '70409',
                                                                       'in_octets': '31887830912',
                                                                       'in_ucast_pkts': '152296330',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5871',
                                                                       'out_octets': '756680875145',
                                                                       'out_ucast_pkts': '540611654',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5871',
                                                                       'in_octets': '756680875145',
                                                                       'in_ucast_pkts': '540611654',
                                                                       'links': [       {       'in_nucast_pkts': '5871',
                                                                                                'in_octets': '756680875145',
                                                                                                'in_ucast_pkts': '540611654',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '70409',
                                                                                                'out_octets': '31887830912',
                                                                                                'out_ucast_pkts': '152296330',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '70409',
                                                                       'out_octets': '31887830912',
                                                                       'out_ucast_pkts': '152296330',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODZDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2045293',
                                                                                                                        'fiber_length': 10744,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODZDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDg2Qw==',
                                         'serial_number': 'ADTN2314486C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2045293'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '43675',
                                                                       'in_octets': '35810310413',
                                                                       'in_ucast_pkts': '103057074',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1455',
                                                                       'out_octets': '440061119703',
                                                                       'out_ucast_pkts': '316118515',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1455',
                                                                       'in_octets': '440061119703',
                                                                       'in_ucast_pkts': '316118515',
                                                                       'links': [       {       'in_nucast_pkts': '1455',
                                                                                                'in_octets': '440061119703',
                                                                                                'in_ucast_pkts': '316118515',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '43675',
                                                                                                'out_octets': '35810310413',
                                                                                                'out_ucast_pkts': '103057074',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '43675',
                                                                       'out_octets': '35810310413',
                                                                       'out_ucast_pkts': '103057074',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0RBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1309470',
                                                                                                                        'fiber_length': 3596,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0RBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdEQQ==',
                                         'serial_number': 'ADTN231447DA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1309470'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-9.1.22.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x9-ONU22_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjJfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0OEI0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEI0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x9-ONU22',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhCNA==',
                                         'serial_number': 'ADTN231448B4',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-6.1.17.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x6-ONU17_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTdfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0NjhDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NjhDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x6-ONU17',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDY4Qw==',
                                         'serial_number': 'ADTN2314468C',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.28.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17347',
                                                                       'in_octets': '18278042675',
                                                                       'in_ucast_pkts': '39931214',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU28_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '187321586366',
                                                                       'out_ucast_pkts': '136018147',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI4X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '187321586366',
                                                                       'in_ucast_pkts': '136018147',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '187321586366',
                                                                                                'in_ucast_pkts': '136018147',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17347',
                                                                                                'out_octets': '18278042675',
                                                                                                'out_ucast_pkts': '39931214',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17347',
                                                                       'out_octets': '18278042675',
                                                                       'out_ucast_pkts': '39931214',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0FCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511498',
                                                                                                                        'fiber_length': 4238,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU28',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdBQg==',
                                         'serial_number': 'ADTN231447AB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '165877',
                                                                       'in_octets': '94609140575',
                                                                       'in_ucast_pkts': '332441703',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13938',
                                                                       'out_octets': '1245110213602',
                                                                       'out_ucast_pkts': '928888614',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '13938',
                                                                       'in_octets': '1245110213602',
                                                                       'in_ucast_pkts': '928888614',
                                                                       'links': [       {       'in_nucast_pkts': '13938',
                                                                                                'in_octets': '1245110213602',
                                                                                                'in_ucast_pkts': '928888614',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '165877',
                                                                                                'out_octets': '94609140575',
                                                                                                'out_ucast_pkts': '332441703',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '165877',
                                                                       'out_octets': '94609140575',
                                                                       'out_ucast_pkts': '332441703',
                                                                       'poller_hash': 'QURUTjIzMTI1MTRBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4857791',
                                                                                                                        'fiber_length': 14481,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTRBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0QQ==',
                                         'serial_number': 'ADTN2312514A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '4857791'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.25.0.eth.1.phy',
                                                                       'in_nucast_pkts': '640',
                                                                       'in_octets': '20128057580',
                                                                       'in_ucast_pkts': '58996755',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU25_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3021',
                                                                       'out_octets': '223867537482',
                                                                       'out_ucast_pkts': '167630986',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3021',
                                                                       'in_octets': '223867537482',
                                                                       'in_ucast_pkts': '167630986',
                                                                       'links': [       {       'in_nucast_pkts': '3021',
                                                                                                'in_octets': '223867537482',
                                                                                                'in_ucast_pkts': '167630986',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '640',
                                                                                                'out_octets': '20128057580',
                                                                                                'out_ucast_pkts': '58996755',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '640',
                                                                       'out_octets': '20128057580',
                                                                       'out_ucast_pkts': '58996755',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzM1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1053537',
                                                                                                                        'fiber_length': 4136,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzM1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU25',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDczNQ==',
                                         'serial_number': 'ADTN23144735',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1053537'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17378',
                                                                       'in_octets': '53122213483',
                                                                       'in_ucast_pkts': '167810645',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1469',
                                                                       'out_octets': '420548488076',
                                                                       'out_ucast_pkts': '318233832',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE1X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1469',
                                                                       'in_octets': '420548488076',
                                                                       'in_ucast_pkts': '318233832',
                                                                       'links': [       {       'in_nucast_pkts': '1469',
                                                                                                'in_octets': '420548488076',
                                                                                                'in_ucast_pkts': '318233832',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17378',
                                                                                                'out_octets': '53122213483',
                                                                                                'out_ucast_pkts': '167810645',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17378',
                                                                       'out_octets': '53122213483',
                                                                       'out_ucast_pkts': '167810645',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkNFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511501',
                                                                                                                        'fiber_length': 7294,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZDRQ==',
                                         'serial_number': 'ADTN231446CE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511501'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17077',
                                                                       'in_octets': '5288053961',
                                                                       'in_ucast_pkts': '18085701',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '63945371536',
                                                                       'out_ucast_pkts': '50263467',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '63945371536',
                                                                       'in_ucast_pkts': '50263467',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '63945371536',
                                                                                                'in_ucast_pkts': '50263467',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17077',
                                                                                                'out_octets': '5288053961',
                                                                                                'out_ucast_pkts': '18085701',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17077',
                                                                       'out_octets': '5288053961',
                                                                       'out_ucast_pkts': '18085701',
                                                                       'poller_hash': 'QURUTjIzMTQ0OEQxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511504',
                                                                                                                        'fiber_length': 2661,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEQxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhEMQ==',
                                         'serial_number': 'ADTN231448D1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511504'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.29.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17676',
                                                                       'in_octets': '174845768581',
                                                                       'in_ucast_pkts': '251770340',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU29_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '606185853600',
                                                                       'out_ucast_pkts': '484055879',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI5X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '606185853600',
                                                                       'in_ucast_pkts': '484055879',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '606185853600',
                                                                                                'in_ucast_pkts': '484055879',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17676',
                                                                                                'out_octets': '174845768581',
                                                                                                'out_ucast_pkts': '251770340',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17676',
                                                                       'out_octets': '174845768581',
                                                                       'out_ucast_pkts': '251770340',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0YzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511504',
                                                                                                                        'fiber_length': 4128,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0YzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU29',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdGMw==',
                                         'serial_number': 'ADTN231447F3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511504'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '186260',
                                                                       'in_octets': '32708626057',
                                                                       'in_ucast_pkts': '75413117',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15613',
                                                                       'out_octets': '151746807797',
                                                                       'out_ucast_pkts': '132901226',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15613',
                                                                       'in_octets': '151746807797',
                                                                       'in_ucast_pkts': '132901226',
                                                                       'links': [       {       'in_nucast_pkts': '15613',
                                                                                                'in_octets': '151746807797',
                                                                                                'in_ucast_pkts': '132901226',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '186260',
                                                                                                'out_octets': '32708626057',
                                                                                                'out_ucast_pkts': '75413117',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '186260',
                                                                       'out_octets': '32708626057',
                                                                       'out_ucast_pkts': '75413117',
                                                                       'poller_hash': 'QURUTjIzMTQ0Njk5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440101',
                                                                                                                        'fiber_length': 8892,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0Njk5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDY5OQ==',
                                         'serial_number': 'ADTN23144699',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440101'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.31.0.eth.1.phy',
                                                                       'in_nucast_pkts': '314',
                                                                       'in_octets': '9818933607',
                                                                       'in_ucast_pkts': '26316509',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU31_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '108363902187',
                                                                       'out_ucast_pkts': '77185946',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMzFfRVRIXzE=',
                                                                       'speed': '100000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '108363902187',
                                                                       'in_ucast_pkts': '77185946',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '108363902187',
                                                                                                'in_ucast_pkts': '77185946',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '314',
                                                                                                'out_octets': '9818933607',
                                                                                                'out_ucast_pkts': '26316509',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '314',
                                                                       'out_octets': '9818933607',
                                                                       'out_ucast_pkts': '26316509',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0E0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511505',
                                                                                                                        'fiber_length': 5397,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0E0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU31',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdBNA==',
                                         'serial_number': 'ADTN231447A4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511505'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6380',
                                                                       'in_octets': '10392725188',
                                                                       'in_ucast_pkts': '13276517',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '214',
                                                                       'out_octets': '42341526171',
                                                                       'out_ucast_pkts': '33214746',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '214',
                                                                       'in_octets': '42341526171',
                                                                       'in_ucast_pkts': '33214746',
                                                                       'links': [       {       'in_nucast_pkts': '214',
                                                                                                'in_octets': '42341526171',
                                                                                                'in_ucast_pkts': '33214746',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6380',
                                                                                                'out_octets': '10392725188',
                                                                                                'out_ucast_pkts': '13276517',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6380',
                                                                       'out_octets': '10392725188',
                                                                       'out_ucast_pkts': '13276517',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzU5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '190728',
                                                                                                                        'fiber_length': 6744,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzU5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDc1OQ==',
                                         'serial_number': 'ADTN23144759',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '190728'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17358',
                                                                       'in_octets': '5720460979',
                                                                       'in_ucast_pkts': '54734286',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '170594327927',
                                                                       'out_ucast_pkts': '116816421',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '170594327927',
                                                                       'in_ucast_pkts': '116816421',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '170594327927',
                                                                                                'in_ucast_pkts': '116816421',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17358',
                                                                                                'out_octets': '5720460979',
                                                                                                'out_ucast_pkts': '54734286',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17358',
                                                                       'out_octets': '5720460979',
                                                                       'out_ucast_pkts': '54734286',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTlDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 3282,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTlDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE5Qw==',
                                         'serial_number': 'ADTN23144A9C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17771',
                                                                       'in_octets': '3369267906',
                                                                       'in_ucast_pkts': '19106450',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '93231884104',
                                                                       'out_ucast_pkts': '66152081',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '93231884104',
                                                                       'in_ucast_pkts': '66152081',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '93231884104',
                                                                                                'in_ucast_pkts': '66152081',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17771',
                                                                                                'out_octets': '3369267906',
                                                                                                'out_ucast_pkts': '19106450',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17771',
                                                                       'out_octets': '3369267906',
                                                                       'out_ucast_pkts': '19106450',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjNBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511502',
                                                                                                                        'fiber_length': 545,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjNBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIzQQ==',
                                         'serial_number': 'ADTN23144B3A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511502'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77257',
                                                                       'in_octets': '37842338820',
                                                                       'in_ucast_pkts': '103822789',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2573',
                                                                       'out_octets': '392270827975',
                                                                       'out_ucast_pkts': '296845783',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2573',
                                                                       'in_octets': '392270827975',
                                                                       'in_ucast_pkts': '296845783',
                                                                       'links': [       {       'in_nucast_pkts': '2573',
                                                                                                'in_octets': '392270827975',
                                                                                                'in_ucast_pkts': '296845783',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77257',
                                                                                                'out_octets': '37842338820',
                                                                                                'out_ucast_pkts': '103822789',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77257',
                                                                       'out_octets': '37842338820',
                                                                       'out_ucast_pkts': '103822789',
                                                                       'poller_hash': 'QURUTjIzMTQ0QUIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317047',
                                                                                                                        'fiber_length': 4574,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QUIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEFCMA==',
                                         'serial_number': 'ADTN23144AB0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317047'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11978',
                                                                       'in_octets': '3250210157',
                                                                       'in_ucast_pkts': '24466505',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1013',
                                                                       'out_octets': '75017766361',
                                                                       'out_ucast_pkts': '51821106',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1013',
                                                                       'in_octets': '75017766361',
                                                                       'in_ucast_pkts': '51821106',
                                                                       'links': [       {       'in_nucast_pkts': '1013',
                                                                                                'in_octets': '75017766361',
                                                                                                'in_ucast_pkts': '51821106',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11978',
                                                                                                'out_octets': '3250210157',
                                                                                                'out_ucast_pkts': '24466505',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11978',
                                                                       'out_octets': '3250210157',
                                                                       'out_ucast_pkts': '24466505',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '352937',
                                                                                                                        'fiber_length': 4011,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIyQw==',
                                         'serial_number': 'ADTN23144B2C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '352937'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '32089',
                                                                       'in_octets': '11857620441',
                                                                       'in_ucast_pkts': '43266679',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2715',
                                                                       'out_octets': '364213727415',
                                                                       'out_ucast_pkts': '253032203',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE2X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2715',
                                                                       'in_octets': '364213727415',
                                                                       'in_ucast_pkts': '253032203',
                                                                       'links': [       {       'in_nucast_pkts': '2715',
                                                                                                'in_octets': '364213727415',
                                                                                                'in_ucast_pkts': '253032203',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '32089',
                                                                                                'out_octets': '11857620441',
                                                                                                'out_ucast_pkts': '43266679',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '32089',
                                                                       'out_octets': '11857620441',
                                                                       'out_ucast_pkts': '43266679',
                                                                       'poller_hash': 'QURUTjIzMTQ0RERGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '946257',
                                                                                                                        'fiber_length': 8094,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RERGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NERERg==',
                                         'serial_number': 'ADTN23144DDF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '946257'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17772',
                                                                       'in_octets': '17672935333',
                                                                       'in_ucast_pkts': '33715104',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '74712486667',
                                                                       'out_ucast_pkts': '61735837',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE3X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '74712486667',
                                                                       'in_ucast_pkts': '61735837',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '74712486667',
                                                                                                'in_ucast_pkts': '61735837',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17772',
                                                                                                'out_octets': '17672935333',
                                                                                                'out_ucast_pkts': '33715104',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17772',
                                                                       'out_octets': '17672935333',
                                                                       'out_ucast_pkts': '33715104',
                                                                       'poller_hash': 'QURUTjIzMTQ0RDUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511501',
                                                                                                                        'fiber_length': 7706,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RDUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEQ1MA==',
                                         'serial_number': 'ADTN23144D50',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511501'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14079',
                                                                       'in_octets': '3578998606',
                                                                       'in_ucast_pkts': '25157851',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '92781727659',
                                                                       'out_ucast_pkts': '65792280',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTdfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '92781727659',
                                                                       'in_ucast_pkts': '65792280',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '92781727659',
                                                                                                'in_ucast_pkts': '65792280',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14079',
                                                                                                'out_octets': '3578998606',
                                                                                                'out_ucast_pkts': '25157851',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14079',
                                                                       'out_octets': '3578998606',
                                                                       'out_ucast_pkts': '25157851',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTE1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411495',
                                                                                                                        'fiber_length': 7986,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTE1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEExNQ==',
                                         'serial_number': 'ADTN23144A15',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411495'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '56138',
                                                                       'in_octets': '70658652100',
                                                                       'in_ucast_pkts': '260887099',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4750',
                                                                       'out_octets': '1013212977198',
                                                                       'out_ucast_pkts': '735130401',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTlfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4750',
                                                                       'in_octets': '1013212977198',
                                                                       'in_ucast_pkts': '735130401',
                                                                       'links': [       {       'in_nucast_pkts': '4750',
                                                                                                'in_octets': '1013212977198',
                                                                                                'in_ucast_pkts': '735130401',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '56138',
                                                                                                'out_octets': '70658652100',
                                                                                                'out_ucast_pkts': '260887099',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '56138',
                                                                       'out_octets': '70658652100',
                                                                       'out_ucast_pkts': '260887099',
                                                                       'poller_hash': 'QURUTjIzMTQ0Q0MzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1656089',
                                                                                                                        'fiber_length': 10521,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0MzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzE0NENDMw==',
                                         'serial_number': 'ADTN23144CC3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1656089'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17349',
                                                                       'in_octets': '505527167',
                                                                       'in_ucast_pkts': '748493',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x14-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '1389952277',
                                                                       'out_ucast_pkts': '1338405',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDE0LU9OVTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '1389952277',
                                                                       'in_ucast_pkts': '1338405',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '1389952277',
                                                                                                'in_ucast_pkts': '1338405',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17349',
                                                                                                'out_octets': '505527167',
                                                                                                'out_ucast_pkts': '748493',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17349',
                                                                       'out_octets': '505527167',
                                                                       'out_ucast_pkts': '748493',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzMzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511505',
                                                                                                                        'fiber_length': 539,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzMzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x14-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEMzMw==',
                                         'serial_number': 'ADTN23144C33',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511505'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13741',
                                                                       'in_octets': '14008554476',
                                                                       'in_ucast_pkts': '40711560',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '109204659528',
                                                                       'out_ucast_pkts': '84841729',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '109204659528',
                                                                       'in_ucast_pkts': '84841729',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '109204659528',
                                                                                                'in_ucast_pkts': '84841729',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13741',
                                                                                                'out_octets': '14008554476',
                                                                                                'out_ucast_pkts': '40711560',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13741',
                                                                       'out_octets': '14008554476',
                                                                       'out_ucast_pkts': '40711560',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTVBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411498',
                                                                                                                        'fiber_length': 9303,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTVBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE1QQ==',
                                         'serial_number': 'ADTN23144A5A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17345',
                                                                       'in_octets': '5263934024',
                                                                       'in_ucast_pkts': '12460366',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '31009469267',
                                                                       'out_ucast_pkts': '25092088',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '31009469267',
                                                                       'in_ucast_pkts': '25092088',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '31009469267',
                                                                                                'in_ucast_pkts': '25092088',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17345',
                                                                                                'out_octets': '5263934024',
                                                                                                'out_ucast_pkts': '12460366',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17345',
                                                                       'out_octets': '5263934024',
                                                                       'out_ucast_pkts': '12460366',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzEwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511498',
                                                                                                                        'fiber_length': 1734,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzEwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEMxMA==',
                                         'serial_number': 'ADTN23144C10',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13740',
                                                                       'in_octets': '7157217416',
                                                                       'in_ucast_pkts': '14080599',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '62900310056',
                                                                       'out_ucast_pkts': '46212700',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE2X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '62900310056',
                                                                       'in_ucast_pkts': '46212700',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '62900310056',
                                                                                                'in_ucast_pkts': '46212700',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13740',
                                                                                                'out_octets': '7157217416',
                                                                                                'out_ucast_pkts': '14080599',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13740',
                                                                       'out_octets': '7157217416',
                                                                       'out_ucast_pkts': '14080599',
                                                                       'poller_hash': 'QURUTjIzMTQ0QkY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411498',
                                                                                                                        'fiber_length': 8538,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QkY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEJGNw==',
                                         'serial_number': 'ADTN23144BF7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13433',
                                                                       'in_octets': '7408149367',
                                                                       'in_ucast_pkts': '37311232',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1108',
                                                                       'out_octets': '127573221452',
                                                                       'out_ucast_pkts': '92544293',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTRfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1108',
                                                                       'in_octets': '127573221452',
                                                                       'in_ucast_pkts': '92544293',
                                                                       'links': [       {       'in_nucast_pkts': '1108',
                                                                                                'in_octets': '127573221452',
                                                                                                'in_ucast_pkts': '92544293',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13433',
                                                                                                'out_octets': '7408149367',
                                                                                                'out_ucast_pkts': '37311232',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13433',
                                                                       'out_octets': '7408149367',
                                                                       'out_ucast_pkts': '37311232',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTUzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '386516',
                                                                                                                        'fiber_length': 9292,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTUzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE1Mw==',
                                         'serial_number': 'ADTN23144A53',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '386516'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13963',
                                                                       'in_octets': '8590335792',
                                                                       'in_ucast_pkts': '49772757',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '279308620557',
                                                                       'out_ucast_pkts': '191253374',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '279308620557',
                                                                       'in_ucast_pkts': '191253374',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '279308620557',
                                                                                                'in_ucast_pkts': '191253374',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13963',
                                                                                                'out_octets': '8590335792',
                                                                                                'out_ucast_pkts': '49772757',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13963',
                                                                       'out_octets': '8590335792',
                                                                       'out_ucast_pkts': '49772757',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTg0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411499',
                                                                                                                        'fiber_length': 8903,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTg0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE4NA==',
                                         'serial_number': 'ADTN23144A84',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78633',
                                                                       'in_octets': '41130133183',
                                                                       'in_ucast_pkts': '267578468',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6644',
                                                                       'out_octets': '983360137787',
                                                                       'out_ucast_pkts': '737983039',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6644',
                                                                       'in_octets': '983360137787',
                                                                       'in_ucast_pkts': '737983039',
                                                                       'links': [       {       'in_nucast_pkts': '6644',
                                                                                                'in_octets': '983360137787',
                                                                                                'in_ucast_pkts': '737983039',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78633',
                                                                                                'out_octets': '41130133183',
                                                                                                'out_ucast_pkts': '267578468',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78633',
                                                                       'out_octets': '41130133183',
                                                                       'out_ucast_pkts': '267578468',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTk3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317055',
                                                                                                                        'fiber_length': 6051,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTk3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE5Nw==',
                                         'serial_number': 'ADTN23144A97',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317055'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.32.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17075',
                                                                       'in_octets': '11614398483',
                                                                       'in_ucast_pkts': '26579067',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU32_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '72537821329',
                                                                       'out_ucast_pkts': '60433602',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMzJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '72537821329',
                                                                       'in_ucast_pkts': '60433602',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '72537821329',
                                                                                                'in_ucast_pkts': '60433602',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17075',
                                                                                                'out_octets': '11614398483',
                                                                                                'out_ucast_pkts': '26579067',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17075',
                                                                       'out_octets': '11614398483',
                                                                       'out_ucast_pkts': '26579067',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTY5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511511',
                                                                                                                        'fiber_length': 4383,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTY5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU32',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE2OQ==',
                                         'serial_number': 'ADTN23144A69',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511511'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13738',
                                                                       'in_octets': '4460347522',
                                                                       'in_ucast_pkts': '17508885',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '79367287959',
                                                                       'out_ucast_pkts': '57326821',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '79367287959',
                                                                       'in_ucast_pkts': '57326821',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '79367287959',
                                                                                                'in_ucast_pkts': '57326821',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13738',
                                                                                                'out_octets': '4460347522',
                                                                                                'out_ucast_pkts': '17508885',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13738',
                                                                       'out_octets': '4460347522',
                                                                       'out_ucast_pkts': '17508885',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTVDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411500',
                                                                                                                        'fiber_length': 9318,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTVDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE1Qw==',
                                         'serial_number': 'ADTN23144A5C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17353',
                                                                       'in_octets': '429884396',
                                                                       'in_ucast_pkts': '648423',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '463891147',
                                                                       'out_ucast_pkts': '612318',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '463891147',
                                                                       'in_ucast_pkts': '612318',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '463891147',
                                                                                                'in_ucast_pkts': '612318',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17353',
                                                                                                'out_octets': '429884396',
                                                                                                'out_ucast_pkts': '648423',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17353',
                                                                       'out_octets': '429884396',
                                                                       'out_ucast_pkts': '648423',
                                                                       'poller_hash': 'QURUTjIzMTQ0Q0Q1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 3249,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0Q1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzE0NENENQ==',
                                         'serial_number': 'ADTN23144CD5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-7.1.19.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x7-ONU19_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTlfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'Q1hOSzAwQTQzMDVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'Q1hOSzAwQTQzMDVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x7-ONU19',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IENYTkswMEE0MzA1Qg==',
                                         'serial_number': 'CXNK00A4305B',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17344',
                                                                       'in_octets': '4382109537',
                                                                       'in_ucast_pkts': '9379472',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1465',
                                                                       'out_octets': '26920753660',
                                                                       'out_ucast_pkts': '21845818',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1465',
                                                                       'in_octets': '26920753660',
                                                                       'in_ucast_pkts': '21845818',
                                                                       'links': [       {       'in_nucast_pkts': '1465',
                                                                                                'in_octets': '26920753660',
                                                                                                'in_ucast_pkts': '21845818',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17344',
                                                                                                'out_octets': '4382109537',
                                                                                                'out_ucast_pkts': '9379472',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17344',
                                                                       'out_octets': '4382109537',
                                                                       'out_ucast_pkts': '9379472',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzQ2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511503',
                                                                                                                        'fiber_length': 2665,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzQ2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEM0Ng==',
                                         'serial_number': 'ADTN23144C46',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511503'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17069',
                                                                       'in_octets': '5898394823',
                                                                       'in_ucast_pkts': '22128245',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '92783601530',
                                                                       'out_ucast_pkts': '65414083',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTdfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '92783601530',
                                                                       'in_ucast_pkts': '65414083',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '92783601530',
                                                                                                'in_ucast_pkts': '65414083',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17069',
                                                                                                'out_octets': '5898394823',
                                                                                                'out_ucast_pkts': '22128245',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17069',
                                                                       'out_octets': '5898394823',
                                                                       'out_ucast_pkts': '22128245',
                                                                       'poller_hash': 'QURUTjIzMTQ0RDk5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511505',
                                                                                                                        'fiber_length': 9196,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RDk5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEQ5OQ==',
                                         'serial_number': 'ADTN23144D99',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511505'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13737',
                                                                       'in_octets': '8181387735',
                                                                       'in_ucast_pkts': '17123621',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '105546938363',
                                                                       'out_ucast_pkts': '77495536',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVThfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '105546938363',
                                                                       'in_ucast_pkts': '77495536',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '105546938363',
                                                                                                'in_ucast_pkts': '77495536',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13737',
                                                                                                'out_octets': '8181387735',
                                                                                                'out_ucast_pkts': '17123621',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13737',
                                                                       'out_octets': '8181387735',
                                                                       'out_ucast_pkts': '17123621',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjNGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411497',
                                                                                                                        'fiber_length': 9156,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjNGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIzRg==',
                                         'serial_number': 'ADTN23144B3F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411497'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13740',
                                                                       'in_octets': '14872244755',
                                                                       'in_ucast_pkts': '66466167',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '278553882577',
                                                                       'out_ucast_pkts': '195340506',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTEwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '278553882577',
                                                                       'in_ucast_pkts': '195340506',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '278553882577',
                                                                                                'in_ucast_pkts': '195340506',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13740',
                                                                                                'out_octets': '14872244755',
                                                                                                'out_ucast_pkts': '66466167',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13740',
                                                                       'out_octets': '14872244755',
                                                                       'out_ucast_pkts': '66466167',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjVFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411497',
                                                                                                                        'fiber_length': 9074,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjVFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEI1RQ==',
                                         'serial_number': 'ADTN23144B5E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411497'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13738',
                                                                       'in_octets': '5679942087',
                                                                       'in_ucast_pkts': '27282218',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '160675401445',
                                                                       'out_ucast_pkts': '123185468',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTExX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '160675401445',
                                                                       'in_ucast_pkts': '123185468',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '160675401445',
                                                                                                'in_ucast_pkts': '123185468',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13738',
                                                                                                'out_octets': '5679942087',
                                                                                                'out_ucast_pkts': '27282218',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13738',
                                                                       'out_octets': '5679942087',
                                                                       'out_ucast_pkts': '27282218',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTlCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411499',
                                                                                                                        'fiber_length': 9481,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTlCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE5Qg==',
                                         'serial_number': 'ADTN23144A9B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13771',
                                                                       'in_octets': '9428401483',
                                                                       'in_ucast_pkts': '20053837',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '460',
                                                                       'out_octets': '84235522516',
                                                                       'out_ucast_pkts': '66454311',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTEyX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '460',
                                                                       'in_octets': '84235522516',
                                                                       'in_ucast_pkts': '66454311',
                                                                       'links': [       {       'in_nucast_pkts': '460',
                                                                                                'in_octets': '84235522516',
                                                                                                'in_ucast_pkts': '66454311',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13771',
                                                                                                'out_octets': '9428401483',
                                                                                                'out_ucast_pkts': '20053837',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13771',
                                                                       'out_octets': '9428401483',
                                                                       'out_ucast_pkts': '20053837',
                                                                       'poller_hash': 'QURUTjIzMTQ0QUNGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411497',
                                                                                                                        'fiber_length': 9284,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QUNGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEFDRg==',
                                         'serial_number': 'ADTN23144ACF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411497'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17786',
                                                                       'in_octets': '15221587024',
                                                                       'in_ucast_pkts': '84353268',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1469',
                                                                       'out_octets': '395982070421',
                                                                       'out_ucast_pkts': '292015605',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1469',
                                                                       'in_octets': '395982070421',
                                                                       'in_ucast_pkts': '292015605',
                                                                       'links': [       {       'in_nucast_pkts': '1469',
                                                                                                'in_octets': '395982070421',
                                                                                                'in_ucast_pkts': '292015605',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17786',
                                                                                                'out_octets': '15221587024',
                                                                                                'out_ucast_pkts': '84353268',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17786',
                                                                       'out_octets': '15221587024',
                                                                       'out_ucast_pkts': '84353268',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTJCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511493',
                                                                                                                        'fiber_length': 324,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTJCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU22',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEEyQg==',
                                         'serial_number': 'ADTN23144A2B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511493'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17077',
                                                                       'in_octets': '21087915910',
                                                                       'in_ucast_pkts': '52964384',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '151984788371',
                                                                       'out_ucast_pkts': '113414646',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '151984788371',
                                                                       'in_ucast_pkts': '113414646',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '151984788371',
                                                                                                'in_ucast_pkts': '113414646',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17077',
                                                                                                'out_octets': '21087915910',
                                                                                                'out_ucast_pkts': '52964384',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17077',
                                                                       'out_octets': '21087915910',
                                                                       'out_ucast_pkts': '52964384',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511508',
                                                                                                                        'fiber_length': 3298,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEI0RA==',
                                         'serial_number': 'ADTN23144B4D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511508'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17095',
                                                                       'in_octets': '70748521978',
                                                                       'in_ucast_pkts': '73321371',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '571',
                                                                       'out_octets': '77868860229',
                                                                       'out_ucast_pkts': '64782879',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTEzX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '571',
                                                                       'in_octets': '77868860229',
                                                                       'in_ucast_pkts': '64782879',
                                                                       'links': [       {       'in_nucast_pkts': '571',
                                                                                                'in_octets': '77868860229',
                                                                                                'in_ucast_pkts': '64782879',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17095',
                                                                                                'out_octets': '70748521978',
                                                                                                'out_ucast_pkts': '73321371',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17095',
                                                                       'out_octets': '70748521978',
                                                                       'out_ucast_pkts': '73321371',
                                                                       'poller_hash': 'QURUTjIzMTI1MDM3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511504',
                                                                                                                        'fiber_length': 9249,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDM3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTAzNw==',
                                         'serial_number': 'ADTN23125037',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511504'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77274',
                                                                       'in_octets': '19847184105',
                                                                       'in_ucast_pkts': '60122034',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '187773959886',
                                                                       'out_ucast_pkts': '141393438',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '187773959886',
                                                                       'in_ucast_pkts': '141393438',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '187773959886',
                                                                                                'in_ucast_pkts': '141393438',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77274',
                                                                                                'out_octets': '19847184105',
                                                                                                'out_ucast_pkts': '60122034',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77274',
                                                                       'out_octets': '19847184105',
                                                                       'out_ucast_pkts': '60122034',
                                                                       'poller_hash': 'QURUTjIzMTI0MDk5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317059',
                                                                                                                        'fiber_length': 6279,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MDk5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDA5OQ==',
                                         'serial_number': 'ADTN23124099',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317059'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14078',
                                                                       'in_octets': '1317198415',
                                                                       'in_ucast_pkts': '5401632',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '14367286856',
                                                                       'out_ucast_pkts': '11234680',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE3X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '14367286856',
                                                                       'in_ucast_pkts': '11234680',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '14367286856',
                                                                                                'in_ucast_pkts': '11234680',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14078',
                                                                                                'out_octets': '1317198415',
                                                                                                'out_ucast_pkts': '5401632',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14078',
                                                                       'out_octets': '1317198415',
                                                                       'out_ucast_pkts': '5401632',
                                                                       'poller_hash': 'QURUTjIzMTI0RTVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411499',
                                                                                                                        'fiber_length': 8775,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU1Qg==',
                                         'serial_number': 'ADTN23124E5B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411499'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-7.1.23.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x7-ONU23_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjNfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI1MThEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MThEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x7-ONU23',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE4RA==',
                                         'serial_number': 'ADTN2312518D',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13737',
                                                                       'in_octets': '4536473801',
                                                                       'in_ucast_pkts': '13816992',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '72110420027',
                                                                       'out_ucast_pkts': '51855448',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTE0X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '72110420027',
                                                                       'in_ucast_pkts': '51855448',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '72110420027',
                                                                                                'in_ucast_pkts': '51855448',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13737',
                                                                                                'out_octets': '4536473801',
                                                                                                'out_ucast_pkts': '13816992',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13737',
                                                                       'out_octets': '4536473801',
                                                                       'out_ucast_pkts': '13816992',
                                                                       'poller_hash': 'QURUTjIzMTI0QjdFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411500',
                                                                                                                        'fiber_length': 9207,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QjdFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEI3RQ==',
                                         'serial_number': 'ADTN23124B7E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.33.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17349',
                                                                       'in_octets': '32971990275',
                                                                       'in_ucast_pkts': '47577357',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU33_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '89779815388',
                                                                       'out_ucast_pkts': '72520182',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMzNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '89779815388',
                                                                       'in_ucast_pkts': '72520182',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '89779815388',
                                                                                                'in_ucast_pkts': '72520182',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17349',
                                                                                                'out_octets': '32971990275',
                                                                                                'out_ucast_pkts': '47577357',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17349',
                                                                       'out_octets': '32971990275',
                                                                       'out_ucast_pkts': '47577357',
                                                                       'poller_hash': 'QURUTjIzMTI0MDA4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511513',
                                                                                                                        'fiber_length': 5786,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MDA4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU33',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDAwOA==',
                                         'serial_number': 'ADTN23124008',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511513'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11789',
                                                                       'in_octets': '1576302843',
                                                                       'in_ucast_pkts': '7857963',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '394',
                                                                       'out_octets': '29168052431',
                                                                       'out_ucast_pkts': '21875490',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '394',
                                                                       'in_octets': '29168052431',
                                                                       'in_ucast_pkts': '21875490',
                                                                       'links': [       {       'in_nucast_pkts': '394',
                                                                                                'in_octets': '29168052431',
                                                                                                'in_ucast_pkts': '21875490',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11789',
                                                                                                'out_octets': '1576302843',
                                                                                                'out_ucast_pkts': '7857963',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11789',
                                                                       'out_octets': '1576302843',
                                                                       'out_ucast_pkts': '7857963',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTcwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '352937',
                                                                                                                        'fiber_length': 3865,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTcwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE3MA==',
                                         'serial_number': 'ADTN23144A70',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '352937'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '108558',
                                                                       'in_octets': '128047818368',
                                                                       'in_ucast_pkts': '389186495',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3756',
                                                                       'out_octets': '1605817453286',
                                                                       'out_ucast_pkts': '1183465374',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMjBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3756',
                                                                       'in_octets': '1605817453286',
                                                                       'in_ucast_pkts': '1183465374',
                                                                       'links': [       {       'in_nucast_pkts': '3756',
                                                                                                'in_octets': '1605817453286',
                                                                                                'in_ucast_pkts': '1183465374',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '108558',
                                                                                                'out_octets': '128047818368',
                                                                                                'out_ucast_pkts': '389186495',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '108558',
                                                                       'out_octets': '128047818368',
                                                                       'out_ucast_pkts': '389186495',
                                                                       'poller_hash': 'QURUTjIzMTI0MjE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3232209',
                                                                                                                        'fiber_length': 9856,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MjE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDIxNA==',
                                         'serial_number': 'ADTN23124214',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3232209'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17356',
                                                                       'in_octets': '7171049590',
                                                                       'in_ucast_pkts': '46866609',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '260566772494',
                                                                       'out_ucast_pkts': '186045618',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE4X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '260566772494',
                                                                       'in_ucast_pkts': '186045618',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '260566772494',
                                                                                                'in_ucast_pkts': '186045618',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17356',
                                                                                                'out_octets': '7171049590',
                                                                                                'out_ucast_pkts': '46866609',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17356',
                                                                       'out_octets': '7171049590',
                                                                       'out_ucast_pkts': '46866609',
                                                                       'poller_hash': 'QURUTjIzMTI0QzczIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511499',
                                                                                                                        'fiber_length': 4666,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QzczIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEM3Mw==',
                                         'serial_number': 'ADTN23124C73',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511499'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.27.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77255',
                                                                       'in_octets': '80112491001',
                                                                       'in_ucast_pkts': '170775637',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU27_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2573',
                                                                       'out_octets': '569526737669',
                                                                       'out_ucast_pkts': '427832231',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjdfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2573',
                                                                       'in_octets': '569526737669',
                                                                       'in_ucast_pkts': '427832231',
                                                                       'links': [       {       'in_nucast_pkts': '2573',
                                                                                                'in_octets': '569526737669',
                                                                                                'in_ucast_pkts': '427832231',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77255',
                                                                                                'out_octets': '80112491001',
                                                                                                'out_ucast_pkts': '170775637',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77255',
                                                                       'out_octets': '80112491001',
                                                                       'out_ucast_pkts': '170775637',
                                                                       'poller_hash': 'QURUTjIzMTI0QzU1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317058',
                                                                                                                        'fiber_length': 4209,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QzU1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU27',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEM1NQ==',
                                         'serial_number': 'ADTN23124C55',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317058'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '25782',
                                                                       'in_octets': '37542868176',
                                                                       'in_ucast_pkts': '76101459',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '849',
                                                                       'out_octets': '149492399337',
                                                                       'out_ucast_pkts': '149742341',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE5X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '849',
                                                                       'in_octets': '149492399337',
                                                                       'in_ucast_pkts': '149742341',
                                                                       'links': [       {       'in_nucast_pkts': '849',
                                                                                                'in_octets': '149492399337',
                                                                                                'in_ucast_pkts': '149742341',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '25782',
                                                                                                'out_octets': '37542868176',
                                                                                                'out_ucast_pkts': '76101459',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '25782',
                                                                       'out_octets': '37542868176',
                                                                       'out_ucast_pkts': '76101459',
                                                                       'poller_hash': 'QURUTjIzMTQ0RURGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '762405',
                                                                                                                        'fiber_length': 8136,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RURGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVERg==',
                                         'serial_number': 'ADTN23144EDF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '762405'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '89585',
                                                                       'in_octets': '44680359865',
                                                                       'in_ucast_pkts': '158497249',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2988',
                                                                       'out_octets': '563961505377',
                                                                       'out_ucast_pkts': '406261121',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMjBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2988',
                                                                       'in_octets': '563961505377',
                                                                       'in_ucast_pkts': '406261121',
                                                                       'links': [       {       'in_nucast_pkts': '2988',
                                                                                                'in_octets': '563961505377',
                                                                                                'in_ucast_pkts': '406261121',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '89585',
                                                                                                'out_octets': '44680359865',
                                                                                                'out_ucast_pkts': '158497249',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '89585',
                                                                       'out_octets': '44680359865',
                                                                       'out_ucast_pkts': '158497249',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUZGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2684908',
                                                                                                                        'fiber_length': 14568,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -25.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVGRg==',
                                         'serial_number': 'ADTN23144EFF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2684908'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78181',
                                                                       'in_octets': '27899552121',
                                                                       'in_ucast_pkts': '54651869',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6617',
                                                                       'out_octets': '90806962833',
                                                                       'out_ucast_pkts': '86369262',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMjFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6617',
                                                                       'in_octets': '90806962833',
                                                                       'in_ucast_pkts': '86369262',
                                                                       'links': [       {       'in_nucast_pkts': '6617',
                                                                                                'in_octets': '90806962833',
                                                                                                'in_ucast_pkts': '86369262',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78181',
                                                                                                'out_octets': '27899552121',
                                                                                                'out_ucast_pkts': '54651869',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78181',
                                                                       'out_octets': '27899552121',
                                                                       'out_ucast_pkts': '54651869',
                                                                       'poller_hash': 'QURUTjIzMTI0QTE4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2312767',
                                                                                                                        'fiber_length': 14439,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QTE4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEExOA==',
                                         'serial_number': 'ADTN23124A18',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2312767'},
                                 {       'interfaces': [       {       'admin_up': False,
                                                                       'id': 'onu-subscr-if-7.1.25.0.eth.1.phy',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 0,
                                                                       'name': 'IndianBayou-x7-ONU25_ETH_1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjVfRVRIXzE=',
                                                                       'type': 6},
                                                               {       'admin_up': False,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': False,
                                                                                                'out_nucast_pkts': '0',
                                                                                                'out_octets': '0',
                                                                                                'out_ucast_pkts': '0',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'QURUTjIzMTQ0RjAwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511502',
                                                                                                                        'fiber_length': 3323,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RjAwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU25',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEYwMA==',
                                         'serial_number': 'ADTN23144F00',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511502'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '71032',
                                                                       'in_octets': '91458886374',
                                                                       'in_ucast_pkts': '156074061',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6014',
                                                                       'out_octets': '492238996224',
                                                                       'out_ucast_pkts': '357362480',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6014',
                                                                       'in_octets': '492238996224',
                                                                       'in_ucast_pkts': '357362480',
                                                                       'links': [       {       'in_nucast_pkts': '6014',
                                                                                                'in_octets': '492238996224',
                                                                                                'in_ucast_pkts': '357362480',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '71032',
                                                                                                'out_octets': '91458886374',
                                                                                                'out_ucast_pkts': '156074061',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '71032',
                                                                       'out_octets': '91458886374',
                                                                       'out_ucast_pkts': '156074061',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUZCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2096136',
                                                                                                                        'fiber_length': 6816,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVGQg==',
                                         'serial_number': 'ADTN23144EFB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2096136'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '151308',
                                                                       'in_octets': '8129995719',
                                                                       'in_ucast_pkts': '14979672',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4924',
                                                                       'out_octets': '4163271384',
                                                                       'out_ucast_pkts': '8737748',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4924',
                                                                       'in_octets': '4163271384',
                                                                       'in_ucast_pkts': '8737748',
                                                                       'links': [       {       'in_nucast_pkts': '4924',
                                                                                                'in_octets': '4163271384',
                                                                                                'in_ucast_pkts': '8737748',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '151308',
                                                                                                'out_octets': '8129995719',
                                                                                                'out_ucast_pkts': '14979672',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '151308',
                                                                       'out_octets': '8129995719',
                                                                       'out_ucast_pkts': '14979672',
                                                                       'poller_hash': 'QURUTjIzMTI0NEJBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440101',
                                                                                                                        'fiber_length': 7484,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NEJBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDRCQQ==',
                                         'serial_number': 'ADTN231244BA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440101'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.28.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78535',
                                                                       'in_octets': '48505866974',
                                                                       'in_ucast_pkts': '136961601',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU28_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6626',
                                                                       'out_octets': '478541428408',
                                                                       'out_ucast_pkts': '341019414',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjhfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6626',
                                                                       'in_octets': '478541428408',
                                                                       'in_ucast_pkts': '341019414',
                                                                       'links': [       {       'in_nucast_pkts': '6626',
                                                                                                'in_octets': '478541428408',
                                                                                                'in_ucast_pkts': '341019414',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78535',
                                                                                                'out_octets': '48505866974',
                                                                                                'out_ucast_pkts': '136961601',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78535',
                                                                       'out_octets': '48505866974',
                                                                       'out_ucast_pkts': '136961601',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUUzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317048',
                                                                                                                        'fiber_length': 4287,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUUzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU28',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVFMw==',
                                         'serial_number': 'ADTN23144EE3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317048'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '51748',
                                                                       'in_octets': '449522113442',
                                                                       'in_ucast_pkts': '526457874',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1725',
                                                                       'out_octets': '848622428896',
                                                                       'out_ucast_pkts': '737635649',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1725',
                                                                       'in_octets': '848622428896',
                                                                       'in_ucast_pkts': '737635649',
                                                                       'links': [       {       'in_nucast_pkts': '1725',
                                                                                                'in_octets': '848622428896',
                                                                                                'in_ucast_pkts': '737635649',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '51748',
                                                                                                'out_octets': '449522113442',
                                                                                                'out_ucast_pkts': '526457874',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '51748',
                                                                       'out_octets': '449522113442',
                                                                       'out_ucast_pkts': '526457874',
                                                                       'poller_hash': 'QURUTjIzMTI0QTc2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1551603',
                                                                                                                        'fiber_length': 12765,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QTc2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEE3Ng==',
                                         'serial_number': 'ADTN23124A76',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1551603'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.25.0.eth.1.phy',
                                                                       'in_nucast_pkts': '8119817',
                                                                       'in_octets': '1884647079326',
                                                                       'in_ucast_pkts': '9369806084',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU25_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '155642',
                                                                       'out_octets': '28843170418904',
                                                                       'out_ucast_pkts': '21400391889',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '155642',
                                                                       'in_octets': '28843170418904',
                                                                       'in_ucast_pkts': '21400391889',
                                                                       'links': [       {       'in_nucast_pkts': '155642',
                                                                                                'in_octets': '28843170418904',
                                                                                                'in_ucast_pkts': '21400391889',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '8119817',
                                                                                                'out_octets': '1884647079326',
                                                                                                'out_ucast_pkts': '9369806084',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '8119817',
                                                                       'out_octets': '1884647079326',
                                                                       'out_ucast_pkts': '9369806084',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTg2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440103',
                                                                                                                        'fiber_length': 7783,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTg2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU25',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE4Ng==',
                                         'serial_number': 'ADTN23144A86',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440103'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '5130',
                                                                       'in_octets': '831887605',
                                                                       'in_ucast_pkts': '2077424',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '172',
                                                                       'out_octets': '7914458678',
                                                                       'out_ucast_pkts': '6076230',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '172',
                                                                       'in_octets': '7914458678',
                                                                       'in_ucast_pkts': '6076230',
                                                                       'links': [       {       'in_nucast_pkts': '172',
                                                                                                'in_octets': '7914458678',
                                                                                                'in_ucast_pkts': '6076230',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '5130',
                                                                                                'out_octets': '831887605',
                                                                                                'out_ucast_pkts': '2077424',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5130',
                                                                       'out_octets': '831887605',
                                                                       'out_ucast_pkts': '2077424',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '153366',
                                                                                                                        'fiber_length': 700,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVFMA==',
                                         'serial_number': 'ADTN23144EE0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '153366'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17074',
                                                                       'in_octets': '41487522417',
                                                                       'in_ucast_pkts': '46058779',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '86523696291',
                                                                       'out_ucast_pkts': '70721780',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '86523696291',
                                                                       'in_ucast_pkts': '70721780',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '86523696291',
                                                                                                'in_ucast_pkts': '70721780',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17074',
                                                                                                'out_octets': '41487522417',
                                                                                                'out_ucast_pkts': '46058779',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17074',
                                                                       'out_octets': '41487522417',
                                                                       'out_ucast_pkts': '46058779',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUM4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511498',
                                                                                                                        'fiber_length': 489,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUM4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVDOA==',
                                         'serial_number': 'ADTN23144EC8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '40455',
                                                                       'in_octets': '13102866261',
                                                                       'in_ucast_pkts': '36130618',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3423',
                                                                       'out_octets': '106071734827',
                                                                       'out_ucast_pkts': '90461016',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3423',
                                                                       'in_octets': '106071734827',
                                                                       'in_ucast_pkts': '90461016',
                                                                       'links': [       {       'in_nucast_pkts': '3423',
                                                                                                'in_octets': '106071734827',
                                                                                                'in_ucast_pkts': '90461016',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '40455',
                                                                                                'out_octets': '13102866261',
                                                                                                'out_ucast_pkts': '36130618',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '40455',
                                                                       'out_octets': '13102866261',
                                                                       'out_ucast_pkts': '36130618',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0FDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1193288',
                                                                                                                        'fiber_length': 6748,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdBQw==',
                                         'serial_number': 'ADTN231447AC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1193288'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '78536',
                                                                       'in_octets': '30883850198',
                                                                       'in_ucast_pkts': '148123087',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6651',
                                                                       'out_octets': '425517749953',
                                                                       'out_ucast_pkts': '389834749',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6651',
                                                                       'in_octets': '425517749953',
                                                                       'in_ucast_pkts': '389834749',
                                                                       'links': [       {       'in_nucast_pkts': '6651',
                                                                                                'in_octets': '425517749953',
                                                                                                'in_ucast_pkts': '389834749',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '78536',
                                                                                                'out_octets': '30883850198',
                                                                                                'out_ucast_pkts': '148123087',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78536',
                                                                       'out_octets': '30883850198',
                                                                       'out_ucast_pkts': '148123087',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUZEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317058',
                                                                                                                        'fiber_length': 6232,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVGRA==',
                                         'serial_number': 'ADTN23144EFD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317058'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.30.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17435',
                                                                       'in_octets': '18736611599',
                                                                       'in_ucast_pkts': '29611412',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU30_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '569',
                                                                       'out_octets': '70520406471',
                                                                       'out_ucast_pkts': '60883818',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTMwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '569',
                                                                       'in_octets': '70520406471',
                                                                       'in_ucast_pkts': '60883818',
                                                                       'links': [       {       'in_nucast_pkts': '569',
                                                                                                'in_octets': '70520406471',
                                                                                                'in_ucast_pkts': '60883818',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17435',
                                                                                                'out_octets': '18736611599',
                                                                                                'out_ucast_pkts': '29611412',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17435',
                                                                       'out_octets': '18736611599',
                                                                       'out_ucast_pkts': '29611412',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUQyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '508250',
                                                                                                                        'fiber_length': 4439,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUQyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU30',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVEMg==',
                                         'serial_number': 'ADTN23144ED2',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '508250'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.34.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17079',
                                                                       'in_octets': '21339841748',
                                                                       'in_ucast_pkts': '84681430',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU34_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '343899216165',
                                                                       'out_ucast_pkts': '254910822',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMzRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '343899216165',
                                                                       'in_ucast_pkts': '254910822',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '343899216165',
                                                                                                'in_ucast_pkts': '254910822',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17079',
                                                                                                'out_octets': '21339841748',
                                                                                                'out_ucast_pkts': '84681430',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17079',
                                                                       'out_octets': '21339841748',
                                                                       'out_ucast_pkts': '84681430',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUQzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511498',
                                                                                                                        'fiber_length': 5736,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUQzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU34',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVEMw==',
                                         'serial_number': 'ADTN23144ED3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17346',
                                                                       'in_octets': '6349422540',
                                                                       'in_ucast_pkts': '27790797',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '121891202937',
                                                                       'out_ucast_pkts': '95008852',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTIwX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '121891202937',
                                                                       'in_ucast_pkts': '95008852',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '121891202937',
                                                                                                'in_ucast_pkts': '95008852',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17346',
                                                                                                'out_octets': '6349422540',
                                                                                                'out_ucast_pkts': '27790797',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17346',
                                                                       'out_octets': '6349422540',
                                                                       'out_ucast_pkts': '27790797',
                                                                       'poller_hash': 'QURUTjIzMTQ0MjhBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511501',
                                                                                                                        'fiber_length': 7421,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0MjhBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDI4QQ==',
                                         'serial_number': 'ADTN2314428A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511501'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.25.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17067',
                                                                       'in_octets': '5557607180',
                                                                       'in_ucast_pkts': '33598451',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU25_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '102148766689',
                                                                       'out_ucast_pkts': '72254854',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '102148766689',
                                                                       'in_ucast_pkts': '72254854',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '102148766689',
                                                                                                'in_ucast_pkts': '72254854',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17067',
                                                                                                'out_octets': '5557607180',
                                                                                                'out_ucast_pkts': '33598451',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17067',
                                                                       'out_octets': '5557607180',
                                                                       'out_ucast_pkts': '33598451',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzE3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511498',
                                                                                                                        'fiber_length': 716,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzE3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU25',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEMxNw==',
                                         'serial_number': 'ADTN23144C17',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17355',
                                                                       'in_octets': '13123606756',
                                                                       'in_ucast_pkts': '69752375',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '290532092288',
                                                                       'out_ucast_pkts': '204848809',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjRfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '290532092288',
                                                                       'in_ucast_pkts': '204848809',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '290532092288',
                                                                                                'in_ucast_pkts': '204848809',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17355',
                                                                                                'out_octets': '13123606756',
                                                                                                'out_ucast_pkts': '69752375',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17355',
                                                                       'out_octets': '13123606756',
                                                                       'out_ucast_pkts': '69752375',
                                                                       'poller_hash': 'QURUTjIzMTQ0NjlFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511513',
                                                                                                                        'fiber_length': 6818,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NjlFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDY5RQ==',
                                         'serial_number': 'ADTN2314469E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511513'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '168',
                                                                       'in_octets': '24287069',
                                                                       'in_ucast_pkts': '268180',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13',
                                                                       'out_octets': '723445891',
                                                                       'out_ucast_pkts': '488924',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '13',
                                                                       'in_octets': '723445891',
                                                                       'in_ucast_pkts': '488924',
                                                                       'links': [       {       'in_nucast_pkts': '13',
                                                                                                'in_octets': '723445891',
                                                                                                'in_ucast_pkts': '488924',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '168',
                                                                                                'out_octets': '24287069',
                                                                                                'out_ucast_pkts': '268180',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '168',
                                                                       'out_octets': '24287069',
                                                                       'out_ucast_pkts': '268180',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjI0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4440',
                                                                                                                        'fiber_length': 6580,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjI0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIyNA==',
                                         'serial_number': 'ADTN23144B24',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '4440'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '19235',
                                                                       'in_octets': '9067388101',
                                                                       'in_ucast_pkts': '11640227',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '642',
                                                                       'out_octets': '32123065177',
                                                                       'out_ucast_pkts': '25321888',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMjFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '642',
                                                                       'in_octets': '32123065177',
                                                                       'in_ucast_pkts': '25321888',
                                                                       'links': [       {       'in_nucast_pkts': '642',
                                                                                                'in_octets': '32123065177',
                                                                                                'in_ucast_pkts': '25321888',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '19235',
                                                                                                'out_octets': '9067388101',
                                                                                                'out_ucast_pkts': '11640227',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '19235',
                                                                       'out_octets': '9067388101',
                                                                       'out_ucast_pkts': '11640227',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzRFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '576233',
                                                                                                                        'fiber_length': 10106,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzRFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEM0RQ==',
                                         'serial_number': 'ADTN23144C4E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '576233'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17070',
                                                                       'in_octets': '6446512919',
                                                                       'in_ucast_pkts': '9277120',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '25774365898',
                                                                       'out_ucast_pkts': '22077595',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '25774365898',
                                                                       'in_ucast_pkts': '22077595',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '25774365898',
                                                                                                'in_ucast_pkts': '22077595',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17070',
                                                                                                'out_octets': '6446512919',
                                                                                                'out_ucast_pkts': '9277120',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17070',
                                                                       'out_octets': '6446512919',
                                                                       'out_ucast_pkts': '9277120',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzhCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511505',
                                                                                                                        'fiber_length': 8911,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzhCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEM4Qg==',
                                         'serial_number': 'ADTN23144C8B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511505'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '2481',
                                                                       'in_octets': '671175529',
                                                                       'in_ucast_pkts': '4503121',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '84',
                                                                       'out_octets': '23274656091',
                                                                       'out_ucast_pkts': '16045359',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMjBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '84',
                                                                       'in_octets': '23274656091',
                                                                       'in_ucast_pkts': '16045359',
                                                                       'links': [       {       'in_nucast_pkts': '84',
                                                                                                'in_octets': '23274656091',
                                                                                                'in_ucast_pkts': '16045359',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '2481',
                                                                                                'out_octets': '671175529',
                                                                                                'out_ucast_pkts': '4503121',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2481',
                                                                       'out_octets': '671175529',
                                                                       'out_ucast_pkts': '4503121',
                                                                       'poller_hash': 'QURUTjIzMTQ1MTIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '96470',
                                                                                                                        'fiber_length': 5781,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ1MTIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NTEyMA==',
                                         'serial_number': 'ADTN23145120',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '96470'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13735',
                                                                       'in_octets': '5787604886',
                                                                       'in_ucast_pkts': '50188010',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '140051627721',
                                                                       'out_ucast_pkts': '98055630',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTE5X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '140051627721',
                                                                       'in_ucast_pkts': '98055630',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '140051627721',
                                                                                                'in_ucast_pkts': '98055630',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13735',
                                                                                                'out_octets': '5787604886',
                                                                                                'out_ucast_pkts': '50188010',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13735',
                                                                       'out_octets': '5787604886',
                                                                       'out_ucast_pkts': '50188010',
                                                                       'poller_hash': 'QURUTjIzMTI1MDZDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411498',
                                                                                                                        'fiber_length': 6899,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDZDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA2Qw==',
                                         'serial_number': 'ADTN2312506C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411498'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181510',
                                                                       'in_octets': '112941005951',
                                                                       'in_ucast_pkts': '351102468',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6045',
                                                                       'out_octets': '946005969035',
                                                                       'out_ucast_pkts': '699659776',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMjJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6045',
                                                                       'in_octets': '946005969035',
                                                                       'in_ucast_pkts': '699659776',
                                                                       'links': [       {       'in_nucast_pkts': '6045',
                                                                                                'in_octets': '946005969035',
                                                                                                'in_ucast_pkts': '699659776',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181510',
                                                                                                'out_octets': '112941005951',
                                                                                                'out_ucast_pkts': '351102468',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181510',
                                                                       'out_octets': '112941005951',
                                                                       'out_ucast_pkts': '351102468',
                                                                       'poller_hash': 'QURUTjIzMTI0QjNGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440065',
                                                                                                                        'fiber_length': 13633,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QjNGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU22',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEIzRg==',
                                         'serial_number': 'ADTN23124B3F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440065'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17069',
                                                                       'in_octets': '17043278935',
                                                                       'in_ucast_pkts': '35921917',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '275488666428',
                                                                       'out_ucast_pkts': '194127485',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '275488666428',
                                                                       'in_ucast_pkts': '194127485',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '275488666428',
                                                                                                'in_ucast_pkts': '194127485',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17069',
                                                                                                'out_octets': '17043278935',
                                                                                                'out_ucast_pkts': '35921917',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17069',
                                                                       'out_octets': '17043278935',
                                                                       'out_ucast_pkts': '35921917',
                                                                       'poller_hash': 'QURUTjIzMTI0NTEwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511500',
                                                                                                                        'fiber_length': 313,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NTEwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDUxMA==',
                                         'serial_number': 'ADTN23124510',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511500'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-5.1.19.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x5-ONU19_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTlfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0QjU4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjU4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x5-ONU19',
                                         'offline_reason': 'OFFLINE_REASON_POWER_OFF',
                                         'offline_reason_detail': 'A local power '
                                                                  'outage has occurred '
                                                                  'for ONU "onu '
                                                                  '5.1.19"(ADTN23144B58).',
                                         'offline_time': '2025-07-16T16:31:45Z',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEI1OA==',
                                         'serial_number': 'ADTN23144B58',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '29660',
                                                                       'in_octets': '92826400199',
                                                                       'in_ucast_pkts': '188162824',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2513',
                                                                       'out_octets': '453129660973',
                                                                       'out_ucast_pkts': '337137185',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMjNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2513',
                                                                       'in_octets': '453129660973',
                                                                       'in_ucast_pkts': '337137185',
                                                                       'links': [       {       'in_nucast_pkts': '2513',
                                                                                                'in_octets': '453129660973',
                                                                                                'in_ucast_pkts': '337137185',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '29660',
                                                                                                'out_octets': '92826400199',
                                                                                                'out_ucast_pkts': '188162824',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '29660',
                                                                       'out_octets': '92826400199',
                                                                       'out_ucast_pkts': '188162824',
                                                                       'poller_hash': 'QURUTjIzMTI0QkE1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '873712',
                                                                                                                        'fiber_length': 14906,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QkE1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEJBNQ==',
                                         'serial_number': 'ADTN23124BA5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '873712'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17074',
                                                                       'in_octets': '8007726891',
                                                                       'in_ucast_pkts': '57195326',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '170355661509',
                                                                       'out_ucast_pkts': '126103840',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '170355661509',
                                                                       'in_ucast_pkts': '126103840',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '170355661509',
                                                                                                'in_ucast_pkts': '126103840',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17074',
                                                                                                'out_octets': '8007726891',
                                                                                                'out_ucast_pkts': '57195326',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17074',
                                                                       'out_octets': '8007726891',
                                                                       'out_ucast_pkts': '57195326',
                                                                       'poller_hash': 'QURUTjIzMTI0N0ExIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511512',
                                                                                                                        'fiber_length': 3094,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0N0ExIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDdBMQ==',
                                         'serial_number': 'ADTN231247A1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511512'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13737',
                                                                       'in_octets': '13904986517',
                                                                       'in_ucast_pkts': '44885617',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '182674651329',
                                                                       'out_ucast_pkts': '136382090',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTE1X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '182674651329',
                                                                       'in_ucast_pkts': '136382090',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '182674651329',
                                                                                                'in_ucast_pkts': '136382090',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13737',
                                                                                                'out_octets': '13904986517',
                                                                                                'out_ucast_pkts': '44885617',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13737',
                                                                       'out_octets': '13904986517',
                                                                       'out_ucast_pkts': '44885617',
                                                                       'poller_hash': 'QURUTjIzMTI0OTY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411500',
                                                                                                                        'fiber_length': 9245,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OTY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDk2Ng==',
                                         'serial_number': 'ADTN23124966',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '411500'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '69483',
                                                                       'in_octets': '9445157701',
                                                                       'in_ucast_pkts': '80859241',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2316',
                                                                       'out_octets': '255804624224',
                                                                       'out_ucast_pkts': '177152668',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2316',
                                                                       'in_octets': '255804624224',
                                                                       'in_ucast_pkts': '177152668',
                                                                       'links': [       {       'in_nucast_pkts': '2316',
                                                                                                'in_octets': '255804624224',
                                                                                                'in_ucast_pkts': '177152668',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '69483',
                                                                                                'out_octets': '9445157701',
                                                                                                'out_ucast_pkts': '80859241',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '69483',
                                                                       'out_octets': '9445157701',
                                                                       'out_ucast_pkts': '80859241',
                                                                       'poller_hash': 'QURUTjIzMTI0NjBBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2083695',
                                                                                                                        'fiber_length': 4437,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NjBBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDYwQQ==',
                                         'serial_number': 'ADTN2312460A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2083695'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '87861',
                                                                       'in_octets': '47614043797',
                                                                       'in_ucast_pkts': '200344149',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2878',
                                                                       'out_octets': '609293328531',
                                                                       'out_ucast_pkts': '439693210',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2878',
                                                                       'in_octets': '609293328531',
                                                                       'in_ucast_pkts': '439693210',
                                                                       'links': [       {       'in_nucast_pkts': '2878',
                                                                                                'in_octets': '609293328531',
                                                                                                'in_ucast_pkts': '439693210',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '87861',
                                                                                                'out_octets': '47614043797',
                                                                                                'out_ucast_pkts': '200344149',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '87861',
                                                                       'out_octets': '47614043797',
                                                                       'out_ucast_pkts': '200344149',
                                                                       'poller_hash': 'QURUTjIzMTI1MTM1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2588442',
                                                                                                                        'fiber_length': 6346,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTM1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEzNQ==',
                                         'serial_number': 'ADTN23125135',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2588442'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.27.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17076',
                                                                       'in_octets': '5211792737',
                                                                       'in_ucast_pkts': '32682412',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU27_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '104083040915',
                                                                       'out_ucast_pkts': '72720282',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjdfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '104083040915',
                                                                       'in_ucast_pkts': '72720282',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '104083040915',
                                                                                                'in_ucast_pkts': '72720282',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17076',
                                                                                                'out_octets': '5211792737',
                                                                                                'out_ucast_pkts': '32682412',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17076',
                                                                       'out_octets': '5211792737',
                                                                       'out_ucast_pkts': '32682412',
                                                                       'poller_hash': 'QURUTjIzMTI0QzZBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511512',
                                                                                                                        'fiber_length': 3062,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QzZBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU27',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEM2QQ==',
                                         'serial_number': 'ADTN23124C6A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511512'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.29.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77256',
                                                                       'in_octets': '8706303577',
                                                                       'in_ucast_pkts': '42490684',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU29_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2571',
                                                                       'out_octets': '215562903845',
                                                                       'out_ucast_pkts': '162529006',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2571',
                                                                       'in_octets': '215562903845',
                                                                       'in_ucast_pkts': '162529006',
                                                                       'links': [       {       'in_nucast_pkts': '2571',
                                                                                                'in_octets': '215562903845',
                                                                                                'in_ucast_pkts': '162529006',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77256',
                                                                                                'out_octets': '8706303577',
                                                                                                'out_ucast_pkts': '42490684',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77256',
                                                                       'out_octets': '8706303577',
                                                                       'out_ucast_pkts': '42490684',
                                                                       'poller_hash': 'QURUTjI0MzYzQUU3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317066',
                                                                                                                        'fiber_length': 3543,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQUU3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU29',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0FFNw==',
                                         'serial_number': 'ADTN24363AE7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2317066'},
                                 {       'interfaces': [       {       'admin_up': False,
                                                                       'id': 'onu-subscr-if-5.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 0,
                                                                       'name': 'IndianBayou-x5-ONU21_ETH_1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMjFfRVRIXzE=',
                                                                       'type': 6},
                                                               {       'admin_up': False,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': False,
                                                                                                'out_nucast_pkts': '0',
                                                                                                'out_octets': '0',
                                                                                                'out_ucast_pkts': '0',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'QURUTjI0MzYzRDNBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1634735',
                                                                                                                        'fiber_length': 6741,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRDNBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU21',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0QzQQ==',
                                         'serial_number': 'ADTN24363D3A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1634735'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '182366',
                                                                       'in_octets': '200222667955',
                                                                       'in_ucast_pkts': '1068086035',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6047',
                                                                       'out_octets': '5909796819926',
                                                                       'out_ucast_pkts': '4494340833',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6047',
                                                                       'in_octets': '5909796819926',
                                                                       'in_ucast_pkts': '4494340833',
                                                                       'links': [       {       'in_nucast_pkts': '6047',
                                                                                                'in_octets': '5909796819926',
                                                                                                'in_ucast_pkts': '4494340833',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '182366',
                                                                                                'out_octets': '200222667955',
                                                                                                'out_ucast_pkts': '1068086035',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '182366',
                                                                       'out_octets': '200222667955',
                                                                       'out_ucast_pkts': '1068086035',
                                                                       'poller_hash': 'QURUTjI0MzYzQ0M2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5440065',
                                                                                                                        'fiber_length': 13230,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQ0M2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU13',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0NDNg==',
                                         'serial_number': 'ADTN24363CC6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5440065'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.32.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17073',
                                                                       'in_octets': '5040846635',
                                                                       'in_ucast_pkts': '18204867',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU32_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '29200136912',
                                                                       'out_ucast_pkts': '21341048',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTMyX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '29200136912',
                                                                       'in_ucast_pkts': '21341048',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '29200136912',
                                                                                                'in_ucast_pkts': '21341048',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17073',
                                                                                                'out_octets': '5040846635',
                                                                                                'out_ucast_pkts': '18204867',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17073',
                                                                       'out_octets': '5040846635',
                                                                       'out_ucast_pkts': '18204867',
                                                                       'poller_hash': 'QURUTjI0MzYzRDREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511510',
                                                                                                                        'fiber_length': 3928,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRDREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU32',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0Q0RA==',
                                         'serial_number': 'ADTN24363D4D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511510'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '116608',
                                                                       'in_octets': '51659450960',
                                                                       'in_ucast_pkts': '382203301',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3831',
                                                                       'out_octets': '1410544228418',
                                                                       'out_ucast_pkts': '992370722',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTIxX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3831',
                                                                       'in_octets': '1410544228418',
                                                                       'in_ucast_pkts': '992370722',
                                                                       'links': [       {       'in_nucast_pkts': '3831',
                                                                                                'in_octets': '1410544228418',
                                                                                                'in_ucast_pkts': '992370722',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '116608',
                                                                                                'out_octets': '51659450960',
                                                                                                'out_ucast_pkts': '382203301',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '116608',
                                                                       'out_octets': '51659450960',
                                                                       'out_ucast_pkts': '382203301',
                                                                       'poller_hash': 'QURUTjIzMTI0MUEyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3449046',
                                                                                                                        'fiber_length': 8116,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MUEyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDFBMg==',
                                         'serial_number': 'ADTN231241A2',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_1-31',
                                         'uptime': '3449046'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13738',
                                                                       'in_octets': '6350521169',
                                                                       'in_ucast_pkts': '9334062',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '36665106028',
                                                                       'out_ucast_pkts': '28912537',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTE2X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '36665106028',
                                                                       'in_ucast_pkts': '28912537',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '36665106028',
                                                                                                'in_ucast_pkts': '28912537',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13738',
                                                                                                'out_octets': '6350521169',
                                                                                                'out_ucast_pkts': '9334062',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13738',
                                                                       'out_octets': '6350521169',
                                                                       'out_ucast_pkts': '9334062',
                                                                       'poller_hash': 'QURUTjI0MzYzREI5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411504',
                                                                                                                        'fiber_length': 9079,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzREI5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU16',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0RCOQ==',
                                         'serial_number': 'ADTN24363DB9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '411504'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.28.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17229',
                                                                       'in_octets': '4548076901',
                                                                       'in_ucast_pkts': '56225134',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU28_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '178557922115',
                                                                       'out_ucast_pkts': '119393201',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjhfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '178557922115',
                                                                       'in_ucast_pkts': '119393201',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '178557922115',
                                                                                                'in_ucast_pkts': '119393201',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17229',
                                                                                                'out_octets': '4548076901',
                                                                                                'out_ucast_pkts': '56225134',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17229',
                                                                       'out_octets': '4548076901',
                                                                       'out_ucast_pkts': '56225134',
                                                                       'poller_hash': 'QURUTjI0MzYzREM5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511516',
                                                                                                                        'fiber_length': 3347,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzREM5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU28',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0RDOQ==',
                                         'serial_number': 'ADTN24363DC9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '511516'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13739',
                                                                       'in_octets': '3138746474',
                                                                       'in_ucast_pkts': '11181202',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '56765714029',
                                                                       'out_ucast_pkts': '41327644',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE4X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '56765714029',
                                                                       'in_ucast_pkts': '41327644',
                                                                       'links': [       {       'in_nucast_pkts': '459',
                                                                                                'in_octets': '56765714029',
                                                                                                'in_ucast_pkts': '41327644',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13739',
                                                                                                'out_octets': '3138746474',
                                                                                                'out_ucast_pkts': '11181202',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13739',
                                                                       'out_octets': '3138746474',
                                                                       'out_ucast_pkts': '11181202',
                                                                       'poller_hash': 'QURUTjI0MzYzQkJBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411503',
                                                                                                                        'fiber_length': 8469,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQkJBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU18',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0JCQQ==',
                                         'serial_number': 'ADTN24363BBA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '411503'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '19253',
                                                                       'in_octets': '8221915797',
                                                                       'in_ucast_pkts': '19612117',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '643',
                                                                       'out_octets': '68669345709',
                                                                       'out_ucast_pkts': '50805687',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VNl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '643',
                                                                       'in_octets': '68669345709',
                                                                       'in_ucast_pkts': '50805687',
                                                                       'links': [       {       'in_nucast_pkts': '643',
                                                                                                'in_octets': '68669345709',
                                                                                                'in_ucast_pkts': '50805687',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '19253',
                                                                                                'out_octets': '8221915797',
                                                                                                'out_ucast_pkts': '19612117',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '19253',
                                                                       'out_octets': '8221915797',
                                                                       'out_ucast_pkts': '19612117',
                                                                       'poller_hash': 'QURUTjI0MzE0NEZCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '576799',
                                                                                                                        'fiber_length': 10046,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0NEZCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU6',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDRGQg==',
                                         'serial_number': 'ADTN243144FB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '576799'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.30.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77256',
                                                                       'in_octets': '17543890243',
                                                                       'in_ucast_pkts': '16181738',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU30_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2570',
                                                                       'out_octets': '20443373744',
                                                                       'out_ucast_pkts': '19280419',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMzBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2570',
                                                                       'in_octets': '20443373744',
                                                                       'in_ucast_pkts': '19280419',
                                                                       'links': [       {       'in_nucast_pkts': '2570',
                                                                                                'in_octets': '20443373744',
                                                                                                'in_ucast_pkts': '19280419',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77256',
                                                                                                'out_octets': '17543890243',
                                                                                                'out_ucast_pkts': '16181738',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77256',
                                                                       'out_octets': '17543890243',
                                                                       'out_ucast_pkts': '16181738',
                                                                       'poller_hash': 'QURUTjI0Mzg5NkY5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317062',
                                                                                                                        'fiber_length': 3918,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0Mzg5NkY5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU30',
                                         'poller_hash': 'b250IEFEVE4yNDM4OTZGOQ==',
                                         'serial_number': 'ADTN243896F9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '2317062'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.28.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17070',
                                                                       'in_octets': '7687648925',
                                                                       'in_ucast_pkts': '67999725',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU28_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '327100804142',
                                                                       'out_ucast_pkts': '223208202',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjhfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '327100804142',
                                                                       'in_ucast_pkts': '223208202',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '327100804142',
                                                                                                'in_ucast_pkts': '223208202',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17070',
                                                                                                'out_octets': '7687648925',
                                                                                                'out_ucast_pkts': '67999725',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17070',
                                                                       'out_octets': '7687648925',
                                                                       'out_ucast_pkts': '67999725',
                                                                       'poller_hash': 'QURUTjI0MzE0NzY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511505',
                                                                                                                        'fiber_length': 859,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0NzY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU28',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDc2Ng==',
                                         'serial_number': 'ADTN24314766',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '511505'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '15132',
                                                                       'in_octets': '30461726021',
                                                                       'in_ucast_pkts': '153669174',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '505',
                                                                       'out_octets': '472024713335',
                                                                       'out_ucast_pkts': '353532698',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '505',
                                                                       'in_octets': '472024713335',
                                                                       'in_ucast_pkts': '353532698',
                                                                       'links': [       {       'in_nucast_pkts': '505',
                                                                                                'in_octets': '472024713335',
                                                                                                'in_ucast_pkts': '353532698',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '15132',
                                                                                                'out_octets': '30461726021',
                                                                                                'out_ucast_pkts': '153669174',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15132',
                                                                       'out_octets': '30461726021',
                                                                       'out_ucast_pkts': '153669174',
                                                                       'poller_hash': 'QURUTjI0MzE0MzE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '453158',
                                                                                                                        'fiber_length': 3270,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MzE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU22',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDMxNA==',
                                         'serial_number': 'ADTN24314314',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '453158'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '80304',
                                                                       'in_octets': '183035148948',
                                                                       'in_ucast_pkts': '1236860971',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6637',
                                                                       'out_octets': '4528321524493',
                                                                       'out_ucast_pkts': '3140107222',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6637',
                                                                       'in_octets': '4528321524493',
                                                                       'in_ucast_pkts': '3140107222',
                                                                       'links': [       {       'in_nucast_pkts': '6637',
                                                                                                'in_octets': '4528321524493',
                                                                                                'in_ucast_pkts': '3140107222',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '80304',
                                                                                                'out_octets': '183035148948',
                                                                                                'out_ucast_pkts': '1236860971',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '80304',
                                                                       'out_octets': '183035148948',
                                                                       'out_ucast_pkts': '1236860971',
                                                                       'poller_hash': 'QURUTjI0MzE0MTk1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317064',
                                                                                                                        'fiber_length': 6741,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MTk1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU10',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDE5NQ==',
                                         'serial_number': 'ADTN24314195',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '2317064'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.31.0.eth.1.phy',
                                                                       'in_nucast_pkts': '307',
                                                                       'in_octets': '12297137773',
                                                                       'in_ucast_pkts': '74863842',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU31_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1467',
                                                                       'out_octets': '254976806134',
                                                                       'out_ucast_pkts': '187481803',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTMxX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1467',
                                                                       'in_octets': '254976806134',
                                                                       'in_ucast_pkts': '187481803',
                                                                       'links': [       {       'in_nucast_pkts': '1467',
                                                                                                'in_octets': '254976806134',
                                                                                                'in_ucast_pkts': '187481803',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '307',
                                                                                                'out_octets': '12297137773',
                                                                                                'out_ucast_pkts': '74863842',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '307',
                                                                       'out_octets': '12297137773',
                                                                       'out_ucast_pkts': '74863842',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511504',
                                                                                                                        'fiber_length': 5562,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU31',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIyMA==',
                                         'serial_number': 'ADTN23144B20',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '511504'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77272',
                                                                       'in_octets': '33000539155',
                                                                       'in_ucast_pkts': '261722329',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2576',
                                                                       'out_octets': '754928010546',
                                                                       'out_ucast_pkts': '529398495',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMjJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2576',
                                                                       'in_octets': '754928010546',
                                                                       'in_ucast_pkts': '529398495',
                                                                       'links': [       {       'in_nucast_pkts': '2576',
                                                                                                'in_octets': '754928010546',
                                                                                                'in_ucast_pkts': '529398495',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77272',
                                                                                                'out_octets': '33000539155',
                                                                                                'out_ucast_pkts': '261722329',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77272',
                                                                       'out_octets': '33000539155',
                                                                       'out_ucast_pkts': '261722329',
                                                                       'poller_hash': 'QURUTjI0MzE0MUEzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2317064',
                                                                                                                        'fiber_length': 6541,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MUEzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU22',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDFBMw==',
                                         'serial_number': 'ADTN243141A3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '2317064'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '13960',
                                                                       'in_octets': '2291465599',
                                                                       'in_ucast_pkts': '10180171',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1182',
                                                                       'out_octets': '27268442391',
                                                                       'out_ucast_pkts': '21783386',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE5X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1182',
                                                                       'in_octets': '27268442391',
                                                                       'in_ucast_pkts': '21783386',
                                                                       'links': [       {       'in_nucast_pkts': '1182',
                                                                                                'in_octets': '27268442391',
                                                                                                'in_ucast_pkts': '21783386',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '13960',
                                                                                                'out_octets': '2291465599',
                                                                                                'out_ucast_pkts': '10180171',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13960',
                                                                       'out_octets': '2291465599',
                                                                       'out_ucast_pkts': '10180171',
                                                                       'poller_hash': 'QURUTjI0MzE0MUFDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '411503',
                                                                                                                        'fiber_length': 9546,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MUFDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU19',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDFBQw==',
                                         'serial_number': 'ADTN243141AC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '411503'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-1.1.27.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x1-ONU27_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjdfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjI0MzYzQjZCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQjZCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x1-ONU27',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '1.1.27"(ADTN24363B6B) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/1.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-07-07T20:35:31Z',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0I2Qg==',
                                         'serial_number': 'ADTN24363B6B',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.31.0.eth.1.phy',
                                                                       'in_nucast_pkts': '56791',
                                                                       'in_octets': '36629619784',
                                                                       'in_ucast_pkts': '86480359',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU31_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1897',
                                                                       'out_octets': '570731716285',
                                                                       'out_ucast_pkts': '410052208',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMzFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1897',
                                                                       'in_octets': '570731716285',
                                                                       'in_ucast_pkts': '410052208',
                                                                       'links': [       {       'in_nucast_pkts': '1897',
                                                                                                'in_octets': '570731716285',
                                                                                                'in_ucast_pkts': '410052208',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '56791',
                                                                                                'out_octets': '36629619784',
                                                                                                'out_ucast_pkts': '86480359',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '56791',
                                                                       'out_octets': '36629619784',
                                                                       'out_ucast_pkts': '86480359',
                                                                       'poller_hash': 'QURUTjI0MzYzQzQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1702391',
                                                                                                                        'fiber_length': 4116,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU31',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M0OQ==',
                                         'serial_number': 'ADTN24363C49',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '1702391'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '17496',
                                                                       'in_octets': '57766222085',
                                                                       'in_ucast_pkts': '69006479',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '570',
                                                                       'out_octets': '153363791304',
                                                                       'out_ucast_pkts': '125730792',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTIyX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '570',
                                                                       'in_octets': '153363791304',
                                                                       'in_ucast_pkts': '125730792',
                                                                       'links': [       {       'in_nucast_pkts': '570',
                                                                                                'in_octets': '153363791304',
                                                                                                'in_ucast_pkts': '125730792',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '17496',
                                                                                                'out_octets': '57766222085',
                                                                                                'out_ucast_pkts': '69006479',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17496',
                                                                       'out_octets': '57766222085',
                                                                       'out_ucast_pkts': '69006479',
                                                                       'poller_hash': 'QURUTjI0NDkxN0E0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '511506',
                                                                                                                        'fiber_length': 4703,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0NDkxN0E0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU22',
                                         'poller_hash': 'b250IEFEVE4yNDQ5MTdBNA==',
                                         'serial_number': 'ADTN244917A4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '511506'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '8180',
                                                                       'in_octets': '18576547566',
                                                                       'in_ucast_pkts': '55485289',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '268',
                                                                       'out_octets': '163409177184',
                                                                       'out_ucast_pkts': '128401759',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTE3X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '268',
                                                                       'in_octets': '163409177184',
                                                                       'in_ucast_pkts': '128401759',
                                                                       'links': [       {       'in_nucast_pkts': '268',
                                                                                                'in_octets': '163409177184',
                                                                                                'in_ucast_pkts': '128401759',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '8180',
                                                                                                'out_octets': '18576547566',
                                                                                                'out_ucast_pkts': '55485289',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '8180',
                                                                       'out_octets': '18576547566',
                                                                       'out_ucast_pkts': '55485289',
                                                                       'poller_hash': 'QURUTjI1MTUyMUEyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '238122',
                                                                                                                        'fiber_length': 9416,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI1MTUyMUEyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU17',
                                         'poller_hash': 'b250IEFEVE4yNTE1MjFBMg==',
                                         'serial_number': 'ADTN251521A2',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '238122'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '53768',
                                                                       'in_octets': '90783772980',
                                                                       'in_ucast_pkts': '337362943',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4457',
                                                                       'out_octets': '888976983721',
                                                                       'out_ucast_pkts': '643600902',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4457',
                                                                       'in_octets': '888976983721',
                                                                       'in_ucast_pkts': '643600902',
                                                                       'links': [       {       'in_nucast_pkts': '4457',
                                                                                                'in_octets': '888976983721',
                                                                                                'in_ucast_pkts': '643600902',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '53768',
                                                                                                'out_octets': '90783772980',
                                                                                                'out_ucast_pkts': '337362943',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '53768',
                                                                       'out_octets': '90783772980',
                                                                       'out_ucast_pkts': '337362943',
                                                                       'poller_hash': 'QURUTjIzMTQ0OEI4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1553615',
                                                                                                                        'fiber_length': 13206,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEI4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhCOA==',
                                         'serial_number': 'ADTN231448B8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1553615'}]
        }
        id_field = 'mac_address' if not obj.stations else ('mac_address' if hasattr(obj.stations[0], 'mac_address') and getattr(obj.stations[0], 'mac_address') else ('imsi' if hasattr(obj.stations[0], 'imsi') else 'serial'))
        cur_info = {x: getattr(obj, x, None) for (x, data_type) in self.apdd.get_metric_fields('info', 'ap', obj.model) + [('vendor', 'str'), ('model', 'str'), ('sw_version', 'str')]}
        if 'rf_freq' in cur_info and cur_info.get('rf_freq') and (cur_info['rf_freq'] <= 0 or int.bit_length(cur_info['rf_freq']) >= 32):
            cur_info.pop('rf_freq')
        cur_stats = {x: getattr(obj, x, None) for (x, data_type) in self.apdd.get_metric_fields('stats', 'ap', obj.model)}
        cur_sta_stats = {getattr(sta, id_field): {x: getattr(sta, x, None) for (x, data_type) in sorted(self.apdd.get_metric_fields('stats', 'cpe', obj.model)) + [(id_field, 'str'), ('ip_address', 'str'), ('name', 'str'), ('dev_macs', 'str')]} for sta in sorted(obj.stations)}
        cur_ne_msg = getattr(obj, 'network_element_update_msg', None)
        cur_ne_msg = cur_ne_msg.pb if cur_ne_msg else None
        self.assertEqual(cur_info, exp_info)
        self.assertEqual(cur_stats, exp_stats)
        self.assertEqual(cur_sta_stats, exp_sta_stats)
        self.assertTrue(self.compare_pb_dict(cur_ne_msg, exp_ne_msg))

    def test_adtran_sdx6000_SDX_6320_16_sdx6320_RL_23_5_33_10_59_5_254_02(self):
        if DeviceTestCase.TestDisabled:
                DeviceTestCase.skipTest(self, "Python 3.7 doesn't support this library")
        obj = self.run_data("adtran.sdx6000.SDX 6320-16.sdx6320-RL-23.5-33.10_59_5_254_02")
        exp_info = {
                'model': 'SDX 6320-16',
                'sta_count': 338,
                'sw_version': 'sdx6320-RL-23.5-33',
                'vendor': 'Adtran'
        }
        exp_stats = {
         
        }
        exp_sta_stats = {
                'ADTN23123E9C': {       'dev_macs': ['38:f8:f6:52:b1:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU2',
                                        'serial': 'ADTN23123E9C'},
                'ADTN23123E9D': {       'dev_macs': ['cc:66:18:dd:8f:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU16',
                                        'serial': 'ADTN23123E9D'},
                'ADTN23123E9E': {       'dev_macs': ['cc:66:18:50:b7:70'],
                                        'ip_address': None,
                                        'name': 'ADTN23123E9E',
                                        'serial': 'ADTN23123E9E'},
                'ADTN23123EA1': {       'dev_macs': ['cc:66:18:db:77:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU17',
                                        'serial': 'ADTN23123EA1'},
                'ADTN23123EA8': {       'dev_macs': ['cc:66:18:dd:88:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU29',
                                        'serial': 'ADTN23123EA8'},
                'ADTN23123EB7': {       'dev_macs': ['38:f8:f6:52:ac:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU10',
                                        'serial': 'ADTN23123EB7'},
                'ADTN23123EBC': {       'dev_macs': ['cc:66:18:dc:77:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU13',
                                        'serial': 'ADTN23123EBC'},
                'ADTN23123EC0': {       'dev_macs': ['cc:66:18:db:65:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU11',
                                        'serial': 'ADTN23123EC0'},
                'ADTN23123F18': {       'dev_macs': ['cc:66:18:db:7d:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU4',
                                        'serial': 'ADTN23123F18'},
                'ADTN23123F2B': {       'dev_macs': ['cc:66:18:50:5c:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU11',
                                        'serial': 'ADTN23123F2B'},
                'ADTN23123F57': {       'dev_macs': ['cc:66:18:dc:68:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU2',
                                        'serial': 'ADTN23123F57'},
                'ADTN23123FAD': {       'dev_macs': ['cc:66:18:dc:76:60'],
                                        'ip_address': None,
                                        'name': 'ADTN23123FAD',
                                        'serial': 'ADTN23123FAD'},
                'ADTN23123FC4': {       'dev_macs': ['cc:66:18:dd:89:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU22',
                                        'serial': 'ADTN23123FC4'},
                'ADTN23123FDC': {       'dev_macs': ['cc:66:18:d6:37:81'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU23',
                                        'serial': 'ADTN23123FDC'},
                'ADTN23123FE9': {       'dev_macs': ['cc:66:18:50:b1:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU13',
                                        'serial': 'ADTN23123FE9'},
                'ADTN23123FEC': {       'dev_macs': ['cc:66:18:4c:03:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU28',
                                        'serial': 'ADTN23123FEC'},
                'ADTN23123FEE': {       'dev_macs': ['cc:66:18:db:65:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU12',
                                        'serial': 'ADTN23123FEE'},
                'ADTN23123FF5': {       'dev_macs': ['cc:66:18:d6:0c:10'],
                                        'ip_address': None,
                                        'name': 'ADTN23123FF5',
                                        'serial': 'ADTN23123FF5'},
                'ADTN23124008': {       'dev_macs': ['cc:66:18:d6:3b:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU33',
                                        'serial': 'ADTN23124008'},
                'ADTN2312408C': {       'dev_macs': ['38:f8:f6:52:ad:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU17',
                                        'serial': 'ADTN2312408C'},
                'ADTN23124097': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU17',
                                        'serial': 'ADTN23124097'},
                'ADTN23124099': {       'dev_macs': ['cc:66:18:51:50:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU19',
                                        'serial': 'ADTN23124099'},
                'ADTN231240A6': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU13',
                                        'serial': 'ADTN231240A6'},
                'ADTN231240B0': {       'dev_macs': ['cc:66:18:dc:5d:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU1',
                                        'serial': 'ADTN231240B0'},
                'ADTN23124128': {       'dev_macs': ['cc:66:18:dc:82:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU8',
                                        'serial': 'ADTN23124128'},
                'ADTN231241A2': {       'dev_macs': ['cc:66:18:50:ae:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU21',
                                        'serial': 'ADTN231241A2'},
                'ADTN231241BF': {       'dev_macs': ['cc:66:18:50:50:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU22',
                                        'serial': 'ADTN231241BF'},
                'ADTN231241C0': {       'dev_macs': ['cc:66:18:dd:8f:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU10',
                                        'serial': 'ADTN231241C0'},
                'ADTN231241D0': {       'dev_macs': ['cc:66:18:dd:7c:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU8',
                                        'serial': 'ADTN231241D0'},
                'ADTN231241D1': {       'dev_macs': ['38:f8:f6:52:ad:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU3',
                                        'serial': 'ADTN231241D1'},
                'ADTN23124214': {       'dev_macs': ['cc:66:18:50:f1:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU20',
                                        'serial': 'ADTN23124214'},
                'ADTN23124304': {       'dev_macs': ['cc:66:18:d6:0b:60'],
                                        'ip_address': None,
                                        'name': 'ADTN23124304',
                                        'serial': 'ADTN23124304'},
                'ADTN2312432E': {       'dev_macs': ['cc:66:18:d6:0a:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU0',
                                        'serial': 'ADTN2312432E'},
                'ADTN2312445C': {       'dev_macs': ['cc:66:18:50:d8:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU7',
                                        'serial': 'ADTN2312445C'},
                'ADTN23124491': {       'dev_macs': ['cc:66:18:d5:fd:61'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU19',
                                        'serial': 'ADTN23124491'},
                'ADTN231244BA': {       'dev_macs': ['38:f8:f6:52:aa:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU0',
                                        'serial': 'ADTN231244BA'},
                'ADTN231244BC': {       'dev_macs': ['38:f8:f6:52:a7:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU8',
                                        'serial': 'ADTN231244BC'},
                'ADTN231244EB': {       'dev_macs': ['cc:66:18:d6:3a:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU17',
                                        'serial': 'ADTN231244EB'},
                'ADTN23124510': {       'dev_macs': ['cc:66:18:50:3d:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU26',
                                        'serial': 'ADTN23124510'},
                'ADTN231245C6': {       'dev_macs': ['cc:66:18:4b:fa:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU6',
                                        'serial': 'ADTN231245C6'},
                'ADTN2312460A': {       'dev_macs': ['38:f8:f6:52:a3:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU2',
                                        'serial': 'ADTN2312460A'},
                'ADTN23124650': {       'dev_macs': ['cc:66:18:dc:81:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU3',
                                        'serial': 'ADTN23124650'},
                'ADTN23124713': {       'dev_macs': ['cc:66:18:dc:88:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU20',
                                        'serial': 'ADTN23124713'},
                'ADTN231247A1': {       'dev_macs': ['cc:66:18:50:51:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU26',
                                        'serial': 'ADTN231247A1'},
                'ADTN23124849': {       'dev_macs': ['cc:66:18:d6:07:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU1',
                                        'serial': 'ADTN23124849'},
                'ADTN231248A8': {       'dev_macs': ['cc:66:18:50:6c:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU9',
                                        'serial': 'ADTN231248A8'},
                'ADTN23124900': {       'dev_macs': ['cc:66:18:dc:72:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU7',
                                        'serial': 'ADTN23124900'},
                'ADTN2312491E': {       'dev_macs': ['cc:66:18:dd:88:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU6',
                                        'serial': 'ADTN2312491E'},
                'ADTN23124966': {       'dev_macs': ['cc:66:18:50:37:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU15',
                                        'serial': 'ADTN23124966'},
                'ADTN23124A18': {       'dev_macs': ['cc:66:18:d6:38:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU21',
                                        'serial': 'ADTN23124A18'},
                'ADTN23124A23': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU3',
                                        'serial': 'ADTN23124A23'},
                'ADTN23124A76': {       'dev_macs': [       '36:3a:c5:c7:0e:34',
                                                            'cc:66:18:50:b4:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU19',
                                        'serial': 'ADTN23124A76'},
                'ADTN23124B3F': {       'dev_macs': ['cc:66:18:50:43:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU22',
                                        'serial': 'ADTN23124B3F'},
                'ADTN23124B7E': {       'dev_macs': ['cc:66:18:50:ef:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU14',
                                        'serial': 'ADTN23124B7E'},
                'ADTN23124BA5': {       'dev_macs': ['cc:66:18:d6:38:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU23',
                                        'serial': 'ADTN23124BA5'},
                'ADTN23124C55': {       'dev_macs': ['cc:66:18:50:81:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU27',
                                        'serial': 'ADTN23124C55'},
                'ADTN23124C6A': {       'dev_macs': ['cc:66:18:51:26:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU27',
                                        'serial': 'ADTN23124C6A'},
                'ADTN23124C73': {       'dev_macs': ['cc:66:18:d6:3c:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU18',
                                        'serial': 'ADTN23124C73'},
                'ADTN23124D38': {       'dev_macs': ['cc:66:18:dc:77:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU6',
                                        'serial': 'ADTN23124D38'},
                'ADTN23124D41': {       'dev_macs': ['cc:66:18:dc:71:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU0',
                                        'serial': 'ADTN23124D41'},
                'ADTN23124D43': {       'dev_macs': ['cc:66:18:dc:56:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU1',
                                        'serial': 'ADTN23124D43'},
                'ADTN23124D46': {       'dev_macs': ['38:f8:f6:52:b4:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU5',
                                        'serial': 'ADTN23124D46'},
                'ADTN23124D50': {       'dev_macs': ['cc:66:18:dc:78:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU5',
                                        'serial': 'ADTN23124D50'},
                'ADTN23124D68': {       'dev_macs': ['cc:66:18:dc:88:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU0',
                                        'serial': 'ADTN23124D68'},
                'ADTN23124D7D': {       'dev_macs': ['38:f8:f6:52:7a:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU7',
                                        'serial': 'ADTN23124D7D'},
                'ADTN23124D7E': {       'dev_macs': ['cc:66:18:dc:74:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU5',
                                        'serial': 'ADTN23124D7E'},
                'ADTN23124D91': {       'dev_macs': ['38:f8:f6:52:a9:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU0',
                                        'serial': 'ADTN23124D91'},
                'ADTN23124D93': {       'dev_macs': ['cc:66:18:dc:87:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU4',
                                        'serial': 'ADTN23124D93'},
                'ADTN23124D98': {       'dev_macs': ['cc:66:18:dd:92:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU4',
                                        'serial': 'ADTN23124D98'},
                'ADTN23124D9E': {       'dev_macs': ['cc:66:18:db:66:60'],
                                        'ip_address': None,
                                        'name': 'ADTN23124D9E',
                                        'serial': 'ADTN23124D9E'},
                'ADTN23124DA0': {       'dev_macs': ['cc:66:18:50:48:80'],
                                        'ip_address': None,
                                        'name': 'ADTN23124DA0',
                                        'serial': 'ADTN23124DA0'},
                'ADTN23124DA6': {       'dev_macs': ['cc:66:18:dc:77:f0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124DA6',
                                        'serial': 'ADTN23124DA6'},
                'ADTN23124DB0': {       'dev_macs': ['cc:66:18:dd:7e:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU7',
                                        'serial': 'ADTN23124DB0'},
                'ADTN23124DB2': {       'dev_macs': ['cc:66:18:dc:80:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU6',
                                        'serial': 'ADTN23124DB2'},
                'ADTN23124DB7': {       'dev_macs': ['cc:66:18:4b:fa:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU8',
                                        'serial': 'ADTN23124DB7'},
                'ADTN23124DBC': {       'dev_macs': ['38:f8:f6:52:ac:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU9',
                                        'serial': 'ADTN23124DBC'},
                'ADTN23124DC7': {       'dev_macs': ['cc:66:18:dd:75:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU1',
                                        'serial': 'ADTN23124DC7'},
                'ADTN23124DCE': {       'dev_macs': ['cc:66:18:dd:2a:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU11',
                                        'serial': 'ADTN23124DCE'},
                'ADTN23124DD9': {       'dev_macs': ['cc:66:18:dd:7e:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU2',
                                        'serial': 'ADTN23124DD9'},
                'ADTN23124DDA': {       'dev_macs': ['cc:66:18:dc:6e:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU1',
                                        'serial': 'ADTN23124DDA'},
                'ADTN23124DDD': {       'dev_macs': ['cc:66:18:dc:57:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU5',
                                        'serial': 'ADTN23124DDD'},
                'ADTN23124DDF': {       'dev_macs': ['cc:66:18:dd:24:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU16',
                                        'serial': 'ADTN23124DDF'},
                'ADTN23124DE0': {       'dev_macs': ['cc:66:18:dd:8e:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU2',
                                        'serial': 'ADTN23124DE0'},
                'ADTN23124DE4': {       'dev_macs': ['cc:66:18:dc:76:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU6',
                                        'serial': 'ADTN23124DE4'},
                'ADTN23124DE8': {       'dev_macs': ['cc:66:18:50:b5:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU12',
                                        'serial': 'ADTN23124DE8'},
                'ADTN23124DEB': {       'dev_macs': ['38:f8:f6:52:9c:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU3',
                                        'serial': 'ADTN23124DEB'},
                'ADTN23124DF1': {       'dev_macs': ['cc:66:18:dc:7f:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU0',
                                        'serial': 'ADTN23124DF1'},
                'ADTN23124DF3': {       'dev_macs': ['cc:66:18:dc:7d:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU9',
                                        'serial': 'ADTN23124DF3'},
                'ADTN23124DF5': {       'dev_macs': ['cc:66:18:dc:6e:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU19',
                                        'serial': 'ADTN23124DF5'},
                'ADTN23124DF6': {       'dev_macs': ['38:f8:f6:52:ac:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU3',
                                        'serial': 'ADTN23124DF6'},
                'ADTN23124DF7': {       'dev_macs': ['cc:66:18:dd:7f:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU14',
                                        'serial': 'ADTN23124DF7'},
                'ADTN23124DFB': {       'dev_macs': ['cc:66:18:dd:73:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU0',
                                        'serial': 'ADTN23124DFB'},
                'ADTN23124DFC': {       'dev_macs': ['cc:66:18:dc:77:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU5',
                                        'serial': 'ADTN23124DFC'},
                'ADTN23124E04': {       'dev_macs': ['cc:66:18:4c:01:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU4',
                                        'serial': 'ADTN23124E04'},
                'ADTN23124E07': {       'dev_macs': ['cc:66:18:dc:56:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU0',
                                        'serial': 'ADTN23124E07'},
                'ADTN23124E0D': {       'dev_macs': ['cc:66:18:dc:75:a0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124E0D',
                                        'serial': 'ADTN23124E0D'},
                'ADTN23124E14': {       'dev_macs': ['cc:66:18:dc:77:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU1',
                                        'serial': 'ADTN23124E14'},
                'ADTN23124E16': {       'dev_macs': ['cc:66:18:dc:73:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU9',
                                        'serial': 'ADTN23124E16'},
                'ADTN23124E17': {       'dev_macs': ['cc:66:18:dc:7d:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU15',
                                        'serial': 'ADTN23124E17'},
                'ADTN23124E18': {       'dev_macs': ['cc:66:18:dc:88:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU1',
                                        'serial': 'ADTN23124E18'},
                'ADTN23124E19': {       'dev_macs': ['cc:66:18:dd:7b:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU21',
                                        'serial': 'ADTN23124E19'},
                'ADTN23124E1F': {       'dev_macs': ['cc:66:18:dd:8b:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU30',
                                        'serial': 'ADTN23124E1F'},
                'ADTN23124E22': {       'dev_macs': ['cc:66:18:dd:8d:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU12',
                                        'serial': 'ADTN23124E22'},
                'ADTN23124E23': {       'dev_macs': ['cc:66:18:4c:03:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU24',
                                        'serial': 'ADTN23124E23'},
                'ADTN23124E25': {       'dev_macs': ['38:f8:f6:52:af:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU13',
                                        'serial': 'ADTN23124E25'},
                'ADTN23124E26': {       'dev_macs': ['cc:66:18:dd:93:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU3',
                                        'serial': 'ADTN23124E26'},
                'ADTN23124E27': {       'dev_macs': ['cc:66:18:dc:70:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU25',
                                        'serial': 'ADTN23124E27'},
                'ADTN23124E29': {       'dev_macs': ['cc:66:18:dd:68:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU7',
                                        'serial': 'ADTN23124E29'},
                'ADTN23124E2A': {       'dev_macs': ['cc:66:18:d6:08:e0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124E2A',
                                        'serial': 'ADTN23124E2A'},
                'ADTN23124E2B': {       'dev_macs': ['38:f8:f6:52:b2:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU10',
                                        'serial': 'ADTN23124E2B'},
                'ADTN23124E2D': {       'dev_macs': ['cc:66:18:dd:8a:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU15',
                                        'serial': 'ADTN23124E2D'},
                'ADTN23124E30': {       'dev_macs': ['cc:66:18:dc:79:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU5',
                                        'serial': 'ADTN23124E30'},
                'ADTN23124E33': {       'dev_macs': ['cc:66:18:d6:08:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU17',
                                        'serial': 'ADTN23124E33'},
                'ADTN23124E35': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU3',
                                        'serial': 'ADTN23124E35'},
                'ADTN23124E37': {       'dev_macs': ['cc:66:18:dc:6e:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU11',
                                        'serial': 'ADTN23124E37'},
                'ADTN23124E3A': {       'dev_macs': ['cc:66:18:dd:8a:b1'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU7',
                                        'serial': 'ADTN23124E3A'},
                'ADTN23124E3E': {       'dev_macs': ['cc:66:18:dc:70:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU4',
                                        'serial': 'ADTN23124E3E'},
                'ADTN23124E3F': {       'dev_macs': ['cc:66:18:dd:83:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU23',
                                        'serial': 'ADTN23124E3F'},
                'ADTN23124E40': {       'dev_macs': ['cc:66:18:dd:8a:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU0',
                                        'serial': 'ADTN23124E40'},
                'ADTN23124E43': {       'dev_macs': ['cc:66:18:d6:03:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU18',
                                        'serial': 'ADTN23124E43'},
                'ADTN23124E45': {       'dev_macs': ['cc:66:18:dc:83:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU0',
                                        'serial': 'ADTN23124E45'},
                'ADTN23124E4B': {       'dev_macs': ['cc:66:18:dd:8f:b0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124E4B',
                                        'serial': 'ADTN23124E4B'},
                'ADTN23124E4D': {       'dev_macs': ['cc:66:18:d6:08:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU4',
                                        'serial': 'ADTN23124E4D'},
                'ADTN23124E50': {       'dev_macs': ['cc:66:18:d6:00:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU11',
                                        'serial': 'ADTN23124E50'},
                'ADTN23124E52': {       'dev_macs': ['cc:66:18:dd:82:c0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124E52',
                                        'serial': 'ADTN23124E52'},
                'ADTN23124E57': {       'dev_macs': ['38:f8:f6:52:ab:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU9',
                                        'serial': 'ADTN23124E57'},
                'ADTN23124E5B': {       'dev_macs': ['cc:66:18:4c:02:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU17',
                                        'serial': 'ADTN23124E5B'},
                'ADTN23124E64': {       'dev_macs': ['cc:66:18:4b:fa:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU26',
                                        'serial': 'ADTN23124E64'},
                'ADTN23124E67': {       'dev_macs': ['cc:66:18:dd:82:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU22',
                                        'serial': 'ADTN23124E67'},
                'ADTN23124E6A': {       'dev_macs': ['cc:66:18:d6:3a:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU5',
                                        'serial': 'ADTN23124E6A'},
                'ADTN23124E6B': {       'dev_macs': ['cc:66:18:dc:87:c0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124E6B',
                                        'serial': 'ADTN23124E6B'},
                'ADTN23124E71': {       'dev_macs': ['cc:66:18:dd:84:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU0',
                                        'serial': 'ADTN23124E71'},
                'ADTN23124E73': {       'dev_macs': ['cc:66:18:50:52:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU13',
                                        'serial': 'ADTN23124E73'},
                'ADTN23124E7A': {       'dev_macs': ['cc:66:18:dd:7e:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU1',
                                        'serial': 'ADTN23124E7A'},
                'ADTN23124E84': {       'dev_macs': ['cc:66:18:dc:71:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU5',
                                        'serial': 'ADTN23124E84'},
                'ADTN23124E86': {       'dev_macs': ['cc:66:18:dc:6d:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU8',
                                        'serial': 'ADTN23124E86'},
                'ADTN23124E8A': {       'dev_macs': ['cc:66:18:51:30:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU14',
                                        'serial': 'ADTN23124E8A'},
                'ADTN23124E92': {       'dev_macs': ['38:f8:f6:52:a6:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU6',
                                        'serial': 'ADTN23124E92'},
                'ADTN23124EA4': {       'dev_macs': ['cc:66:18:4c:8d:d0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124EA4',
                                        'serial': 'ADTN23124EA4'},
                'ADTN23124EA5': {       'dev_macs': ['cc:66:18:dc:8a:10'],
                                        'ip_address': None,
                                        'name': 'ADTN23124EA5',
                                        'serial': 'ADTN23124EA5'},
                'ADTN23124ED3': {       'dev_macs': ['cc:66:18:d6:32:11'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU7',
                                        'serial': 'ADTN23124ED3'},
                'ADTN23124ED9': {       'dev_macs': ['cc:66:18:dd:7c:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU3',
                                        'serial': 'ADTN23124ED9'},
                'ADTN23124EDF': {       'dev_macs': ['cc:66:18:dc:81:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU3',
                                        'serial': 'ADTN23124EDF'},
                'ADTN23124EFA': {       'dev_macs': ['cc:66:18:db:76:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU2',
                                        'serial': 'ADTN23124EFA'},
                'ADTN23124F06': {       'dev_macs': ['38:f8:f6:52:ab:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU10',
                                        'serial': 'ADTN23124F06'},
                'ADTN23124F14': {       'dev_macs': ['cc:66:18:dc:6b:c0'],
                                        'ip_address': None,
                                        'name': 'ADTN23124F14',
                                        'serial': 'ADTN23124F14'},
                'ADTN23124F2B': {       'dev_macs': ['cc:66:18:dc:81:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU10',
                                        'serial': 'ADTN23124F2B'},
                'ADTN23124F2D': {       'dev_macs': ['38:f8:f6:52:af:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU0',
                                        'serial': 'ADTN23124F2D'},
                'ADTN23124F30': {       'dev_macs': ['cc:66:18:dc:75:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU6',
                                        'serial': 'ADTN23124F30'},
                'ADTN23124F34': {       'dev_macs': ['38:f8:f6:52:ac:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU8',
                                        'serial': 'ADTN23124F34'},
                'ADTN23124F37': {       'dev_macs': ['cc:66:18:dd:89:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU5',
                                        'serial': 'ADTN23124F37'},
                'ADTN23124F3C': {       'dev_macs': ['cc:66:18:dc:78:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU4',
                                        'serial': 'ADTN23124F3C'},
                'ADTN23124F74': {       'dev_macs': ['cc:66:18:dd:83:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU16',
                                        'serial': 'ADTN23124F74'},
                'ADTN23124F99': {       'dev_macs': ['cc:66:18:dc:75:51'],
                                        'ip_address': None,
                                        'name': 'ADTN23124F99',
                                        'serial': 'ADTN23124F99'},
                'ADTN2312501D': {       'dev_macs': ['cc:66:18:dc:73:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU18',
                                        'serial': 'ADTN2312501D'},
                'ADTN23125020': {       'dev_macs': ['cc:66:18:dd:7d:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU4',
                                        'serial': 'ADTN23125020'},
                'ADTN23125026': {       'dev_macs': ['cc:66:18:dd:79:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU1',
                                        'serial': 'ADTN23125026'},
                'ADTN23125037': {       'dev_macs': ['38:f8:f6:52:ac:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU13',
                                        'serial': 'ADTN23125037'},
                'ADTN23125055': {       'dev_macs': ['cc:66:18:dc:81:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU2',
                                        'serial': 'ADTN23125055'},
                'ADTN23125061': {       'dev_macs': ['cc:66:18:4c:01:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU0',
                                        'serial': 'ADTN23125061'},
                'ADTN23125065': {       'dev_macs': ['cc:66:18:50:4f:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU15',
                                        'serial': 'ADTN23125065'},
                'ADTN2312506C': {       'dev_macs': ['cc:66:18:51:42:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU19',
                                        'serial': 'ADTN2312506C'},
                'ADTN23125088': {       'dev_macs': ['cc:66:18:dd:8b:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU3',
                                        'serial': 'ADTN23125088'},
                'ADTN2312509E': {       'dev_macs': ['cc:66:18:d6:08:f0'],
                                        'ip_address': None,
                                        'name': 'ADTN2312509E',
                                        'serial': 'ADTN2312509E'},
                'ADTN231250BB': {       'dev_macs': ['cc:66:18:db:65:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU13',
                                        'serial': 'ADTN231250BB'},
                'ADTN231250BF': {       'dev_macs': ['cc:66:18:d6:08:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU10',
                                        'serial': 'ADTN231250BF'},
                'ADTN231250C0': {       'dev_macs': ['cc:66:18:d6:09:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU9',
                                        'serial': 'ADTN231250C0'},
                'ADTN231250C3': {       'dev_macs': ['cc:66:18:dc:77:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU23',
                                        'serial': 'ADTN231250C3'},
                'ADTN231250C6': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU2',
                                        'serial': 'ADTN231250C6'},
                'ADTN231250C8': {       'dev_macs': ['cc:66:18:51:50:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU16',
                                        'serial': 'ADTN231250C8'},
                'ADTN231250CC': {       'dev_macs': ['cc:66:18:dd:7b:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU6',
                                        'serial': 'ADTN231250CC'},
                'ADTN231250CD': {       'dev_macs': ['cc:66:18:50:63:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU11',
                                        'serial': 'ADTN231250CD'},
                'ADTN231250D1': {       'dev_macs': ['cc:66:18:50:f5:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU14',
                                        'serial': 'ADTN231250D1'},
                'ADTN231250DD': {       'dev_macs': ['cc:66:18:4c:02:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU12',
                                        'serial': 'ADTN231250DD'},
                'ADTN231250FE': {       'dev_macs': ['cc:66:18:d6:02:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU8',
                                        'serial': 'ADTN231250FE'},
                'ADTN23125103': {       'dev_macs': ['cc:66:18:d6:04:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU9',
                                        'serial': 'ADTN23125103'},
                'ADTN2312510B': {       'dev_macs': ['cc:66:18:dc:88:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU20',
                                        'serial': 'ADTN2312510B'},
                'ADTN2312510C': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU11',
                                        'serial': 'ADTN2312510C'},
                'ADTN23125111': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU7',
                                        'serial': 'ADTN23125111'},
                'ADTN23125116': {       'dev_macs': ['cc:66:18:dc:79:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU14',
                                        'serial': 'ADTN23125116'},
                'ADTN2312511D': {       'dev_macs': ['cc:66:18:d6:0b:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU12',
                                        'serial': 'ADTN2312511D'},
                'ADTN23125126': {       'dev_macs': ['cc:66:18:dc:6a:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU18',
                                        'serial': 'ADTN23125126'},
                'ADTN23125129': {       'dev_macs': ['cc:66:18:d6:09:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU16',
                                        'serial': 'ADTN23125129'},
                'ADTN2312512C': {       'dev_macs': ['cc:66:18:dc:6c:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU16',
                                        'serial': 'ADTN2312512C'},
                'ADTN2312512E': {       'dev_macs': ['cc:66:18:dd:8e:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU10',
                                        'serial': 'ADTN2312512E'},
                'ADTN2312512F': {       'dev_macs': ['cc:66:18:4b:fa:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU15',
                                        'serial': 'ADTN2312512F'},
                'ADTN23125133': {       'dev_macs': ['cc:66:18:db:76:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU18',
                                        'serial': 'ADTN23125133'},
                'ADTN23125135': {       'dev_macs': ['cc:66:18:50:3e:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU26',
                                        'serial': 'ADTN23125135'},
                'ADTN2312513A': {       'dev_macs': ['cc:66:18:50:50:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU9',
                                        'serial': 'ADTN2312513A'},
                'ADTN23125143': {       'dev_macs': ['cc:66:18:d6:07:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU8',
                                        'serial': 'ADTN23125143'},
                'ADTN23125146': {       'dev_macs': ['cc:66:18:dd:91:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU7',
                                        'serial': 'ADTN23125146'},
                'ADTN23125148': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU12',
                                        'serial': 'ADTN23125148'},
                'ADTN23125149': {       'dev_macs': ['cc:66:18:d6:36:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU1',
                                        'serial': 'ADTN23125149'},
                'ADTN2312514A': {       'dev_macs': ['cc:66:18:dc:83:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU18',
                                        'serial': 'ADTN2312514A'},
                'ADTN2312514B': {       'dev_macs': ['cc:66:18:d6:35:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU12',
                                        'serial': 'ADTN2312514B'},
                'ADTN2312514C': {       'dev_macs': ['cc:66:18:50:87:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU21',
                                        'serial': 'ADTN2312514C'},
                'ADTN2312514D': {       'dev_macs': ['38:f8:f6:52:b3:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU9',
                                        'serial': 'ADTN2312514D'},
                'ADTN2312514E': {       'dev_macs': ['cc:66:18:dd:8a:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU11',
                                        'serial': 'ADTN2312514E'},
                'ADTN2312514F': {       'dev_macs': ['cc:66:18:51:24:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU21',
                                        'serial': 'ADTN2312514F'},
                'ADTN23125150': {       'dev_macs': ['cc:66:18:d6:36:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x14-ONU0',
                                        'serial': 'ADTN23125150'},
                'ADTN23125153': {       'dev_macs': ['38:f8:f6:52:a8:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU19',
                                        'serial': 'ADTN23125153'},
                'ADTN23125158': {       'dev_macs': ['cc:66:18:dd:92:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU11',
                                        'serial': 'ADTN23125158'},
                'ADTN2312515B': {       'dev_macs': ['cc:66:18:dc:7f:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU12',
                                        'serial': 'ADTN2312515B'},
                'ADTN23125160': {       'dev_macs': ['cc:66:18:dd:69:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU4',
                                        'serial': 'ADTN23125160'},
                'ADTN23125161': {       'dev_macs': ['cc:66:18:dc:65:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU6',
                                        'serial': 'ADTN23125161'},
                'ADTN23125166': {       'dev_macs': ['cc:66:18:dd:78:e0'],
                                        'ip_address': None,
                                        'name': 'ADTN23125166',
                                        'serial': 'ADTN23125166'},
                'ADTN23125171': {       'dev_macs': ['cc:66:18:4c:04:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU2',
                                        'serial': 'ADTN23125171'},
                'ADTN2312517D': {       'dev_macs': ['cc:66:18:dd:78:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU11',
                                        'serial': 'ADTN2312517D'},
                'ADTN2312517F': {       'dev_macs': ['cc:66:18:dd:23:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU11',
                                        'serial': 'ADTN2312517F'},
                'ADTN2312518D': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU23',
                                        'serial': 'ADTN2312518D'},
                'ADTN2312518E': {       'dev_macs': ['cc:66:18:db:74:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU10',
                                        'serial': 'ADTN2312518E'},
                'ADTN23125190': {       'dev_macs': ['cc:66:18:50:75:70'],
                                        'ip_address': None,
                                        'name': 'ADTN23125190',
                                        'serial': 'ADTN23125190'},
                'ADTN23125194': {       'dev_macs': ['cc:66:18:50:81:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU10',
                                        'serial': 'ADTN23125194'},
                'ADTN23125195': {       'dev_macs': ['cc:66:18:50:b8:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU5',
                                        'serial': 'ADTN23125195'},
                'ADTN231251AA': {       'dev_macs': ['cc:66:18:d6:37:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU20',
                                        'serial': 'ADTN231251AA'},
                'ADTN231251AB': {       'dev_macs': ['cc:66:18:4c:a3:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU20',
                                        'serial': 'ADTN231251AB'},
                'ADTN231251B0': {       'dev_macs': ['cc:66:18:dd:8d:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU15',
                                        'serial': 'ADTN231251B0'},
                'ADTN231251B1': {       'dev_macs': ['cc:66:18:dd:6a:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU13',
                                        'serial': 'ADTN231251B1'},
                'ADTN231251E2': {       'dev_macs': ['38:f8:f6:52:71:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU2',
                                        'serial': 'ADTN231251E2'},
                'ADTN231251EB': {       'dev_macs': ['cc:66:18:dc:56:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU5',
                                        'serial': 'ADTN231251EB'},
                'ADTN231251EE': {       'dev_macs': ['cc:66:18:dd:93:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU13',
                                        'serial': 'ADTN231251EE'},
                'ADTN231251F6': {       'dev_macs': ['cc:66:18:d6:08:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU6',
                                        'serial': 'ADTN231251F6'},
                'ADTN231251F7': {       'dev_macs': ['cc:66:18:dc:79:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU4',
                                        'serial': 'ADTN231251F7'},
                'ADTN23125206': {       'dev_macs': ['cc:66:18:50:6e:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU8',
                                        'serial': 'ADTN23125206'},
                'ADTN2314428A': {       'dev_macs': ['cc:66:18:dc:97:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU20',
                                        'serial': 'ADTN2314428A'},
                'ADTN23144666': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU25',
                                        'serial': 'ADTN23144666'},
                'ADTN23144681': {       'dev_macs': ['cc:66:18:d6:3b:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU5',
                                        'serial': 'ADTN23144681'},
                'ADTN2314468C': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU17',
                                        'serial': 'ADTN2314468C'},
                'ADTN23144699': {       'dev_macs': ['cc:66:18:dc:78:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU18',
                                        'serial': 'ADTN23144699'},
                'ADTN2314469E': {       'dev_macs': ['cc:66:18:dd:87:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU24',
                                        'serial': 'ADTN2314469E'},
                'ADTN231446A3': {       'dev_macs': ['38:f8:f6:52:a9:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU13',
                                        'serial': 'ADTN231446A3'},
                'ADTN231446A5': {       'dev_macs': ['cc:66:18:d6:3a:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU17',
                                        'serial': 'ADTN231446A5'},
                'ADTN231446B4': {       'dev_macs': ['cc:66:18:50:23:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU14',
                                        'serial': 'ADTN231446B4'},
                'ADTN231446B8': {       'dev_macs': ['38:f8:f6:52:a8:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU13',
                                        'serial': 'ADTN231446B8'},
                'ADTN231446BC': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU12',
                                        'serial': 'ADTN231446BC'},
                'ADTN231446C0': {       'dev_macs': ['cc:66:18:50:5b:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU18',
                                        'serial': 'ADTN231446C0'},
                'ADTN231446CB': {       'dev_macs': ['cc:66:18:dd:6a:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU14',
                                        'serial': 'ADTN231446CB'},
                'ADTN231446CC': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU14',
                                        'serial': 'ADTN231446CC'},
                'ADTN231446CE': {       'dev_macs': ['cc:66:18:dc:89:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU15',
                                        'serial': 'ADTN231446CE'},
                'ADTN23144702': {       'dev_macs': ['cc:66:18:d6:08:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU11',
                                        'serial': 'ADTN23144702'},
                'ADTN23144707': {       'dev_macs': ['cc:66:18:dd:52:20'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU10',
                                        'serial': 'ADTN23144707'},
                'ADTN23144714': {       'dev_macs': ['cc:66:18:dd:8d:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x14-ONU1',
                                        'serial': 'ADTN23144714'},
                'ADTN23144735': {       'dev_macs': ['cc:66:18:dc:84:51'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU25',
                                        'serial': 'ADTN23144735'},
                'ADTN23144748': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU14',
                                        'serial': 'ADTN23144748'},
                'ADTN23144759': {       'dev_macs': ['cc:66:18:4b:fc:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU15',
                                        'serial': 'ADTN23144759'},
                'ADTN23144773': {       'dev_macs': ['cc:66:18:dd:29:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU17',
                                        'serial': 'ADTN23144773'},
                'ADTN2314477A': {       'dev_macs': ['cc:66:18:dd:8a:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU26',
                                        'serial': 'ADTN2314477A'},
                'ADTN23144796': {       'dev_macs': ['cc:66:18:50:ce:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU14',
                                        'serial': 'ADTN23144796'},
                'ADTN231447A4': {       'dev_macs': ['cc:66:18:dc:86:d1'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU31',
                                        'serial': 'ADTN231447A4'},
                'ADTN231447A5': {       'dev_macs': ['cc:66:18:50:58:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU24',
                                        'serial': 'ADTN231447A5'},
                'ADTN231447AB': {       'dev_macs': ['cc:66:18:dc:b3:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU28',
                                        'serial': 'ADTN231447AB'},
                'ADTN231447AC': {       'dev_macs': ['cc:66:18:dc:b3:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU23',
                                        'serial': 'ADTN231447AC'},
                'ADTN231447AE': {       'dev_macs': ['cc:66:18:d6:03:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU15',
                                        'serial': 'ADTN231447AE'},
                'ADTN231447BD': {       'dev_macs': ['38:f8:f6:52:a8:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU12',
                                        'serial': 'ADTN231447BD'},
                'ADTN231447DA': {       'dev_macs': ['38:f8:f6:52:ab:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU24',
                                        'serial': 'ADTN231447DA'},
                'ADTN231447E3': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU15',
                                        'serial': 'ADTN231447E3'},
                'ADTN231447F3': {       'dev_macs': ['cc:66:18:dd:89:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU29',
                                        'serial': 'ADTN231447F3'},
                'ADTN231447F9': {       'dev_macs': ['38:f8:f6:52:a8:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU4',
                                        'serial': 'ADTN231447F9'},
                'ADTN2314480D': {       'dev_macs': ['cc:66:18:d6:0a:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU27',
                                        'serial': 'ADTN2314480D'},
                'ADTN2314483B': {       'dev_macs': ['cc:66:18:d6:07:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU13',
                                        'serial': 'ADTN2314483B'},
                'ADTN23144861': {       'dev_macs': ['cc:66:18:50:9a:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU15',
                                        'serial': 'ADTN23144861'},
                'ADTN2314486C': {       'dev_macs': ['cc:66:18:dc:84:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU16',
                                        'serial': 'ADTN2314486C'},
                'ADTN23144883': {       'dev_macs': ['cc:66:18:d6:0a:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU9',
                                        'serial': 'ADTN23144883'},
                'ADTN2314489C': {       'dev_macs': ['cc:66:18:d6:0c:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU6',
                                        'serial': 'ADTN2314489C'},
                'ADTN2314489D': {       'dev_macs': ['cc:66:18:50:6e:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU8',
                                        'serial': 'ADTN2314489D'},
                'ADTN231448B4': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU22',
                                        'serial': 'ADTN231448B4'},
                'ADTN231448B8': {       'dev_macs': ['cc:66:18:dd:8c:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU3',
                                        'serial': 'ADTN231448B8'},
                'ADTN231448BA': {       'dev_macs': ['cc:66:18:d6:38:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU7',
                                        'serial': 'ADTN231448BA'},
                'ADTN231448C9': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU18',
                                        'serial': 'ADTN231448C9'},
                'ADTN231448D1': {       'dev_macs': ['cc:66:18:50:b3:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU15',
                                        'serial': 'ADTN231448D1'},
                'ADTN231448D5': {       'dev_macs': ['cc:66:18:d6:0c:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU21',
                                        'serial': 'ADTN231448D5'},
                'ADTN231448FC': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU2',
                                        'serial': 'ADTN231448FC'},
                'ADTN231448FE': {       'dev_macs': ['cc:66:18:d6:0b:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU16',
                                        'serial': 'ADTN231448FE'},
                'ADTN23144916': {       'dev_macs': ['38:f8:f6:52:a8:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU19',
                                        'serial': 'ADTN23144916'},
                'ADTN23144A15': {       'dev_macs': ['cc:66:18:50:6e:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU7',
                                        'serial': 'ADTN23144A15'},
                'ADTN23144A2B': {       'dev_macs': ['cc:66:18:dc:89:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU22',
                                        'serial': 'ADTN23144A2B'},
                'ADTN23144A53': {       'dev_macs': ['cc:66:18:d6:38:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU4',
                                        'serial': 'ADTN23144A53'},
                'ADTN23144A5A': {       'dev_macs': ['cc:66:18:4c:03:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU3',
                                        'serial': 'ADTN23144A5A'},
                'ADTN23144A5C': {       'dev_macs': ['cc:66:18:4c:00:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU6',
                                        'serial': 'ADTN23144A5C'},
                'ADTN23144A69': {       'dev_macs': ['cc:66:18:51:04:e0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU32',
                                        'serial': 'ADTN23144A69'},
                'ADTN23144A70': {       'dev_macs': ['cc:66:18:50:d0:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU24',
                                        'serial': 'ADTN23144A70'},
                'ADTN23144A84': {       'dev_macs': ['cc:66:18:d6:34:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU5',
                                        'serial': 'ADTN23144A84'},
                'ADTN23144A86': {       'dev_macs': [       '08:55:31:e1:ea:4b',
                                                            '18:e8:29:34:ed:33',
                                                            '18:e8:29:36:d4:71',
                                                            '18:e8:29:3a:1a:3d',
                                                            '18:e8:29:8e:ed:6f',
                                                            '18:e8:29:8e:ed:78',
                                                            '18:e8:29:8e:f1:06',
                                                            '24:5a:4c:e4:c8:a1',
                                                            '74:83:c2:a6:5a:93',
                                                            '74:83:c2:a6:7f:9f',
                                                            '74:83:c2:a6:86:4e',
                                                            '74:83:c2:e8:a2:61',
                                                            '74:83:c2:ea:a6:01',
                                                            '74:83:c2:ea:a6:28',
                                                            '74:83:c2:ec:11:8b',
                                                            '74:ac:b9:80:d4:53',
                                                            '94:a6:7e:dd:ad:81',
                                                            'b4:fb:e4:5a:4f:8e',
                                                            'b8:94:70:f8:2b:2a',
                                                            'dc:2c:6e:dd:2e:26',
                                                            'f4:92:bf:38:a5:74',
                                                            'f4:92:bf:38:a7:25',
                                                            'f4:92:bf:38:a7:c2',
                                                            'f4:92:bf:d6:a8:ac'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x9-ONU25',
                                        'serial': 'ADTN23144A86'},
                'ADTN23144A97': {       'dev_macs': ['cc:66:18:d6:37:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU16',
                                        'serial': 'ADTN23144A97'},
                'ADTN23144A9B': {       'dev_macs': ['cc:66:18:51:21:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU11',
                                        'serial': 'ADTN23144A9B'},
                'ADTN23144A9C': {       'dev_macs': ['cc:66:18:dd:8a:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU16',
                                        'serial': 'ADTN23144A9C'},
                'ADTN23144AB0': {       'dev_macs': ['cc:66:18:4c:01:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU26',
                                        'serial': 'ADTN23144AB0'},
                'ADTN23144ACF': {       'dev_macs': ['38:f8:f6:52:a1:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU12',
                                        'serial': 'ADTN23144ACF'},
                'ADTN23144B20': {       'dev_macs': ['cc:66:18:dc:aa:e1'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU31',
                                        'serial': 'ADTN23144B20'},
                'ADTN23144B24': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU18',
                                        'serial': 'ADTN23144B24'},
                'ADTN23144B2C': {       'dev_macs': ['cc:66:18:d6:3c:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU17',
                                        'serial': 'ADTN23144B2C'},
                'ADTN23144B3A': {       'dev_macs': ['cc:66:18:db:7e:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU20',
                                        'serial': 'ADTN23144B3A'},
                'ADTN23144B3F': {       'dev_macs': ['cc:66:18:50:63:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU8',
                                        'serial': 'ADTN23144B3F'},
                'ADTN23144B4D': {       'dev_macs': ['cc:66:18:50:9c:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU21',
                                        'serial': 'ADTN23144B4D'},
                'ADTN23144B58': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU19',
                                        'serial': 'ADTN23144B58'},
                'ADTN23144B5E': {       'dev_macs': ['cc:66:18:51:31:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU10',
                                        'serial': 'ADTN23144B5E'},
                'ADTN23144BF7': {       'dev_macs': ['cc:66:18:4c:55:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU16',
                                        'serial': 'ADTN23144BF7'},
                'ADTN23144C10': {       'dev_macs': ['cc:66:18:d6:36:70'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU21',
                                        'serial': 'ADTN23144C10'},
                'ADTN23144C17': {       'dev_macs': ['cc:66:18:51:04:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU25',
                                        'serial': 'ADTN23144C17'},
                'ADTN23144C33': {       'dev_macs': ['cc:66:18:dd:91:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x14-ONU2',
                                        'serial': 'ADTN23144C33'},
                'ADTN23144C46': {       'dev_macs': ['cc:66:18:dc:75:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU20',
                                        'serial': 'ADTN23144C46'},
                'ADTN23144C4E': {       'dev_macs': ['cc:66:18:50:a9:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU21',
                                        'serial': 'ADTN23144C4E'},
                'ADTN23144C8B': {       'dev_macs': ['cc:66:18:50:62:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU9',
                                        'serial': 'ADTN23144C8B'},
                'ADTN23144CC3': {       'dev_macs': ['cc:66:18:dd:8a:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU19',
                                        'serial': 'ADTN23144CC3'},
                'ADTN23144CD5': {       'dev_macs': ['cc:66:18:dd:93:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU18',
                                        'serial': 'ADTN23144CD5'},
                'ADTN23144D50': {       'dev_macs': ['cc:66:18:d6:0c:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU17',
                                        'serial': 'ADTN23144D50'},
                'ADTN23144D99': {       'dev_macs': ['cc:66:18:50:3f:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU7',
                                        'serial': 'ADTN23144D99'},
                'ADTN23144DDF': {       'dev_macs': ['cc:66:18:d6:37:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU16',
                                        'serial': 'ADTN23144DDF'},
                'ADTN23144EC8': {       'dev_macs': ['cc:66:18:50:ad:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU24',
                                        'serial': 'ADTN23144EC8'},
                'ADTN23144ED2': {       'dev_macs': ['cc:66:18:50:7b:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU30',
                                        'serial': 'ADTN23144ED2'},
                'ADTN23144ED3': {       'dev_macs': ['cc:66:18:50:b7:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x8-ONU34',
                                        'serial': 'ADTN23144ED3'},
                'ADTN23144EDF': {       'dev_macs': ['cc:66:18:50:39:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU19',
                                        'serial': 'ADTN23144EDF'},
                'ADTN23144EE0': {       'dev_macs': ['cc:66:18:50:b6:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU23',
                                        'serial': 'ADTN23144EE0'},
                'ADTN23144EE3': {       'dev_macs': ['cc:66:18:dc:a8:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU28',
                                        'serial': 'ADTN23144EE3'},
                'ADTN23144EFB': {       'dev_macs': ['cc:66:18:dc:b4:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU17',
                                        'serial': 'ADTN23144EFB'},
                'ADTN23144EFD': {       'dev_macs': ['cc:66:18:dc:71:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x4-ONU20',
                                        'serial': 'ADTN23144EFD'},
                'ADTN23144EFF': {       'dev_macs': ['cc:66:18:50:4a:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU20',
                                        'serial': 'ADTN23144EFF'},
                'ADTN23144F00': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU25',
                                        'serial': 'ADTN23144F00'},
                'ADTN23145120': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU20',
                                        'serial': 'ADTN23145120'},
                'ADTN24314195': {       'dev_macs': ['cc:66:18:d6:39:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU10',
                                        'serial': 'ADTN24314195'},
                'ADTN243141A3': {       'dev_macs': ['cc:66:18:50:e6:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU22',
                                        'serial': 'ADTN243141A3'},
                'ADTN243141AC': {       'dev_macs': ['cc:66:18:dd:81:80'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU19',
                                        'serial': 'ADTN243141AC'},
                'ADTN24314314': {       'dev_macs': ['cc:66:18:51:27:10'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU22',
                                        'serial': 'ADTN24314314'},
                'ADTN243144FB': {       'dev_macs': ['cc:66:18:50:fd:b0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x6-ONU6',
                                        'serial': 'ADTN243144FB'},
                'ADTN24314766': {       'dev_macs': ['cc:66:18:50:3e:40'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU28',
                                        'serial': 'ADTN24314766'},
                'ADTN24363AE7': {       'dev_macs': ['38:f8:f6:52:ab:a0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU29',
                                        'serial': 'ADTN24363AE7'},
                'ADTN24363B6B': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x1-ONU27',
                                        'serial': 'ADTN24363B6B'},
                'ADTN24363BBA': {       'dev_macs': ['cc:66:18:50:b6:c0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x12-ONU18',
                                        'serial': 'ADTN24363BBA'},
                'ADTN24363C49': {       'dev_macs': ['cc:66:18:51:2e:00'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU31',
                                        'serial': 'ADTN24363C49'},
                'ADTN24363CC6': {       'dev_macs': ['cc:66:18:50:62:90'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x2-ONU13',
                                        'serial': 'ADTN24363CC6'},
                'ADTN24363D3A': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x5-ONU21',
                                        'serial': 'ADTN24363D3A'},
                'ADTN24363D4D': {       'dev_macs': ['cc:66:18:50:6a:d0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x11-ONU32',
                                        'serial': 'ADTN24363D4D'},
                'ADTN24363DB9': {       'dev_macs': ['cc:66:18:50:b1:30'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU16',
                                        'serial': 'ADTN24363DB9'},
                'ADTN24363DC9': {       'dev_macs': ['cc:66:18:50:ac:60'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU28',
                                        'serial': 'ADTN24363DC9'},
                'ADTN243896F9': {       'dev_macs': ['cc:66:18:51:25:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x3-ONU30',
                                        'serial': 'ADTN243896F9'},
                'ADTN244917A4': {       'dev_macs': ['cc:66:18:50:9b:50'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x10-ONU22',
                                        'serial': 'ADTN244917A4'},
                'ADTN251521A2': {       'dev_macs': ['cc:66:18:50:ba:f0'],
                                        'ip_address': None,
                                        'name': 'IndianBayou-x13-ONU17',
                                        'serial': 'ADTN251521A2'},
                'CXNK00A4305B': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'IndianBayou-x7-ONU19',
                                        'serial': 'CXNK00A4305B'}
        }
        exp_ne_msg = {
                'data': {       'cpu': 39.37,
                                'interfaces': [       {       'admin_up': True,
                                                              'id': '0/1',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/1',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8x',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/1',
                                                                                      'id': 'xgspon '
                                                                                            '0/1',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/1',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8x',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/10',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dc:88:30'],
                                                                                       'in_nucast_pkts': '14488',
                                                                                       'in_octets': '10550726467',
                                                                                       'in_ucast_pkts': '78959047',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '372201121743',
                                                                                       'out_ucast_pkts': '254564839',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDY4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:88:a0'],
                                                                                       'in_nucast_pkts': '14491',
                                                                                       'in_octets': '2971603812',
                                                                                       'in_ucast_pkts': '6478397',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '32270863710',
                                                                                       'out_ucast_pkts': '26537850',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTE4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:81:d0'],
                                                                                       'in_nucast_pkts': '114032',
                                                                                       'in_octets': '34146902875',
                                                                                       'in_ucast_pkts': '130691237',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9654',
                                                                                       'out_octets': '471523767208',
                                                                                       'out_ucast_pkts': '370552154',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDU1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:81:e0'],
                                                                                       'in_nucast_pkts': '114027',
                                                                                       'in_octets': '48597335655',
                                                                                       'in_ucast_pkts': '274600145',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9650',
                                                                                       'out_octets': '1294769624337',
                                                                                       'out_ucast_pkts': '894488302',
                                                                                       'poller_hash': 'QURUTjIzMTI0NjUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7d:00'],
                                                                                       'in_nucast_pkts': '21619',
                                                                                       'in_octets': '5281560674',
                                                                                       'in_ucast_pkts': '11555894',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1828',
                                                                                       'out_octets': '37139094766',
                                                                                       'out_ucast_pkts': '31182483',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDIwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:74:90'],
                                                                                       'in_nucast_pkts': '182420',
                                                                                       'in_octets': '61330425016',
                                                                                       'in_ucast_pkts': '233445066',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15381',
                                                                                       'out_octets': '863383765062',
                                                                                       'out_ucast_pkts': '644759562',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDdFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:80:50'],
                                                                                       'in_nucast_pkts': '14474',
                                                                                       'in_octets': '9698208206',
                                                                                       'in_ucast_pkts': '63773063',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '245329448787',
                                                                                       'out_ucast_pkts': '170570158',
                                                                                       'poller_hash': 'QURUTjIzMTI0REIyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:68:b0'],
                                                                                       'in_nucast_pkts': '14492',
                                                                                       'in_octets': '2697660819',
                                                                                       'in_ucast_pkts': '10709779',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '43209532271',
                                                                                       'out_ucast_pkts': '33612964',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTI5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7c:d0'],
                                                                                       'in_nucast_pkts': '14496',
                                                                                       'in_octets': '6562879945',
                                                                                       'in_ucast_pkts': '44066207',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '250735238169',
                                                                                       'out_ucast_pkts': '178667406',
                                                                                       'poller_hash': 'QURUTjIzMTI0MUQwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:d0'],
                                                                                       'in_nucast_pkts': '14611',
                                                                                       'in_octets': '10465029520',
                                                                                       'in_ucast_pkts': '32886879',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '178250999676',
                                                                                       'out_ucast_pkts': '132715489',
                                                                                       'poller_hash': 'QURUTjIzMTI0REJDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:b2:80'],
                                                                                       'in_nucast_pkts': '14258',
                                                                                       'in_octets': '4222787347',
                                                                                       'in_ucast_pkts': '10744367',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '31820503360',
                                                                                       'out_ucast_pkts': '28427247',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTJCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:23:20'],
                                                                                       'in_nucast_pkts': '53346',
                                                                                       'in_octets': '71092519004',
                                                                                       'in_ucast_pkts': '217419329',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4406',
                                                                                       'out_octets': '911286037291',
                                                                                       'out_ucast_pkts': '655351516',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTdGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:7f:60'],
                                                                                       'in_nucast_pkts': '14487',
                                                                                       'in_octets': '22467858323',
                                                                                       'in_ucast_pkts': '66788495',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '192046021310',
                                                                                       'out_ucast_pkts': '146416225',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTVCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:6a:40'],
                                                                                       'in_nucast_pkts': '14495',
                                                                                       'in_octets': '10077191716',
                                                                                       'in_ucast_pkts': '46633002',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '121818231744',
                                                                                       'out_ucast_pkts': '104751984',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUIxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:30:c0'],
                                                                                       'in_nucast_pkts': '112169',
                                                                                       'in_octets': '21016518482',
                                                                                       'in_ucast_pkts': '99931401',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3740',
                                                                                       'out_octets': '441127091202',
                                                                                       'out_ucast_pkts': '309978043',
                                                                                       'poller_hash': 'QURUTjIzMTI0RThBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:4f:90'],
                                                                                       'in_nucast_pkts': '112172',
                                                                                       'in_octets': '29451437801',
                                                                                       'in_ucast_pkts': '146969540',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3740',
                                                                                       'out_octets': '477818042319',
                                                                                       'out_ucast_pkts': '349944811',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDY1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8f:50'],
                                                                                       'in_nucast_pkts': '14491',
                                                                                       'in_octets': '25747209778',
                                                                                       'in_ucast_pkts': '86361868',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '258710619021',
                                                                                       'out_ucast_pkts': '205310075',
                                                                                       'poller_hash': 'QURUTjIzMTIzRTlEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:30'],
                                                                                       'in_nucast_pkts': '14843',
                                                                                       'in_octets': '86559397424',
                                                                                       'in_ucast_pkts': '191752268',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '511160420142',
                                                                                       'out_ucast_pkts': '393278257',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTMzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:03:40'],
                                                                                       'in_nucast_pkts': '114024',
                                                                                       'in_octets': '132343926606',
                                                                                       'in_ucast_pkts': '358229940',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9652',
                                                                                       'out_octets': '1077935386793',
                                                                                       'out_ucast_pkts': '789288784',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTQzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d5:fd:61'],
                                                                                       'in_nucast_pkts': '261',
                                                                                       'in_octets': '5627074010',
                                                                                       'in_ucast_pkts': '41991202',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '198998846990',
                                                                                       'out_ucast_pkts': '135593488',
                                                                                       'poller_hash': 'QURUTjIzMTI0NDkxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:88:70'],
                                                                                       'in_nucast_pkts': '114024',
                                                                                       'in_octets': '171519733498',
                                                                                       'in_ucast_pkts': '490137850',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9650',
                                                                                       'out_octets': '1102102393438',
                                                                                       'out_ucast_pkts': '920098124',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTBCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0c:90'],
                                                                                       'in_nucast_pkts': '114060',
                                                                                       'in_octets': '89909882389',
                                                                                       'in_ucast_pkts': '158297701',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9654',
                                                                                       'out_octets': '404682300236',
                                                                                       'out_ucast_pkts': '317195659',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OEQ1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '08:55:31:e1:ea:4b',
                                                                                                                        '18:e8:29:34:ed:33',
                                                                                                                        '18:e8:29:36:d4:71',
                                                                                                                        '18:e8:29:3a:1a:3d',
                                                                                                                        '18:e8:29:8e:ed:6f',
                                                                                                                        '18:e8:29:8e:ed:78',
                                                                                                                        '18:e8:29:8e:f1:06',
                                                                                                                        '24:5a:4c:e4:c8:a1',
                                                                                                                        '74:83:c2:a6:5a:93',
                                                                                                                        '74:83:c2:a6:7f:9f',
                                                                                                                        '74:83:c2:a6:86:4e',
                                                                                                                        '74:83:c2:e8:a2:61',
                                                                                                                        '74:83:c2:ea:a6:01',
                                                                                                                        '74:83:c2:ea:a6:28',
                                                                                                                        '74:83:c2:ec:11:8b',
                                                                                                                        '74:ac:b9:80:d4:53',
                                                                                                                        '94:a6:7e:dd:ad:81',
                                                                                                                        'b4:fb:e4:5a:4f:8e',
                                                                                                                        'b8:94:70:f8:2b:2a',
                                                                                                                        'dc:2c:6e:dd:2e:26',
                                                                                                                        'f4:92:bf:38:a5:74',
                                                                                                                        'f4:92:bf:38:a7:25',
                                                                                                                        'f4:92:bf:38:a7:c2',
                                                                                                                        'f4:92:bf:d6:a8:ac'],
                                                                                       'in_nucast_pkts': '7995507',
                                                                                       'in_octets': '1858309971876',
                                                                                       'in_ucast_pkts': '9243126879',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '153208',
                                                                                       'out_octets': '28429412683572',
                                                                                       'out_ucast_pkts': '21092874583',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTg2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:b3:c0'],
                                                                                       'in_nucast_pkts': '37592',
                                                                                       'in_octets': '12713140314',
                                                                                       'in_ucast_pkts': '33577218',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3182',
                                                                                       'out_octets': '98263455711',
                                                                                       'out_ucast_pkts': '84012258',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0FDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:87:20'],
                                                                                       'in_nucast_pkts': '14492',
                                                                                       'in_octets': '9889045754',
                                                                                       'in_ucast_pkts': '57345105',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '242042243181',
                                                                                       'out_ucast_pkts': '170513907',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NjlFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:3e:d0'],
                                                                                       'in_nucast_pkts': '84975',
                                                                                       'in_octets': '45699713263',
                                                                                       'in_ucast_pkts': '190445833',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2784',
                                                                                       'out_octets': '579520275565',
                                                                                       'out_ucast_pkts': '418521378',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTM1IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/10',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/10',
                                                                                      'id': 'xgspon '
                                                                                            '0/10',
                                                                                      'links': [       {       'connected_time': '427041',
                                                                                                               'fiber_length': 5677,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDY4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427040',
                                                                                                               'fiber_length': 6613,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTE4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '3364586',
                                                                                                               'fiber_length': 6229,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDU1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '3364583',
                                                                                                               'fiber_length': 5972,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NjUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '637514',
                                                                                                               'fiber_length': 5554,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '5355633',
                                                                                                               'fiber_length': 6288,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDdFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427041',
                                                                                                               'fiber_length': 5688,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REIyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427041',
                                                                                                               'fiber_length': 5757,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTI5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427037',
                                                                                                               'fiber_length': 5845,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MUQwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427041',
                                                                                                               'fiber_length': 6895,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REJDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427039',
                                                                                                               'fiber_length': 5938,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTJCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '1535983',
                                                                                                               'fiber_length': 7112,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTdGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427039',
                                                                                                               'fiber_length': 6916,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTVCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427037',
                                                                                                               'fiber_length': 6439,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUIxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '3364584',
                                                                                                               'fiber_length': 6277,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RThBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '3364586',
                                                                                                               'fiber_length': 6171,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDY1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427037',
                                                                                                               'fiber_length': 6505,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRTlEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427036',
                                                                                                               'fiber_length': 6490,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTMzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '3364587',
                                                                                                               'fiber_length': 6584,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTQzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427041',
                                                                                                               'fiber_length': 6819,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NDkxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '3364587',
                                                                                                               'fiber_length': 6776,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTBCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '3364586',
                                                                                                               'fiber_length': 5964,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OEQ1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '5355630',
                                                                                                               'fiber_length': 7783,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTg2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '1108815',
                                                                                                               'fiber_length': 6748,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '427040',
                                                                                                               'fiber_length': 6818,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NjlFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649},
                                                                                                       {       'connected_time': '2503969',
                                                                                                               'fiber_length': 6346,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTM1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.649}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/10',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/11',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:af:10'],
                                                                                       'in_nucast_pkts': '14605',
                                                                                       'in_octets': '4074282793',
                                                                                       'in_ucast_pkts': '9703049',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '42205526052',
                                                                                       'out_ucast_pkts': '33256315',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjJEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:79:10'],
                                                                                       'in_nucast_pkts': '14833',
                                                                                       'in_octets': '18005021720',
                                                                                       'in_ucast_pkts': '32996865',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '130581815576',
                                                                                       'out_ucast_pkts': '99475924',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDI2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:68:90'],
                                                                                       'in_nucast_pkts': '14478',
                                                                                       'in_octets': '6509985184',
                                                                                       'in_ucast_pkts': '10001038',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '23997518580',
                                                                                       'out_ucast_pkts': '17479618',
                                                                                       'poller_hash': 'QURUTjIzMTIzRjU3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8b:b0'],
                                                                                       'in_nucast_pkts': '14842',
                                                                                       'in_octets': '17964625070',
                                                                                       'in_ucast_pkts': '72846985',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '261600566960',
                                                                                       'out_ucast_pkts': '192987362',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDg4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6b:c0'],
                                                                                       'in_nucast_pkts': '14478',
                                                                                       'in_octets': '4192434431',
                                                                                       'in_ucast_pkts': '9735185',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '27110650859',
                                                                                       'out_ucast_pkts': '23016861',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjE0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:89:00'],
                                                                                       'in_nucast_pkts': '14836',
                                                                                       'in_octets': '11623302477',
                                                                                       'in_ucast_pkts': '42961367',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '90157267496',
                                                                                       'out_ucast_pkts': '71266091',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjM3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:65:60'],
                                                                                       'in_nucast_pkts': '14481',
                                                                                       'in_octets': '32821474532',
                                                                                       'in_ucast_pkts': '51829091',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '185633626970',
                                                                                       'out_ucast_pkts': '144377013',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTYxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:48:80'],
                                                                                       'in_nucast_pkts': '28745',
                                                                                       'in_octets': '27617544835',
                                                                                       'in_ucast_pkts': '103034840',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '959',
                                                                                       'out_octets': '424807630451',
                                                                                       'out_ucast_pkts': '301205551',
                                                                                       'poller_hash': 'QURUTjIzMTI0REEwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:02:b0'],
                                                                                       'in_nucast_pkts': '14482',
                                                                                       'in_octets': '2081111609',
                                                                                       'in_ucast_pkts': '9893508',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '35193394551',
                                                                                       'out_ucast_pkts': '26825096',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEZFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:09:70'],
                                                                                       'in_nucast_pkts': '29438',
                                                                                       'in_octets': '14131521633',
                                                                                       'in_ucast_pkts': '62966197',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2478',
                                                                                       'out_octets': '288886403265',
                                                                                       'out_ucast_pkts': '207388734',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:40'],
                                                                                       'in_nucast_pkts': '29210',
                                                                                       'in_octets': '13692612528',
                                                                                       'in_ucast_pkts': '90428390',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2472',
                                                                                       'out_octets': '327230078932',
                                                                                       'out_ucast_pkts': '230788998',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEJGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:65:70'],
                                                                                       'in_nucast_pkts': '14483',
                                                                                       'in_octets': '5950362229',
                                                                                       'in_ucast_pkts': '25236495',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '130148994270',
                                                                                       'out_ucast_pkts': '93649767',
                                                                                       'poller_hash': 'QURUTjIzMTIzRUMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:65:d0'],
                                                                                       'in_nucast_pkts': '29209',
                                                                                       'in_octets': '8844157372',
                                                                                       'in_ucast_pkts': '21227697',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2472',
                                                                                       'out_octets': '45819930297',
                                                                                       'out_ucast_pkts': '43332910',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkVFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b1:50'],
                                                                                       'in_nucast_pkts': '112182',
                                                                                       'in_octets': '65293876011',
                                                                                       'in_ucast_pkts': '244388608',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3741',
                                                                                       'out_octets': '1379640415195',
                                                                                       'out_ucast_pkts': '1013050052',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkU5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:f5:40'],
                                                                                       'in_nucast_pkts': '111982',
                                                                                       'in_octets': '55559806667',
                                                                                       'in_ucast_pkts': '326687744',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3734',
                                                                                       'out_octets': '1471543734583',
                                                                                       'out_ucast_pkts': '1139963640',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEQxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:89:30'],
                                                                                       'in_nucast_pkts': '14514',
                                                                                       'in_octets': '44117505825',
                                                                                       'in_ucast_pkts': '140548890',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1228',
                                                                                       'out_octets': '355477843613',
                                                                                       'out_ucast_pkts': '268863394',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkNFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:37:10'],
                                                                                       'in_nucast_pkts': '29225',
                                                                                       'in_octets': '11170979770',
                                                                                       'in_ucast_pkts': '40121588',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2474',
                                                                                       'out_octets': '333470491980',
                                                                                       'out_ucast_pkts': '231950841',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RERGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0c:40'],
                                                                                       'in_nucast_pkts': '14837',
                                                                                       'in_octets': '6618612021',
                                                                                       'in_ucast_pkts': '22102834',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '60367612439',
                                                                                       'out_ucast_pkts': '49153075',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RDUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3c:20'],
                                                                                       'in_nucast_pkts': '14491',
                                                                                       'in_octets': '5945748995',
                                                                                       'in_ucast_pkts': '38403602',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '211846856617',
                                                                                       'out_ucast_pkts': '151900586',
                                                                                       'poller_hash': 'QURUTjIzMTI0QzczIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:39:50'],
                                                                                       'in_nucast_pkts': '22894',
                                                                                       'in_octets': '34295336756',
                                                                                       'in_ucast_pkts': '69203173',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '755',
                                                                                       'out_octets': '137053450797',
                                                                                       'out_ucast_pkts': '137839818',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RURGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:97:70'],
                                                                                       'in_nucast_pkts': '14481',
                                                                                       'in_octets': '5086119669',
                                                                                       'in_ucast_pkts': '22447222',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '99081591641',
                                                                                       'out_ucast_pkts': '78561562',
                                                                                       'poller_hash': 'QURUTjIzMTQ0MjhBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ae:70'],
                                                                                       'in_nucast_pkts': '113720',
                                                                                       'in_octets': '50182795525',
                                                                                       'in_ucast_pkts': '367529678',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3738',
                                                                                       'out_octets': '1355404021797',
                                                                                       'out_ucast_pkts': '954516758',
                                                                                       'poller_hash': 'QURUTjIzMTI0MUEyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:9b:50'],
                                                                                       'in_nucast_pkts': '14609',
                                                                                       'in_octets': '55959688032',
                                                                                       'in_ucast_pkts': '63282989',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '134362196864',
                                                                                       'out_ucast_pkts': '112053823',
                                                                                       'poller_hash': 'QURUTjI0NDkxN0E0IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/11',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/11',
                                                                                      'id': 'xgspon '
                                                                                            '0/11',
                                                                                      'links': [       {       'connected_time': '427032',
                                                                                                               'fiber_length': 4645,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjJEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427032',
                                                                                                               'fiber_length': 4747,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDI2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427033',
                                                                                                               'fiber_length': 4661,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRjU3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427032',
                                                                                                               'fiber_length': 4852,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDg4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427033',
                                                                                                               'fiber_length': 7344,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 7364,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjM3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427032',
                                                                                                               'fiber_length': 5086,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTYxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '861783',
                                                                                                               'fiber_length': 8008,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REEwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 4938,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEZFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '861787',
                                                                                                               'fiber_length': 8251,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '861785',
                                                                                                               'fiber_length': 8000,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEJGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 4770,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRUMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '861785',
                                                                                                               'fiber_length': 8343,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkVFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '3364579',
                                                                                                               'fiber_length': 8068,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkU5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '3364578',
                                                                                                               'fiber_length': 8127,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEQxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427031',
                                                                                                               'fiber_length': 7294,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '861787',
                                                                                                               'fiber_length': 8094,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RERGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427031',
                                                                                                               'fiber_length': 7706,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RDUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 4666,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QzczIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '677935',
                                                                                                               'fiber_length': 8136,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RURGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427031',
                                                                                                               'fiber_length': 7421,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0MjhBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '3364576',
                                                                                                               'fiber_length': 8116,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MUEyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868},
                                                                                                       {       'connected_time': '427036',
                                                                                                               'fiber_length': 4703,
                                                                                                               'poller_hash': 'QURUTjI0NDkxN0E0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4868}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/11',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/12',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dd:78:e0'],
                                                                                       'in_nucast_pkts': '14481',
                                                                                       'in_octets': '12010142280',
                                                                                       'in_ucast_pkts': '40833955',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '139273338352',
                                                                                       'out_ucast_pkts': '125281266',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTY2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:75:70'],
                                                                                       'in_nucast_pkts': '10918',
                                                                                       'in_octets': '4909755664',
                                                                                       'in_ucast_pkts': '21306651',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '71098510942',
                                                                                       'out_ucast_pkts': '52730022',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTkwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:40'],
                                                                                       'in_nucast_pkts': '14484',
                                                                                       'in_octets': '5087837331',
                                                                                       'in_ucast_pkts': '20715264',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '144634995117',
                                                                                       'out_ucast_pkts': '106640068',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTQwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7c:f0'],
                                                                                       'in_nucast_pkts': '14840',
                                                                                       'in_octets': '2117419990',
                                                                                       'in_ucast_pkts': '16432791',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '76155224064',
                                                                                       'out_ucast_pkts': '54375468',
                                                                                       'poller_hash': 'QURUTjIzMTI0RUQ5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:70:a0'],
                                                                                       'in_nucast_pkts': '181493',
                                                                                       'in_octets': '260250861320',
                                                                                       'in_ucast_pkts': '725172539',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15373',
                                                                                       'out_octets': '2021601751723',
                                                                                       'out_ucast_pkts': '1766925366',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTNFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:56:40'],
                                                                                       'in_nucast_pkts': '11095',
                                                                                       'in_octets': '14171450420',
                                                                                       'in_ucast_pkts': '40353367',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '202237039839',
                                                                                       'out_ucast_pkts': '154765636',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUVCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:76:40'],
                                                                                       'in_nucast_pkts': '11096',
                                                                                       'in_octets': '3889093495',
                                                                                       'in_ucast_pkts': '10543113',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '36279580423',
                                                                                       'out_ucast_pkts': '30627633',
                                                                                       'poller_hash': 'QURUTjIzMTI0REU0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7e:00'],
                                                                                       'in_nucast_pkts': '14481',
                                                                                       'in_octets': '69004973439',
                                                                                       'in_ucast_pkts': '106561031',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '235491527014',
                                                                                       'out_ucast_pkts': '189025158',
                                                                                       'poller_hash': 'QURUTjIzMTI0REIwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:20'],
                                                                                       'in_nucast_pkts': '14254',
                                                                                       'in_octets': '5606187983',
                                                                                       'in_ucast_pkts': '27182948',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '242397156682',
                                                                                       'out_ucast_pkts': '177117374',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjM0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ab:30'],
                                                                                       'in_nucast_pkts': '11191',
                                                                                       'in_octets': '9781526284',
                                                                                       'in_ucast_pkts': '24332014',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '119157782184',
                                                                                       'out_ucast_pkts': '89684982',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTU3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:e0'],
                                                                                       'in_nucast_pkts': '14251',
                                                                                       'in_octets': '17584067874',
                                                                                       'in_ucast_pkts': '26547393',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '99416976968',
                                                                                       'out_ucast_pkts': '77910131',
                                                                                       'poller_hash': 'QURUTjIzMTIzRUI3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6e:40'],
                                                                                       'in_nucast_pkts': '14833',
                                                                                       'in_octets': '4278648759',
                                                                                       'in_ucast_pkts': '28535445',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '173726757260',
                                                                                       'out_ucast_pkts': '120494076',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTM3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8f:b0'],
                                                                                       'in_nucast_pkts': '14728',
                                                                                       'in_octets': '24093737952',
                                                                                       'in_ucast_pkts': '106219013',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1223',
                                                                                       'out_octets': '404520882600',
                                                                                       'out_ucast_pkts': '288398654',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTRCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:52:10'],
                                                                                       'in_nucast_pkts': '10919',
                                                                                       'in_octets': '7054881425',
                                                                                       'in_ucast_pkts': '20904973',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '139917712727',
                                                                                       'out_ucast_pkts': '99189673',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTczIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:7d:70'],
                                                                                       'in_nucast_pkts': '14483',
                                                                                       'in_octets': '6218284263',
                                                                                       'in_ucast_pkts': '24698100',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '92812486069',
                                                                                       'out_ucast_pkts': '66531067',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTE3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6c:b0'],
                                                                                       'in_nucast_pkts': '14837',
                                                                                       'in_octets': '8526432020',
                                                                                       'in_ucast_pkts': '61587656',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '260945147663',
                                                                                       'out_ucast_pkts': '182878707',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTJDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:77:30'],
                                                                                       'in_nucast_pkts': '14479',
                                                                                       'in_octets': '3965871616',
                                                                                       'in_ucast_pkts': '17863515',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '83823481543',
                                                                                       'out_ucast_pkts': '61501151',
                                                                                       'poller_hash': 'QURUTjIzMTIzRUExIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:76:40'],
                                                                                       'in_nucast_pkts': '14480',
                                                                                       'in_octets': '31320035261',
                                                                                       'in_ucast_pkts': '78000070',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '276529278879',
                                                                                       'out_ucast_pkts': '201540243',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTMzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:a3:c0'],
                                                                                       'in_nucast_pkts': '14258',
                                                                                       'in_octets': '37144367397',
                                                                                       'in_ucast_pkts': '86633184',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '319146165890',
                                                                                       'out_ucast_pkts': '229692615',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUFCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0b:60'],
                                                                                       'in_nucast_pkts': '11367',
                                                                                       'in_octets': '27663153',
                                                                                       'in_ucast_pkts': '90402',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '18021876',
                                                                                       'out_ucast_pkts': '72331',
                                                                                       'poller_hash': 'QURUTjIzMTI0MzA0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:87:b0'],
                                                                                       'in_nucast_pkts': '14252',
                                                                                       'in_octets': '4570785449',
                                                                                       'in_ucast_pkts': '6288909',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '9000467123',
                                                                                       'out_ucast_pkts': '8758129',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTRDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:89:60'],
                                                                                       'in_nucast_pkts': '14483',
                                                                                       'in_octets': '4288421826',
                                                                                       'in_ucast_pkts': '45952539',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '140797273463',
                                                                                       'out_ucast_pkts': '94877340',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkM0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:10'],
                                                                                       'in_nucast_pkts': '14484',
                                                                                       'in_octets': '3321663322',
                                                                                       'in_ucast_pkts': '3122814',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '3289045895',
                                                                                       'out_ucast_pkts': '3279753',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEMzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:58:40'],
                                                                                       'in_nucast_pkts': '179971',
                                                                                       'in_octets': '121539742062',
                                                                                       'in_ucast_pkts': '283895598',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5954',
                                                                                       'out_octets': '946344226230',
                                                                                       'out_ucast_pkts': '722438556',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0E1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:80'],
                                                                                       'in_nucast_pkts': '14482',
                                                                                       'in_octets': '1923211696',
                                                                                       'in_ucast_pkts': '8016011',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '22878018917',
                                                                                       'out_ucast_pkts': '17994918',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzdBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0a:00'],
                                                                                       'in_nucast_pkts': '31123',
                                                                                       'in_octets': '29998733749',
                                                                                       'in_ucast_pkts': '30479439',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2633',
                                                                                       'out_octets': '15272543552',
                                                                                       'out_ucast_pkts': '17448007',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODBEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:b3:a0'],
                                                                                       'in_nucast_pkts': '14483',
                                                                                       'in_octets': '13072353988',
                                                                                       'in_ucast_pkts': '27611640',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '159270900949',
                                                                                       'out_ucast_pkts': '113977534',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0FCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:89:30'],
                                                                                       'in_nucast_pkts': '14812',
                                                                                       'in_octets': '140478805070',
                                                                                       'in_ucast_pkts': '206090775',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '509510772101',
                                                                                       'out_ucast_pkts': '407999782',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0YzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:7b:50'],
                                                                                       'in_nucast_pkts': '14548',
                                                                                       'in_octets': '16103184483',
                                                                                       'in_ucast_pkts': '24497332',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '475',
                                                                                       'out_octets': '49677874682',
                                                                                       'out_ucast_pkts': '44710176',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUQyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:42:00'],
                                                                                       'in_nucast_pkts': '10918',
                                                                                       'in_octets': '4741021242',
                                                                                       'in_ucast_pkts': '40561298',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '113233366598',
                                                                                       'out_ucast_pkts': '79471298',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDZDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6a:d0'],
                                                                                       'in_nucast_pkts': '14257',
                                                                                       'in_octets': '3901163862',
                                                                                       'in_ucast_pkts': '12196554',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '16966615768',
                                                                                       'out_ucast_pkts': '12577131',
                                                                                       'poller_hash': 'QURUTjI0MzYzRDREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:aa:e1'],
                                                                                       'in_nucast_pkts': '258',
                                                                                       'in_octets': '9643762835',
                                                                                       'in_ucast_pkts': '55490510',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '203171251388',
                                                                                       'out_ucast_pkts': '150409881',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjIwIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/12',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/12',
                                                                                      'id': 'xgspon '
                                                                                            '0/12',
                                                                                      'links': [       {       'connected_time': '427028',
                                                                                                               'fiber_length': 5195,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '327028',
                                                                                                               'fiber_length': 6895,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTkwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 5726,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTQwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427030',
                                                                                                               'fiber_length': 5679,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RUQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '5355592',
                                                                                                               'fiber_length': 5805,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTNFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '327027',
                                                                                                               'fiber_length': 6685,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUVCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '327029',
                                                                                                               'fiber_length': 6822,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REU0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427027',
                                                                                                               'fiber_length': 5405,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427030',
                                                                                                               'fiber_length': 5229,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjM0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '327028',
                                                                                                               'fiber_length': 6531,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTU3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427033',
                                                                                                               'fiber_length': 3552,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRUI3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 5276,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTM3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427025',
                                                                                                               'fiber_length': 5187,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTRCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '327028',
                                                                                                               'fiber_length': 4886,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTczIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427031',
                                                                                                               'fiber_length': 4322,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTE3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427030',
                                                                                                               'fiber_length': 5460,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTJDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427032',
                                                                                                               'fiber_length': 3586,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRUExIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 3613,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTMzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 5452,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUFCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '327029',
                                                                                                               'fiber_length': 4895,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MzA0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 5052,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTRDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427031',
                                                                                                               'fiber_length': 3515,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkM0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427034',
                                                                                                               'fiber_length': 3450,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEMzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '5355590',
                                                                                                               'fiber_length': 4061,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0E1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 3993,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzdBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '918509',
                                                                                                               'fiber_length': 4007,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODBEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427028',
                                                                                                               'fiber_length': 4238,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427034',
                                                                                                               'fiber_length': 4128,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0YzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '423780',
                                                                                                               'fiber_length': 4439,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUQyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '327028',
                                                                                                               'fiber_length': 6899,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDZDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427039',
                                                                                                               'fiber_length': 3928,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRDREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411},
                                                                                                       {       'connected_time': '427033',
                                                                                                               'fiber_length': 5562,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.411}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/12',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/13',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dd:84:00'],
                                                                                       'in_nucast_pkts': '11367',
                                                                                       'in_octets': '15767760415',
                                                                                       'in_ucast_pkts': '30683773',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '73447962648',
                                                                                       'out_ucast_pkts': '60143172',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTcxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6e:f0'],
                                                                                       'in_nucast_pkts': '11367',
                                                                                       'in_octets': '4050843218',
                                                                                       'in_ucast_pkts': '22475505',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '148833022375',
                                                                                       'out_ucast_pkts': '104104029',
                                                                                       'poller_hash': 'QURUTjIzMTI0RERBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8e:50'],
                                                                                       'in_nucast_pkts': '11367',
                                                                                       'in_octets': '4949456657',
                                                                                       'in_ucast_pkts': '16426571',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '72751942766',
                                                                                       'out_ucast_pkts': '56652163',
                                                                                       'poller_hash': 'QURUTjIzMTI0REUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:b0'],
                                                                                       'in_nucast_pkts': '10922',
                                                                                       'in_octets': '12769370752',
                                                                                       'in_ucast_pkts': '61778045',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '284107873798',
                                                                                       'out_ucast_pkts': '200982987',
                                                                                       'poller_hash': 'QURUTjIzMTI0REY2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:01:50'],
                                                                                       'in_nucast_pkts': '10920',
                                                                                       'in_octets': '437383688',
                                                                                       'in_ucast_pkts': '893267',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '308702946',
                                                                                       'out_ucast_pkts': '570377',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTA0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:79:70'],
                                                                                       'in_nucast_pkts': '11093',
                                                                                       'in_octets': '7916900498',
                                                                                       'in_ucast_pkts': '19430276',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '39267826124',
                                                                                       'out_ucast_pkts': '33836502',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a6:80'],
                                                                                       'in_nucast_pkts': '10993',
                                                                                       'in_octets': '11999564548',
                                                                                       'in_ucast_pkts': '69806944',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '352265173015',
                                                                                       'out_ucast_pkts': '253140500',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTkyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6d:50'],
                                                                                       'in_nucast_pkts': '11091',
                                                                                       'in_octets': '1957758834',
                                                                                       'in_ucast_pkts': '11873930',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '46479498752',
                                                                                       'out_ucast_pkts': '34686590',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTg2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:81:b0'],
                                                                                       'in_nucast_pkts': '10921',
                                                                                       'in_octets': '24621439854',
                                                                                       'in_ucast_pkts': '47282680',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '279360494658',
                                                                                       'out_ucast_pkts': '197894118',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTk0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:e0'],
                                                                                       'in_nucast_pkts': '11355',
                                                                                       'in_octets': '2030971762',
                                                                                       'in_ucast_pkts': '8552994',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '39794992093',
                                                                                       'out_ucast_pkts': '27978558',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTRFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:00'],
                                                                                       'in_nucast_pkts': '11192',
                                                                                       'in_octets': '15555701442',
                                                                                       'in_ucast_pkts': '37628460',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '212251197099',
                                                                                       'out_ucast_pkts': '163182738',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkI4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:6a:80'],
                                                                                       'in_nucast_pkts': '181534',
                                                                                       'in_octets': '150318711426',
                                                                                       'in_ucast_pkts': '604576406',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15379',
                                                                                       'out_octets': '2125424430048',
                                                                                       'out_ucast_pkts': '1580422205',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkNCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:9a:d0'],
                                                                                       'in_nucast_pkts': '178542',
                                                                                       'in_octets': '141812945571',
                                                                                       'in_ucast_pkts': '1004023051',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5952',
                                                                                       'out_octets': '4783940559298',
                                                                                       'out_ucast_pkts': '3415646167',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODYxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6e:b0'],
                                                                                       'in_nucast_pkts': '11193',
                                                                                       'in_octets': '2614312908',
                                                                                       'in_ucast_pkts': '17215659',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '65148013224',
                                                                                       'out_ucast_pkts': '46404667',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTE1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:55:80'],
                                                                                       'in_nucast_pkts': '10920',
                                                                                       'in_octets': '5696191608',
                                                                                       'in_ucast_pkts': '11625565',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '57625493901',
                                                                                       'out_ucast_pkts': '41738950',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QkY3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:02:90'],
                                                                                       'in_nucast_pkts': '11192',
                                                                                       'in_octets': '1012604589',
                                                                                       'in_ucast_pkts': '4080931',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '10986661054',
                                                                                       'out_ucast_pkts': '8636314',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTVCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:62:10'],
                                                                                       'in_nucast_pkts': '14253',
                                                                                       'in_octets': '5439487007',
                                                                                       'in_ucast_pkts': '7838907',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '22414068884',
                                                                                       'out_ucast_pkts': '19152768',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzhCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b6:c0'],
                                                                                       'in_nucast_pkts': '10923',
                                                                                       'in_octets': '2414262555',
                                                                                       'in_ucast_pkts': '8667815',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '45045431911',
                                                                                       'out_ucast_pkts': '32186109',
                                                                                       'poller_hash': 'QURUTjI0MzYzQkJBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:81:80'],
                                                                                       'in_nucast_pkts': '11097',
                                                                                       'in_octets': '1764676337',
                                                                                       'in_ucast_pkts': '7642458',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '19920110767',
                                                                                       'out_ucast_pkts': '16158235',
                                                                                       'poller_hash': 'QURUTjI0MzE0MUFDIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/13',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/13',
                                                                                      'id': 'xgspon '
                                                                                            '0/13',
                                                                                      'links': [       {       'connected_time': '327025',
                                                                                                               'fiber_length': 8742,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTcxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327029',
                                                                                                               'fiber_length': 8706,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RERBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327027',
                                                                                                               'fiber_length': 8348,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327030',
                                                                                                               'fiber_length': 8325,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327029',
                                                                                                               'fiber_length': 8705,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTA0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327026',
                                                                                                               'fiber_length': 8651,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327029',
                                                                                                               'fiber_length': 8747,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTkyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327029',
                                                                                                               'fiber_length': 8737,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTg2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327022',
                                                                                                               'fiber_length': 9481,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTk0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327028',
                                                                                                               'fiber_length': 8614,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTRFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327027',
                                                                                                               'fiber_length': 9457,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkI4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '5355622',
                                                                                                               'fiber_length': 8828,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '5355621',
                                                                                                               'fiber_length': 8370,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODYxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327025',
                                                                                                               'fiber_length': 7986,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTE1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327028',
                                                                                                               'fiber_length': 8538,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QkY3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327029',
                                                                                                               'fiber_length': 8775,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTVCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 8911,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzhCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327033',
                                                                                                               'fiber_length': 8469,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQkJBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666},
                                                                                                       {       'connected_time': '327033',
                                                                                                               'fiber_length': 9546,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MUFDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5666}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/13',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/14',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:d6:0a:70'],
                                                                                       'in_nucast_pkts': '11095',
                                                                                       'in_octets': '11364743835',
                                                                                       'in_ucast_pkts': '80052462',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '384501951861',
                                                                                       'out_ucast_pkts': '263215609',
                                                                                       'poller_hash': 'QURUTjIzMTI0MzJFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:07:e0'],
                                                                                       'in_nucast_pkts': '14480',
                                                                                       'in_octets': '5893619517',
                                                                                       'in_ucast_pkts': '50997649',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '125013412365',
                                                                                       'out_ucast_pkts': '93707406',
                                                                                       'poller_hash': 'QURUTjIzMTI0ODQ5IHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '302422',
                                                                                       'in_octets': '93074964',
                                                                                       'in_ucast_pkts': '0',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEM2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:03:f0'],
                                                                                       'in_nucast_pkts': '10925',
                                                                                       'in_octets': '11584582976',
                                                                                       'in_ucast_pkts': '36155516',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '97260334575',
                                                                                       'out_ucast_pkts': '74915668',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTVBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:38:f0'],
                                                                                       'in_nucast_pkts': '10500',
                                                                                       'in_octets': '4315964990',
                                                                                       'in_ucast_pkts': '17363552',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '867',
                                                                                       'out_octets': '57026256345',
                                                                                       'out_ucast_pkts': '42378102',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTUzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:34:50'],
                                                                                       'in_nucast_pkts': '11100',
                                                                                       'in_octets': '6681485962',
                                                                                       'in_ucast_pkts': '42967152',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '249261259637',
                                                                                       'out_ucast_pkts': '170364556',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTg0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:00:f0'],
                                                                                       'in_nucast_pkts': '10921',
                                                                                       'in_octets': '3043675623',
                                                                                       'in_ucast_pkts': '9936676',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '45570325495',
                                                                                       'out_ucast_pkts': '33509700',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTVDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:3f:10'],
                                                                                       'in_nucast_pkts': '14252',
                                                                                       'in_octets': '4816574218',
                                                                                       'in_ucast_pkts': '16714166',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '68442769578',
                                                                                       'out_ucast_pkts': '48338264',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RDk5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:63:f0'],
                                                                                       'in_nucast_pkts': '10920',
                                                                                       'in_octets': '6155899121',
                                                                                       'in_ucast_pkts': '13805941',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '85065250252',
                                                                                       'out_ucast_pkts': '62297141',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjNGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:31:10'],
                                                                                       'in_nucast_pkts': '10923',
                                                                                       'in_octets': '11113504970',
                                                                                       'in_ucast_pkts': '38851489',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '178322644314',
                                                                                       'out_ucast_pkts': '125756425',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjVFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:21:80'],
                                                                                       'in_nucast_pkts': '10921',
                                                                                       'in_octets': '4787032931',
                                                                                       'in_ucast_pkts': '23749412',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '140274218693',
                                                                                       'out_ucast_pkts': '107161210',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTlCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a1:70'],
                                                                                       'in_nucast_pkts': '10953',
                                                                                       'in_octets': '4102645831',
                                                                                       'in_ucast_pkts': '13734254',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '366',
                                                                                       'out_octets': '72279718107',
                                                                                       'out_ucast_pkts': '55541440',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QUNGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:30'],
                                                                                       'in_nucast_pkts': '14277',
                                                                                       'in_octets': '54397660066',
                                                                                       'in_ucast_pkts': '56952448',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '477',
                                                                                       'out_octets': '62495928412',
                                                                                       'out_ucast_pkts': '52136476',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDM3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ef:c0'],
                                                                                       'in_nucast_pkts': '10920',
                                                                                       'in_octets': '3578492675',
                                                                                       'in_ucast_pkts': '10367495',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '53634846440',
                                                                                       'out_ucast_pkts': '38760531',
                                                                                       'poller_hash': 'QURUTjIzMTI0QjdFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:37:30'],
                                                                                       'in_nucast_pkts': '10920',
                                                                                       'in_octets': '12111820901',
                                                                                       'in_ucast_pkts': '34746453',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '138901108970',
                                                                                       'out_ucast_pkts': '106039018',
                                                                                       'poller_hash': 'QURUTjIzMTI0OTY2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b1:30'],
                                                                                       'in_nucast_pkts': '10921',
                                                                                       'in_octets': '5131827869',
                                                                                       'in_ucast_pkts': '7642544',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '365',
                                                                                       'out_octets': '31461333787',
                                                                                       'out_ucast_pkts': '24629401',
                                                                                       'poller_hash': 'QURUTjI0MzYzREI5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ba:f0'],
                                                                                       'in_nucast_pkts': '5293',
                                                                                       'in_octets': '12671935772',
                                                                                       'in_ucast_pkts': '40421903',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '174',
                                                                                       'out_octets': '126488221476',
                                                                                       'out_ucast_pkts': '96314100',
                                                                                       'poller_hash': 'QURUTjI1MTUyMUEyIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/14',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/14',
                                                                                      'id': 'xgspon '
                                                                                            '0/14',
                                                                                      'links': [       {       'connected_time': '327030',
                                                                                                               'fiber_length': 9068,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MzJFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 9154,
                                                                                                               'poller_hash': 'QURUTjIzMTI0ODQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '327029',
                                                                                                               'fiber_length': 9038,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEM2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '327027',
                                                                                                               'fiber_length': 9303,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTVBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '302045',
                                                                                                               'fiber_length': 9292,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTUzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '327028',
                                                                                                               'fiber_length': 8903,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTg0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '327029',
                                                                                                               'fiber_length': 9318,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTVDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '427034',
                                                                                                               'fiber_length': 9196,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RDk5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '327026',
                                                                                                               'fiber_length': 9156,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjNGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '327026',
                                                                                                               'fiber_length': 9074,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjVFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '327028',
                                                                                                               'fiber_length': 9481,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTlCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '327026',
                                                                                                               'fiber_length': 9284,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QUNGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '427033',
                                                                                                               'fiber_length': 9249,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDM3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '327029',
                                                                                                               'fiber_length': 9207,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QjdFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '327029',
                                                                                                               'fiber_length': 9245,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OTY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '327033',
                                                                                                               'fiber_length': 9079,
                                                                                                               'poller_hash': 'QURUTjI0MzYzREI5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476},
                                                                                                       {       'connected_time': '153651',
                                                                                                               'fiber_length': 9416,
                                                                                                               'poller_hash': 'QURUTjI1MTUyMUEyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4476}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/14',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/15',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:d6:36:60'],
                                                                                       'in_nucast_pkts': '14481',
                                                                                       'in_octets': '27007301589',
                                                                                       'in_ucast_pkts': '46146504',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '114919981146',
                                                                                       'out_ucast_pkts': '87784650',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8d:b0'],
                                                                                       'in_nucast_pkts': '14848',
                                                                                       'in_octets': '1440796980',
                                                                                       'in_ucast_pkts': '2796705',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '17436492765',
                                                                                       'out_ucast_pkts': '12461495',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzE0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:91:80'],
                                                                                       'in_nucast_pkts': '14485',
                                                                                       'in_octets': '431823514',
                                                                                       'in_ucast_pkts': '650656',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '1314724262',
                                                                                       'out_ucast_pkts': '1246654',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzMzIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/15',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/15',
                                                                                      'id': 'xgspon '
                                                                                            '0/15',
                                                                                      'links': [       {       'connected_time': '427035',
                                                                                                               'fiber_length': 1195,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3519},
                                                                                                       {       'connected_time': '427034',
                                                                                                               'fiber_length': 780,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3519},
                                                                                                       {       'connected_time': '427034',
                                                                                                               'fiber_length': 539,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzMzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3519}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/15',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/16',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/16',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNg==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/16',
                                                                                      'id': 'xgspon '
                                                                                            '0/16',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/16',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/2',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dc:71:b0'],
                                                                                       'in_nucast_pkts': '14537',
                                                                                       'in_octets': '983568170',
                                                                                       'in_ucast_pkts': '6528585',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '17955758353',
                                                                                       'out_ucast_pkts': '12484499',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDQxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:5d:e0'],
                                                                                       'in_nucast_pkts': '14480',
                                                                                       'in_octets': '1572259937',
                                                                                       'in_ucast_pkts': '6243316',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '29939132609',
                                                                                       'out_ucast_pkts': '21426454',
                                                                                       'poller_hash': 'QURUTjIzMTI0MEIwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7e:90'],
                                                                                       'in_nucast_pkts': '14820',
                                                                                       'in_octets': '9477065744',
                                                                                       'in_ucast_pkts': '46725402',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '131191620822',
                                                                                       'out_ucast_pkts': '105992610',
                                                                                       'poller_hash': 'QURUTjIzMTI0REQ5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ad:c0'],
                                                                                       'in_nucast_pkts': '14280',
                                                                                       'in_octets': '10173859269',
                                                                                       'in_ucast_pkts': '47770475',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '154648734777',
                                                                                       'out_ucast_pkts': '117667977',
                                                                                       'poller_hash': 'QURUTjIzMTI0MUQxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:78:c0'],
                                                                                       'in_nucast_pkts': '14481',
                                                                                       'in_octets': '6742653477',
                                                                                       'in_ucast_pkts': '36837158',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '128934177076',
                                                                                       'out_ucast_pkts': '90637709',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjNDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:70'],
                                                                                       'in_nucast_pkts': '14558',
                                                                                       'in_octets': '6938004105',
                                                                                       'in_ucast_pkts': '19551545',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '65772753626',
                                                                                       'out_ucast_pkts': '52111151',
                                                                                       'poller_hash': 'QURUTjIzMTI0REZDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:75:f0'],
                                                                                       'in_nucast_pkts': '14852',
                                                                                       'in_octets': '5085781923',
                                                                                       'in_ucast_pkts': '20757790',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1226',
                                                                                       'out_octets': '79970505613',
                                                                                       'out_ucast_pkts': '56820648',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:91:b0'],
                                                                                       'in_nucast_pkts': '14483',
                                                                                       'in_octets': '14960213903',
                                                                                       'in_ucast_pkts': '38797550',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '104093424075',
                                                                                       'out_ucast_pkts': '86505373',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTQ2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fa:80'],
                                                                                       'in_nucast_pkts': '74431',
                                                                                       'in_octets': '49201605747',
                                                                                       'in_ucast_pkts': '312371320',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '1167157019048',
                                                                                       'out_ucast_pkts': '818793600',
                                                                                       'poller_hash': 'QURUTjIzMTI0REI3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6c:f0'],
                                                                                       'in_nucast_pkts': '74476',
                                                                                       'in_octets': '192252977689',
                                                                                       'in_ucast_pkts': '847205322',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '3450434962360',
                                                                                       'out_ucast_pkts': '2556552687',
                                                                                       'poller_hash': 'QURUTjIzMTI0OEE4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:f0'],
                                                                                       'in_nucast_pkts': '75663',
                                                                                       'in_octets': '28885852860',
                                                                                       'in_ucast_pkts': '160066746',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6406',
                                                                                       'out_octets': '570354592812',
                                                                                       'out_ucast_pkts': '405768101',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDlFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:00:60'],
                                                                                       'in_nucast_pkts': '77590',
                                                                                       'in_octets': '21008714935',
                                                                                       'in_ucast_pkts': '130418590',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6414',
                                                                                       'out_octets': '494392636187',
                                                                                       'out_ucast_pkts': '353071836',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0b:a0'],
                                                                                       'in_nucast_pkts': '14475',
                                                                                       'in_octets': '5294014205',
                                                                                       'in_ucast_pkts': '24438441',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '92267892089',
                                                                                       'out_ucast_pkts': '68782014',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTFEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:60'],
                                                                                       'in_nucast_pkts': '77020',
                                                                                       'in_octets': '42963108958',
                                                                                       'in_ucast_pkts': '308691492',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6404',
                                                                                       'out_octets': '1383576104760',
                                                                                       'out_ucast_pkts': '949831788',
                                                                                       'poller_hash': 'QURUTjIzMTIzRUJDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:79:f0'],
                                                                                       'in_nucast_pkts': '14564',
                                                                                       'in_octets': '5936188625',
                                                                                       'in_ucast_pkts': '39928554',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '161793611585',
                                                                                       'out_ucast_pkts': '113854837',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTE2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fa:60'],
                                                                                       'in_nucast_pkts': '75200',
                                                                                       'in_octets': '62366534715',
                                                                                       'in_ucast_pkts': '213576772',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '615271594261',
                                                                                       'out_ucast_pkts': '452854306',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTJGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:50:60'],
                                                                                       'in_nucast_pkts': '14604',
                                                                                       'in_octets': '7802515320',
                                                                                       'in_ucast_pkts': '26916176',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '154402388649',
                                                                                       'out_ucast_pkts': '116621269',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEM4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:29:50'],
                                                                                       'in_nucast_pkts': '14477',
                                                                                       'in_octets': '10649180489',
                                                                                       'in_ucast_pkts': '104194724',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '319260136551',
                                                                                       'out_ucast_pkts': '217951826',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzczIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:5b:70'],
                                                                                       'in_nucast_pkts': '14608',
                                                                                       'in_octets': '10170294001',
                                                                                       'in_ucast_pkts': '42209559',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '142833690444',
                                                                                       'out_ucast_pkts': '105137132',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:40'],
                                                                                       'in_nucast_pkts': '14249',
                                                                                       'in_octets': '9238663898',
                                                                                       'in_ucast_pkts': '44807023',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '144197548729',
                                                                                       'out_ucast_pkts': '105308065',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OTE2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:7e:b0'],
                                                                                       'in_nucast_pkts': '14837',
                                                                                       'in_octets': '2937545669',
                                                                                       'in_ucast_pkts': '17554396',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '88281793537',
                                                                                       'out_ucast_pkts': '62529827',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjNBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:36:70'],
                                                                                       'in_nucast_pkts': '14481',
                                                                                       'in_octets': '4562146411',
                                                                                       'in_ucast_pkts': '11084217',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '27854038641',
                                                                                       'out_ucast_pkts': '22226636',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzEwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:89:50'],
                                                                                       'in_nucast_pkts': '14852',
                                                                                       'in_octets': '13081311719',
                                                                                       'in_ucast_pkts': '67658493',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '320476692980',
                                                                                       'out_ucast_pkts': '237606249',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTJCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b6:30'],
                                                                                       'in_nucast_pkts': '2312',
                                                                                       'in_octets': '298049450',
                                                                                       'in_ucast_pkts': '876509',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '78',
                                                                                       'out_octets': '2928972974',
                                                                                       'out_ucast_pkts': '2279665',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ad:d0'],
                                                                                       'in_nucast_pkts': '14254',
                                                                                       'in_octets': '28850937908',
                                                                                       'in_ucast_pkts': '35236369',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '76317764218',
                                                                                       'out_ucast_pkts': '61085997',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUM4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:04:00'],
                                                                                       'in_nucast_pkts': '14249',
                                                                                       'in_octets': '4157814749',
                                                                                       'in_ucast_pkts': '19666598',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '60994460314',
                                                                                       'out_ucast_pkts': '44172062',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzE3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:3d:30'],
                                                                                       'in_nucast_pkts': '14252',
                                                                                       'in_octets': '14656916626',
                                                                                       'in_ucast_pkts': '30525972',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '231750491985',
                                                                                       'out_ucast_pkts': '163367025',
                                                                                       'poller_hash': 'QURUTjIzMTI0NTEwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:3e:40'],
                                                                                       'in_nucast_pkts': '14253',
                                                                                       'in_octets': '6706964484',
                                                                                       'in_ucast_pkts': '58361129',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '283054358159',
                                                                                       'out_ucast_pkts': '193514948',
                                                                                       'poller_hash': 'QURUTjI0MzE0NzY2IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/2',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/2',
                                                                                      'id': 'xgspon '
                                                                                            '0/2',
                                                                                      'links': [       {       'connected_time': '427025',
                                                                                                               'fiber_length': 1926,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDQxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427026',
                                                                                                               'fiber_length': 1969,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MEIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427026',
                                                                                                               'fiber_length': 1780,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427020',
                                                                                                               'fiber_length': 2471,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MUQxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427025',
                                                                                                               'fiber_length': 1807,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjNDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427030',
                                                                                                               'fiber_length': 1986,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REZDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -12.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427027',
                                                                                                               'fiber_length': 2396,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427026',
                                                                                                               'fiber_length': 2088,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '2232581',
                                                                                                               'fiber_length': 2058,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REI3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '2232583',
                                                                                                               'fiber_length': 2069,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OEE4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '2232581',
                                                                                                               'fiber_length': 2313,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDlFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '2232581',
                                                                                                               'fiber_length': 2008,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427024',
                                                                                                               'fiber_length': 540,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTFEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '2232583',
                                                                                                               'fiber_length': 2230,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRUJDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427020',
                                                                                                               'fiber_length': 1746,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTE2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '2232581',
                                                                                                               'fiber_length': 1857,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTJGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427030',
                                                                                                               'fiber_length': 1996,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEM4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427024',
                                                                                                               'fiber_length': 233,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzczIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427032',
                                                                                                               'fiber_length': 1780,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -12.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 2012,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OTE2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427032',
                                                                                                               'fiber_length': 545,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjNBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427028',
                                                                                                               'fiber_length': 1734,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzEwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427023',
                                                                                                               'fiber_length': 324,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTJCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '68896',
                                                                                                               'fiber_length': 700,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427028',
                                                                                                               'fiber_length': 489,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUM4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427028',
                                                                                                               'fiber_length': 716,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzE3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427030',
                                                                                                               'fiber_length': 313,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NTEwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -12.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 859,
                                                                                                               'poller_hash': 'QURUTjI0MzE0NzY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.268}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/2',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/3',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dc:83:00'],
                                                                                       'in_nucast_pkts': '182279',
                                                                                       'in_octets': '113227085394',
                                                                                       'in_ucast_pkts': '281784683',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15383',
                                                                                       'out_octets': '1188113516971',
                                                                                       'out_ucast_pkts': '929359604',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTQ1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:36:e0'],
                                                                                       'in_nucast_pkts': '131312',
                                                                                       'in_octets': '40046711094',
                                                                                       'in_ucast_pkts': '80198771',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '10973',
                                                                                       'out_octets': '343142535187',
                                                                                       'out_ucast_pkts': '256076050',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTQ5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:a0'],
                                                                                       'in_nucast_pkts': '61665',
                                                                                       'in_octets': '95356553219',
                                                                                       'in_ucast_pkts': '401599977',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2006',
                                                                                       'out_octets': '1226720307132',
                                                                                       'out_ucast_pkts': '888283807',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0Y5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3b:b0'],
                                                                                       'in_nucast_pkts': '181509',
                                                                                       'in_octets': '23060195890',
                                                                                       'in_ucast_pkts': '143995367',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15377',
                                                                                       'out_octets': '705058396714',
                                                                                       'out_ucast_pkts': '490329991',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NjgxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0c:50'],
                                                                                       'in_nucast_pkts': '23643',
                                                                                       'in_octets': '15570400974',
                                                                                       'in_ucast_pkts': '93138719',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2000',
                                                                                       'out_octets': '371021531560',
                                                                                       'out_ucast_pkts': '269070296',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODlDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:38:50'],
                                                                                       'in_nucast_pkts': '181515',
                                                                                       'in_octets': '103624194410',
                                                                                       'in_ucast_pkts': '359433717',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15384',
                                                                                       'out_octets': '1507930273863',
                                                                                       'out_ucast_pkts': '1092314172',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OEJBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6e:e0'],
                                                                                       'in_nucast_pkts': '181637',
                                                                                       'in_octets': '747454418392',
                                                                                       'in_ucast_pkts': '1068389034',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5952',
                                                                                       'out_octets': '2679772269223',
                                                                                       'out_ucast_pkts': '2022632883',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODlEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0a:f0'],
                                                                                       'in_nucast_pkts': '181527',
                                                                                       'in_octets': '118694247227',
                                                                                       'in_ucast_pkts': '744512617',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15370',
                                                                                       'out_octets': '1744583538342',
                                                                                       'out_ucast_pkts': '1340512502',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODgzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:52:20'],
                                                                                       'in_nucast_pkts': '181542',
                                                                                       'in_octets': '28908246753',
                                                                                       'in_ucast_pkts': '225045072',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15379',
                                                                                       'out_octets': '696477307941',
                                                                                       'out_ucast_pkts': '493506105',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzA3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:c0'],
                                                                                       'in_nucast_pkts': '181585',
                                                                                       'in_octets': '39407377498',
                                                                                       'in_ucast_pkts': '111477829',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15373',
                                                                                       'out_octets': '296003448852',
                                                                                       'out_ucast_pkts': '256938233',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzAyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:10'],
                                                                                       'in_nucast_pkts': '178548',
                                                                                       'in_octets': '58400515225',
                                                                                       'in_ucast_pkts': '102604284',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5952',
                                                                                       'out_octets': '283805844020',
                                                                                       'out_ucast_pkts': '233979174',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0JEIHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '0',
                                                                                       'in_octets': '0',
                                                                                       'in_ucast_pkts': '0',
                                                                                       'oper_up': False,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkNDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:03:a0'],
                                                                                       'in_nucast_pkts': '181535',
                                                                                       'in_octets': '39608918242',
                                                                                       'in_ucast_pkts': '165154942',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15371',
                                                                                       'out_octets': '880189602473',
                                                                                       'out_ucast_pkts': '626030024',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0FFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0b:70'],
                                                                                       'in_nucast_pkts': '49152',
                                                                                       'in_octets': '1631760670',
                                                                                       'in_ucast_pkts': '2740950',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4159',
                                                                                       'out_octets': '3452620893',
                                                                                       'out_ucast_pkts': '3491273',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OEZFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3a:70'],
                                                                                       'in_nucast_pkts': '31817',
                                                                                       'in_octets': '10028074215',
                                                                                       'in_ucast_pkts': '37876254',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2691',
                                                                                       'out_octets': '121263697438',
                                                                                       'out_ucast_pkts': '94319708',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkE1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:83:50'],
                                                                                       'in_nucast_pkts': '163013',
                                                                                       'in_octets': '89954696087',
                                                                                       'in_ucast_pkts': '322041214',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '13694',
                                                                                       'out_octets': '1215213876404',
                                                                                       'out_ucast_pkts': '907506591',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTRBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:4a:f0'],
                                                                                       'in_nucast_pkts': '86768',
                                                                                       'in_octets': '43839947049',
                                                                                       'in_ucast_pkts': '155302953',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2894',
                                                                                       'out_octets': '549023046624',
                                                                                       'out_ucast_pkts': '395156955',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUZGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:38:00'],
                                                                                       'in_nucast_pkts': '75317',
                                                                                       'in_octets': '26957671546',
                                                                                       'in_ucast_pkts': '52874363',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6375',
                                                                                       'out_octets': '88318159755',
                                                                                       'out_ucast_pkts': '83722263',
                                                                                       'poller_hash': 'QURUTjIzMTI0QTE4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '36:3a:c5:c7:0e:34',
                                                                                                                        'cc:66:18:50:b4:20'],
                                                                                       'in_nucast_pkts': '48931',
                                                                                       'in_octets': '436383774512',
                                                                                       'in_ucast_pkts': '507681784',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1631',
                                                                                       'out_octets': '796936082032',
                                                                                       'out_ucast_pkts': '698475722',
                                                                                       'poller_hash': 'QURUTjIzMTI0QTc2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:43:e0'],
                                                                                       'in_nucast_pkts': '178694',
                                                                                       'in_octets': '110469006288',
                                                                                       'in_ucast_pkts': '344223166',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5951',
                                                                                       'out_octets': '929286519505',
                                                                                       'out_ucast_pkts': '686797219',
                                                                                       'poller_hash': 'QURUTjIzMTI0QjNGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:38:30'],
                                                                                       'in_nucast_pkts': '26797',
                                                                                       'in_octets': '84175078455',
                                                                                       'in_ucast_pkts': '173298172',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2272',
                                                                                       'out_octets': '423674434887',
                                                                                       'out_ucast_pkts': '315344278',
                                                                                       'poller_hash': 'QURUTjIzMTI0QkE1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:62:90'],
                                                                                       'in_nucast_pkts': '179550',
                                                                                       'in_octets': '198479562751',
                                                                                       'in_ucast_pkts': '1057037287',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5953',
                                                                                       'out_octets': '5820242453077',
                                                                                       'out_ucast_pkts': '4428697131',
                                                                                       'poller_hash': 'QURUTjI0MzYzQ0M2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8c:80'],
                                                                                       'in_nucast_pkts': '50835',
                                                                                       'in_octets': '85868828443',
                                                                                       'in_ucast_pkts': '313251806',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4214',
                                                                                       'out_octets': '820631184990',
                                                                                       'out_ucast_pkts': '594871244',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OEI4IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/3',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/3',
                                                                                      'id': 'xgspon '
                                                                                            '0/3',
                                                                                      'links': [       {       'connected_time': '5355625',
                                                                                                               'fiber_length': 13527,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTQ1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '3826297',
                                                                                                               'fiber_length': 13074,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '1804563',
                                                                                                               'fiber_length': 14757,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0Y5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '5355592',
                                                                                                               'fiber_length': 14954,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NjgxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '696994',
                                                                                                               'fiber_length': 14487,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODlDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '5355621',
                                                                                                               'fiber_length': 14745,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OEJBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -24.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '5355622',
                                                                                                               'fiber_length': 14466,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODlEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '5355622',
                                                                                                               'fiber_length': 14826,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODgzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '5355624',
                                                                                                               'fiber_length': 14521,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzA3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '5355626',
                                                                                                               'fiber_length': 14662,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzAyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '5355624',
                                                                                                               'fiber_length': 13218,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0JEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '5355594',
                                                                                                               'fiber_length': 12154,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '5355625',
                                                                                                               'fiber_length': 13016,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '1450250',
                                                                                                               'fiber_length': 14823,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OEZFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '938225',
                                                                                                               'fiber_length': 14178,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkE1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '4773320',
                                                                                                               'fiber_length': 14481,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTRBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '2600437',
                                                                                                               'fiber_length': 14568,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -26.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '2228296',
                                                                                                               'fiber_length': 14439,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QTE4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '1467132',
                                                                                                               'fiber_length': 12765,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QTc2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '5355594',
                                                                                                               'fiber_length': 13633,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QjNGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '789241',
                                                                                                               'fiber_length': 14906,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QkE1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '5355594',
                                                                                                               'fiber_length': 13230,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQ0M2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456},
                                                                                                       {       'connected_time': '1469144',
                                                                                                               'fiber_length': 13206,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OEI4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5456}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/3',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/4',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dc:56:70'],
                                                                                       'in_nucast_pkts': '77530',
                                                                                       'in_octets': '62439363189',
                                                                                       'in_ucast_pkts': '286217570',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6385',
                                                                                       'out_octets': '1268052424980',
                                                                                       'out_ucast_pkts': '909777139',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTA3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:20'],
                                                                                       'in_nucast_pkts': '11726',
                                                                                       'in_octets': '6215172481',
                                                                                       'in_ucast_pkts': '26752970',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '967',
                                                                                       'out_octets': '92677070435',
                                                                                       'out_ucast_pkts': '66622657',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTE0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:69:60'],
                                                                                       'in_nucast_pkts': '63819',
                                                                                       'in_octets': '29946671985',
                                                                                       'in_ucast_pkts': '123997316',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5396',
                                                                                       'out_octets': '358573734229',
                                                                                       'out_ucast_pkts': '321869766',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTYwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:57:60'],
                                                                                       'in_nucast_pkts': '6011',
                                                                                       'in_octets': '763293728',
                                                                                       'in_ucast_pkts': '4151642',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '508',
                                                                                       'out_octets': '26958327833',
                                                                                       'out_ucast_pkts': '19150838',
                                                                                       'poller_hash': 'QURUTjIzMTI0REREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fa:30'],
                                                                                       'in_nucast_pkts': '74433',
                                                                                       'in_octets': '29191281244',
                                                                                       'in_ucast_pkts': '94770633',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2473',
                                                                                       'out_octets': '375087618014',
                                                                                       'out_ucast_pkts': '274711552',
                                                                                       'poller_hash': 'QURUTjIzMTI0NUM2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:7a:e0'],
                                                                                       'in_nucast_pkts': '109387',
                                                                                       'in_octets': '174666695788',
                                                                                       'in_ucast_pkts': '363170979',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3640',
                                                                                       'out_octets': '933167134518',
                                                                                       'out_ucast_pkts': '719789175',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDdEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:76:60'],
                                                                                       'in_nucast_pkts': '77528',
                                                                                       'in_octets': '32641247852',
                                                                                       'in_ucast_pkts': '126568489',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6387',
                                                                                       'out_octets': '362483997378',
                                                                                       'out_ucast_pkts': '260331198',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkFEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:75:51'],
                                                                                       'in_nucast_pkts': '1294',
                                                                                       'in_octets': '29294916315',
                                                                                       'in_ucast_pkts': '132421807',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6385',
                                                                                       'out_octets': '669835731171',
                                                                                       'out_ucast_pkts': '478712474',
                                                                                       'poller_hash': 'QURUTjIzMTI0Rjk5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ab:20'],
                                                                                       'in_nucast_pkts': '59201',
                                                                                       'in_octets': '68196870753',
                                                                                       'in_ucast_pkts': '219278469',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1926',
                                                                                       'out_octets': '551753915216',
                                                                                       'out_ucast_pkts': '400848215',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjA2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:78:a0'],
                                                                                       'in_nucast_pkts': '75678',
                                                                                       'in_octets': '16154703523',
                                                                                       'in_ucast_pkts': '100747676',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6384',
                                                                                       'out_octets': '270403354462',
                                                                                       'out_ucast_pkts': '197093412',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTdEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b5:50'],
                                                                                       'in_nucast_pkts': '74443',
                                                                                       'in_octets': '29275910389',
                                                                                       'in_ucast_pkts': '150037707',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2476',
                                                                                       'out_octets': '892621609407',
                                                                                       'out_ucast_pkts': '650891503',
                                                                                       'poller_hash': 'QURUTjIzMTI0REU4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:93:80'],
                                                                                       'in_nucast_pkts': '75269',
                                                                                       'in_octets': '36222598925',
                                                                                       'in_ucast_pkts': '207835924',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6347',
                                                                                       'out_octets': '689783876933',
                                                                                       'out_ucast_pkts': '491851054',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUVFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:8a:10'],
                                                                                       'in_nucast_pkts': '21001',
                                                                                       'in_octets': '11161088054',
                                                                                       'in_ucast_pkts': '58228056',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1735',
                                                                                       'out_octets': '218225110338',
                                                                                       'out_ucast_pkts': '157171893',
                                                                                       'poller_hash': 'QURUTjIzMTI0RUE1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b7:70'],
                                                                                       'in_nucast_pkts': '74438',
                                                                                       'in_octets': '9283026051',
                                                                                       'in_ucast_pkts': '29323621',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2480',
                                                                                       'out_octets': '82760998155',
                                                                                       'out_ucast_pkts': '67787830',
                                                                                       'poller_hash': 'QURUTjIzMTIzRTlFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:09:30'],
                                                                                       'in_nucast_pkts': '14496',
                                                                                       'in_octets': '17081454320',
                                                                                       'in_ucast_pkts': '118761085',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1226',
                                                                                       'out_octets': '325121783026',
                                                                                       'out_ucast_pkts': '222443442',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTI5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6a:e0'],
                                                                                       'in_nucast_pkts': '76202',
                                                                                       'in_octets': '25252545824',
                                                                                       'in_ucast_pkts': '81251484',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6386',
                                                                                       'out_octets': '401436456616',
                                                                                       'out_ucast_pkts': '292023857',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTI2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:50'],
                                                                                       'in_nucast_pkts': '74436',
                                                                                       'in_octets': '27238749941',
                                                                                       'in_ucast_pkts': '145779371',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2477',
                                                                                       'out_octets': '663778846525',
                                                                                       'out_ucast_pkts': '469434828',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTUzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:37:60'],
                                                                                       'in_nucast_pkts': '75662',
                                                                                       'in_octets': '24561795550',
                                                                                       'in_ucast_pkts': '147022405',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6382',
                                                                                       'out_octets': '527052006636',
                                                                                       'out_ucast_pkts': '366905481',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUFBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:24:e0'],
                                                                                       'in_nucast_pkts': '40298',
                                                                                       'in_octets': '4243954573',
                                                                                       'in_ucast_pkts': '12895145',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1324',
                                                                                       'out_octets': '37437659973',
                                                                                       'out_ucast_pkts': '27245498',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTRGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:50:b0'],
                                                                                       'in_nucast_pkts': '74438',
                                                                                       'in_octets': '18974555818',
                                                                                       'in_ucast_pkts': '69047685',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2478',
                                                                                       'out_octets': '268453639730',
                                                                                       'out_ucast_pkts': '197348696',
                                                                                       'poller_hash': 'QURUTjIzMTI0MUJGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:37:81'],
                                                                                       'in_nucast_pkts': '259',
                                                                                       'in_octets': '5241450276',
                                                                                       'in_ucast_pkts': '29662030',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1224',
                                                                                       'out_octets': '248025810009',
                                                                                       'out_ucast_pkts': '177604089',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkRDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ab:00'],
                                                                                       'in_nucast_pkts': '40855',
                                                                                       'in_octets': '34452701031',
                                                                                       'in_ucast_pkts': '98440496',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1361',
                                                                                       'out_octets': '418703305580',
                                                                                       'out_ucast_pkts': '301311136',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0RBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:84:51'],
                                                                                       'in_nucast_pkts': '592',
                                                                                       'in_octets': '18872725877',
                                                                                       'in_ucast_pkts': '54923384',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2779',
                                                                                       'out_octets': '197713632354',
                                                                                       'out_ucast_pkts': '149092553',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzM1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:01:10'],
                                                                                       'in_nucast_pkts': '74436',
                                                                                       'in_octets': '36425499264',
                                                                                       'in_ucast_pkts': '101071453',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2479',
                                                                                       'out_octets': '385596721031',
                                                                                       'out_ucast_pkts': '291389941',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QUIwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:81:70'],
                                                                                       'in_nucast_pkts': '74437',
                                                                                       'in_octets': '79842049203',
                                                                                       'in_ucast_pkts': '169807253',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2479',
                                                                                       'out_octets': '564242287128',
                                                                                       'out_ucast_pkts': '423924209',
                                                                                       'poller_hash': 'QURUTjIzMTI0QzU1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:a8:60'],
                                                                                       'in_nucast_pkts': '75672',
                                                                                       'in_octets': '45888934802',
                                                                                       'in_ucast_pkts': '130477397',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6384',
                                                                                       'out_octets': '459567680266',
                                                                                       'out_ucast_pkts': '327328544',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUUzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a3:a0'],
                                                                                       'in_nucast_pkts': '66667',
                                                                                       'in_octets': '9328434308',
                                                                                       'in_ucast_pkts': '80529385',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2222',
                                                                                       'out_octets': '255358029288',
                                                                                       'out_ucast_pkts': '176691746',
                                                                                       'poller_hash': 'QURUTjIzMTI0NjBBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ab:a0'],
                                                                                       'in_nucast_pkts': '74439',
                                                                                       'in_octets': '8471193592',
                                                                                       'in_ucast_pkts': '41579304',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2477',
                                                                                       'out_octets': '212613192305',
                                                                                       'out_ucast_pkts': '160210824',
                                                                                       'poller_hash': 'QURUTjI0MzYzQUU3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:25:f0'],
                                                                                       'in_nucast_pkts': '74439',
                                                                                       'in_octets': '16883087413',
                                                                                       'in_ucast_pkts': '15576212',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2477',
                                                                                       'out_octets': '19789729593',
                                                                                       'out_ucast_pkts': '18624355',
                                                                                       'poller_hash': 'QURUTjI0Mzg5NkY5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:2e:00'],
                                                                                       'in_nucast_pkts': '53974',
                                                                                       'in_octets': '35067790418',
                                                                                       'in_ucast_pkts': '83574139',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1803',
                                                                                       'out_octets': '555199089656',
                                                                                       'out_ucast_pkts': '398797477',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzQ5IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/4',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/4',
                                                                                      'id': 'xgspon '
                                                                                            '0/4',
                                                                                      'links': [       {       'connected_time': '2232581',
                                                                                                               'fiber_length': 3992,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTA3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '337326',
                                                                                                               'fiber_length': 4123,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '1882625',
                                                                                                               'fiber_length': 4527,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTYwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '176952',
                                                                                                               'fiber_length': 3424,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232587',
                                                                                                               'fiber_length': 3878,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NUM2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '3278982',
                                                                                                               'fiber_length': 4613,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDdEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232583',
                                                                                                               'fiber_length': 3684,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkFEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232585',
                                                                                                               'fiber_length': 3674,
                                                                                                               'poller_hash': 'QURUTjIzMTI0Rjk5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '1732403',
                                                                                                               'fiber_length': 4267,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjA2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232584',
                                                                                                               'fiber_length': 3861,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTdEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232585',
                                                                                                               'fiber_length': 3367,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REU4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232578',
                                                                                                               'fiber_length': 4569,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUVFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '604300',
                                                                                                               'fiber_length': 3857,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RUE1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232582',
                                                                                                               'fiber_length': 4488,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRTlFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 4696,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTI5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232584',
                                                                                                               'fiber_length': 4079,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTI2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232579',
                                                                                                               'fiber_length': 3719,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTUzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232583',
                                                                                                               'fiber_length': 4022,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUFBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '1208680',
                                                                                                               'fiber_length': 3775,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTRGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -24.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232586',
                                                                                                               'fiber_length': 4059,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MUJGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '427037',
                                                                                                               'fiber_length': 4481,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkRDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '1224999',
                                                                                                               'fiber_length': 3596,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0RBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '969066',
                                                                                                               'fiber_length': 4136,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzM1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232576',
                                                                                                               'fiber_length': 4574,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QUIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232586',
                                                                                                               'fiber_length': 4209,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QzU1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232576',
                                                                                                               'fiber_length': 4287,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUUzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '1999223',
                                                                                                               'fiber_length': 4437,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NjBBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232594',
                                                                                                               'fiber_length': 3543,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQUU3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '2232590',
                                                                                                               'fiber_length': 3918,
                                                                                                               'poller_hash': 'QURUTjI0Mzg5NkY5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749},
                                                                                                       {       'connected_time': '1617919',
                                                                                                               'fiber_length': 4116,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3749}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/4',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/5',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dc:7f:40'],
                                                                                       'in_nucast_pkts': '75670',
                                                                                       'in_octets': '31365538729',
                                                                                       'in_ucast_pkts': '242077913',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6407',
                                                                                       'out_octets': '829089433233',
                                                                                       'out_ucast_pkts': '590504210',
                                                                                       'poller_hash': 'QURUTjIzMTI0REYxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:56:90'],
                                                                                       'in_nucast_pkts': '75665',
                                                                                       'in_octets': '9338582863',
                                                                                       'in_ucast_pkts': '28219211',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6402',
                                                                                       'out_octets': '92332925860',
                                                                                       'out_ucast_pkts': '77335772',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDQzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:71:00'],
                                                                                       'in_nucast_pkts': '76288',
                                                                                       'in_octets': '33947972487',
                                                                                       'in_ucast_pkts': '162391808',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '599415877956',
                                                                                       'out_ucast_pkts': '432388381',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUUyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:93:30'],
                                                                                       'in_nucast_pkts': '34912',
                                                                                       'in_octets': '16610401362',
                                                                                       'in_ucast_pkts': '85236877',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2953',
                                                                                       'out_octets': '458100236742',
                                                                                       'out_ucast_pkts': '324936178',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTI2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:92:20'],
                                                                                       'in_nucast_pkts': '75672',
                                                                                       'in_octets': '85674469971',
                                                                                       'in_ucast_pkts': '348962087',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6402',
                                                                                       'out_octets': '1033461819978',
                                                                                       'out_ucast_pkts': '758406064',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDk4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:78:60'],
                                                                                       'in_nucast_pkts': '75665',
                                                                                       'in_octets': '27739920116',
                                                                                       'in_ucast_pkts': '138623369',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6403',
                                                                                       'out_octets': '922022364186',
                                                                                       'out_ucast_pkts': '680231536',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:c0'],
                                                                                       'in_nucast_pkts': '75767',
                                                                                       'in_octets': '2784186285',
                                                                                       'in_ucast_pkts': '10090190',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6405',
                                                                                       'out_octets': '35990375455',
                                                                                       'out_ucast_pkts': '26460817',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDM4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:72:20'],
                                                                                       'in_nucast_pkts': '77266',
                                                                                       'in_octets': '41123582384',
                                                                                       'in_ucast_pkts': '281917848',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6407',
                                                                                       'out_octets': '1196558938007',
                                                                                       'out_ucast_pkts': '859399516',
                                                                                       'poller_hash': 'QURUTjIzMTI0OTAwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:77:f0'],
                                                                                       'in_nucast_pkts': '26178',
                                                                                       'in_octets': '85650039323',
                                                                                       'in_ucast_pkts': '114289475',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2214',
                                                                                       'out_octets': '250406646440',
                                                                                       'out_ucast_pkts': '232063765',
                                                                                       'poller_hash': 'QURUTjIzMTI0REE2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:73:e0'],
                                                                                       'in_nucast_pkts': '34913',
                                                                                       'in_octets': '2840283832',
                                                                                       'in_ucast_pkts': '26225482',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2953',
                                                                                       'out_octets': '82265357604',
                                                                                       'out_ucast_pkts': '55788751',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTE2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:81:10'],
                                                                                       'in_nucast_pkts': '75672',
                                                                                       'in_octets': '120872407124',
                                                                                       'in_ucast_pkts': '568591069',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6405',
                                                                                       'out_octets': '1832551536574',
                                                                                       'out_ucast_pkts': '1367600034',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjJCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:5c:00'],
                                                                                       'in_nucast_pkts': '48342',
                                                                                       'in_octets': '47394728515',
                                                                                       'in_ucast_pkts': '165295208',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1612',
                                                                                       'out_octets': '587936001059',
                                                                                       'out_ucast_pkts': '429751557',
                                                                                       'poller_hash': 'QURUTjIzMTIzRjJCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:75:a0'],
                                                                                       'in_nucast_pkts': '34947',
                                                                                       'in_octets': '8301950420',
                                                                                       'in_ucast_pkts': '63624076',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2956',
                                                                                       'out_octets': '157804370907',
                                                                                       'out_ucast_pkts': '115544537',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTBEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:af:00'],
                                                                                       'in_nucast_pkts': '74455',
                                                                                       'in_octets': '20700678576',
                                                                                       'in_ucast_pkts': '51489624',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '171434867308',
                                                                                       'out_ucast_pkts': '139976763',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTI1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7f:80'],
                                                                                       'in_nucast_pkts': '58851',
                                                                                       'in_octets': '90918435541',
                                                                                       'in_ucast_pkts': '161774759',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4984',
                                                                                       'out_octets': '394429533551',
                                                                                       'out_ucast_pkts': '335201598',
                                                                                       'poller_hash': 'QURUTjIzMTI0REY3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8d:c0'],
                                                                                       'in_nucast_pkts': '75669',
                                                                                       'in_octets': '24276410552',
                                                                                       'in_ucast_pkts': '74655460',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6403',
                                                                                       'out_octets': '189926341187',
                                                                                       'out_ucast_pkts': '146275978',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUIwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:83:00'],
                                                                                       'in_nucast_pkts': '77462',
                                                                                       'in_octets': '89905790007',
                                                                                       'in_ucast_pkts': '229297951',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6403',
                                                                                       'out_octets': '764692895348',
                                                                                       'out_ucast_pkts': '563283399',
                                                                                       'poller_hash': 'QURUTjIzMTI0Rjc0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3a:f0'],
                                                                                       'in_nucast_pkts': '75669',
                                                                                       'in_octets': '59160943607',
                                                                                       'in_ucast_pkts': '354938436',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6407',
                                                                                       'out_octets': '1144840804202',
                                                                                       'out_ucast_pkts': '838247141',
                                                                                       'poller_hash': 'QURUTjIzMTI0NEVCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:50:e0'],
                                                                                       'in_nucast_pkts': '74457',
                                                                                       'in_octets': '18963024446',
                                                                                       'in_ucast_pkts': '55212384',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '173927629920',
                                                                                       'out_ucast_pkts': '131762938',
                                                                                       'poller_hash': 'QURUTjIzMTI0MDk5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:71:10'],
                                                                                       'in_nucast_pkts': '75672',
                                                                                       'in_octets': '30042834672',
                                                                                       'in_ucast_pkts': '143586465',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6407',
                                                                                       'out_octets': '405418036092',
                                                                                       'out_ucast_pkts': '371197083',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUZEIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/5',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/5',
                                                                                      'id': 'xgspon '
                                                                                            '0/5',
                                                                                      'links': [       {       'connected_time': '2232582',
                                                                                                               'fiber_length': 5944,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REYxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232584',
                                                                                                               'fiber_length': 6063,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDQzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232582',
                                                                                                               'fiber_length': 5212,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUUyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '1029869',
                                                                                                               'fiber_length': 7531,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTI2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232584',
                                                                                                               'fiber_length': 5967,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDk4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232588',
                                                                                                               'fiber_length': 5915,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232582',
                                                                                                               'fiber_length': 6210,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDM4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232584',
                                                                                                               'fiber_length': 6197,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OTAwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '771591',
                                                                                                               'fiber_length': 5510,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REE2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '1029869',
                                                                                                               'fiber_length': 7481,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTE2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232586',
                                                                                                               'fiber_length': 6120,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjJCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '1449540',
                                                                                                               'fiber_length': 5584,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRjJCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '1029870',
                                                                                                               'fiber_length': 7351,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTBEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232587',
                                                                                                               'fiber_length': 5209,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTI1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '1735925',
                                                                                                               'fiber_length': 8168,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REY3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232584',
                                                                                                               'fiber_length': 5245,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232586',
                                                                                                               'fiber_length': 6325,
                                                                                                               'poller_hash': 'QURUTjIzMTI0Rjc0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232588',
                                                                                                               'fiber_length': 8586,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NEVCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232588',
                                                                                                               'fiber_length': 6279,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MDk5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071},
                                                                                                       {       'connected_time': '2232586',
                                                                                                               'fiber_length': 6232,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.1071}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/5',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/6',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:d6:08:e0'],
                                                                                       'in_nucast_pkts': '75673',
                                                                                       'in_octets': '16375391903',
                                                                                       'in_ucast_pkts': '94315921',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6407',
                                                                                       'out_octets': '416345970025',
                                                                                       'out_ucast_pkts': '307636158',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTJBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:8d:d0'],
                                                                                       'in_nucast_pkts': '74566',
                                                                                       'in_octets': '168473594025',
                                                                                       'in_ucast_pkts': '584126252',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2480',
                                                                                       'out_octets': '1789470340955',
                                                                                       'out_ucast_pkts': '1279183727',
                                                                                       'poller_hash': 'QURUTjIzMTI0RUE0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:0c:10'],
                                                                                       'in_nucast_pkts': '165871',
                                                                                       'in_octets': '99808996995',
                                                                                       'in_ucast_pkts': '610693991',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '13915',
                                                                                       'out_octets': '954629699928',
                                                                                       'out_ucast_pkts': '923394512',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkY1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:7d:e0'],
                                                                                       'in_nucast_pkts': '75667',
                                                                                       'in_octets': '11102011713',
                                                                                       'in_ucast_pkts': '70121762',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6403',
                                                                                       'out_octets': '263864899151',
                                                                                       'out_ucast_pkts': '186484050',
                                                                                       'poller_hash': 'QURUTjIzMTIzRjE4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b8:50'],
                                                                                       'in_nucast_pkts': '76288',
                                                                                       'in_octets': '36720060506',
                                                                                       'in_ucast_pkts': '279046156',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '873140339405',
                                                                                       'out_ucast_pkts': '608947037',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTk1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7b:30'],
                                                                                       'in_nucast_pkts': '77528',
                                                                                       'in_octets': '121194056410',
                                                                                       'in_ucast_pkts': '294951548',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6409',
                                                                                       'out_octets': '999471470264',
                                                                                       'out_ucast_pkts': '796610270',
                                                                                       'poller_hash': 'QURUTjIzMTI1MENDIHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '4038463',
                                                                                       'in_octets': '1185329785',
                                                                                       'in_ucast_pkts': '1052807',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTExIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6e:30'],
                                                                                       'in_nucast_pkts': '178546',
                                                                                       'in_octets': '116680277023',
                                                                                       'in_ucast_pkts': '778141606',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5952',
                                                                                       'out_octets': '2677020422440',
                                                                                       'out_ucast_pkts': '1895563390',
                                                                                       'poller_hash': 'QURUTjIzMTI1MjA2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:b3:60'],
                                                                                       'in_nucast_pkts': '74440',
                                                                                       'in_octets': '25203479449',
                                                                                       'in_ucast_pkts': '190860845',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '522456534596',
                                                                                       'out_ucast_pkts': '377160431',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:63:40'],
                                                                                       'in_nucast_pkts': '74441',
                                                                                       'in_octets': '19241109251',
                                                                                       'in_ucast_pkts': '133289436',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '394019870049',
                                                                                       'out_ucast_pkts': '272349105',
                                                                                       'poller_hash': 'QURUTjIzMTI1MENEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:35:20'],
                                                                                       'in_nucast_pkts': '75670',
                                                                                       'in_octets': '22487044850',
                                                                                       'in_ucast_pkts': '116863573',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6405',
                                                                                       'out_octets': '568621225123',
                                                                                       'out_ucast_pkts': '402931634',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTRCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a9:70'],
                                                                                       'in_nucast_pkts': '74456',
                                                                                       'in_octets': '19525953906',
                                                                                       'in_ucast_pkts': '52938837',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '165640553610',
                                                                                       'out_ucast_pkts': '129636471',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkEzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:23:70'],
                                                                                       'in_nucast_pkts': '74451',
                                                                                       'in_octets': '136938371849',
                                                                                       'in_ucast_pkts': '615456612',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '2040535594782',
                                                                                       'out_ucast_pkts': '1469568415',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NkI0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fc:e0'],
                                                                                       'in_nucast_pkts': '3563',
                                                                                       'in_octets': '5015036161',
                                                                                       'in_ucast_pkts': '7132338',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '120',
                                                                                       'out_octets': '29720959044',
                                                                                       'out_ucast_pkts': '22368483',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzU5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:37:40'],
                                                                                       'in_nucast_pkts': '75769',
                                                                                       'in_octets': '40434694715',
                                                                                       'in_ucast_pkts': '264411380',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6404',
                                                                                       'out_octets': '970857885258',
                                                                                       'out_ucast_pkts': '728217544',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTk3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:b4:b0'],
                                                                                       'in_nucast_pkts': '68169',
                                                                                       'in_octets': '84935880657',
                                                                                       'in_ucast_pkts': '148044448',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5773',
                                                                                       'out_octets': '475176904238',
                                                                                       'out_ucast_pkts': '345196782',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUZCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:aa:e0'],
                                                                                       'in_nucast_pkts': '148421',
                                                                                       'in_octets': '7943819348',
                                                                                       'in_ucast_pkts': '14707772',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4830',
                                                                                       'out_octets': '4078142804',
                                                                                       'out_ucast_pkts': '8557943',
                                                                                       'poller_hash': 'QURUTjIzMTI0NEJBIHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '0',
                                                                                       'in_octets': '0',
                                                                                       'in_ucast_pkts': '0',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjIzMTQ1MTIwIHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '0',
                                                                                       'in_octets': '0',
                                                                                       'in_ucast_pkts': '0',
                                                                                       'oper_up': False,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjI0MzYzRDNBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:39:90'],
                                                                                       'in_nucast_pkts': '77372',
                                                                                       'in_octets': '173473435611',
                                                                                       'in_ucast_pkts': '1165260072',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '6395',
                                                                                       'out_octets': '4284904575408',
                                                                                       'out_ucast_pkts': '2972646601',
                                                                                       'poller_hash': 'QURUTjI0MzE0MTk1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:e6:40'],
                                                                                       'in_nucast_pkts': '74456',
                                                                                       'in_octets': '32172783742',
                                                                                       'in_ucast_pkts': '255295421',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '735907349926',
                                                                                       'out_ucast_pkts': '516063150',
                                                                                       'poller_hash': 'QURUTjI0MzE0MUEzIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/6',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/6',
                                                                                      'id': 'xgspon '
                                                                                            '0/6',
                                                                                      'links': [       {       'connected_time': '2232588',
                                                                                                               'fiber_length': 5521,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTJBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232585',
                                                                                                               'fiber_length': 5919,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RUE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '4851897',
                                                                                                               'fiber_length': 5995,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkY1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232589',
                                                                                                               'fiber_length': 5797,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRjE4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232587',
                                                                                                               'fiber_length': 6115,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTk1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232589',
                                                                                                               'fiber_length': 6181,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MENDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232588',
                                                                                                               'fiber_length': 6734,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTExIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '5355625',
                                                                                                               'fiber_length': 7635,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MjA2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232588',
                                                                                                               'fiber_length': 5523,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232584',
                                                                                                               'fiber_length': 5847,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MENEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232587',
                                                                                                               'fiber_length': 6444,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTRCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232580',
                                                                                                               'fiber_length': 6030,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkEzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232586',
                                                                                                               'fiber_length': 6378,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NkI0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '106257',
                                                                                                               'fiber_length': 6744,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzU5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232584',
                                                                                                               'fiber_length': 6051,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTk3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2011665',
                                                                                                               'fiber_length': 6816,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '5355629',
                                                                                                               'fiber_length': 7484,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NEJBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '11998',
                                                                                                               'fiber_length': 5781,
                                                                                                               'poller_hash': 'QURUTjIzMTQ1MTIwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '1550263',
                                                                                                               'fiber_length': 6741,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRDNBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232592',
                                                                                                               'fiber_length': 6741,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MTk1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625},
                                                                                                       {       'connected_time': '2232592',
                                                                                                               'fiber_length': 6541,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MUEzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4625}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/6',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/7',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:4c:01:c0'],
                                                                                       'in_nucast_pkts': '158840',
                                                                                       'in_octets': '49755568489',
                                                                                       'in_ucast_pkts': '151433781',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5302',
                                                                                       'out_octets': '504357300674',
                                                                                       'out_ucast_pkts': '390076108',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDYxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:04:00'],
                                                                                       'in_nucast_pkts': '76290',
                                                                                       'in_octets': '51219008561',
                                                                                       'in_ucast_pkts': '286936107',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2482',
                                                                                       'out_octets': '978368486421',
                                                                                       'out_ucast_pkts': '730330180',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTcxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:82:c0'],
                                                                                       'in_nucast_pkts': '183322',
                                                                                       'in_octets': '237506751290',
                                                                                       'in_ucast_pkts': '1415059809',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15384',
                                                                                       'out_octets': '4720683348841',
                                                                                       'out_ucast_pkts': '3430807496',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTUyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:70'],
                                                                                       'in_nucast_pkts': '90358',
                                                                                       'in_octets': '46234900077',
                                                                                       'in_ucast_pkts': '306741005',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '7625',
                                                                                       'out_octets': '1355647793471',
                                                                                       'out_ucast_pkts': '1038143637',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3a:e0'],
                                                                                       'in_nucast_pkts': '12185',
                                                                                       'in_octets': '7659325026',
                                                                                       'in_ucast_pkts': '25544182',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1032',
                                                                                       'out_octets': '66306366365',
                                                                                       'out_ucast_pkts': '49152844',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTZBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:d8:90'],
                                                                                       'in_nucast_pkts': '160098',
                                                                                       'in_octets': '22879317435',
                                                                                       'in_ucast_pkts': '105664544',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5304',
                                                                                       'out_octets': '461411238133',
                                                                                       'out_ucast_pkts': '341084502',
                                                                                       'poller_hash': 'QURUTjIzMTI0NDVDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a7:30'],
                                                                                       'in_nucast_pkts': '25252',
                                                                                       'in_octets': '20642883768',
                                                                                       'in_ucast_pkts': '113167798',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '843',
                                                                                       'out_octets': '530696181797',
                                                                                       'out_ucast_pkts': '381264008',
                                                                                       'poller_hash': 'QURUTjIzMTI0NEJDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:50:10'],
                                                                                       'in_nucast_pkts': '178545',
                                                                                       'in_octets': '153882517082',
                                                                                       'in_ucast_pkts': '446613554',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5952',
                                                                                       'out_octets': '1172709230629',
                                                                                       'out_ucast_pkts': '920785112',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTNBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8f:d0'],
                                                                                       'in_nucast_pkts': '183654',
                                                                                       'in_octets': '11693088997',
                                                                                       'in_ucast_pkts': '32136695',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15381',
                                                                                       'out_octets': '44985435977',
                                                                                       'out_ucast_pkts': '40222103',
                                                                                       'poller_hash': 'QURUTjIzMTI0MUMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:92:e0'],
                                                                                       'in_nucast_pkts': '114326',
                                                                                       'in_octets': '116923921474',
                                                                                       'in_ucast_pkts': '644934805',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '9678',
                                                                                       'out_octets': '2824710093476',
                                                                                       'out_ucast_pkts': '2018653090',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTU4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:02:f0'],
                                                                                       'in_nucast_pkts': '178527',
                                                                                       'in_octets': '19649965190',
                                                                                       'in_ucast_pkts': '37384091',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5952',
                                                                                       'out_octets': '72406994171',
                                                                                       'out_ucast_pkts': '64572035',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:65:a0'],
                                                                                       'in_nucast_pkts': '181960',
                                                                                       'in_octets': '118797021115',
                                                                                       'in_ucast_pkts': '220053985',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15383',
                                                                                       'out_octets': '700092925660',
                                                                                       'out_ucast_pkts': '526170506',
                                                                                       'poller_hash': 'QURUTjIzMTI1MEJCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ce:70'],
                                                                                       'in_nucast_pkts': '178708',
                                                                                       'in_octets': '78648404371',
                                                                                       'in_ucast_pkts': '538668066',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5952',
                                                                                       'out_octets': '2649428185093',
                                                                                       'out_ucast_pkts': '1818879008',
                                                                                       'poller_hash': 'QURUTjIzMTQ0Nzk2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:84:90'],
                                                                                       'in_nucast_pkts': '67546',
                                                                                       'in_octets': '30381695416',
                                                                                       'in_ucast_pkts': '148236549',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '5627',
                                                                                       'out_octets': '731272078042',
                                                                                       'out_ucast_pkts': '522495069',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODZDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:78:80'],
                                                                                       'in_nucast_pkts': '183326',
                                                                                       'in_octets': '31927005547',
                                                                                       'in_ucast_pkts': '73791556',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '15369',
                                                                                       'out_octets': '148961018578',
                                                                                       'out_ucast_pkts': '130331928',
                                                                                       'poller_hash': 'QURUTjIzMTQ0Njk5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:50'],
                                                                                       'in_nucast_pkts': '53275',
                                                                                       'in_octets': '66181121606',
                                                                                       'in_ucast_pkts': '251617020',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4507',
                                                                                       'out_octets': '975716368421',
                                                                                       'out_ucast_pkts': '706761030',
                                                                                       'poller_hash': 'QURUTjIzMTQ0Q0MzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:f1:00'],
                                                                                       'in_nucast_pkts': '105741',
                                                                                       'in_octets': '125867370590',
                                                                                       'in_ucast_pkts': '380855656',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3662',
                                                                                       'out_octets': '1578961306528',
                                                                                       'out_ucast_pkts': '1164173803',
                                                                                       'poller_hash': 'QURUTjIzMTI0MjE0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:a9:00'],
                                                                                       'in_nucast_pkts': '16418',
                                                                                       'in_octets': '8947474311',
                                                                                       'in_ucast_pkts': '11200592',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '548',
                                                                                       'out_octets': '30754950510',
                                                                                       'out_ucast_pkts': '24258646',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzRFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:fd:b0'],
                                                                                       'in_nucast_pkts': '16436',
                                                                                       'in_octets': '7606830915',
                                                                                       'in_ucast_pkts': '17153075',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '549',
                                                                                       'out_octets': '59503336499',
                                                                                       'out_ucast_pkts': '44103819',
                                                                                       'poller_hash': 'QURUTjI0MzE0NEZCIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/7',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/7',
                                                                                      'id': 'xgspon '
                                                                                            '0/7',
                                                                                      'links': [       {       'connected_time': '4773323',
                                                                                                               'fiber_length': 10820,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDYxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '2232590',
                                                                                                               'fiber_length': 10756,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTcxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '5355629',
                                                                                                               'fiber_length': 9016,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTUyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '2658219',
                                                                                                               'fiber_length': 10425,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '359118',
                                                                                                               'fiber_length': 10462,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTZBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '4773324',
                                                                                                               'fiber_length': 10872,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NDVDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '756858',
                                                                                                               'fiber_length': 9328,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NEJDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '5355629',
                                                                                                               'fiber_length': 9589,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTNBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '5355630',
                                                                                                               'fiber_length': 10748,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MUMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '3371909',
                                                                                                               'fiber_length': 10811,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTU4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '5355597',
                                                                                                               'fiber_length': 8976,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '5355597',
                                                                                                               'fiber_length': 8993,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MEJCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '5355628',
                                                                                                               'fiber_length': 9375,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0Nzk2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '1960821',
                                                                                                               'fiber_length': 10744,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODZDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '5355629',
                                                                                                               'fiber_length': 8892,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0Njk5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '1571617',
                                                                                                               'fiber_length': 10521,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0MzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '3147736',
                                                                                                               'fiber_length': 9856,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MjE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '491760',
                                                                                                               'fiber_length': 10106,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzRFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429},
                                                                                                       {       'connected_time': '492326',
                                                                                                               'fiber_length': 10046,
                                                                                                               'poller_hash': 'QURUTjI0MzE0NEZCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.429}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/7',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/8',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dd:73:50'],
                                                                                       'in_nucast_pkts': '14488',
                                                                                       'in_octets': '11123096046',
                                                                                       'in_ucast_pkts': '13511000',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '30079611459',
                                                                                       'out_ucast_pkts': '25371278',
                                                                                       'poller_hash': 'QURUTjIzMTI0REZCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:75:a0'],
                                                                                       'in_nucast_pkts': '14487',
                                                                                       'in_octets': '3409732582',
                                                                                       'in_ucast_pkts': '24706889',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '69705065738',
                                                                                       'out_ucast_pkts': '48123549',
                                                                                       'poller_hash': 'QURUTjIzMTI0REM3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:76:a0'],
                                                                                       'in_nucast_pkts': '11102',
                                                                                       'in_octets': '8102513606',
                                                                                       'in_ucast_pkts': '28691963',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '938',
                                                                                       'out_octets': '80493637516',
                                                                                       'out_ucast_pkts': '57497526',
                                                                                       'poller_hash': 'QURUTjIzMTI0RUZBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:81:80'],
                                                                                       'in_nucast_pkts': '14711',
                                                                                       'in_octets': '6058566606',
                                                                                       'in_ucast_pkts': '14324366',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '56384755529',
                                                                                       'out_ucast_pkts': '43173980',
                                                                                       'poller_hash': 'QURUTjIzMTI0RURGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:79:10'],
                                                                                       'in_nucast_pkts': '14487',
                                                                                       'in_octets': '1408453082',
                                                                                       'in_ucast_pkts': '7039557',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '16740090927',
                                                                                       'out_ucast_pkts': '12732063',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUY3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:71:e0'],
                                                                                       'in_nucast_pkts': '14489',
                                                                                       'in_octets': '17033134539',
                                                                                       'in_ucast_pkts': '38082733',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '83737263849',
                                                                                       'out_ucast_pkts': '65683823',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTg0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:08:10'],
                                                                                       'in_nucast_pkts': '14491',
                                                                                       'in_octets': '50179152392',
                                                                                       'in_ucast_pkts': '165755454',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '509052632069',
                                                                                       'out_ucast_pkts': '368118322',
                                                                                       'poller_hash': 'QURUTjIzMTI1MUY2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:32:11'],
                                                                                       'in_nucast_pkts': '264',
                                                                                       'in_octets': '16626695046',
                                                                                       'in_ucast_pkts': '31430246',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '110967086335',
                                                                                       'out_ucast_pkts': '87932377',
                                                                                       'poller_hash': 'QURUTjIzMTI0RUQzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:07:b0'],
                                                                                       'in_nucast_pkts': '14486',
                                                                                       'in_octets': '454089503',
                                                                                       'in_ucast_pkts': '1098388',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '2165466570',
                                                                                       'out_ucast_pkts': '1700019',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTQzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:04:10'],
                                                                                       'in_nucast_pkts': '14486',
                                                                                       'in_octets': '46466954692',
                                                                                       'in_ucast_pkts': '162384018',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '629137532340',
                                                                                       'out_ucast_pkts': '440714131',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTAzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8e:b0'],
                                                                                       'in_nucast_pkts': '14494',
                                                                                       'in_octets': '6049417954',
                                                                                       'in_ucast_pkts': '34033927',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1226',
                                                                                       'out_octets': '211119778048',
                                                                                       'out_ucast_pkts': '156321241',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTJFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:07:60'],
                                                                                       'in_nucast_pkts': '14494',
                                                                                       'in_octets': '4484194474',
                                                                                       'in_ucast_pkts': '25912450',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1226',
                                                                                       'out_octets': '193882004883',
                                                                                       'out_ucast_pkts': '132819587',
                                                                                       'poller_hash': 'QURUTjIzMTQ0ODNCIHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '29989',
                                                                                       'in_octets': '1221926735',
                                                                                       'in_ucast_pkts': '1237098',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '451',
                                                                                       'out_octets': '1213350778',
                                                                                       'out_ucast_pkts': '1231323',
                                                                                       'poller_hash': 'QURUTjIzMTQ0NzQ4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b3:a0'],
                                                                                       'in_nucast_pkts': '14261',
                                                                                       'in_octets': '4585573724',
                                                                                       'in_ucast_pkts': '15384444',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '54811201689',
                                                                                       'out_ucast_pkts': '43212303',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OEQxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:10'],
                                                                                       'in_nucast_pkts': '14493',
                                                                                       'in_octets': '4479141903',
                                                                                       'in_ucast_pkts': '41446317',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '130555458155',
                                                                                       'out_ucast_pkts': '89477649',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTlDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3c:30'],
                                                                                       'in_nucast_pkts': '9115',
                                                                                       'in_octets': '2830692519',
                                                                                       'in_ucast_pkts': '22765801',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '772',
                                                                                       'out_octets': '70890071883',
                                                                                       'out_ucast_pkts': '48865187',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjJDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:93:c0'],
                                                                                       'in_nucast_pkts': '14490',
                                                                                       'in_octets': '351591502',
                                                                                       'in_ucast_pkts': '528810',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '372614589',
                                                                                       'out_ucast_pkts': '498076',
                                                                                       'poller_hash': 'QURUTjIzMTQ0Q0Q1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:75:00'],
                                                                                       'in_nucast_pkts': '14481',
                                                                                       'in_octets': '3791904941',
                                                                                       'in_ucast_pkts': '8423097',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1225',
                                                                                       'out_octets': '25578469812',
                                                                                       'out_ucast_pkts': '20420027',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QzQ2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:9c:b0'],
                                                                                       'in_nucast_pkts': '14260',
                                                                                       'in_octets': '18330245862',
                                                                                       'in_ucast_pkts': '46570425',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '133623055334',
                                                                                       'out_ucast_pkts': '99196688',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:d0:70'],
                                                                                       'in_nucast_pkts': '8972',
                                                                                       'in_octets': '1209998692',
                                                                                       'in_ucast_pkts': '5042376',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '300',
                                                                                       'out_octets': '17772443257',
                                                                                       'out_ucast_pkts': '13503826',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTcwIHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '0',
                                                                                       'in_octets': '0',
                                                                                       'in_ucast_pkts': '0',
                                                                                       'oper_up': False,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RjAwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:51:30'],
                                                                                       'in_nucast_pkts': '14258',
                                                                                       'in_octets': '5556961097',
                                                                                       'in_ucast_pkts': '44970447',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '137130796457',
                                                                                       'out_ucast_pkts': '99273180',
                                                                                       'poller_hash': 'QURUTjIzMTI0N0ExIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:26:a0'],
                                                                                       'in_nucast_pkts': '14260',
                                                                                       'in_octets': '4745824547',
                                                                                       'in_ucast_pkts': '28323993',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '91040367405',
                                                                                       'out_ucast_pkts': '63611691',
                                                                                       'poller_hash': 'QURUTjIzMTI0QzZBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ac:60'],
                                                                                       'in_nucast_pkts': '14413',
                                                                                       'in_octets': '3836087366',
                                                                                       'in_ucast_pkts': '47521683',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '150917148362',
                                                                                       'out_ucast_pkts': '100852685',
                                                                                       'poller_hash': 'QURUTjI0MzYzREM5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:27:10'],
                                                                                       'in_nucast_pkts': '12315',
                                                                                       'in_octets': '24721684604',
                                                                                       'in_ucast_pkts': '136340685',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '411',
                                                                                       'out_octets': '411841066531',
                                                                                       'out_ucast_pkts': '304654174',
                                                                                       'poller_hash': 'QURUTjI0MzE0MzE0IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/8',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/8',
                                                                                      'id': 'xgspon '
                                                                                            '0/8',
                                                                                      'links': [       {       'connected_time': '427037',
                                                                                                               'fiber_length': 2183,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REZCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427037',
                                                                                                               'fiber_length': 2394,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REM3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '327189',
                                                                                                               'fiber_length': 2496,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RUZBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427037',
                                                                                                               'fiber_length': 3018,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RURGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 2254,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUY3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427038',
                                                                                                               'fiber_length': 3256,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTg0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427036',
                                                                                                               'fiber_length': 2382,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MUY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427033',
                                                                                                               'fiber_length': 2629,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RUQzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427033',
                                                                                                               'fiber_length': 3430,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTQzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427036',
                                                                                                               'fiber_length': 3301,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTAzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427036',
                                                                                                               'fiber_length': 3383,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTJFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427033',
                                                                                                               'fiber_length': 2182,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0ODNCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 3353,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0NzQ4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427031',
                                                                                                               'fiber_length': 2661,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OEQxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 3282,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTlDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '268464',
                                                                                                               'fiber_length': 4011,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjJDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 3249,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0Q1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427030',
                                                                                                               'fiber_length': 2665,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QzQ2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 3298,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '268464',
                                                                                                               'fiber_length': 3865,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTcwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427029',
                                                                                                               'fiber_length': 3323,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RjAwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427039',
                                                                                                               'fiber_length': 3094,
                                                                                                               'poller_hash': 'QURUTjIzMTI0N0ExIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427039',
                                                                                                               'fiber_length': 3062,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QzZBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '427043',
                                                                                                               'fiber_length': 3347,
                                                                                                               'poller_hash': 'QURUTjI0MzYzREM5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541},
                                                                                                       {       'connected_time': '368685',
                                                                                                               'fiber_length': 3270,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MzE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2541}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/8',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/9',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:a9:50'],
                                                                                       'in_nucast_pkts': '14745',
                                                                                       'in_octets': '27560742812',
                                                                                       'in_ucast_pkts': '124826761',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1226',
                                                                                       'out_octets': '441183690755',
                                                                                       'out_ucast_pkts': '317382052',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDkxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7e:50'],
                                                                                       'in_nucast_pkts': '14487',
                                                                                       'in_octets': '5327360268',
                                                                                       'in_ucast_pkts': '12580045',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '52274437849',
                                                                                       'out_ucast_pkts': '41675343',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTdBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:b1:c0'],
                                                                                       'in_nucast_pkts': '14846',
                                                                                       'in_octets': '77541879154',
                                                                                       'in_ucast_pkts': '144166953',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '363149263500',
                                                                                       'out_ucast_pkts': '278103635',
                                                                                       'poller_hash': 'QURUTjIzMTIzRTlDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:9c:b0'],
                                                                                       'in_nucast_pkts': '14613',
                                                                                       'in_octets': '37044453415',
                                                                                       'in_ucast_pkts': '60328336',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '186031888166',
                                                                                       'out_ucast_pkts': '144934536',
                                                                                       'poller_hash': 'QURUTjIzMTI0REVCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:87:80'],
                                                                                       'in_nucast_pkts': '14488',
                                                                                       'in_octets': '7182857775',
                                                                                       'in_ucast_pkts': '36284466',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '102633796554',
                                                                                       'out_ucast_pkts': '77660339',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDkzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:b4:c0'],
                                                                                       'in_nucast_pkts': '14259',
                                                                                       'in_octets': '8477882963',
                                                                                       'in_ucast_pkts': '18924820',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '84457489770',
                                                                                       'out_ucast_pkts': '65016933',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDQ2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:88:60'],
                                                                                       'in_nucast_pkts': '14487',
                                                                                       'in_octets': '7352355370',
                                                                                       'in_ucast_pkts': '32956583',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '201948414523',
                                                                                       'out_ucast_pkts': '142178378',
                                                                                       'poller_hash': 'QURUTjIzMTI0OTFFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:b1'],
                                                                                       'in_nucast_pkts': '342',
                                                                                       'in_octets': '9898390739',
                                                                                       'in_ucast_pkts': '22190380',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '80594217954',
                                                                                       'out_ucast_pkts': '61996459',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTNBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:7d:c0'],
                                                                                       'in_nucast_pkts': '14846',
                                                                                       'in_octets': '10434238744',
                                                                                       'in_ucast_pkts': '50310389',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '257756685243',
                                                                                       'out_ucast_pkts': '190759006',
                                                                                       'poller_hash': 'QURUTjIzMTI0REYzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:74:b0'],
                                                                                       'in_nucast_pkts': '14487',
                                                                                       'in_octets': '790422525',
                                                                                       'in_ucast_pkts': '2958689',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '7130903493',
                                                                                       'out_ucast_pkts': '5656972',
                                                                                       'poller_hash': 'QURUTjIzMTI1MThFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:2a:40'],
                                                                                       'in_nucast_pkts': '14845',
                                                                                       'in_octets': '12476280102',
                                                                                       'in_ucast_pkts': '51926230',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '266208528670',
                                                                                       'out_ucast_pkts': '191938908',
                                                                                       'poller_hash': 'QURUTjIzMTI0RENFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8d:d0'],
                                                                                       'in_nucast_pkts': '14487',
                                                                                       'in_octets': '1975280843',
                                                                                       'in_ucast_pkts': '6538177',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '17893158561',
                                                                                       'out_ucast_pkts': '14760521',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTIyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:db:66:60'],
                                                                                       'in_nucast_pkts': '14845',
                                                                                       'in_octets': '11065164565',
                                                                                       'in_ucast_pkts': '73336480',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '276809440559',
                                                                                       'out_ucast_pkts': '202956573',
                                                                                       'poller_hash': 'QURUTjIzMTI0RDlFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8a:c0'],
                                                                                       'in_nucast_pkts': '14515',
                                                                                       'in_octets': '3639779298',
                                                                                       'in_ucast_pkts': '7358348',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '26974714761',
                                                                                       'out_ucast_pkts': '22277576',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTJEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:24:f0'],
                                                                                       'in_nucast_pkts': '3685',
                                                                                       'in_octets': '5017196441',
                                                                                       'in_ucast_pkts': '12594970',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '312',
                                                                                       'out_octets': '43937685021',
                                                                                       'out_ucast_pkts': '32852011',
                                                                                       'poller_hash': 'QURUTjIzMTI0RERGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ad:50'],
                                                                                       'in_nucast_pkts': '14258',
                                                                                       'in_octets': '8179937368',
                                                                                       'in_ucast_pkts': '48036719',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '197105393156',
                                                                                       'out_ucast_pkts': '143812436',
                                                                                       'poller_hash': 'QURUTjIzMTI0MDhDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:73:90'],
                                                                                       'in_nucast_pkts': '14487',
                                                                                       'in_octets': '41206052671',
                                                                                       'in_ucast_pkts': '67857055',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '212344229386',
                                                                                       'out_ucast_pkts': '157460453',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDFEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:6e:20'],
                                                                                       'in_nucast_pkts': '14496',
                                                                                       'in_octets': '7685322089',
                                                                                       'in_ucast_pkts': '61007810',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '289160274295',
                                                                                       'out_ucast_pkts': '198393026',
                                                                                       'poller_hash': 'QURUTjIzMTI0REY1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:88:c0'],
                                                                                       'in_nucast_pkts': '14487',
                                                                                       'in_octets': '11594417795',
                                                                                       'in_ucast_pkts': '74432212',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '237652363977',
                                                                                       'out_ucast_pkts': '165639326',
                                                                                       'poller_hash': 'QURUTjIzMTI0NzEzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:7b:70'],
                                                                                       'in_nucast_pkts': '14779',
                                                                                       'in_octets': '11099335936',
                                                                                       'in_ucast_pkts': '67664231',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '185374996354',
                                                                                       'out_ucast_pkts': '129135376',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTE5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:82:00'],
                                                                                       'in_nucast_pkts': '14488',
                                                                                       'in_octets': '3843776317',
                                                                                       'in_ucast_pkts': '11663473',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '54436132368',
                                                                                       'out_ucast_pkts': '43264779',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTY3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:83:40'],
                                                                                       'in_nucast_pkts': '14538',
                                                                                       'in_octets': '19593481234',
                                                                                       'in_ucast_pkts': '59439003',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1229',
                                                                                       'out_octets': '246726206538',
                                                                                       'out_ucast_pkts': '183721766',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTNGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:03:e0'],
                                                                                       'in_nucast_pkts': '14687',
                                                                                       'in_octets': '16436354372',
                                                                                       'in_ucast_pkts': '47215406',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '487',
                                                                                       'out_octets': '132008364272',
                                                                                       'out_ucast_pkts': '102344793',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTIzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:70:f0'],
                                                                                       'in_nucast_pkts': '14493',
                                                                                       'in_octets': '7921703286',
                                                                                       'in_ucast_pkts': '58170493',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '169020056180',
                                                                                       'out_ucast_pkts': '133811964',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTI3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fa:00'],
                                                                                       'in_nucast_pkts': '14260',
                                                                                       'in_octets': '6557138016',
                                                                                       'in_ucast_pkts': '37645152',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '163595952937',
                                                                                       'out_ucast_pkts': '114245219',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTY0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:87:c0'],
                                                                                       'in_nucast_pkts': '14490',
                                                                                       'in_octets': '3504213634',
                                                                                       'in_ucast_pkts': '17893442',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '60693102340',
                                                                                       'out_ucast_pkts': '43963678',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTZCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:03:d0'],
                                                                                       'in_nucast_pkts': '2283',
                                                                                       'in_octets': '217397666',
                                                                                       'in_ucast_pkts': '576131',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '76',
                                                                                       'out_octets': '699514356',
                                                                                       'out_ucast_pkts': '834306',
                                                                                       'poller_hash': 'QURUTjIzMTIzRkVDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:88:b0'],
                                                                                       'in_nucast_pkts': '14491',
                                                                                       'in_octets': '1222611241',
                                                                                       'in_ucast_pkts': '4483675',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '14554324780',
                                                                                       'out_ucast_pkts': '11327632',
                                                                                       'poller_hash': 'QURUTjIzMTIzRUE4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:8b:c0'],
                                                                                       'in_nucast_pkts': '14488',
                                                                                       'in_octets': '2563843765',
                                                                                       'in_ucast_pkts': '23288569',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '94069604523',
                                                                                       'out_ucast_pkts': '65745034',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTFGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:82:30'],
                                                                                       'in_nucast_pkts': '14489',
                                                                                       'in_octets': '4871916230',
                                                                                       'in_ucast_pkts': '19295162',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '53219380465',
                                                                                       'out_ucast_pkts': '46179521',
                                                                                       'poller_hash': 'QURUTjIzMTI0MTI4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:86:d1'],
                                                                                       'in_nucast_pkts': '265',
                                                                                       'in_octets': '9081539991',
                                                                                       'in_ucast_pkts': '24965850',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '102797141730',
                                                                                       'out_ucast_pkts': '72870428',
                                                                                       'poller_hash': 'QURUTjIzMTQ0N0E0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:04:e0'],
                                                                                       'in_nucast_pkts': '14259',
                                                                                       'in_octets': '9556335117',
                                                                                       'in_ucast_pkts': '22133906',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '62700366416',
                                                                                       'out_ucast_pkts': '51739233',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTY5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3b:c0'],
                                                                                       'in_nucast_pkts': '14487',
                                                                                       'in_octets': '31752714694',
                                                                                       'in_ucast_pkts': '44450073',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1227',
                                                                                       'out_octets': '81359961853',
                                                                                       'out_ucast_pkts': '65945504',
                                                                                       'poller_hash': 'QURUTjIzMTI0MDA4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b7:d0'],
                                                                                       'in_nucast_pkts': '14260',
                                                                                       'in_octets': '17809587612',
                                                                                       'in_ucast_pkts': '67320477',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '476',
                                                                                       'out_octets': '270026241546',
                                                                                       'out_ucast_pkts': '199676509',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUQzIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/9',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/9',
                                                                                      'id': 'xgspon '
                                                                                            '0/9',
                                                                                      'links': [       {       'connected_time': '427021',
                                                                                                               'fiber_length': 5783,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDkxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427033',
                                                                                                               'fiber_length': 5853,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTdBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427040',
                                                                                                               'fiber_length': 5669,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRTlDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 3861,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REVCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427024',
                                                                                                               'fiber_length': 4090,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDkzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427036',
                                                                                                               'fiber_length': 4612,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDQ2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427022',
                                                                                                               'fiber_length': 5476,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OTFFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427025',
                                                                                                               'fiber_length': 5824,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTNBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427040',
                                                                                                               'fiber_length': 3643,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REYzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427024',
                                                                                                               'fiber_length': 3909,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MThFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 5338,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RENFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427034',
                                                                                                               'fiber_length': 3743,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTIyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 4273,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RDlFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427034',
                                                                                                               'fiber_length': 4867,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTJEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '108153',
                                                                                                               'fiber_length': 4825,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RERGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427027',
                                                                                                               'fiber_length': 5423,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MDhDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427032',
                                                                                                               'fiber_length': 4938,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDFEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427034',
                                                                                                               'fiber_length': 5198,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REY1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427026',
                                                                                                               'fiber_length': 5184,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NzEzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427040',
                                                                                                               'fiber_length': 5202,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTE5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427038',
                                                                                                               'fiber_length': 5100,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTY3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427039',
                                                                                                               'fiber_length': 5421,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTNGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427039',
                                                                                                               'fiber_length': 4748,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTIzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427035',
                                                                                                               'fiber_length': 4716,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTI3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427034',
                                                                                                               'fiber_length': 4657,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTY0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427034',
                                                                                                               'fiber_length': 5879,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTZCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '68291',
                                                                                                               'fiber_length': 5819,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRkVDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427036',
                                                                                                               'fiber_length': 4449,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRUE4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427039',
                                                                                                               'fiber_length': 4391,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTFGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427040',
                                                                                                               'fiber_length': 5794,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MTI4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427032',
                                                                                                               'fiber_length': 5397,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0N0E0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427038',
                                                                                                               'fiber_length': 4383,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTY5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427040',
                                                                                                               'fiber_length': 5786,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MDA4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955},
                                                                                                       {       'connected_time': '427025',
                                                                                                               'fiber_length': 5736,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUQzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3955}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/9',
                                                                                      'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/1',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:ce',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/1',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMQ==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/2',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:cf',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/2',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMg==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/3',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:d0',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/3',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMw==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/4',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:d1',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/4',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvNA==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'management 0/1',
                                                              'mac_address': '38:f8:f6:ac:65:c8',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'management 0/1',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgbWFuYWdlbWVudCAwLzE=',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/1',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '56931807',
                                                              'in_octets': '40995186676973',
                                                              'in_ucast_pkts': '443630565079',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:ca',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/1',
                                                              'oper_up': True,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '60638424',
                                                              'out_octets': '48218558459893',
                                                              'out_ucast_pkts': '173991433096',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC8x',
                                                              'speed': '10000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/2',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:cb',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/2',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC8y',
                                                              'speed': '10000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/3',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:cc',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/3',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC8z',
                                                              'speed': '10000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/4',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:ac:65:cd',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/4',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC80',
                                                              'speed': '10000000000',
                                                              'type': 6}],
                                'manufacturer': 'Adtran',
                                'mem': 53.82,
                                'model': 'SDX 6320-16',
                                'name': 'IndianBayou-x1',
                                'poller_hash': 'b2x0IExCQURUTjIzMjJBQTIzNA==',
                                'serial_number': 'LBADTN2322AA234',
                                'sw_version': 'sdx6320-RL-23.5-33',
                                'temperature': 72.0,
                                'uptime': '5355835'},
                'peers': [       {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14745',
                                                                       'in_octets': '27560742812',
                                                                       'in_ucast_pkts': '124826761',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1226',
                                                                       'out_octets': '441183690755',
                                                                       'out_ucast_pkts': '317382052',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1226',
                                                                       'in_octets': '441183690755',
                                                                       'in_ucast_pkts': '317382052',
                                                                       'links': [       {       'in_nucast_pkts': '1226',
                                                                                                'in_octets': '441183690755',
                                                                                                'in_ucast_pkts': '317382052',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14745',
                                                                                                'out_octets': '27560742812',
                                                                                                'out_ucast_pkts': '124826761',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14745',
                                                                       'out_octets': '27560742812',
                                                                       'out_ucast_pkts': '124826761',
                                                                       'poller_hash': 'QURUTjIzMTI0RDkxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427021',
                                                                                                                        'fiber_length': 5783,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDkxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ5MQ==',
                                         'serial_number': 'ADTN23124D91',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427021'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14488',
                                                                       'in_octets': '10550726467',
                                                                       'in_ucast_pkts': '78959047',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '372201121743',
                                                                       'out_ucast_pkts': '254564839',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '372201121743',
                                                                       'in_ucast_pkts': '254564839',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '372201121743',
                                                                                                'in_ucast_pkts': '254564839',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14488',
                                                                                                'out_octets': '10550726467',
                                                                                                'out_ucast_pkts': '78959047',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14488',
                                                                       'out_octets': '10550726467',
                                                                       'out_ucast_pkts': '78959047',
                                                                       'poller_hash': 'QURUTjIzMTI0RDY4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427041',
                                                                                                                        'fiber_length': 5677,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDY4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ2OA==',
                                         'serial_number': 'ADTN23124D68',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427041'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14487',
                                                                       'in_octets': '5327360268',
                                                                       'in_ucast_pkts': '12580045',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '52274437849',
                                                                       'out_ucast_pkts': '41675343',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '52274437849',
                                                                       'in_ucast_pkts': '41675343',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '52274437849',
                                                                                                'in_ucast_pkts': '41675343',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14487',
                                                                                                'out_octets': '5327360268',
                                                                                                'out_ucast_pkts': '12580045',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14487',
                                                                       'out_octets': '5327360268',
                                                                       'out_ucast_pkts': '12580045',
                                                                       'poller_hash': 'QURUTjIzMTI0RTdBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427033',
                                                                                                                        'fiber_length': 5853,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTdBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU3QQ==',
                                         'serial_number': 'ADTN23124E7A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427033'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14491',
                                                                       'in_octets': '2971603812',
                                                                       'in_ucast_pkts': '6478397',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '32270863710',
                                                                       'out_ucast_pkts': '26537850',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '32270863710',
                                                                       'in_ucast_pkts': '26537850',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '32270863710',
                                                                                                'in_ucast_pkts': '26537850',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14491',
                                                                                                'out_octets': '2971603812',
                                                                                                'out_ucast_pkts': '6478397',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14491',
                                                                       'out_octets': '2971603812',
                                                                       'out_ucast_pkts': '6478397',
                                                                       'poller_hash': 'QURUTjIzMTI0RTE4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427040',
                                                                                                                        'fiber_length': 6613,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTE4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxOA==',
                                         'serial_number': 'ADTN23124E18',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427040'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14846',
                                                                       'in_octets': '77541879154',
                                                                       'in_ucast_pkts': '144166953',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '363149263500',
                                                                       'out_ucast_pkts': '278103635',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '363149263500',
                                                                       'in_ucast_pkts': '278103635',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '363149263500',
                                                                                                'in_ucast_pkts': '278103635',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14846',
                                                                                                'out_octets': '77541879154',
                                                                                                'out_ucast_pkts': '144166953',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14846',
                                                                       'out_octets': '77541879154',
                                                                       'out_ucast_pkts': '144166953',
                                                                       'poller_hash': 'QURUTjIzMTIzRTlDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427040',
                                                                                                                        'fiber_length': 5669,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRTlDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0U5Qw==',
                                         'serial_number': 'ADTN23123E9C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427040'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14613',
                                                                       'in_octets': '37044453415',
                                                                       'in_ucast_pkts': '60328336',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '186031888166',
                                                                       'out_ucast_pkts': '144934536',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '186031888166',
                                                                       'in_ucast_pkts': '144934536',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '186031888166',
                                                                                                'in_ucast_pkts': '144934536',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14613',
                                                                                                'out_octets': '37044453415',
                                                                                                'out_ucast_pkts': '60328336',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14613',
                                                                       'out_octets': '37044453415',
                                                                       'out_ucast_pkts': '60328336',
                                                                       'poller_hash': 'QURUTjIzMTI0REVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 3861,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERFQg==',
                                         'serial_number': 'ADTN23124DEB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14488',
                                                                       'in_octets': '7182857775',
                                                                       'in_ucast_pkts': '36284466',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '102633796554',
                                                                       'out_ucast_pkts': '77660339',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '102633796554',
                                                                       'in_ucast_pkts': '77660339',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '102633796554',
                                                                                                'in_ucast_pkts': '77660339',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14488',
                                                                                                'out_octets': '7182857775',
                                                                                                'out_ucast_pkts': '36284466',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14488',
                                                                       'out_octets': '7182857775',
                                                                       'out_ucast_pkts': '36284466',
                                                                       'poller_hash': 'QURUTjIzMTI0RDkzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427024',
                                                                                                                        'fiber_length': 4090,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDkzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ5Mw==',
                                         'serial_number': 'ADTN23124D93',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427024'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14259',
                                                                       'in_octets': '8477882963',
                                                                       'in_ucast_pkts': '18924820',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '84457489770',
                                                                       'out_ucast_pkts': '65016933',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VNV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '84457489770',
                                                                       'in_ucast_pkts': '65016933',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '84457489770',
                                                                                                'in_ucast_pkts': '65016933',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14259',
                                                                                                'out_octets': '8477882963',
                                                                                                'out_ucast_pkts': '18924820',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14259',
                                                                       'out_octets': '8477882963',
                                                                       'out_ucast_pkts': '18924820',
                                                                       'poller_hash': 'QURUTjIzMTI0RDQ2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427036',
                                                                                                                        'fiber_length': 4612,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDQ2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ0Ng==',
                                         'serial_number': 'ADTN23124D46',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427036'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14487',
                                                                       'in_octets': '7352355370',
                                                                       'in_ucast_pkts': '32956583',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '201948414523',
                                                                       'out_ucast_pkts': '142178378',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '201948414523',
                                                                       'in_ucast_pkts': '142178378',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '201948414523',
                                                                                                'in_ucast_pkts': '142178378',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14487',
                                                                                                'out_octets': '7352355370',
                                                                                                'out_ucast_pkts': '32956583',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14487',
                                                                       'out_octets': '7352355370',
                                                                       'out_ucast_pkts': '32956583',
                                                                       'poller_hash': 'QURUTjIzMTI0OTFFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427022',
                                                                                                                        'fiber_length': 5476,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OTFFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDkxRQ==',
                                         'serial_number': 'ADTN2312491E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427022'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '342',
                                                                       'in_octets': '9898390739',
                                                                       'in_ucast_pkts': '22190380',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '80594217954',
                                                                       'out_ucast_pkts': '61996459',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '80594217954',
                                                                       'in_ucast_pkts': '61996459',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '80594217954',
                                                                                                'in_ucast_pkts': '61996459',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '342',
                                                                                                'out_octets': '9898390739',
                                                                                                'out_ucast_pkts': '22190380',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '342',
                                                                       'out_octets': '9898390739',
                                                                       'out_ucast_pkts': '22190380',
                                                                       'poller_hash': 'QURUTjIzMTI0RTNBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427025',
                                                                                                                        'fiber_length': 5824,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTNBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzQQ==',
                                         'serial_number': 'ADTN23124E3A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427025'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14846',
                                                                       'in_octets': '10434238744',
                                                                       'in_ucast_pkts': '50310389',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '257756685243',
                                                                       'out_ucast_pkts': '190759006',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VOV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '257756685243',
                                                                       'in_ucast_pkts': '190759006',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '257756685243',
                                                                                                'in_ucast_pkts': '190759006',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14846',
                                                                                                'out_octets': '10434238744',
                                                                                                'out_ucast_pkts': '50310389',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14846',
                                                                       'out_octets': '10434238744',
                                                                       'out_ucast_pkts': '50310389',
                                                                       'poller_hash': 'QURUTjIzMTI0REYzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427040',
                                                                                                                        'fiber_length': 3643,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REYzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGMw==',
                                         'serial_number': 'ADTN23124DF3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5-5-b',
                                         'uptime': '427040'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14487',
                                                                       'in_octets': '790422525',
                                                                       'in_ucast_pkts': '2958689',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '7130903493',
                                                                       'out_ucast_pkts': '5656972',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '7130903493',
                                                                       'in_ucast_pkts': '5656972',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '7130903493',
                                                                                                'in_ucast_pkts': '5656972',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14487',
                                                                                                'out_octets': '790422525',
                                                                                                'out_ucast_pkts': '2958689',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14487',
                                                                       'out_octets': '790422525',
                                                                       'out_ucast_pkts': '2958689',
                                                                       'poller_hash': 'QURUTjIzMTI1MThFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427024',
                                                                                                                        'fiber_length': 3909,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MThFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE4RQ==',
                                         'serial_number': 'ADTN2312518E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427024'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14845',
                                                                       'in_octets': '12476280102',
                                                                       'in_ucast_pkts': '51926230',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '266208528670',
                                                                       'out_ucast_pkts': '191938908',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '266208528670',
                                                                       'in_ucast_pkts': '191938908',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '266208528670',
                                                                                                'in_ucast_pkts': '191938908',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14845',
                                                                                                'out_octets': '12476280102',
                                                                                                'out_ucast_pkts': '51926230',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14845',
                                                                       'out_octets': '12476280102',
                                                                       'out_ucast_pkts': '51926230',
                                                                       'poller_hash': 'QURUTjIzMTI0RENFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 5338,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RENFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERDRQ==',
                                         'serial_number': 'ADTN23124DCE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14487',
                                                                       'in_octets': '1975280843',
                                                                       'in_ucast_pkts': '6538177',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '17893158561',
                                                                       'out_ucast_pkts': '14760521',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '17893158561',
                                                                       'in_ucast_pkts': '14760521',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '17893158561',
                                                                                                'in_ucast_pkts': '14760521',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14487',
                                                                                                'out_octets': '1975280843',
                                                                                                'out_ucast_pkts': '6538177',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14487',
                                                                       'out_octets': '1975280843',
                                                                       'out_ucast_pkts': '6538177',
                                                                       'poller_hash': 'QURUTjIzMTI0RTIyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427034',
                                                                                                                        'fiber_length': 3743,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTIyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyMg==',
                                         'serial_number': 'ADTN23124E22',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427034'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '114032',
                                                                       'in_octets': '34146902875',
                                                                       'in_ucast_pkts': '130691237',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9654',
                                                                       'out_octets': '471523767208',
                                                                       'out_ucast_pkts': '370552154',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9654',
                                                                       'in_octets': '471523767208',
                                                                       'in_ucast_pkts': '370552154',
                                                                       'links': [       {       'in_nucast_pkts': '9654',
                                                                                                'in_octets': '471523767208',
                                                                                                'in_ucast_pkts': '370552154',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '114032',
                                                                                                'out_octets': '34146902875',
                                                                                                'out_ucast_pkts': '130691237',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '114032',
                                                                       'out_octets': '34146902875',
                                                                       'out_ucast_pkts': '130691237',
                                                                       'poller_hash': 'QURUTjIzMTI1MDU1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3364586',
                                                                                                                        'fiber_length': 6229,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDU1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA1NQ==',
                                         'serial_number': 'ADTN23125055',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3364586'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14537',
                                                                       'in_octets': '983568170',
                                                                       'in_ucast_pkts': '6528585',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '17955758353',
                                                                       'out_ucast_pkts': '12484499',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '17955758353',
                                                                       'in_ucast_pkts': '12484499',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '17955758353',
                                                                                                'in_ucast_pkts': '12484499',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14537',
                                                                                                'out_octets': '983568170',
                                                                                                'out_ucast_pkts': '6528585',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14537',
                                                                       'out_octets': '983568170',
                                                                       'out_ucast_pkts': '6528585',
                                                                       'poller_hash': 'QURUTjIzMTI0RDQxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427025',
                                                                                                                        'fiber_length': 1926,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDQxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ0MQ==',
                                         'serial_number': 'ADTN23124D41',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427025'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14480',
                                                                       'in_octets': '1572259937',
                                                                       'in_ucast_pkts': '6243316',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '29939132609',
                                                                       'out_ucast_pkts': '21426454',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '29939132609',
                                                                       'in_ucast_pkts': '21426454',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '29939132609',
                                                                                                'in_ucast_pkts': '21426454',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14480',
                                                                                                'out_octets': '1572259937',
                                                                                                'out_ucast_pkts': '6243316',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14480',
                                                                       'out_octets': '1572259937',
                                                                       'out_ucast_pkts': '6243316',
                                                                       'poller_hash': 'QURUTjIzMTI0MEIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427026',
                                                                                                                        'fiber_length': 1969,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MEIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDBCMA==',
                                         'serial_number': 'ADTN231240B0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427026'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14820',
                                                                       'in_octets': '9477065744',
                                                                       'in_ucast_pkts': '46725402',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '131191620822',
                                                                       'out_ucast_pkts': '105992610',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '131191620822',
                                                                       'in_ucast_pkts': '105992610',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '131191620822',
                                                                                                'in_ucast_pkts': '105992610',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14820',
                                                                                                'out_octets': '9477065744',
                                                                                                'out_ucast_pkts': '46725402',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14820',
                                                                       'out_octets': '9477065744',
                                                                       'out_ucast_pkts': '46725402',
                                                                       'poller_hash': 'QURUTjIzMTI0REQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427026',
                                                                                                                        'fiber_length': 1780,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEREOQ==',
                                         'serial_number': 'ADTN23124DD9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427026'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-8.1.13.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x8-ONU13_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTNfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0MEE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MEE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x8-ONU13',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDBBNg==',
                                         'serial_number': 'ADTN231240A6',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14845',
                                                                       'in_octets': '11065164565',
                                                                       'in_ucast_pkts': '73336480',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124D9E_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '276809440559',
                                                                       'out_ucast_pkts': '202956573',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RDlFX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '276809440559',
                                                                       'in_ucast_pkts': '202956573',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '276809440559',
                                                                                                'in_ucast_pkts': '202956573',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14845',
                                                                                                'out_octets': '11065164565',
                                                                                                'out_ucast_pkts': '73336480',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14845',
                                                                       'out_octets': '11065164565',
                                                                       'out_ucast_pkts': '73336480',
                                                                       'poller_hash': 'QURUTjIzMTI0RDlFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 4273,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDlFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124D9E',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ5RQ==',
                                         'serial_number': 'ADTN23124D9E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75670',
                                                                       'in_octets': '31365538729',
                                                                       'in_ucast_pkts': '242077913',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6407',
                                                                       'out_octets': '829089433233',
                                                                       'out_ucast_pkts': '590504210',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6407',
                                                                       'in_octets': '829089433233',
                                                                       'in_ucast_pkts': '590504210',
                                                                       'links': [       {       'in_nucast_pkts': '6407',
                                                                                                'in_octets': '829089433233',
                                                                                                'in_ucast_pkts': '590504210',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75670',
                                                                                                'out_octets': '31365538729',
                                                                                                'out_ucast_pkts': '242077913',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75670',
                                                                       'out_octets': '31365538729',
                                                                       'out_ucast_pkts': '242077913',
                                                                       'poller_hash': 'QURUTjIzMTI0REYxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232582',
                                                                                                                        'fiber_length': 5944,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REYxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGMQ==',
                                         'serial_number': 'ADTN23124DF1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232582'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77530',
                                                                       'in_octets': '62439363189',
                                                                       'in_ucast_pkts': '286217570',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6385',
                                                                       'out_octets': '1268052424980',
                                                                       'out_ucast_pkts': '909777139',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6385',
                                                                       'in_octets': '1268052424980',
                                                                       'in_ucast_pkts': '909777139',
                                                                       'links': [       {       'in_nucast_pkts': '6385',
                                                                                                'in_octets': '1268052424980',
                                                                                                'in_ucast_pkts': '909777139',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77530',
                                                                                                'out_octets': '62439363189',
                                                                                                'out_ucast_pkts': '286217570',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77530',
                                                                       'out_octets': '62439363189',
                                                                       'out_ucast_pkts': '286217570',
                                                                       'poller_hash': 'QURUTjIzMTI0RTA3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232581',
                                                                                                                        'fiber_length': 3992,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTA3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUwNw==',
                                         'serial_number': 'ADTN23124E07',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232581'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11726',
                                                                       'in_octets': '6215172481',
                                                                       'in_ucast_pkts': '26752970',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '967',
                                                                       'out_octets': '92677070435',
                                                                       'out_ucast_pkts': '66622657',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '967',
                                                                       'in_octets': '92677070435',
                                                                       'in_ucast_pkts': '66622657',
                                                                       'links': [       {       'in_nucast_pkts': '967',
                                                                                                'in_octets': '92677070435',
                                                                                                'in_ucast_pkts': '66622657',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11726',
                                                                                                'out_octets': '6215172481',
                                                                                                'out_ucast_pkts': '26752970',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11726',
                                                                       'out_octets': '6215172481',
                                                                       'out_ucast_pkts': '26752970',
                                                                       'poller_hash': 'QURUTjIzMTI0RTE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '337326',
                                                                                                                        'fiber_length': 4123,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxNA==',
                                         'serial_number': 'ADTN23124E14',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '337326'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75665',
                                                                       'in_octets': '9338582863',
                                                                       'in_ucast_pkts': '28219211',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6402',
                                                                       'out_octets': '92332925860',
                                                                       'out_ucast_pkts': '77335772',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6402',
                                                                       'in_octets': '92332925860',
                                                                       'in_ucast_pkts': '77335772',
                                                                       'links': [       {       'in_nucast_pkts': '6402',
                                                                                                'in_octets': '92332925860',
                                                                                                'in_ucast_pkts': '77335772',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75665',
                                                                                                'out_octets': '9338582863',
                                                                                                'out_ucast_pkts': '28219211',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75665',
                                                                       'out_octets': '9338582863',
                                                                       'out_ucast_pkts': '28219211',
                                                                       'poller_hash': 'QURUTjIzMTI0RDQzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232584',
                                                                                                                        'fiber_length': 6063,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDQzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ0Mw==',
                                         'serial_number': 'ADTN23124D43',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232584'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '76288',
                                                                       'in_octets': '33947972487',
                                                                       'in_ucast_pkts': '162391808',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '599415877956',
                                                                       'out_ucast_pkts': '432388381',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '599415877956',
                                                                       'in_ucast_pkts': '432388381',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '599415877956',
                                                                                                'in_ucast_pkts': '432388381',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '76288',
                                                                                                'out_octets': '33947972487',
                                                                                                'out_ucast_pkts': '162391808',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '76288',
                                                                       'out_octets': '33947972487',
                                                                       'out_ucast_pkts': '162391808',
                                                                       'poller_hash': 'QURUTjIzMTI1MUUyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232582',
                                                                                                                        'fiber_length': 5212,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUUyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFFMg==',
                                         'serial_number': 'ADTN231251E2',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232582'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '114027',
                                                                       'in_octets': '48597335655',
                                                                       'in_ucast_pkts': '274600145',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9650',
                                                                       'out_octets': '1294769624337',
                                                                       'out_ucast_pkts': '894488302',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9650',
                                                                       'in_octets': '1294769624337',
                                                                       'in_ucast_pkts': '894488302',
                                                                       'links': [       {       'in_nucast_pkts': '9650',
                                                                                                'in_octets': '1294769624337',
                                                                                                'in_ucast_pkts': '894488302',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '114027',
                                                                                                'out_octets': '48597335655',
                                                                                                'out_ucast_pkts': '274600145',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '114027',
                                                                       'out_octets': '48597335655',
                                                                       'out_ucast_pkts': '274600145',
                                                                       'poller_hash': 'QURUTjIzMTI0NjUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3364583',
                                                                                                                        'fiber_length': 5972,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NjUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDY1MA==',
                                         'serial_number': 'ADTN23124650',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3364583'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '34912',
                                                                       'in_octets': '16610401362',
                                                                       'in_ucast_pkts': '85236877',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2953',
                                                                       'out_octets': '458100236742',
                                                                       'out_ucast_pkts': '324936178',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2953',
                                                                       'in_octets': '458100236742',
                                                                       'in_ucast_pkts': '324936178',
                                                                       'links': [       {       'in_nucast_pkts': '2953',
                                                                                                'in_octets': '458100236742',
                                                                                                'in_ucast_pkts': '324936178',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '34912',
                                                                                                'out_octets': '16610401362',
                                                                                                'out_ucast_pkts': '85236877',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '34912',
                                                                       'out_octets': '16610401362',
                                                                       'out_ucast_pkts': '85236877',
                                                                       'poller_hash': 'QURUTjIzMTI0RTI2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1029869',
                                                                                                                        'fiber_length': 7531,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTI2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyNg==',
                                         'serial_number': 'ADTN23124E26',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1029869'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75672',
                                                                       'in_octets': '85674469971',
                                                                       'in_ucast_pkts': '348962087',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6402',
                                                                       'out_octets': '1033461819978',
                                                                       'out_ucast_pkts': '758406064',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6402',
                                                                       'in_octets': '1033461819978',
                                                                       'in_ucast_pkts': '758406064',
                                                                       'links': [       {       'in_nucast_pkts': '6402',
                                                                                                'in_octets': '1033461819978',
                                                                                                'in_ucast_pkts': '758406064',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75672',
                                                                                                'out_octets': '85674469971',
                                                                                                'out_ucast_pkts': '348962087',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75672',
                                                                       'out_octets': '85674469971',
                                                                       'out_ucast_pkts': '348962087',
                                                                       'poller_hash': 'QURUTjIzMTI0RDk4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232584',
                                                                                                                        'fiber_length': 5967,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDk4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ5OA==',
                                         'serial_number': 'ADTN23124D98',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232584'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75665',
                                                                       'in_octets': '27739920116',
                                                                       'in_ucast_pkts': '138623369',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6403',
                                                                       'out_octets': '922022364186',
                                                                       'out_ucast_pkts': '680231536',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6403',
                                                                       'in_octets': '922022364186',
                                                                       'in_ucast_pkts': '680231536',
                                                                       'links': [       {       'in_nucast_pkts': '6403',
                                                                                                'in_octets': '922022364186',
                                                                                                'in_ucast_pkts': '680231536',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75665',
                                                                                                'out_octets': '27739920116',
                                                                                                'out_ucast_pkts': '138623369',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75665',
                                                                       'out_octets': '27739920116',
                                                                       'out_ucast_pkts': '138623369',
                                                                       'poller_hash': 'QURUTjIzMTI0RDUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232588',
                                                                                                                        'fiber_length': 5915,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ1MA==',
                                         'serial_number': 'ADTN23124D50',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232588'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75767',
                                                                       'in_octets': '2784186285',
                                                                       'in_ucast_pkts': '10090190',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6405',
                                                                       'out_octets': '35990375455',
                                                                       'out_ucast_pkts': '26460817',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6405',
                                                                       'in_octets': '35990375455',
                                                                       'in_ucast_pkts': '26460817',
                                                                       'links': [       {       'in_nucast_pkts': '6405',
                                                                                                'in_octets': '35990375455',
                                                                                                'in_ucast_pkts': '26460817',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75767',
                                                                                                'out_octets': '2784186285',
                                                                                                'out_ucast_pkts': '10090190',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75767',
                                                                       'out_octets': '2784186285',
                                                                       'out_ucast_pkts': '10090190',
                                                                       'poller_hash': 'QURUTjIzMTI0RDM4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232582',
                                                                                                                        'fiber_length': 6210,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDM4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQzOA==',
                                         'serial_number': 'ADTN23124D38',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232582'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-3.1.3.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x3-ONU3_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VM19FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0RTM1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTM1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x3-ONU3',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzNQ==',
                                         'serial_number': 'ADTN23124E35',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77266',
                                                                       'in_octets': '41123582384',
                                                                       'in_ucast_pkts': '281917848',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6407',
                                                                       'out_octets': '1196558938007',
                                                                       'out_ucast_pkts': '859399516',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6407',
                                                                       'in_octets': '1196558938007',
                                                                       'in_ucast_pkts': '859399516',
                                                                       'links': [       {       'in_nucast_pkts': '6407',
                                                                                                'in_octets': '1196558938007',
                                                                                                'in_ucast_pkts': '859399516',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77266',
                                                                                                'out_octets': '41123582384',
                                                                                                'out_ucast_pkts': '281917848',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77266',
                                                                       'out_octets': '41123582384',
                                                                       'out_ucast_pkts': '281917848',
                                                                       'poller_hash': 'QURUTjIzMTI0OTAwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232584',
                                                                                                                        'fiber_length': 6197,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OTAwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDkwMA==',
                                         'serial_number': 'ADTN23124900',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232584'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '63819',
                                                                       'in_octets': '29946671985',
                                                                       'in_ucast_pkts': '123997316',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5396',
                                                                       'out_octets': '358573734229',
                                                                       'out_ucast_pkts': '321869766',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5396',
                                                                       'in_octets': '358573734229',
                                                                       'in_ucast_pkts': '321869766',
                                                                       'links': [       {       'in_nucast_pkts': '5396',
                                                                                                'in_octets': '358573734229',
                                                                                                'in_ucast_pkts': '321869766',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '63819',
                                                                                                'out_octets': '29946671985',
                                                                                                'out_ucast_pkts': '123997316',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '63819',
                                                                       'out_octets': '29946671985',
                                                                       'out_ucast_pkts': '123997316',
                                                                       'poller_hash': 'QURUTjIzMTI1MTYwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1882625',
                                                                                                                        'fiber_length': 4527,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTYwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE2MA==',
                                         'serial_number': 'ADTN23125160',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1882625'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6011',
                                                                       'in_octets': '763293728',
                                                                       'in_ucast_pkts': '4151642',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '508',
                                                                       'out_octets': '26958327833',
                                                                       'out_ucast_pkts': '19150838',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '508',
                                                                       'in_octets': '26958327833',
                                                                       'in_ucast_pkts': '19150838',
                                                                       'links': [       {       'in_nucast_pkts': '508',
                                                                                                'in_octets': '26958327833',
                                                                                                'in_ucast_pkts': '19150838',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6011',
                                                                                                'out_octets': '763293728',
                                                                                                'out_ucast_pkts': '4151642',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6011',
                                                                       'out_octets': '763293728',
                                                                       'out_ucast_pkts': '4151642',
                                                                       'poller_hash': 'QURUTjIzMTI0REREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '176952',
                                                                                                                        'fiber_length': 3424,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERERA==',
                                         'serial_number': 'ADTN23124DDD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '176952'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14515',
                                                                       'in_octets': '3639779298',
                                                                       'in_ucast_pkts': '7358348',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '26974714761',
                                                                       'out_ucast_pkts': '22277576',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '26974714761',
                                                                       'in_ucast_pkts': '22277576',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '26974714761',
                                                                                                'in_ucast_pkts': '22277576',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14515',
                                                                                                'out_octets': '3639779298',
                                                                                                'out_ucast_pkts': '7358348',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14515',
                                                                       'out_octets': '3639779298',
                                                                       'out_ucast_pkts': '7358348',
                                                                       'poller_hash': 'QURUTjIzMTI0RTJEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427034',
                                                                                                                        'fiber_length': 4867,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTJEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyRA==',
                                         'serial_number': 'ADTN23124E2D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427034'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74433',
                                                                       'in_octets': '29191281244',
                                                                       'in_ucast_pkts': '94770633',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2473',
                                                                       'out_octets': '375087618014',
                                                                       'out_ucast_pkts': '274711552',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VNl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2473',
                                                                       'in_octets': '375087618014',
                                                                       'in_ucast_pkts': '274711552',
                                                                       'links': [       {       'in_nucast_pkts': '2473',
                                                                                                'in_octets': '375087618014',
                                                                                                'in_ucast_pkts': '274711552',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74433',
                                                                                                'out_octets': '29191281244',
                                                                                                'out_ucast_pkts': '94770633',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74433',
                                                                       'out_octets': '29191281244',
                                                                       'out_ucast_pkts': '94770633',
                                                                       'poller_hash': 'QURUTjIzMTI0NUM2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232587',
                                                                                                                        'fiber_length': 3878,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NUM2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDVDNg==',
                                         'serial_number': 'ADTN231245C6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232587'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14280',
                                                                       'in_octets': '10173859269',
                                                                       'in_ucast_pkts': '47770475',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '154648734777',
                                                                       'out_ucast_pkts': '117667977',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '154648734777',
                                                                       'in_ucast_pkts': '117667977',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '154648734777',
                                                                                                'in_ucast_pkts': '117667977',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14280',
                                                                                                'out_octets': '10173859269',
                                                                                                'out_ucast_pkts': '47770475',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14280',
                                                                       'out_octets': '10173859269',
                                                                       'out_ucast_pkts': '47770475',
                                                                       'poller_hash': 'QURUTjIzMTI0MUQxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427020',
                                                                                                                        'fiber_length': 2471,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MUQxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDFEMQ==',
                                         'serial_number': 'ADTN231241D1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427020'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '3685',
                                                                       'in_octets': '5017196441',
                                                                       'in_ucast_pkts': '12594970',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '312',
                                                                       'out_octets': '43937685021',
                                                                       'out_ucast_pkts': '32852011',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '312',
                                                                       'in_octets': '43937685021',
                                                                       'in_ucast_pkts': '32852011',
                                                                       'links': [       {       'in_nucast_pkts': '312',
                                                                                                'in_octets': '43937685021',
                                                                                                'in_ucast_pkts': '32852011',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '3685',
                                                                                                'out_octets': '5017196441',
                                                                                                'out_ucast_pkts': '12594970',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3685',
                                                                       'out_octets': '5017196441',
                                                                       'out_ucast_pkts': '12594970',
                                                                       'poller_hash': 'QURUTjIzMTI0RERGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '108153',
                                                                                                                        'fiber_length': 4825,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RERGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERERg==',
                                         'serial_number': 'ADTN23124DDF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '108153'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '26178',
                                                                       'in_octets': '85650039323',
                                                                       'in_ucast_pkts': '114289475',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124DA6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2214',
                                                                       'out_octets': '250406646440',
                                                                       'out_ucast_pkts': '232063765',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0REE2X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2214',
                                                                       'in_octets': '250406646440',
                                                                       'in_ucast_pkts': '232063765',
                                                                       'links': [       {       'in_nucast_pkts': '2214',
                                                                                                'in_octets': '250406646440',
                                                                                                'in_ucast_pkts': '232063765',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '26178',
                                                                                                'out_octets': '85650039323',
                                                                                                'out_ucast_pkts': '114289475',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '26178',
                                                                       'out_octets': '85650039323',
                                                                       'out_ucast_pkts': '114289475',
                                                                       'poller_hash': 'QURUTjIzMTI0REE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '771591',
                                                                                                                        'fiber_length': 5510,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124DA6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERBNg==',
                                         'serial_number': 'ADTN23124DA6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '771591'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '21619',
                                                                       'in_octets': '5281560674',
                                                                       'in_ucast_pkts': '11555894',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1828',
                                                                       'out_octets': '37139094766',
                                                                       'out_ucast_pkts': '31182483',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1828',
                                                                       'in_octets': '37139094766',
                                                                       'in_ucast_pkts': '31182483',
                                                                       'links': [       {       'in_nucast_pkts': '1828',
                                                                                                'in_octets': '37139094766',
                                                                                                'in_ucast_pkts': '31182483',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '21619',
                                                                                                'out_octets': '5281560674',
                                                                                                'out_ucast_pkts': '11555894',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '21619',
                                                                       'out_octets': '5281560674',
                                                                       'out_ucast_pkts': '11555894',
                                                                       'poller_hash': 'QURUTjIzMTI1MDIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '637514',
                                                                                                                        'fiber_length': 5554,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTAyMA==',
                                         'serial_number': 'ADTN23125020',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '637514'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '34913',
                                                                       'in_octets': '2840283832',
                                                                       'in_ucast_pkts': '26225482',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2953',
                                                                       'out_octets': '82265357604',
                                                                       'out_ucast_pkts': '55788751',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VOV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2953',
                                                                       'in_octets': '82265357604',
                                                                       'in_ucast_pkts': '55788751',
                                                                       'links': [       {       'in_nucast_pkts': '2953',
                                                                                                'in_octets': '82265357604',
                                                                                                'in_ucast_pkts': '55788751',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '34913',
                                                                                                'out_octets': '2840283832',
                                                                                                'out_ucast_pkts': '26225482',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '34913',
                                                                       'out_octets': '2840283832',
                                                                       'out_ucast_pkts': '26225482',
                                                                       'poller_hash': 'QURUTjIzMTI0RTE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1029869',
                                                                                                                        'fiber_length': 7481,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxNg==',
                                         'serial_number': 'ADTN23124E16',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1029869'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14481',
                                                                       'in_octets': '6742653477',
                                                                       'in_ucast_pkts': '36837158',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '128934177076',
                                                                       'out_ucast_pkts': '90637709',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '128934177076',
                                                                       'in_ucast_pkts': '90637709',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '128934177076',
                                                                                                'in_ucast_pkts': '90637709',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14481',
                                                                                                'out_octets': '6742653477',
                                                                                                'out_ucast_pkts': '36837158',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14481',
                                                                       'out_octets': '6742653477',
                                                                       'out_ucast_pkts': '36837158',
                                                                       'poller_hash': 'QURUTjIzMTI0RjNDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427025',
                                                                                                                        'fiber_length': 1807,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjNDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYzQw==',
                                         'serial_number': 'ADTN23124F3C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427025'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14605',
                                                                       'in_octets': '4074282793',
                                                                       'in_ucast_pkts': '9703049',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '42205526052',
                                                                       'out_ucast_pkts': '33256315',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '42205526052',
                                                                       'in_ucast_pkts': '33256315',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '42205526052',
                                                                                                'in_ucast_pkts': '33256315',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14605',
                                                                                                'out_octets': '4074282793',
                                                                                                'out_ucast_pkts': '9703049',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14605',
                                                                       'out_octets': '4074282793',
                                                                       'out_ucast_pkts': '9703049',
                                                                       'poller_hash': 'QURUTjIzMTI0RjJEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427032',
                                                                                                                        'fiber_length': 4645,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjJEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYyRA==',
                                         'serial_number': 'ADTN23124F2D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427032'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14258',
                                                                       'in_octets': '8179937368',
                                                                       'in_ucast_pkts': '48036719',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '197105393156',
                                                                       'out_ucast_pkts': '143812436',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTdfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '197105393156',
                                                                       'in_ucast_pkts': '143812436',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '197105393156',
                                                                                                'in_ucast_pkts': '143812436',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14258',
                                                                                                'out_octets': '8179937368',
                                                                                                'out_ucast_pkts': '48036719',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14258',
                                                                       'out_octets': '8179937368',
                                                                       'out_ucast_pkts': '48036719',
                                                                       'poller_hash': 'QURUTjIzMTI0MDhDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427027',
                                                                                                                        'fiber_length': 5423,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MDhDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDA4Qw==',
                                         'serial_number': 'ADTN2312408C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427027'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '182420',
                                                                       'in_octets': '61330425016',
                                                                       'in_ucast_pkts': '233445066',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15381',
                                                                       'out_octets': '863383765062',
                                                                       'out_ucast_pkts': '644759562',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15381',
                                                                       'in_octets': '863383765062',
                                                                       'in_ucast_pkts': '644759562',
                                                                       'links': [       {       'in_nucast_pkts': '15381',
                                                                                                'in_octets': '863383765062',
                                                                                                'in_ucast_pkts': '644759562',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '182420',
                                                                                                'out_octets': '61330425016',
                                                                                                'out_ucast_pkts': '233445066',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '182420',
                                                                       'out_octets': '61330425016',
                                                                       'out_ucast_pkts': '233445066',
                                                                       'poller_hash': 'QURUTjIzMTI0RDdFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355633',
                                                                                                                        'fiber_length': 6288,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDdFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ3RQ==',
                                         'serial_number': 'ADTN23124D7E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355633'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '109387',
                                                                       'in_octets': '174666695788',
                                                                       'in_ucast_pkts': '363170979',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3640',
                                                                       'out_octets': '933167134518',
                                                                       'out_ucast_pkts': '719789175',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3640',
                                                                       'in_octets': '933167134518',
                                                                       'in_ucast_pkts': '719789175',
                                                                       'links': [       {       'in_nucast_pkts': '3640',
                                                                                                'in_octets': '933167134518',
                                                                                                'in_ucast_pkts': '719789175',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '109387',
                                                                                                'out_octets': '174666695788',
                                                                                                'out_ucast_pkts': '363170979',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '109387',
                                                                       'out_octets': '174666695788',
                                                                       'out_ucast_pkts': '363170979',
                                                                       'poller_hash': 'QURUTjIzMTI0RDdEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3278982',
                                                                                                                        'fiber_length': 4613,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RDdEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEQ3RA==',
                                         'serial_number': 'ADTN23124D7D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3278982'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14487',
                                                                       'in_octets': '41206052671',
                                                                       'in_ucast_pkts': '67857055',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '212344229386',
                                                                       'out_ucast_pkts': '157460453',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '212344229386',
                                                                       'in_ucast_pkts': '157460453',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '212344229386',
                                                                                                'in_ucast_pkts': '157460453',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14487',
                                                                                                'out_octets': '41206052671',
                                                                                                'out_ucast_pkts': '67857055',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14487',
                                                                       'out_octets': '41206052671',
                                                                       'out_ucast_pkts': '67857055',
                                                                       'poller_hash': 'QURUTjIzMTI1MDFEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427032',
                                                                                                                        'fiber_length': 4938,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDFEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTAxRA==',
                                         'serial_number': 'ADTN2312501D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427032'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14558',
                                                                       'in_octets': '6938004105',
                                                                       'in_ucast_pkts': '19551545',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '65772753626',
                                                                       'out_ucast_pkts': '52111151',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '65772753626',
                                                                       'in_ucast_pkts': '52111151',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '65772753626',
                                                                                                'in_ucast_pkts': '52111151',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14558',
                                                                                                'out_octets': '6938004105',
                                                                                                'out_ucast_pkts': '19551545',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14558',
                                                                       'out_octets': '6938004105',
                                                                       'out_ucast_pkts': '19551545',
                                                                       'poller_hash': 'QURUTjIzMTI0REZDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427030',
                                                                                                                        'fiber_length': 1986,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REZDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGQw==',
                                         'serial_number': 'ADTN23124DFC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427030'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14496',
                                                                       'in_octets': '7685322089',
                                                                       'in_ucast_pkts': '61007810',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '289160274295',
                                                                       'out_ucast_pkts': '198393026',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMTlfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '289160274295',
                                                                       'in_ucast_pkts': '198393026',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '289160274295',
                                                                                                'in_ucast_pkts': '198393026',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14496',
                                                                                                'out_octets': '7685322089',
                                                                                                'out_ucast_pkts': '61007810',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14496',
                                                                       'out_octets': '7685322089',
                                                                       'out_ucast_pkts': '61007810',
                                                                       'poller_hash': 'QURUTjIzMTI0REY1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427034',
                                                                                                                        'fiber_length': 5198,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REY1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGNQ==',
                                         'serial_number': 'ADTN23124DF5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427034'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14852',
                                                                       'in_octets': '5085781923',
                                                                       'in_ucast_pkts': '20757790',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1226',
                                                                       'out_octets': '79970505613',
                                                                       'out_ucast_pkts': '56820648',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1226',
                                                                       'in_octets': '79970505613',
                                                                       'in_ucast_pkts': '56820648',
                                                                       'links': [       {       'in_nucast_pkts': '1226',
                                                                                                'in_octets': '79970505613',
                                                                                                'in_ucast_pkts': '56820648',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14852',
                                                                                                'out_octets': '5085781923',
                                                                                                'out_ucast_pkts': '20757790',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14852',
                                                                       'out_octets': '5085781923',
                                                                       'out_ucast_pkts': '20757790',
                                                                       'poller_hash': 'QURUTjIzMTI0RjMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427027',
                                                                                                                        'fiber_length': 2396,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYzMA==',
                                         'serial_number': 'ADTN23124F30',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427027'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75672',
                                                                       'in_octets': '120872407124',
                                                                       'in_ucast_pkts': '568591069',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6405',
                                                                       'out_octets': '1832551536574',
                                                                       'out_ucast_pkts': '1367600034',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6405',
                                                                       'in_octets': '1832551536574',
                                                                       'in_ucast_pkts': '1367600034',
                                                                       'links': [       {       'in_nucast_pkts': '6405',
                                                                                                'in_octets': '1832551536574',
                                                                                                'in_ucast_pkts': '1367600034',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75672',
                                                                                                'out_octets': '120872407124',
                                                                                                'out_ucast_pkts': '568591069',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75672',
                                                                       'out_octets': '120872407124',
                                                                       'out_ucast_pkts': '568591069',
                                                                       'poller_hash': 'QURUTjIzMTI0RjJCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232586',
                                                                                                                        'fiber_length': 6120,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjJCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYyQg==',
                                         'serial_number': 'ADTN23124F2B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232586'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77528',
                                                                       'in_octets': '32641247852',
                                                                       'in_ucast_pkts': '126568489',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23123FAD_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6387',
                                                                       'out_octets': '362483997378',
                                                                       'out_ucast_pkts': '260331198',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTIzRkFEX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6387',
                                                                       'in_octets': '362483997378',
                                                                       'in_ucast_pkts': '260331198',
                                                                       'links': [       {       'in_nucast_pkts': '6387',
                                                                                                'in_octets': '362483997378',
                                                                                                'in_ucast_pkts': '260331198',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77528',
                                                                                                'out_octets': '32641247852',
                                                                                                'out_ucast_pkts': '126568489',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77528',
                                                                       'out_octets': '32641247852',
                                                                       'out_ucast_pkts': '126568489',
                                                                       'poller_hash': 'QURUTjIzMTIzRkFEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232583',
                                                                                                                        'fiber_length': 3684,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkFEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23123FAD',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZBRA==',
                                         'serial_number': 'ADTN23123FAD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232583'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14487',
                                                                       'in_octets': '11594417795',
                                                                       'in_ucast_pkts': '74432212',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '237652363977',
                                                                       'out_ucast_pkts': '165639326',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '237652363977',
                                                                       'in_ucast_pkts': '165639326',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '237652363977',
                                                                                                'in_ucast_pkts': '165639326',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14487',
                                                                                                'out_octets': '11594417795',
                                                                                                'out_ucast_pkts': '74432212',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14487',
                                                                       'out_octets': '11594417795',
                                                                       'out_ucast_pkts': '74432212',
                                                                       'poller_hash': 'QURUTjIzMTI0NzEzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427026',
                                                                                                                        'fiber_length': 5184,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NzEzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDcxMw==',
                                         'serial_number': 'ADTN23124713',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427026'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '1294',
                                                                       'in_octets': '29294916315',
                                                                       'in_ucast_pkts': '132421807',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124F99_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6385',
                                                                       'out_octets': '669835731171',
                                                                       'out_ucast_pkts': '478712474',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0Rjk5X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6385',
                                                                       'in_octets': '669835731171',
                                                                       'in_ucast_pkts': '478712474',
                                                                       'links': [       {       'in_nucast_pkts': '6385',
                                                                                                'in_octets': '669835731171',
                                                                                                'in_ucast_pkts': '478712474',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '1294',
                                                                                                'out_octets': '29294916315',
                                                                                                'out_ucast_pkts': '132421807',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1294',
                                                                       'out_octets': '29294916315',
                                                                       'out_ucast_pkts': '132421807',
                                                                       'poller_hash': 'QURUTjIzMTI0Rjk5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232585',
                                                                                                                        'fiber_length': 3674,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0Rjk5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124F99',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEY5OQ==',
                                         'serial_number': 'ADTN23124F99',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232585'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14779',
                                                                       'in_octets': '11099335936',
                                                                       'in_ucast_pkts': '67664231',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '185374996354',
                                                                       'out_ucast_pkts': '129135376',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '185374996354',
                                                                       'in_ucast_pkts': '129135376',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '185374996354',
                                                                                                'in_ucast_pkts': '129135376',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14779',
                                                                                                'out_octets': '11099335936',
                                                                                                'out_ucast_pkts': '67664231',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14779',
                                                                       'out_octets': '11099335936',
                                                                       'out_ucast_pkts': '67664231',
                                                                       'poller_hash': 'QURUTjIzMTI0RTE5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427040',
                                                                                                                        'fiber_length': 5202,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTE5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxOQ==',
                                         'serial_number': 'ADTN23124E19',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427040'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14833',
                                                                       'in_octets': '18005021720',
                                                                       'in_ucast_pkts': '32996865',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '130581815576',
                                                                       'out_ucast_pkts': '99475924',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '130581815576',
                                                                       'in_ucast_pkts': '99475924',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '130581815576',
                                                                                                'in_ucast_pkts': '99475924',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14833',
                                                                                                'out_octets': '18005021720',
                                                                                                'out_ucast_pkts': '32996865',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14833',
                                                                       'out_octets': '18005021720',
                                                                       'out_ucast_pkts': '32996865',
                                                                       'poller_hash': 'QURUTjIzMTI1MDI2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427032',
                                                                                                                        'fiber_length': 4747,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDI2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTAyNg==',
                                         'serial_number': 'ADTN23125026',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427032'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '59201',
                                                                       'in_octets': '68196870753',
                                                                       'in_ucast_pkts': '219278469',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1926',
                                                                       'out_octets': '551753915216',
                                                                       'out_ucast_pkts': '400848215',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1926',
                                                                       'in_octets': '551753915216',
                                                                       'in_ucast_pkts': '400848215',
                                                                       'links': [       {       'in_nucast_pkts': '1926',
                                                                                                'in_octets': '551753915216',
                                                                                                'in_ucast_pkts': '400848215',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '59201',
                                                                                                'out_octets': '68196870753',
                                                                                                'out_ucast_pkts': '219278469',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '59201',
                                                                       'out_octets': '68196870753',
                                                                       'out_ucast_pkts': '219278469',
                                                                       'poller_hash': 'QURUTjIzMTI0RjA2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1732403',
                                                                                                                        'fiber_length': 4267,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjA2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYwNg==',
                                         'serial_number': 'ADTN23124F06',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1732403'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14478',
                                                                       'in_octets': '6509985184',
                                                                       'in_ucast_pkts': '10001038',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '23997518580',
                                                                       'out_ucast_pkts': '17479618',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '23997518580',
                                                                       'in_ucast_pkts': '17479618',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '23997518580',
                                                                                                'in_ucast_pkts': '17479618',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14478',
                                                                                                'out_octets': '6509985184',
                                                                                                'out_ucast_pkts': '10001038',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14478',
                                                                       'out_octets': '6509985184',
                                                                       'out_ucast_pkts': '10001038',
                                                                       'poller_hash': 'QURUTjIzMTIzRjU3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427033',
                                                                                                                        'fiber_length': 4661,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRjU3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0Y1Nw==',
                                         'serial_number': 'ADTN23123F57',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427033'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14474',
                                                                       'in_octets': '9698208206',
                                                                       'in_ucast_pkts': '63773063',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '245329448787',
                                                                       'out_ucast_pkts': '170570158',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '245329448787',
                                                                       'in_ucast_pkts': '170570158',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '245329448787',
                                                                                                'in_ucast_pkts': '170570158',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14474',
                                                                                                'out_octets': '9698208206',
                                                                                                'out_ucast_pkts': '63773063',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14474',
                                                                       'out_octets': '9698208206',
                                                                       'out_ucast_pkts': '63773063',
                                                                       'poller_hash': 'QURUTjIzMTI0REIyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427041',
                                                                                                                        'fiber_length': 5688,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REIyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERCMg==',
                                         'serial_number': 'ADTN23124DB2',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427041'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '48342',
                                                                       'in_octets': '47394728515',
                                                                       'in_ucast_pkts': '165295208',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1612',
                                                                       'out_octets': '587936001059',
                                                                       'out_ucast_pkts': '429751557',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1612',
                                                                       'in_octets': '587936001059',
                                                                       'in_ucast_pkts': '429751557',
                                                                       'links': [       {       'in_nucast_pkts': '1612',
                                                                                                'in_octets': '587936001059',
                                                                                                'in_ucast_pkts': '429751557',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '48342',
                                                                                                'out_octets': '47394728515',
                                                                                                'out_ucast_pkts': '165295208',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '48342',
                                                                       'out_octets': '47394728515',
                                                                       'out_ucast_pkts': '165295208',
                                                                       'poller_hash': 'QURUTjIzMTIzRjJCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1449540',
                                                                                                                        'fiber_length': 5584,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRjJCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0YyQg==',
                                         'serial_number': 'ADTN23123F2B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1449540'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14492',
                                                                       'in_octets': '2697660819',
                                                                       'in_ucast_pkts': '10709779',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '43209532271',
                                                                       'out_ucast_pkts': '33612964',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '43209532271',
                                                                       'in_ucast_pkts': '33612964',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '43209532271',
                                                                                                'in_ucast_pkts': '33612964',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14492',
                                                                                                'out_octets': '2697660819',
                                                                                                'out_ucast_pkts': '10709779',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14492',
                                                                       'out_octets': '2697660819',
                                                                       'out_ucast_pkts': '10709779',
                                                                       'poller_hash': 'QURUTjIzMTI0RTI5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427041',
                                                                                                                        'fiber_length': 5757,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTI5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyOQ==',
                                         'serial_number': 'ADTN23124E29',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427041'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14842',
                                                                       'in_octets': '17964625070',
                                                                       'in_ucast_pkts': '72846985',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '261600566960',
                                                                       'out_ucast_pkts': '192987362',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '261600566960',
                                                                       'in_ucast_pkts': '192987362',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '261600566960',
                                                                                                'in_ucast_pkts': '192987362',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14842',
                                                                                                'out_octets': '17964625070',
                                                                                                'out_ucast_pkts': '72846985',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14842',
                                                                       'out_octets': '17964625070',
                                                                       'out_ucast_pkts': '72846985',
                                                                       'poller_hash': 'QURUTjIzMTI1MDg4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427032',
                                                                                                                        'fiber_length': 4852,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDg4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA4OA==',
                                         'serial_number': 'ADTN23125088',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427032'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14483',
                                                                       'in_octets': '14960213903',
                                                                       'in_ucast_pkts': '38797550',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '104093424075',
                                                                       'out_ucast_pkts': '86505373',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '104093424075',
                                                                       'in_ucast_pkts': '86505373',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '104093424075',
                                                                                                'in_ucast_pkts': '86505373',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14483',
                                                                                                'out_octets': '14960213903',
                                                                                                'out_ucast_pkts': '38797550',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14483',
                                                                       'out_octets': '14960213903',
                                                                       'out_ucast_pkts': '38797550',
                                                                       'poller_hash': 'QURUTjIzMTI1MTQ2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427026',
                                                                                                                        'fiber_length': 2088,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0Ng==',
                                         'serial_number': 'ADTN23125146',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427026'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75678',
                                                                       'in_octets': '16154703523',
                                                                       'in_ucast_pkts': '100747676',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6384',
                                                                       'out_octets': '270403354462',
                                                                       'out_ucast_pkts': '197093412',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6384',
                                                                       'in_octets': '270403354462',
                                                                       'in_ucast_pkts': '197093412',
                                                                       'links': [       {       'in_nucast_pkts': '6384',
                                                                                                'in_octets': '270403354462',
                                                                                                'in_ucast_pkts': '197093412',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75678',
                                                                                                'out_octets': '16154703523',
                                                                                                'out_ucast_pkts': '100747676',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75678',
                                                                       'out_octets': '16154703523',
                                                                       'out_ucast_pkts': '100747676',
                                                                       'poller_hash': 'QURUTjIzMTI1MTdEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232584',
                                                                                                                        'fiber_length': 3861,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTdEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE3RA==',
                                         'serial_number': 'ADTN2312517D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232584'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14481',
                                                                       'in_octets': '12010142280',
                                                                       'in_ucast_pkts': '40833955',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23125166_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '139273338352',
                                                                       'out_ucast_pkts': '125281266',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI1MTY2X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '139273338352',
                                                                       'in_ucast_pkts': '125281266',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '139273338352',
                                                                                                'in_ucast_pkts': '125281266',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14481',
                                                                                                'out_octets': '12010142280',
                                                                                                'out_ucast_pkts': '40833955',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14481',
                                                                       'out_octets': '12010142280',
                                                                       'out_ucast_pkts': '40833955',
                                                                       'poller_hash': 'QURUTjIzMTI1MTY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427028',
                                                                                                                        'fiber_length': 5195,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23125166',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE2Ng==',
                                         'serial_number': 'ADTN23125166',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14488',
                                                                       'in_octets': '3843776317',
                                                                       'in_ucast_pkts': '11663473',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '54436132368',
                                                                       'out_ucast_pkts': '43264779',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '54436132368',
                                                                       'in_ucast_pkts': '43264779',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '54436132368',
                                                                                                'in_ucast_pkts': '43264779',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14488',
                                                                                                'out_octets': '3843776317',
                                                                                                'out_ucast_pkts': '11663473',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14488',
                                                                       'out_octets': '3843776317',
                                                                       'out_ucast_pkts': '11663473',
                                                                       'poller_hash': 'QURUTjIzMTI0RTY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427038',
                                                                                                                        'fiber_length': 5100,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU22',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU2Nw==',
                                         'serial_number': 'ADTN23124E67',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427038'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10918',
                                                                       'in_octets': '4909755664',
                                                                       'in_ucast_pkts': '21306651',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23125190_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '71098510942',
                                                                       'out_ucast_pkts': '52730022',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI1MTkwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '71098510942',
                                                                       'in_ucast_pkts': '52730022',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '71098510942',
                                                                                                'in_ucast_pkts': '52730022',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10918',
                                                                                                'out_octets': '4909755664',
                                                                                                'out_ucast_pkts': '21306651',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10918',
                                                                       'out_octets': '4909755664',
                                                                       'out_ucast_pkts': '21306651',
                                                                       'poller_hash': 'QURUTjIzMTI1MTkwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327028',
                                                                                                                        'fiber_length': 6895,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTkwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23125190',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE5MA==',
                                         'serial_number': 'ADTN23125190',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14496',
                                                                       'in_octets': '6562879945',
                                                                       'in_ucast_pkts': '44066207',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '250735238169',
                                                                       'out_ucast_pkts': '178667406',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VOF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '250735238169',
                                                                       'in_ucast_pkts': '178667406',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '250735238169',
                                                                                                'in_ucast_pkts': '178667406',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14496',
                                                                                                'out_octets': '6562879945',
                                                                                                'out_ucast_pkts': '44066207',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14496',
                                                                       'out_octets': '6562879945',
                                                                       'out_ucast_pkts': '44066207',
                                                                       'poller_hash': 'QURUTjIzMTI0MUQwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427037',
                                                                                                                        'fiber_length': 5845,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MUQwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDFEMA==',
                                         'serial_number': 'ADTN231241D0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427037'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14484',
                                                                       'in_octets': '5087837331',
                                                                       'in_ucast_pkts': '20715264',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '144634995117',
                                                                       'out_ucast_pkts': '106640068',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '144634995117',
                                                                       'in_ucast_pkts': '106640068',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '144634995117',
                                                                                                'in_ucast_pkts': '106640068',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14484',
                                                                                                'out_octets': '5087837331',
                                                                                                'out_ucast_pkts': '20715264',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14484',
                                                                       'out_octets': '5087837331',
                                                                       'out_ucast_pkts': '20715264',
                                                                       'poller_hash': 'QURUTjIzMTI0RTQwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 5726,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTQwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU0MA==',
                                         'serial_number': 'ADTN23124E40',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '34947',
                                                                       'in_octets': '8301950420',
                                                                       'in_ucast_pkts': '63624076',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124E0D_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2956',
                                                                       'out_octets': '157804370907',
                                                                       'out_ucast_pkts': '115544537',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RTBEX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2956',
                                                                       'in_octets': '157804370907',
                                                                       'in_ucast_pkts': '115544537',
                                                                       'links': [       {       'in_nucast_pkts': '2956',
                                                                                                'in_octets': '157804370907',
                                                                                                'in_ucast_pkts': '115544537',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '34947',
                                                                                                'out_octets': '8301950420',
                                                                                                'out_ucast_pkts': '63624076',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '34947',
                                                                       'out_octets': '8301950420',
                                                                       'out_ucast_pkts': '63624076',
                                                                       'poller_hash': 'QURUTjIzMTI0RTBEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1029870',
                                                                                                                        'fiber_length': 7351,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTBEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124E0D',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUwRA==',
                                         'serial_number': 'ADTN23124E0D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1029870'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11367',
                                                                       'in_octets': '15767760415',
                                                                       'in_ucast_pkts': '30683773',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '73447962648',
                                                                       'out_ucast_pkts': '60143172',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '73447962648',
                                                                       'in_ucast_pkts': '60143172',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '73447962648',
                                                                                                'in_ucast_pkts': '60143172',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11367',
                                                                                                'out_octets': '15767760415',
                                                                                                'out_ucast_pkts': '30683773',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11367',
                                                                       'out_octets': '15767760415',
                                                                       'out_ucast_pkts': '30683773',
                                                                       'poller_hash': 'QURUTjIzMTI0RTcxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327025',
                                                                                                                        'fiber_length': 8742,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTcxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU3MQ==',
                                         'serial_number': 'ADTN23124E71',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327025'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14538',
                                                                       'in_octets': '19593481234',
                                                                       'in_ucast_pkts': '59439003',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1229',
                                                                       'out_octets': '246726206538',
                                                                       'out_ucast_pkts': '183721766',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1229',
                                                                       'in_octets': '246726206538',
                                                                       'in_ucast_pkts': '183721766',
                                                                       'links': [       {       'in_nucast_pkts': '1229',
                                                                                                'in_octets': '246726206538',
                                                                                                'in_ucast_pkts': '183721766',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14538',
                                                                                                'out_octets': '19593481234',
                                                                                                'out_ucast_pkts': '59439003',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14538',
                                                                       'out_octets': '19593481234',
                                                                       'out_ucast_pkts': '59439003',
                                                                       'poller_hash': 'QURUTjIzMTI0RTNGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427039',
                                                                                                                        'fiber_length': 5421,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTNGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzRg==',
                                         'serial_number': 'ADTN23124E3F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427039'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14840',
                                                                       'in_octets': '2117419990',
                                                                       'in_ucast_pkts': '16432791',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '76155224064',
                                                                       'out_ucast_pkts': '54375468',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '76155224064',
                                                                       'in_ucast_pkts': '54375468',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '76155224064',
                                                                                                'in_ucast_pkts': '54375468',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14840',
                                                                                                'out_octets': '2117419990',
                                                                                                'out_ucast_pkts': '16432791',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14840',
                                                                       'out_octets': '2117419990',
                                                                       'out_ucast_pkts': '16432791',
                                                                       'poller_hash': 'QURUTjIzMTI0RUQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427030',
                                                                                                                        'fiber_length': 5679,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RUQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVEOQ==',
                                         'serial_number': 'ADTN23124ED9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427030'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74443',
                                                                       'in_octets': '29275910389',
                                                                       'in_ucast_pkts': '150037707',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2476',
                                                                       'out_octets': '892621609407',
                                                                       'out_ucast_pkts': '650891503',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2476',
                                                                       'in_octets': '892621609407',
                                                                       'in_ucast_pkts': '650891503',
                                                                       'links': [       {       'in_nucast_pkts': '2476',
                                                                                                'in_octets': '892621609407',
                                                                                                'in_ucast_pkts': '650891503',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74443',
                                                                                                'out_octets': '29275910389',
                                                                                                'out_ucast_pkts': '150037707',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74443',
                                                                       'out_octets': '29275910389',
                                                                       'out_ucast_pkts': '150037707',
                                                                       'poller_hash': 'QURUTjIzMTI0REU4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232585',
                                                                                                                        'fiber_length': 3367,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REU4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERFOA==',
                                         'serial_number': 'ADTN23124DE8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232585'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181493',
                                                                       'in_octets': '260250861320',
                                                                       'in_ucast_pkts': '725172539',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15373',
                                                                       'out_octets': '2021601751723',
                                                                       'out_ucast_pkts': '1766925366',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTRfRVRIXzE=',
                                                                       'speed': '100000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15373',
                                                                       'in_octets': '2021601751723',
                                                                       'in_ucast_pkts': '1766925366',
                                                                       'links': [       {       'in_nucast_pkts': '15373',
                                                                                                'in_octets': '2021601751723',
                                                                                                'in_ucast_pkts': '1766925366',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181493',
                                                                                                'out_octets': '260250861320',
                                                                                                'out_ucast_pkts': '725172539',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181493',
                                                                       'out_octets': '260250861320',
                                                                       'out_ucast_pkts': '725172539',
                                                                       'poller_hash': 'QURUTjIzMTI0RTNFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355592',
                                                                                                                        'fiber_length': 5805,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTNFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzRQ==',
                                         'serial_number': 'ADTN23124E3E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5-5-b',
                                         'uptime': '5355592'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11095',
                                                                       'in_octets': '14171450420',
                                                                       'in_ucast_pkts': '40353367',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '202237039839',
                                                                       'out_ucast_pkts': '154765636',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '202237039839',
                                                                       'in_ucast_pkts': '154765636',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '202237039839',
                                                                                                'in_ucast_pkts': '154765636',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11095',
                                                                                                'out_octets': '14171450420',
                                                                                                'out_ucast_pkts': '40353367',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11095',
                                                                       'out_octets': '14171450420',
                                                                       'out_ucast_pkts': '40353367',
                                                                       'poller_hash': 'QURUTjIzMTI1MUVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327027',
                                                                                                                        'fiber_length': 6685,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFFQg==',
                                         'serial_number': 'ADTN231251EB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327027'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11367',
                                                                       'in_octets': '4050843218',
                                                                       'in_ucast_pkts': '22475505',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '148833022375',
                                                                       'out_ucast_pkts': '104104029',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '148833022375',
                                                                       'in_ucast_pkts': '104104029',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '148833022375',
                                                                                                'in_ucast_pkts': '104104029',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11367',
                                                                                                'out_octets': '4050843218',
                                                                                                'out_ucast_pkts': '22475505',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11367',
                                                                       'out_octets': '4050843218',
                                                                       'out_ucast_pkts': '22475505',
                                                                       'poller_hash': 'QURUTjIzMTI0RERBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327029',
                                                                                                                        'fiber_length': 8706,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RERBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEREQQ==',
                                         'serial_number': 'ADTN23124DDA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11367',
                                                                       'in_octets': '4949456657',
                                                                       'in_ucast_pkts': '16426571',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '72751942766',
                                                                       'out_ucast_pkts': '56652163',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '72751942766',
                                                                       'in_ucast_pkts': '56652163',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '72751942766',
                                                                                                'in_ucast_pkts': '56652163',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11367',
                                                                                                'out_octets': '4949456657',
                                                                                                'out_ucast_pkts': '16426571',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11367',
                                                                       'out_octets': '4949456657',
                                                                       'out_ucast_pkts': '16426571',
                                                                       'poller_hash': 'QURUTjIzMTI0REUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327027',
                                                                                                                        'fiber_length': 8348,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERFMA==',
                                         'serial_number': 'ADTN23124DE0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327027'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14488',
                                                                       'in_octets': '11123096046',
                                                                       'in_ucast_pkts': '13511000',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '30079611459',
                                                                       'out_ucast_pkts': '25371278',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '30079611459',
                                                                       'in_ucast_pkts': '25371278',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '30079611459',
                                                                                                'in_ucast_pkts': '25371278',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14488',
                                                                                                'out_octets': '11123096046',
                                                                                                'out_ucast_pkts': '13511000',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14488',
                                                                       'out_octets': '11123096046',
                                                                       'out_ucast_pkts': '13511000',
                                                                       'poller_hash': 'QURUTjIzMTI0REZCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427037',
                                                                                                                        'fiber_length': 2183,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REZCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGQg==',
                                         'serial_number': 'ADTN23124DFB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427037'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11096',
                                                                       'in_octets': '3889093495',
                                                                       'in_ucast_pkts': '10543113',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '36279580423',
                                                                       'out_ucast_pkts': '30627633',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '36279580423',
                                                                       'in_ucast_pkts': '30627633',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '36279580423',
                                                                                                'in_ucast_pkts': '30627633',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11096',
                                                                                                'out_octets': '3889093495',
                                                                                                'out_ucast_pkts': '10543113',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11096',
                                                                       'out_octets': '3889093495',
                                                                       'out_ucast_pkts': '10543113',
                                                                       'poller_hash': 'QURUTjIzMTI0REU0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327029',
                                                                                                                        'fiber_length': 6822,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REU0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERFNA==',
                                         'serial_number': 'ADTN23124DE4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14611',
                                                                       'in_octets': '10465029520',
                                                                       'in_ucast_pkts': '32886879',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '178250999676',
                                                                       'out_ucast_pkts': '132715489',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VOV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '178250999676',
                                                                       'in_ucast_pkts': '132715489',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '178250999676',
                                                                                                'in_ucast_pkts': '132715489',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14611',
                                                                                                'out_octets': '10465029520',
                                                                                                'out_ucast_pkts': '32886879',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14611',
                                                                       'out_octets': '10465029520',
                                                                       'out_ucast_pkts': '32886879',
                                                                       'poller_hash': 'QURUTjIzMTI0REJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427041',
                                                                                                                        'fiber_length': 6895,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERCQw==',
                                         'serial_number': 'ADTN23124DBC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427041'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74455',
                                                                       'in_octets': '20700678576',
                                                                       'in_ucast_pkts': '51489624',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '171434867308',
                                                                       'out_ucast_pkts': '139976763',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '171434867308',
                                                                       'in_ucast_pkts': '139976763',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '171434867308',
                                                                                                'in_ucast_pkts': '139976763',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74455',
                                                                                                'out_octets': '20700678576',
                                                                                                'out_ucast_pkts': '51489624',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74455',
                                                                       'out_octets': '20700678576',
                                                                       'out_ucast_pkts': '51489624',
                                                                       'poller_hash': 'QURUTjIzMTI0RTI1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232587',
                                                                                                                        'fiber_length': 5209,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTI1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyNQ==',
                                         'serial_number': 'ADTN23124E25',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232587'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14481',
                                                                       'in_octets': '69004973439',
                                                                       'in_ucast_pkts': '106561031',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '235491527014',
                                                                       'out_ucast_pkts': '189025158',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '235491527014',
                                                                       'in_ucast_pkts': '189025158',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '235491527014',
                                                                                                'in_ucast_pkts': '189025158',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14481',
                                                                                                'out_octets': '69004973439',
                                                                                                'out_ucast_pkts': '106561031',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14481',
                                                                       'out_octets': '69004973439',
                                                                       'out_ucast_pkts': '106561031',
                                                                       'poller_hash': 'QURUTjIzMTI0REIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427027',
                                                                                                                        'fiber_length': 5405,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERCMA==',
                                         'serial_number': 'ADTN23124DB0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427027'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14254',
                                                                       'in_octets': '5606187983',
                                                                       'in_ucast_pkts': '27182948',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '242397156682',
                                                                       'out_ucast_pkts': '177117374',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVThfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '242397156682',
                                                                       'in_ucast_pkts': '177117374',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '242397156682',
                                                                                                'in_ucast_pkts': '177117374',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14254',
                                                                                                'out_octets': '5606187983',
                                                                                                'out_ucast_pkts': '27182948',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14254',
                                                                       'out_octets': '5606187983',
                                                                       'out_ucast_pkts': '27182948',
                                                                       'poller_hash': 'QURUTjIzMTI0RjM0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427030',
                                                                                                                        'fiber_length': 5229,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjM0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYzNA==',
                                         'serial_number': 'ADTN23124F34',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427030'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10922',
                                                                       'in_octets': '12769370752',
                                                                       'in_ucast_pkts': '61778045',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '284107873798',
                                                                       'out_ucast_pkts': '200982987',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '284107873798',
                                                                       'in_ucast_pkts': '200982987',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '284107873798',
                                                                                                'in_ucast_pkts': '200982987',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10922',
                                                                                                'out_octets': '12769370752',
                                                                                                'out_ucast_pkts': '61778045',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10922',
                                                                       'out_octets': '12769370752',
                                                                       'out_ucast_pkts': '61778045',
                                                                       'poller_hash': 'QURUTjIzMTI0REY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327030',
                                                                                                                        'fiber_length': 8325,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGNg==',
                                         'serial_number': 'ADTN23124DF6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327030'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10920',
                                                                       'in_octets': '437383688',
                                                                       'in_ucast_pkts': '893267',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '308702946',
                                                                       'out_ucast_pkts': '570377',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '308702946',
                                                                       'in_ucast_pkts': '570377',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '308702946',
                                                                                                'in_ucast_pkts': '570377',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10920',
                                                                                                'out_octets': '437383688',
                                                                                                'out_ucast_pkts': '893267',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10920',
                                                                       'out_octets': '437383688',
                                                                       'out_ucast_pkts': '893267',
                                                                       'poller_hash': 'QURUTjIzMTI0RTA0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327029',
                                                                                                                        'fiber_length': 8705,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTA0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUwNA==',
                                         'serial_number': 'ADTN23124E04',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11093',
                                                                       'in_octets': '7916900498',
                                                                       'in_ucast_pkts': '19430276',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '39267826124',
                                                                       'out_ucast_pkts': '33836502',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '39267826124',
                                                                       'in_ucast_pkts': '33836502',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '39267826124',
                                                                                                'in_ucast_pkts': '33836502',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11093',
                                                                                                'out_octets': '7916900498',
                                                                                                'out_ucast_pkts': '19430276',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11093',
                                                                       'out_octets': '7916900498',
                                                                       'out_ucast_pkts': '19430276',
                                                                       'poller_hash': 'QURUTjIzMTI0RTMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327026',
                                                                                                                        'fiber_length': 8651,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzMA==',
                                         'serial_number': 'ADTN23124E30',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327026'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11191',
                                                                       'in_octets': '9781526284',
                                                                       'in_ucast_pkts': '24332014',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '119157782184',
                                                                       'out_ucast_pkts': '89684982',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '119157782184',
                                                                       'in_ucast_pkts': '89684982',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '119157782184',
                                                                                                'in_ucast_pkts': '89684982',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11191',
                                                                                                'out_octets': '9781526284',
                                                                                                'out_ucast_pkts': '24332014',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11191',
                                                                       'out_octets': '9781526284',
                                                                       'out_ucast_pkts': '24332014',
                                                                       'poller_hash': 'QURUTjIzMTI0RTU3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327028',
                                                                                                                        'fiber_length': 6531,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTU3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU1Nw==',
                                         'serial_number': 'ADTN23124E57',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14251',
                                                                       'in_octets': '17584067874',
                                                                       'in_ucast_pkts': '26547393',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '99416976968',
                                                                       'out_ucast_pkts': '77910131',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTEwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '99416976968',
                                                                       'in_ucast_pkts': '77910131',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '99416976968',
                                                                                                'in_ucast_pkts': '77910131',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14251',
                                                                                                'out_octets': '17584067874',
                                                                                                'out_ucast_pkts': '26547393',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14251',
                                                                       'out_octets': '17584067874',
                                                                       'out_ucast_pkts': '26547393',
                                                                       'poller_hash': 'QURUTjIzMTIzRUI3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427033',
                                                                                                                        'fiber_length': 3552,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRUI3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0VCNw==',
                                         'serial_number': 'ADTN23123EB7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427033'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14833',
                                                                       'in_octets': '4278648759',
                                                                       'in_ucast_pkts': '28535445',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '173726757260',
                                                                       'out_ucast_pkts': '120494076',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTExX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '173726757260',
                                                                       'in_ucast_pkts': '120494076',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '173726757260',
                                                                                                'in_ucast_pkts': '120494076',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14833',
                                                                                                'out_octets': '4278648759',
                                                                                                'out_ucast_pkts': '28535445',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14833',
                                                                       'out_octets': '4278648759',
                                                                       'out_ucast_pkts': '28535445',
                                                                       'poller_hash': 'QURUTjIzMTI0RTM3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 5276,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTM3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzNw==',
                                         'serial_number': 'ADTN23124E37',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14728',
                                                                       'in_octets': '24093737952',
                                                                       'in_ucast_pkts': '106219013',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124E4B_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1223',
                                                                       'out_octets': '404520882600',
                                                                       'out_ucast_pkts': '288398654',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RTRCX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1223',
                                                                       'in_octets': '404520882600',
                                                                       'in_ucast_pkts': '288398654',
                                                                       'links': [       {       'in_nucast_pkts': '1223',
                                                                                                'in_octets': '404520882600',
                                                                                                'in_ucast_pkts': '288398654',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14728',
                                                                                                'out_octets': '24093737952',
                                                                                                'out_ucast_pkts': '106219013',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14728',
                                                                       'out_octets': '24093737952',
                                                                       'out_ucast_pkts': '106219013',
                                                                       'poller_hash': 'QURUTjIzMTI0RTRCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427025',
                                                                                                                        'fiber_length': 5187,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTRCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124E4B',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU0Qg==',
                                         'serial_number': 'ADTN23124E4B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427025'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '58851',
                                                                       'in_octets': '90918435541',
                                                                       'in_ucast_pkts': '161774759',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4984',
                                                                       'out_octets': '394429533551',
                                                                       'out_ucast_pkts': '335201598',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTRfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4984',
                                                                       'in_octets': '394429533551',
                                                                       'in_ucast_pkts': '335201598',
                                                                       'links': [       {       'in_nucast_pkts': '4984',
                                                                                                'in_octets': '394429533551',
                                                                                                'in_ucast_pkts': '335201598',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '58851',
                                                                                                'out_octets': '90918435541',
                                                                                                'out_ucast_pkts': '161774759',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '58851',
                                                                       'out_octets': '90918435541',
                                                                       'out_ucast_pkts': '161774759',
                                                                       'poller_hash': 'QURUTjIzMTI0REY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1735925',
                                                                                                                        'fiber_length': 8168,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGNw==',
                                         'serial_number': 'ADTN23124DF7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1735925'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10919',
                                                                       'in_octets': '7054881425',
                                                                       'in_ucast_pkts': '20904973',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '139917712727',
                                                                       'out_ucast_pkts': '99189673',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTEzX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '139917712727',
                                                                       'in_ucast_pkts': '99189673',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '139917712727',
                                                                                                'in_ucast_pkts': '99189673',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10919',
                                                                                                'out_octets': '7054881425',
                                                                                                'out_ucast_pkts': '20904973',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10919',
                                                                       'out_octets': '7054881425',
                                                                       'out_ucast_pkts': '20904973',
                                                                       'poller_hash': 'QURUTjIzMTI0RTczIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327028',
                                                                                                                        'fiber_length': 4886,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTczIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU3Mw==',
                                         'serial_number': 'ADTN23124E73',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14483',
                                                                       'in_octets': '6218284263',
                                                                       'in_ucast_pkts': '24698100',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '92812486069',
                                                                       'out_ucast_pkts': '66531067',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTE1X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '92812486069',
                                                                       'in_ucast_pkts': '66531067',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '92812486069',
                                                                                                'in_ucast_pkts': '66531067',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14483',
                                                                                                'out_octets': '6218284263',
                                                                                                'out_ucast_pkts': '24698100',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14483',
                                                                       'out_octets': '6218284263',
                                                                       'out_ucast_pkts': '24698100',
                                                                       'poller_hash': 'QURUTjIzMTI0RTE3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427031',
                                                                                                                        'fiber_length': 4322,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTE3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxNw==',
                                         'serial_number': 'ADTN23124E17',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427031'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75669',
                                                                       'in_octets': '24276410552',
                                                                       'in_ucast_pkts': '74655460',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6403',
                                                                       'out_octets': '189926341187',
                                                                       'out_ucast_pkts': '146275978',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6403',
                                                                       'in_octets': '189926341187',
                                                                       'in_ucast_pkts': '146275978',
                                                                       'links': [       {       'in_nucast_pkts': '6403',
                                                                                                'in_octets': '189926341187',
                                                                                                'in_ucast_pkts': '146275978',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75669',
                                                                                                'out_octets': '24276410552',
                                                                                                'out_ucast_pkts': '74655460',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75669',
                                                                       'out_octets': '24276410552',
                                                                       'out_ucast_pkts': '74655460',
                                                                       'poller_hash': 'QURUTjIzMTI1MUIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232584',
                                                                                                                        'fiber_length': 5245,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFCMA==',
                                         'serial_number': 'ADTN231251B0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232584'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14258',
                                                                       'in_octets': '4222787347',
                                                                       'in_ucast_pkts': '10744367',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '31820503360',
                                                                       'out_ucast_pkts': '28427247',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '31820503360',
                                                                       'in_ucast_pkts': '28427247',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '31820503360',
                                                                                                'in_ucast_pkts': '28427247',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14258',
                                                                                                'out_octets': '4222787347',
                                                                                                'out_ucast_pkts': '10744367',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14258',
                                                                       'out_octets': '4222787347',
                                                                       'out_ucast_pkts': '10744367',
                                                                       'poller_hash': 'QURUTjIzMTI0RTJCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427039',
                                                                                                                        'fiber_length': 5938,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTJCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyQg==',
                                         'serial_number': 'ADTN23124E2B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427039'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75269',
                                                                       'in_octets': '36222598925',
                                                                       'in_ucast_pkts': '207835924',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6347',
                                                                       'out_octets': '689783876933',
                                                                       'out_ucast_pkts': '491851054',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6347',
                                                                       'in_octets': '689783876933',
                                                                       'in_ucast_pkts': '491851054',
                                                                       'links': [       {       'in_nucast_pkts': '6347',
                                                                                                'in_octets': '689783876933',
                                                                                                'in_ucast_pkts': '491851054',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75269',
                                                                                                'out_octets': '36222598925',
                                                                                                'out_ucast_pkts': '207835924',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75269',
                                                                       'out_octets': '36222598925',
                                                                       'out_ucast_pkts': '207835924',
                                                                       'poller_hash': 'QURUTjIzMTI1MUVFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232578',
                                                                                                                        'fiber_length': 4569,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUVFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFFRQ==',
                                         'serial_number': 'ADTN231251EE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232578'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10993',
                                                                       'in_octets': '11999564548',
                                                                       'in_ucast_pkts': '69806944',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '352265173015',
                                                                       'out_ucast_pkts': '253140500',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '352265173015',
                                                                       'in_ucast_pkts': '253140500',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '352265173015',
                                                                                                'in_ucast_pkts': '253140500',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10993',
                                                                                                'out_octets': '11999564548',
                                                                                                'out_ucast_pkts': '69806944',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10993',
                                                                       'out_octets': '11999564548',
                                                                       'out_ucast_pkts': '69806944',
                                                                       'poller_hash': 'QURUTjIzMTI0RTkyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327029',
                                                                                                                        'fiber_length': 8747,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTkyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU5Mg==',
                                         'serial_number': 'ADTN23124E92',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14687',
                                                                       'in_octets': '16436354372',
                                                                       'in_ucast_pkts': '47215406',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '487',
                                                                       'out_octets': '132008364272',
                                                                       'out_ucast_pkts': '102344793',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '487',
                                                                       'in_octets': '132008364272',
                                                                       'in_ucast_pkts': '102344793',
                                                                       'links': [       {       'in_nucast_pkts': '487',
                                                                                                'in_octets': '132008364272',
                                                                                                'in_ucast_pkts': '102344793',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14687',
                                                                                                'out_octets': '16436354372',
                                                                                                'out_ucast_pkts': '47215406',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14687',
                                                                       'out_octets': '16436354372',
                                                                       'out_ucast_pkts': '47215406',
                                                                       'poller_hash': 'QURUTjIzMTI0RTIzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427039',
                                                                                                                        'fiber_length': 4748,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTIzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyMw==',
                                         'serial_number': 'ADTN23124E23',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427039'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.25.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14493',
                                                                       'in_octets': '7921703286',
                                                                       'in_ucast_pkts': '58170493',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU25_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '169020056180',
                                                                       'out_ucast_pkts': '133811964',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '169020056180',
                                                                       'in_ucast_pkts': '133811964',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '169020056180',
                                                                                                'in_ucast_pkts': '133811964',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14493',
                                                                                                'out_octets': '7921703286',
                                                                                                'out_ucast_pkts': '58170493',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14493',
                                                                       'out_octets': '7921703286',
                                                                       'out_ucast_pkts': '58170493',
                                                                       'poller_hash': 'QURUTjIzMTI0RTI3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 4716,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTI3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU25',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyNw==',
                                         'serial_number': 'ADTN23124E27',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14260',
                                                                       'in_octets': '6557138016',
                                                                       'in_ucast_pkts': '37645152',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '163595952937',
                                                                       'out_ucast_pkts': '114245219',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '163595952937',
                                                                       'in_ucast_pkts': '114245219',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '163595952937',
                                                                                                'in_ucast_pkts': '114245219',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14260',
                                                                                                'out_octets': '6557138016',
                                                                                                'out_ucast_pkts': '37645152',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14260',
                                                                       'out_octets': '6557138016',
                                                                       'out_ucast_pkts': '37645152',
                                                                       'poller_hash': 'QURUTjIzMTI0RTY0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427034',
                                                                                                                        'fiber_length': 4657,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTY0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU2NA==',
                                         'serial_number': 'ADTN23124E64',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427034'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14478',
                                                                       'in_octets': '4192434431',
                                                                       'in_ucast_pkts': '9735185',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124F14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '27110650859',
                                                                       'out_ucast_pkts': '23016861',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RjE0X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '27110650859',
                                                                       'in_ucast_pkts': '23016861',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '27110650859',
                                                                                                'in_ucast_pkts': '23016861',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14478',
                                                                                                'out_octets': '4192434431',
                                                                                                'out_ucast_pkts': '9735185',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14478',
                                                                       'out_octets': '4192434431',
                                                                       'out_ucast_pkts': '9735185',
                                                                       'poller_hash': 'QURUTjIzMTI0RjE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427033',
                                                                                                                        'fiber_length': 7344,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124F14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYxNA==',
                                         'serial_number': 'ADTN23124F14',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427033'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.27.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14490',
                                                                       'in_octets': '3504213634',
                                                                       'in_ucast_pkts': '17893442',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124E6B_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '60693102340',
                                                                       'out_ucast_pkts': '43963678',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RTZCX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '60693102340',
                                                                       'in_ucast_pkts': '43963678',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '60693102340',
                                                                                                'in_ucast_pkts': '43963678',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14490',
                                                                                                'out_octets': '3504213634',
                                                                                                'out_ucast_pkts': '17893442',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14490',
                                                                       'out_octets': '3504213634',
                                                                       'out_ucast_pkts': '17893442',
                                                                       'poller_hash': 'QURUTjIzMTI0RTZCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427034',
                                                                                                                        'fiber_length': 5879,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTZCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124E6B',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU2Qg==',
                                         'serial_number': 'ADTN23124E6B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427034'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '53346',
                                                                       'in_octets': '71092519004',
                                                                       'in_ucast_pkts': '217419329',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4406',
                                                                       'out_octets': '911286037291',
                                                                       'out_ucast_pkts': '655351516',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4406',
                                                                       'in_octets': '911286037291',
                                                                       'in_ucast_pkts': '655351516',
                                                                       'links': [       {       'in_nucast_pkts': '4406',
                                                                                                'in_octets': '911286037291',
                                                                                                'in_ucast_pkts': '655351516',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '53346',
                                                                                                'out_octets': '71092519004',
                                                                                                'out_ucast_pkts': '217419329',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '53346',
                                                                       'out_octets': '71092519004',
                                                                       'out_ucast_pkts': '217419329',
                                                                       'poller_hash': 'QURUTjIzMTI1MTdGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1535983',
                                                                                                                        'fiber_length': 7112,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTdGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE3Rg==',
                                         'serial_number': 'ADTN2312517F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1535983'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77462',
                                                                       'in_octets': '89905790007',
                                                                       'in_ucast_pkts': '229297951',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6403',
                                                                       'out_octets': '764692895348',
                                                                       'out_ucast_pkts': '563283399',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6403',
                                                                       'in_octets': '764692895348',
                                                                       'in_ucast_pkts': '563283399',
                                                                       'links': [       {       'in_nucast_pkts': '6403',
                                                                                                'in_octets': '764692895348',
                                                                                                'in_ucast_pkts': '563283399',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77462',
                                                                                                'out_octets': '89905790007',
                                                                                                'out_ucast_pkts': '229297951',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77462',
                                                                       'out_octets': '89905790007',
                                                                       'out_ucast_pkts': '229297951',
                                                                       'poller_hash': 'QURUTjIzMTI0Rjc0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232586',
                                                                                                                        'fiber_length': 6325,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0Rjc0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEY3NA==',
                                         'serial_number': 'ADTN23124F74',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232586'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.28.0.eth.1.phy',
                                                                       'in_nucast_pkts': '2283',
                                                                       'in_octets': '217397666',
                                                                       'in_ucast_pkts': '576131',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU28_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '76',
                                                                       'out_octets': '699514356',
                                                                       'out_ucast_pkts': '834306',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjhfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '76',
                                                                       'in_octets': '699514356',
                                                                       'in_ucast_pkts': '834306',
                                                                       'links': [       {       'in_nucast_pkts': '76',
                                                                                                'in_octets': '699514356',
                                                                                                'in_ucast_pkts': '834306',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '2283',
                                                                                                'out_octets': '217397666',
                                                                                                'out_ucast_pkts': '576131',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2283',
                                                                       'out_octets': '217397666',
                                                                       'out_ucast_pkts': '576131',
                                                                       'poller_hash': 'QURUTjIzMTIzRkVDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '68291',
                                                                                                                        'fiber_length': 5819,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkVDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU28',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZFQw==',
                                         'serial_number': 'ADTN23123FEC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '68291'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '158840',
                                                                       'in_octets': '49755568489',
                                                                       'in_ucast_pkts': '151433781',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5302',
                                                                       'out_octets': '504357300674',
                                                                       'out_ucast_pkts': '390076108',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5302',
                                                                       'in_octets': '504357300674',
                                                                       'in_ucast_pkts': '390076108',
                                                                       'links': [       {       'in_nucast_pkts': '5302',
                                                                                                'in_octets': '504357300674',
                                                                                                'in_ucast_pkts': '390076108',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '158840',
                                                                                                'out_octets': '49755568489',
                                                                                                'out_ucast_pkts': '151433781',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '158840',
                                                                       'out_octets': '49755568489',
                                                                       'out_ucast_pkts': '151433781',
                                                                       'poller_hash': 'QURUTjIzMTI1MDYxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4773323',
                                                                                                                        'fiber_length': 10820,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDYxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA2MQ==',
                                         'serial_number': 'ADTN23125061',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '4773323'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74431',
                                                                       'in_octets': '49201605747',
                                                                       'in_ucast_pkts': '312371320',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '1167157019048',
                                                                       'out_ucast_pkts': '818793600',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '1167157019048',
                                                                       'in_ucast_pkts': '818793600',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '1167157019048',
                                                                                                'in_ucast_pkts': '818793600',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74431',
                                                                                                'out_octets': '49201605747',
                                                                                                'out_ucast_pkts': '312371320',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74431',
                                                                       'out_octets': '49201605747',
                                                                       'out_ucast_pkts': '312371320',
                                                                       'poller_hash': 'QURUTjIzMTI0REI3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232581',
                                                                                                                        'fiber_length': 2058,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REI3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERCNw==',
                                         'serial_number': 'ADTN23124DB7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232581'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14837',
                                                                       'in_octets': '8526432020',
                                                                       'in_ucast_pkts': '61587656',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '260945147663',
                                                                       'out_ucast_pkts': '182878707',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTE2X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '260945147663',
                                                                       'in_ucast_pkts': '182878707',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '260945147663',
                                                                                                'in_ucast_pkts': '182878707',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14837',
                                                                                                'out_octets': '8526432020',
                                                                                                'out_ucast_pkts': '61587656',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14837',
                                                                       'out_octets': '8526432020',
                                                                       'out_ucast_pkts': '61587656',
                                                                       'poller_hash': 'QURUTjIzMTI1MTJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427030',
                                                                                                                        'fiber_length': 5460,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEyQw==',
                                         'serial_number': 'ADTN2312512C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427030'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14479',
                                                                       'in_octets': '3965871616',
                                                                       'in_ucast_pkts': '17863515',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '83823481543',
                                                                       'out_ucast_pkts': '61501151',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTE3X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '83823481543',
                                                                       'in_ucast_pkts': '61501151',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '83823481543',
                                                                                                'in_ucast_pkts': '61501151',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14479',
                                                                                                'out_octets': '3965871616',
                                                                                                'out_ucast_pkts': '17863515',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14479',
                                                                       'out_octets': '3965871616',
                                                                       'out_ucast_pkts': '17863515',
                                                                       'poller_hash': 'QURUTjIzMTIzRUExIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427032',
                                                                                                                        'fiber_length': 3586,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRUExIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0VBMQ==',
                                         'serial_number': 'ADTN23123EA1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427032'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14487',
                                                                       'in_octets': '3409732582',
                                                                       'in_ucast_pkts': '24706889',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '69705065738',
                                                                       'out_ucast_pkts': '48123549',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '69705065738',
                                                                       'in_ucast_pkts': '48123549',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '69705065738',
                                                                                                'in_ucast_pkts': '48123549',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14487',
                                                                                                'out_octets': '3409732582',
                                                                                                'out_ucast_pkts': '24706889',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14487',
                                                                       'out_octets': '3409732582',
                                                                       'out_ucast_pkts': '24706889',
                                                                       'poller_hash': 'QURUTjIzMTI0REM3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427037',
                                                                                                                        'fiber_length': 2394,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REM3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERDNw==',
                                         'serial_number': 'ADTN23124DC7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427037'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11102',
                                                                       'in_octets': '8102513606',
                                                                       'in_ucast_pkts': '28691963',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '80493637516',
                                                                       'out_ucast_pkts': '57497526',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '80493637516',
                                                                       'in_ucast_pkts': '57497526',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '80493637516',
                                                                                                'in_ucast_pkts': '57497526',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11102',
                                                                                                'out_octets': '8102513606',
                                                                                                'out_ucast_pkts': '28691963',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11102',
                                                                       'out_octets': '8102513606',
                                                                       'out_ucast_pkts': '28691963',
                                                                       'poller_hash': 'QURUTjIzMTI0RUZBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327189',
                                                                                                                        'fiber_length': 2496,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RUZBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVGQQ==',
                                         'serial_number': 'ADTN23124EFA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327189'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14480',
                                                                       'in_octets': '31320035261',
                                                                       'in_ucast_pkts': '78000070',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '276529278879',
                                                                       'out_ucast_pkts': '201540243',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTE4X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '276529278879',
                                                                       'in_ucast_pkts': '201540243',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '276529278879',
                                                                                                'in_ucast_pkts': '201540243',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14480',
                                                                                                'out_octets': '31320035261',
                                                                                                'out_ucast_pkts': '78000070',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14480',
                                                                       'out_octets': '31320035261',
                                                                       'out_ucast_pkts': '78000070',
                                                                       'poller_hash': 'QURUTjIzMTI1MTMzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 3613,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTMzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEzMw==',
                                         'serial_number': 'ADTN23125133',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14711',
                                                                       'in_octets': '6058566606',
                                                                       'in_ucast_pkts': '14324366',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '56384755529',
                                                                       'out_ucast_pkts': '43173980',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '56384755529',
                                                                       'in_ucast_pkts': '43173980',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '56384755529',
                                                                                                'in_ucast_pkts': '43173980',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14711',
                                                                                                'out_octets': '6058566606',
                                                                                                'out_ucast_pkts': '14324366',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14711',
                                                                       'out_octets': '6058566606',
                                                                       'out_ucast_pkts': '14324366',
                                                                       'poller_hash': 'QURUTjIzMTI0RURGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427037',
                                                                                                                        'fiber_length': 3018,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RURGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVERg==',
                                         'serial_number': 'ADTN23124EDF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427037'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '76290',
                                                                       'in_octets': '51219008561',
                                                                       'in_ucast_pkts': '286936107',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '978368486421',
                                                                       'out_ucast_pkts': '730330180',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '978368486421',
                                                                       'in_ucast_pkts': '730330180',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '978368486421',
                                                                                                'in_ucast_pkts': '730330180',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '76290',
                                                                                                'out_octets': '51219008561',
                                                                                                'out_ucast_pkts': '286936107',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '76290',
                                                                       'out_octets': '51219008561',
                                                                       'out_ucast_pkts': '286936107',
                                                                       'poller_hash': 'QURUTjIzMTI1MTcxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232590',
                                                                                                                        'fiber_length': 10756,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTcxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE3MQ==',
                                         'serial_number': 'ADTN23125171',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232590'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '21001',
                                                                       'in_octets': '11161088054',
                                                                       'in_ucast_pkts': '58228056',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124EA5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1735',
                                                                       'out_octets': '218225110338',
                                                                       'out_ucast_pkts': '157171893',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RUE1X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1735',
                                                                       'in_octets': '218225110338',
                                                                       'in_ucast_pkts': '157171893',
                                                                       'links': [       {       'in_nucast_pkts': '1735',
                                                                                                'in_octets': '218225110338',
                                                                                                'in_ucast_pkts': '157171893',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '21001',
                                                                                                'out_octets': '11161088054',
                                                                                                'out_ucast_pkts': '58228056',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '21001',
                                                                       'out_octets': '11161088054',
                                                                       'out_ucast_pkts': '58228056',
                                                                       'poller_hash': 'QURUTjIzMTI0RUE1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '604300',
                                                                                                                        'fiber_length': 3857,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RUE1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124EA5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVBNQ==',
                                         'serial_number': 'ADTN23124EA5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '604300'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74476',
                                                                       'in_octets': '192252977689',
                                                                       'in_ucast_pkts': '847205322',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '3450434962360',
                                                                       'out_ucast_pkts': '2556552687',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VOV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '3450434962360',
                                                                       'in_ucast_pkts': '2556552687',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '3450434962360',
                                                                                                'in_ucast_pkts': '2556552687',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74476',
                                                                                                'out_octets': '192252977689',
                                                                                                'out_ucast_pkts': '847205322',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74476',
                                                                       'out_octets': '192252977689',
                                                                       'out_ucast_pkts': '847205322',
                                                                       'poller_hash': 'QURUTjIzMTI0OEE4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232583',
                                                                                                                        'fiber_length': 2069,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OEE4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDhBOA==',
                                         'serial_number': 'ADTN231248A8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232583'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.29.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14491',
                                                                       'in_octets': '1222611241',
                                                                       'in_ucast_pkts': '4483675',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU29_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '14554324780',
                                                                       'out_ucast_pkts': '11327632',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMjlfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '14554324780',
                                                                       'in_ucast_pkts': '11327632',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '14554324780',
                                                                                                'in_ucast_pkts': '11327632',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14491',
                                                                                                'out_octets': '1222611241',
                                                                                                'out_ucast_pkts': '4483675',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14491',
                                                                       'out_octets': '1222611241',
                                                                       'out_ucast_pkts': '4483675',
                                                                       'poller_hash': 'QURUTjIzMTIzRUE4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427036',
                                                                                                                        'fiber_length': 4449,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRUE4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU29',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0VBOA==',
                                         'serial_number': 'ADTN23123EA8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427036'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '183322',
                                                                       'in_octets': '237506751290',
                                                                       'in_ucast_pkts': '1415059809',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124E52_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15384',
                                                                       'out_octets': '4720683348841',
                                                                       'out_ucast_pkts': '3430807496',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RTUyX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15384',
                                                                       'in_octets': '4720683348841',
                                                                       'in_ucast_pkts': '3430807496',
                                                                       'links': [       {       'in_nucast_pkts': '15384',
                                                                                                'in_octets': '4720683348841',
                                                                                                'in_ucast_pkts': '3430807496',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '183322',
                                                                                                'out_octets': '237506751290',
                                                                                                'out_ucast_pkts': '1415059809',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '183322',
                                                                       'out_octets': '237506751290',
                                                                       'out_ucast_pkts': '1415059809',
                                                                       'poller_hash': 'QURUTjIzMTI0RTUyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355629',
                                                                                                                        'fiber_length': 9016,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTUyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124E52',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU1Mg==',
                                         'serial_number': 'ADTN23124E52',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355629'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14487',
                                                                       'in_octets': '22467858323',
                                                                       'in_ucast_pkts': '66788495',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '192046021310',
                                                                       'out_ucast_pkts': '146416225',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '192046021310',
                                                                       'in_ucast_pkts': '146416225',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '192046021310',
                                                                                                'in_ucast_pkts': '146416225',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14487',
                                                                                                'out_octets': '22467858323',
                                                                                                'out_ucast_pkts': '66788495',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14487',
                                                                       'out_octets': '22467858323',
                                                                       'out_ucast_pkts': '66788495',
                                                                       'poller_hash': 'QURUTjIzMTI1MTVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427039',
                                                                                                                        'fiber_length': 6916,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE1Qg==',
                                         'serial_number': 'ADTN2312515B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427039'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14495',
                                                                       'in_octets': '10077191716',
                                                                       'in_ucast_pkts': '46633002',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '121818231744',
                                                                       'out_ucast_pkts': '104751984',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '121818231744',
                                                                       'in_ucast_pkts': '104751984',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '121818231744',
                                                                                                'in_ucast_pkts': '104751984',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14495',
                                                                                                'out_octets': '10077191716',
                                                                                                'out_ucast_pkts': '46633002',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14495',
                                                                       'out_octets': '10077191716',
                                                                       'out_ucast_pkts': '46633002',
                                                                       'poller_hash': 'QURUTjIzMTI1MUIxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427037',
                                                                                                                        'fiber_length': 6439,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUIxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFCMQ==',
                                         'serial_number': 'ADTN231251B1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427037'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14487',
                                                                       'in_octets': '1408453082',
                                                                       'in_ucast_pkts': '7039557',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '16740090927',
                                                                       'out_ucast_pkts': '12732063',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '16740090927',
                                                                       'in_ucast_pkts': '12732063',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '16740090927',
                                                                                                'in_ucast_pkts': '12732063',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14487',
                                                                                                'out_octets': '1408453082',
                                                                                                'out_ucast_pkts': '7039557',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14487',
                                                                       'out_octets': '1408453082',
                                                                       'out_ucast_pkts': '7039557',
                                                                       'poller_hash': 'QURUTjIzMTI1MUY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 2254,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFGNw==',
                                         'serial_number': 'ADTN231251F7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14489',
                                                                       'in_octets': '17033134539',
                                                                       'in_ucast_pkts': '38082733',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '83737263849',
                                                                       'out_ucast_pkts': '65683823',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '83737263849',
                                                                       'in_ucast_pkts': '65683823',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '83737263849',
                                                                                                'in_ucast_pkts': '65683823',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14489',
                                                                                                'out_octets': '17033134539',
                                                                                                'out_ucast_pkts': '38082733',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14489',
                                                                       'out_octets': '17033134539',
                                                                       'out_ucast_pkts': '38082733',
                                                                       'poller_hash': 'QURUTjIzMTI0RTg0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427038',
                                                                                                                        'fiber_length': 3256,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTg0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU4NA==',
                                         'serial_number': 'ADTN23124E84',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427038'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-6.1.3.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x6-ONU3_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VM19FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0QTIzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QTIzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x6-ONU3',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '6.1.3"(ADTN23124A23) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/6.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-08-14T13:51:35Z',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEEyMw==',
                                         'serial_number': 'ADTN23124A23',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.30.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14488',
                                                                       'in_octets': '2563843765',
                                                                       'in_ucast_pkts': '23288569',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU30_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '94069604523',
                                                                       'out_ucast_pkts': '65745034',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMzBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '94069604523',
                                                                       'in_ucast_pkts': '65745034',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '94069604523',
                                                                                                'in_ucast_pkts': '65745034',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14488',
                                                                                                'out_octets': '2563843765',
                                                                                                'out_ucast_pkts': '23288569',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14488',
                                                                       'out_octets': '2563843765',
                                                                       'out_ucast_pkts': '23288569',
                                                                       'poller_hash': 'QURUTjIzMTI0RTFGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427039',
                                                                                                                        'fiber_length': 4391,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTFGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU30',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxRg==',
                                         'serial_number': 'ADTN23124E1F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427039'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14836',
                                                                       'in_octets': '11623302477',
                                                                       'in_ucast_pkts': '42961367',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '90157267496',
                                                                       'out_ucast_pkts': '71266091',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '90157267496',
                                                                       'in_ucast_pkts': '71266091',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '90157267496',
                                                                                                'in_ucast_pkts': '71266091',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14836',
                                                                                                'out_octets': '11623302477',
                                                                                                'out_ucast_pkts': '42961367',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14836',
                                                                       'out_octets': '11623302477',
                                                                       'out_ucast_pkts': '42961367',
                                                                       'poller_hash': 'QURUTjIzMTI0RjM3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 7364,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjM3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYzNw==',
                                         'serial_number': 'ADTN23124F37',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '112169',
                                                                       'in_octets': '21016518482',
                                                                       'in_ucast_pkts': '99931401',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3740',
                                                                       'out_octets': '441127091202',
                                                                       'out_ucast_pkts': '309978043',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3740',
                                                                       'in_octets': '441127091202',
                                                                       'in_ucast_pkts': '309978043',
                                                                       'links': [       {       'in_nucast_pkts': '3740',
                                                                                                'in_octets': '441127091202',
                                                                                                'in_ucast_pkts': '309978043',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '112169',
                                                                                                'out_octets': '21016518482',
                                                                                                'out_ucast_pkts': '99931401',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '112169',
                                                                       'out_octets': '21016518482',
                                                                       'out_ucast_pkts': '99931401',
                                                                       'poller_hash': 'QURUTjIzMTI0RThBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3364584',
                                                                                                                        'fiber_length': 6277,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RThBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU4QQ==',
                                         'serial_number': 'ADTN23124E8A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3364584'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '112172',
                                                                       'in_octets': '29451437801',
                                                                       'in_ucast_pkts': '146969540',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3740',
                                                                       'out_octets': '477818042319',
                                                                       'out_ucast_pkts': '349944811',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3740',
                                                                       'in_octets': '477818042319',
                                                                       'in_ucast_pkts': '349944811',
                                                                       'links': [       {       'in_nucast_pkts': '3740',
                                                                                                'in_octets': '477818042319',
                                                                                                'in_ucast_pkts': '349944811',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '112172',
                                                                                                'out_octets': '29451437801',
                                                                                                'out_ucast_pkts': '146969540',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '112172',
                                                                       'out_octets': '29451437801',
                                                                       'out_ucast_pkts': '146969540',
                                                                       'poller_hash': 'QURUTjIzMTI1MDY1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3364586',
                                                                                                                        'fiber_length': 6171,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDY1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA2NQ==',
                                         'serial_number': 'ADTN23125065',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3364586'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14491',
                                                                       'in_octets': '50179152392',
                                                                       'in_ucast_pkts': '165755454',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '509052632069',
                                                                       'out_ucast_pkts': '368118322',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '509052632069',
                                                                       'in_ucast_pkts': '368118322',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '509052632069',
                                                                                                'in_ucast_pkts': '368118322',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14491',
                                                                                                'out_octets': '50179152392',
                                                                                                'out_ucast_pkts': '165755454',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14491',
                                                                       'out_octets': '50179152392',
                                                                       'out_ucast_pkts': '165755454',
                                                                       'poller_hash': 'QURUTjIzMTI1MUY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427036',
                                                                                                                        'fiber_length': 2382,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFGNg==',
                                         'serial_number': 'ADTN231251F6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427036'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14491',
                                                                       'in_octets': '25747209778',
                                                                       'in_ucast_pkts': '86361868',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '258710619021',
                                                                       'out_ucast_pkts': '205310075',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '258710619021',
                                                                       'in_ucast_pkts': '205310075',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '258710619021',
                                                                                                'in_ucast_pkts': '205310075',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14491',
                                                                                                'out_octets': '25747209778',
                                                                                                'out_ucast_pkts': '86361868',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14491',
                                                                       'out_octets': '25747209778',
                                                                       'out_ucast_pkts': '86361868',
                                                                       'poller_hash': 'QURUTjIzMTIzRTlEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427037',
                                                                                                                        'fiber_length': 6505,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRTlEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0U5RA==',
                                         'serial_number': 'ADTN23123E9D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427037'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14258',
                                                                       'in_octets': '37144367397',
                                                                       'in_ucast_pkts': '86633184',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '319146165890',
                                                                       'out_ucast_pkts': '229692615',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTIwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '319146165890',
                                                                       'in_ucast_pkts': '229692615',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '319146165890',
                                                                                                'in_ucast_pkts': '229692615',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14258',
                                                                                                'out_octets': '37144367397',
                                                                                                'out_ucast_pkts': '86633184',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14258',
                                                                       'out_octets': '37144367397',
                                                                       'out_ucast_pkts': '86633184',
                                                                       'poller_hash': 'QURUTjIzMTI1MUFCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 5452,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUFCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFBQg==',
                                         'serial_number': 'ADTN231251AB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '264',
                                                                       'in_octets': '16626695046',
                                                                       'in_ucast_pkts': '31430246',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '110967086335',
                                                                       'out_ucast_pkts': '87932377',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VN19FVEhfMQ==',
                                                                       'speed': '100000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '110967086335',
                                                                       'in_ucast_pkts': '87932377',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '110967086335',
                                                                                                'in_ucast_pkts': '87932377',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '264',
                                                                                                'out_octets': '16626695046',
                                                                                                'out_ucast_pkts': '31430246',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '264',
                                                                       'out_octets': '16626695046',
                                                                       'out_ucast_pkts': '31430246',
                                                                       'poller_hash': 'QURUTjIzMTI0RUQzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427033',
                                                                                                                        'fiber_length': 2629,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RUQzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVEMw==',
                                         'serial_number': 'ADTN23124ED3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427033'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14843',
                                                                       'in_octets': '86559397424',
                                                                       'in_ucast_pkts': '191752268',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '511160420142',
                                                                       'out_ucast_pkts': '393278257',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '511160420142',
                                                                       'in_ucast_pkts': '393278257',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '511160420142',
                                                                                                'in_ucast_pkts': '393278257',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14843',
                                                                                                'out_octets': '86559397424',
                                                                                                'out_ucast_pkts': '191752268',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14843',
                                                                       'out_octets': '86559397424',
                                                                       'out_ucast_pkts': '191752268',
                                                                       'poller_hash': 'QURUTjIzMTI0RTMzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427036',
                                                                                                                        'fiber_length': 6490,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTMzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUzMw==',
                                         'serial_number': 'ADTN23124E33',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427036'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '90358',
                                                                       'in_octets': '46234900077',
                                                                       'in_ucast_pkts': '306741005',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '7625',
                                                                       'out_octets': '1355647793471',
                                                                       'out_ucast_pkts': '1038143637',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '7625',
                                                                       'in_octets': '1355647793471',
                                                                       'in_ucast_pkts': '1038143637',
                                                                       'links': [       {       'in_nucast_pkts': '7625',
                                                                                                'in_octets': '1355647793471',
                                                                                                'in_ucast_pkts': '1038143637',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '90358',
                                                                                                'out_octets': '46234900077',
                                                                                                'out_ucast_pkts': '306741005',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '90358',
                                                                       'out_octets': '46234900077',
                                                                       'out_ucast_pkts': '306741005',
                                                                       'poller_hash': 'QURUTjIzMTI0RTREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2658219',
                                                                                                                        'fiber_length': 10425,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU0RA==',
                                         'serial_number': 'ADTN23124E4D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2658219'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75663',
                                                                       'in_octets': '28885852860',
                                                                       'in_ucast_pkts': '160066746',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN2312509E_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6406',
                                                                       'out_octets': '570354592812',
                                                                       'out_ucast_pkts': '405768101',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI1MDlFX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6406',
                                                                       'in_octets': '570354592812',
                                                                       'in_ucast_pkts': '405768101',
                                                                       'links': [       {       'in_nucast_pkts': '6406',
                                                                                                'in_octets': '570354592812',
                                                                                                'in_ucast_pkts': '405768101',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75663',
                                                                                                'out_octets': '28885852860',
                                                                                                'out_ucast_pkts': '160066746',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75663',
                                                                       'out_octets': '28885852860',
                                                                       'out_ucast_pkts': '160066746',
                                                                       'poller_hash': 'QURUTjIzMTI1MDlFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232581',
                                                                                                                        'fiber_length': 2313,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDlFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN2312509E',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA5RQ==',
                                         'serial_number': 'ADTN2312509E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232581'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74438',
                                                                       'in_octets': '9283026051',
                                                                       'in_ucast_pkts': '29323621',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23123E9E_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2480',
                                                                       'out_octets': '82760998155',
                                                                       'out_ucast_pkts': '67787830',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTIzRTlFX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2480',
                                                                       'in_octets': '82760998155',
                                                                       'in_ucast_pkts': '67787830',
                                                                       'links': [       {       'in_nucast_pkts': '2480',
                                                                                                'in_octets': '82760998155',
                                                                                                'in_ucast_pkts': '67787830',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74438',
                                                                                                'out_octets': '9283026051',
                                                                                                'out_ucast_pkts': '29323621',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74438',
                                                                       'out_octets': '9283026051',
                                                                       'out_ucast_pkts': '29323621',
                                                                       'poller_hash': 'QURUTjIzMTIzRTlFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232582',
                                                                                                                        'fiber_length': 4488,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRTlFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23123E9E',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0U5RQ==',
                                         'serial_number': 'ADTN23123E9E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232582'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75673',
                                                                       'in_octets': '16375391903',
                                                                       'in_ucast_pkts': '94315921',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124E2A_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6407',
                                                                       'out_octets': '416345970025',
                                                                       'out_ucast_pkts': '307636158',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RTJBX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6407',
                                                                       'in_octets': '416345970025',
                                                                       'in_ucast_pkts': '307636158',
                                                                       'links': [       {       'in_nucast_pkts': '6407',
                                                                                                'in_octets': '416345970025',
                                                                                                'in_ucast_pkts': '307636158',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75673',
                                                                                                'out_octets': '16375391903',
                                                                                                'out_ucast_pkts': '94315921',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75673',
                                                                       'out_octets': '16375391903',
                                                                       'out_ucast_pkts': '94315921',
                                                                       'poller_hash': 'QURUTjIzMTI0RTJBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232588',
                                                                                                                        'fiber_length': 5521,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTJBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124E2A',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyQQ==',
                                         'serial_number': 'ADTN23124E2A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232588'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74566',
                                                                       'in_octets': '168473594025',
                                                                       'in_ucast_pkts': '584126252',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124EA4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2480',
                                                                       'out_octets': '1789470340955',
                                                                       'out_ucast_pkts': '1279183727',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0RUE0X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2480',
                                                                       'in_octets': '1789470340955',
                                                                       'in_ucast_pkts': '1279183727',
                                                                       'links': [       {       'in_nucast_pkts': '2480',
                                                                                                'in_octets': '1789470340955',
                                                                                                'in_ucast_pkts': '1279183727',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74566',
                                                                                                'out_octets': '168473594025',
                                                                                                'out_ucast_pkts': '584126252',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74566',
                                                                       'out_octets': '168473594025',
                                                                       'out_ucast_pkts': '584126252',
                                                                       'poller_hash': 'QURUTjIzMTI0RUE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232585',
                                                                                                                        'fiber_length': 5919,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RUE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124EA4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEVBNA==',
                                         'serial_number': 'ADTN23124EA4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232585'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '114024',
                                                                       'in_octets': '132343926606',
                                                                       'in_ucast_pkts': '358229940',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9652',
                                                                       'out_octets': '1077935386793',
                                                                       'out_ucast_pkts': '789288784',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9652',
                                                                       'in_octets': '1077935386793',
                                                                       'in_ucast_pkts': '789288784',
                                                                       'links': [       {       'in_nucast_pkts': '9652',
                                                                                                'in_octets': '1077935386793',
                                                                                                'in_ucast_pkts': '789288784',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '114024',
                                                                                                'out_octets': '132343926606',
                                                                                                'out_ucast_pkts': '358229940',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '114024',
                                                                       'out_octets': '132343926606',
                                                                       'out_ucast_pkts': '358229940',
                                                                       'poller_hash': 'QURUTjIzMTI0RTQzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3364587',
                                                                                                                        'fiber_length': 6584,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTQzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU0Mw==',
                                         'serial_number': 'ADTN23124E43',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3364587'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '12185',
                                                                       'in_octets': '7659325026',
                                                                       'in_ucast_pkts': '25544182',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1032',
                                                                       'out_octets': '66306366365',
                                                                       'out_ucast_pkts': '49152844',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1032',
                                                                       'in_octets': '66306366365',
                                                                       'in_ucast_pkts': '49152844',
                                                                       'links': [       {       'in_nucast_pkts': '1032',
                                                                                                'in_octets': '66306366365',
                                                                                                'in_ucast_pkts': '49152844',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '12185',
                                                                                                'out_octets': '7659325026',
                                                                                                'out_ucast_pkts': '25544182',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '12185',
                                                                       'out_octets': '7659325026',
                                                                       'out_ucast_pkts': '25544182',
                                                                       'poller_hash': 'QURUTjIzMTI0RTZBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '359118',
                                                                                                                        'fiber_length': 10462,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTZBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU2QQ==',
                                         'serial_number': 'ADTN23124E6A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '359118'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14481',
                                                                       'in_octets': '32821474532',
                                                                       'in_ucast_pkts': '51829091',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '185633626970',
                                                                       'out_ucast_pkts': '144377013',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '185633626970',
                                                                       'in_ucast_pkts': '144377013',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '185633626970',
                                                                                                'in_ucast_pkts': '144377013',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14481',
                                                                                                'out_octets': '32821474532',
                                                                                                'out_ucast_pkts': '51829091',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14481',
                                                                       'out_octets': '32821474532',
                                                                       'out_ucast_pkts': '51829091',
                                                                       'poller_hash': 'QURUTjIzMTI1MTYxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427032',
                                                                                                                        'fiber_length': 5086,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTYxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE2MQ==',
                                         'serial_number': 'ADTN23125161',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427032'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11091',
                                                                       'in_octets': '1957758834',
                                                                       'in_ucast_pkts': '11873930',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '46479498752',
                                                                       'out_ucast_pkts': '34686590',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '46479498752',
                                                                       'in_ucast_pkts': '34686590',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '46479498752',
                                                                                                'in_ucast_pkts': '34686590',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11091',
                                                                                                'out_octets': '1957758834',
                                                                                                'out_ucast_pkts': '11873930',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11091',
                                                                       'out_octets': '1957758834',
                                                                       'out_ucast_pkts': '11873930',
                                                                       'poller_hash': 'QURUTjIzMTI0RTg2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327029',
                                                                                                                        'fiber_length': 8737,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTg2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU4Ng==',
                                         'serial_number': 'ADTN23124E86',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77590',
                                                                       'in_octets': '21008714935',
                                                                       'in_ucast_pkts': '130418590',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6414',
                                                                       'out_octets': '494392636187',
                                                                       'out_ucast_pkts': '353071836',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6414',
                                                                       'in_octets': '494392636187',
                                                                       'in_ucast_pkts': '353071836',
                                                                       'links': [       {       'in_nucast_pkts': '6414',
                                                                                                'in_octets': '494392636187',
                                                                                                'in_ucast_pkts': '353071836',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77590',
                                                                                                'out_octets': '21008714935',
                                                                                                'out_ucast_pkts': '130418590',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77590',
                                                                       'out_octets': '21008714935',
                                                                       'out_ucast_pkts': '130418590',
                                                                       'poller_hash': 'QURUTjIzMTI0RTUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232581',
                                                                                                                        'fiber_length': 2008,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU1MA==',
                                         'serial_number': 'ADTN23124E50',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232581'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '28745',
                                                                       'in_octets': '27617544835',
                                                                       'in_ucast_pkts': '103034840',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124DA0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '959',
                                                                       'out_octets': '424807630451',
                                                                       'out_ucast_pkts': '301205551',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0REEwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '959',
                                                                       'in_octets': '424807630451',
                                                                       'in_ucast_pkts': '301205551',
                                                                       'links': [       {       'in_nucast_pkts': '959',
                                                                                                'in_octets': '424807630451',
                                                                                                'in_ucast_pkts': '301205551',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '28745',
                                                                                                'out_octets': '27617544835',
                                                                                                'out_ucast_pkts': '103034840',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '28745',
                                                                       'out_octets': '27617544835',
                                                                       'out_ucast_pkts': '103034840',
                                                                       'poller_hash': 'QURUTjIzMTI0REEwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '861783',
                                                                                                                        'fiber_length': 8008,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REEwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124DA0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERBMA==',
                                         'serial_number': 'ADTN23124DA0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '861783'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14489',
                                                                       'in_octets': '4871916230',
                                                                       'in_ucast_pkts': '19295162',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '53219380465',
                                                                       'out_ucast_pkts': '46179521',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VOF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '53219380465',
                                                                       'in_ucast_pkts': '46179521',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '53219380465',
                                                                                                'in_ucast_pkts': '46179521',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14489',
                                                                                                'out_octets': '4871916230',
                                                                                                'out_ucast_pkts': '19295162',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14489',
                                                                       'out_octets': '4871916230',
                                                                       'out_ucast_pkts': '19295162',
                                                                       'poller_hash': 'QURUTjIzMTI0MTI4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427040',
                                                                                                                        'fiber_length': 5794,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MTI4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDEyOA==',
                                         'serial_number': 'ADTN23124128',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427040'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '160098',
                                                                       'in_octets': '22879317435',
                                                                       'in_ucast_pkts': '105664544',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5304',
                                                                       'out_octets': '461411238133',
                                                                       'out_ucast_pkts': '341084502',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5304',
                                                                       'in_octets': '461411238133',
                                                                       'in_ucast_pkts': '341084502',
                                                                       'links': [       {       'in_nucast_pkts': '5304',
                                                                                                'in_octets': '461411238133',
                                                                                                'in_ucast_pkts': '341084502',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '160098',
                                                                                                'out_octets': '22879317435',
                                                                                                'out_ucast_pkts': '105664544',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '160098',
                                                                       'out_octets': '22879317435',
                                                                       'out_ucast_pkts': '105664544',
                                                                       'poller_hash': 'QURUTjIzMTI0NDVDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4773324',
                                                                                                                        'fiber_length': 10872,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NDVDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDQ1Qw==',
                                         'serial_number': 'ADTN2312445C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '4773324'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11367',
                                                                       'in_octets': '27663153',
                                                                       'in_ucast_pkts': '90402',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23124304_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '18021876',
                                                                       'out_ucast_pkts': '72331',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTI0MzA0X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '18021876',
                                                                       'in_ucast_pkts': '72331',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '18021876',
                                                                                                'in_ucast_pkts': '72331',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11367',
                                                                                                'out_octets': '27663153',
                                                                                                'out_ucast_pkts': '90402',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11367',
                                                                       'out_octets': '27663153',
                                                                       'out_ucast_pkts': '90402',
                                                                       'poller_hash': 'QURUTjIzMTI0MzA0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327029',
                                                                                                                        'fiber_length': 4895,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MzA0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23124304',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDMwNA==',
                                         'serial_number': 'ADTN23124304',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '25252',
                                                                       'in_octets': '20642883768',
                                                                       'in_ucast_pkts': '113167798',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '843',
                                                                       'out_octets': '530696181797',
                                                                       'out_ucast_pkts': '381264008',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '843',
                                                                       'in_octets': '530696181797',
                                                                       'in_ucast_pkts': '381264008',
                                                                       'links': [       {       'in_nucast_pkts': '843',
                                                                                                'in_octets': '530696181797',
                                                                                                'in_ucast_pkts': '381264008',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '25252',
                                                                                                'out_octets': '20642883768',
                                                                                                'out_ucast_pkts': '113167798',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '25252',
                                                                       'out_octets': '20642883768',
                                                                       'out_ucast_pkts': '113167798',
                                                                       'poller_hash': 'QURUTjIzMTI0NEJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '756858',
                                                                                                                        'fiber_length': 9328,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NEJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDRCQw==',
                                         'serial_number': 'ADTN231244BC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '756858'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14475',
                                                                       'in_octets': '5294014205',
                                                                       'in_ucast_pkts': '24438441',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '92267892089',
                                                                       'out_ucast_pkts': '68782014',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '92267892089',
                                                                       'in_ucast_pkts': '68782014',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '92267892089',
                                                                                                'in_ucast_pkts': '68782014',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14475',
                                                                                                'out_octets': '5294014205',
                                                                                                'out_ucast_pkts': '24438441',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14475',
                                                                       'out_octets': '5294014205',
                                                                       'out_ucast_pkts': '24438441',
                                                                       'poller_hash': 'QURUTjIzMTI1MTFEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427024',
                                                                                                                        'fiber_length': 540,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTFEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTExRA==',
                                         'serial_number': 'ADTN2312511D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427024'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '261',
                                                                       'in_octets': '5627074010',
                                                                       'in_ucast_pkts': '41991202',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '198998846990',
                                                                       'out_ucast_pkts': '135593488',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '198998846990',
                                                                       'in_ucast_pkts': '135593488',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '198998846990',
                                                                                                'in_ucast_pkts': '135593488',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '261',
                                                                                                'out_octets': '5627074010',
                                                                                                'out_ucast_pkts': '41991202',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '261',
                                                                       'out_octets': '5627074010',
                                                                       'out_ucast_pkts': '41991202',
                                                                       'poller_hash': 'QURUTjIzMTI0NDkxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427041',
                                                                                                                        'fiber_length': 6819,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NDkxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDQ5MQ==',
                                         'serial_number': 'ADTN23124491',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427041'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14486',
                                                                       'in_octets': '454089503',
                                                                       'in_ucast_pkts': '1098388',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '2165466570',
                                                                       'out_ucast_pkts': '1700019',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VOF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '2165466570',
                                                                       'in_ucast_pkts': '1700019',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '2165466570',
                                                                                                'in_ucast_pkts': '1700019',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14486',
                                                                                                'out_octets': '454089503',
                                                                                                'out_ucast_pkts': '1098388',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14486',
                                                                       'out_octets': '454089503',
                                                                       'out_ucast_pkts': '1098388',
                                                                       'poller_hash': 'QURUTjIzMTI1MTQzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427033',
                                                                                                                        'fiber_length': 3430,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTQzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0Mw==',
                                         'serial_number': 'ADTN23125143',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427033'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14482',
                                                                       'in_octets': '2081111609',
                                                                       'in_ucast_pkts': '9893508',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '35193394551',
                                                                       'out_ucast_pkts': '26825096',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '35193394551',
                                                                       'in_ucast_pkts': '26825096',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '35193394551',
                                                                                                'in_ucast_pkts': '26825096',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14482',
                                                                                                'out_octets': '2081111609',
                                                                                                'out_ucast_pkts': '9893508',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14482',
                                                                       'out_octets': '2081111609',
                                                                       'out_ucast_pkts': '9893508',
                                                                       'poller_hash': 'QURUTjIzMTI1MEZFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 4938,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEZFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBGRQ==',
                                         'serial_number': 'ADTN231250FE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '165871',
                                                                       'in_octets': '99808996995',
                                                                       'in_ucast_pkts': '610693991',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'ADTN23123FF5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13915',
                                                                       'out_octets': '954629699928',
                                                                       'out_ucast_pkts': '923394512',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgQURUTjIzMTIzRkY1X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '13915',
                                                                       'in_octets': '954629699928',
                                                                       'in_ucast_pkts': '923394512',
                                                                       'links': [       {       'in_nucast_pkts': '13915',
                                                                                                'in_octets': '954629699928',
                                                                                                'in_ucast_pkts': '923394512',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '165871',
                                                                                                'out_octets': '99808996995',
                                                                                                'out_ucast_pkts': '610693991',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '165871',
                                                                       'out_octets': '99808996995',
                                                                       'out_ucast_pkts': '610693991',
                                                                       'poller_hash': 'QURUTjIzMTIzRkY1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4851897',
                                                                                                                        'fiber_length': 5995,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkY1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'ADTN23123FF5',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZGNQ==',
                                         'serial_number': 'ADTN23123FF5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '4851897'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14486',
                                                                       'in_octets': '46466954692',
                                                                       'in_ucast_pkts': '162384018',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '629137532340',
                                                                       'out_ucast_pkts': '440714131',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VOV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '629137532340',
                                                                       'in_ucast_pkts': '440714131',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '629137532340',
                                                                                                'in_ucast_pkts': '440714131',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14486',
                                                                                                'out_octets': '46466954692',
                                                                                                'out_ucast_pkts': '162384018',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14486',
                                                                       'out_octets': '46466954692',
                                                                       'out_ucast_pkts': '162384018',
                                                                       'poller_hash': 'QURUTjIzMTI1MTAzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427036',
                                                                                                                        'fiber_length': 3301,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTAzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEwMw==',
                                         'serial_number': 'ADTN23125103',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427036'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14496',
                                                                       'in_octets': '17081454320',
                                                                       'in_ucast_pkts': '118761085',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1226',
                                                                       'out_octets': '325121783026',
                                                                       'out_ucast_pkts': '222443442',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1226',
                                                                       'in_octets': '325121783026',
                                                                       'in_ucast_pkts': '222443442',
                                                                       'links': [       {       'in_nucast_pkts': '1226',
                                                                                                'in_octets': '325121783026',
                                                                                                'in_ucast_pkts': '222443442',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14496',
                                                                                                'out_octets': '17081454320',
                                                                                                'out_ucast_pkts': '118761085',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14496',
                                                                       'out_octets': '17081454320',
                                                                       'out_ucast_pkts': '118761085',
                                                                       'poller_hash': 'QURUTjIzMTI1MTI5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 4696,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTI5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEyOQ==',
                                         'serial_number': 'ADTN23125129',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11095',
                                                                       'in_octets': '11364743835',
                                                                       'in_ucast_pkts': '80052462',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '384501951861',
                                                                       'out_ucast_pkts': '263215609',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '384501951861',
                                                                       'in_ucast_pkts': '263215609',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '384501951861',
                                                                                                'in_ucast_pkts': '263215609',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11095',
                                                                                                'out_octets': '11364743835',
                                                                                                'out_ucast_pkts': '80052462',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11095',
                                                                       'out_octets': '11364743835',
                                                                       'out_ucast_pkts': '80052462',
                                                                       'poller_hash': 'QURUTjIzMTI0MzJFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327030',
                                                                                                                        'fiber_length': 9068,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MzJFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDMyRQ==',
                                         'serial_number': 'ADTN2312432E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327030'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '29438',
                                                                       'in_octets': '14131521633',
                                                                       'in_ucast_pkts': '62966197',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2478',
                                                                       'out_octets': '288886403265',
                                                                       'out_ucast_pkts': '207388734',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTlfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2478',
                                                                       'in_octets': '288886403265',
                                                                       'in_ucast_pkts': '207388734',
                                                                       'links': [       {       'in_nucast_pkts': '2478',
                                                                                                'in_octets': '288886403265',
                                                                                                'in_ucast_pkts': '207388734',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '29438',
                                                                                                'out_octets': '14131521633',
                                                                                                'out_ucast_pkts': '62966197',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '29438',
                                                                       'out_octets': '14131521633',
                                                                       'out_ucast_pkts': '62966197',
                                                                       'poller_hash': 'QURUTjIzMTI1MEMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '861787',
                                                                                                                        'fiber_length': 8251,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDMA==',
                                         'serial_number': 'ADTN231250C0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '861787'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '29210',
                                                                       'in_octets': '13692612528',
                                                                       'in_ucast_pkts': '90428390',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2472',
                                                                       'out_octets': '327230078932',
                                                                       'out_ucast_pkts': '230788998',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTEwX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2472',
                                                                       'in_octets': '327230078932',
                                                                       'in_ucast_pkts': '230788998',
                                                                       'links': [       {       'in_nucast_pkts': '2472',
                                                                                                'in_octets': '327230078932',
                                                                                                'in_ucast_pkts': '230788998',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '29210',
                                                                                                'out_octets': '13692612528',
                                                                                                'out_ucast_pkts': '90428390',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '29210',
                                                                       'out_octets': '13692612528',
                                                                       'out_ucast_pkts': '90428390',
                                                                       'poller_hash': 'QURUTjIzMTI1MEJGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '861785',
                                                                                                                        'fiber_length': 8000,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEJGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBCRg==',
                                         'serial_number': 'ADTN231250BF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '861785'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14480',
                                                                       'in_octets': '5893619517',
                                                                       'in_ucast_pkts': '50997649',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '125013412365',
                                                                       'out_ucast_pkts': '93707406',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '125013412365',
                                                                       'in_ucast_pkts': '93707406',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '125013412365',
                                                                                                'in_ucast_pkts': '93707406',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14480',
                                                                                                'out_octets': '5893619517',
                                                                                                'out_ucast_pkts': '50997649',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14480',
                                                                       'out_octets': '5893619517',
                                                                       'out_ucast_pkts': '50997649',
                                                                       'poller_hash': 'QURUTjIzMTI0ODQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 9154,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0ODQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDg0OQ==',
                                         'serial_number': 'ADTN23124849',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77020',
                                                                       'in_octets': '42963108958',
                                                                       'in_ucast_pkts': '308691492',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6404',
                                                                       'out_octets': '1383576104760',
                                                                       'out_ucast_pkts': '949831788',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6404',
                                                                       'in_octets': '1383576104760',
                                                                       'in_ucast_pkts': '949831788',
                                                                       'links': [       {       'in_nucast_pkts': '6404',
                                                                                                'in_octets': '1383576104760',
                                                                                                'in_ucast_pkts': '949831788',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77020',
                                                                                                'out_octets': '42963108958',
                                                                                                'out_ucast_pkts': '308691492',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77020',
                                                                       'out_octets': '42963108958',
                                                                       'out_ucast_pkts': '308691492',
                                                                       'poller_hash': 'QURUTjIzMTIzRUJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232583',
                                                                                                                        'fiber_length': 2230,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRUJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0VCQw==',
                                         'serial_number': 'ADTN23123EBC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232583'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14564',
                                                                       'in_octets': '5936188625',
                                                                       'in_ucast_pkts': '39928554',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '161793611585',
                                                                       'out_ucast_pkts': '113854837',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTRfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '161793611585',
                                                                       'in_ucast_pkts': '113854837',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '161793611585',
                                                                                                'in_ucast_pkts': '113854837',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14564',
                                                                                                'out_octets': '5936188625',
                                                                                                'out_ucast_pkts': '39928554',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14564',
                                                                       'out_octets': '5936188625',
                                                                       'out_ucast_pkts': '39928554',
                                                                       'poller_hash': 'QURUTjIzMTI1MTE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427020',
                                                                                                                        'fiber_length': 1746,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTExNg==',
                                         'serial_number': 'ADTN23125116',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427020'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75667',
                                                                       'in_octets': '11102011713',
                                                                       'in_ucast_pkts': '70121762',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6403',
                                                                       'out_octets': '263864899151',
                                                                       'out_ucast_pkts': '186484050',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6403',
                                                                       'in_octets': '263864899151',
                                                                       'in_ucast_pkts': '186484050',
                                                                       'links': [       {       'in_nucast_pkts': '6403',
                                                                                                'in_octets': '263864899151',
                                                                                                'in_ucast_pkts': '186484050',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75667',
                                                                                                'out_octets': '11102011713',
                                                                                                'out_ucast_pkts': '70121762',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75667',
                                                                       'out_octets': '11102011713',
                                                                       'out_ucast_pkts': '70121762',
                                                                       'poller_hash': 'QURUTjIzMTIzRjE4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232589',
                                                                                                                        'fiber_length': 5797,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRjE4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0YxOA==',
                                         'serial_number': 'ADTN23123F18',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232589'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '76288',
                                                                       'in_octets': '36720060506',
                                                                       'in_ucast_pkts': '279046156',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '873140339405',
                                                                       'out_ucast_pkts': '608947037',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VNV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '873140339405',
                                                                       'in_ucast_pkts': '608947037',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '873140339405',
                                                                                                'in_ucast_pkts': '608947037',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '76288',
                                                                                                'out_octets': '36720060506',
                                                                                                'out_ucast_pkts': '279046156',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '76288',
                                                                       'out_octets': '36720060506',
                                                                       'out_ucast_pkts': '279046156',
                                                                       'poller_hash': 'QURUTjIzMTI1MTk1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232587',
                                                                                                                        'fiber_length': 6115,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTk1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE5NQ==',
                                         'serial_number': 'ADTN23125195',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232587'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14483',
                                                                       'in_octets': '5950362229',
                                                                       'in_ucast_pkts': '25236495',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '130148994270',
                                                                       'out_ucast_pkts': '93649767',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTExX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '130148994270',
                                                                       'in_ucast_pkts': '93649767',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '130148994270',
                                                                                                'in_ucast_pkts': '93649767',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14483',
                                                                                                'out_octets': '5950362229',
                                                                                                'out_ucast_pkts': '25236495',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14483',
                                                                       'out_octets': '5950362229',
                                                                       'out_ucast_pkts': '25236495',
                                                                       'poller_hash': 'QURUTjIzMTIzRUMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 4770,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRUMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0VDMA==',
                                         'serial_number': 'ADTN23123EC0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75669',
                                                                       'in_octets': '59160943607',
                                                                       'in_ucast_pkts': '354938436',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6407',
                                                                       'out_octets': '1144840804202',
                                                                       'out_ucast_pkts': '838247141',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6407',
                                                                       'in_octets': '1144840804202',
                                                                       'in_ucast_pkts': '838247141',
                                                                       'links': [       {       'in_nucast_pkts': '6407',
                                                                                                'in_octets': '1144840804202',
                                                                                                'in_ucast_pkts': '838247141',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75669',
                                                                                                'out_octets': '59160943607',
                                                                                                'out_ucast_pkts': '354938436',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75669',
                                                                       'out_octets': '59160943607',
                                                                       'out_ucast_pkts': '354938436',
                                                                       'poller_hash': 'QURUTjIzMTI0NEVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232588',
                                                                                                                        'fiber_length': 8586,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NEVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDRFQg==',
                                         'serial_number': 'ADTN231244EB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232588'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '29209',
                                                                       'in_octets': '8844157372',
                                                                       'in_ucast_pkts': '21227697',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2472',
                                                                       'out_octets': '45819930297',
                                                                       'out_ucast_pkts': '43332910',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTEyX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2472',
                                                                       'in_octets': '45819930297',
                                                                       'in_ucast_pkts': '43332910',
                                                                       'links': [       {       'in_nucast_pkts': '2472',
                                                                                                'in_octets': '45819930297',
                                                                                                'in_ucast_pkts': '43332910',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '29209',
                                                                                                'out_octets': '8844157372',
                                                                                                'out_ucast_pkts': '21227697',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '29209',
                                                                       'out_octets': '8844157372',
                                                                       'out_ucast_pkts': '21227697',
                                                                       'poller_hash': 'QURUTjIzMTIzRkVFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '861785',
                                                                                                                        'fiber_length': 8343,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkVFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZFRQ==',
                                         'serial_number': 'ADTN23123FEE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '861785'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '302422',
                                                                       'in_octets': '93074964',
                                                                       'in_ucast_pkts': '0',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '302422',
                                                                                                'out_octets': '93074964',
                                                                                                'out_ucast_pkts': '0',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '302422',
                                                                       'out_octets': '93074964',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'QURUTjIzMTI1MEM2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327029',
                                                                                                                        'fiber_length': 9038,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEM2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDNg==',
                                         'serial_number': 'ADTN231250C6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '178545',
                                                                       'in_octets': '153882517082',
                                                                       'in_ucast_pkts': '446613554',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5952',
                                                                       'out_octets': '1172709230629',
                                                                       'out_ucast_pkts': '920785112',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VOV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5952',
                                                                       'in_octets': '1172709230629',
                                                                       'in_ucast_pkts': '920785112',
                                                                       'links': [       {       'in_nucast_pkts': '5952',
                                                                                                'in_octets': '1172709230629',
                                                                                                'in_ucast_pkts': '920785112',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '178545',
                                                                                                'out_octets': '153882517082',
                                                                                                'out_ucast_pkts': '446613554',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '178545',
                                                                       'out_octets': '153882517082',
                                                                       'out_ucast_pkts': '446613554',
                                                                       'poller_hash': 'QURUTjIzMTI1MTNBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355629',
                                                                                                                        'fiber_length': 9589,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTNBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEzQQ==',
                                         'serial_number': 'ADTN2312513A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355629'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '183654',
                                                                       'in_octets': '11693088997',
                                                                       'in_ucast_pkts': '32136695',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15381',
                                                                       'out_octets': '44985435977',
                                                                       'out_ucast_pkts': '40222103',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15381',
                                                                       'in_octets': '44985435977',
                                                                       'in_ucast_pkts': '40222103',
                                                                       'links': [       {       'in_nucast_pkts': '15381',
                                                                                                'in_octets': '44985435977',
                                                                                                'in_ucast_pkts': '40222103',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '183654',
                                                                                                'out_octets': '11693088997',
                                                                                                'out_ucast_pkts': '32136695',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '183654',
                                                                       'out_octets': '11693088997',
                                                                       'out_ucast_pkts': '32136695',
                                                                       'poller_hash': 'QURUTjIzMTI0MUMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355630',
                                                                                                                        'fiber_length': 10748,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MUMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDFDMA==',
                                         'serial_number': 'ADTN231241C0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355630'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75200',
                                                                       'in_octets': '62366534715',
                                                                       'in_ucast_pkts': '213576772',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '615271594261',
                                                                       'out_ucast_pkts': '452854306',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '615271594261',
                                                                       'in_ucast_pkts': '452854306',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '615271594261',
                                                                                                'in_ucast_pkts': '452854306',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75200',
                                                                                                'out_octets': '62366534715',
                                                                                                'out_ucast_pkts': '213576772',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75200',
                                                                       'out_octets': '62366534715',
                                                                       'out_ucast_pkts': '213576772',
                                                                       'poller_hash': 'QURUTjIzMTI1MTJGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232581',
                                                                                                                        'fiber_length': 1857,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTJGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEyRg==',
                                         'serial_number': 'ADTN2312512F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232581'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77528',
                                                                       'in_octets': '121194056410',
                                                                       'in_ucast_pkts': '294951548',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6409',
                                                                       'out_octets': '999471470264',
                                                                       'out_ucast_pkts': '796610270',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6409',
                                                                       'in_octets': '999471470264',
                                                                       'in_ucast_pkts': '796610270',
                                                                       'links': [       {       'in_nucast_pkts': '6409',
                                                                                                'in_octets': '999471470264',
                                                                                                'in_ucast_pkts': '796610270',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77528',
                                                                                                'out_octets': '121194056410',
                                                                                                'out_ucast_pkts': '294951548',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77528',
                                                                       'out_octets': '121194056410',
                                                                       'out_ucast_pkts': '294951548',
                                                                       'poller_hash': 'QURUTjIzMTI1MENDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232589',
                                                                                                                        'fiber_length': 6181,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MENDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDQw==',
                                         'serial_number': 'ADTN231250CC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232589'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '114326',
                                                                       'in_octets': '116923921474',
                                                                       'in_ucast_pkts': '644934805',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9678',
                                                                       'out_octets': '2824710093476',
                                                                       'out_ucast_pkts': '2018653090',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9678',
                                                                       'in_octets': '2824710093476',
                                                                       'in_ucast_pkts': '2018653090',
                                                                       'links': [       {       'in_nucast_pkts': '9678',
                                                                                                'in_octets': '2824710093476',
                                                                                                'in_ucast_pkts': '2018653090',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '114326',
                                                                                                'out_octets': '116923921474',
                                                                                                'out_ucast_pkts': '644934805',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '114326',
                                                                       'out_octets': '116923921474',
                                                                       'out_ucast_pkts': '644934805',
                                                                       'poller_hash': 'QURUTjIzMTI1MTU4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3371909',
                                                                                                                        'fiber_length': 10811,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTU4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE1OA==',
                                         'serial_number': 'ADTN23125158',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3371909'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '112182',
                                                                       'in_octets': '65293876011',
                                                                       'in_ucast_pkts': '244388608',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3741',
                                                                       'out_octets': '1379640415195',
                                                                       'out_ucast_pkts': '1013050052',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTEzX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3741',
                                                                       'in_octets': '1379640415195',
                                                                       'in_ucast_pkts': '1013050052',
                                                                       'links': [       {       'in_nucast_pkts': '3741',
                                                                                                'in_octets': '1379640415195',
                                                                                                'in_ucast_pkts': '1013050052',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '112182',
                                                                                                'out_octets': '65293876011',
                                                                                                'out_ucast_pkts': '244388608',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '112182',
                                                                       'out_octets': '65293876011',
                                                                       'out_ucast_pkts': '244388608',
                                                                       'poller_hash': 'QURUTjIzMTIzRkU5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3364579',
                                                                                                                        'fiber_length': 8068,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkU5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZFOQ==',
                                         'serial_number': 'ADTN23123FE9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3364579'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-3.1.17.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x3-ONU17_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTdfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0MDk3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MDk3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x3-ONU17',
                                         'offline_reason': 'OFFLINE_REASON_POWER_OFF',
                                         'offline_reason_detail': 'A local power '
                                                                  'outage has occurred '
                                                                  'for ONU "onu '
                                                                  '3.1.17"(ADTN23124097).',
                                         'offline_time': '2025-07-31T22:19:54Z',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDA5Nw==',
                                         'serial_number': 'ADTN23124097',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '4038463',
                                                                       'in_octets': '1185329785',
                                                                       'in_ucast_pkts': '1052807',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '4038463',
                                                                                                'out_octets': '1185329785',
                                                                                                'out_ucast_pkts': '1052807',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4038463',
                                                                       'out_octets': '1185329785',
                                                                       'out_ucast_pkts': '1052807',
                                                                       'poller_hash': 'QURUTjIzMTI1MTExIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232588',
                                                                                                                        'fiber_length': 6734,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTExIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTExMQ==',
                                         'serial_number': 'ADTN23125111',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232588'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14494',
                                                                       'in_octets': '6049417954',
                                                                       'in_ucast_pkts': '34033927',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1226',
                                                                       'out_octets': '211119778048',
                                                                       'out_ucast_pkts': '156321241',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1226',
                                                                       'in_octets': '211119778048',
                                                                       'in_ucast_pkts': '156321241',
                                                                       'links': [       {       'in_nucast_pkts': '1226',
                                                                                                'in_octets': '211119778048',
                                                                                                'in_ucast_pkts': '156321241',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14494',
                                                                                                'out_octets': '6049417954',
                                                                                                'out_ucast_pkts': '34033927',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14494',
                                                                       'out_octets': '6049417954',
                                                                       'out_ucast_pkts': '34033927',
                                                                       'poller_hash': 'QURUTjIzMTI1MTJFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427036',
                                                                                                                        'fiber_length': 3383,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTJFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEyRQ==',
                                         'serial_number': 'ADTN2312512E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427036'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '76202',
                                                                       'in_octets': '25252545824',
                                                                       'in_ucast_pkts': '81251484',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6386',
                                                                       'out_octets': '401436456616',
                                                                       'out_ucast_pkts': '292023857',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6386',
                                                                       'in_octets': '401436456616',
                                                                       'in_ucast_pkts': '292023857',
                                                                       'links': [       {       'in_nucast_pkts': '6386',
                                                                                                'in_octets': '401436456616',
                                                                                                'in_ucast_pkts': '292023857',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '76202',
                                                                                                'out_octets': '25252545824',
                                                                                                'out_ucast_pkts': '81251484',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '76202',
                                                                       'out_octets': '25252545824',
                                                                       'out_ucast_pkts': '81251484',
                                                                       'poller_hash': 'QURUTjIzMTI1MTI2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232584',
                                                                                                                        'fiber_length': 4079,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTI2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEyNg==',
                                         'serial_number': 'ADTN23125126',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232584'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14252',
                                                                       'in_octets': '4570785449',
                                                                       'in_ucast_pkts': '6288909',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '9000467123',
                                                                       'out_ucast_pkts': '8758129',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTIxX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '9000467123',
                                                                       'in_ucast_pkts': '8758129',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '9000467123',
                                                                                                'in_ucast_pkts': '8758129',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14252',
                                                                                                'out_octets': '4570785449',
                                                                                                'out_ucast_pkts': '6288909',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14252',
                                                                       'out_octets': '4570785449',
                                                                       'out_ucast_pkts': '6288909',
                                                                       'poller_hash': 'QURUTjIzMTI1MTRDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 5052,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTRDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0Qw==',
                                         'serial_number': 'ADTN2312514C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-7.1.11.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x7-ONU11_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTFfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI1MTBDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTBDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x7-ONU11',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEwQw==',
                                         'serial_number': 'ADTN2312510C',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '178546',
                                                                       'in_octets': '116680277023',
                                                                       'in_ucast_pkts': '778141606',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5952',
                                                                       'out_octets': '2677020422440',
                                                                       'out_ucast_pkts': '1895563390',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5952',
                                                                       'in_octets': '2677020422440',
                                                                       'in_ucast_pkts': '1895563390',
                                                                       'links': [       {       'in_nucast_pkts': '5952',
                                                                                                'in_octets': '2677020422440',
                                                                                                'in_ucast_pkts': '1895563390',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '178546',
                                                                                                'out_octets': '116680277023',
                                                                                                'out_ucast_pkts': '778141606',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '178546',
                                                                       'out_octets': '116680277023',
                                                                       'out_ucast_pkts': '778141606',
                                                                       'poller_hash': 'QURUTjIzMTI1MjA2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355625',
                                                                                                                        'fiber_length': 7635,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MjA2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTIwNg==',
                                         'serial_number': 'ADTN23125206',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355625'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '178527',
                                                                       'in_octets': '19649965190',
                                                                       'in_ucast_pkts': '37384091',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5952',
                                                                       'out_octets': '72406994171',
                                                                       'out_ucast_pkts': '64572035',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5952',
                                                                       'in_octets': '72406994171',
                                                                       'in_ucast_pkts': '64572035',
                                                                       'links': [       {       'in_nucast_pkts': '5952',
                                                                                                'in_octets': '72406994171',
                                                                                                'in_ucast_pkts': '64572035',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '178527',
                                                                                                'out_octets': '19649965190',
                                                                                                'out_ucast_pkts': '37384091',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '178527',
                                                                       'out_octets': '19649965190',
                                                                       'out_ucast_pkts': '37384091',
                                                                       'poller_hash': 'QURUTjIzMTI1MEREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355597',
                                                                                                                        'fiber_length': 8976,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBERA==',
                                         'serial_number': 'ADTN231250DD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355597'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181960',
                                                                       'in_octets': '118797021115',
                                                                       'in_ucast_pkts': '220053985',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15383',
                                                                       'out_octets': '700092925660',
                                                                       'out_ucast_pkts': '526170506',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15383',
                                                                       'in_octets': '700092925660',
                                                                       'in_ucast_pkts': '526170506',
                                                                       'links': [       {       'in_nucast_pkts': '15383',
                                                                                                'in_octets': '700092925660',
                                                                                                'in_ucast_pkts': '526170506',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181960',
                                                                                                'out_octets': '118797021115',
                                                                                                'out_ucast_pkts': '220053985',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181960',
                                                                       'out_octets': '118797021115',
                                                                       'out_ucast_pkts': '220053985',
                                                                       'poller_hash': 'QURUTjIzMTI1MEJCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355597',
                                                                                                                        'fiber_length': 8993,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEJCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBCQg==',
                                         'serial_number': 'ADTN231250BB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355597'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '114024',
                                                                       'in_octets': '171519733498',
                                                                       'in_ucast_pkts': '490137850',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9650',
                                                                       'out_octets': '1102102393438',
                                                                       'out_ucast_pkts': '920098124',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9650',
                                                                       'in_octets': '1102102393438',
                                                                       'in_ucast_pkts': '920098124',
                                                                       'links': [       {       'in_nucast_pkts': '9650',
                                                                                                'in_octets': '1102102393438',
                                                                                                'in_ucast_pkts': '920098124',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '114024',
                                                                                                'out_octets': '171519733498',
                                                                                                'out_ucast_pkts': '490137850',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '114024',
                                                                       'out_octets': '171519733498',
                                                                       'out_ucast_pkts': '490137850',
                                                                       'poller_hash': 'QURUTjIzMTI1MTBCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3364587',
                                                                                                                        'fiber_length': 6776,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTBCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEwQg==',
                                         'serial_number': 'ADTN2312510B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3364587'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10921',
                                                                       'in_octets': '24621439854',
                                                                       'in_ucast_pkts': '47282680',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '279360494658',
                                                                       'out_ucast_pkts': '197894118',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTEwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '279360494658',
                                                                       'in_ucast_pkts': '197894118',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '279360494658',
                                                                                                'in_ucast_pkts': '197894118',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10921',
                                                                                                'out_octets': '24621439854',
                                                                                                'out_ucast_pkts': '47282680',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10921',
                                                                       'out_octets': '24621439854',
                                                                       'out_ucast_pkts': '47282680',
                                                                       'poller_hash': 'QURUTjIzMTI1MTk0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327022',
                                                                                                                        'fiber_length': 9481,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTk0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE5NA==',
                                         'serial_number': 'ADTN23125194',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327022'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-7.1.12.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x7-ONU12_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTJfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI1MTQ4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x7-ONU12',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0OA==',
                                         'serial_number': 'ADTN23125148',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11355',
                                                                       'in_octets': '2030971762',
                                                                       'in_ucast_pkts': '8552994',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '39794992093',
                                                                       'out_ucast_pkts': '27978558',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTExX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '39794992093',
                                                                       'in_ucast_pkts': '27978558',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '39794992093',
                                                                                                'in_ucast_pkts': '27978558',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11355',
                                                                                                'out_octets': '2030971762',
                                                                                                'out_ucast_pkts': '8552994',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11355',
                                                                       'out_octets': '2030971762',
                                                                       'out_ucast_pkts': '8552994',
                                                                       'poller_hash': 'QURUTjIzMTI1MTRFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327028',
                                                                                                                        'fiber_length': 8614,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTRFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0RQ==',
                                         'serial_number': 'ADTN2312514E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74440',
                                                                       'in_octets': '25203479449',
                                                                       'in_ucast_pkts': '190860845',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '522456534596',
                                                                       'out_ucast_pkts': '377160431',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VOV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '522456534596',
                                                                       'in_ucast_pkts': '377160431',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '522456534596',
                                                                                                'in_ucast_pkts': '377160431',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74440',
                                                                                                'out_octets': '25203479449',
                                                                                                'out_ucast_pkts': '190860845',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74440',
                                                                       'out_octets': '25203479449',
                                                                       'out_ucast_pkts': '190860845',
                                                                       'poller_hash': 'QURUTjIzMTI1MTREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232588',
                                                                                                                        'fiber_length': 5523,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0RA==',
                                         'serial_number': 'ADTN2312514D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232588'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74441',
                                                                       'in_octets': '19241109251',
                                                                       'in_ucast_pkts': '133289436',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '394019870049',
                                                                       'out_ucast_pkts': '272349105',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '394019870049',
                                                                       'in_ucast_pkts': '272349105',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '394019870049',
                                                                                                'in_ucast_pkts': '272349105',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74441',
                                                                                                'out_octets': '19241109251',
                                                                                                'out_ucast_pkts': '133289436',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74441',
                                                                       'out_octets': '19241109251',
                                                                       'out_ucast_pkts': '133289436',
                                                                       'poller_hash': 'QURUTjIzMTI1MENEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232584',
                                                                                                                        'fiber_length': 5847,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MENEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDRA==',
                                         'serial_number': 'ADTN231250CD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232584'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74436',
                                                                       'in_octets': '27238749941',
                                                                       'in_ucast_pkts': '145779371',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2477',
                                                                       'out_octets': '663778846525',
                                                                       'out_ucast_pkts': '469434828',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2477',
                                                                       'in_octets': '663778846525',
                                                                       'in_ucast_pkts': '469434828',
                                                                       'links': [       {       'in_nucast_pkts': '2477',
                                                                                                'in_octets': '663778846525',
                                                                                                'in_ucast_pkts': '469434828',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74436',
                                                                                                'out_octets': '27238749941',
                                                                                                'out_ucast_pkts': '145779371',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74436',
                                                                       'out_octets': '27238749941',
                                                                       'out_ucast_pkts': '145779371',
                                                                       'poller_hash': 'QURUTjIzMTI1MTUzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232579',
                                                                                                                        'fiber_length': 3719,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTUzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE1Mw==',
                                         'serial_number': 'ADTN23125153',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232579'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14483',
                                                                       'in_octets': '4288421826',
                                                                       'in_ucast_pkts': '45952539',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '140797273463',
                                                                       'out_ucast_pkts': '94877340',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTIyX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '140797273463',
                                                                       'in_ucast_pkts': '94877340',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '140797273463',
                                                                                                'in_ucast_pkts': '94877340',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14483',
                                                                                                'out_octets': '4288421826',
                                                                                                'out_ucast_pkts': '45952539',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14483',
                                                                       'out_octets': '4288421826',
                                                                       'out_ucast_pkts': '45952539',
                                                                       'poller_hash': 'QURUTjIzMTIzRkM0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427031',
                                                                                                                        'fiber_length': 3515,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkM0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU22',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZDNA==',
                                         'serial_number': 'ADTN23123FC4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427031'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14481',
                                                                       'in_octets': '27007301589',
                                                                       'in_ucast_pkts': '46146504',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x14-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '114919981146',
                                                                       'out_ucast_pkts': '87784650',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDE0LU9OVTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '114919981146',
                                                                       'in_ucast_pkts': '87784650',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '114919981146',
                                                                                                'in_ucast_pkts': '87784650',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14481',
                                                                                                'out_octets': '27007301589',
                                                                                                'out_ucast_pkts': '46146504',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14481',
                                                                       'out_octets': '27007301589',
                                                                       'out_ucast_pkts': '46146504',
                                                                       'poller_hash': 'QURUTjIzMTI1MTUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 1195,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x14-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE1MA==',
                                         'serial_number': 'ADTN23125150',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5-5-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75662',
                                                                       'in_octets': '24561795550',
                                                                       'in_ucast_pkts': '147022405',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6382',
                                                                       'out_octets': '527052006636',
                                                                       'out_ucast_pkts': '366905481',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6382',
                                                                       'in_octets': '527052006636',
                                                                       'in_ucast_pkts': '366905481',
                                                                       'links': [       {       'in_nucast_pkts': '6382',
                                                                                                'in_octets': '527052006636',
                                                                                                'in_ucast_pkts': '366905481',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75662',
                                                                                                'out_octets': '24561795550',
                                                                                                'out_ucast_pkts': '147022405',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75662',
                                                                       'out_octets': '24561795550',
                                                                       'out_ucast_pkts': '147022405',
                                                                       'poller_hash': 'QURUTjIzMTI1MUFBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232583',
                                                                                                                        'fiber_length': 4022,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MUFBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTFBQQ==',
                                         'serial_number': 'ADTN231251AA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232583'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '40298',
                                                                       'in_octets': '4243954573',
                                                                       'in_ucast_pkts': '12895145',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1324',
                                                                       'out_octets': '37437659973',
                                                                       'out_ucast_pkts': '27245498',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1324',
                                                                       'in_octets': '37437659973',
                                                                       'in_ucast_pkts': '27245498',
                                                                       'links': [       {       'in_nucast_pkts': '1324',
                                                                                                'in_octets': '37437659973',
                                                                                                'in_ucast_pkts': '27245498',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '40298',
                                                                                                'out_octets': '4243954573',
                                                                                                'out_ucast_pkts': '12895145',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '40298',
                                                                       'out_octets': '4243954573',
                                                                       'out_ucast_pkts': '12895145',
                                                                       'poller_hash': 'QURUTjIzMTI1MTRGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1208680',
                                                                                                                        'fiber_length': 3775,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTRGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0Rg==',
                                         'serial_number': 'ADTN2312514F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1208680'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74438',
                                                                       'in_octets': '18974555818',
                                                                       'in_ucast_pkts': '69047685',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2478',
                                                                       'out_octets': '268453639730',
                                                                       'out_ucast_pkts': '197348696',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2478',
                                                                       'in_octets': '268453639730',
                                                                       'in_ucast_pkts': '197348696',
                                                                       'links': [       {       'in_nucast_pkts': '2478',
                                                                                                'in_octets': '268453639730',
                                                                                                'in_ucast_pkts': '197348696',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74438',
                                                                                                'out_octets': '18974555818',
                                                                                                'out_ucast_pkts': '69047685',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74438',
                                                                       'out_octets': '18974555818',
                                                                       'out_ucast_pkts': '69047685',
                                                                       'poller_hash': 'QURUTjIzMTI0MUJGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232586',
                                                                                                                        'fiber_length': 4059,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MUJGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU22',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDFCRg==',
                                         'serial_number': 'ADTN231241BF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232586'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '259',
                                                                       'in_octets': '5241450276',
                                                                       'in_ucast_pkts': '29662030',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '248025810009',
                                                                       'out_ucast_pkts': '177604089',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '248025810009',
                                                                       'in_ucast_pkts': '177604089',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '248025810009',
                                                                                                'in_ucast_pkts': '177604089',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '259',
                                                                                                'out_octets': '5241450276',
                                                                                                'out_ucast_pkts': '29662030',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '259',
                                                                       'out_octets': '5241450276',
                                                                       'out_ucast_pkts': '29662030',
                                                                       'poller_hash': 'QURUTjIzMTIzRkRDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427037',
                                                                                                                        'fiber_length': 4481,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRkRDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0ZEQw==',
                                         'serial_number': 'ADTN23123FDC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427037'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '182279',
                                                                       'in_octets': '113227085394',
                                                                       'in_ucast_pkts': '281784683',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15383',
                                                                       'out_octets': '1188113516971',
                                                                       'out_ucast_pkts': '929359604',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15383',
                                                                       'in_octets': '1188113516971',
                                                                       'in_ucast_pkts': '929359604',
                                                                       'links': [       {       'in_nucast_pkts': '15383',
                                                                                                'in_octets': '1188113516971',
                                                                                                'in_ucast_pkts': '929359604',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '182279',
                                                                                                'out_octets': '113227085394',
                                                                                                'out_ucast_pkts': '281784683',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '182279',
                                                                       'out_octets': '113227085394',
                                                                       'out_ucast_pkts': '281784683',
                                                                       'poller_hash': 'QURUTjIzMTI0RTQ1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355625',
                                                                                                                        'fiber_length': 13527,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTQ1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU0NQ==',
                                         'serial_number': 'ADTN23124E45',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355625'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75670',
                                                                       'in_octets': '22487044850',
                                                                       'in_ucast_pkts': '116863573',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6405',
                                                                       'out_octets': '568621225123',
                                                                       'out_ucast_pkts': '402931634',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6405',
                                                                       'in_octets': '568621225123',
                                                                       'in_ucast_pkts': '402931634',
                                                                       'links': [       {       'in_nucast_pkts': '6405',
                                                                                                'in_octets': '568621225123',
                                                                                                'in_ucast_pkts': '402931634',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75670',
                                                                                                'out_octets': '22487044850',
                                                                                                'out_ucast_pkts': '116863573',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75670',
                                                                       'out_octets': '22487044850',
                                                                       'out_ucast_pkts': '116863573',
                                                                       'poller_hash': 'QURUTjIzMTI1MTRCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232587',
                                                                                                                        'fiber_length': 6444,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTRCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0Qg==',
                                         'serial_number': 'ADTN2312514B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232587'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14484',
                                                                       'in_octets': '3321663322',
                                                                       'in_ucast_pkts': '3122814',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '3289045895',
                                                                       'out_ucast_pkts': '3279753',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTIzX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '3289045895',
                                                                       'in_ucast_pkts': '3279753',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '3289045895',
                                                                                                'in_ucast_pkts': '3279753',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14484',
                                                                                                'out_octets': '3321663322',
                                                                                                'out_ucast_pkts': '3122814',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14484',
                                                                       'out_octets': '3321663322',
                                                                       'out_ucast_pkts': '3122814',
                                                                       'poller_hash': 'QURUTjIzMTI1MEMzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427034',
                                                                                                                        'fiber_length': 3450,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEMzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDMw==',
                                         'serial_number': 'ADTN231250C3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427034'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '131312',
                                                                       'in_octets': '40046711094',
                                                                       'in_ucast_pkts': '80198771',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10973',
                                                                       'out_octets': '343142535187',
                                                                       'out_ucast_pkts': '256076050',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '10973',
                                                                       'in_octets': '343142535187',
                                                                       'in_ucast_pkts': '256076050',
                                                                       'links': [       {       'in_nucast_pkts': '10973',
                                                                                                'in_octets': '343142535187',
                                                                                                'in_ucast_pkts': '256076050',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '131312',
                                                                                                'out_octets': '40046711094',
                                                                                                'out_ucast_pkts': '80198771',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '131312',
                                                                       'out_octets': '40046711094',
                                                                       'out_ucast_pkts': '80198771',
                                                                       'poller_hash': 'QURUTjIzMTI1MTQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3826297',
                                                                                                                        'fiber_length': 13074,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0OQ==',
                                         'serial_number': 'ADTN23125149',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3826297'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14604',
                                                                       'in_octets': '7802515320',
                                                                       'in_ucast_pkts': '26916176',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '154402388649',
                                                                       'out_ucast_pkts': '116621269',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '154402388649',
                                                                       'in_ucast_pkts': '116621269',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '154402388649',
                                                                                                'in_ucast_pkts': '116621269',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14604',
                                                                                                'out_octets': '7802515320',
                                                                                                'out_ucast_pkts': '26916176',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14604',
                                                                       'out_octets': '7802515320',
                                                                       'out_ucast_pkts': '26916176',
                                                                       'poller_hash': 'QURUTjIzMTI1MEM4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427030',
                                                                                                                        'fiber_length': 1996,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEM4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDOA==',
                                         'serial_number': 'ADTN231250C8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427030'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '111982',
                                                                       'in_octets': '55559806667',
                                                                       'in_ucast_pkts': '326687744',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3734',
                                                                       'out_octets': '1471543734583',
                                                                       'out_ucast_pkts': '1139963640',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE0X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3734',
                                                                       'in_octets': '1471543734583',
                                                                       'in_ucast_pkts': '1139963640',
                                                                       'links': [       {       'in_nucast_pkts': '3734',
                                                                                                'in_octets': '1471543734583',
                                                                                                'in_ucast_pkts': '1139963640',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '111982',
                                                                                                'out_octets': '55559806667',
                                                                                                'out_ucast_pkts': '326687744',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '111982',
                                                                       'out_octets': '55559806667',
                                                                       'out_ucast_pkts': '326687744',
                                                                       'poller_hash': 'QURUTjIzMTI1MEQxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3364578',
                                                                                                                        'fiber_length': 8127,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MEQxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBEMQ==',
                                         'serial_number': 'ADTN231250D1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3364578'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '179971',
                                                                       'in_octets': '121539742062',
                                                                       'in_ucast_pkts': '283895598',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5954',
                                                                       'out_octets': '946344226230',
                                                                       'out_ucast_pkts': '722438556',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI0X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5954',
                                                                       'in_octets': '946344226230',
                                                                       'in_ucast_pkts': '722438556',
                                                                       'links': [       {       'in_nucast_pkts': '5954',
                                                                                                'in_octets': '946344226230',
                                                                                                'in_ucast_pkts': '722438556',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '179971',
                                                                                                'out_octets': '121539742062',
                                                                                                'out_ucast_pkts': '283895598',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '179971',
                                                                       'out_octets': '121539742062',
                                                                       'out_ucast_pkts': '283895598',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0E1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355590',
                                                                                                                        'fiber_length': 4061,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0E1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdBNQ==',
                                         'serial_number': 'ADTN231447A5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355590'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-2.1.2.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x2-ONU2_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMl9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0OEZDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEZDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x2-ONU2',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhGQw==',
                                         'serial_number': 'ADTN231448FC',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '61665',
                                                                       'in_octets': '95356553219',
                                                                       'in_ucast_pkts': '401599977',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2006',
                                                                       'out_octets': '1226720307132',
                                                                       'out_ucast_pkts': '888283807',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VNF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2006',
                                                                       'in_octets': '1226720307132',
                                                                       'in_ucast_pkts': '888283807',
                                                                       'links': [       {       'in_nucast_pkts': '2006',
                                                                                                'in_octets': '1226720307132',
                                                                                                'in_ucast_pkts': '888283807',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '61665',
                                                                                                'out_octets': '95356553219',
                                                                                                'out_ucast_pkts': '401599977',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '61665',
                                                                       'out_octets': '95356553219',
                                                                       'out_ucast_pkts': '401599977',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0Y5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1804563',
                                                                                                                        'fiber_length': 14757,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0Y5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdGOQ==',
                                         'serial_number': 'ADTN231447F9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1804563'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181509',
                                                                       'in_octets': '23060195890',
                                                                       'in_ucast_pkts': '143995367',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15377',
                                                                       'out_octets': '705058396714',
                                                                       'out_ucast_pkts': '490329991',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15377',
                                                                       'in_octets': '705058396714',
                                                                       'in_ucast_pkts': '490329991',
                                                                       'links': [       {       'in_nucast_pkts': '15377',
                                                                                                'in_octets': '705058396714',
                                                                                                'in_ucast_pkts': '490329991',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181509',
                                                                                                'out_octets': '23060195890',
                                                                                                'out_ucast_pkts': '143995367',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181509',
                                                                       'out_octets': '23060195890',
                                                                       'out_ucast_pkts': '143995367',
                                                                       'poller_hash': 'QURUTjIzMTQ0NjgxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355592',
                                                                                                                        'fiber_length': 14954,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NjgxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDY4MQ==',
                                         'serial_number': 'ADTN23144681',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355592'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-11.1.25.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x11-ONU25_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI1X0VUSF8x'},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0NjY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NjY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x11-ONU25',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDY2Ng==',
                                         'serial_number': 'ADTN23144666',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74456',
                                                                       'in_octets': '19525953906',
                                                                       'in_ucast_pkts': '52938837',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '165640553610',
                                                                       'out_ucast_pkts': '129636471',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '165640553610',
                                                                       'in_ucast_pkts': '129636471',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '165640553610',
                                                                                                'in_ucast_pkts': '129636471',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74456',
                                                                                                'out_octets': '19525953906',
                                                                                                'out_ucast_pkts': '52938837',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74456',
                                                                       'out_octets': '19525953906',
                                                                       'out_ucast_pkts': '52938837',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkEzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232580',
                                                                                                                        'fiber_length': 6030,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkEzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZBMw==',
                                         'serial_number': 'ADTN231446A3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232580'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74451',
                                                                       'in_octets': '136938371849',
                                                                       'in_ucast_pkts': '615456612',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '2040535594782',
                                                                       'out_ucast_pkts': '1469568415',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '2040535594782',
                                                                       'in_ucast_pkts': '1469568415',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '2040535594782',
                                                                                                'in_ucast_pkts': '1469568415',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74451',
                                                                                                'out_octets': '136938371849',
                                                                                                'out_ucast_pkts': '615456612',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74451',
                                                                       'out_octets': '136938371849',
                                                                       'out_ucast_pkts': '615456612',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkI0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232586',
                                                                                                                        'fiber_length': 6378,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkI0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZCNA==',
                                         'serial_number': 'ADTN231446B4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232586'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '23643',
                                                                       'in_octets': '15570400974',
                                                                       'in_ucast_pkts': '93138719',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2000',
                                                                       'out_octets': '371021531560',
                                                                       'out_ucast_pkts': '269070296',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2000',
                                                                       'in_octets': '371021531560',
                                                                       'in_ucast_pkts': '269070296',
                                                                       'links': [       {       'in_nucast_pkts': '2000',
                                                                                                'in_octets': '371021531560',
                                                                                                'in_ucast_pkts': '269070296',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '23643',
                                                                                                'out_octets': '15570400974',
                                                                                                'out_ucast_pkts': '93138719',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '23643',
                                                                       'out_octets': '15570400974',
                                                                       'out_ucast_pkts': '93138719',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODlDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '696994',
                                                                                                                        'fiber_length': 14487,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODlDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDg5Qw==',
                                         'serial_number': 'ADTN2314489C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '696994'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181515',
                                                                       'in_octets': '103624194410',
                                                                       'in_ucast_pkts': '359433717',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15384',
                                                                       'out_octets': '1507930273863',
                                                                       'out_ucast_pkts': '1092314172',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15384',
                                                                       'in_octets': '1507930273863',
                                                                       'in_ucast_pkts': '1092314172',
                                                                       'links': [       {       'in_nucast_pkts': '15384',
                                                                                                'in_octets': '1507930273863',
                                                                                                'in_ucast_pkts': '1092314172',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181515',
                                                                                                'out_octets': '103624194410',
                                                                                                'out_ucast_pkts': '359433717',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181515',
                                                                       'out_octets': '103624194410',
                                                                       'out_ucast_pkts': '359433717',
                                                                       'poller_hash': 'QURUTjIzMTQ0OEJBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355621',
                                                                                                                        'fiber_length': 14745,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEJBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhCQQ==',
                                         'serial_number': 'ADTN231448BA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355621'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181637',
                                                                       'in_octets': '747454418392',
                                                                       'in_ucast_pkts': '1068389034',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5952',
                                                                       'out_octets': '2679772269223',
                                                                       'out_ucast_pkts': '2022632883',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5952',
                                                                       'in_octets': '2679772269223',
                                                                       'in_ucast_pkts': '2022632883',
                                                                       'links': [       {       'in_nucast_pkts': '5952',
                                                                                                'in_octets': '2679772269223',
                                                                                                'in_ucast_pkts': '2022632883',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181637',
                                                                                                'out_octets': '747454418392',
                                                                                                'out_ucast_pkts': '1068389034',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181637',
                                                                       'out_octets': '747454418392',
                                                                       'out_ucast_pkts': '1068389034',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODlEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355622',
                                                                                                                        'fiber_length': 14466,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODlEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDg5RA==',
                                         'serial_number': 'ADTN2314489D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355622'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14494',
                                                                       'in_octets': '4484194474',
                                                                       'in_ucast_pkts': '25912450',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1226',
                                                                       'out_octets': '193882004883',
                                                                       'out_ucast_pkts': '132819587',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1226',
                                                                       'in_octets': '193882004883',
                                                                       'in_ucast_pkts': '132819587',
                                                                       'links': [       {       'in_nucast_pkts': '1226',
                                                                                                'in_octets': '193882004883',
                                                                                                'in_ucast_pkts': '132819587',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14494',
                                                                                                'out_octets': '4484194474',
                                                                                                'out_ucast_pkts': '25912450',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14494',
                                                                       'out_octets': '4484194474',
                                                                       'out_ucast_pkts': '25912450',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODNCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427033',
                                                                                                                        'fiber_length': 2182,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODNCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDgzQg==',
                                         'serial_number': 'ADTN2314483B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427033'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '178708',
                                                                       'in_octets': '78648404371',
                                                                       'in_ucast_pkts': '538668066',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5952',
                                                                       'out_octets': '2649428185093',
                                                                       'out_ucast_pkts': '1818879008',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5952',
                                                                       'in_octets': '2649428185093',
                                                                       'in_ucast_pkts': '1818879008',
                                                                       'links': [       {       'in_nucast_pkts': '5952',
                                                                                                'in_octets': '2649428185093',
                                                                                                'in_ucast_pkts': '1818879008',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '178708',
                                                                                                'out_octets': '78648404371',
                                                                                                'out_ucast_pkts': '538668066',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '178708',
                                                                       'out_octets': '78648404371',
                                                                       'out_ucast_pkts': '538668066',
                                                                       'poller_hash': 'QURUTjIzMTQ0Nzk2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355628',
                                                                                                                        'fiber_length': 9375,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0Nzk2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDc5Ng==',
                                         'serial_number': 'ADTN23144796',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355628'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181527',
                                                                       'in_octets': '118694247227',
                                                                       'in_ucast_pkts': '744512617',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15370',
                                                                       'out_octets': '1744583538342',
                                                                       'out_ucast_pkts': '1340512502',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VOV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15370',
                                                                       'in_octets': '1744583538342',
                                                                       'in_ucast_pkts': '1340512502',
                                                                       'links': [       {       'in_nucast_pkts': '15370',
                                                                                                'in_octets': '1744583538342',
                                                                                                'in_ucast_pkts': '1340512502',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181527',
                                                                                                'out_octets': '118694247227',
                                                                                                'out_ucast_pkts': '744512617',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181527',
                                                                       'out_octets': '118694247227',
                                                                       'out_ucast_pkts': '744512617',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODgzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355622',
                                                                                                                        'fiber_length': 14826,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODgzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDg4Mw==',
                                         'serial_number': 'ADTN23144883',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355622'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-4.1.18.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x4-ONU18_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMThfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0OEM5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEM5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x4-ONU18',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhDOQ==',
                                         'serial_number': 'ADTN231448C9',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181542',
                                                                       'in_octets': '28908246753',
                                                                       'in_ucast_pkts': '225045072',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15379',
                                                                       'out_octets': '696477307941',
                                                                       'out_ucast_pkts': '493506105',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15379',
                                                                       'in_octets': '696477307941',
                                                                       'in_ucast_pkts': '493506105',
                                                                       'links': [       {       'in_nucast_pkts': '15379',
                                                                                                'in_octets': '696477307941',
                                                                                                'in_ucast_pkts': '493506105',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181542',
                                                                                                'out_octets': '28908246753',
                                                                                                'out_ucast_pkts': '225045072',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181542',
                                                                       'out_octets': '28908246753',
                                                                       'out_ucast_pkts': '225045072',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzA3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355624',
                                                                                                                        'fiber_length': 14521,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzA3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDcwNw==',
                                         'serial_number': 'ADTN23144707',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355624'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-12.1.12.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x12-ONU12_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTEyX0VUSF8x'},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0NkJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x12-ONU12',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZCQw==',
                                         'serial_number': 'ADTN231446BC',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14482',
                                                                       'in_octets': '1923211696',
                                                                       'in_ucast_pkts': '8016011',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '22878018917',
                                                                       'out_ucast_pkts': '17994918',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI2X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '22878018917',
                                                                       'in_ucast_pkts': '17994918',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '22878018917',
                                                                                                'in_ucast_pkts': '17994918',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14482',
                                                                                                'out_octets': '1923211696',
                                                                                                'out_ucast_pkts': '8016011',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14482',
                                                                       'out_octets': '1923211696',
                                                                       'out_ucast_pkts': '8016011',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzdBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 3993,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzdBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDc3QQ==',
                                         'serial_number': 'ADTN2314477A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181585',
                                                                       'in_octets': '39407377498',
                                                                       'in_ucast_pkts': '111477829',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15373',
                                                                       'out_octets': '296003448852',
                                                                       'out_ucast_pkts': '256938233',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15373',
                                                                       'in_octets': '296003448852',
                                                                       'in_ucast_pkts': '256938233',
                                                                       'links': [       {       'in_nucast_pkts': '15373',
                                                                                                'in_octets': '296003448852',
                                                                                                'in_ucast_pkts': '256938233',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181585',
                                                                                                'out_octets': '39407377498',
                                                                                                'out_ucast_pkts': '111477829',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181585',
                                                                       'out_octets': '39407377498',
                                                                       'out_ucast_pkts': '111477829',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzAyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355626',
                                                                                                                        'fiber_length': 14662,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzAyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDcwMg==',
                                         'serial_number': 'ADTN23144702',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355626'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '178548',
                                                                       'in_octets': '58400515225',
                                                                       'in_ucast_pkts': '102604284',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5952',
                                                                       'out_octets': '283805844020',
                                                                       'out_ucast_pkts': '233979174',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5952',
                                                                       'in_octets': '283805844020',
                                                                       'in_ucast_pkts': '233979174',
                                                                       'links': [       {       'in_nucast_pkts': '5952',
                                                                                                'in_octets': '283805844020',
                                                                                                'in_ucast_pkts': '233979174',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '178548',
                                                                                                'out_octets': '58400515225',
                                                                                                'out_ucast_pkts': '102604284',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '178548',
                                                                       'out_octets': '58400515225',
                                                                       'out_ucast_pkts': '102604284',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0JEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355624',
                                                                                                                        'fiber_length': 13218,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0JEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdCRA==',
                                         'serial_number': 'ADTN231447BD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355624'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11192',
                                                                       'in_octets': '15555701442',
                                                                       'in_ucast_pkts': '37628460',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '212251197099',
                                                                       'out_ucast_pkts': '163182738',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTEzX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '212251197099',
                                                                       'in_ucast_pkts': '163182738',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '212251197099',
                                                                                                'in_ucast_pkts': '163182738',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11192',
                                                                                                'out_octets': '15555701442',
                                                                                                'out_ucast_pkts': '37628460',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11192',
                                                                       'out_octets': '15555701442',
                                                                       'out_ucast_pkts': '37628460',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkI4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327027',
                                                                                                                        'fiber_length': 9457,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkI4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZCOA==',
                                         'serial_number': 'ADTN231446B8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327027'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '114060',
                                                                       'in_octets': '89909882389',
                                                                       'in_ucast_pkts': '158297701',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9654',
                                                                       'out_octets': '404682300236',
                                                                       'out_ucast_pkts': '317195659',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '9654',
                                                                       'in_octets': '404682300236',
                                                                       'in_ucast_pkts': '317195659',
                                                                       'links': [       {       'in_nucast_pkts': '9654',
                                                                                                'in_octets': '404682300236',
                                                                                                'in_ucast_pkts': '317195659',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '114060',
                                                                                                'out_octets': '89909882389',
                                                                                                'out_ucast_pkts': '158297701',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '114060',
                                                                       'out_octets': '89909882389',
                                                                       'out_ucast_pkts': '158297701',
                                                                       'poller_hash': 'QURUTjIzMTQ0OEQ1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3364586',
                                                                                                                        'fiber_length': 5964,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEQ1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhENQ==',
                                         'serial_number': 'ADTN231448D5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3364586'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '29989',
                                                                       'in_octets': '1221926735',
                                                                       'in_ucast_pkts': '1237098',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '451',
                                                                       'out_octets': '1213350778',
                                                                       'out_ucast_pkts': '1231323',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '451',
                                                                       'in_octets': '1213350778',
                                                                       'in_ucast_pkts': '1231323',
                                                                       'links': [       {       'in_nucast_pkts': '451',
                                                                                                'in_octets': '1213350778',
                                                                                                'in_ucast_pkts': '1231323',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '29989',
                                                                                                'out_octets': '1221926735',
                                                                                                'out_ucast_pkts': '1237098',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '29989',
                                                                       'out_octets': '1221926735',
                                                                       'out_ucast_pkts': '1237098',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzQ4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 3353,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzQ4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDc0OA==',
                                         'serial_number': 'ADTN23144748',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14848',
                                                                       'in_octets': '1440796980',
                                                                       'in_ucast_pkts': '2796705',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x14-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '17436492765',
                                                                       'out_ucast_pkts': '12461495',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDE0LU9OVTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '17436492765',
                                                                       'in_ucast_pkts': '12461495',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '17436492765',
                                                                                                'in_ucast_pkts': '12461495',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14848',
                                                                                                'out_octets': '1440796980',
                                                                                                'out_ucast_pkts': '2796705',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14848',
                                                                       'out_octets': '1440796980',
                                                                       'out_ucast_pkts': '2796705',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427034',
                                                                                                                        'fiber_length': 780,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x14-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDcxNA==',
                                         'serial_number': 'ADTN23144714',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427034'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181534',
                                                                       'in_octets': '150318711426',
                                                                       'in_ucast_pkts': '604576406',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15379',
                                                                       'out_octets': '2125424430048',
                                                                       'out_ucast_pkts': '1580422205',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE0X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15379',
                                                                       'in_octets': '2125424430048',
                                                                       'in_ucast_pkts': '1580422205',
                                                                       'links': [       {       'in_nucast_pkts': '15379',
                                                                                                'in_octets': '2125424430048',
                                                                                                'in_ucast_pkts': '1580422205',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181534',
                                                                                                'out_octets': '150318711426',
                                                                                                'out_ucast_pkts': '604576406',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181534',
                                                                       'out_octets': '150318711426',
                                                                       'out_ucast_pkts': '604576406',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkNCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355622',
                                                                                                                        'fiber_length': 8828,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZDQg==',
                                         'serial_number': 'ADTN231446CB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355622'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-6.1.15.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x6-ONU15_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTVfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0N0UzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0UzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x6-ONU15',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdFMw==',
                                         'serial_number': 'ADTN231447E3',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.27.0.eth.1.phy',
                                                                       'in_nucast_pkts': '31123',
                                                                       'in_octets': '29998733749',
                                                                       'in_ucast_pkts': '30479439',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU27_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2633',
                                                                       'out_octets': '15272543552',
                                                                       'out_ucast_pkts': '17448007',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI3X0VUSF8x',
                                                                       'speed': '100000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2633',
                                                                       'in_octets': '15272543552',
                                                                       'in_ucast_pkts': '17448007',
                                                                       'links': [       {       'in_nucast_pkts': '2633',
                                                                                                'in_octets': '15272543552',
                                                                                                'in_ucast_pkts': '17448007',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '31123',
                                                                                                'out_octets': '29998733749',
                                                                                                'out_ucast_pkts': '30479439',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '31123',
                                                                       'out_octets': '29998733749',
                                                                       'out_ucast_pkts': '30479439',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODBEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '918509',
                                                                                                                        'fiber_length': 4007,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODBEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU27',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDgwRA==',
                                         'serial_number': 'ADTN2314480D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '918509'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '178542',
                                                                       'in_octets': '141812945571',
                                                                       'in_ucast_pkts': '1004023051',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5952',
                                                                       'out_octets': '4783940559298',
                                                                       'out_ucast_pkts': '3415646167',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE1X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5952',
                                                                       'in_octets': '4783940559298',
                                                                       'in_ucast_pkts': '3415646167',
                                                                       'links': [       {       'in_nucast_pkts': '5952',
                                                                                                'in_octets': '4783940559298',
                                                                                                'in_ucast_pkts': '3415646167',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '178542',
                                                                                                'out_octets': '141812945571',
                                                                                                'out_ucast_pkts': '1004023051',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '178542',
                                                                       'out_octets': '141812945571',
                                                                       'out_ucast_pkts': '1004023051',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODYxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355621',
                                                                                                                        'fiber_length': 8370,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODYxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDg2MQ==',
                                         'serial_number': 'ADTN23144861',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355621'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14477',
                                                                       'in_octets': '10649180489',
                                                                       'in_ucast_pkts': '104194724',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '319260136551',
                                                                       'out_ucast_pkts': '217951826',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '319260136551',
                                                                       'in_ucast_pkts': '217951826',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '319260136551',
                                                                                                'in_ucast_pkts': '217951826',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14477',
                                                                                                'out_octets': '10649180489',
                                                                                                'out_ucast_pkts': '104194724',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14477',
                                                                       'out_octets': '10649180489',
                                                                       'out_ucast_pkts': '104194724',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzczIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427024',
                                                                                                                        'fiber_length': 233,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzczIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDc3Mw==',
                                         'serial_number': 'ADTN23144773',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427024'},
                                 {       'interfaces': [       {       'admin_up': False,
                                                                       'id': 'onu-subscr-if-2.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 0,
                                                                       'name': 'IndianBayou-x2-ONU14_ETH_1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTRfRVRIXzE=',
                                                                       'type': 6},
                                                               {       'admin_up': False,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': False,
                                                                                                'out_nucast_pkts': '0',
                                                                                                'out_octets': '0',
                                                                                                'out_ucast_pkts': '0',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkNDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355594',
                                                                                                                        'fiber_length': 12154,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZDQw==',
                                         'serial_number': 'ADTN231446CC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355594'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14608',
                                                                       'in_octets': '10170294001',
                                                                       'in_ucast_pkts': '42209559',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '142833690444',
                                                                       'out_ucast_pkts': '105137132',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMThfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '142833690444',
                                                                       'in_ucast_pkts': '105137132',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '142833690444',
                                                                                                'in_ucast_pkts': '105137132',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14608',
                                                                                                'out_octets': '10170294001',
                                                                                                'out_ucast_pkts': '42209559',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14608',
                                                                       'out_octets': '10170294001',
                                                                       'out_ucast_pkts': '42209559',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427032',
                                                                                                                        'fiber_length': 1780,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZDMA==',
                                         'serial_number': 'ADTN231446C0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427032'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '181535',
                                                                       'in_octets': '39608918242',
                                                                       'in_ucast_pkts': '165154942',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15371',
                                                                       'out_octets': '880189602473',
                                                                       'out_ucast_pkts': '626030024',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15371',
                                                                       'in_octets': '880189602473',
                                                                       'in_ucast_pkts': '626030024',
                                                                       'links': [       {       'in_nucast_pkts': '15371',
                                                                                                'in_octets': '880189602473',
                                                                                                'in_ucast_pkts': '626030024',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '181535',
                                                                                                'out_octets': '39608918242',
                                                                                                'out_ucast_pkts': '165154942',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '181535',
                                                                       'out_octets': '39608918242',
                                                                       'out_ucast_pkts': '165154942',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0FFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355625',
                                                                                                                        'fiber_length': 13016,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdBRQ==',
                                         'serial_number': 'ADTN231447AE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355625'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '49152',
                                                                       'in_octets': '1631760670',
                                                                       'in_ucast_pkts': '2740950',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4159',
                                                                       'out_octets': '3452620893',
                                                                       'out_ucast_pkts': '3491273',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4159',
                                                                       'in_octets': '3452620893',
                                                                       'in_ucast_pkts': '3491273',
                                                                       'links': [       {       'in_nucast_pkts': '4159',
                                                                                                'in_octets': '3452620893',
                                                                                                'in_ucast_pkts': '3491273',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '49152',
                                                                                                'out_octets': '1631760670',
                                                                                                'out_ucast_pkts': '2740950',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '49152',
                                                                       'out_octets': '1631760670',
                                                                       'out_ucast_pkts': '2740950',
                                                                       'poller_hash': 'QURUTjIzMTQ0OEZFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1450250',
                                                                                                                        'fiber_length': 14823,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEZFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhGRQ==',
                                         'serial_number': 'ADTN231448FE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1450250'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '31817',
                                                                       'in_octets': '10028074215',
                                                                       'in_ucast_pkts': '37876254',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2691',
                                                                       'out_octets': '121263697438',
                                                                       'out_ucast_pkts': '94319708',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2691',
                                                                       'in_octets': '121263697438',
                                                                       'in_ucast_pkts': '94319708',
                                                                       'links': [       {       'in_nucast_pkts': '2691',
                                                                                                'in_octets': '121263697438',
                                                                                                'in_ucast_pkts': '94319708',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '31817',
                                                                                                'out_octets': '10028074215',
                                                                                                'out_ucast_pkts': '37876254',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '31817',
                                                                       'out_octets': '10028074215',
                                                                       'out_ucast_pkts': '37876254',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkE1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '938225',
                                                                                                                        'fiber_length': 14178,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkE1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZBNQ==',
                                         'serial_number': 'ADTN231446A5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '938225'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14249',
                                                                       'in_octets': '9238663898',
                                                                       'in_ucast_pkts': '44807023',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '144197548729',
                                                                       'out_ucast_pkts': '105308065',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '144197548729',
                                                                       'in_ucast_pkts': '105308065',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '144197548729',
                                                                                                'in_ucast_pkts': '105308065',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14249',
                                                                                                'out_octets': '9238663898',
                                                                                                'out_ucast_pkts': '44807023',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14249',
                                                                       'out_octets': '9238663898',
                                                                       'out_ucast_pkts': '44807023',
                                                                       'poller_hash': 'QURUTjIzMTQ0OTE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 2012,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OTE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDkxNg==',
                                         'serial_number': 'ADTN23144916',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '67546',
                                                                       'in_octets': '30381695416',
                                                                       'in_ucast_pkts': '148236549',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5627',
                                                                       'out_octets': '731272078042',
                                                                       'out_ucast_pkts': '522495069',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5627',
                                                                       'in_octets': '731272078042',
                                                                       'in_ucast_pkts': '522495069',
                                                                       'links': [       {       'in_nucast_pkts': '5627',
                                                                                                'in_octets': '731272078042',
                                                                                                'in_ucast_pkts': '522495069',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '67546',
                                                                                                'out_octets': '30381695416',
                                                                                                'out_ucast_pkts': '148236549',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '67546',
                                                                       'out_octets': '30381695416',
                                                                       'out_ucast_pkts': '148236549',
                                                                       'poller_hash': 'QURUTjIzMTQ0ODZDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1960821',
                                                                                                                        'fiber_length': 10744,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0ODZDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDg2Qw==',
                                         'serial_number': 'ADTN2314486C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1960821'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '40855',
                                                                       'in_octets': '34452701031',
                                                                       'in_ucast_pkts': '98440496',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1361',
                                                                       'out_octets': '418703305580',
                                                                       'out_ucast_pkts': '301311136',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1361',
                                                                       'in_octets': '418703305580',
                                                                       'in_ucast_pkts': '301311136',
                                                                       'links': [       {       'in_nucast_pkts': '1361',
                                                                                                'in_octets': '418703305580',
                                                                                                'in_ucast_pkts': '301311136',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '40855',
                                                                                                'out_octets': '34452701031',
                                                                                                'out_ucast_pkts': '98440496',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '40855',
                                                                       'out_octets': '34452701031',
                                                                       'out_ucast_pkts': '98440496',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0RBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1224999',
                                                                                                                        'fiber_length': 3596,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0RBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdEQQ==',
                                         'serial_number': 'ADTN231447DA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1224999'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-9.1.22.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x9-ONU22_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjJfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0OEI0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEI0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x9-ONU22',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhCNA==',
                                         'serial_number': 'ADTN231448B4',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-6.1.17.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x6-ONU17_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTdfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0NjhDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NjhDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x6-ONU17',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDY4Qw==',
                                         'serial_number': 'ADTN2314468C',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.28.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14483',
                                                                       'in_octets': '13072353988',
                                                                       'in_ucast_pkts': '27611640',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU28_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '159270900949',
                                                                       'out_ucast_pkts': '113977534',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI4X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '159270900949',
                                                                       'in_ucast_pkts': '113977534',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '159270900949',
                                                                                                'in_ucast_pkts': '113977534',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14483',
                                                                                                'out_octets': '13072353988',
                                                                                                'out_ucast_pkts': '27611640',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14483',
                                                                       'out_octets': '13072353988',
                                                                       'out_ucast_pkts': '27611640',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0FCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427028',
                                                                                                                        'fiber_length': 4238,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU28',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdBQg==',
                                         'serial_number': 'ADTN231447AB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '163013',
                                                                       'in_octets': '89954696087',
                                                                       'in_ucast_pkts': '322041214',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '13694',
                                                                       'out_octets': '1215213876404',
                                                                       'out_ucast_pkts': '907506591',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '13694',
                                                                       'in_octets': '1215213876404',
                                                                       'in_ucast_pkts': '907506591',
                                                                       'links': [       {       'in_nucast_pkts': '13694',
                                                                                                'in_octets': '1215213876404',
                                                                                                'in_ucast_pkts': '907506591',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '163013',
                                                                                                'out_octets': '89954696087',
                                                                                                'out_ucast_pkts': '322041214',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '163013',
                                                                       'out_octets': '89954696087',
                                                                       'out_ucast_pkts': '322041214',
                                                                       'poller_hash': 'QURUTjIzMTI1MTRBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4773320',
                                                                                                                        'fiber_length': 14481,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTRBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0QQ==',
                                         'serial_number': 'ADTN2312514A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '4773320'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.25.0.eth.1.phy',
                                                                       'in_nucast_pkts': '592',
                                                                       'in_octets': '18872725877',
                                                                       'in_ucast_pkts': '54923384',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU25_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2779',
                                                                       'out_octets': '197713632354',
                                                                       'out_ucast_pkts': '149092553',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2779',
                                                                       'in_octets': '197713632354',
                                                                       'in_ucast_pkts': '149092553',
                                                                       'links': [       {       'in_nucast_pkts': '2779',
                                                                                                'in_octets': '197713632354',
                                                                                                'in_ucast_pkts': '149092553',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '592',
                                                                                                'out_octets': '18872725877',
                                                                                                'out_ucast_pkts': '54923384',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '592',
                                                                       'out_octets': '18872725877',
                                                                       'out_ucast_pkts': '54923384',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzM1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '969066',
                                                                                                                        'fiber_length': 4136,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzM1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU25',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDczNQ==',
                                         'serial_number': 'ADTN23144735',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '969066'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14514',
                                                                       'in_octets': '44117505825',
                                                                       'in_ucast_pkts': '140548890',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1228',
                                                                       'out_octets': '355477843613',
                                                                       'out_ucast_pkts': '268863394',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE1X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1228',
                                                                       'in_octets': '355477843613',
                                                                       'in_ucast_pkts': '268863394',
                                                                       'links': [       {       'in_nucast_pkts': '1228',
                                                                                                'in_octets': '355477843613',
                                                                                                'in_ucast_pkts': '268863394',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14514',
                                                                                                'out_octets': '44117505825',
                                                                                                'out_ucast_pkts': '140548890',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14514',
                                                                       'out_octets': '44117505825',
                                                                       'out_ucast_pkts': '140548890',
                                                                       'poller_hash': 'QURUTjIzMTQ0NkNFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427031',
                                                                                                                        'fiber_length': 7294,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NkNFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDZDRQ==',
                                         'serial_number': 'ADTN231446CE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427031'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14261',
                                                                       'in_octets': '4585573724',
                                                                       'in_ucast_pkts': '15384444',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '54811201689',
                                                                       'out_ucast_pkts': '43212303',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '54811201689',
                                                                       'in_ucast_pkts': '43212303',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '54811201689',
                                                                                                'in_ucast_pkts': '43212303',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14261',
                                                                                                'out_octets': '4585573724',
                                                                                                'out_ucast_pkts': '15384444',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14261',
                                                                       'out_octets': '4585573724',
                                                                       'out_ucast_pkts': '15384444',
                                                                       'poller_hash': 'QURUTjIzMTQ0OEQxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427031',
                                                                                                                        'fiber_length': 2661,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEQxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhEMQ==',
                                         'serial_number': 'ADTN231448D1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427031'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.29.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14812',
                                                                       'in_octets': '140478805070',
                                                                       'in_ucast_pkts': '206090775',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU29_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '509510772101',
                                                                       'out_ucast_pkts': '407999782',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTI5X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '509510772101',
                                                                       'in_ucast_pkts': '407999782',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '509510772101',
                                                                                                'in_ucast_pkts': '407999782',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14812',
                                                                                                'out_octets': '140478805070',
                                                                                                'out_ucast_pkts': '206090775',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14812',
                                                                       'out_octets': '140478805070',
                                                                       'out_ucast_pkts': '206090775',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0YzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427034',
                                                                                                                        'fiber_length': 4128,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0YzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU29',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdGMw==',
                                         'serial_number': 'ADTN231447F3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427034'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '183326',
                                                                       'in_octets': '31927005547',
                                                                       'in_ucast_pkts': '73791556',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '15369',
                                                                       'out_octets': '148961018578',
                                                                       'out_ucast_pkts': '130331928',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '15369',
                                                                       'in_octets': '148961018578',
                                                                       'in_ucast_pkts': '130331928',
                                                                       'links': [       {       'in_nucast_pkts': '15369',
                                                                                                'in_octets': '148961018578',
                                                                                                'in_ucast_pkts': '130331928',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '183326',
                                                                                                'out_octets': '31927005547',
                                                                                                'out_ucast_pkts': '73791556',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '183326',
                                                                       'out_octets': '31927005547',
                                                                       'out_ucast_pkts': '73791556',
                                                                       'poller_hash': 'QURUTjIzMTQ0Njk5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355629',
                                                                                                                        'fiber_length': 8892,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0Njk5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDY5OQ==',
                                         'serial_number': 'ADTN23144699',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355629'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.31.0.eth.1.phy',
                                                                       'in_nucast_pkts': '265',
                                                                       'in_octets': '9081539991',
                                                                       'in_ucast_pkts': '24965850',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU31_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '102797141730',
                                                                       'out_ucast_pkts': '72870428',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMzFfRVRIXzE=',
                                                                       'speed': '100000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '102797141730',
                                                                       'in_ucast_pkts': '72870428',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '102797141730',
                                                                                                'in_ucast_pkts': '72870428',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '265',
                                                                                                'out_octets': '9081539991',
                                                                                                'out_ucast_pkts': '24965850',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '265',
                                                                       'out_octets': '9081539991',
                                                                       'out_ucast_pkts': '24965850',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0E0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427032',
                                                                                                                        'fiber_length': 5397,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0E0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU31',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdBNA==',
                                         'serial_number': 'ADTN231447A4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427032'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '3563',
                                                                       'in_octets': '5015036161',
                                                                       'in_ucast_pkts': '7132338',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '120',
                                                                       'out_octets': '29720959044',
                                                                       'out_ucast_pkts': '22368483',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '120',
                                                                       'in_octets': '29720959044',
                                                                       'in_ucast_pkts': '22368483',
                                                                       'links': [       {       'in_nucast_pkts': '120',
                                                                                                'in_octets': '29720959044',
                                                                                                'in_ucast_pkts': '22368483',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '3563',
                                                                                                'out_octets': '5015036161',
                                                                                                'out_ucast_pkts': '7132338',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3563',
                                                                       'out_octets': '5015036161',
                                                                       'out_ucast_pkts': '7132338',
                                                                       'poller_hash': 'QURUTjIzMTQ0NzU5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '106257',
                                                                                                                        'fiber_length': 6744,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NzU5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDc1OQ==',
                                         'serial_number': 'ADTN23144759',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '106257'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14493',
                                                                       'in_octets': '4479141903',
                                                                       'in_ucast_pkts': '41446317',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '130555458155',
                                                                       'out_ucast_pkts': '89477649',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '130555458155',
                                                                       'in_ucast_pkts': '89477649',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '130555458155',
                                                                                                'in_ucast_pkts': '89477649',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14493',
                                                                                                'out_octets': '4479141903',
                                                                                                'out_ucast_pkts': '41446317',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14493',
                                                                       'out_octets': '4479141903',
                                                                       'out_ucast_pkts': '41446317',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTlDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 3282,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTlDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE5Qw==',
                                         'serial_number': 'ADTN23144A9C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14837',
                                                                       'in_octets': '2937545669',
                                                                       'in_ucast_pkts': '17554396',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '88281793537',
                                                                       'out_ucast_pkts': '62529827',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '88281793537',
                                                                       'in_ucast_pkts': '62529827',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '88281793537',
                                                                                                'in_ucast_pkts': '62529827',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14837',
                                                                                                'out_octets': '2937545669',
                                                                                                'out_ucast_pkts': '17554396',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14837',
                                                                       'out_octets': '2937545669',
                                                                       'out_ucast_pkts': '17554396',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjNBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427032',
                                                                                                                        'fiber_length': 545,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjNBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIzQQ==',
                                         'serial_number': 'ADTN23144B3A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427032'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74436',
                                                                       'in_octets': '36425499264',
                                                                       'in_ucast_pkts': '101071453',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2479',
                                                                       'out_octets': '385596721031',
                                                                       'out_ucast_pkts': '291389941',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2479',
                                                                       'in_octets': '385596721031',
                                                                       'in_ucast_pkts': '291389941',
                                                                       'links': [       {       'in_nucast_pkts': '2479',
                                                                                                'in_octets': '385596721031',
                                                                                                'in_ucast_pkts': '291389941',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74436',
                                                                                                'out_octets': '36425499264',
                                                                                                'out_ucast_pkts': '101071453',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74436',
                                                                       'out_octets': '36425499264',
                                                                       'out_ucast_pkts': '101071453',
                                                                       'poller_hash': 'QURUTjIzMTQ0QUIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232576',
                                                                                                                        'fiber_length': 4574,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QUIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEFCMA==',
                                         'serial_number': 'ADTN23144AB0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232576'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '9115',
                                                                       'in_octets': '2830692519',
                                                                       'in_ucast_pkts': '22765801',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '772',
                                                                       'out_octets': '70890071883',
                                                                       'out_ucast_pkts': '48865187',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '772',
                                                                       'in_octets': '70890071883',
                                                                       'in_ucast_pkts': '48865187',
                                                                       'links': [       {       'in_nucast_pkts': '772',
                                                                                                'in_octets': '70890071883',
                                                                                                'in_ucast_pkts': '48865187',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '9115',
                                                                                                'out_octets': '2830692519',
                                                                                                'out_ucast_pkts': '22765801',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '9115',
                                                                       'out_octets': '2830692519',
                                                                       'out_ucast_pkts': '22765801',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '268464',
                                                                                                                        'fiber_length': 4011,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIyQw==',
                                         'serial_number': 'ADTN23144B2C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '268464'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '29225',
                                                                       'in_octets': '11170979770',
                                                                       'in_ucast_pkts': '40121588',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2474',
                                                                       'out_octets': '333470491980',
                                                                       'out_ucast_pkts': '231950841',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE2X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2474',
                                                                       'in_octets': '333470491980',
                                                                       'in_ucast_pkts': '231950841',
                                                                       'links': [       {       'in_nucast_pkts': '2474',
                                                                                                'in_octets': '333470491980',
                                                                                                'in_ucast_pkts': '231950841',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '29225',
                                                                                                'out_octets': '11170979770',
                                                                                                'out_ucast_pkts': '40121588',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '29225',
                                                                       'out_octets': '11170979770',
                                                                       'out_ucast_pkts': '40121588',
                                                                       'poller_hash': 'QURUTjIzMTQ0RERGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '861787',
                                                                                                                        'fiber_length': 8094,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RERGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NERERg==',
                                         'serial_number': 'ADTN23144DDF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '861787'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14837',
                                                                       'in_octets': '6618612021',
                                                                       'in_ucast_pkts': '22102834',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '60367612439',
                                                                       'out_ucast_pkts': '49153075',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE3X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '60367612439',
                                                                       'in_ucast_pkts': '49153075',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '60367612439',
                                                                                                'in_ucast_pkts': '49153075',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14837',
                                                                                                'out_octets': '6618612021',
                                                                                                'out_ucast_pkts': '22102834',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14837',
                                                                       'out_octets': '6618612021',
                                                                       'out_ucast_pkts': '22102834',
                                                                       'poller_hash': 'QURUTjIzMTQ0RDUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427031',
                                                                                                                        'fiber_length': 7706,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RDUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEQ1MA==',
                                         'serial_number': 'ADTN23144D50',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427031'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11193',
                                                                       'in_octets': '2614312908',
                                                                       'in_ucast_pkts': '17215659',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '65148013224',
                                                                       'out_ucast_pkts': '46404667',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTdfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '65148013224',
                                                                       'in_ucast_pkts': '46404667',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '65148013224',
                                                                                                'in_ucast_pkts': '46404667',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11193',
                                                                                                'out_octets': '2614312908',
                                                                                                'out_ucast_pkts': '17215659',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11193',
                                                                       'out_octets': '2614312908',
                                                                       'out_ucast_pkts': '17215659',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTE1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327025',
                                                                                                                        'fiber_length': 7986,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTE1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEExNQ==',
                                         'serial_number': 'ADTN23144A15',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327025'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '53275',
                                                                       'in_octets': '66181121606',
                                                                       'in_ucast_pkts': '251617020',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4507',
                                                                       'out_octets': '975716368421',
                                                                       'out_ucast_pkts': '706761030',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMTlfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4507',
                                                                       'in_octets': '975716368421',
                                                                       'in_ucast_pkts': '706761030',
                                                                       'links': [       {       'in_nucast_pkts': '4507',
                                                                                                'in_octets': '975716368421',
                                                                                                'in_ucast_pkts': '706761030',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '53275',
                                                                                                'out_octets': '66181121606',
                                                                                                'out_ucast_pkts': '251617020',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '53275',
                                                                       'out_octets': '66181121606',
                                                                       'out_ucast_pkts': '251617020',
                                                                       'poller_hash': 'QURUTjIzMTQ0Q0MzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1571617',
                                                                                                                        'fiber_length': 10521,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0MzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzE0NENDMw==',
                                         'serial_number': 'ADTN23144CC3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1571617'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14485',
                                                                       'in_octets': '431823514',
                                                                       'in_ucast_pkts': '650656',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x14-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '1314724262',
                                                                       'out_ucast_pkts': '1246654',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDE0LU9OVTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '1314724262',
                                                                       'in_ucast_pkts': '1246654',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '1314724262',
                                                                                                'in_ucast_pkts': '1246654',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14485',
                                                                                                'out_octets': '431823514',
                                                                                                'out_ucast_pkts': '650656',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14485',
                                                                       'out_octets': '431823514',
                                                                       'out_ucast_pkts': '650656',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzMzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427034',
                                                                                                                        'fiber_length': 539,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzMzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x14-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEMzMw==',
                                         'serial_number': 'ADTN23144C33',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427034'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10925',
                                                                       'in_octets': '11584582976',
                                                                       'in_ucast_pkts': '36155516',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '97260334575',
                                                                       'out_ucast_pkts': '74915668',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '97260334575',
                                                                       'in_ucast_pkts': '74915668',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '97260334575',
                                                                                                'in_ucast_pkts': '74915668',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10925',
                                                                                                'out_octets': '11584582976',
                                                                                                'out_ucast_pkts': '36155516',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10925',
                                                                       'out_octets': '11584582976',
                                                                       'out_ucast_pkts': '36155516',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTVBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327027',
                                                                                                                        'fiber_length': 9303,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTVBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE1QQ==',
                                         'serial_number': 'ADTN23144A5A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327027'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14481',
                                                                       'in_octets': '4562146411',
                                                                       'in_ucast_pkts': '11084217',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '27854038641',
                                                                       'out_ucast_pkts': '22226636',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '27854038641',
                                                                       'in_ucast_pkts': '22226636',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '27854038641',
                                                                                                'in_ucast_pkts': '22226636',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14481',
                                                                                                'out_octets': '4562146411',
                                                                                                'out_ucast_pkts': '11084217',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14481',
                                                                       'out_octets': '4562146411',
                                                                       'out_ucast_pkts': '11084217',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzEwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427028',
                                                                                                                        'fiber_length': 1734,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzEwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEMxMA==',
                                         'serial_number': 'ADTN23144C10',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10920',
                                                                       'in_octets': '5696191608',
                                                                       'in_ucast_pkts': '11625565',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '57625493901',
                                                                       'out_ucast_pkts': '41738950',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE2X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '57625493901',
                                                                       'in_ucast_pkts': '41738950',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '57625493901',
                                                                                                'in_ucast_pkts': '41738950',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10920',
                                                                                                'out_octets': '5696191608',
                                                                                                'out_ucast_pkts': '11625565',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10920',
                                                                       'out_octets': '5696191608',
                                                                       'out_ucast_pkts': '11625565',
                                                                       'poller_hash': 'QURUTjIzMTQ0QkY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327028',
                                                                                                                        'fiber_length': 8538,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QkY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEJGNw==',
                                         'serial_number': 'ADTN23144BF7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10500',
                                                                       'in_octets': '4315964990',
                                                                       'in_ucast_pkts': '17363552',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '867',
                                                                       'out_octets': '57026256345',
                                                                       'out_ucast_pkts': '42378102',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTRfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '867',
                                                                       'in_octets': '57026256345',
                                                                       'in_ucast_pkts': '42378102',
                                                                       'links': [       {       'in_nucast_pkts': '867',
                                                                                                'in_octets': '57026256345',
                                                                                                'in_ucast_pkts': '42378102',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10500',
                                                                                                'out_octets': '4315964990',
                                                                                                'out_ucast_pkts': '17363552',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10500',
                                                                       'out_octets': '4315964990',
                                                                       'out_ucast_pkts': '17363552',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTUzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '302045',
                                                                                                                        'fiber_length': 9292,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTUzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE1Mw==',
                                         'serial_number': 'ADTN23144A53',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '302045'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11100',
                                                                       'in_octets': '6681485962',
                                                                       'in_ucast_pkts': '42967152',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '249261259637',
                                                                       'out_ucast_pkts': '170364556',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTVfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '249261259637',
                                                                       'in_ucast_pkts': '170364556',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '249261259637',
                                                                                                'in_ucast_pkts': '170364556',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11100',
                                                                                                'out_octets': '6681485962',
                                                                                                'out_ucast_pkts': '42967152',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11100',
                                                                       'out_octets': '6681485962',
                                                                       'out_ucast_pkts': '42967152',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTg0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327028',
                                                                                                                        'fiber_length': 8903,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTg0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE4NA==',
                                         'serial_number': 'ADTN23144A84',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75769',
                                                                       'in_octets': '40434694715',
                                                                       'in_ucast_pkts': '264411380',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6404',
                                                                       'out_octets': '970857885258',
                                                                       'out_ucast_pkts': '728217544',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6404',
                                                                       'in_octets': '970857885258',
                                                                       'in_ucast_pkts': '728217544',
                                                                       'links': [       {       'in_nucast_pkts': '6404',
                                                                                                'in_octets': '970857885258',
                                                                                                'in_ucast_pkts': '728217544',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75769',
                                                                                                'out_octets': '40434694715',
                                                                                                'out_ucast_pkts': '264411380',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75769',
                                                                       'out_octets': '40434694715',
                                                                       'out_ucast_pkts': '264411380',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTk3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232584',
                                                                                                                        'fiber_length': 6051,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTk3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU16',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE5Nw==',
                                         'serial_number': 'ADTN23144A97',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232584'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.32.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14259',
                                                                       'in_octets': '9556335117',
                                                                       'in_ucast_pkts': '22133906',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU32_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '62700366416',
                                                                       'out_ucast_pkts': '51739233',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMzJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '62700366416',
                                                                       'in_ucast_pkts': '51739233',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '62700366416',
                                                                                                'in_ucast_pkts': '51739233',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14259',
                                                                                                'out_octets': '9556335117',
                                                                                                'out_ucast_pkts': '22133906',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14259',
                                                                       'out_octets': '9556335117',
                                                                       'out_ucast_pkts': '22133906',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTY5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427038',
                                                                                                                        'fiber_length': 4383,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTY5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU32',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE2OQ==',
                                         'serial_number': 'ADTN23144A69',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427038'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10921',
                                                                       'in_octets': '3043675623',
                                                                       'in_ucast_pkts': '9936676',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '45570325495',
                                                                       'out_ucast_pkts': '33509700',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '45570325495',
                                                                       'in_ucast_pkts': '33509700',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '45570325495',
                                                                                                'in_ucast_pkts': '33509700',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10921',
                                                                                                'out_octets': '3043675623',
                                                                                                'out_ucast_pkts': '9936676',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10921',
                                                                       'out_octets': '3043675623',
                                                                       'out_ucast_pkts': '9936676',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTVDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327029',
                                                                                                                        'fiber_length': 9318,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTVDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE1Qw==',
                                         'serial_number': 'ADTN23144A5C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14490',
                                                                       'in_octets': '351591502',
                                                                       'in_ucast_pkts': '528810',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '372614589',
                                                                       'out_ucast_pkts': '498076',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMThfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '372614589',
                                                                       'in_ucast_pkts': '498076',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '372614589',
                                                                                                'in_ucast_pkts': '498076',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14490',
                                                                                                'out_octets': '351591502',
                                                                                                'out_ucast_pkts': '528810',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14490',
                                                                       'out_octets': '351591502',
                                                                       'out_ucast_pkts': '528810',
                                                                       'poller_hash': 'QURUTjIzMTQ0Q0Q1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 3249,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0Q1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzE0NENENQ==',
                                         'serial_number': 'ADTN23144CD5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-7.1.19.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x7-ONU19_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMTlfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'Q1hOSzAwQTQzMDVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'Q1hOSzAwQTQzMDVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x7-ONU19',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IENYTkswMEE0MzA1Qg==',
                                         'serial_number': 'CXNK00A4305B',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14481',
                                                                       'in_octets': '3791904941',
                                                                       'in_ucast_pkts': '8423097',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '25578469812',
                                                                       'out_ucast_pkts': '20420027',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '25578469812',
                                                                       'in_ucast_pkts': '20420027',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '25578469812',
                                                                                                'in_ucast_pkts': '20420027',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14481',
                                                                                                'out_octets': '3791904941',
                                                                                                'out_ucast_pkts': '8423097',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14481',
                                                                       'out_octets': '3791904941',
                                                                       'out_ucast_pkts': '8423097',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzQ2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427030',
                                                                                                                        'fiber_length': 2665,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzQ2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEM0Ng==',
                                         'serial_number': 'ADTN23144C46',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427030'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14252',
                                                                       'in_octets': '4816574218',
                                                                       'in_ucast_pkts': '16714166',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '68442769578',
                                                                       'out_ucast_pkts': '48338264',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTdfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '68442769578',
                                                                       'in_ucast_pkts': '48338264',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '68442769578',
                                                                                                'in_ucast_pkts': '48338264',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14252',
                                                                                                'out_octets': '4816574218',
                                                                                                'out_ucast_pkts': '16714166',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14252',
                                                                       'out_octets': '4816574218',
                                                                       'out_ucast_pkts': '16714166',
                                                                       'poller_hash': 'QURUTjIzMTQ0RDk5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427034',
                                                                                                                        'fiber_length': 9196,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RDk5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEQ5OQ==',
                                         'serial_number': 'ADTN23144D99',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427034'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10920',
                                                                       'in_octets': '6155899121',
                                                                       'in_ucast_pkts': '13805941',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '85065250252',
                                                                       'out_ucast_pkts': '62297141',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVThfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '85065250252',
                                                                       'in_ucast_pkts': '62297141',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '85065250252',
                                                                                                'in_ucast_pkts': '62297141',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10920',
                                                                                                'out_octets': '6155899121',
                                                                                                'out_ucast_pkts': '13805941',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10920',
                                                                       'out_octets': '6155899121',
                                                                       'out_ucast_pkts': '13805941',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjNGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327026',
                                                                                                                        'fiber_length': 9156,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjNGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIzRg==',
                                         'serial_number': 'ADTN23144B3F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327026'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10923',
                                                                       'in_octets': '11113504970',
                                                                       'in_ucast_pkts': '38851489',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '178322644314',
                                                                       'out_ucast_pkts': '125756425',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTEwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '178322644314',
                                                                       'in_ucast_pkts': '125756425',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '178322644314',
                                                                                                'in_ucast_pkts': '125756425',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10923',
                                                                                                'out_octets': '11113504970',
                                                                                                'out_ucast_pkts': '38851489',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10923',
                                                                       'out_octets': '11113504970',
                                                                       'out_ucast_pkts': '38851489',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjVFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327026',
                                                                                                                        'fiber_length': 9074,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjVFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU10',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEI1RQ==',
                                         'serial_number': 'ADTN23144B5E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327026'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10921',
                                                                       'in_octets': '4787032931',
                                                                       'in_ucast_pkts': '23749412',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '140274218693',
                                                                       'out_ucast_pkts': '107161210',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTExX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '140274218693',
                                                                       'in_ucast_pkts': '107161210',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '140274218693',
                                                                                                'in_ucast_pkts': '107161210',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10921',
                                                                                                'out_octets': '4787032931',
                                                                                                'out_ucast_pkts': '23749412',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10921',
                                                                       'out_octets': '4787032931',
                                                                       'out_ucast_pkts': '23749412',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTlCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327028',
                                                                                                                        'fiber_length': 9481,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTlCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE5Qg==',
                                         'serial_number': 'ADTN23144A9B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10953',
                                                                       'in_octets': '4102645831',
                                                                       'in_ucast_pkts': '13734254',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '366',
                                                                       'out_octets': '72279718107',
                                                                       'out_ucast_pkts': '55541440',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTEyX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '366',
                                                                       'in_octets': '72279718107',
                                                                       'in_ucast_pkts': '55541440',
                                                                       'links': [       {       'in_nucast_pkts': '366',
                                                                                                'in_octets': '72279718107',
                                                                                                'in_ucast_pkts': '55541440',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10953',
                                                                                                'out_octets': '4102645831',
                                                                                                'out_ucast_pkts': '13734254',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10953',
                                                                       'out_octets': '4102645831',
                                                                       'out_ucast_pkts': '13734254',
                                                                       'poller_hash': 'QURUTjIzMTQ0QUNGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327026',
                                                                                                                        'fiber_length': 9284,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QUNGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU12',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEFDRg==',
                                         'serial_number': 'ADTN23144ACF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327026'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14852',
                                                                       'in_octets': '13081311719',
                                                                       'in_ucast_pkts': '67658493',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '320476692980',
                                                                       'out_ucast_pkts': '237606249',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '320476692980',
                                                                       'in_ucast_pkts': '237606249',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '320476692980',
                                                                                                'in_ucast_pkts': '237606249',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14852',
                                                                                                'out_octets': '13081311719',
                                                                                                'out_ucast_pkts': '67658493',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14852',
                                                                       'out_octets': '13081311719',
                                                                       'out_ucast_pkts': '67658493',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTJCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427023',
                                                                                                                        'fiber_length': 324,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTJCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU22',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEEyQg==',
                                         'serial_number': 'ADTN23144A2B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427023'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14260',
                                                                       'in_octets': '18330245862',
                                                                       'in_ucast_pkts': '46570425',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '133623055334',
                                                                       'out_ucast_pkts': '99196688',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '133623055334',
                                                                       'in_ucast_pkts': '99196688',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '133623055334',
                                                                                                'in_ucast_pkts': '99196688',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14260',
                                                                                                'out_octets': '18330245862',
                                                                                                'out_ucast_pkts': '46570425',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14260',
                                                                       'out_octets': '18330245862',
                                                                       'out_ucast_pkts': '46570425',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 3298,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEI0RA==',
                                         'serial_number': 'ADTN23144B4D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14277',
                                                                       'in_octets': '54397660066',
                                                                       'in_ucast_pkts': '56952448',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '477',
                                                                       'out_octets': '62495928412',
                                                                       'out_ucast_pkts': '52136476',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTEzX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '477',
                                                                       'in_octets': '62495928412',
                                                                       'in_ucast_pkts': '52136476',
                                                                       'links': [       {       'in_nucast_pkts': '477',
                                                                                                'in_octets': '62495928412',
                                                                                                'in_ucast_pkts': '52136476',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14277',
                                                                                                'out_octets': '54397660066',
                                                                                                'out_ucast_pkts': '56952448',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14277',
                                                                       'out_octets': '54397660066',
                                                                       'out_ucast_pkts': '56952448',
                                                                       'poller_hash': 'QURUTjIzMTI1MDM3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427033',
                                                                                                                        'fiber_length': 9249,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDM3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU13',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTAzNw==',
                                         'serial_number': 'ADTN23125037',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427033'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74457',
                                                                       'in_octets': '18963024446',
                                                                       'in_ucast_pkts': '55212384',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '173927629920',
                                                                       'out_ucast_pkts': '131762938',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '173927629920',
                                                                       'in_ucast_pkts': '131762938',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '173927629920',
                                                                                                'in_ucast_pkts': '131762938',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74457',
                                                                                                'out_octets': '18963024446',
                                                                                                'out_ucast_pkts': '55212384',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74457',
                                                                       'out_octets': '18963024446',
                                                                       'out_ucast_pkts': '55212384',
                                                                       'poller_hash': 'QURUTjIzMTI0MDk5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232588',
                                                                                                                        'fiber_length': 6279,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MDk5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDA5OQ==',
                                         'serial_number': 'ADTN23124099',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232588'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11192',
                                                                       'in_octets': '1012604589',
                                                                       'in_ucast_pkts': '4080931',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '10986661054',
                                                                       'out_ucast_pkts': '8636314',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE3X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '10986661054',
                                                                       'in_ucast_pkts': '8636314',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '10986661054',
                                                                                                'in_ucast_pkts': '8636314',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11192',
                                                                                                'out_octets': '1012604589',
                                                                                                'out_ucast_pkts': '4080931',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11192',
                                                                       'out_octets': '1012604589',
                                                                       'out_ucast_pkts': '4080931',
                                                                       'poller_hash': 'QURUTjIzMTI0RTVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327029',
                                                                                                                        'fiber_length': 8775,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU1Qg==',
                                         'serial_number': 'ADTN23124E5B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327029'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-7.1.23.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x7-ONU23_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjNfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI1MThEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MThEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x7-ONU23',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE4RA==',
                                         'serial_number': 'ADTN2312518D',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10920',
                                                                       'in_octets': '3578492675',
                                                                       'in_ucast_pkts': '10367495',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '53634846440',
                                                                       'out_ucast_pkts': '38760531',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTE0X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '53634846440',
                                                                       'in_ucast_pkts': '38760531',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '53634846440',
                                                                                                'in_ucast_pkts': '38760531',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10920',
                                                                                                'out_octets': '3578492675',
                                                                                                'out_ucast_pkts': '10367495',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10920',
                                                                       'out_octets': '3578492675',
                                                                       'out_ucast_pkts': '10367495',
                                                                       'poller_hash': 'QURUTjIzMTI0QjdFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327029',
                                                                                                                        'fiber_length': 9207,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QjdFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEI3RQ==',
                                         'serial_number': 'ADTN23124B7E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.33.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14487',
                                                                       'in_octets': '31752714694',
                                                                       'in_ucast_pkts': '44450073',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU33_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '81359961853',
                                                                       'out_ucast_pkts': '65945504',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMzNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '81359961853',
                                                                       'in_ucast_pkts': '65945504',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '81359961853',
                                                                                                'in_ucast_pkts': '65945504',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14487',
                                                                                                'out_octets': '31752714694',
                                                                                                'out_ucast_pkts': '44450073',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14487',
                                                                       'out_octets': '31752714694',
                                                                       'out_ucast_pkts': '44450073',
                                                                       'poller_hash': 'QURUTjIzMTI0MDA4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427040',
                                                                                                                        'fiber_length': 5786,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MDA4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU33',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDAwOA==',
                                         'serial_number': 'ADTN23124008',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427040'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '8972',
                                                                       'in_octets': '1209998692',
                                                                       'in_ucast_pkts': '5042376',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '300',
                                                                       'out_octets': '17772443257',
                                                                       'out_ucast_pkts': '13503826',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '300',
                                                                       'in_octets': '17772443257',
                                                                       'in_ucast_pkts': '13503826',
                                                                       'links': [       {       'in_nucast_pkts': '300',
                                                                                                'in_octets': '17772443257',
                                                                                                'in_ucast_pkts': '13503826',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '8972',
                                                                                                'out_octets': '1209998692',
                                                                                                'out_ucast_pkts': '5042376',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '8972',
                                                                       'out_octets': '1209998692',
                                                                       'out_ucast_pkts': '5042376',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTcwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '268464',
                                                                                                                        'fiber_length': 3865,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTcwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE3MA==',
                                         'serial_number': 'ADTN23144A70',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '268464'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '105741',
                                                                       'in_octets': '125867370590',
                                                                       'in_ucast_pkts': '380855656',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3662',
                                                                       'out_octets': '1578961306528',
                                                                       'out_ucast_pkts': '1164173803',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMjBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3662',
                                                                       'in_octets': '1578961306528',
                                                                       'in_ucast_pkts': '1164173803',
                                                                       'links': [       {       'in_nucast_pkts': '3662',
                                                                                                'in_octets': '1578961306528',
                                                                                                'in_ucast_pkts': '1164173803',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '105741',
                                                                                                'out_octets': '125867370590',
                                                                                                'out_ucast_pkts': '380855656',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '105741',
                                                                       'out_octets': '125867370590',
                                                                       'out_ucast_pkts': '380855656',
                                                                       'poller_hash': 'QURUTjIzMTI0MjE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3147736',
                                                                                                                        'fiber_length': 9856,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MjE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDIxNA==',
                                         'serial_number': 'ADTN23124214',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '3147736'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14491',
                                                                       'in_octets': '5945748995',
                                                                       'in_ucast_pkts': '38403602',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '211846856617',
                                                                       'out_ucast_pkts': '151900586',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE4X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '211846856617',
                                                                       'in_ucast_pkts': '151900586',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '211846856617',
                                                                                                'in_ucast_pkts': '151900586',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14491',
                                                                                                'out_octets': '5945748995',
                                                                                                'out_ucast_pkts': '38403602',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14491',
                                                                       'out_octets': '5945748995',
                                                                       'out_ucast_pkts': '38403602',
                                                                       'poller_hash': 'QURUTjIzMTI0QzczIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 4666,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QzczIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU18',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEM3Mw==',
                                         'serial_number': 'ADTN23124C73',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.27.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74437',
                                                                       'in_octets': '79842049203',
                                                                       'in_ucast_pkts': '169807253',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU27_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2479',
                                                                       'out_octets': '564242287128',
                                                                       'out_ucast_pkts': '423924209',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjdfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2479',
                                                                       'in_octets': '564242287128',
                                                                       'in_ucast_pkts': '423924209',
                                                                       'links': [       {       'in_nucast_pkts': '2479',
                                                                                                'in_octets': '564242287128',
                                                                                                'in_ucast_pkts': '423924209',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74437',
                                                                                                'out_octets': '79842049203',
                                                                                                'out_ucast_pkts': '169807253',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74437',
                                                                       'out_octets': '79842049203',
                                                                       'out_ucast_pkts': '169807253',
                                                                       'poller_hash': 'QURUTjIzMTI0QzU1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232586',
                                                                                                                        'fiber_length': 4209,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QzU1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU27',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEM1NQ==',
                                         'serial_number': 'ADTN23124C55',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232586'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '22894',
                                                                       'in_octets': '34295336756',
                                                                       'in_ucast_pkts': '69203173',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '755',
                                                                       'out_octets': '137053450797',
                                                                       'out_ucast_pkts': '137839818',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTE5X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '755',
                                                                       'in_octets': '137053450797',
                                                                       'in_ucast_pkts': '137839818',
                                                                       'links': [       {       'in_nucast_pkts': '755',
                                                                                                'in_octets': '137053450797',
                                                                                                'in_ucast_pkts': '137839818',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '22894',
                                                                                                'out_octets': '34295336756',
                                                                                                'out_ucast_pkts': '69203173',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '22894',
                                                                       'out_octets': '34295336756',
                                                                       'out_ucast_pkts': '69203173',
                                                                       'poller_hash': 'QURUTjIzMTQ0RURGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '677935',
                                                                                                                        'fiber_length': 8136,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RURGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVERg==',
                                         'serial_number': 'ADTN23144EDF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '677935'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '86768',
                                                                       'in_octets': '43839947049',
                                                                       'in_ucast_pkts': '155302953',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2894',
                                                                       'out_octets': '549023046624',
                                                                       'out_ucast_pkts': '395156955',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMjBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2894',
                                                                       'in_octets': '549023046624',
                                                                       'in_ucast_pkts': '395156955',
                                                                       'links': [       {       'in_nucast_pkts': '2894',
                                                                                                'in_octets': '549023046624',
                                                                                                'in_ucast_pkts': '395156955',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '86768',
                                                                                                'out_octets': '43839947049',
                                                                                                'out_ucast_pkts': '155302953',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '86768',
                                                                       'out_octets': '43839947049',
                                                                       'out_ucast_pkts': '155302953',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUZGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2600437',
                                                                                                                        'fiber_length': 14568,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -25.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVGRg==',
                                         'serial_number': 'ADTN23144EFF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2600437'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75317',
                                                                       'in_octets': '26957671546',
                                                                       'in_ucast_pkts': '52874363',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6375',
                                                                       'out_octets': '88318159755',
                                                                       'out_ucast_pkts': '83722263',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMjFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6375',
                                                                       'in_octets': '88318159755',
                                                                       'in_ucast_pkts': '83722263',
                                                                       'links': [       {       'in_nucast_pkts': '6375',
                                                                                                'in_octets': '88318159755',
                                                                                                'in_ucast_pkts': '83722263',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75317',
                                                                                                'out_octets': '26957671546',
                                                                                                'out_ucast_pkts': '52874363',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75317',
                                                                       'out_octets': '26957671546',
                                                                       'out_ucast_pkts': '52874363',
                                                                       'poller_hash': 'QURUTjIzMTI0QTE4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2228296',
                                                                                                                        'fiber_length': 14439,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QTE4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEExOA==',
                                         'serial_number': 'ADTN23124A18',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2228296'},
                                 {       'interfaces': [       {       'admin_up': False,
                                                                       'id': 'onu-subscr-if-7.1.25.0.eth.1.phy',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 0,
                                                                       'name': 'IndianBayou-x7-ONU25_ETH_1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjVfRVRIXzE=',
                                                                       'type': 6},
                                                               {       'admin_up': False,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': False,
                                                                                                'out_nucast_pkts': '0',
                                                                                                'out_octets': '0',
                                                                                                'out_ucast_pkts': '0',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'QURUTjIzMTQ0RjAwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427029',
                                                                                                                        'fiber_length': 3323,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RjAwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU25',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEYwMA==',
                                         'serial_number': 'ADTN23144F00',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '68169',
                                                                       'in_octets': '84935880657',
                                                                       'in_ucast_pkts': '148044448',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5773',
                                                                       'out_octets': '475176904238',
                                                                       'out_ucast_pkts': '345196782',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTdfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5773',
                                                                       'in_octets': '475176904238',
                                                                       'in_ucast_pkts': '345196782',
                                                                       'links': [       {       'in_nucast_pkts': '5773',
                                                                                                'in_octets': '475176904238',
                                                                                                'in_ucast_pkts': '345196782',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '68169',
                                                                                                'out_octets': '84935880657',
                                                                                                'out_ucast_pkts': '148044448',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '68169',
                                                                       'out_octets': '84935880657',
                                                                       'out_ucast_pkts': '148044448',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUZCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2011665',
                                                                                                                        'fiber_length': 6816,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU17',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVGQg==',
                                         'serial_number': 'ADTN23144EFB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2011665'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '148421',
                                                                       'in_octets': '7943819348',
                                                                       'in_ucast_pkts': '14707772',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4830',
                                                                       'out_octets': '4078142804',
                                                                       'out_ucast_pkts': '8557943',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4830',
                                                                       'in_octets': '4078142804',
                                                                       'in_ucast_pkts': '8557943',
                                                                       'links': [       {       'in_nucast_pkts': '4830',
                                                                                                'in_octets': '4078142804',
                                                                                                'in_ucast_pkts': '8557943',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '148421',
                                                                                                'out_octets': '7943819348',
                                                                                                'out_ucast_pkts': '14707772',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '148421',
                                                                       'out_octets': '7943819348',
                                                                       'out_ucast_pkts': '14707772',
                                                                       'poller_hash': 'QURUTjIzMTI0NEJBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355629',
                                                                                                                        'fiber_length': 7484,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NEJBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDRCQQ==',
                                         'serial_number': 'ADTN231244BA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355629'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.28.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75672',
                                                                       'in_octets': '45888934802',
                                                                       'in_ucast_pkts': '130477397',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU28_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6384',
                                                                       'out_octets': '459567680266',
                                                                       'out_ucast_pkts': '327328544',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjhfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6384',
                                                                       'in_octets': '459567680266',
                                                                       'in_ucast_pkts': '327328544',
                                                                       'links': [       {       'in_nucast_pkts': '6384',
                                                                                                'in_octets': '459567680266',
                                                                                                'in_ucast_pkts': '327328544',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75672',
                                                                                                'out_octets': '45888934802',
                                                                                                'out_ucast_pkts': '130477397',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75672',
                                                                       'out_octets': '45888934802',
                                                                       'out_ucast_pkts': '130477397',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUUzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232576',
                                                                                                                        'fiber_length': 4287,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUUzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU28',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVFMw==',
                                         'serial_number': 'ADTN23144EE3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232576'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '48931',
                                                                       'in_octets': '436383774512',
                                                                       'in_ucast_pkts': '507681784',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1631',
                                                                       'out_octets': '796936082032',
                                                                       'out_ucast_pkts': '698475722',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1631',
                                                                       'in_octets': '796936082032',
                                                                       'in_ucast_pkts': '698475722',
                                                                       'links': [       {       'in_nucast_pkts': '1631',
                                                                                                'in_octets': '796936082032',
                                                                                                'in_ucast_pkts': '698475722',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '48931',
                                                                                                'out_octets': '436383774512',
                                                                                                'out_ucast_pkts': '507681784',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '48931',
                                                                       'out_octets': '436383774512',
                                                                       'out_ucast_pkts': '507681784',
                                                                       'poller_hash': 'QURUTjIzMTI0QTc2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1467132',
                                                                                                                        'fiber_length': 12765,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QTc2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEE3Ng==',
                                         'serial_number': 'ADTN23124A76',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1467132'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.25.0.eth.1.phy',
                                                                       'in_nucast_pkts': '7995507',
                                                                       'in_octets': '1858309971876',
                                                                       'in_ucast_pkts': '9243126879',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU25_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '153208',
                                                                       'out_octets': '28429412683572',
                                                                       'out_ucast_pkts': '21092874583',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '153208',
                                                                       'in_octets': '28429412683572',
                                                                       'in_ucast_pkts': '21092874583',
                                                                       'links': [       {       'in_nucast_pkts': '153208',
                                                                                                'in_octets': '28429412683572',
                                                                                                'in_ucast_pkts': '21092874583',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '7995507',
                                                                                                'out_octets': '1858309971876',
                                                                                                'out_ucast_pkts': '9243126879',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '7995507',
                                                                       'out_octets': '1858309971876',
                                                                       'out_ucast_pkts': '9243126879',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTg2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355630',
                                                                                                                        'fiber_length': 7783,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTg2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU25',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE4Ng==',
                                         'serial_number': 'ADTN23144A86',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355630'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '2312',
                                                                       'in_octets': '298049450',
                                                                       'in_ucast_pkts': '876509',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '78',
                                                                       'out_octets': '2928972974',
                                                                       'out_ucast_pkts': '2279665',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '78',
                                                                       'in_octets': '2928972974',
                                                                       'in_ucast_pkts': '2279665',
                                                                       'links': [       {       'in_nucast_pkts': '78',
                                                                                                'in_octets': '2928972974',
                                                                                                'in_ucast_pkts': '2279665',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '2312',
                                                                                                'out_octets': '298049450',
                                                                                                'out_ucast_pkts': '876509',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2312',
                                                                       'out_octets': '298049450',
                                                                       'out_ucast_pkts': '876509',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '68896',
                                                                                                                        'fiber_length': 700,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVFMA==',
                                         'serial_number': 'ADTN23144EE0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '68896'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14254',
                                                                       'in_octets': '28850937908',
                                                                       'in_ucast_pkts': '35236369',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '76317764218',
                                                                       'out_ucast_pkts': '61085997',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '76317764218',
                                                                       'in_ucast_pkts': '61085997',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '76317764218',
                                                                                                'in_ucast_pkts': '61085997',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14254',
                                                                                                'out_octets': '28850937908',
                                                                                                'out_ucast_pkts': '35236369',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14254',
                                                                       'out_octets': '28850937908',
                                                                       'out_ucast_pkts': '35236369',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUM4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427028',
                                                                                                                        'fiber_length': 489,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUM4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVDOA==',
                                         'serial_number': 'ADTN23144EC8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '37592',
                                                                       'in_octets': '12713140314',
                                                                       'in_ucast_pkts': '33577218',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3182',
                                                                       'out_octets': '98263455711',
                                                                       'out_ucast_pkts': '84012258',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3182',
                                                                       'in_octets': '98263455711',
                                                                       'in_ucast_pkts': '84012258',
                                                                       'links': [       {       'in_nucast_pkts': '3182',
                                                                                                'in_octets': '98263455711',
                                                                                                'in_ucast_pkts': '84012258',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '37592',
                                                                                                'out_octets': '12713140314',
                                                                                                'out_ucast_pkts': '33577218',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '37592',
                                                                       'out_octets': '12713140314',
                                                                       'out_ucast_pkts': '33577218',
                                                                       'poller_hash': 'QURUTjIzMTQ0N0FDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1108815',
                                                                                                                        'fiber_length': 6748,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0N0FDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDdBQw==',
                                         'serial_number': 'ADTN231447AC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1108815'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '75672',
                                                                       'in_octets': '30042834672',
                                                                       'in_ucast_pkts': '143586465',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x4-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6407',
                                                                       'out_octets': '405418036092',
                                                                       'out_ucast_pkts': '371197083',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDQtT05VMjBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6407',
                                                                       'in_octets': '405418036092',
                                                                       'in_ucast_pkts': '371197083',
                                                                       'links': [       {       'in_nucast_pkts': '6407',
                                                                                                'in_octets': '405418036092',
                                                                                                'in_ucast_pkts': '371197083',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '75672',
                                                                                                'out_octets': '30042834672',
                                                                                                'out_ucast_pkts': '143586465',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '75672',
                                                                       'out_octets': '30042834672',
                                                                       'out_ucast_pkts': '143586465',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUZEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232586',
                                                                                                                        'fiber_length': 6232,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x4-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVGRA==',
                                         'serial_number': 'ADTN23144EFD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232586'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.30.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14548',
                                                                       'in_octets': '16103184483',
                                                                       'in_ucast_pkts': '24497332',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU30_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '475',
                                                                       'out_octets': '49677874682',
                                                                       'out_ucast_pkts': '44710176',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTMwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '475',
                                                                       'in_octets': '49677874682',
                                                                       'in_ucast_pkts': '44710176',
                                                                       'links': [       {       'in_nucast_pkts': '475',
                                                                                                'in_octets': '49677874682',
                                                                                                'in_ucast_pkts': '44710176',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14548',
                                                                                                'out_octets': '16103184483',
                                                                                                'out_ucast_pkts': '24497332',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14548',
                                                                       'out_octets': '16103184483',
                                                                       'out_ucast_pkts': '24497332',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUQyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '423780',
                                                                                                                        'fiber_length': 4439,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUQyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU30',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVEMg==',
                                         'serial_number': 'ADTN23144ED2',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '423780'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.34.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14260',
                                                                       'in_octets': '17809587612',
                                                                       'in_ucast_pkts': '67320477',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x8-ONU34_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '270026241546',
                                                                       'out_ucast_pkts': '199676509',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDgtT05VMzRfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '270026241546',
                                                                       'in_ucast_pkts': '199676509',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '270026241546',
                                                                                                'in_ucast_pkts': '199676509',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14260',
                                                                                                'out_octets': '17809587612',
                                                                                                'out_ucast_pkts': '67320477',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14260',
                                                                       'out_octets': '17809587612',
                                                                       'out_ucast_pkts': '67320477',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUQzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427025',
                                                                                                                        'fiber_length': 5736,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUQzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x8-ONU34',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVEMw==',
                                         'serial_number': 'ADTN23144ED3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427025'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14481',
                                                                       'in_octets': '5086119669',
                                                                       'in_ucast_pkts': '22447222',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1224',
                                                                       'out_octets': '99081591641',
                                                                       'out_ucast_pkts': '78561562',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTIwX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1224',
                                                                       'in_octets': '99081591641',
                                                                       'in_ucast_pkts': '78561562',
                                                                       'links': [       {       'in_nucast_pkts': '1224',
                                                                                                'in_octets': '99081591641',
                                                                                                'in_ucast_pkts': '78561562',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14481',
                                                                                                'out_octets': '5086119669',
                                                                                                'out_ucast_pkts': '22447222',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14481',
                                                                       'out_octets': '5086119669',
                                                                       'out_ucast_pkts': '22447222',
                                                                       'poller_hash': 'QURUTjIzMTQ0MjhBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427031',
                                                                                                                        'fiber_length': 7421,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0MjhBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDI4QQ==',
                                         'serial_number': 'ADTN2314428A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427031'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.25.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14249',
                                                                       'in_octets': '4157814749',
                                                                       'in_ucast_pkts': '19666598',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU25_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '60994460314',
                                                                       'out_ucast_pkts': '44172062',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjVfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '60994460314',
                                                                       'in_ucast_pkts': '44172062',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '60994460314',
                                                                                                'in_ucast_pkts': '44172062',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14249',
                                                                                                'out_octets': '4157814749',
                                                                                                'out_ucast_pkts': '19666598',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14249',
                                                                       'out_octets': '4157814749',
                                                                       'out_ucast_pkts': '19666598',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzE3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427028',
                                                                                                                        'fiber_length': 716,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzE3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU25',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEMxNw==',
                                         'serial_number': 'ADTN23144C17',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.24.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14492',
                                                                       'in_octets': '9889045754',
                                                                       'in_ucast_pkts': '57345105',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU24_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1227',
                                                                       'out_octets': '242042243181',
                                                                       'out_ucast_pkts': '170513907',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjRfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1227',
                                                                       'in_octets': '242042243181',
                                                                       'in_ucast_pkts': '170513907',
                                                                       'links': [       {       'in_nucast_pkts': '1227',
                                                                                                'in_octets': '242042243181',
                                                                                                'in_ucast_pkts': '170513907',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14492',
                                                                                                'out_octets': '9889045754',
                                                                                                'out_ucast_pkts': '57345105',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14492',
                                                                       'out_octets': '9889045754',
                                                                       'out_ucast_pkts': '57345105',
                                                                       'poller_hash': 'QURUTjIzMTQ0NjlFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427040',
                                                                                                                        'fiber_length': 6818,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0NjlFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU24',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDY5RQ==',
                                         'serial_number': 'ADTN2314469E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427040'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-5.1.18.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x5-ONU18_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMThfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0QjI0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjI0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x5-ONU18',
                                         'offline_reason': 'OFFLINE_REASON_POWER_OFF',
                                         'offline_reason_detail': 'A local power '
                                                                  'outage has occurred '
                                                                  'for ONU "onu '
                                                                  '5.1.18"(ADTN23144B24).',
                                         'offline_time': '2025-08-25T19:05:47Z',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIyNA==',
                                         'serial_number': 'ADTN23144B24',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '16418',
                                                                       'in_octets': '8947474311',
                                                                       'in_ucast_pkts': '11200592',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '548',
                                                                       'out_octets': '30754950510',
                                                                       'out_ucast_pkts': '24258646',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VMjFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '548',
                                                                       'in_octets': '30754950510',
                                                                       'in_ucast_pkts': '24258646',
                                                                       'links': [       {       'in_nucast_pkts': '548',
                                                                                                'in_octets': '30754950510',
                                                                                                'in_ucast_pkts': '24258646',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '16418',
                                                                                                'out_octets': '8947474311',
                                                                                                'out_ucast_pkts': '11200592',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '16418',
                                                                       'out_octets': '8947474311',
                                                                       'out_ucast_pkts': '11200592',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzRFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '491760',
                                                                                                                        'fiber_length': 10106,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzRFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEM0RQ==',
                                         'serial_number': 'ADTN23144C4E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '491760'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14253',
                                                                       'in_octets': '5439487007',
                                                                       'in_ucast_pkts': '7838907',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '22414068884',
                                                                       'out_ucast_pkts': '19152768',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '22414068884',
                                                                       'in_ucast_pkts': '19152768',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '22414068884',
                                                                                                'in_ucast_pkts': '19152768',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14253',
                                                                                                'out_octets': '5439487007',
                                                                                                'out_ucast_pkts': '7838907',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14253',
                                                                       'out_octets': '5439487007',
                                                                       'out_ucast_pkts': '7838907',
                                                                       'poller_hash': 'QURUTjIzMTQ0QzhCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 8911,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QzhCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU9',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEM4Qg==',
                                         'serial_number': 'ADTN23144C8B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.20.0.eth.1.phy',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU20_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMjBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '0',
                                                                                                'out_octets': '0',
                                                                                                'out_ucast_pkts': '0',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'QURUTjIzMTQ1MTIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11998',
                                                                                                                        'fiber_length': 5781,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ1MTIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU20',
                                         'poller_hash': 'b250IEFEVE4yMzE0NTEyMA==',
                                         'serial_number': 'ADTN23145120',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '11998'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10918',
                                                                       'in_octets': '4741021242',
                                                                       'in_ucast_pkts': '40561298',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '113233366598',
                                                                       'out_ucast_pkts': '79471298',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTE5X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '113233366598',
                                                                       'in_ucast_pkts': '79471298',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '113233366598',
                                                                                                'in_ucast_pkts': '79471298',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10918',
                                                                                                'out_octets': '4741021242',
                                                                                                'out_ucast_pkts': '40561298',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10918',
                                                                       'out_octets': '4741021242',
                                                                       'out_ucast_pkts': '40561298',
                                                                       'poller_hash': 'QURUTjIzMTI1MDZDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327028',
                                                                                                                        'fiber_length': 6899,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDZDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU19',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTA2Qw==',
                                         'serial_number': 'ADTN2312506C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327028'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '178694',
                                                                       'in_octets': '110469006288',
                                                                       'in_ucast_pkts': '344223166',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5951',
                                                                       'out_octets': '929286519505',
                                                                       'out_ucast_pkts': '686797219',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMjJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5951',
                                                                       'in_octets': '929286519505',
                                                                       'in_ucast_pkts': '686797219',
                                                                       'links': [       {       'in_nucast_pkts': '5951',
                                                                                                'in_octets': '929286519505',
                                                                                                'in_ucast_pkts': '686797219',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '178694',
                                                                                                'out_octets': '110469006288',
                                                                                                'out_ucast_pkts': '344223166',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '178694',
                                                                       'out_octets': '110469006288',
                                                                       'out_ucast_pkts': '344223166',
                                                                       'poller_hash': 'QURUTjIzMTI0QjNGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355594',
                                                                                                                        'fiber_length': 13633,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QjNGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU22',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEIzRg==',
                                         'serial_number': 'ADTN23124B3F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355594'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14252',
                                                                       'in_octets': '14656916626',
                                                                       'in_ucast_pkts': '30525972',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '231750491985',
                                                                       'out_ucast_pkts': '163367025',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '231750491985',
                                                                       'in_ucast_pkts': '163367025',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '231750491985',
                                                                                                'in_ucast_pkts': '163367025',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14252',
                                                                                                'out_octets': '14656916626',
                                                                                                'out_ucast_pkts': '30525972',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14252',
                                                                       'out_octets': '14656916626',
                                                                       'out_ucast_pkts': '30525972',
                                                                       'poller_hash': 'QURUTjIzMTI0NTEwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427030',
                                                                                                                        'fiber_length': 313,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NTEwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDUxMA==',
                                         'serial_number': 'ADTN23124510',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427030'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-5.1.19.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x5-ONU19_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTlfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0QjU4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjU4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x5-ONU19',
                                         'offline_reason': 'OFFLINE_REASON_POWER_OFF',
                                         'offline_reason_detail': 'A local power '
                                                                  'outage has occurred '
                                                                  'for ONU "onu '
                                                                  '5.1.19"(ADTN23144B58).',
                                         'offline_time': '2025-07-16T16:31:45Z',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEI1OA==',
                                         'serial_number': 'ADTN23144B58',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.23.0.eth.1.phy',
                                                                       'in_nucast_pkts': '26797',
                                                                       'in_octets': '84175078455',
                                                                       'in_ucast_pkts': '173298172',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU23_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2272',
                                                                       'out_octets': '423674434887',
                                                                       'out_ucast_pkts': '315344278',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMjNfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2272',
                                                                       'in_octets': '423674434887',
                                                                       'in_ucast_pkts': '315344278',
                                                                       'links': [       {       'in_nucast_pkts': '2272',
                                                                                                'in_octets': '423674434887',
                                                                                                'in_ucast_pkts': '315344278',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '26797',
                                                                                                'out_octets': '84175078455',
                                                                                                'out_ucast_pkts': '173298172',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '26797',
                                                                       'out_octets': '84175078455',
                                                                       'out_ucast_pkts': '173298172',
                                                                       'poller_hash': 'QURUTjIzMTI0QkE1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '789241',
                                                                                                                        'fiber_length': 14906,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QkE1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU23',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEJBNQ==',
                                         'serial_number': 'ADTN23124BA5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '789241'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14258',
                                                                       'in_octets': '5556961097',
                                                                       'in_ucast_pkts': '44970447',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '137130796457',
                                                                       'out_ucast_pkts': '99273180',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '137130796457',
                                                                       'in_ucast_pkts': '99273180',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '137130796457',
                                                                                                'in_ucast_pkts': '99273180',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14258',
                                                                                                'out_octets': '5556961097',
                                                                                                'out_ucast_pkts': '44970447',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14258',
                                                                       'out_octets': '5556961097',
                                                                       'out_ucast_pkts': '44970447',
                                                                       'poller_hash': 'QURUTjIzMTI0N0ExIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427039',
                                                                                                                        'fiber_length': 3094,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0N0ExIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDdBMQ==',
                                         'serial_number': 'ADTN231247A1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427039'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.15.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10920',
                                                                       'in_octets': '12111820901',
                                                                       'in_ucast_pkts': '34746453',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU15_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '138901108970',
                                                                       'out_ucast_pkts': '106039018',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTE1X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '138901108970',
                                                                       'in_ucast_pkts': '106039018',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '138901108970',
                                                                                                'in_ucast_pkts': '106039018',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10920',
                                                                                                'out_octets': '12111820901',
                                                                                                'out_ucast_pkts': '34746453',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10920',
                                                                       'out_octets': '12111820901',
                                                                       'out_ucast_pkts': '34746453',
                                                                       'poller_hash': 'QURUTjIzMTI0OTY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327029',
                                                                                                                        'fiber_length': 9245,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OTY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU15',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDk2Ng==',
                                         'serial_number': 'ADTN23124966',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '327029'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '66667',
                                                                       'in_octets': '9328434308',
                                                                       'in_ucast_pkts': '80529385',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2222',
                                                                       'out_octets': '255358029288',
                                                                       'out_ucast_pkts': '176691746',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2222',
                                                                       'in_octets': '255358029288',
                                                                       'in_ucast_pkts': '176691746',
                                                                       'links': [       {       'in_nucast_pkts': '2222',
                                                                                                'in_octets': '255358029288',
                                                                                                'in_ucast_pkts': '176691746',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '66667',
                                                                                                'out_octets': '9328434308',
                                                                                                'out_ucast_pkts': '80529385',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '66667',
                                                                       'out_octets': '9328434308',
                                                                       'out_ucast_pkts': '80529385',
                                                                       'poller_hash': 'QURUTjIzMTI0NjBBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1999223',
                                                                                                                        'fiber_length': 4437,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NjBBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDYwQQ==',
                                         'serial_number': 'ADTN2312460A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1999223'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.26.0.eth.1.phy',
                                                                       'in_nucast_pkts': '84975',
                                                                       'in_octets': '45699713263',
                                                                       'in_ucast_pkts': '190445833',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x9-ONU26_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2784',
                                                                       'out_octets': '579520275565',
                                                                       'out_ucast_pkts': '418521378',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDktT05VMjZfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2784',
                                                                       'in_octets': '579520275565',
                                                                       'in_ucast_pkts': '418521378',
                                                                       'links': [       {       'in_nucast_pkts': '2784',
                                                                                                'in_octets': '579520275565',
                                                                                                'in_ucast_pkts': '418521378',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '84975',
                                                                                                'out_octets': '45699713263',
                                                                                                'out_ucast_pkts': '190445833',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '84975',
                                                                       'out_octets': '45699713263',
                                                                       'out_ucast_pkts': '190445833',
                                                                       'poller_hash': 'QURUTjIzMTI1MTM1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2503969',
                                                                                                                        'fiber_length': 6346,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTM1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x9-ONU26',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEzNQ==',
                                         'serial_number': 'ADTN23125135',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2503969'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.27.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14260',
                                                                       'in_octets': '4745824547',
                                                                       'in_ucast_pkts': '28323993',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU27_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '91040367405',
                                                                       'out_ucast_pkts': '63611691',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjdfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '91040367405',
                                                                       'in_ucast_pkts': '63611691',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '91040367405',
                                                                                                'in_ucast_pkts': '63611691',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14260',
                                                                                                'out_octets': '4745824547',
                                                                                                'out_ucast_pkts': '28323993',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14260',
                                                                       'out_octets': '4745824547',
                                                                       'out_ucast_pkts': '28323993',
                                                                       'poller_hash': 'QURUTjIzMTI0QzZBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427039',
                                                                                                                        'fiber_length': 3062,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QzZBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU27',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEM2QQ==',
                                         'serial_number': 'ADTN23124C6A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427039'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.29.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74439',
                                                                       'in_octets': '8471193592',
                                                                       'in_ucast_pkts': '41579304',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU29_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2477',
                                                                       'out_octets': '212613192305',
                                                                       'out_ucast_pkts': '160210824',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMjlfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2477',
                                                                       'in_octets': '212613192305',
                                                                       'in_ucast_pkts': '160210824',
                                                                       'links': [       {       'in_nucast_pkts': '2477',
                                                                                                'in_octets': '212613192305',
                                                                                                'in_ucast_pkts': '160210824',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74439',
                                                                                                'out_octets': '8471193592',
                                                                                                'out_ucast_pkts': '41579304',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74439',
                                                                       'out_octets': '8471193592',
                                                                       'out_ucast_pkts': '41579304',
                                                                       'poller_hash': 'QURUTjI0MzYzQUU3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232594',
                                                                                                                        'fiber_length': 3543,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQUU3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU29',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0FFNw==',
                                         'serial_number': 'ADTN24363AE7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '2232594'},
                                 {       'interfaces': [       {       'admin_up': False,
                                                                       'id': 'onu-subscr-if-5.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 0,
                                                                       'name': 'IndianBayou-x5-ONU21_ETH_1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMjFfRVRIXzE=',
                                                                       'type': 6},
                                                               {       'admin_up': False,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': False,
                                                                                                'out_nucast_pkts': '0',
                                                                                                'out_octets': '0',
                                                                                                'out_ucast_pkts': '0',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'QURUTjI0MzYzRDNBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1550263',
                                                                                                                        'fiber_length': 6741,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRDNBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU21',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0QzQQ==',
                                         'serial_number': 'ADTN24363D3A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1550263'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '179550',
                                                                       'in_octets': '198479562751',
                                                                       'in_ucast_pkts': '1057037287',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5953',
                                                                       'out_octets': '5820242453077',
                                                                       'out_ucast_pkts': '4428697131',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VMTNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '5953',
                                                                       'in_octets': '5820242453077',
                                                                       'in_ucast_pkts': '4428697131',
                                                                       'links': [       {       'in_nucast_pkts': '5953',
                                                                                                'in_octets': '5820242453077',
                                                                                                'in_ucast_pkts': '4428697131',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '179550',
                                                                                                'out_octets': '198479562751',
                                                                                                'out_ucast_pkts': '1057037287',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '179550',
                                                                       'out_octets': '198479562751',
                                                                       'out_ucast_pkts': '1057037287',
                                                                       'poller_hash': 'QURUTjI0MzYzQ0M2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5355594',
                                                                                                                        'fiber_length': 13230,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQ0M2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU13',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0NDNg==',
                                         'serial_number': 'ADTN24363CC6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '5355594'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.32.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14257',
                                                                       'in_octets': '3901163862',
                                                                       'in_ucast_pkts': '12196554',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU32_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '16966615768',
                                                                       'out_ucast_pkts': '12577131',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTMyX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '16966615768',
                                                                       'in_ucast_pkts': '12577131',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '16966615768',
                                                                                                'in_ucast_pkts': '12577131',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14257',
                                                                                                'out_octets': '3901163862',
                                                                                                'out_ucast_pkts': '12196554',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14257',
                                                                       'out_octets': '3901163862',
                                                                       'out_ucast_pkts': '12196554',
                                                                       'poller_hash': 'QURUTjI0MzYzRDREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427039',
                                                                                                                        'fiber_length': 3928,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRDREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU32',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0Q0RA==',
                                         'serial_number': 'ADTN24363D4D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427039'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.21.0.eth.1.phy',
                                                                       'in_nucast_pkts': '113720',
                                                                       'in_octets': '50182795525',
                                                                       'in_ucast_pkts': '367529678',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU21_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3738',
                                                                       'out_octets': '1355404021797',
                                                                       'out_ucast_pkts': '954516758',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTIxX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3738',
                                                                       'in_octets': '1355404021797',
                                                                       'in_ucast_pkts': '954516758',
                                                                       'links': [       {       'in_nucast_pkts': '3738',
                                                                                                'in_octets': '1355404021797',
                                                                                                'in_ucast_pkts': '954516758',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '113720',
                                                                                                'out_octets': '50182795525',
                                                                                                'out_ucast_pkts': '367529678',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '113720',
                                                                       'out_octets': '50182795525',
                                                                       'out_ucast_pkts': '367529678',
                                                                       'poller_hash': 'QURUTjIzMTI0MUEyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3364576',
                                                                                                                        'fiber_length': 8116,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MUEyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU21',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDFBMg==',
                                         'serial_number': 'ADTN231241A2',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_1-31',
                                         'uptime': '3364576'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.16.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10921',
                                                                       'in_octets': '5131827869',
                                                                       'in_ucast_pkts': '7642544',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU16_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '31461333787',
                                                                       'out_ucast_pkts': '24629401',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTE2X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '31461333787',
                                                                       'in_ucast_pkts': '24629401',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '31461333787',
                                                                                                'in_ucast_pkts': '24629401',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10921',
                                                                                                'out_octets': '5131827869',
                                                                                                'out_ucast_pkts': '7642544',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10921',
                                                                       'out_octets': '5131827869',
                                                                       'out_ucast_pkts': '7642544',
                                                                       'poller_hash': 'QURUTjI0MzYzREI5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327033',
                                                                                                                        'fiber_length': 9079,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzREI5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU16',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0RCOQ==',
                                         'serial_number': 'ADTN24363DB9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '327033'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.28.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14413',
                                                                       'in_octets': '3836087366',
                                                                       'in_ucast_pkts': '47521683',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU28_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '150917148362',
                                                                       'out_ucast_pkts': '100852685',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjhfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '150917148362',
                                                                       'in_ucast_pkts': '100852685',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '150917148362',
                                                                                                'in_ucast_pkts': '100852685',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14413',
                                                                                                'out_octets': '3836087366',
                                                                                                'out_ucast_pkts': '47521683',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14413',
                                                                       'out_octets': '3836087366',
                                                                       'out_ucast_pkts': '47521683',
                                                                       'poller_hash': 'QURUTjI0MzYzREM5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427043',
                                                                                                                        'fiber_length': 3347,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzREM5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU28',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0RDOQ==',
                                         'serial_number': 'ADTN24363DC9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '427043'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.18.0.eth.1.phy',
                                                                       'in_nucast_pkts': '10923',
                                                                       'in_octets': '2414262555',
                                                                       'in_ucast_pkts': '8667815',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU18_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '365',
                                                                       'out_octets': '45045431911',
                                                                       'out_ucast_pkts': '32186109',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE4X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '365',
                                                                       'in_octets': '45045431911',
                                                                       'in_ucast_pkts': '32186109',
                                                                       'links': [       {       'in_nucast_pkts': '365',
                                                                                                'in_octets': '45045431911',
                                                                                                'in_ucast_pkts': '32186109',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '10923',
                                                                                                'out_octets': '2414262555',
                                                                                                'out_ucast_pkts': '8667815',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '10923',
                                                                       'out_octets': '2414262555',
                                                                       'out_ucast_pkts': '8667815',
                                                                       'poller_hash': 'QURUTjI0MzYzQkJBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327033',
                                                                                                                        'fiber_length': 8469,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQkJBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU18',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0JCQQ==',
                                         'serial_number': 'ADTN24363BBA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '327033'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '16436',
                                                                       'in_octets': '7606830915',
                                                                       'in_ucast_pkts': '17153075',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x6-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '549',
                                                                       'out_octets': '59503336499',
                                                                       'out_ucast_pkts': '44103819',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDYtT05VNl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '549',
                                                                       'in_octets': '59503336499',
                                                                       'in_ucast_pkts': '44103819',
                                                                       'links': [       {       'in_nucast_pkts': '549',
                                                                                                'in_octets': '59503336499',
                                                                                                'in_ucast_pkts': '44103819',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '16436',
                                                                                                'out_octets': '7606830915',
                                                                                                'out_ucast_pkts': '17153075',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '16436',
                                                                       'out_octets': '7606830915',
                                                                       'out_ucast_pkts': '17153075',
                                                                       'poller_hash': 'QURUTjI0MzE0NEZCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '492326',
                                                                                                                        'fiber_length': 10046,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0NEZCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x6-ONU6',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDRGQg==',
                                         'serial_number': 'ADTN243144FB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '492326'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.30.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74439',
                                                                       'in_octets': '16883087413',
                                                                       'in_ucast_pkts': '15576212',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU30_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2477',
                                                                       'out_octets': '19789729593',
                                                                       'out_ucast_pkts': '18624355',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMzBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2477',
                                                                       'in_octets': '19789729593',
                                                                       'in_ucast_pkts': '18624355',
                                                                       'links': [       {       'in_nucast_pkts': '2477',
                                                                                                'in_octets': '19789729593',
                                                                                                'in_ucast_pkts': '18624355',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74439',
                                                                                                'out_octets': '16883087413',
                                                                                                'out_ucast_pkts': '15576212',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74439',
                                                                       'out_octets': '16883087413',
                                                                       'out_ucast_pkts': '15576212',
                                                                       'poller_hash': 'QURUTjI0Mzg5NkY5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232590',
                                                                                                                        'fiber_length': 3918,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0Mzg5NkY5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU30',
                                         'poller_hash': 'b250IEFEVE4yNDM4OTZGOQ==',
                                         'serial_number': 'ADTN243896F9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '2232590'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.28.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14253',
                                                                       'in_octets': '6706964484',
                                                                       'in_ucast_pkts': '58361129',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x1-ONU28_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '283054358159',
                                                                       'out_ucast_pkts': '193514948',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjhfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '283054358159',
                                                                       'in_ucast_pkts': '193514948',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '283054358159',
                                                                                                'in_ucast_pkts': '193514948',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14253',
                                                                                                'out_octets': '6706964484',
                                                                                                'out_ucast_pkts': '58361129',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14253',
                                                                       'out_octets': '6706964484',
                                                                       'out_ucast_pkts': '58361129',
                                                                       'poller_hash': 'QURUTjI0MzE0NzY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427035',
                                                                                                                        'fiber_length': 859,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0NzY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x1-ONU28',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDc2Ng==',
                                         'serial_number': 'ADTN24314766',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '427035'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-7.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '12315',
                                                                       'in_octets': '24721684604',
                                                                       'in_ucast_pkts': '136340685',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x7-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '411',
                                                                       'out_octets': '411841066531',
                                                                       'out_ucast_pkts': '304654174',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDctT05VMjJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '411',
                                                                       'in_octets': '411841066531',
                                                                       'in_ucast_pkts': '304654174',
                                                                       'links': [       {       'in_nucast_pkts': '411',
                                                                                                'in_octets': '411841066531',
                                                                                                'in_ucast_pkts': '304654174',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '12315',
                                                                                                'out_octets': '24721684604',
                                                                                                'out_ucast_pkts': '136340685',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC84'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '12315',
                                                                       'out_octets': '24721684604',
                                                                       'out_ucast_pkts': '136340685',
                                                                       'poller_hash': 'QURUTjI0MzE0MzE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '368685',
                                                                                                                        'fiber_length': 3270,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MzE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x7-ONU22',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDMxNA==',
                                         'serial_number': 'ADTN24314314',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '368685'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '77372',
                                                                       'in_octets': '173473435611',
                                                                       'in_ucast_pkts': '1165260072',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6395',
                                                                       'out_octets': '4284904575408',
                                                                       'out_ucast_pkts': '2972646601',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '6395',
                                                                       'in_octets': '4284904575408',
                                                                       'in_ucast_pkts': '2972646601',
                                                                       'links': [       {       'in_nucast_pkts': '6395',
                                                                                                'in_octets': '4284904575408',
                                                                                                'in_ucast_pkts': '2972646601',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '77372',
                                                                                                'out_octets': '173473435611',
                                                                                                'out_ucast_pkts': '1165260072',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '77372',
                                                                       'out_octets': '173473435611',
                                                                       'out_ucast_pkts': '1165260072',
                                                                       'poller_hash': 'QURUTjI0MzE0MTk1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232592',
                                                                                                                        'fiber_length': 6741,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MTk1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU10',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDE5NQ==',
                                         'serial_number': 'ADTN24314195',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '2232592'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.31.0.eth.1.phy',
                                                                       'in_nucast_pkts': '258',
                                                                       'in_octets': '9643762835',
                                                                       'in_ucast_pkts': '55490510',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x11-ONU31_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1225',
                                                                       'out_octets': '203171251388',
                                                                       'out_ucast_pkts': '150409881',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDExLU9OVTMxX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1225',
                                                                       'in_octets': '203171251388',
                                                                       'in_ucast_pkts': '150409881',
                                                                       'links': [       {       'in_nucast_pkts': '1225',
                                                                                                'in_octets': '203171251388',
                                                                                                'in_ucast_pkts': '150409881',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '258',
                                                                                                'out_octets': '9643762835',
                                                                                                'out_ucast_pkts': '55490510',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '258',
                                                                       'out_octets': '9643762835',
                                                                       'out_ucast_pkts': '55490510',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjIwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427033',
                                                                                                                        'fiber_length': 5562,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjIwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x11-ONU31',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIyMA==',
                                         'serial_number': 'ADTN23144B20',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '427033'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74456',
                                                                       'in_octets': '32172783742',
                                                                       'in_ucast_pkts': '255295421',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x5-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2482',
                                                                       'out_octets': '735907349926',
                                                                       'out_ucast_pkts': '516063150',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDUtT05VMjJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2482',
                                                                       'in_octets': '735907349926',
                                                                       'in_ucast_pkts': '516063150',
                                                                       'links': [       {       'in_nucast_pkts': '2482',
                                                                                                'in_octets': '735907349926',
                                                                                                'in_ucast_pkts': '516063150',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74456',
                                                                                                'out_octets': '32172783742',
                                                                                                'out_ucast_pkts': '255295421',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74456',
                                                                       'out_octets': '32172783742',
                                                                       'out_ucast_pkts': '255295421',
                                                                       'poller_hash': 'QURUTjI0MzE0MUEzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2232592',
                                                                                                                        'fiber_length': 6541,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MUEzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x5-ONU22',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDFBMw==',
                                         'serial_number': 'ADTN243141A3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '2232592'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.19.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11097',
                                                                       'in_octets': '1764676337',
                                                                       'in_ucast_pkts': '7642458',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x12-ONU19_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '938',
                                                                       'out_octets': '19920110767',
                                                                       'out_ucast_pkts': '16158235',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEyLU9OVTE5X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '938',
                                                                       'in_octets': '19920110767',
                                                                       'in_ucast_pkts': '16158235',
                                                                       'links': [       {       'in_nucast_pkts': '938',
                                                                                                'in_octets': '19920110767',
                                                                                                'in_ucast_pkts': '16158235',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11097',
                                                                                                'out_octets': '1764676337',
                                                                                                'out_ucast_pkts': '7642458',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11097',
                                                                       'out_octets': '1764676337',
                                                                       'out_ucast_pkts': '7642458',
                                                                       'poller_hash': 'QURUTjI0MzE0MUFDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '327033',
                                                                                                                        'fiber_length': 9546,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MUFDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x12-ONU19',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDFBQw==',
                                         'serial_number': 'ADTN243141AC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '327033'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-1.1.27.0.eth.1.phy',
                                                                       'name': 'IndianBayou-x1-ONU27_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEtT05VMjdfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjI0MzYzQjZCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQjZCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'IndianBayou-x1-ONU27',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '1.1.27"(ADTN24363B6B) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/1.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-07-07T20:35:31Z',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0I2Qg==',
                                         'serial_number': 'ADTN24363B6B',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.31.0.eth.1.phy',
                                                                       'in_nucast_pkts': '53974',
                                                                       'in_octets': '35067790418',
                                                                       'in_ucast_pkts': '83574139',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x3-ONU31_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1803',
                                                                       'out_octets': '555199089656',
                                                                       'out_ucast_pkts': '398797477',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDMtT05VMzFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1803',
                                                                       'in_octets': '555199089656',
                                                                       'in_ucast_pkts': '398797477',
                                                                       'links': [       {       'in_nucast_pkts': '1803',
                                                                                                'in_octets': '555199089656',
                                                                                                'in_ucast_pkts': '398797477',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '53974',
                                                                                                'out_octets': '35067790418',
                                                                                                'out_ucast_pkts': '83574139',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '53974',
                                                                       'out_octets': '35067790418',
                                                                       'out_ucast_pkts': '83574139',
                                                                       'poller_hash': 'QURUTjI0MzYzQzQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1617919',
                                                                                                                        'fiber_length': 4116,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x3-ONU31',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M0OQ==',
                                         'serial_number': 'ADTN24363C49',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '1617919'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.22.0.eth.1.phy',
                                                                       'in_nucast_pkts': '14609',
                                                                       'in_octets': '55959688032',
                                                                       'in_ucast_pkts': '63282989',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x10-ONU22_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '134362196864',
                                                                       'out_ucast_pkts': '112053823',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEwLU9OVTIyX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '134362196864',
                                                                       'in_ucast_pkts': '112053823',
                                                                       'links': [       {       'in_nucast_pkts': '476',
                                                                                                'in_octets': '134362196864',
                                                                                                'in_ucast_pkts': '112053823',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '14609',
                                                                                                'out_octets': '55959688032',
                                                                                                'out_ucast_pkts': '63282989',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '14609',
                                                                       'out_octets': '55959688032',
                                                                       'out_ucast_pkts': '63282989',
                                                                       'poller_hash': 'QURUTjI0NDkxN0E0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '427036',
                                                                                                                        'fiber_length': 4703,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0NDkxN0E0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x10-ONU22',
                                         'poller_hash': 'b250IEFEVE4yNDQ5MTdBNA==',
                                         'serial_number': 'ADTN244917A4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '427036'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.17.0.eth.1.phy',
                                                                       'in_nucast_pkts': '5293',
                                                                       'in_octets': '12671935772',
                                                                       'in_ucast_pkts': '40421903',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x13-ONU17_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '174',
                                                                       'out_octets': '126488221476',
                                                                       'out_ucast_pkts': '96314100',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDEzLU9OVTE3X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '174',
                                                                       'in_octets': '126488221476',
                                                                       'in_ucast_pkts': '96314100',
                                                                       'links': [       {       'in_nucast_pkts': '174',
                                                                                                'in_octets': '126488221476',
                                                                                                'in_ucast_pkts': '96314100',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '5293',
                                                                                                'out_octets': '12671935772',
                                                                                                'out_ucast_pkts': '40421903',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '5293',
                                                                       'out_octets': '12671935772',
                                                                       'out_ucast_pkts': '40421903',
                                                                       'poller_hash': 'QURUTjI1MTUyMUEyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '153651',
                                                                                                                        'fiber_length': 9416,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI1MTUyMUEyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x13-ONU17',
                                         'poller_hash': 'b250IEFEVE4yNTE1MjFBMg==',
                                         'serial_number': 'ADTN251521A2',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_4-4-b',
                                         'uptime': '153651'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '50835',
                                                                       'in_octets': '85868828443',
                                                                       'in_ucast_pkts': '313251806',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'IndianBayou-x2-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4214',
                                                                       'out_octets': '820631184990',
                                                                       'out_ucast_pkts': '594871244',
                                                                       'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgSW5kaWFuQmF5b3UteDItT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4214',
                                                                       'in_octets': '820631184990',
                                                                       'in_ucast_pkts': '594871244',
                                                                       'links': [       {       'in_nucast_pkts': '4214',
                                                                                                'in_octets': '820631184990',
                                                                                                'in_ucast_pkts': '594871244',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '50835',
                                                                                                'out_octets': '85868828443',
                                                                                                'out_ucast_pkts': '313251806',
                                                                                                'poller_hash': 'TEJBRFROMjMyMkFBMjM0IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '50835',
                                                                       'out_octets': '85868828443',
                                                                       'out_ucast_pkts': '313251806',
                                                                       'poller_hash': 'QURUTjIzMTQ0OEI4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1469144',
                                                                                                                        'fiber_length': 13206,
                                                                                                                        'poller_hash': 'TEJBRFROMjMyMkFBMjM0IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OEI4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'IndianBayou-x2-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDhCOA==',
                                         'serial_number': 'ADTN231448B8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '1469144'}]
        }
        id_field = 'mac_address' if not obj.stations else ('mac_address' if hasattr(obj.stations[0], 'mac_address') and getattr(obj.stations[0], 'mac_address') else ('imsi' if hasattr(obj.stations[0], 'imsi') else 'serial'))
        cur_info = {x: getattr(obj, x, None) for (x, data_type) in self.apdd.get_metric_fields('info', 'ap', obj.model) + [('vendor', 'str'), ('model', 'str'), ('sw_version', 'str')]}
        if 'rf_freq' in cur_info and cur_info.get('rf_freq') and (cur_info['rf_freq'] <= 0 or int.bit_length(cur_info['rf_freq']) >= 32):
            cur_info.pop('rf_freq')
        cur_stats = {x: getattr(obj, x, None) for (x, data_type) in self.apdd.get_metric_fields('stats', 'ap', obj.model)}
        cur_sta_stats = {getattr(sta, id_field): {x: getattr(sta, x, None) for (x, data_type) in sorted(self.apdd.get_metric_fields('stats', 'cpe', obj.model)) + [(id_field, 'str'), ('ip_address', 'str'), ('name', 'str'), ('dev_macs', 'str')]} for sta in sorted(obj.stations)}
        cur_ne_msg = getattr(obj, 'network_element_update_msg', None)
        cur_ne_msg = cur_ne_msg.pb if cur_ne_msg else None
        self.assertEqual(cur_info, exp_info)
        self.assertEqual(cur_stats, exp_stats)
        self.assertEqual(cur_sta_stats, exp_sta_stats)
        self.assertTrue(self.compare_pb_dict(cur_ne_msg, exp_ne_msg))

    def test_adtran_sdx6000_SDX_6320_16_sdx6320_RL_25_2_11_10_45_10_12(self):
        if DeviceTestCase.TestDisabled:
                DeviceTestCase.skipTest(self, "Python 3.7 doesn't support this library")
        obj = self.run_data("adtran.sdx6000.SDX 6320-16.sdx6320-RL-25.2-11.10_45_10_12")
        exp_info = {
                'model': 'SDX 6320-16',
                'sta_count': 65,
                'sw_version': 'sdx6320-RL-25.2-11',
                'vendor': 'Adtran'
        }
        exp_stats = {
         
        }
        exp_sta_stats = {
                'ADTN232215D8': {       'dev_macs': ['38:f8:f6:42:1d:a0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU1',
                                        'serial': 'ADTN232215D8'},
                'ADTN232215E4': {       'dev_macs': ['54:22:e0:3c:20:40'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU7',
                                        'serial': 'ADTN232215E4'},
                'ADTN23221633': {       'dev_macs': ['38:f8:f6:42:38:20'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x4-ONU4',
                                        'serial': 'ADTN23221633'},
                'ADTN23221644': {       'dev_macs': ['38:f8:f6:42:1a:e0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU5',
                                        'serial': 'ADTN23221644'},
                'ADTN23221649': {       'dev_macs': ['54:22:e0:3c:66:80'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x9-ONU4',
                                        'serial': 'ADTN23221649'},
                'ADTN23221681': {       'dev_macs': ['38:f8:f6:42:1c:f0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU2',
                                        'serial': 'ADTN23221681'},
                'ADTN232216D3': {       'dev_macs': ['38:f8:f6:42:11:e0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x4-ONU2',
                                        'serial': 'ADTN232216D3'},
                'ADTN232216EF': {       'dev_macs': ['38:f8:f6:42:22:30'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x7-ONU3',
                                        'serial': 'ADTN232216EF'},
                'ADTN2322177F': {       'dev_macs': ['38:f8:f6:42:4e:30'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x7-ONU2',
                                        'serial': 'ADTN2322177F'},
                'ADTN232217C4': {       'dev_macs': ['38:f8:f6:42:35:40'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU0',
                                        'serial': 'ADTN232217C4'},
                'ADTN2322181F': {       'dev_macs': ['38:f8:f6:42:52:a0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x16-ONU2',
                                        'serial': 'ADTN2322181F'},
                'ADTN23221829': {       'dev_macs': ['38:f8:f6:42:4c:90'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x4-ONU1',
                                        'serial': 'ADTN23221829'},
                'ADTN2322183F': {       'dev_macs': ['38:f8:f6:42:55:60'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU3',
                                        'serial': 'ADTN2322183F'},
                'ADTN23221849': {       'dev_macs': ['38:f8:f6:42:2d:70'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x3-ONU3',
                                        'serial': 'ADTN23221849'},
                'ADTN23221879': {       'dev_macs': ['38:f8:f6:42:1c:c0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x9-ONU0',
                                        'serial': 'ADTN23221879'},
                'ADTN2322188F': {       'dev_macs': ['38:f8:f6:42:1a:80'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x4-ONU3',
                                        'serial': 'ADTN2322188F'},
                'ADTN2322189C': {       'dev_macs': ['38:f8:f6:42:1a:70'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU4',
                                        'serial': 'ADTN2322189C'},
                'ADTN232218BC': {       'dev_macs': ['54:22:e0:3c:14:c0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x9-ONU3',
                                        'serial': 'ADTN232218BC'},
                'ADTN232218C9': {       'dev_macs': ['38:f8:f6:41:dd:60'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x4-ONU0',
                                        'serial': 'ADTN232218C9'},
                'ADTN232218D4': {       'dev_macs': ['38:f8:f6:42:35:20'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x16-ONU1',
                                        'serial': 'ADTN232218D4'},
                'ADTN24160005': {       'dev_macs': ['cc:66:18:4a:d9:b0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x10-ONU0',
                                        'serial': 'ADTN24160005'},
                'ADTN2416000A': {       'dev_macs': ['cc:66:18:4b:39:90'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x9-ONU1',
                                        'serial': 'ADTN2416000A'},
                'ADTN24160022': {       'dev_macs': ['cc:66:18:4a:d8:70'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x3-ONU6',
                                        'serial': 'ADTN24160022'},
                'ADTN24160039': {       'dev_macs': ['cc:66:18:4a:d8:c0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x6-ONU3',
                                        'serial': 'ADTN24160039'},
                'ADTN24160054': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x16-ONU3',
                                        'serial': 'ADTN24160054'},
                'ADTN24160057': {       'dev_macs': ['cc:66:18:4a:be:d0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x3-ONU7',
                                        'serial': 'ADTN24160057'},
                'ADTN24160063': {       'dev_macs': ['18:e8:29:4c:44:56'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x12-ONU0',
                                        'serial': 'ADTN24160063'},
                'ADTN24300066': {       'dev_macs': ['cc:66:18:4b:1a:b0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x6-ONU0',
                                        'serial': 'ADTN24300066'},
                'ADTN24300281': {       'dev_macs': ['cc:66:18:4a:cd:80'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x9-ONU2',
                                        'serial': 'ADTN24300281'},
                'ADTN243002B7': {       'dev_macs': ['cc:66:18:4a:d7:00'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x6-ONU2',
                                        'serial': 'ADTN243002B7'},
                'ADTN24300308': {       'dev_macs': ['cc:66:18:4a:d9:d0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x7-ONU1',
                                        'serial': 'ADTN24300308'},
                'ADTN24300323': {       'dev_macs': ['cc:66:18:4a:d7:70'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x6-ONU1',
                                        'serial': 'ADTN24300323'},
                'ADTN2430043D': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x11-ONU1',
                                        'serial': 'ADTN2430043D'},
                'ADTN243004A7': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU6',
                                        'serial': 'ADTN243004A7'},
                'ADTN243004B7': {       'dev_macs': ['cc:66:18:4a:d8:f0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x4-ONU5',
                                        'serial': 'ADTN243004B7'},
                'ADTN243004DF': {       'dev_macs': ['cc:66:18:4b:26:70'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x3-ONU1',
                                        'serial': 'ADTN243004DF'},
                'ADTN2430055C': {       'dev_macs': ['cc:66:18:4a:a2:00'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU12',
                                        'serial': 'ADTN2430055C'},
                'ADTN2430073D': {       'dev_macs': ['cc:66:18:4b:5c:20'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x3-ONU5',
                                        'serial': 'ADTN2430073D'},
                'ADTN243007BA': {       'dev_macs': ['38:f8:f6:42:36:e0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x3-ONU8',
                                        'serial': 'ADTN243007BA'},
                'ADTN243155C0': {       'dev_macs': ['54:22:e0:3c:0c:20'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x6-ONU4',
                                        'serial': 'ADTN243155C0'},
                'ADTN24315628': {       'dev_macs': ['54:22:e0:3c:7c:f0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x6-ONU5',
                                        'serial': 'ADTN24315628'},
                'ADTN2431569E': {       'dev_macs': ['54:22:e0:3c:0c:40'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x15-ONU0',
                                        'serial': 'ADTN2431569E'},
                'ADTN24315712': {       'dev_macs': ['54:22:e0:3c:06:a0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x16-ONU4',
                                        'serial': 'ADTN24315712'},
                'ADTN24315767': {       'dev_macs': ['54:22:e0:3c:12:d0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x12-ONU1',
                                        'serial': 'ADTN24315767'},
                'ADTN2431578B': {       'dev_macs': ['54:22:e0:3c:8a:f0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x4-ONU6',
                                        'serial': 'ADTN2431578B'},
                'ADTN243158A5': {       'dev_macs': ['54:22:e0:3c:aa:b0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU8',
                                        'serial': 'ADTN243158A5'},
                'ADTN243158F4': {       'dev_macs': ['54:22:e0:3c:30:d0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x9-ONU6',
                                        'serial': 'ADTN243158F4'},
                'ADTN243158FA': {       'dev_macs': ['54:22:e0:3c:2c:c0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x7-ONU4',
                                        'serial': 'ADTN243158FA'},
                'ADTN24315948': {       'dev_macs': ['38:f8:f6:83:18:70'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x10-ONU3',
                                        'serial': 'ADTN24315948'},
                'ADTN2431599F': {       'dev_macs': ['54:22:e0:56:15:50'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x7-ONU0',
                                        'serial': 'ADTN2431599F'},
                'ADTN243159C6': {       'dev_macs': ['54:22:e0:3c:35:d0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU10',
                                        'serial': 'ADTN243159C6'},
                'ADTN243159E5': {       'dev_macs': ['54:22:e0:3c:2e:10'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x16-ONU6',
                                        'serial': 'ADTN243159E5'},
                'ADTN24315A51': {       'dev_macs': ['54:22:e0:56:26:40'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x16-ONU7',
                                        'serial': 'ADTN24315A51'},
                'ADTN24315AC5': {       'dev_macs': ['54:22:e0:3c:80:c0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x10-ONU2',
                                        'serial': 'ADTN24315AC5'},
                'ADTN24315B0E': {       'dev_macs': ['54:22:e0:3c:15:30'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x7-ONU5',
                                        'serial': 'ADTN24315B0E'},
                'ADTN24315B5B': {       'dev_macs': ['38:f8:f6:96:38:40'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x9-ONU5',
                                        'serial': 'ADTN24315B5B'},
                'ADTN24315B77': {       'dev_macs': ['54:22:e0:56:00:a0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x6-ONU8',
                                        'serial': 'ADTN24315B77'},
                'ADTN24315B9B': {       'dev_macs': ['54:22:e0:3c:5e:00'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU11',
                                        'serial': 'ADTN24315B9B'},
                'ADTN24315BBD': {       'dev_macs': ['54:22:e0:3c:67:f0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x10-ONU1',
                                        'serial': 'ADTN24315BBD'},
                'ADTN24315BCB': {       'dev_macs': ['54:22:e0:3c:65:e0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x7-ONU6',
                                        'serial': 'ADTN24315BCB'},
                'ADTN24315BEE': {       'dev_macs': ['54:22:e0:3c:31:a0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x6-ONU6',
                                        'serial': 'ADTN24315BEE'},
                'ADTN24315C0B': {       'dev_macs': ['54:22:e0:3c:ad:d0'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x5-ONU9',
                                        'serial': 'ADTN24315C0B'},
                'ADTN24315C28': {       'dev_macs': ['54:22:e0:56:01:60'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x10-ONU4',
                                        'serial': 'ADTN24315C28'},
                'ADTN250239B7': {       'dev_macs': ['54:22:e0:56:2e:10'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x10-ONU5',
                                        'serial': 'ADTN250239B7'},
                'ADTN250553A8': {       'dev_macs': ['54:22:e0:56:17:80'],
                                        'ip_address': None,
                                        'name': 'COURT-OLT02-x13-ONU0',
                                        'serial': 'ADTN250553A8'}
        }
        exp_ne_msg = {
                'data': {       'cpu': 15.41,
                                'interfaces': [       {       'admin_up': True,
                                                              'id': '0/1',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/1',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8x',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/1',
                                                                                      'id': 'gpon '
                                                                                            '0/1',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/1',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvMQ==',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/1',
                                                                                      'id': 'xgspon '
                                                                                            '0/1',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/1',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8x',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/10',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:4a:d9:b0'],
                                                                                       'in_nucast_pkts': '189115',
                                                                                       'in_octets': '102395467977',
                                                                                       'in_ucast_pkts': '274128907',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '17',
                                                                                       'out_octets': '1399185605831',
                                                                                       'out_ucast_pkts': '974106150',
                                                                                       'poller_hash': 'QURUTjI0MTYwMDA1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:83:18:70'],
                                                                                       'in_nucast_pkts': '48585',
                                                                                       'in_octets': '432033988003',
                                                                                       'in_ucast_pkts': '1254602937',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '17',
                                                                                       'out_octets': '3673298764763',
                                                                                       'out_ucast_pkts': '2776210294',
                                                                                       'poller_hash': 'QURUTjI0MzE1OTQ4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:67:f0'],
                                                                                       'in_nucast_pkts': '20177',
                                                                                       'in_octets': '8340237255',
                                                                                       'in_ucast_pkts': '20275285',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '98518662832',
                                                                                       'out_ucast_pkts': '74426625',
                                                                                       'poller_hash': 'QURUTjI0MzE1QkJEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:80:c0'],
                                                                                       'in_nucast_pkts': '196854',
                                                                                       'in_octets': '60520873610',
                                                                                       'in_ucast_pkts': '104712196',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '17',
                                                                                       'out_octets': '464654638005',
                                                                                       'out_ucast_pkts': '341504519',
                                                                                       'poller_hash': 'QURUTjI0MzE1QUM1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:56:01:60'],
                                                                                       'in_nucast_pkts': '30037',
                                                                                       'in_octets': '56944581543',
                                                                                       'in_ucast_pkts': '281172927',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1329089123637',
                                                                                       'out_ucast_pkts': '956083166',
                                                                                       'poller_hash': 'QURUTjI0MzE1QzI4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:56:2e:10'],
                                                                                       'in_nucast_pkts': '18725',
                                                                                       'in_octets': '19143027006',
                                                                                       'in_ucast_pkts': '129203556',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '439504573732',
                                                                                       'out_ucast_pkts': '310649488',
                                                                                       'poller_hash': 'QURUTjI1MDIzOUI3IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/10',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMA==',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/10',
                                                                                      'id': 'gpon '
                                                                                            '0/10',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/10',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvMTA=',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/10',
                                                                                      'id': 'xgspon '
                                                                                            '0/10',
                                                                                      'links': [       {       'connected_time': '5677492',
                                                                                                               'fiber_length': 2550,
                                                                                                               'poller_hash': 'QURUTjI0MTYwMDA1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7364},
                                                                                                       {       'connected_time': '5908159',
                                                                                                               'fiber_length': 2534,
                                                                                                               'poller_hash': 'QURUTjI0MzE1OTQ4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7364},
                                                                                                       {       'connected_time': '605160',
                                                                                                               'fiber_length': 2799,
                                                                                                               'poller_hash': 'QURUTjI0MzE1QkJEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7364},
                                                                                                       {       'connected_time': '5908148',
                                                                                                               'fiber_length': 2864,
                                                                                                               'poller_hash': 'QURUTjI0MzE1QUM1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7364},
                                                                                                       {       'connected_time': '3627734',
                                                                                                               'fiber_length': 2599,
                                                                                                               'poller_hash': 'QURUTjI0MzE1QzI4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -24.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7364},
                                                                                                       {       'connected_time': '2270614',
                                                                                                               'fiber_length': 2406,
                                                                                                               'poller_hash': 'QURUTjI1MDIzOUI3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7364}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/10',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/11',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/11',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMQ==',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/11',
                                                                                      'id': 'gpon '
                                                                                            '0/11',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/11',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvMTE=',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/11',
                                                                                      'id': 'xgspon '
                                                                                            '0/11',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/11',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/12',
                                                              'links': [       {       'bridged_mac_addresses': [       '18:e8:29:4c:44:56'],
                                                                                       'in_nucast_pkts': '1613334',
                                                                                       'in_octets': '1729504242720',
                                                                                       'in_ucast_pkts': '8061892975',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '11467',
                                                                                       'out_octets': '24730548291125',
                                                                                       'out_ucast_pkts': '17961538604',
                                                                                       'poller_hash': 'QURUTjI0MTYwMDYzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:12:d0'],
                                                                                       'in_nucast_pkts': '162839',
                                                                                       'in_octets': '48514770810',
                                                                                       'in_ucast_pkts': '55141234',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3',
                                                                                       'out_octets': '128188762850',
                                                                                       'out_ucast_pkts': '108691185',
                                                                                       'poller_hash': 'QURUTjI0MzE1NzY3IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/12',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMg==',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/12',
                                                                                      'id': 'gpon '
                                                                                            '0/12',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/12',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvMTI=',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/12',
                                                                                      'id': 'xgspon '
                                                                                            '0/12',
                                                                                      'links': [       {       'connected_time': '13779834',
                                                                                                               'fiber_length': 5770,
                                                                                                               'poller_hash': 'QURUTjI0MTYwMDYzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.9767},
                                                                                                       {       'connected_time': '4888203',
                                                                                                               'fiber_length': 4903,
                                                                                                               'poller_hash': 'QURUTjI0MzE1NzY3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.9767}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/12',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/13',
                                                              'links': [       {       'bridged_mac_addresses': [       '54:22:e0:56:17:80'],
                                                                                       'in_nucast_pkts': '6139',
                                                                                       'in_octets': '18541776867',
                                                                                       'in_ucast_pkts': '103601902',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '296637308288',
                                                                                       'out_ucast_pkts': '215189445',
                                                                                       'poller_hash': 'QURUTjI1MDU1M0E4IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/13',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMw==',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/13',
                                                                                      'id': 'gpon '
                                                                                            '0/13',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/13',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvMTM=',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/13',
                                                                                      'id': 'xgspon '
                                                                                            '0/13',
                                                                                      'links': [       {       'connected_time': '720719',
                                                                                                               'fiber_length': 6839,
                                                                                                               'poller_hash': 'QURUTjI1MDU1M0E4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7859}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/13',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/14',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/14',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xNA==',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/14',
                                                                                      'id': 'gpon '
                                                                                            '0/14',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/14',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvMTQ=',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/14',
                                                                                      'id': 'xgspon '
                                                                                            '0/14',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/14',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/15',
                                                              'links': [       {       'bridged_mac_addresses': [       '54:22:e0:3c:0c:40'],
                                                                                       'in_nucast_pkts': '458987',
                                                                                       'in_octets': '302050302113',
                                                                                       'in_ucast_pkts': '763611898',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '3925281094956',
                                                                                       'out_ucast_pkts': '2927259512',
                                                                                       'poller_hash': 'QURUTjI0MzE1NjlFIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/15',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xNQ==',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/15',
                                                                                      'id': 'gpon '
                                                                                            '0/15',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/15',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvMTU=',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/15',
                                                                                      'id': 'xgspon '
                                                                                            '0/15',
                                                                                      'links': [       {       'connected_time': '13779834',
                                                                                                               'fiber_length': 9891,
                                                                                                               'poller_hash': 'QURUTjI0MzE1NjlFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.9167}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/15',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/16',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:42:35:20'],
                                                                                       'in_nucast_pkts': '381330',
                                                                                       'in_octets': '130525431506',
                                                                                       'in_ucast_pkts': '109835826',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '86996469274',
                                                                                       'out_ucast_pkts': '89108554',
                                                                                       'poller_hash': 'QURUTjIzMjIxOEQ0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:52:a0'],
                                                                                       'in_nucast_pkts': '458991',
                                                                                       'in_octets': '84314490392',
                                                                                       'in_ucast_pkts': '71374313',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '86783430412',
                                                                                       'out_ucast_pkts': '82404978',
                                                                                       'poller_hash': 'QURUTjIzMjIxODFGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:06:a0'],
                                                                                       'in_nucast_pkts': '166922',
                                                                                       'in_octets': '422045420615',
                                                                                       'in_ucast_pkts': '941436428',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '2842783666142',
                                                                                       'out_ucast_pkts': '2125498705',
                                                                                       'poller_hash': 'QURUTjI0MzE1NzEyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:2e:10'],
                                                                                       'in_nucast_pkts': '299933',
                                                                                       'in_octets': '321235332415',
                                                                                       'in_ucast_pkts': '733356586',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '26',
                                                                                       'out_octets': '2863523369542',
                                                                                       'out_ucast_pkts': '2086051068',
                                                                                       'poller_hash': 'QURUTjI0MzE1OUU1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:56:26:40'],
                                                                                       'in_nucast_pkts': '31769',
                                                                                       'in_octets': '55467852678',
                                                                                       'in_ucast_pkts': '407752377',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '2272910568794',
                                                                                       'out_ucast_pkts': '1585293024',
                                                                                       'poller_hash': 'QURUTjI0MzE1QTUxIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/16',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xNg==',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/16',
                                                                                      'id': 'gpon '
                                                                                            '0/16',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/16',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvMTY=',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/16',
                                                                                      'id': 'xgspon '
                                                                                            '0/16',
                                                                                      'links': [       {       'connected_time': '11448280',
                                                                                                               'fiber_length': 10066,
                                                                                                               'poller_hash': 'QURUTjIzMjIxOEQ0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 6.0593},
                                                                                                       {       'connected_time': '13779832',
                                                                                                               'fiber_length': 9854,
                                                                                                               'poller_hash': 'QURUTjIzMjIxODFGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 6.0593},
                                                                                                       {       'connected_time': '5011300',
                                                                                                               'fiber_length': 9990,
                                                                                                               'poller_hash': 'QURUTjI0MzE1NzEyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -24.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 6.0593},
                                                                                                       {       'connected_time': '9026294',
                                                                                                               'fiber_length': 10175,
                                                                                                               'poller_hash': 'QURUTjI0MzE1OUU1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 6.0593},
                                                                                                       {       'connected_time': '3822939',
                                                                                                               'fiber_length': 10256,
                                                                                                               'poller_hash': 'QURUTjI0MzE1QTUxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 6.0593}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/16',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/2',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/2',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8y',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/2',
                                                                                      'id': 'gpon '
                                                                                            '0/2',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/2',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvMg==',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/2',
                                                                                      'id': 'xgspon '
                                                                                            '0/2',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/2',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/3',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:4b:26:70'],
                                                                                       'in_nucast_pkts': '137793',
                                                                                       'in_octets': '90610808846',
                                                                                       'in_ucast_pkts': '271918889',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1064990737067',
                                                                                       'out_ucast_pkts': '762817018',
                                                                                       'poller_hash': 'QURUTjI0MzAwNERGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:2d:70'],
                                                                                       'in_nucast_pkts': '139138',
                                                                                       'in_octets': '49349372535',
                                                                                       'in_ucast_pkts': '163301225',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '618562252857',
                                                                                       'out_ucast_pkts': '430564767',
                                                                                       'poller_hash': 'QURUTjIzMjIxODQ5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:5c:20'],
                                                                                       'in_nucast_pkts': '137833',
                                                                                       'in_octets': '407700754614',
                                                                                       'in_ucast_pkts': '2313261064',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '8284105140008',
                                                                                       'out_ucast_pkts': '6610442600',
                                                                                       'poller_hash': 'QURUTjI0MzAwNzNEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4a:d8:70'],
                                                                                       'in_nucast_pkts': '140734',
                                                                                       'in_octets': '83887642786',
                                                                                       'in_ucast_pkts': '223401903',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1828299533183',
                                                                                       'out_ucast_pkts': '1360643676',
                                                                                       'poller_hash': 'QURUTjI0MTYwMDIyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:36:e0'],
                                                                                       'in_nucast_pkts': '137796',
                                                                                       'in_octets': '45669814792',
                                                                                       'in_ucast_pkts': '141765226',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '618582346966',
                                                                                       'out_ucast_pkts': '435043496',
                                                                                       'poller_hash': 'QURUTjI0MzAwN0JBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4a:be:d0'],
                                                                                       'in_nucast_pkts': '91732',
                                                                                       'in_octets': '38369584713',
                                                                                       'in_ucast_pkts': '88848776',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '394459532009',
                                                                                       'out_ucast_pkts': '279410318',
                                                                                       'poller_hash': 'QURUTjI0MTYwMDU3IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/3',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8z',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/3',
                                                                                      'id': 'gpon '
                                                                                            '0/3',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/3',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvMw==',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/3',
                                                                                      'id': 'xgspon '
                                                                                            '0/3',
                                                                                      'links': [       {       'connected_time': '4136716',
                                                                                                               'fiber_length': 9886,
                                                                                                               'poller_hash': 'QURUTjI0MzAwNERGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -26.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.9066},
                                                                                                       {       'connected_time': '4136660',
                                                                                                               'fiber_length': 9354,
                                                                                                               'poller_hash': 'QURUTjIzMjIxODQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.9066},
                                                                                                       {       'connected_time': '4136712',
                                                                                                               'fiber_length': 9454,
                                                                                                               'poller_hash': 'QURUTjI0MzAwNzNEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.9066},
                                                                                                       {       'connected_time': '4136697',
                                                                                                               'fiber_length': 10424,
                                                                                                               'poller_hash': 'QURUTjI0MTYwMDIyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.9066},
                                                                                                       {       'connected_time': '4136716',
                                                                                                               'fiber_length': 9904,
                                                                                                               'poller_hash': 'QURUTjI0MzAwN0JBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -26.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.9066},
                                                                                                       {       'connected_time': '2723488',
                                                                                                               'fiber_length': 10138,
                                                                                                               'poller_hash': 'QURUTjI0MTYwMDU3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.9066}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/3',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/4',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:41:dd:60'],
                                                                                       'in_nucast_pkts': '40962',
                                                                                       'in_octets': '51695716200',
                                                                                       'in_ucast_pkts': '96031189',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '310991547144',
                                                                                       'out_ucast_pkts': '249182815',
                                                                                       'poller_hash': 'QURUTjIzMjIxOEM5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:4c:90'],
                                                                                       'in_nucast_pkts': '25248',
                                                                                       'in_octets': '23805090915',
                                                                                       'in_ucast_pkts': '96963903',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '312873467600',
                                                                                       'out_ucast_pkts': '228052239',
                                                                                       'poller_hash': 'QURUTjIzMjIxODI5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:11:e0'],
                                                                                       'in_nucast_pkts': '436538',
                                                                                       'in_octets': '481891173105',
                                                                                       'in_ucast_pkts': '783578234',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '39',
                                                                                       'out_octets': '2812166926036',
                                                                                       'out_ucast_pkts': '2096165690',
                                                                                       'poller_hash': 'QURUTjIzMjIxNkQzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:1a:80'],
                                                                                       'in_nucast_pkts': '440884',
                                                                                       'in_octets': '243985211088',
                                                                                       'in_ucast_pkts': '366933122',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '39',
                                                                                       'out_octets': '1913087271517',
                                                                                       'out_ucast_pkts': '1442759623',
                                                                                       'poller_hash': 'QURUTjIzMjIxODhGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:38:20'],
                                                                                       'in_nucast_pkts': '436579',
                                                                                       'in_octets': '1166022885485',
                                                                                       'in_ucast_pkts': '870718780',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '39',
                                                                                       'out_octets': '649903137417',
                                                                                       'out_ucast_pkts': '806761959',
                                                                                       'poller_hash': 'QURUTjIzMjIxNjMzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4a:d8:f0'],
                                                                                       'in_nucast_pkts': '137795',
                                                                                       'in_octets': '42449103146',
                                                                                       'in_ucast_pkts': '76480480',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '402353232636',
                                                                                       'out_ucast_pkts': '289174922',
                                                                                       'poller_hash': 'QURUTjI0MzAwNEI3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:8a:f0'],
                                                                                       'in_nucast_pkts': '6234',
                                                                                       'in_octets': '239081542',
                                                                                       'in_ucast_pkts': '1041263',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '4411657978',
                                                                                       'out_ucast_pkts': '3529479',
                                                                                       'poller_hash': 'QURUTjI0MzE1NzhCIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/4',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC80',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/4',
                                                                                      'id': 'gpon '
                                                                                            '0/4',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/4',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvNA==',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/4',
                                                                                      'id': 'xgspon '
                                                                                            '0/4',
                                                                                      'links': [       {       'connected_time': '1202460',
                                                                                                               'fiber_length': 10753,
                                                                                                               'poller_hash': 'QURUTjIzMjIxOEM5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8499},
                                                                                                       {       'connected_time': '738982',
                                                                                                               'fiber_length': 10870,
                                                                                                               'poller_hash': 'QURUTjIzMjIxODI5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8499},
                                                                                                       {       'connected_time': '13105117',
                                                                                                               'fiber_length': 10858,
                                                                                                               'poller_hash': 'QURUTjIzMjIxNkQzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8499},
                                                                                                       {       'connected_time': '13105116',
                                                                                                               'fiber_length': 11036,
                                                                                                               'poller_hash': 'QURUTjIzMjIxODhGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8499},
                                                                                                       {       'connected_time': '13105117',
                                                                                                               'fiber_length': 10850,
                                                                                                               'poller_hash': 'QURUTjIzMjIxNjMzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8499},
                                                                                                       {       'connected_time': '4136716',
                                                                                                               'fiber_length': 10499,
                                                                                                               'poller_hash': 'QURUTjI0MzAwNEI3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8499},
                                                                                                       {       'connected_time': '206922',
                                                                                                               'fiber_length': 11870,
                                                                                                               'poller_hash': 'QURUTjI0MzE1NzhCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8499}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/4',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/5',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:42:35:40'],
                                                                                       'in_nucast_pkts': '463305',
                                                                                       'in_octets': '1178984076318',
                                                                                       'in_ucast_pkts': '1223805426',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '35',
                                                                                       'out_octets': '3369976835188',
                                                                                       'out_ucast_pkts': '2591958082',
                                                                                       'poller_hash': 'QURUTjIzMjIxN0M0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:1d:a0'],
                                                                                       'in_nucast_pkts': '7078',
                                                                                       'in_octets': '38371444454',
                                                                                       'in_ucast_pkts': '38109253',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '58012669622',
                                                                                       'out_ucast_pkts': '54013743',
                                                                                       'poller_hash': 'QURUTjIzMjIxNUQ4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:1c:f0'],
                                                                                       'in_nucast_pkts': '6911',
                                                                                       'in_octets': '4172251496',
                                                                                       'in_ucast_pkts': '4166041',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '9678544072',
                                                                                       'out_ucast_pkts': '7908162',
                                                                                       'poller_hash': 'QURUTjIzMjIxNjgxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:55:60'],
                                                                                       'in_nucast_pkts': '4976',
                                                                                       'in_octets': '4620588177',
                                                                                       'in_ucast_pkts': '14150873',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '60015295518',
                                                                                       'out_ucast_pkts': '43673788',
                                                                                       'poller_hash': 'QURUTjIzMjIxODNGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:1a:70'],
                                                                                       'in_nucast_pkts': '6910',
                                                                                       'in_octets': '8527519977',
                                                                                       'in_ucast_pkts': '9544082',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '37307519656',
                                                                                       'out_ucast_pkts': '27693170',
                                                                                       'poller_hash': 'QURUTjIzMjIxODlDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:1a:e0'],
                                                                                       'in_nucast_pkts': '7084',
                                                                                       'in_octets': '8166394517',
                                                                                       'in_ucast_pkts': '10758213',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '49876785651',
                                                                                       'out_ucast_pkts': '36189011',
                                                                                       'poller_hash': 'QURUTjIzMjIxNjQ0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:20:40'],
                                                                                       'in_nucast_pkts': '7084',
                                                                                       'in_octets': '2928782530',
                                                                                       'in_ucast_pkts': '7239204',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '28832359733',
                                                                                       'out_ucast_pkts': '21454166',
                                                                                       'poller_hash': 'QURUTjIzMjIxNUU0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:aa:b0'],
                                                                                       'in_nucast_pkts': '6893',
                                                                                       'in_octets': '15263930760',
                                                                                       'in_ucast_pkts': '24578082',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '94619947938',
                                                                                       'out_ucast_pkts': '75160872',
                                                                                       'poller_hash': 'QURUTjI0MzE1OEE1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:ad:d0'],
                                                                                       'in_nucast_pkts': '6136',
                                                                                       'in_octets': '295915251',
                                                                                       'in_ucast_pkts': '1410902',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '12576670318',
                                                                                       'out_ucast_pkts': '8838864',
                                                                                       'poller_hash': 'QURUTjI0MzE1QzBCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:35:d0'],
                                                                                       'in_nucast_pkts': '6912',
                                                                                       'in_octets': '4212425014',
                                                                                       'in_ucast_pkts': '7070404',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '26673574979',
                                                                                       'out_ucast_pkts': '19634393',
                                                                                       'poller_hash': 'QURUTjI0MzE1OUM2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:5e:00'],
                                                                                       'in_nucast_pkts': '6910',
                                                                                       'in_octets': '3701614432',
                                                                                       'in_ucast_pkts': '19536998',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '129073834011',
                                                                                       'out_ucast_pkts': '90093128',
                                                                                       'poller_hash': 'QURUTjI0MzE1QjlCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4a:a2:00'],
                                                                                       'in_nucast_pkts': '2596',
                                                                                       'in_octets': '1774795493',
                                                                                       'in_ucast_pkts': '1460464',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '2590042477',
                                                                                       'out_ucast_pkts': '2082155',
                                                                                       'poller_hash': 'QURUTjI0MzAwNTVDIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/5',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/5',
                                                                                      'id': 'gpon '
                                                                                            '0/5',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/5',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvNQ==',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/5',
                                                                                      'id': 'xgspon '
                                                                                            '0/5',
                                                                                      'links': [       {       'connected_time': '13779830',
                                                                                                               'fiber_length': 12942,
                                                                                                               'poller_hash': 'QURUTjIzMjIxN0M0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921},
                                                                                                       {       'connected_time': '206918',
                                                                                                               'fiber_length': 11422,
                                                                                                               'poller_hash': 'QURUTjIzMjIxNUQ4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921},
                                                                                                       {       'connected_time': '206917',
                                                                                                               'fiber_length': 12679,
                                                                                                               'poller_hash': 'QURUTjIzMjIxNjgxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921},
                                                                                                       {       'connected_time': '148812',
                                                                                                               'fiber_length': 11627,
                                                                                                               'poller_hash': 'QURUTjIzMjIxODNGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921},
                                                                                                       {       'connected_time': '206917',
                                                                                                               'fiber_length': 11739,
                                                                                                               'poller_hash': 'QURUTjIzMjIxODlDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921},
                                                                                                       {       'connected_time': '206921',
                                                                                                               'fiber_length': 13393,
                                                                                                               'poller_hash': 'QURUTjIzMjIxNjQ0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921},
                                                                                                       {       'connected_time': '206922',
                                                                                                               'fiber_length': 11662,
                                                                                                               'poller_hash': 'QURUTjIzMjIxNUU0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921},
                                                                                                       {       'connected_time': '206920',
                                                                                                               'fiber_length': 11475,
                                                                                                               'poller_hash': 'QURUTjI0MzE1OEE1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921},
                                                                                                       {       'connected_time': '206914',
                                                                                                               'fiber_length': 12572,
                                                                                                               'poller_hash': 'QURUTjI0MzE1QzBCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921},
                                                                                                       {       'connected_time': '206916',
                                                                                                               'fiber_length': 11660,
                                                                                                               'poller_hash': 'QURUTjI0MzE1OUM2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921},
                                                                                                       {       'connected_time': '206911',
                                                                                                               'fiber_length': 12568,
                                                                                                               'poller_hash': 'QURUTjI0MzE1QjlCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921},
                                                                                                       {       'connected_time': '77607',
                                                                                                               'fiber_length': 11204,
                                                                                                               'poller_hash': 'QURUTjI0MzAwNTVDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8921}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/5',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/6',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:4b:1a:b0'],
                                                                                       'in_nucast_pkts': '6898',
                                                                                       'in_octets': '4335408878',
                                                                                       'in_ucast_pkts': '13781104',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '53697149611',
                                                                                       'out_ucast_pkts': '38952159',
                                                                                       'poller_hash': 'QURUTjI0MzAwMDY2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4a:d7:70'],
                                                                                       'in_nucast_pkts': '138374',
                                                                                       'in_octets': '94469407751',
                                                                                       'in_ucast_pkts': '183714353',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1084260257589',
                                                                                       'out_ucast_pkts': '789178650',
                                                                                       'poller_hash': 'QURUTjI0MzAwMzIzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4a:d7:00'],
                                                                                       'in_nucast_pkts': '6211',
                                                                                       'in_octets': '10313301081',
                                                                                       'in_ucast_pkts': '13663816',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '29892262028',
                                                                                       'out_ucast_pkts': '26727189',
                                                                                       'poller_hash': 'QURUTjI0MzAwMkI3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4a:d8:c0'],
                                                                                       'in_nucast_pkts': '124176',
                                                                                       'in_octets': '41429781080',
                                                                                       'in_ucast_pkts': '64600333',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '248299874813',
                                                                                       'out_ucast_pkts': '191355149',
                                                                                       'poller_hash': 'QURUTjI0MTYwMDM5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:0c:20'],
                                                                                       'in_nucast_pkts': '6911',
                                                                                       'in_octets': '3844849862',
                                                                                       'in_ucast_pkts': '4038723',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '9793497912',
                                                                                       'out_ucast_pkts': '7827170',
                                                                                       'poller_hash': 'QURUTjI0MzE1NUMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:7c:f0'],
                                                                                       'in_nucast_pkts': '7005',
                                                                                       'in_octets': '2190059537',
                                                                                       'in_ucast_pkts': '2572408',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '19641550979',
                                                                                       'out_ucast_pkts': '14027337',
                                                                                       'poller_hash': 'QURUTjI0MzE1NjI4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:31:a0'],
                                                                                       'in_nucast_pkts': '6913',
                                                                                       'in_octets': '1240930819',
                                                                                       'in_ucast_pkts': '1496449',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1134399324',
                                                                                       'out_ucast_pkts': '1548196',
                                                                                       'poller_hash': 'QURUTjI0MzE1QkVFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:56:00:a0'],
                                                                                       'in_nucast_pkts': '26588',
                                                                                       'in_octets': '30145450304',
                                                                                       'in_ucast_pkts': '120560216',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '523603012985',
                                                                                       'out_ucast_pkts': '392718548',
                                                                                       'poller_hash': 'QURUTjI0MzE1Qjc3IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/6',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC82',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/6',
                                                                                      'id': 'gpon '
                                                                                            '0/6',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/6',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvNg==',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/6',
                                                                                      'id': 'xgspon '
                                                                                            '0/6',
                                                                                      'links': [       {       'connected_time': '206962',
                                                                                                               'fiber_length': 11616,
                                                                                                               'poller_hash': 'QURUTjI0MzAwMDY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8726},
                                                                                                       {       'connected_time': '4136711',
                                                                                                               'fiber_length': 10052,
                                                                                                               'poller_hash': 'QURUTjI0MzAwMzIzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8726},
                                                                                                       {       'connected_time': '206961',
                                                                                                               'fiber_length': 12375,
                                                                                                               'poller_hash': 'QURUTjI0MzAwMkI3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8726},
                                                                                                       {       'connected_time': '3723722',
                                                                                                               'fiber_length': 12490,
                                                                                                               'poller_hash': 'QURUTjI0MTYwMDM5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8726},
                                                                                                       {       'connected_time': '206922',
                                                                                                               'fiber_length': 10500,
                                                                                                               'poller_hash': 'QURUTjI0MzE1NUMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8726},
                                                                                                       {       'connected_time': '206921',
                                                                                                               'fiber_length': 11997,
                                                                                                               'poller_hash': 'QURUTjI0MzE1NjI4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8726},
                                                                                                       {       'connected_time': '206921',
                                                                                                               'fiber_length': 10516,
                                                                                                               'poller_hash': 'QURUTjI0MzE1QkVFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -24.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8726},
                                                                                                       {       'connected_time': '3195115',
                                                                                                               'fiber_length': 10130,
                                                                                                               'poller_hash': 'QURUTjI0MzE1Qjc3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.8726}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/6',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/7',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:4a:d9:d0'],
                                                                                       'in_nucast_pkts': '6899',
                                                                                       'in_octets': '2293319646',
                                                                                       'in_ucast_pkts': '6032134',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '50897462074',
                                                                                       'out_ucast_pkts': '37601861',
                                                                                       'poller_hash': 'QURUTjI0MzAwMzA4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:4e:30'],
                                                                                       'in_nucast_pkts': '6891',
                                                                                       'in_octets': '4515107611',
                                                                                       'in_ucast_pkts': '5055281',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '11693955671',
                                                                                       'out_ucast_pkts': '9169094',
                                                                                       'poller_hash': 'QURUTjIzMjIxNzdGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:42:22:30'],
                                                                                       'in_nucast_pkts': '436986',
                                                                                       'in_octets': '149471470469',
                                                                                       'in_ucast_pkts': '1007607824',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '39',
                                                                                       'out_octets': '3745748850436',
                                                                                       'out_ucast_pkts': '2538430349',
                                                                                       'poller_hash': 'QURUTjIzMjIxNkVGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:2c:c0'],
                                                                                       'in_nucast_pkts': '6891',
                                                                                       'in_octets': '1723790967',
                                                                                       'in_ucast_pkts': '2741088',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '6339162646',
                                                                                       'out_ucast_pkts': '5486602',
                                                                                       'poller_hash': 'QURUTjI0MzE1OEZBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:15:30'],
                                                                                       'in_nucast_pkts': '6891',
                                                                                       'in_octets': '1599169368',
                                                                                       'in_ucast_pkts': '2219771',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '3638074989',
                                                                                       'out_ucast_pkts': '3033043',
                                                                                       'poller_hash': 'QURUTjI0MzE1QjBFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:65:e0'],
                                                                                       'in_nucast_pkts': '19400',
                                                                                       'in_octets': '8398195044',
                                                                                       'in_ucast_pkts': '13260049',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '65479166013',
                                                                                       'out_ucast_pkts': '50562876',
                                                                                       'poller_hash': 'QURUTjI0MzE1QkNCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:56:15:50'],
                                                                                       'in_nucast_pkts': '1716',
                                                                                       'in_octets': '980352418',
                                                                                       'in_ucast_pkts': '1189294',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '869742044',
                                                                                       'out_ucast_pkts': '1179019',
                                                                                       'poller_hash': 'QURUTjI0MzE1OTlGIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/7',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC83',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/7',
                                                                                      'id': 'gpon '
                                                                                            '0/7',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/7',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvNw==',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/7',
                                                                                      'id': 'xgspon '
                                                                                            '0/7',
                                                                                      'links': [       {       'connected_time': '206960',
                                                                                                               'fiber_length': 10845,
                                                                                                               'poller_hash': 'QURUTjI0MzAwMzA4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7292},
                                                                                                       {       'connected_time': '206922',
                                                                                                               'fiber_length': 11203,
                                                                                                               'poller_hash': 'QURUTjIzMjIxNzdGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7292},
                                                                                                       {       'connected_time': '13105120',
                                                                                                               'fiber_length': 9898,
                                                                                                               'poller_hash': 'QURUTjIzMjIxNkVGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7292},
                                                                                                       {       'connected_time': '206921',
                                                                                                               'fiber_length': 11449,
                                                                                                               'poller_hash': 'QURUTjI0MzE1OEZBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7292},
                                                                                                       {       'connected_time': '206921',
                                                                                                               'fiber_length': 10867,
                                                                                                               'poller_hash': 'QURUTjI0MzE1QjBFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7292},
                                                                                                       {       'connected_time': '575896',
                                                                                                               'fiber_length': 9690,
                                                                                                               'poller_hash': 'QURUTjI0MzE1QkNCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7292},
                                                                                                       {       'connected_time': '206913',
                                                                                                               'fiber_length': 11739,
                                                                                                               'poller_hash': 'QURUTjI0MzE1OTlGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.7292}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/7',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/8',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/8',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC84',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/8',
                                                                                      'id': 'gpon '
                                                                                            '0/8',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/8',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvOA==',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/8',
                                                                                      'id': 'xgspon '
                                                                                            '0/8',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/8',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/9',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:42:1c:c0'],
                                                                                       'in_nucast_pkts': '6912',
                                                                                       'in_octets': '1445252179',
                                                                                       'in_ucast_pkts': '4677018',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '8480173783',
                                                                                       'out_ucast_pkts': '6128584',
                                                                                       'poller_hash': 'QURUTjIzMjIxODc5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:39:90'],
                                                                                       'in_nucast_pkts': '196834',
                                                                                       'in_octets': '101367607766',
                                                                                       'in_ucast_pkts': '250824266',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '17',
                                                                                       'out_octets': '662658758350',
                                                                                       'out_ucast_pkts': '505342698',
                                                                                       'poller_hash': 'QURUTjI0MTYwMDBBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4a:cd:80'],
                                                                                       'in_nucast_pkts': '7069',
                                                                                       'in_octets': '4325067771',
                                                                                       'in_ucast_pkts': '25959835',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '55474574769',
                                                                                       'out_ucast_pkts': '38459970',
                                                                                       'poller_hash': 'QURUTjI0MzAwMjgxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:14:c0'],
                                                                                       'in_nucast_pkts': '6912',
                                                                                       'in_octets': '5994443149',
                                                                                       'in_ucast_pkts': '31463857',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '225178642937',
                                                                                       'out_ucast_pkts': '154279979',
                                                                                       'poller_hash': 'QURUTjIzMjIxOEJDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:66:80'],
                                                                                       'in_nucast_pkts': '74572',
                                                                                       'in_octets': '30850596248',
                                                                                       'in_ucast_pkts': '56146805',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '306723675869',
                                                                                       'out_ucast_pkts': '227068520',
                                                                                       'poller_hash': 'QURUTjIzMjIxNjQ5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '54:22:e0:3c:30:d0'],
                                                                                       'in_nucast_pkts': '6893',
                                                                                       'in_octets': '13725175614',
                                                                                       'in_ucast_pkts': '25214964',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '64942887137',
                                                                                       'out_ucast_pkts': '47888503',
                                                                                       'poller_hash': 'QURUTjI0MzE1OEY0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:96:38:40'],
                                                                                       'in_nucast_pkts': '11317',
                                                                                       'in_octets': '54740200397',
                                                                                       'in_ucast_pkts': '90335240',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '204672909790',
                                                                                       'out_ucast_pkts': '190254684',
                                                                                       'poller_hash': 'QURUTjI0MzE1QjVCIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442543F2',
                                                              'name': '0/9',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC85',
                                                              'pons': [       {       'description': 'gpon '
                                                                                                     '0/9',
                                                                                      'id': 'gpon '
                                                                                            '0/9',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -30.0,
                                                                                      'name': 'gpon '
                                                                                              '0/9',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiBncG9uIDAvOQ==',
                                                                                      'pon_technology': 'GPON',
                                                                                      'rx_link_rate': '1244160000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '2488320000'},
                                                                              {       'description': 'xgspon '
                                                                                                     '0/9',
                                                                                      'id': 'xgspon '
                                                                                            '0/9',
                                                                                      'links': [       {       'connected_time': '206922',
                                                                                                               'fiber_length': 894,
                                                                                                               'poller_hash': 'QURUTjIzMjIxODc5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5763},
                                                                                                       {       'connected_time': '5908197',
                                                                                                               'fiber_length': 1695,
                                                                                                               'poller_hash': 'QURUTjI0MTYwMDBBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5763},
                                                                                                       {       'connected_time': '206962',
                                                                                                               'fiber_length': 1686,
                                                                                                               'poller_hash': 'QURUTjI0MzAwMjgxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5763},
                                                                                                       {       'connected_time': '206922',
                                                                                                               'fiber_length': 1504,
                                                                                                               'poller_hash': 'QURUTjIzMjIxOEJDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5763},
                                                                                                       {       'connected_time': '2236837',
                                                                                                               'fiber_length': 1360,
                                                                                                               'poller_hash': 'QURUTjIzMjIxNjQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5763},
                                                                                                       {       'connected_time': '206922',
                                                                                                               'fiber_length': 1606,
                                                                                                               'poller_hash': 'QURUTjI0MzE1OEY0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5763},
                                                                                                       {       'connected_time': '1375369',
                                                                                                               'fiber_length': 1687,
                                                                                                               'poller_hash': 'QURUTjI0MzE1QjVCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -26.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5763}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/9',
                                                                                      'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/1',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '42252152',
                                                              'in_octets': '156491151941696',
                                                              'in_ucast_pkts': '116360757205',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:2e:ae:74',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/1',
                                                              'oper_up': True,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '26535877',
                                                              'out_octets': '16168535040397',
                                                              'out_ucast_pkts': '47891525477',
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMQ==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/1:2',
                                                              'mac_address': '38:f8:f6:2e:ae:75',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/1:2',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMToy',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/1:3',
                                                              'mac_address': '38:f8:f6:2e:ae:76',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/1:3',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMToz',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/1:4',
                                                              'mac_address': '38:f8:f6:2e:ae:77',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/1:4',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMTo0',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/2',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '503720992',
                                                              'in_octets': '160894305393555',
                                                              'in_ucast_pkts': '119896861796',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:2e:ae:78',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/2',
                                                              'oper_up': True,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '27928937',
                                                              'out_octets': '15913633760768',
                                                              'out_ucast_pkts': '46299056077',
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMg==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/2:2',
                                                              'mac_address': '38:f8:f6:2e:ae:79',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/2:2',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMjoy',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/2:3',
                                                              'mac_address': '38:f8:f6:2e:ae:7a',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/2:3',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMjoz',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/2:4',
                                                              'mac_address': '38:f8:f6:2e:ae:7b',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/2:4',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMjo0',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/3',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:2e:ae:7c',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/3',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMw==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/3:2',
                                                              'mac_address': '38:f8:f6:2e:ae:7d',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/3:2',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMzoy',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/3:3',
                                                              'mac_address': '38:f8:f6:2e:ae:7e',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/3:3',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMzoz',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/3:4',
                                                              'mac_address': '38:f8:f6:2e:ae:7f',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/3:4',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvMzo0',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/4',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:2e:ae:80',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/4',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvNA==',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/4:2',
                                                              'mac_address': '38:f8:f6:2e:ae:81',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/4:2',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvNDoy',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/4:3',
                                                              'mac_address': '38:f8:f6:2e:ae:82',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/4:3',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvNDoz',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'hundred-gigabit-ethernet '
                                                                    '0/4:4',
                                                              'mac_address': '38:f8:f6:2e:ae:83',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'hundred-gigabit-ethernet '
                                                                      '0/4:4',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgaHVuZHJlZC1naWdhYml0LWV0aGVybmV0IDAvNDo0',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'management 0/1',
                                                              'mac_address': '38:f8:f6:2e:ae:6e',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'management 0/1',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgbWFuYWdlbWVudCAwLzE=',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/1',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:2e:ae:70',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/1',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC8x',
                                                              'speed': '10000000000',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/2',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:2e:ae:71',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/2',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC8y',
                                                              'speed': '10000000000',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/3',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:2e:ae:72',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/3',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC8z',
                                                              'speed': '10000000000',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'ten-gigabit-ethernet '
                                                                    '0/4',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '38:f8:f6:2e:ae:73',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ten-gigabit-ethernet '
                                                                      '0/4',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgdGVuLWdpZ2FiaXQtZXRoZXJuZXQgMC80',
                                                              'speed': '10000000000',
                                                              'type': 6}],
                                'manufacturer': 'Adtran',
                                'mem': 26.76,
                                'model': 'SDX 6320-16',
                                'name': 'COURT-OLT02-g1',
                                'poller_hash': 'b2x0IExCQURUTjIzMTFBQzQzMw==',
                                'serial_number': 'LBADTN2311AC433',
                                'sw_version': 'sdx6320-RL-25.2-11',
                                'temperature': 51.0,
                                'uptime': '13780018'},
                'peers': [       {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6912',
                                                                       'in_octets': '1445252179',
                                                                       'in_ucast_pkts': '4677018',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x9-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '8480173783',
                                                                       'out_ucast_pkts': '6128584',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDktT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '8480173783',
                                                                       'in_ucast_pkts': '6128584',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '8480173783',
                                                                                                'in_ucast_pkts': '6128584',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6912',
                                                                                                'out_octets': '1445252179',
                                                                                                'out_ucast_pkts': '4677018',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6912',
                                                                       'out_octets': '1445252179',
                                                                       'out_ucast_pkts': '4677018',
                                                                       'poller_hash': 'QURUTjIzMjIxODc5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206922',
                                                                                                                        'fiber_length': 894,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxODc5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x9-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTg3OQ==',
                                         'serial_number': 'ADTN23221879',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206922'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '463305',
                                                                       'in_octets': '1178984076318',
                                                                       'in_ucast_pkts': '1223805426',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '35',
                                                                       'out_octets': '3369976835188',
                                                                       'out_ucast_pkts': '2591958082',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '35',
                                                                       'in_octets': '3369976835188',
                                                                       'in_ucast_pkts': '2591958082',
                                                                       'links': [       {       'in_nucast_pkts': '35',
                                                                                                'in_octets': '3369976835188',
                                                                                                'in_ucast_pkts': '2591958082',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '463305',
                                                                                                'out_octets': '1178984076318',
                                                                                                'out_ucast_pkts': '1223805426',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '463305',
                                                                       'out_octets': '1178984076318',
                                                                       'out_ucast_pkts': '1223805426',
                                                                       'poller_hash': 'QURUTjIzMjIxN0M0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13779830',
                                                                                                                        'fiber_length': 12942,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxN0M0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x5-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTdDNA==',
                                         'serial_number': 'ADTN232217C4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '13779830'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '7078',
                                                                       'in_octets': '38371444454',
                                                                       'in_ucast_pkts': '38109253',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '58012669622',
                                                                       'out_ucast_pkts': '54013743',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '58012669622',
                                                                       'in_ucast_pkts': '54013743',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '58012669622',
                                                                                                'in_ucast_pkts': '54013743',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '7078',
                                                                                                'out_octets': '38371444454',
                                                                                                'out_ucast_pkts': '38109253',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '7078',
                                                                       'out_octets': '38371444454',
                                                                       'out_ucast_pkts': '38109253',
                                                                       'poller_hash': 'QURUTjIzMjIxNUQ4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206918',
                                                                                                                        'fiber_length': 11422,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxNUQ4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x5-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTVEOA==',
                                         'serial_number': 'ADTN232215D8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206918'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6911',
                                                                       'in_octets': '4172251496',
                                                                       'in_ucast_pkts': '4166041',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '9678544072',
                                                                       'out_ucast_pkts': '7908162',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '9678544072',
                                                                       'in_ucast_pkts': '7908162',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '9678544072',
                                                                                                'in_ucast_pkts': '7908162',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6911',
                                                                                                'out_octets': '4172251496',
                                                                                                'out_ucast_pkts': '4166041',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6911',
                                                                       'out_octets': '4172251496',
                                                                       'out_ucast_pkts': '4166041',
                                                                       'poller_hash': 'QURUTjIzMjIxNjgxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206917',
                                                                                                                        'fiber_length': 12679,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxNjgxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x5-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTY4MQ==',
                                         'serial_number': 'ADTN23221681',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206917'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '40962',
                                                                       'in_octets': '51695716200',
                                                                       'in_ucast_pkts': '96031189',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x4-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '310991547144',
                                                                       'out_ucast_pkts': '249182815',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDQtT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '310991547144',
                                                                       'in_ucast_pkts': '249182815',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '310991547144',
                                                                                                'in_ucast_pkts': '249182815',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '40962',
                                                                                                'out_octets': '51695716200',
                                                                                                'out_ucast_pkts': '96031189',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '40962',
                                                                       'out_octets': '51695716200',
                                                                       'out_ucast_pkts': '96031189',
                                                                       'poller_hash': 'QURUTjIzMjIxOEM5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1202460',
                                                                                                                        'fiber_length': 10753,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxOEM5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x4-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzIyMThDOQ==',
                                         'serial_number': 'ADTN232218C9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '1202460'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '25248',
                                                                       'in_octets': '23805090915',
                                                                       'in_ucast_pkts': '96963903',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x4-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '312873467600',
                                                                       'out_ucast_pkts': '228052239',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDQtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '312873467600',
                                                                       'in_ucast_pkts': '228052239',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '312873467600',
                                                                                                'in_ucast_pkts': '228052239',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '25248',
                                                                                                'out_octets': '23805090915',
                                                                                                'out_ucast_pkts': '96963903',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '25248',
                                                                       'out_octets': '23805090915',
                                                                       'out_ucast_pkts': '96963903',
                                                                       'poller_hash': 'QURUTjIzMjIxODI5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '738982',
                                                                                                                        'fiber_length': 10870,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxODI5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x4-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTgyOQ==',
                                         'serial_number': 'ADTN23221829',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '738982'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '4976',
                                                                       'in_octets': '4620588177',
                                                                       'in_ucast_pkts': '14150873',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '60015295518',
                                                                       'out_ucast_pkts': '43673788',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '60015295518',
                                                                       'in_ucast_pkts': '43673788',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '60015295518',
                                                                                                'in_ucast_pkts': '43673788',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '4976',
                                                                                                'out_octets': '4620588177',
                                                                                                'out_ucast_pkts': '14150873',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4976',
                                                                       'out_octets': '4620588177',
                                                                       'out_ucast_pkts': '14150873',
                                                                       'poller_hash': 'QURUTjIzMjIxODNGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '148812',
                                                                                                                        'fiber_length': 11627,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxODNGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x5-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTgzRg==',
                                         'serial_number': 'ADTN2322183F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '148812'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '436538',
                                                                       'in_octets': '481891173105',
                                                                       'in_ucast_pkts': '783578234',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x4-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '39',
                                                                       'out_octets': '2812166926036',
                                                                       'out_ucast_pkts': '2096165690',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDQtT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '39',
                                                                       'in_octets': '2812166926036',
                                                                       'in_ucast_pkts': '2096165690',
                                                                       'links': [       {       'in_nucast_pkts': '39',
                                                                                                'in_octets': '2812166926036',
                                                                                                'in_ucast_pkts': '2096165690',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '436538',
                                                                                                'out_octets': '481891173105',
                                                                                                'out_ucast_pkts': '783578234',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '436538',
                                                                       'out_octets': '481891173105',
                                                                       'out_ucast_pkts': '783578234',
                                                                       'poller_hash': 'QURUTjIzMjIxNkQzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13105117',
                                                                                                                        'fiber_length': 10858,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxNkQzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x4-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTZEMw==',
                                         'serial_number': 'ADTN232216D3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '13105117'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6910',
                                                                       'in_octets': '8527519977',
                                                                       'in_ucast_pkts': '9544082',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '37307519656',
                                                                       'out_ucast_pkts': '27693170',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '37307519656',
                                                                       'in_ucast_pkts': '27693170',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '37307519656',
                                                                                                'in_ucast_pkts': '27693170',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6910',
                                                                                                'out_octets': '8527519977',
                                                                                                'out_ucast_pkts': '9544082',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6910',
                                                                       'out_octets': '8527519977',
                                                                       'out_ucast_pkts': '9544082',
                                                                       'poller_hash': 'QURUTjIzMjIxODlDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206917',
                                                                                                                        'fiber_length': 11739,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxODlDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x5-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTg5Qw==',
                                         'serial_number': 'ADTN2322189C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206917'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '440884',
                                                                       'in_octets': '243985211088',
                                                                       'in_ucast_pkts': '366933122',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x4-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '39',
                                                                       'out_octets': '1913087271517',
                                                                       'out_ucast_pkts': '1442759623',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDQtT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '39',
                                                                       'in_octets': '1913087271517',
                                                                       'in_ucast_pkts': '1442759623',
                                                                       'links': [       {       'in_nucast_pkts': '39',
                                                                                                'in_octets': '1913087271517',
                                                                                                'in_ucast_pkts': '1442759623',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '440884',
                                                                                                'out_octets': '243985211088',
                                                                                                'out_ucast_pkts': '366933122',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '440884',
                                                                       'out_octets': '243985211088',
                                                                       'out_ucast_pkts': '366933122',
                                                                       'poller_hash': 'QURUTjIzMjIxODhGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13105116',
                                                                                                                        'fiber_length': 11036,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxODhGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x4-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTg4Rg==',
                                         'serial_number': 'ADTN2322188F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '13105116'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '436579',
                                                                       'in_octets': '1166022885485',
                                                                       'in_ucast_pkts': '870718780',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x4-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '39',
                                                                       'out_octets': '649903137417',
                                                                       'out_ucast_pkts': '806761959',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDQtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '39',
                                                                       'in_octets': '649903137417',
                                                                       'in_ucast_pkts': '806761959',
                                                                       'links': [       {       'in_nucast_pkts': '39',
                                                                                                'in_octets': '649903137417',
                                                                                                'in_ucast_pkts': '806761959',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '436579',
                                                                                                'out_octets': '1166022885485',
                                                                                                'out_ucast_pkts': '870718780',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '436579',
                                                                       'out_octets': '1166022885485',
                                                                       'out_ucast_pkts': '870718780',
                                                                       'poller_hash': 'QURUTjIzMjIxNjMzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13105117',
                                                                                                                        'fiber_length': 10850,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxNjMzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x4-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTYzMw==',
                                         'serial_number': 'ADTN23221633',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '13105117'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '7084',
                                                                       'in_octets': '8166394517',
                                                                       'in_ucast_pkts': '10758213',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '49876785651',
                                                                       'out_ucast_pkts': '36189011',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '49876785651',
                                                                       'in_ucast_pkts': '36189011',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '49876785651',
                                                                                                'in_ucast_pkts': '36189011',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '7084',
                                                                                                'out_octets': '8166394517',
                                                                                                'out_ucast_pkts': '10758213',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '7084',
                                                                       'out_octets': '8166394517',
                                                                       'out_ucast_pkts': '10758213',
                                                                       'poller_hash': 'QURUTjIzMjIxNjQ0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206921',
                                                                                                                        'fiber_length': 13393,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxNjQ0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x5-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTY0NA==',
                                         'serial_number': 'ADTN23221644',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206921'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '381330',
                                                                       'in_octets': '130525431506',
                                                                       'in_ucast_pkts': '109835826',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x16-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '86996469274',
                                                                       'out_ucast_pkts': '89108554',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDE2LU9OVTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '86996469274',
                                                                       'in_ucast_pkts': '89108554',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '86996469274',
                                                                                                'in_ucast_pkts': '89108554',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '381330',
                                                                                                'out_octets': '130525431506',
                                                                                                'out_ucast_pkts': '109835826',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '381330',
                                                                       'out_octets': '130525431506',
                                                                       'out_ucast_pkts': '109835826',
                                                                       'poller_hash': 'QURUTjIzMjIxOEQ0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11448280',
                                                                                                                        'fiber_length': 10066,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxOEQ0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x16-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzIyMThENA==',
                                         'serial_number': 'ADTN232218D4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '11448280'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '458991',
                                                                       'in_octets': '84314490392',
                                                                       'in_ucast_pkts': '71374313',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x16-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '86783430412',
                                                                       'out_ucast_pkts': '82404978',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDE2LU9OVTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '86783430412',
                                                                       'in_ucast_pkts': '82404978',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '86783430412',
                                                                                                'in_ucast_pkts': '82404978',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '458991',
                                                                                                'out_octets': '84314490392',
                                                                                                'out_ucast_pkts': '71374313',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '458991',
                                                                       'out_octets': '84314490392',
                                                                       'out_ucast_pkts': '71374313',
                                                                       'poller_hash': 'QURUTjIzMjIxODFGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13779832',
                                                                                                                        'fiber_length': 9854,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxODFGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x16-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTgxRg==',
                                         'serial_number': 'ADTN2322181F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '13779832'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-15.1.3.0.eth.1.phy',
                                                                       'name': 'COURT-OLT02-x16-ONU3_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDE2LU9OVTNfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjI0MTYwMDU0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MTYwMDU0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'COURT-OLT02-x16-ONU3',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '15.1.3"(ADTN24160054) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/16.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-06-12T19:57:46Z',
                                         'poller_hash': 'b250IEFEVE4yNDE2MDA1NA==',
                                         'serial_number': 'ADTN24160054',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-4.1.6.0.eth.1.phy',
                                                                       'name': 'COURT-OLT02-x5-ONU6_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VNl9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjI0MzAwNEE3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwNEE3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'COURT-OLT02-x5-ONU6',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '4.1.6"(ADTN243004A7) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/5.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-09-02T20:45:10Z',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDRBNw==',
                                         'serial_number': 'ADTN243004A7',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6898',
                                                                       'in_octets': '4335408878',
                                                                       'in_ucast_pkts': '13781104',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x6-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '53697149611',
                                                                       'out_ucast_pkts': '38952159',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDYtT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '53697149611',
                                                                       'in_ucast_pkts': '38952159',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '53697149611',
                                                                                                'in_ucast_pkts': '38952159',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6898',
                                                                                                'out_octets': '4335408878',
                                                                                                'out_ucast_pkts': '13781104',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6898',
                                                                       'out_octets': '4335408878',
                                                                       'out_ucast_pkts': '13781104',
                                                                       'poller_hash': 'QURUTjI0MzAwMDY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206962',
                                                                                                                        'fiber_length': 11616,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwMDY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x6-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDA2Ng==',
                                         'serial_number': 'ADTN24300066',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '206962'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '196834',
                                                                       'in_octets': '101367607766',
                                                                       'in_ucast_pkts': '250824266',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x9-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17',
                                                                       'out_octets': '662658758350',
                                                                       'out_ucast_pkts': '505342698',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDktT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '17',
                                                                       'in_octets': '662658758350',
                                                                       'in_ucast_pkts': '505342698',
                                                                       'links': [       {       'in_nucast_pkts': '17',
                                                                                                'in_octets': '662658758350',
                                                                                                'in_ucast_pkts': '505342698',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '196834',
                                                                                                'out_octets': '101367607766',
                                                                                                'out_ucast_pkts': '250824266',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '196834',
                                                                       'out_octets': '101367607766',
                                                                       'out_ucast_pkts': '250824266',
                                                                       'poller_hash': 'QURUTjI0MTYwMDBBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5908197',
                                                                                                                        'fiber_length': 1695,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.2}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MTYwMDBBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x9-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDE2MDAwQQ==',
                                         'serial_number': 'ADTN2416000A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '5908197'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '138374',
                                                                       'in_octets': '94469407751',
                                                                       'in_ucast_pkts': '183714353',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x6-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1084260257589',
                                                                       'out_ucast_pkts': '789178650',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDYtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1084260257589',
                                                                       'in_ucast_pkts': '789178650',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1084260257589',
                                                                                                'in_ucast_pkts': '789178650',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '138374',
                                                                                                'out_octets': '94469407751',
                                                                                                'out_ucast_pkts': '183714353',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '138374',
                                                                       'out_octets': '94469407751',
                                                                       'out_ucast_pkts': '183714353',
                                                                       'poller_hash': 'QURUTjI0MzAwMzIzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4136711',
                                                                                                                        'fiber_length': 10052,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwMzIzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x6-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDMyMw==',
                                         'serial_number': 'ADTN24300323',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '4136711'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '7069',
                                                                       'in_octets': '4325067771',
                                                                       'in_ucast_pkts': '25959835',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x9-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '55474574769',
                                                                       'out_ucast_pkts': '38459970',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDktT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '55474574769',
                                                                       'in_ucast_pkts': '38459970',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '55474574769',
                                                                                                'in_ucast_pkts': '38459970',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '7069',
                                                                                                'out_octets': '4325067771',
                                                                                                'out_ucast_pkts': '25959835',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '7069',
                                                                       'out_octets': '4325067771',
                                                                       'out_ucast_pkts': '25959835',
                                                                       'poller_hash': 'QURUTjI0MzAwMjgxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206962',
                                                                                                                        'fiber_length': 1686,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwMjgxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x9-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDI4MQ==',
                                         'serial_number': 'ADTN24300281',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '206962'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6211',
                                                                       'in_octets': '10313301081',
                                                                       'in_ucast_pkts': '13663816',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x6-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '29892262028',
                                                                       'out_ucast_pkts': '26727189',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDYtT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '29892262028',
                                                                       'in_ucast_pkts': '26727189',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '29892262028',
                                                                                                'in_ucast_pkts': '26727189',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6211',
                                                                                                'out_octets': '10313301081',
                                                                                                'out_ucast_pkts': '13663816',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6211',
                                                                       'out_octets': '10313301081',
                                                                       'out_ucast_pkts': '13663816',
                                                                       'poller_hash': 'QURUTjI0MzAwMkI3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206961',
                                                                                                                        'fiber_length': 12375,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwMkI3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x6-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDJCNw==',
                                         'serial_number': 'ADTN243002B7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '206961'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '137793',
                                                                       'in_octets': '90610808846',
                                                                       'in_ucast_pkts': '271918889',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x3-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1064990737067',
                                                                       'out_ucast_pkts': '762817018',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDMtT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1064990737067',
                                                                       'in_ucast_pkts': '762817018',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1064990737067',
                                                                                                'in_ucast_pkts': '762817018',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '137793',
                                                                                                'out_octets': '90610808846',
                                                                                                'out_ucast_pkts': '271918889',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '137793',
                                                                       'out_octets': '90610808846',
                                                                       'out_ucast_pkts': '271918889',
                                                                       'poller_hash': 'QURUTjI0MzAwNERGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4136716',
                                                                                                                        'fiber_length': 9886,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwNERGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x3-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDRERg==',
                                         'serial_number': 'ADTN243004DF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '4136716'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6899',
                                                                       'in_octets': '2293319646',
                                                                       'in_ucast_pkts': '6032134',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x7-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '50897462074',
                                                                       'out_ucast_pkts': '37601861',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDctT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '50897462074',
                                                                       'in_ucast_pkts': '37601861',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '50897462074',
                                                                                                'in_ucast_pkts': '37601861',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6899',
                                                                                                'out_octets': '2293319646',
                                                                                                'out_ucast_pkts': '6032134',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6899',
                                                                       'out_octets': '2293319646',
                                                                       'out_ucast_pkts': '6032134',
                                                                       'poller_hash': 'QURUTjI0MzAwMzA4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206960',
                                                                                                                        'fiber_length': 10845,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.1}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwMzA4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x7-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDMwOA==',
                                         'serial_number': 'ADTN24300308',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '206960'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '139138',
                                                                       'in_octets': '49349372535',
                                                                       'in_ucast_pkts': '163301225',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x3-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '618562252857',
                                                                       'out_ucast_pkts': '430564767',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDMtT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '618562252857',
                                                                       'in_ucast_pkts': '430564767',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '618562252857',
                                                                                                'in_ucast_pkts': '430564767',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '139138',
                                                                                                'out_octets': '49349372535',
                                                                                                'out_ucast_pkts': '163301225',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '139138',
                                                                       'out_octets': '49349372535',
                                                                       'out_ucast_pkts': '163301225',
                                                                       'poller_hash': 'QURUTjIzMjIxODQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4136660',
                                                                                                                        'fiber_length': 9354,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxODQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x3-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTg0OQ==',
                                         'serial_number': 'ADTN23221849',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '4136660'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '137795',
                                                                       'in_octets': '42449103146',
                                                                       'in_ucast_pkts': '76480480',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x4-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '402353232636',
                                                                       'out_ucast_pkts': '289174922',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDQtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '402353232636',
                                                                       'in_ucast_pkts': '289174922',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '402353232636',
                                                                                                'in_ucast_pkts': '289174922',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '137795',
                                                                                                'out_octets': '42449103146',
                                                                                                'out_ucast_pkts': '76480480',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '137795',
                                                                       'out_octets': '42449103146',
                                                                       'out_ucast_pkts': '76480480',
                                                                       'poller_hash': 'QURUTjI0MzAwNEI3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4136716',
                                                                                                                        'fiber_length': 10499,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwNEI3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x4-ONU5',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDRCNw==',
                                         'serial_number': 'ADTN243004B7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '4136716'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '137833',
                                                                       'in_octets': '407700754614',
                                                                       'in_ucast_pkts': '2313261064',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x3-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '8284105140008',
                                                                       'out_ucast_pkts': '6610442600',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDMtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '8284105140008',
                                                                       'in_ucast_pkts': '6610442600',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '8284105140008',
                                                                                                'in_ucast_pkts': '6610442600',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '137833',
                                                                                                'out_octets': '407700754614',
                                                                                                'out_ucast_pkts': '2313261064',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '137833',
                                                                       'out_octets': '407700754614',
                                                                       'out_ucast_pkts': '2313261064',
                                                                       'poller_hash': 'QURUTjI0MzAwNzNEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4136712',
                                                                                                                        'fiber_length': 9454,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.1}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwNzNEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x3-ONU5',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDczRA==',
                                         'serial_number': 'ADTN2430073D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '4136712'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '124176',
                                                                       'in_octets': '41429781080',
                                                                       'in_ucast_pkts': '64600333',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x6-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '248299874813',
                                                                       'out_ucast_pkts': '191355149',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDYtT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '248299874813',
                                                                       'in_ucast_pkts': '191355149',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '248299874813',
                                                                                                'in_ucast_pkts': '191355149',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '124176',
                                                                                                'out_octets': '41429781080',
                                                                                                'out_ucast_pkts': '64600333',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '124176',
                                                                       'out_octets': '41429781080',
                                                                       'out_ucast_pkts': '64600333',
                                                                       'poller_hash': 'QURUTjI0MTYwMDM5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3723722',
                                                                                                                        'fiber_length': 12490,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MTYwMDM5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x6-ONU3',
                                         'poller_hash': 'b250IEFEVE4yNDE2MDAzOQ==',
                                         'serial_number': 'ADTN24160039',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '3723722'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '140734',
                                                                       'in_octets': '83887642786',
                                                                       'in_ucast_pkts': '223401903',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x3-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1828299533183',
                                                                       'out_ucast_pkts': '1360643676',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDMtT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1828299533183',
                                                                       'in_ucast_pkts': '1360643676',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1828299533183',
                                                                                                'in_ucast_pkts': '1360643676',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '140734',
                                                                                                'out_octets': '83887642786',
                                                                                                'out_ucast_pkts': '223401903',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '140734',
                                                                       'out_octets': '83887642786',
                                                                       'out_ucast_pkts': '223401903',
                                                                       'poller_hash': 'QURUTjI0MTYwMDIyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4136697',
                                                                                                                        'fiber_length': 10424,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MTYwMDIyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x3-ONU6',
                                         'poller_hash': 'b250IEFEVE4yNDE2MDAyMg==',
                                         'serial_number': 'ADTN24160022',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '4136697'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '189115',
                                                                       'in_octets': '102395467977',
                                                                       'in_ucast_pkts': '274128907',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x10-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17',
                                                                       'out_octets': '1399185605831',
                                                                       'out_ucast_pkts': '974106150',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDEwLU9OVTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '17',
                                                                       'in_octets': '1399185605831',
                                                                       'in_ucast_pkts': '974106150',
                                                                       'links': [       {       'in_nucast_pkts': '17',
                                                                                                'in_octets': '1399185605831',
                                                                                                'in_ucast_pkts': '974106150',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '189115',
                                                                                                'out_octets': '102395467977',
                                                                                                'out_ucast_pkts': '274128907',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '189115',
                                                                       'out_octets': '102395467977',
                                                                       'out_ucast_pkts': '274128907',
                                                                       'poller_hash': 'QURUTjI0MTYwMDA1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5677492',
                                                                                                                        'fiber_length': 2550,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MTYwMDA1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x10-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDE2MDAwNQ==',
                                         'serial_number': 'ADTN24160005',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '5677492'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '1613334',
                                                                       'in_octets': '1729504242720',
                                                                       'in_ucast_pkts': '8061892975',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x12-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11467',
                                                                       'out_octets': '24730548291125',
                                                                       'out_ucast_pkts': '17961538604',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDEyLU9OVTBfRVRIXzE=',
                                                                       'speed': '100000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '11467',
                                                                       'in_octets': '24730548291125',
                                                                       'in_ucast_pkts': '17961538604',
                                                                       'links': [       {       'in_nucast_pkts': '11467',
                                                                                                'in_octets': '24730548291125',
                                                                                                'in_ucast_pkts': '17961538604',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '1613334',
                                                                                                'out_octets': '1729504242720',
                                                                                                'out_ucast_pkts': '8061892975',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1613334',
                                                                       'out_octets': '1729504242720',
                                                                       'out_ucast_pkts': '8061892975',
                                                                       'poller_hash': 'QURUTjI0MTYwMDYzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13779834',
                                                                                                                        'fiber_length': 5770,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.1}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MTYwMDYzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x12-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDE2MDA2Mw==',
                                         'serial_number': 'ADTN24160063',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '13779834'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6891',
                                                                       'in_octets': '4515107611',
                                                                       'in_ucast_pkts': '5055281',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x7-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '11693955671',
                                                                       'out_ucast_pkts': '9169094',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDctT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '11693955671',
                                                                       'in_ucast_pkts': '9169094',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '11693955671',
                                                                                                'in_ucast_pkts': '9169094',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6891',
                                                                                                'out_octets': '4515107611',
                                                                                                'out_ucast_pkts': '5055281',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6891',
                                                                       'out_octets': '4515107611',
                                                                       'out_ucast_pkts': '5055281',
                                                                       'poller_hash': 'QURUTjIzMjIxNzdGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206922',
                                                                                                                        'fiber_length': 11203,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxNzdGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x7-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTc3Rg==',
                                         'serial_number': 'ADTN2322177F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206922'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '436986',
                                                                       'in_octets': '149471470469',
                                                                       'in_ucast_pkts': '1007607824',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x7-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '39',
                                                                       'out_octets': '3745748850436',
                                                                       'out_ucast_pkts': '2538430349',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDctT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '39',
                                                                       'in_octets': '3745748850436',
                                                                       'in_ucast_pkts': '2538430349',
                                                                       'links': [       {       'in_nucast_pkts': '39',
                                                                                                'in_octets': '3745748850436',
                                                                                                'in_ucast_pkts': '2538430349',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '436986',
                                                                                                'out_octets': '149471470469',
                                                                                                'out_ucast_pkts': '1007607824',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '436986',
                                                                       'out_octets': '149471470469',
                                                                       'out_ucast_pkts': '1007607824',
                                                                       'poller_hash': 'QURUTjIzMjIxNkVGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13105120',
                                                                                                                        'fiber_length': 9898,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxNkVGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x7-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTZFRg==',
                                         'serial_number': 'ADTN232216EF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '13105120'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6912',
                                                                       'in_octets': '5994443149',
                                                                       'in_ucast_pkts': '31463857',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x9-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '225178642937',
                                                                       'out_ucast_pkts': '154279979',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDktT05VM19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '225178642937',
                                                                       'in_ucast_pkts': '154279979',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '225178642937',
                                                                                                'in_ucast_pkts': '154279979',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6912',
                                                                                                'out_octets': '5994443149',
                                                                                                'out_ucast_pkts': '31463857',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6912',
                                                                       'out_octets': '5994443149',
                                                                       'out_ucast_pkts': '31463857',
                                                                       'poller_hash': 'QURUTjIzMjIxOEJDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206922',
                                                                                                                        'fiber_length': 1504,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxOEJDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x9-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzIyMThCQw==',
                                         'serial_number': 'ADTN232218BC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206922'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '137796',
                                                                       'in_octets': '45669814792',
                                                                       'in_ucast_pkts': '141765226',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x3-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '618582346966',
                                                                       'out_ucast_pkts': '435043496',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDMtT05VOF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '618582346966',
                                                                       'in_ucast_pkts': '435043496',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '618582346966',
                                                                                                'in_ucast_pkts': '435043496',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '137796',
                                                                                                'out_octets': '45669814792',
                                                                                                'out_ucast_pkts': '141765226',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '137796',
                                                                       'out_octets': '45669814792',
                                                                       'out_ucast_pkts': '141765226',
                                                                       'poller_hash': 'QURUTjI0MzAwN0JBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4136716',
                                                                                                                        'fiber_length': 9904,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwN0JBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x3-ONU8',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDdCQQ==',
                                         'serial_number': 'ADTN243007BA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-3-g',
                                         'uptime': '4136716'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '7084',
                                                                       'in_octets': '2928782530',
                                                                       'in_ucast_pkts': '7239204',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '28832359733',
                                                                       'out_ucast_pkts': '21454166',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '28832359733',
                                                                       'in_ucast_pkts': '21454166',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '28832359733',
                                                                                                'in_ucast_pkts': '21454166',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '7084',
                                                                                                'out_octets': '2928782530',
                                                                                                'out_ucast_pkts': '7239204',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '7084',
                                                                       'out_octets': '2928782530',
                                                                       'out_ucast_pkts': '7239204',
                                                                       'poller_hash': 'QURUTjIzMjIxNUU0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206922',
                                                                                                                        'fiber_length': 11662,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxNUU0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x5-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTVFNA==',
                                         'serial_number': 'ADTN232215E4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206922'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '458987',
                                                                       'in_octets': '302050302113',
                                                                       'in_ucast_pkts': '763611898',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x15-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '3925281094956',
                                                                       'out_ucast_pkts': '2927259512',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDE1LU9OVTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '3925281094956',
                                                                       'in_ucast_pkts': '2927259512',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '3925281094956',
                                                                                                'in_ucast_pkts': '2927259512',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '458987',
                                                                                                'out_octets': '302050302113',
                                                                                                'out_ucast_pkts': '763611898',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '458987',
                                                                       'out_octets': '302050302113',
                                                                       'out_ucast_pkts': '763611898',
                                                                       'poller_hash': 'QURUTjI0MzE1NjlFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13779834',
                                                                                                                        'fiber_length': 9891,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1NjlFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x15-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDMxNTY5RQ==',
                                         'serial_number': 'ADTN2431569E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '13779834'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '162839',
                                                                       'in_octets': '48514770810',
                                                                       'in_ucast_pkts': '55141234',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x12-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3',
                                                                       'out_octets': '128188762850',
                                                                       'out_ucast_pkts': '108691185',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDEyLU9OVTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3',
                                                                       'in_octets': '128188762850',
                                                                       'in_ucast_pkts': '108691185',
                                                                       'links': [       {       'in_nucast_pkts': '3',
                                                                                                'in_octets': '128188762850',
                                                                                                'in_ucast_pkts': '108691185',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '162839',
                                                                                                'out_octets': '48514770810',
                                                                                                'out_ucast_pkts': '55141234',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '162839',
                                                                       'out_octets': '48514770810',
                                                                       'out_ucast_pkts': '55141234',
                                                                       'poller_hash': 'QURUTjI0MzE1NzY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4888203',
                                                                                                                        'fiber_length': 4903,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1NzY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x12-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDMxNTc2Nw==',
                                         'serial_number': 'ADTN24315767',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '4888203'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6911',
                                                                       'in_octets': '3844849862',
                                                                       'in_ucast_pkts': '4038723',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x6-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '9793497912',
                                                                       'out_ucast_pkts': '7827170',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDYtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '9793497912',
                                                                       'in_ucast_pkts': '7827170',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '9793497912',
                                                                                                'in_ucast_pkts': '7827170',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6911',
                                                                                                'out_octets': '3844849862',
                                                                                                'out_ucast_pkts': '4038723',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6911',
                                                                       'out_octets': '3844849862',
                                                                       'out_ucast_pkts': '4038723',
                                                                       'poller_hash': 'QURUTjI0MzE1NUMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206922',
                                                                                                                        'fiber_length': 10500,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1NUMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x6-ONU4',
                                         'poller_hash': 'b250IEFEVE4yNDMxNTVDMA==',
                                         'serial_number': 'ADTN243155C0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206922'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '166922',
                                                                       'in_octets': '422045420615',
                                                                       'in_ucast_pkts': '941436428',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x16-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '2842783666142',
                                                                       'out_ucast_pkts': '2125498705',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDE2LU9OVTRfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '2842783666142',
                                                                       'in_ucast_pkts': '2125498705',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '2842783666142',
                                                                                                'in_ucast_pkts': '2125498705',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '166922',
                                                                                                'out_octets': '422045420615',
                                                                                                'out_ucast_pkts': '941436428',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '166922',
                                                                       'out_octets': '422045420615',
                                                                       'out_ucast_pkts': '941436428',
                                                                       'poller_hash': 'QURUTjI0MzE1NzEyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5011300',
                                                                                                                        'fiber_length': 9990,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1NzEyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x16-ONU4',
                                         'poller_hash': 'b250IEFEVE4yNDMxNTcxMg==',
                                         'serial_number': 'ADTN24315712',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '5011300'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6234',
                                                                       'in_octets': '239081542',
                                                                       'in_ucast_pkts': '1041263',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x4-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '4411657978',
                                                                       'out_ucast_pkts': '3529479',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDQtT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '4411657978',
                                                                       'in_ucast_pkts': '3529479',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '4411657978',
                                                                                                'in_ucast_pkts': '3529479',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6234',
                                                                                                'out_octets': '239081542',
                                                                                                'out_ucast_pkts': '1041263',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6234',
                                                                       'out_octets': '239081542',
                                                                       'out_ucast_pkts': '1041263',
                                                                       'poller_hash': 'QURUTjI0MzE1NzhCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206922',
                                                                                                                        'fiber_length': 11870,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1NzhCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x4-ONU6',
                                         'poller_hash': 'b250IEFEVE4yNDMxNTc4Qg==',
                                         'serial_number': 'ADTN2431578B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206922'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '7005',
                                                                       'in_octets': '2190059537',
                                                                       'in_ucast_pkts': '2572408',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x6-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '19641550979',
                                                                       'out_ucast_pkts': '14027337',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDYtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '19641550979',
                                                                       'in_ucast_pkts': '14027337',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '19641550979',
                                                                                                'in_ucast_pkts': '14027337',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '7005',
                                                                                                'out_octets': '2190059537',
                                                                                                'out_ucast_pkts': '2572408',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '7005',
                                                                       'out_octets': '2190059537',
                                                                       'out_ucast_pkts': '2572408',
                                                                       'poller_hash': 'QURUTjI0MzE1NjI4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206921',
                                                                                                                        'fiber_length': 11997,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1NjI4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x6-ONU5',
                                         'poller_hash': 'b250IEFEVE4yNDMxNTYyOA==',
                                         'serial_number': 'ADTN24315628',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206921'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6891',
                                                                       'in_octets': '1723790967',
                                                                       'in_ucast_pkts': '2741088',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x7-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '6339162646',
                                                                       'out_ucast_pkts': '5486602',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDctT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '6339162646',
                                                                       'in_ucast_pkts': '5486602',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '6339162646',
                                                                                                'in_ucast_pkts': '5486602',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6891',
                                                                                                'out_octets': '1723790967',
                                                                                                'out_ucast_pkts': '2741088',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6891',
                                                                       'out_octets': '1723790967',
                                                                       'out_ucast_pkts': '2741088',
                                                                       'poller_hash': 'QURUTjI0MzE1OEZBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206921',
                                                                                                                        'fiber_length': 11449,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1OEZBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x7-ONU4',
                                         'poller_hash': 'b250IEFEVE4yNDMxNThGQQ==',
                                         'serial_number': 'ADTN243158FA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206921'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '299933',
                                                                       'in_octets': '321235332415',
                                                                       'in_ucast_pkts': '733356586',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x16-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '26',
                                                                       'out_octets': '2863523369542',
                                                                       'out_ucast_pkts': '2086051068',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDE2LU9OVTZfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '26',
                                                                       'in_octets': '2863523369542',
                                                                       'in_ucast_pkts': '2086051068',
                                                                       'links': [       {       'in_nucast_pkts': '26',
                                                                                                'in_octets': '2863523369542',
                                                                                                'in_ucast_pkts': '2086051068',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '299933',
                                                                                                'out_octets': '321235332415',
                                                                                                'out_ucast_pkts': '733356586',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '299933',
                                                                       'out_octets': '321235332415',
                                                                       'out_ucast_pkts': '733356586',
                                                                       'poller_hash': 'QURUTjI0MzE1OUU1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9026294',
                                                                                                                        'fiber_length': 10175,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1OUU1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x16-ONU6',
                                         'poller_hash': 'b250IEFEVE4yNDMxNTlFNQ==',
                                         'serial_number': 'ADTN243159E5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '9026294'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '74572',
                                                                       'in_octets': '30850596248',
                                                                       'in_ucast_pkts': '56146805',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x9-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '306723675869',
                                                                       'out_ucast_pkts': '227068520',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDktT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '306723675869',
                                                                       'in_ucast_pkts': '227068520',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '306723675869',
                                                                                                'in_ucast_pkts': '227068520',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '74572',
                                                                                                'out_octets': '30850596248',
                                                                                                'out_ucast_pkts': '56146805',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '74572',
                                                                       'out_octets': '30850596248',
                                                                       'out_ucast_pkts': '56146805',
                                                                       'poller_hash': 'QURUTjIzMjIxNjQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2236837',
                                                                                                                        'fiber_length': 1360,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMjIxNjQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x9-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzIyMTY0OQ==',
                                         'serial_number': 'ADTN23221649',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '2236837'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6893',
                                                                       'in_octets': '15263930760',
                                                                       'in_ucast_pkts': '24578082',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '94619947938',
                                                                       'out_ucast_pkts': '75160872',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VOF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '94619947938',
                                                                       'in_ucast_pkts': '75160872',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '94619947938',
                                                                                                'in_ucast_pkts': '75160872',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6893',
                                                                                                'out_octets': '15263930760',
                                                                                                'out_ucast_pkts': '24578082',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6893',
                                                                       'out_octets': '15263930760',
                                                                       'out_ucast_pkts': '24578082',
                                                                       'poller_hash': 'QURUTjI0MzE1OEE1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206920',
                                                                                                                        'fiber_length': 11475,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1OEE1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x5-ONU8',
                                         'poller_hash': 'b250IEFEVE4yNDMxNThBNQ==',
                                         'serial_number': 'ADTN243158A5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206920'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6136',
                                                                       'in_octets': '295915251',
                                                                       'in_ucast_pkts': '1410902',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '12576670318',
                                                                       'out_ucast_pkts': '8838864',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VOV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '12576670318',
                                                                       'in_ucast_pkts': '8838864',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '12576670318',
                                                                                                'in_ucast_pkts': '8838864',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6136',
                                                                                                'out_octets': '295915251',
                                                                                                'out_ucast_pkts': '1410902',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6136',
                                                                       'out_octets': '295915251',
                                                                       'out_ucast_pkts': '1410902',
                                                                       'poller_hash': 'QURUTjI0MzE1QzBCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206914',
                                                                                                                        'fiber_length': 12572,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1QzBCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x5-ONU9',
                                         'poller_hash': 'b250IEFEVE4yNDMxNUMwQg==',
                                         'serial_number': 'ADTN24315C0B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206914'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '48585',
                                                                       'in_octets': '432033988003',
                                                                       'in_ucast_pkts': '1254602937',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x10-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17',
                                                                       'out_octets': '3673298764763',
                                                                       'out_ucast_pkts': '2776210294',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDEwLU9OVTNfRVRIXzE=',
                                                                       'speed': '10000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '17',
                                                                       'in_octets': '3673298764763',
                                                                       'in_ucast_pkts': '2776210294',
                                                                       'links': [       {       'in_nucast_pkts': '17',
                                                                                                'in_octets': '3673298764763',
                                                                                                'in_ucast_pkts': '2776210294',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '48585',
                                                                                                'out_octets': '432033988003',
                                                                                                'out_ucast_pkts': '1254602937',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '48585',
                                                                       'out_octets': '432033988003',
                                                                       'out_ucast_pkts': '1254602937',
                                                                       'poller_hash': 'QURUTjI0MzE1OTQ4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5908159',
                                                                                                                        'fiber_length': 2534,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1OTQ4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x10-ONU3',
                                         'poller_hash': 'b250IEFEVE4yNDMxNTk0OA==',
                                         'serial_number': 'ADTN24315948',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '5908159'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6891',
                                                                       'in_octets': '1599169368',
                                                                       'in_ucast_pkts': '2219771',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x7-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '3638074989',
                                                                       'out_ucast_pkts': '3033043',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDctT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '3638074989',
                                                                       'in_ucast_pkts': '3033043',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '3638074989',
                                                                                                'in_ucast_pkts': '3033043',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6891',
                                                                                                'out_octets': '1599169368',
                                                                                                'out_ucast_pkts': '2219771',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6891',
                                                                       'out_octets': '1599169368',
                                                                       'out_ucast_pkts': '2219771',
                                                                       'poller_hash': 'QURUTjI0MzE1QjBFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206921',
                                                                                                                        'fiber_length': 10867,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1QjBFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x7-ONU5',
                                         'poller_hash': 'b250IEFEVE4yNDMxNUIwRQ==',
                                         'serial_number': 'ADTN24315B0E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206921'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6893',
                                                                       'in_octets': '13725175614',
                                                                       'in_ucast_pkts': '25214964',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x9-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '64942887137',
                                                                       'out_ucast_pkts': '47888503',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDktT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '64942887137',
                                                                       'in_ucast_pkts': '47888503',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '64942887137',
                                                                                                'in_ucast_pkts': '47888503',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6893',
                                                                                                'out_octets': '13725175614',
                                                                                                'out_ucast_pkts': '25214964',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6893',
                                                                       'out_octets': '13725175614',
                                                                       'out_ucast_pkts': '25214964',
                                                                       'poller_hash': 'QURUTjI0MzE1OEY0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206922',
                                                                                                                        'fiber_length': 1606,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1OEY0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x9-ONU6',
                                         'poller_hash': 'b250IEFEVE4yNDMxNThGNA==',
                                         'serial_number': 'ADTN243158F4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206922'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6913',
                                                                       'in_octets': '1240930819',
                                                                       'in_ucast_pkts': '1496449',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x6-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1134399324',
                                                                       'out_ucast_pkts': '1548196',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDYtT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1134399324',
                                                                       'in_ucast_pkts': '1548196',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1134399324',
                                                                                                'in_ucast_pkts': '1548196',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6913',
                                                                                                'out_octets': '1240930819',
                                                                                                'out_ucast_pkts': '1496449',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6913',
                                                                       'out_octets': '1240930819',
                                                                       'out_ucast_pkts': '1496449',
                                                                       'poller_hash': 'QURUTjI0MzE1QkVFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206921',
                                                                                                                        'fiber_length': 10516,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1QkVFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x6-ONU6',
                                         'poller_hash': 'b250IEFEVE4yNDMxNUJFRQ==',
                                         'serial_number': 'ADTN24315BEE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206921'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6912',
                                                                       'in_octets': '4212425014',
                                                                       'in_ucast_pkts': '7070404',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '26673574979',
                                                                       'out_ucast_pkts': '19634393',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VMTBfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '26673574979',
                                                                       'in_ucast_pkts': '19634393',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '26673574979',
                                                                                                'in_ucast_pkts': '19634393',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6912',
                                                                                                'out_octets': '4212425014',
                                                                                                'out_ucast_pkts': '7070404',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6912',
                                                                       'out_octets': '4212425014',
                                                                       'out_ucast_pkts': '7070404',
                                                                       'poller_hash': 'QURUTjI0MzE1OUM2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206916',
                                                                                                                        'fiber_length': 11660,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1OUM2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x5-ONU10',
                                         'poller_hash': 'b250IEFEVE4yNDMxNTlDNg==',
                                         'serial_number': 'ADTN243159C6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206916'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6910',
                                                                       'in_octets': '3701614432',
                                                                       'in_ucast_pkts': '19536998',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '129073834011',
                                                                       'out_ucast_pkts': '90093128',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VMTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '129073834011',
                                                                       'in_ucast_pkts': '90093128',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '129073834011',
                                                                                                'in_ucast_pkts': '90093128',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6910',
                                                                                                'out_octets': '3701614432',
                                                                                                'out_ucast_pkts': '19536998',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6910',
                                                                       'out_octets': '3701614432',
                                                                       'out_ucast_pkts': '19536998',
                                                                       'poller_hash': 'QURUTjI0MzE1QjlCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206911',
                                                                                                                        'fiber_length': 12568,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1QjlCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x5-ONU11',
                                         'poller_hash': 'b250IEFEVE4yNDMxNUI5Qg==',
                                         'serial_number': 'ADTN24315B9B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206911'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '91732',
                                                                       'in_octets': '38369584713',
                                                                       'in_ucast_pkts': '88848776',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x3-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '394459532009',
                                                                       'out_ucast_pkts': '279410318',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDMtT05VN19FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '394459532009',
                                                                       'in_ucast_pkts': '279410318',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '394459532009',
                                                                                                'in_ucast_pkts': '279410318',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '91732',
                                                                                                'out_octets': '38369584713',
                                                                                                'out_ucast_pkts': '88848776',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '91732',
                                                                       'out_octets': '38369584713',
                                                                       'out_ucast_pkts': '88848776',
                                                                       'poller_hash': 'QURUTjI0MTYwMDU3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2723488',
                                                                                                                        'fiber_length': 10138,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MTYwMDU3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x3-ONU7',
                                         'poller_hash': 'b250IEFEVE4yNDE2MDA1Nw==',
                                         'serial_number': 'ADTN24160057',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-2-g',
                                         'uptime': '2723488'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '20177',
                                                                       'in_octets': '8340237255',
                                                                       'in_ucast_pkts': '20275285',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x10-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '98518662832',
                                                                       'out_ucast_pkts': '74426625',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDEwLU9OVTFfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '98518662832',
                                                                       'in_ucast_pkts': '74426625',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '98518662832',
                                                                                                'in_ucast_pkts': '74426625',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '20177',
                                                                                                'out_octets': '8340237255',
                                                                                                'out_ucast_pkts': '20275285',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '20177',
                                                                       'out_octets': '8340237255',
                                                                       'out_ucast_pkts': '20275285',
                                                                       'poller_hash': 'QURUTjI0MzE1QkJEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '605160',
                                                                                                                        'fiber_length': 2799,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1QkJEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x10-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDMxNUJCRA==',
                                         'serial_number': 'ADTN24315BBD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '605160'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '31769',
                                                                       'in_octets': '55467852678',
                                                                       'in_ucast_pkts': '407752377',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x16-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '2272910568794',
                                                                       'out_ucast_pkts': '1585293024',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDE2LU9OVTdfRVRIXzE=',
                                                                       'speed': '10000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '2272910568794',
                                                                       'in_ucast_pkts': '1585293024',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '2272910568794',
                                                                                                'in_ucast_pkts': '1585293024',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '31769',
                                                                                                'out_octets': '55467852678',
                                                                                                'out_ucast_pkts': '407752377',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '31769',
                                                                       'out_octets': '55467852678',
                                                                       'out_ucast_pkts': '407752377',
                                                                       'poller_hash': 'QURUTjI0MzE1QTUxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3822939',
                                                                                                                        'fiber_length': 10256,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1QTUxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x16-ONU7',
                                         'poller_hash': 'b250IEFEVE4yNDMxNUE1MQ==',
                                         'serial_number': 'ADTN24315A51',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '3822939'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '196854',
                                                                       'in_octets': '60520873610',
                                                                       'in_ucast_pkts': '104712196',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x10-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '17',
                                                                       'out_octets': '464654638005',
                                                                       'out_ucast_pkts': '341504519',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDEwLU9OVTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '17',
                                                                       'in_octets': '464654638005',
                                                                       'in_ucast_pkts': '341504519',
                                                                       'links': [       {       'in_nucast_pkts': '17',
                                                                                                'in_octets': '464654638005',
                                                                                                'in_ucast_pkts': '341504519',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '196854',
                                                                                                'out_octets': '60520873610',
                                                                                                'out_ucast_pkts': '104712196',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '196854',
                                                                       'out_octets': '60520873610',
                                                                       'out_ucast_pkts': '104712196',
                                                                       'poller_hash': 'QURUTjI0MzE1QUM1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5908148',
                                                                                                                        'fiber_length': 2864,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1QUM1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x10-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDMxNUFDNQ==',
                                         'serial_number': 'ADTN24315AC5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '5908148'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '19400',
                                                                       'in_octets': '8398195044',
                                                                       'in_ucast_pkts': '13260049',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x7-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '65479166013',
                                                                       'out_ucast_pkts': '50562876',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDctT05VNl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '65479166013',
                                                                       'in_ucast_pkts': '50562876',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '65479166013',
                                                                                                'in_ucast_pkts': '50562876',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '19400',
                                                                                                'out_octets': '8398195044',
                                                                                                'out_ucast_pkts': '13260049',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '19400',
                                                                       'out_octets': '8398195044',
                                                                       'out_ucast_pkts': '13260049',
                                                                       'poller_hash': 'QURUTjI0MzE1QkNCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '575896',
                                                                                                                        'fiber_length': 9690,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1QkNCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x7-ONU6',
                                         'poller_hash': 'b250IEFEVE4yNDMxNUJDQg==',
                                         'serial_number': 'ADTN24315BCB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '575896'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-6.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '1716',
                                                                       'in_octets': '980352418',
                                                                       'in_ucast_pkts': '1189294',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x7-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '869742044',
                                                                       'out_ucast_pkts': '1179019',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDctT05VMF9FVEhfMQ==',
                                                                       'speed': '10000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '869742044',
                                                                       'in_ucast_pkts': '1179019',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '869742044',
                                                                                                'in_ucast_pkts': '1179019',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '1716',
                                                                                                'out_octets': '980352418',
                                                                                                'out_ucast_pkts': '1189294',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC83'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1716',
                                                                       'out_octets': '980352418',
                                                                       'out_ucast_pkts': '1189294',
                                                                       'poller_hash': 'QURUTjI0MzE1OTlGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '206913',
                                                                                                                        'fiber_length': 11739,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1OTlGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x7-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDMxNTk5Rg==',
                                         'serial_number': 'ADTN2431599F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '206913'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '11317',
                                                                       'in_octets': '54740200397',
                                                                       'in_ucast_pkts': '90335240',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x9-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '204672909790',
                                                                       'out_ucast_pkts': '190254684',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDktT05VNV9FVEhfMQ==',
                                                                       'speed': '10000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '204672909790',
                                                                       'in_ucast_pkts': '190254684',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '204672909790',
                                                                                                'in_ucast_pkts': '190254684',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '11317',
                                                                                                'out_octets': '54740200397',
                                                                                                'out_ucast_pkts': '90335240',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '11317',
                                                                       'out_octets': '54740200397',
                                                                       'out_ucast_pkts': '90335240',
                                                                       'poller_hash': 'QURUTjI0MzE1QjVCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1375369',
                                                                                                                        'fiber_length': 1687,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1QjVCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x9-ONU5',
                                         'poller_hash': 'b250IEFEVE4yNDMxNUI1Qg==',
                                         'serial_number': 'ADTN24315B5B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '1375369'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '26588',
                                                                       'in_octets': '30145450304',
                                                                       'in_ucast_pkts': '120560216',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x6-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '523603012985',
                                                                       'out_ucast_pkts': '392718548',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDYtT05VOF9FVEhfMQ==',
                                                                       'speed': '10000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '523603012985',
                                                                       'in_ucast_pkts': '392718548',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '523603012985',
                                                                                                'in_ucast_pkts': '392718548',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '26588',
                                                                                                'out_octets': '30145450304',
                                                                                                'out_ucast_pkts': '120560216',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '26588',
                                                                       'out_octets': '30145450304',
                                                                       'out_ucast_pkts': '120560216',
                                                                       'poller_hash': 'QURUTjI0MzE1Qjc3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3195115',
                                                                                                                        'fiber_length': 10130,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1Qjc3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x6-ONU8',
                                         'poller_hash': 'b250IEFEVE4yNDMxNUI3Nw==',
                                         'serial_number': 'ADTN24315B77',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '3195115'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-10.1.1.0.eth.1.phy',
                                                                       'name': 'COURT-OLT02-x11-ONU1_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDExLU9OVTFfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjI0MzAwNDNEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwNDNEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'COURT-OLT02-x11-ONU1',
                                         'offline_reason': 'OFFLINE_REASON_POWER_OFF',
                                         'offline_reason_detail': 'A local power '
                                                                  'outage has occurred '
                                                                  'for ONU "onu '
                                                                  '10.1.1"(ADTN2430043D).',
                                         'offline_time': '2025-07-22T22:27:37Z',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDQzRA==',
                                         'serial_number': 'ADTN2430043D',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '30037',
                                                                       'in_octets': '56944581543',
                                                                       'in_ucast_pkts': '281172927',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x10-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1329089123637',
                                                                       'out_ucast_pkts': '956083166',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDEwLU9OVTRfRVRIXzE=',
                                                                       'speed': '10000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1329089123637',
                                                                       'in_ucast_pkts': '956083166',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1329089123637',
                                                                                                'in_ucast_pkts': '956083166',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '30037',
                                                                                                'out_octets': '56944581543',
                                                                                                'out_ucast_pkts': '281172927',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '30037',
                                                                       'out_octets': '56944581543',
                                                                       'out_ucast_pkts': '281172927',
                                                                       'poller_hash': 'QURUTjI0MzE1QzI4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '3627734',
                                                                                                                        'fiber_length': 2599,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE1QzI4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x10-ONU4',
                                         'poller_hash': 'b250IEFEVE4yNDMxNUMyOA==',
                                         'serial_number': 'ADTN24315C28',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '3627734'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '18725',
                                                                       'in_octets': '19143027006',
                                                                       'in_ucast_pkts': '129203556',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x10-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '439504573732',
                                                                       'out_ucast_pkts': '310649488',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDEwLU9OVTVfRVRIXzE=',
                                                                       'speed': '10000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '439504573732',
                                                                       'in_ucast_pkts': '310649488',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '439504573732',
                                                                                                'in_ucast_pkts': '310649488',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '18725',
                                                                                                'out_octets': '19143027006',
                                                                                                'out_ucast_pkts': '129203556',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '18725',
                                                                       'out_octets': '19143027006',
                                                                       'out_ucast_pkts': '129203556',
                                                                       'poller_hash': 'QURUTjI1MDIzOUI3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2270614',
                                                                                                                        'fiber_length': 2406,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI1MDIzOUI3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x10-ONU5',
                                         'poller_hash': 'b250IEFEVE4yNTAyMzlCNw==',
                                         'serial_number': 'ADTN250239B7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '2270614'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '6139',
                                                                       'in_octets': '18541776867',
                                                                       'in_ucast_pkts': '103601902',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x13-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '296637308288',
                                                                       'out_ucast_pkts': '215189445',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDEzLU9OVTBfRVRIXzE=',
                                                                       'speed': '10000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '296637308288',
                                                                       'in_ucast_pkts': '215189445',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '296637308288',
                                                                                                'in_ucast_pkts': '215189445',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '6139',
                                                                                                'out_octets': '18541776867',
                                                                                                'out_ucast_pkts': '103601902',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '6139',
                                                                       'out_octets': '18541776867',
                                                                       'out_ucast_pkts': '103601902',
                                                                       'poller_hash': 'QURUTjI1MDU1M0E4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '720719',
                                                                                                                        'fiber_length': 6839,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI1MDU1M0E4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 630',
                                         'name': 'COURT-OLT02-x13-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNTA1NTNBOA==',
                                         'serial_number': 'ADTN250553A8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '24_3-1-c',
                                         'uptime': '720719'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '2596',
                                                                       'in_octets': '1774795493',
                                                                       'in_ucast_pkts': '1460464',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'COURT-OLT02-x5-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '2590042477',
                                                                       'out_ucast_pkts': '2082155',
                                                                       'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgQ09VUlQtT0xUMDIteDUtT05VMTJfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '2590042477',
                                                                       'in_ucast_pkts': '2082155',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '2590042477',
                                                                                                'in_ucast_pkts': '2082155',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '2596',
                                                                                                'out_octets': '1774795493',
                                                                                                'out_ucast_pkts': '1460464',
                                                                                                'poller_hash': 'TEJBRFROMjMxMUFDNDMzIGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2596',
                                                                       'out_octets': '1774795493',
                                                                       'out_ucast_pkts': '1460464',
                                                                       'poller_hash': 'QURUTjI0MzAwNTVDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '77607',
                                                                                                                        'fiber_length': 11204,
                                                                                                                        'poller_hash': 'TEJBRFROMjMxMUFDNDMzIG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.1}],
                                                                                               'max_distance': 20000,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzAwNTVDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': '1287970F1',
                                         'name': 'COURT-OLT02-x5-ONU12',
                                         'poller_hash': 'b250IEFEVE4yNDMwMDU1Qw==',
                                         'serial_number': 'ADTN2430055C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_1-5-g',
                                         'uptime': '77607'}]
        }
        id_field = 'mac_address' if not obj.stations else ('mac_address' if hasattr(obj.stations[0], 'mac_address') and getattr(obj.stations[0], 'mac_address') else ('imsi' if hasattr(obj.stations[0], 'imsi') else 'serial'))
        cur_info = {x: getattr(obj, x, None) for (x, data_type) in self.apdd.get_metric_fields('info', 'ap', obj.model) + [('vendor', 'str'), ('model', 'str'), ('sw_version', 'str')]}
        if 'rf_freq' in cur_info and cur_info.get('rf_freq') and (cur_info['rf_freq'] <= 0 or int.bit_length(cur_info['rf_freq']) >= 32):
            cur_info.pop('rf_freq')
        cur_stats = {x: getattr(obj, x, None) for (x, data_type) in self.apdd.get_metric_fields('stats', 'ap', obj.model)}
        cur_sta_stats = {getattr(sta, id_field): {x: getattr(sta, x, None) for (x, data_type) in sorted(self.apdd.get_metric_fields('stats', 'cpe', obj.model)) + [(id_field, 'str'), ('ip_address', 'str'), ('name', 'str'), ('dev_macs', 'str')]} for sta in sorted(obj.stations)}
        cur_ne_msg = getattr(obj, 'network_element_update_msg', None)
        cur_ne_msg = cur_ne_msg.pb if cur_ne_msg else None
        self.assertEqual(cur_info, exp_info)
        self.assertEqual(cur_stats, exp_stats)
        self.assertEqual(cur_sta_stats, exp_sta_stats)
        self.assertTrue(self.compare_pb_dict(cur_ne_msg, exp_ne_msg))

    def test_adtran_sdx6000_SDX_6330_48_sdx6330_RL_23_5_30_10_59_5_252(self):
        if DeviceTestCase.TestDisabled:
                DeviceTestCase.skipTest(self, "Python 3.7 doesn't support this library")
        obj = self.run_data("adtran.sdx6000.SDX 6330-48.sdx6330-RL-23.5-30.10_59_5_252")
        exp_info = {
                'model': 'SDX 6330-48',
                'sta_count': 173,
                'sw_version': 'sdx6330-RL-23.5-30',
                'vendor': 'Adtran'
        }
        exp_stats = {
         
        }
        exp_sta_stats = {
                'ADTN23123E8E': {       'dev_macs': ['cc:66:18:51:29:d0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU7',
                                        'serial': 'ADTN23123E8E'},
                'ADTN23123ED1': {       'dev_macs': ['38:f8:f6:52:a1:e0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x5-ONU7',
                                        'serial': 'ADTN23123ED1'},
                'ADTN23123F0D': {       'dev_macs': ['cc:66:18:50:fe:90'],
                                        'ip_address': None,
                                        'name': 'Maurice-x9-ONU1',
                                        'serial': 'ADTN23123F0D'},
                'ADTN23124048': {       'dev_macs': ['cc:66:18:50:64:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU4',
                                        'serial': 'ADTN23124048'},
                'ADTN23124085': {       'dev_macs': ['38:f8:f6:52:a2:40'],
                                        'ip_address': None,
                                        'name': 'Maurice-x27-ONU0',
                                        'serial': 'ADTN23124085'},
                'ADTN23124115': {       'dev_macs': ['38:f8:f6:52:a2:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU3',
                                        'serial': 'ADTN23124115'},
                'ADTN2312418F': {       'dev_macs': ['cc:66:18:51:51:b0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x41-ONU0',
                                        'serial': 'ADTN2312418F'},
                'ADTN231241A6': {       'dev_macs': ['cc:66:18:50:34:10'],
                                        'ip_address': None,
                                        'name': 'Maurice-x18-ONU1',
                                        'serial': 'ADTN231241A6'},
                'ADTN23124261': {       'dev_macs': ['cc:66:18:50:40:f0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x43-ONU0',
                                        'serial': 'ADTN23124261'},
                'ADTN23124313': {       'dev_macs': ['38:f8:f6:52:ac:c0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x34-ONU0',
                                        'serial': 'ADTN23124313'},
                'ADTN2312441B': {       'dev_macs': ['38:f8:f6:52:a9:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x43-ONU1',
                                        'serial': 'ADTN2312441B'},
                'ADTN2312466B': {       'dev_macs': ['cc:66:18:51:0a:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU4',
                                        'serial': 'ADTN2312466B'},
                'ADTN23124746': {       'dev_macs': ['cc:66:18:50:7a:f0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU8',
                                        'serial': 'ADTN23124746'},
                'ADTN23124785': {       'dev_macs': ['cc:66:18:51:03:90'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU3',
                                        'serial': 'ADTN23124785'},
                'ADTN231247B0': {       'dev_macs': ['38:f8:f6:52:ae:80'],
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU3',
                                        'serial': 'ADTN231247B0'},
                'ADTN231247D6': {       'dev_macs': ['cc:66:18:50:ac:30'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU0',
                                        'serial': 'ADTN231247D6'},
                'ADTN23124802': {       'dev_macs': ['38:f8:f6:52:a5:80'],
                                        'ip_address': None,
                                        'name': 'Maurice-x35-ONU0',
                                        'serial': 'ADTN23124802'},
                'ADTN23124838': {       'dev_macs': ['cc:66:18:50:b6:80'],
                                        'ip_address': None,
                                        'name': 'Maurice-x15-ONU1',
                                        'serial': 'ADTN23124838'},
                'ADTN2312486D': {       'dev_macs': ['cc:66:18:51:05:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x14-ONU1',
                                        'serial': 'ADTN2312486D'},
                'ADTN23124891': {       'dev_macs': ['cc:66:18:50:5c:b0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU6',
                                        'serial': 'ADTN23124891'},
                'ADTN231248AD': {       'dev_macs': ['cc:66:18:51:51:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x29-ONU0',
                                        'serial': 'ADTN231248AD'},
                'ADTN231248D7': {       'dev_macs': ['cc:66:18:51:22:50'],
                                        'ip_address': None,
                                        'name': 'Maurice-x14-ONU4',
                                        'serial': 'ADTN231248D7'},
                'ADTN231248E2': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x14-ONU6',
                                        'serial': 'ADTN231248E2'},
                'ADTN2312491B': {       'dev_macs': ['38:f8:f6:52:a8:d0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x39-ONU2',
                                        'serial': 'ADTN2312491B'},
                'ADTN2312498A': {       'dev_macs': ['cc:66:18:51:39:d0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x14-ONU3',
                                        'serial': 'ADTN2312498A'},
                'ADTN231249DD': {       'dev_macs': ['8c:30:66:a0:51:91'],
                                        'ip_address': None,
                                        'name': 'Maurice-x44-ONU0',
                                        'serial': 'ADTN231249DD'},
                'ADTN231249DF': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x45-ONU0',
                                        'serial': 'ADTN231249DF'},
                'ADTN23124A0B': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x44-ONU2',
                                        'serial': 'ADTN23124A0B'},
                'ADTN23124A19': {       'dev_macs': ['cc:66:18:51:4c:30'],
                                        'ip_address': None,
                                        'name': 'Maurice-x40-ONU0',
                                        'serial': 'ADTN23124A19'},
                'ADTN23124A1D': {       'dev_macs': ['cc:66:18:51:0e:c0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU6',
                                        'serial': 'ADTN23124A1D'},
                'ADTN23124A2F': {       'dev_macs': [],
                                        'ip_address': None,
                                        'name': 'Maurice-x3-ONU3',
                                        'serial': 'ADTN23124A2F'},
                'ADTN23124A94': {       'dev_macs': ['38:f8:f6:52:a9:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x44-ONU1',
                                        'serial': 'ADTN23124A94'},
                'ADTN23124ACA': {       'dev_macs': ['cc:66:18:50:4d:a0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x13-ONU0',
                                        'serial': 'ADTN23124ACA'},
                'ADTN23124B25': {       'dev_macs': ['38:f8:f6:52:ac:f0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU1',
                                        'serial': 'ADTN23124B25'},
                'ADTN23124B6D': {       'dev_macs': ['cc:66:18:51:50:b0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU1',
                                        'serial': 'ADTN23124B6D'},
                'ADTN23124B75': {       'dev_macs': ['38:f8:f6:52:a1:40'],
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU6',
                                        'serial': 'ADTN23124B75'},
                'ADTN23124B7C': {       'dev_macs': ['38:f8:f6:52:a6:f0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x31-ONU0',
                                        'serial': 'ADTN23124B7C'},
                'ADTN23124BB8': {       'dev_macs': ['cc:66:18:50:6e:c0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x39-ONU0',
                                        'serial': 'ADTN23124BB8'},
                'ADTN23124BEE': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU5',
                                        'serial': 'ADTN23124BEE'},
                'ADTN23124BFD': {       'dev_macs': ['cc:66:18:51:32:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x15-ONU2',
                                        'serial': 'ADTN23124BFD'},
                'ADTN23124C47': {       'dev_macs': ['cc:66:18:50:82:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x39-ONU1',
                                        'serial': 'ADTN23124C47'},
                'ADTN23124CB3': {       'dev_macs': ['38:f8:f6:52:ad:e0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x41-ONU2',
                                        'serial': 'ADTN23124CB3'},
                'ADTN23124CE5': {       'dev_macs': ['cc:66:18:50:a7:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU0',
                                        'serial': 'ADTN23124CE5'},
                'ADTN23124DAC': {       'dev_macs': ['38:f8:f6:52:af:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU4',
                                        'serial': 'ADTN23124DAC'},
                'ADTN23124DCC': {       'dev_macs': ['cc:66:18:d6:3b:f0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU2',
                                        'serial': 'ADTN23124DCC'},
                'ADTN23124DF7': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x43-ONU4',
                                        'serial': 'ADTN23124DF7'},
                'ADTN23124E05': {       'dev_macs': ['38:f8:f6:52:ac:80'],
                                        'ip_address': None,
                                        'name': 'Maurice-x36-ONU0',
                                        'serial': 'ADTN23124E05'},
                'ADTN23124E0A': {       'dev_macs': ['38:f8:f6:52:a8:70'],
                                        'ip_address': None,
                                        'name': 'Maurice-x12-ONU0',
                                        'serial': 'ADTN23124E0A'},
                'ADTN23124E0D': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x43-ONU6',
                                        'serial': 'ADTN23124E0D'},
                'ADTN23124E16': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x43-ONU5',
                                        'serial': 'ADTN23124E16'},
                'ADTN23124E26': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x43-ONU7',
                                        'serial': 'ADTN23124E26'},
                'ADTN23124E63': {       'dev_macs': ['cc:66:18:51:27:c0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x20-ONU0',
                                        'serial': 'ADTN23124E63'},
                'ADTN23124F11': {       'dev_macs': ['cc:66:18:51:13:c0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x18-ONU0',
                                        'serial': 'ADTN23124F11'},
                'ADTN23124F3E': {       'dev_macs': ['cc:66:18:dc:a9:90'],
                                        'ip_address': None,
                                        'name': 'Maurice-x30-ONU0',
                                        'serial': 'ADTN23124F3E'},
                'ADTN23125030': {       'dev_macs': ['cc:66:18:9c:2f:90'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU5',
                                        'serial': 'ADTN23125030'},
                'ADTN231250CE': {       'dev_macs': ['cc:66:18:51:03:40'],
                                        'ip_address': None,
                                        'name': 'Maurice-x38-ONU0',
                                        'serial': 'ADTN231250CE'},
                'ADTN23125139': {       'dev_macs': ['38:f8:f6:52:ad:70'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU2',
                                        'serial': 'ADTN23125139'},
                'ADTN23125144': {       'dev_macs': ['cc:66:18:50:58:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU7',
                                        'serial': 'ADTN23125144'},
                'ADTN231449BB': {       'dev_macs': ['cc:66:18:51:25:10'],
                                        'ip_address': None,
                                        'name': 'Maurice-x4-ONU1',
                                        'serial': 'ADTN231449BB'},
                'ADTN23144A18': {       'dev_macs': ['cc:66:18:50:a1:b0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x5-ONU6',
                                        'serial': 'ADTN23144A18'},
                'ADTN23144A24': {       'dev_macs': ['cc:66:18:50:a7:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x5-ONU2',
                                        'serial': 'ADTN23144A24'},
                'ADTN23144A29': {       'dev_macs': ['38:f8:f6:52:b2:30'],
                                        'ip_address': None,
                                        'name': 'Maurice-x6-ONU1',
                                        'serial': 'ADTN23144A29'},
                'ADTN23144A48': {       'dev_macs': ['cc:66:18:51:13:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x3-ONU2',
                                        'serial': 'ADTN23144A48'},
                'ADTN23144A66': {       'dev_macs': ['cc:66:18:51:06:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x14-ONU0',
                                        'serial': 'ADTN23144A66'},
                'ADTN23144A85': {       'dev_macs': ['cc:66:18:50:59:e0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x9-ONU0',
                                        'serial': 'ADTN23144A85'},
                'ADTN23144A98': {       'dev_macs': ['cc:66:18:d6:3e:f0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x37-ONU1',
                                        'serial': 'ADTN23144A98'},
                'ADTN23144A9D': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x1-ONU0',
                                        'serial': 'ADTN23144A9D'},
                'ADTN23144B2F': {       'dev_macs': ['cc:66:18:51:18:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x3-ONU1',
                                        'serial': 'ADTN23144B2F'},
                'ADTN23144B38': {       'dev_macs': ['38:f8:f6:52:ad:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x4-ONU2',
                                        'serial': 'ADTN23144B38'},
                'ADTN23144B79': {       'dev_macs': ['cc:66:18:50:5b:f0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x11-ONU0',
                                        'serial': 'ADTN23144B79'},
                'ADTN23144BA4': {       'dev_macs': ['cc:66:18:50:28:80'],
                                        'ip_address': None,
                                        'name': 'Maurice-x5-ONU5',
                                        'serial': 'ADTN23144BA4'},
                'ADTN23144CB6': {       'dev_macs': ['cc:66:18:dc:5d:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x5-ONU4',
                                        'serial': 'ADTN23144CB6'},
                'ADTN23144CD9': {       'dev_macs': ['cc:66:18:dc:a9:10'],
                                        'ip_address': None,
                                        'name': 'Maurice-x4-ONU3',
                                        'serial': 'ADTN23144CD9'},
                'ADTN23144D47': {       'dev_macs': ['cc:66:18:d6:36:d0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x1-ONU1',
                                        'serial': 'ADTN23144D47'},
                'ADTN23144E7A': {       'dev_macs': ['cc:66:18:dd:83:70'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU14',
                                        'serial': 'ADTN23144E7A'},
                'ADTN23144E7B': {       'dev_macs': ['cc:66:18:4c:02:e0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU11',
                                        'serial': 'ADTN23144E7B'},
                'ADTN23144E95': {       'dev_macs': ['c4:41:1e:6d:d4:36'],
                                        'ip_address': None,
                                        'name': 'Maurice-x1-ONU3',
                                        'serial': 'ADTN23144E95'},
                'ADTN23144ED5': {       'dev_macs': ['e8:9f:80:eb:9e:5b'],
                                        'ip_address': None,
                                        'name': 'Maurice-x5-ONU0',
                                        'serial': 'ADTN23144ED5'},
                'ADTN23144EE7': {       'dev_macs': ['cc:66:18:51:4f:e0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x3-ONU0',
                                        'serial': 'ADTN23144EE7'},
                'ADTN23144EF2': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x48-ONU0',
                                        'serial': 'ADTN23144EF2'},
                'ADTN23144EFA': {       'dev_macs': ['cc:66:18:50:bb:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x4-ONU0',
                                        'serial': 'ADTN23144EFA'},
                'ADTN23144F1F': {       'dev_macs': ['cc:66:18:51:2d:a0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x5-ONU3',
                                        'serial': 'ADTN23144F1F'},
                'ADTN2314515D': {       'dev_macs': ['cc:66:18:50:ff:a0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x44-ONU7',
                                        'serial': 'ADTN2314515D'},
                'ADTN231451E9': {       'dev_macs': ['cc:66:18:4b:fa:a0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x5-ONU1',
                                        'serial': 'ADTN231451E9'},
                'ADTN23145232': {       'dev_macs': ['38:f8:f6:52:b1:10'],
                                        'ip_address': None,
                                        'name': 'Maurice-x2-ONU0',
                                        'serial': 'ADTN23145232'},
                'ADTN231452FD': {       'dev_macs': ['cc:66:18:50:5c:10'],
                                        'ip_address': None,
                                        'name': 'Maurice-x6-ONU0',
                                        'serial': 'ADTN231452FD'},
                'ADTN24314158': {       'dev_macs': ['cc:66:18:50:60:c0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x37-ONU4',
                                        'serial': 'ADTN24314158'},
                'ADTN2431416E': {       'dev_macs': ['cc:66:18:50:42:30'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU11',
                                        'serial': 'ADTN2431416E'},
                'ADTN24314189': {       'dev_macs': ['cc:66:18:50:cc:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x39-ONU8',
                                        'serial': 'ADTN24314189'},
                'ADTN2431419B': {       'dev_macs': ['cc:66:18:50:b7:b0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x20-ONU1',
                                        'serial': 'ADTN2431419B'},
                'ADTN243141AA': {       'dev_macs': ['cc:66:18:51:04:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x22-ONU1',
                                        'serial': 'ADTN243141AA'},
                'ADTN243141B8': {       'dev_macs': ['cc:66:18:51:30:90'],
                                        'ip_address': None,
                                        'name': 'Maurice-x26-ONU0',
                                        'serial': 'ADTN243141B8'},
                'ADTN243141B9': {       'dev_macs': ['cc:66:18:51:29:b0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU12',
                                        'serial': 'ADTN243141B9'},
                'ADTN243141C7': {       'dev_macs': ['cc:66:18:50:61:b0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x37-ONU5',
                                        'serial': 'ADTN243141C7'},
                'ADTN243141E0': {       'dev_macs': ['cc:66:18:50:50:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x32-ONU2',
                                        'serial': 'ADTN243141E0'},
                'ADTN243141EF': {       'dev_macs': ['cc:66:18:51:35:40'],
                                        'ip_address': None,
                                        'name': 'Maurice-x22-ONU0',
                                        'serial': 'ADTN243141EF'},
                'ADTN243141F7': {       'dev_macs': ['cc:66:18:50:42:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x38-ONU3',
                                        'serial': 'ADTN243141F7'},
                'ADTN24314222': {       'dev_macs': ['cc:66:18:51:19:50'],
                                        'ip_address': None,
                                        'name': 'Maurice-x29-ONU2',
                                        'serial': 'ADTN24314222'},
                'ADTN2431422E': {       'dev_macs': ['cc:66:18:4c:00:10'],
                                        'ip_address': None,
                                        'name': 'Maurice-x19-ONU1',
                                        'serial': 'ADTN2431422E'},
                'ADTN24314236': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x28-ONU0',
                                        'serial': 'ADTN24314236'},
                'ADTN24314239': {       'dev_macs': ['cc:66:18:dd:67:50'],
                                        'ip_address': None,
                                        'name': 'Maurice-x15-ONU4',
                                        'serial': 'ADTN24314239'},
                'ADTN2431423E': {       'dev_macs': ['cc:66:18:51:11:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x18-ONU2',
                                        'serial': 'ADTN2431423E'},
                'ADTN24314242': {       'dev_macs': ['cc:66:18:51:20:e0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x39-ONU7',
                                        'serial': 'ADTN24314242'},
                'ADTN24314260': {       'dev_macs': ['38:f8:f6:52:ad:80'],
                                        'ip_address': None,
                                        'name': 'Maurice-x42-ONU2',
                                        'serial': 'ADTN24314260'},
                'ADTN24314261': {       'dev_macs': ['cc:66:18:51:21:30'],
                                        'ip_address': None,
                                        'name': 'Maurice-x28-ONU1',
                                        'serial': 'ADTN24314261'},
                'ADTN24314262': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x10-ONU1',
                                        'serial': 'ADTN24314262'},
                'ADTN2431428C': {       'dev_macs': ['cc:66:18:50:27:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x32-ONU0',
                                        'serial': 'ADTN2431428C'},
                'ADTN24314293': {       'dev_macs': ['cc:66:18:dd:69:f0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x38-ONU4',
                                        'serial': 'ADTN24314293'},
                'ADTN24314295': {       'dev_macs': ['cc:66:18:51:51:30'],
                                        'ip_address': None,
                                        'name': 'Maurice-x22-ONU2',
                                        'serial': 'ADTN24314295'},
                'ADTN2431430E': {       'dev_macs': ['cc:66:18:51:1b:f0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x18-ONU3',
                                        'serial': 'ADTN2431430E'},
                'ADTN243144F4': {       'dev_macs': ['cc:66:18:51:2a:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x44-ONU5',
                                        'serial': 'ADTN243144F4'},
                'ADTN24314500': {       'dev_macs': ['cc:66:18:50:64:70'],
                                        'ip_address': None,
                                        'name': 'Maurice-x42-ONU1',
                                        'serial': 'ADTN24314500'},
                'ADTN2431453F': {       'dev_macs': ['38:f8:f6:52:b1:90'],
                                        'ip_address': None,
                                        'name': 'Maurice-x45-ONU1',
                                        'serial': 'ADTN2431453F'},
                'ADTN24314573': {       'dev_macs': ['cc:66:18:50:b9:b0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x31-ONU3',
                                        'serial': 'ADTN24314573'},
                'ADTN2431466E': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x5-ONU8',
                                        'serial': 'ADTN2431466E'},
                'ADTN24314771': {       'dev_macs': ['cc:66:18:51:39:70'],
                                        'ip_address': None,
                                        'name': 'Maurice-x1-ONU2',
                                        'serial': 'ADTN24314771'},
                'ADTN24314779': {       'dev_macs': ['cc:66:18:4c:05:80'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU13',
                                        'serial': 'ADTN24314779'},
                'ADTN24363ABD': {       'dev_macs': ['cc:66:18:51:51:a0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x14-ONU7',
                                        'serial': 'ADTN24363ABD'},
                'ADTN24363AEA': {       'dev_macs': ['38:f8:f6:52:ac:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x12-ONU2',
                                        'serial': 'ADTN24363AEA'},
                'ADTN24363B07': {       'dev_macs': ['cc:66:18:4c:0a:40'],
                                        'ip_address': None,
                                        'name': 'Maurice-x40-ONU2',
                                        'serial': 'ADTN24363B07'},
                'ADTN24363B11': {       'dev_macs': ['cc:66:18:51:2a:f0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU8',
                                        'serial': 'ADTN24363B11'},
                'ADTN24363B5A': {       'dev_macs': ['cc:66:18:4b:ff:c0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU9',
                                        'serial': 'ADTN24363B5A'},
                'ADTN24363B78': {       'dev_macs': ['cc:66:18:50:aa:90'],
                                        'ip_address': None,
                                        'name': 'Maurice-x12-ONU7',
                                        'serial': 'ADTN24363B78'},
                'ADTN24363BBF': {       'dev_macs': ['cc:66:18:50:ab:e0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x44-ONU4',
                                        'serial': 'ADTN24363BBF'},
                'ADTN24363BC6': {       'dev_macs': ['cc:66:18:dd:47:40'],
                                        'ip_address': None,
                                        'name': 'Maurice-x27-ONU1',
                                        'serial': 'ADTN24363BC6'},
                'ADTN24363BC8': {       'dev_macs': ['cc:66:18:50:7c:90'],
                                        'ip_address': None,
                                        'name': 'Maurice-x12-ONU5',
                                        'serial': 'ADTN24363BC8'},
                'ADTN24363BCE': {       'dev_macs': ['38:f8:f6:52:9d:e0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x31-ONU1',
                                        'serial': 'ADTN24363BCE'},
                'ADTN24363BF4': {       'dev_macs': ['cc:66:18:50:b7:10'],
                                        'ip_address': None,
                                        'name': 'Maurice-x12-ONU8',
                                        'serial': 'ADTN24363BF4'},
                'ADTN24363BF9': {       'dev_macs': ['cc:66:18:50:6e:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x6-ONU2',
                                        'serial': 'ADTN24363BF9'},
                'ADTN24363C10': {       'dev_macs': ['cc:66:18:dc:ac:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU9',
                                        'serial': 'ADTN24363C10'},
                'ADTN24363C4F': {       'dev_macs': ['cc:66:18:51:2d:80'],
                                        'ip_address': None,
                                        'name': 'Maurice-x13-ONU1',
                                        'serial': 'ADTN24363C4F'},
                'ADTN24363C5C': {       'dev_macs': ['cc:66:18:50:ae:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x39-ONU5',
                                        'serial': 'ADTN24363C5C'},
                'ADTN24363C5D': {       'dev_macs': ['cc:66:18:51:34:c0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x14-ONU8',
                                        'serial': 'ADTN24363C5D'},
                'ADTN24363C65': {       'dev_macs': ['cc:66:18:4c:01:b0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x44-ONU3',
                                        'serial': 'ADTN24363C65'},
                'ADTN24363C67': {       'dev_macs': ['cc:66:18:50:c8:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU1',
                                        'serial': 'ADTN24363C67'},
                'ADTN24363C6F': {       'dev_macs': ['cc:66:18:51:51:90'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU7',
                                        'serial': 'ADTN24363C6F'},
                'ADTN24363C73': {       'dev_macs': ['38:f8:f6:52:ab:c0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU9',
                                        'serial': 'ADTN24363C73'},
                'ADTN24363C77': {       'dev_macs': ['cc:66:18:4c:08:c0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x37-ONU0',
                                        'serial': 'ADTN24363C77'},
                'ADTN24363C78': {       'dev_macs': ['cc:66:18:51:51:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x12-ONU4',
                                        'serial': 'ADTN24363C78'},
                'ADTN24363C94': {       'dev_macs': ['38:f8:f6:52:ad:90'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU0',
                                        'serial': 'ADTN24363C94'},
                'ADTN24363C9E': {       'dev_macs': ['cc:66:18:4b:fb:40'],
                                        'ip_address': None,
                                        'name': 'Maurice-x11-ONU1',
                                        'serial': 'ADTN24363C9E'},
                'ADTN24363CAE': {       'dev_macs': ['cc:66:18:50:b4:50'],
                                        'ip_address': None,
                                        'name': 'Maurice-x31-ONU2',
                                        'serial': 'ADTN24363CAE'},
                'ADTN24363CDD': {       'dev_macs': ['38:f8:f6:52:ac:50'],
                                        'ip_address': None,
                                        'name': 'Maurice-x38-ONU1',
                                        'serial': 'ADTN24363CDD'},
                'ADTN24363CE0': {       'dev_macs': ['cc:66:18:50:5d:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x39-ONU4',
                                        'serial': 'ADTN24363CE0'},
                'ADTN24363CE2': {       'dev_macs': ['cc:66:18:51:18:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x39-ONU3',
                                        'serial': 'ADTN24363CE2'},
                'ADTN24363CE6': {       'dev_macs': ['cc:66:18:50:b9:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x14-ONU5',
                                        'serial': 'ADTN24363CE6'},
                'ADTN24363CFF': {       'dev_macs': ['cc:66:18:50:5b:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU2',
                                        'serial': 'ADTN24363CFF'},
                'ADTN24363D07': {       'dev_macs': ['38:f8:f6:52:ae:e0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x12-ONU1',
                                        'serial': 'ADTN24363D07'},
                'ADTN24363D3B': {       'dev_macs': ['cc:66:18:51:51:80'],
                                        'ip_address': None,
                                        'name': 'Maurice-x37-ONU2',
                                        'serial': 'ADTN24363D3B'},
                'ADTN24363D6E': {       'dev_macs': ['38:f8:f6:52:ae:10'],
                                        'ip_address': None,
                                        'name': 'Maurice-x32-ONU1',
                                        'serial': 'ADTN24363D6E'},
                'ADTN24363D71': {       'dev_macs': ['38:f8:f6:52:ae:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x44-ONU6',
                                        'serial': 'ADTN24363D71'},
                'ADTN24363D7D': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x12-ONU3',
                                        'serial': 'ADTN24363D7D'},
                'ADTN24363D8F': {       'dev_macs': ['cc:66:18:4c:06:50'],
                                        'ip_address': None,
                                        'name': 'Maurice-x32-ONU3',
                                        'serial': 'ADTN24363D8F'},
                'ADTN24363D9A': {       'dev_macs': ['cc:66:18:50:ae:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU8',
                                        'serial': 'ADTN24363D9A'},
                'ADTN24363DB1': {       'dev_macs': ['cc:66:18:51:3c:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x12-ONU6',
                                        'serial': 'ADTN24363DB1'},
                'ADTN24363DD7': {       'dev_macs': ['cc:66:18:51:04:50'],
                                        'ip_address': None,
                                        'name': 'Maurice-x19-ONU0',
                                        'serial': 'ADTN24363DD7'},
                'ADTN24363DDA': {       'dev_macs': ['cc:66:18:4b:fa:d0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x10-ONU0',
                                        'serial': 'ADTN24363DDA'},
                'ADTN24363E05': {       'dev_macs': ['cc:66:18:4c:01:40'],
                                        'ip_address': None,
                                        'name': 'Maurice-x42-ONU3',
                                        'serial': 'ADTN24363E05'},
                'ADTN24363E0E': {       'dev_macs': ['cc:66:18:50:55:60'],
                                        'ip_address': None,
                                        'name': 'Maurice-x37-ONU3',
                                        'serial': 'ADTN24363E0E'},
                'ADTN24363E22': {       'dev_macs': ['cc:66:18:50:e2:00'],
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU12',
                                        'serial': 'ADTN24363E22'},
                'ADTN24363E39': {       'dev_macs': ['cc:66:18:50:51:d0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x40-ONU1',
                                        'serial': 'ADTN24363E39'},
                'ADTN24363E3D': {       'dev_macs': ['cc:66:18:50:aa:80'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU10',
                                        'serial': 'ADTN24363E3D'},
                'ADTN24363E5C': {       'dev_macs': ['cc:66:18:50:59:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x43-ONU2',
                                        'serial': 'ADTN24363E5C'},
                'ADTN24363E65': {       'dev_macs': ['cc:66:18:50:af:10'],
                                        'ip_address': None,
                                        'name': 'Maurice-x39-ONU6',
                                        'serial': 'ADTN24363E65'},
                'ADTN24389722': {       'dev_macs': ['cc:66:18:51:37:c0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x15-ONU0',
                                        'serial': 'ADTN24389722'},
                'ADTN24389729': {       'dev_macs': ['38:f8:f6:52:b1:b0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x17-ONU10',
                                        'serial': 'ADTN24389729'},
                'ADTN24389754': {       'dev_macs': ['cc:66:18:51:2d:10'],
                                        'ip_address': None,
                                        'name': 'Maurice-x15-ONU3',
                                        'serial': 'ADTN24389754'},
                'ADTN24389760': {       'dev_macs': ['cc:66:18:51:24:20'],
                                        'ip_address': None,
                                        'name': 'Maurice-x29-ONU1',
                                        'serial': 'ADTN24389760'},
                'ADTN24389794': {       'dev_macs': ['cc:66:18:4c:00:e0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x42-ONU0',
                                        'serial': 'ADTN24389794'},
                'ADTN24389796': {       'dev_macs': set(),
                                        'ip_address': None,
                                        'name': 'Maurice-x16-ONU10',
                                        'serial': 'ADTN24389796'},
                'ADTN2438985A': {       'dev_macs': ['cc:66:18:50:8b:e0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x38-ONU2',
                                        'serial': 'ADTN2438985A'},
                'ADTN243899D9': {       'dev_macs': ['cc:66:18:50:6e:80'],
                                        'ip_address': None,
                                        'name': 'Maurice-x24-ONU11',
                                        'serial': 'ADTN243899D9'},
                'ADTN24389AD7': {       'dev_macs': ['cc:66:18:4b:ff:d0'],
                                        'ip_address': None,
                                        'name': 'Maurice-x36-ONU1',
                                        'serial': 'ADTN24389AD7'}
        }
        exp_ne_msg = {
                'data': {       'cpu': 4.2,
                                'interfaces': [       {       'admin_up': True,
                                                              'id': '0/1',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:d6:36:d0'],
                                                                                       'in_nucast_pkts': '451',
                                                                                       'in_octets': '242864315',
                                                                                       'in_ucast_pkts': '448335',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '961946008',
                                                                                       'out_ucast_pkts': '914137',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RDQ3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:39:70'],
                                                                                       'in_nucast_pkts': '477',
                                                                                       'in_octets': '386990823',
                                                                                       'in_ucast_pkts': '759975',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1656358492',
                                                                                       'out_ucast_pkts': '1417813',
                                                                                       'poller_hash': 'QURUTjI0MzE0NzcxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'c4:41:1e:6d:d4:36'],
                                                                                       'in_nucast_pkts': '833',
                                                                                       'in_octets': '90243343',
                                                                                       'in_ucast_pkts': '319978',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '196904150',
                                                                                       'out_ucast_pkts': '369594',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RTk1IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/1',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8x',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/1',
                                                                                      'id': 'xgspon '
                                                                                            '0/1',
                                                                                      'links': [       {       'connected_time': '13217',
                                                                                                               'fiber_length': 11625,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RDQ3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.326},
                                                                                                       {       'connected_time': '14147',
                                                                                                               'fiber_length': 11724,
                                                                                                               'poller_hash': 'QURUTjI0MzE0NzcxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.326},
                                                                                                       {       'connected_time': '13986',
                                                                                                               'fiber_length': 10921,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RTk1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.326}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/1',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8x',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/10',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:4b:fa:d0'],
                                                                                       'in_nucast_pkts': '476',
                                                                                       'in_octets': '50590282',
                                                                                       'in_ucast_pkts': '185582',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '147152550',
                                                                                       'out_ucast_pkts': '202288',
                                                                                       'poller_hash': 'QURUTjI0MzYzRERBIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/10',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/10',
                                                                                      'id': 'xgspon '
                                                                                            '0/10',
                                                                                      'links': [       {       'connected_time': '13827',
                                                                                                               'fiber_length': 17649,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRERBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4215}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/10',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/11',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:50:5b:f0'],
                                                                                       'in_nucast_pkts': '465',
                                                                                       'in_octets': '856146',
                                                                                       'in_ucast_pkts': '3319',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '249954',
                                                                                       'out_ucast_pkts': '2410',
                                                                                       'poller_hash': 'QURUTjIzMTQ0Qjc5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fb:40'],
                                                                                       'in_nucast_pkts': '465',
                                                                                       'in_octets': '718482852',
                                                                                       'in_ucast_pkts': '3061983',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '9215264617',
                                                                                       'out_ucast_pkts': '6426261',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzlFIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/11',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/11',
                                                                                      'id': 'xgspon '
                                                                                            '0/11',
                                                                                      'links': [       {       'connected_time': '13470',
                                                                                                               'fiber_length': 18588,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0Qjc5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2778},
                                                                                                       {       'connected_time': '13221',
                                                                                                               'fiber_length': 18740,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzlFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2778}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/11',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/12',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:70'],
                                                                                       'in_nucast_pkts': '499',
                                                                                       'in_octets': '97705072',
                                                                                       'in_ucast_pkts': '551175',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1170351240',
                                                                                       'out_ucast_pkts': '974025',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTBBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ae:e0'],
                                                                                       'in_nucast_pkts': '470',
                                                                                       'in_octets': '1026782676',
                                                                                       'in_ucast_pkts': '1405990',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1870686990',
                                                                                       'out_ucast_pkts': '1866863',
                                                                                       'poller_hash': 'QURUTjI0MzYzRDA3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:60'],
                                                                                       'in_nucast_pkts': '475',
                                                                                       'in_octets': '143127976',
                                                                                       'in_ucast_pkts': '223240',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '452710226',
                                                                                       'out_ucast_pkts': '375162',
                                                                                       'poller_hash': 'QURUTjI0MzYzQUVBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:51:00'],
                                                                                       'in_nucast_pkts': '463',
                                                                                       'in_octets': '32646246',
                                                                                       'in_ucast_pkts': '164103',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '442333405',
                                                                                       'out_ucast_pkts': '363467',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzc4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:7c:90'],
                                                                                       'in_nucast_pkts': '453',
                                                                                       'in_octets': '33980506',
                                                                                       'in_ucast_pkts': '378028',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '2024115893',
                                                                                       'out_ucast_pkts': '1360137',
                                                                                       'poller_hash': 'QURUTjI0MzYzQkM4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:3c:60'],
                                                                                       'in_nucast_pkts': '477',
                                                                                       'in_octets': '74726912',
                                                                                       'in_ucast_pkts': '246695',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '628628742',
                                                                                       'out_ucast_pkts': '546779',
                                                                                       'poller_hash': 'QURUTjI0MzYzREIxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:aa:90'],
                                                                                       'in_nucast_pkts': '482',
                                                                                       'in_octets': '147895232',
                                                                                       'in_ucast_pkts': '1084132',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '11084945193',
                                                                                       'out_ucast_pkts': '8436789',
                                                                                       'poller_hash': 'QURUTjI0MzYzQjc4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b7:10'],
                                                                                       'in_nucast_pkts': '475',
                                                                                       'in_octets': '180046930',
                                                                                       'in_ucast_pkts': '395488',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1547014878',
                                                                                       'out_ucast_pkts': '1222738',
                                                                                       'poller_hash': 'QURUTjI0MzYzQkY0IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/12',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMg==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/12',
                                                                                      'id': 'xgspon '
                                                                                            '0/12',
                                                                                      'links': [       {       'connected_time': '14137',
                                                                                                               'fiber_length': 11276,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTBBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4927},
                                                                                                       {       'connected_time': '13996',
                                                                                                               'fiber_length': 11960,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRDA3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4927},
                                                                                                       {       'connected_time': '13827',
                                                                                                               'fiber_length': 12184,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQUVBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4927},
                                                                                                       {       'connected_time': '13476',
                                                                                                               'fiber_length': 11077,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzc4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4927},
                                                                                                       {       'connected_time': '13217',
                                                                                                               'fiber_length': 11983,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQkM4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4927},
                                                                                                       {       'connected_time': '14148',
                                                                                                               'fiber_length': 11015,
                                                                                                               'poller_hash': 'QURUTjI0MzYzREIxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4927},
                                                                                                       {       'connected_time': '13997',
                                                                                                               'fiber_length': 10680,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQjc4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4927},
                                                                                                       {       'connected_time': '13826',
                                                                                                               'fiber_length': 10619,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQkY0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -12.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4927}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/12',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/13',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:50:4d:a0'],
                                                                                       'in_nucast_pkts': '481',
                                                                                       'in_octets': '410242455',
                                                                                       'in_ucast_pkts': '551868',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1042134894',
                                                                                       'out_ucast_pkts': '924503',
                                                                                       'poller_hash': 'QURUTjIzMTI0QUNBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:2d:80'],
                                                                                       'in_nucast_pkts': '464',
                                                                                       'in_octets': '13566675',
                                                                                       'in_ucast_pkts': '89916',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '97973196',
                                                                                       'out_ucast_pkts': '101853',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzRGIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/13',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMw==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/13',
                                                                                      'id': 'xgspon '
                                                                                            '0/13',
                                                                                      'links': [       {       'connected_time': '13674',
                                                                                                               'fiber_length': 11798,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QUNBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5186},
                                                                                                       {       'connected_time': '13474',
                                                                                                               'fiber_length': 11903,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzRGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5186}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/13',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/14',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:06:60'],
                                                                                       'in_nucast_pkts': '446',
                                                                                       'in_octets': '11778979',
                                                                                       'in_ucast_pkts': '63827',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '238381558',
                                                                                       'out_ucast_pkts': '195348',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTY2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:05:60'],
                                                                                       'in_nucast_pkts': '486',
                                                                                       'in_octets': '448932237',
                                                                                       'in_ucast_pkts': '454935',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1090082492',
                                                                                       'out_ucast_pkts': '961874',
                                                                                       'poller_hash': 'QURUTjIzMTI0ODZEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:39:d0'],
                                                                                       'in_nucast_pkts': '481',
                                                                                       'in_octets': '965316008',
                                                                                       'in_ucast_pkts': '1586410',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '6978071567',
                                                                                       'out_ucast_pkts': '5410288',
                                                                                       'poller_hash': 'QURUTjIzMTI0OThBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:22:50'],
                                                                                       'in_nucast_pkts': '473',
                                                                                       'in_octets': '160826096',
                                                                                       'in_ucast_pkts': '1877214',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '6590958232',
                                                                                       'out_ucast_pkts': '4558090',
                                                                                       'poller_hash': 'QURUTjIzMTI0OEQ3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b9:00'],
                                                                                       'in_nucast_pkts': '471',
                                                                                       'in_octets': '1774270060',
                                                                                       'in_ucast_pkts': '2666821',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '7485507904',
                                                                                       'out_ucast_pkts': '5484464',
                                                                                       'poller_hash': 'QURUTjI0MzYzQ0U2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:51:a0'],
                                                                                       'in_nucast_pkts': '455',
                                                                                       'in_octets': '10875783',
                                                                                       'in_ucast_pkts': '43047',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '24822738',
                                                                                       'out_ucast_pkts': '42142',
                                                                                       'poller_hash': 'QURUTjI0MzYzQUJEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:34:c0'],
                                                                                       'in_nucast_pkts': '485',
                                                                                       'in_octets': '190135208',
                                                                                       'in_ucast_pkts': '1406309',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '3792845478',
                                                                                       'out_ucast_pkts': '2666815',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzVEIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/14',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/14',
                                                                                      'id': 'xgspon '
                                                                                            '0/14',
                                                                                      'links': [       {       'connected_time': '13214',
                                                                                                               'fiber_length': 13407,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTY2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3281},
                                                                                                       {       'connected_time': '14143',
                                                                                                               'fiber_length': 12827,
                                                                                                               'poller_hash': 'QURUTjIzMTI0ODZEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3281},
                                                                                                       {       'connected_time': '13985',
                                                                                                               'fiber_length': 13210,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OThBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3281},
                                                                                                       {       'connected_time': '13824',
                                                                                                               'fiber_length': 13796,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OEQ3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3281},
                                                                                                       {       'connected_time': '13684',
                                                                                                               'fiber_length': 13958,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQ0U2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3281},
                                                                                                       {       'connected_time': '13223',
                                                                                                               'fiber_length': 12895,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQUJEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3281},
                                                                                                       {       'connected_time': '14143',
                                                                                                               'fiber_length': 13024,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzVEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3281}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/14',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/15',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:32:00'],
                                                                                       'in_nucast_pkts': '481',
                                                                                       'in_octets': '331645176',
                                                                                       'in_ucast_pkts': '717159',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1016931757',
                                                                                       'out_ucast_pkts': '1123194',
                                                                                       'poller_hash': 'QURUTjIzMTI0QkZEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b6:80'],
                                                                                       'in_nucast_pkts': '466',
                                                                                       'in_octets': '168223928',
                                                                                       'in_ucast_pkts': '324871',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '896806012',
                                                                                       'out_ucast_pkts': '718117',
                                                                                       'poller_hash': 'QURUTjIzMTI0ODM4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:37:c0'],
                                                                                       'in_nucast_pkts': '472',
                                                                                       'in_octets': '1020052778',
                                                                                       'in_ucast_pkts': '1498303',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '3821608507',
                                                                                       'out_ucast_pkts': '3181992',
                                                                                       'poller_hash': 'QURUTjI0Mzg5NzIyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:2d:10'],
                                                                                       'in_nucast_pkts': '454',
                                                                                       'in_octets': '44051786',
                                                                                       'in_ucast_pkts': '202316',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '488754397',
                                                                                       'out_ucast_pkts': '389744',
                                                                                       'poller_hash': 'QURUTjI0Mzg5NzU0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:67:50'],
                                                                                       'in_nucast_pkts': '449',
                                                                                       'in_octets': '20061858',
                                                                                       'in_ucast_pkts': '92108',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '56360098',
                                                                                       'out_ucast_pkts': '91493',
                                                                                       'poller_hash': 'QURUTjI0MzE0MjM5IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/15',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/15',
                                                                                      'id': 'xgspon '
                                                                                            '0/15',
                                                                                      'links': [       {       'connected_time': '13674',
                                                                                                               'fiber_length': 14227,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QkZEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5167},
                                                                                                       {       'connected_time': '13824',
                                                                                                               'fiber_length': 14418,
                                                                                                               'poller_hash': 'QURUTjIzMTI0ODM4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5167},
                                                                                                       {       'connected_time': '13994',
                                                                                                               'fiber_length': 14621,
                                                                                                               'poller_hash': 'QURUTjI0Mzg5NzIyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5167},
                                                                                                       {       'connected_time': '13474',
                                                                                                               'fiber_length': 14296,
                                                                                                               'poller_hash': 'QURUTjI0Mzg5NzU0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5167},
                                                                                                       {       'connected_time': '13213',
                                                                                                               'fiber_length': 13780,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MjM5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5167}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/15',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/16',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:50:ac:30'],
                                                                                       'in_nucast_pkts': '351',
                                                                                       'in_octets': '135656681',
                                                                                       'in_ucast_pkts': '145060',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '142610193',
                                                                                       'out_ucast_pkts': '140424',
                                                                                       'poller_hash': 'QURUTjIzMTI0N0Q2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:50:b0'],
                                                                                       'in_nucast_pkts': '342',
                                                                                       'in_octets': '129347451',
                                                                                       'in_ucast_pkts': '125597',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '138429097',
                                                                                       'out_ucast_pkts': '127664',
                                                                                       'poller_hash': 'QURUTjIzMTI0QjZEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3b:f0'],
                                                                                       'in_nucast_pkts': '391',
                                                                                       'in_octets': '16872461',
                                                                                       'in_ucast_pkts': '62795',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '208640346',
                                                                                       'out_ucast_pkts': '186120',
                                                                                       'poller_hash': 'QURUTjIzMTI0RENDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:03:90'],
                                                                                       'in_nucast_pkts': '372',
                                                                                       'in_octets': '96698803',
                                                                                       'in_ucast_pkts': '1106465',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '6589630494',
                                                                                       'out_ucast_pkts': '4465065',
                                                                                       'poller_hash': 'QURUTjIzMTI0Nzg1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:64:00'],
                                                                                       'in_nucast_pkts': '361',
                                                                                       'in_octets': '166981861',
                                                                                       'in_ucast_pkts': '256927',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '618995496',
                                                                                       'out_ucast_pkts': '498480',
                                                                                       'poller_hash': 'QURUTjIzMTI0MDQ4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:9c:2f:90'],
                                                                                       'in_nucast_pkts': '341',
                                                                                       'in_octets': '88559950',
                                                                                       'in_ucast_pkts': '533666',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '3581600402',
                                                                                       'out_ucast_pkts': '2622981',
                                                                                       'poller_hash': 'QURUTjIzMTI1MDMwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:0e:c0'],
                                                                                       'in_nucast_pkts': '334',
                                                                                       'in_octets': '58899795',
                                                                                       'in_ucast_pkts': '204814',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1128714277',
                                                                                       'out_ucast_pkts': '889018',
                                                                                       'poller_hash': 'QURUTjIzMTI0QTFEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:29:d0'],
                                                                                       'in_nucast_pkts': '319',
                                                                                       'in_octets': '78198887',
                                                                                       'in_ucast_pkts': '303965',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1119873683',
                                                                                       'out_ucast_pkts': '976914',
                                                                                       'poller_hash': 'QURUTjIzMTIzRThFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:7a:f0'],
                                                                                       'in_nucast_pkts': '413',
                                                                                       'in_octets': '93745932',
                                                                                       'in_ucast_pkts': '705884',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1',
                                                                                       'out_octets': '2192693191',
                                                                                       'out_ucast_pkts': '1588392',
                                                                                       'poller_hash': 'QURUTjIzMTI0NzQ2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ab:c0'],
                                                                                       'in_nucast_pkts': '402',
                                                                                       'in_octets': '195815673',
                                                                                       'in_ucast_pkts': '1253370',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1',
                                                                                       'out_octets': '2858079183',
                                                                                       'out_ucast_pkts': '2091469',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzczIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:42:30'],
                                                                                       'in_nucast_pkts': '416',
                                                                                       'in_octets': '370352738',
                                                                                       'in_ucast_pkts': '2517837',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '10119353774',
                                                                                       'out_ucast_pkts': '7166713',
                                                                                       'poller_hash': 'QURUTjI0MzE0MTZFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:e2:00'],
                                                                                       'in_nucast_pkts': '399',
                                                                                       'in_octets': '340370588',
                                                                                       'in_ucast_pkts': '2029064',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '6082153947',
                                                                                       'out_ucast_pkts': '4547465',
                                                                                       'poller_hash': 'QURUTjI0MzYzRTIyIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/16',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/16',
                                                                                      'id': 'xgspon '
                                                                                            '0/16',
                                                                                      'links': [       {       'connected_time': '10224',
                                                                                                               'fiber_length': 10077,
                                                                                                               'poller_hash': 'QURUTjIzMTI0N0Q2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461},
                                                                                                       {       'connected_time': '9593',
                                                                                                               'fiber_length': 10108,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QjZEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461},
                                                                                                       {       'connected_time': '11334',
                                                                                                               'fiber_length': 10168,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RENDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461},
                                                                                                       {       'connected_time': '10734',
                                                                                                               'fiber_length': 10205,
                                                                                                               'poller_hash': 'QURUTjIzMTI0Nzg1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461},
                                                                                                       {       'connected_time': '10394',
                                                                                                               'fiber_length': 10416,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MDQ4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461},
                                                                                                       {       'connected_time': '10234',
                                                                                                               'fiber_length': 10562,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MDMwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461},
                                                                                                       {       'connected_time': '9593',
                                                                                                               'fiber_length': 10214,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QTFEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461},
                                                                                                       {       'connected_time': '9144',
                                                                                                               'fiber_length': 10440,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRThFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461},
                                                                                                       {       'connected_time': '12084',
                                                                                                               'fiber_length': 10759,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NzQ2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461},
                                                                                                       {       'connected_time': '11754',
                                                                                                               'fiber_length': 10470,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzczIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461},
                                                                                                       {       'connected_time': '12083',
                                                                                                               'fiber_length': 10093,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MTZFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461},
                                                                                                       {       'connected_time': '11754',
                                                                                                               'fiber_length': 10578,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRTIyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4461}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/16',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/17',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:50:a7:20'],
                                                                                       'in_nucast_pkts': '384',
                                                                                       'in_octets': '130081128',
                                                                                       'in_ucast_pkts': '127560',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '137853785',
                                                                                       'out_ucast_pkts': '131574',
                                                                                       'poller_hash': 'QURUTjIzMTI0Q0U1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:c8:20'],
                                                                                       'in_nucast_pkts': '372',
                                                                                       'in_octets': '509766003',
                                                                                       'in_ucast_pkts': '691910',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1712492583',
                                                                                       'out_ucast_pkts': '1528806',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzY3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:5b:00'],
                                                                                       'in_nucast_pkts': '334',
                                                                                       'in_octets': '24651498',
                                                                                       'in_ucast_pkts': '146394',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1252514528',
                                                                                       'out_ucast_pkts': '993141',
                                                                                       'poller_hash': 'QURUTjI0MzYzQ0ZGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ae:80'],
                                                                                       'in_nucast_pkts': '327',
                                                                                       'in_octets': '543857883',
                                                                                       'in_ucast_pkts': '1468691',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '6250712841',
                                                                                       'out_ucast_pkts': '4357999',
                                                                                       'poller_hash': 'QURUTjIzMTI0N0IwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:af:60'],
                                                                                       'in_nucast_pkts': '409',
                                                                                       'in_octets': '1074153',
                                                                                       'in_ucast_pkts': '4234',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1032682',
                                                                                       'out_ucast_pkts': '3572',
                                                                                       'poller_hash': 'QURUTjIzMTI0REFDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a1:40'],
                                                                                       'in_nucast_pkts': '470',
                                                                                       'in_octets': '226059110',
                                                                                       'in_ucast_pkts': '401961',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1335899184',
                                                                                       'out_ucast_pkts': '1128684',
                                                                                       'poller_hash': 'QURUTjIzMTI0Qjc1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:58:00'],
                                                                                       'in_nucast_pkts': '452',
                                                                                       'in_octets': '541486255',
                                                                                       'in_ucast_pkts': '1784498',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '3202701440',
                                                                                       'out_ucast_pkts': '3004730',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTQ0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:2a:f0'],
                                                                                       'in_nucast_pkts': '456',
                                                                                       'in_octets': '45022957',
                                                                                       'in_ucast_pkts': '274008',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '122106102',
                                                                                       'out_ucast_pkts': '316102',
                                                                                       'poller_hash': 'QURUTjI0MzYzQjExIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:ff:c0'],
                                                                                       'in_nucast_pkts': '498',
                                                                                       'in_octets': '235958907',
                                                                                       'in_ucast_pkts': '2297650',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '12723574439',
                                                                                       'out_ucast_pkts': '8782231',
                                                                                       'poller_hash': 'QURUTjI0MzYzQjVBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:b1:b0'],
                                                                                       'in_nucast_pkts': '363',
                                                                                       'in_octets': '144139860',
                                                                                       'in_ucast_pkts': '1057494',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '4670288854',
                                                                                       'out_ucast_pkts': '3250941',
                                                                                       'poller_hash': 'QURUTjI0Mzg5NzI5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:02:e0'],
                                                                                       'in_nucast_pkts': '355',
                                                                                       'in_octets': '614206',
                                                                                       'in_ucast_pkts': '2446',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '180776',
                                                                                       'out_ucast_pkts': '1782',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RTdCIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/17',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/17',
                                                                                      'id': 'xgspon '
                                                                                            '0/17',
                                                                                      'links': [       {       'connected_time': '11338',
                                                                                                               'fiber_length': 11030,
                                                                                                               'poller_hash': 'QURUTjIzMTI0Q0U1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.418},
                                                                                                       {       'connected_time': '10739',
                                                                                                               'fiber_length': 10813,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzY3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.418},
                                                                                                       {       'connected_time': '9609',
                                                                                                               'fiber_length': 11213,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQ0ZGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.418},
                                                                                                       {       'connected_time': '9149',
                                                                                                               'fiber_length': 11290,
                                                                                                               'poller_hash': 'QURUTjIzMTI0N0IwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.418},
                                                                                                       {       'connected_time': '12079',
                                                                                                               'fiber_length': 11290,
                                                                                                               'poller_hash': 'QURUTjIzMTI0REFDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.418},
                                                                                                       {       'connected_time': '13669',
                                                                                                               'fiber_length': 10837,
                                                                                                               'poller_hash': 'QURUTjIzMTI0Qjc1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.418},
                                                                                                       {       'connected_time': '13468',
                                                                                                               'fiber_length': 10680,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.418},
                                                                                                       {       'connected_time': '13218',
                                                                                                               'fiber_length': 10874,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQjExIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.418},
                                                                                                       {       'connected_time': '14148',
                                                                                                               'fiber_length': 11228,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQjVBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.418},
                                                                                                       {       'connected_time': '10408',
                                                                                                               'fiber_length': 11321,
                                                                                                               'poller_hash': 'QURUTjI0Mzg5NzI5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.418},
                                                                                                       {       'connected_time': '10228',
                                                                                                               'fiber_length': 10732,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RTdCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.418}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/17',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/18',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:13:c0'],
                                                                                       'in_nucast_pkts': '483',
                                                                                       'in_octets': '1026813',
                                                                                       'in_ucast_pkts': '4111',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '376306',
                                                                                       'out_ucast_pkts': '3108',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjExIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:34:10'],
                                                                                       'in_nucast_pkts': '486',
                                                                                       'in_octets': '891543894',
                                                                                       'in_ucast_pkts': '778638',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '417433523',
                                                                                       'out_ucast_pkts': '452066',
                                                                                       'poller_hash': 'QURUTjIzMTI0MUE2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:11:00'],
                                                                                       'in_nucast_pkts': '459',
                                                                                       'in_octets': '7256001',
                                                                                       'in_ucast_pkts': '52116',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '107197188',
                                                                                       'out_ucast_pkts': '84441',
                                                                                       'poller_hash': 'QURUTjI0MzE0MjNFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:1b:f0'],
                                                                                       'in_nucast_pkts': '454',
                                                                                       'in_octets': '16054503',
                                                                                       'in_ucast_pkts': '76659',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '114169609',
                                                                                       'out_ucast_pkts': '98571',
                                                                                       'poller_hash': 'QURUTjI0MzE0MzBFIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/18',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xOA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/18',
                                                                                      'id': 'xgspon '
                                                                                            '0/18',
                                                                                      'links': [       {       'connected_time': '13991',
                                                                                                               'fiber_length': 13282,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjExIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3163},
                                                                                                       {       'connected_time': '13821',
                                                                                                               'fiber_length': 13833,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MUE2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3163},
                                                                                                       {       'connected_time': '13671',
                                                                                                               'fiber_length': 13806,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MjNFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3163},
                                                                                                       {       'connected_time': '13471',
                                                                                                               'fiber_length': 12282,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MzBFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3163}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/18',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xOA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/19',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:04:50'],
                                                                                       'in_nucast_pkts': '446',
                                                                                       'in_octets': '121809105',
                                                                                       'in_ucast_pkts': '351198',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1022799880',
                                                                                       'out_ucast_pkts': '852967',
                                                                                       'poller_hash': 'QURUTjI0MzYzREQ3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:00:10'],
                                                                                       'in_nucast_pkts': '481',
                                                                                       'in_octets': '121064401',
                                                                                       'in_ucast_pkts': '1607299',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '4878114535',
                                                                                       'out_ucast_pkts': '3323816',
                                                                                       'poller_hash': 'QURUTjI0MzE0MjJFIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/19',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xOQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/19',
                                                                                      'id': 'xgspon '
                                                                                            '0/19',
                                                                                      'links': [       {       'connected_time': '13219',
                                                                                                               'fiber_length': 13158,
                                                                                                               'poller_hash': 'QURUTjI0MzYzREQ3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -12.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '14138',
                                                                                                               'fiber_length': 13053,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MjJFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -11.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/19',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xOQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/2',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:b1:10'],
                                                                                       'in_nucast_pkts': '491',
                                                                                       'in_octets': '33125671',
                                                                                       'in_ucast_pkts': '99597',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '326285482',
                                                                                       'out_ucast_pkts': '264921',
                                                                                       'poller_hash': 'QURUTjIzMTQ1MjMyIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/2',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8y',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/2',
                                                                                      'id': 'xgspon '
                                                                                            '0/2',
                                                                                      'links': [       {       'connected_time': '13992',
                                                                                                               'fiber_length': 11584,
                                                                                                               'poller_hash': 'QURUTjIzMTQ1MjMyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3286}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/2',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/20',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:27:c0'],
                                                                                       'in_nucast_pkts': '464',
                                                                                       'in_octets': '170233309',
                                                                                       'in_ucast_pkts': '209683',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '103661230',
                                                                                       'out_ucast_pkts': '208638',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTYzIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b7:b0'],
                                                                                       'in_nucast_pkts': '461',
                                                                                       'in_octets': '4873684',
                                                                                       'in_ucast_pkts': '39084',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '73121332',
                                                                                       'out_ucast_pkts': '62546',
                                                                                       'poller_hash': 'QURUTjI0MzE0MTlCIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/20',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yMA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/20',
                                                                                      'id': 'xgspon '
                                                                                            '0/20',
                                                                                      'links': [       {       'connected_time': '13827',
                                                                                                               'fiber_length': 14841,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTYzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.427},
                                                                                                       {       'connected_time': '13677',
                                                                                                               'fiber_length': 14803,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MTlCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.427}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/20',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yMA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/21',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/21',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yMQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/21',
                                                                                      'id': 'xgspon '
                                                                                            '0/21',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/21',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yMQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/22',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:35:40'],
                                                                                       'in_nucast_pkts': '414',
                                                                                       'in_octets': '672959934',
                                                                                       'in_ucast_pkts': '1315820',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '4342032313',
                                                                                       'out_ucast_pkts': '3372938',
                                                                                       'poller_hash': 'QURUTjI0MzE0MUVGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:04:20'],
                                                                                       'in_nucast_pkts': '444',
                                                                                       'in_octets': '1254326334',
                                                                                       'in_ucast_pkts': '995741',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1131627148',
                                                                                       'out_ucast_pkts': '1072018',
                                                                                       'poller_hash': 'QURUTjI0MzE0MUFBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:51:30'],
                                                                                       'in_nucast_pkts': '86',
                                                                                       'in_octets': '23044891',
                                                                                       'in_ucast_pkts': '133642',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '887837606',
                                                                                       'out_ucast_pkts': '609426',
                                                                                       'poller_hash': 'QURUTjI0MzE0Mjk1IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/22',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yMg==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/22',
                                                                                      'id': 'xgspon '
                                                                                            '0/22',
                                                                                      'links': [       {       'connected_time': '11897',
                                                                                                               'fiber_length': 14374,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MUVGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.423},
                                                                                                       {       'connected_time': '13216',
                                                                                                               'fiber_length': 14505,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MUFBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.423},
                                                                                                       {       'connected_time': '2167',
                                                                                                               'fiber_length': 14572,
                                                                                                               'poller_hash': 'QURUTjI0MzE0Mjk1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.423}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/22',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yMg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/23',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/23',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yMw==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/23',
                                                                                      'id': 'xgspon '
                                                                                            '0/23',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/23',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yMw==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/24',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:ad:90'],
                                                                                       'in_nucast_pkts': '472',
                                                                                       'in_octets': '326530641',
                                                                                       'in_ucast_pkts': '310190',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '388114042',
                                                                                       'out_ucast_pkts': '363110',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzk0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:f0'],
                                                                                       'in_nucast_pkts': '476',
                                                                                       'in_octets': '142525263',
                                                                                       'in_ucast_pkts': '536733',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '3748978229',
                                                                                       'out_ucast_pkts': '2730483',
                                                                                       'poller_hash': 'QURUTjIzMTI0QjI1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ad:70'],
                                                                                       'in_nucast_pkts': '466',
                                                                                       'in_octets': '9302477',
                                                                                       'in_ucast_pkts': '18880',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '12413789',
                                                                                       'out_ucast_pkts': '22434',
                                                                                       'poller_hash': 'QURUTjIzMTI1MTM5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a2:20'],
                                                                                       'in_nucast_pkts': '461',
                                                                                       'in_octets': '5741096',
                                                                                       'in_ucast_pkts': '21426',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '38726554',
                                                                                       'out_ucast_pkts': '40615',
                                                                                       'poller_hash': 'QURUTjIzMTI0MTE1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:0a:60'],
                                                                                       'in_nucast_pkts': '460',
                                                                                       'in_octets': '39321449',
                                                                                       'in_ucast_pkts': '53652',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1',
                                                                                       'out_octets': '42406679',
                                                                                       'out_ucast_pkts': '51054',
                                                                                       'poller_hash': 'QURUTjIzMTI0NjZCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:5c:b0'],
                                                                                       'in_nucast_pkts': '466',
                                                                                       'in_octets': '17555200',
                                                                                       'in_ucast_pkts': '121207',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '276319426',
                                                                                       'out_ucast_pkts': '215064',
                                                                                       'poller_hash': 'QURUTjIzMTI0ODkxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:51:90'],
                                                                                       'in_nucast_pkts': '477',
                                                                                       'in_octets': '74649393',
                                                                                       'in_ucast_pkts': '124008',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '52665276',
                                                                                       'out_ucast_pkts': '112193',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzZGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ae:20'],
                                                                                       'in_nucast_pkts': '472',
                                                                                       'in_octets': '34363364',
                                                                                       'in_ucast_pkts': '197604',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '253902422',
                                                                                       'out_ucast_pkts': '302249',
                                                                                       'poller_hash': 'QURUTjI0MzYzRDlBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:ac:00'],
                                                                                       'in_nucast_pkts': '482',
                                                                                       'in_octets': '1698528',
                                                                                       'in_ucast_pkts': '5694',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '2263753',
                                                                                       'out_ucast_pkts': '5032',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzEwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:aa:80'],
                                                                                       'in_nucast_pkts': '460',
                                                                                       'in_octets': '55807624',
                                                                                       'in_ucast_pkts': '182498',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '2662806574',
                                                                                       'out_ucast_pkts': '1801793',
                                                                                       'poller_hash': 'QURUTjI0MzYzRTNEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6e:80'],
                                                                                       'in_nucast_pkts': '452',
                                                                                       'in_octets': '6100443',
                                                                                       'in_ucast_pkts': '18941',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '12040094',
                                                                                       'out_ucast_pkts': '18524',
                                                                                       'poller_hash': 'QURUTjI0Mzg5OUQ5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:29:b0'],
                                                                                       'in_nucast_pkts': '455',
                                                                                       'in_octets': '76744010',
                                                                                       'in_ucast_pkts': '414343',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '472202097',
                                                                                       'out_ucast_pkts': '575679',
                                                                                       'poller_hash': 'QURUTjI0MzE0MUI5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:05:80'],
                                                                                       'in_nucast_pkts': '486',
                                                                                       'in_octets': '90624732',
                                                                                       'in_ucast_pkts': '959739',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '5417896669',
                                                                                       'out_ucast_pkts': '3938892',
                                                                                       'poller_hash': 'QURUTjI0MzE0Nzc5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:83:70'],
                                                                                       'in_nucast_pkts': '487',
                                                                                       'in_octets': '854654976',
                                                                                       'in_ucast_pkts': '3209662',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '5691213118',
                                                                                       'out_ucast_pkts': '5581369',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RTdBIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/24',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/24',
                                                                                      'id': 'xgspon '
                                                                                            '0/24',
                                                                                      'links': [       {       'connected_time': '13997',
                                                                                                               'fiber_length': 15866,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzk0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '13827',
                                                                                                               'fiber_length': 16307,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QjI1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '13826',
                                                                                                               'fiber_length': 15592,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MTM5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '13677',
                                                                                                               'fiber_length': 16513,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MTE1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '13467',
                                                                                                               'fiber_length': 16138,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NjZCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '13206',
                                                                                                               'fiber_length': 16559,
                                                                                                               'poller_hash': 'QURUTjIzMTI0ODkxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '14146',
                                                                                                               'fiber_length': 15994,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzZGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '13996',
                                                                                                               'fiber_length': 16064,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRDlBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '13826',
                                                                                                               'fiber_length': 16032,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzEwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '13687',
                                                                                                               'fiber_length': 15731,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRTNEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '13476',
                                                                                                               'fiber_length': 16307,
                                                                                                               'poller_hash': 'QURUTjI0Mzg5OUQ5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '13217',
                                                                                                               'fiber_length': 15966,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MUI5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '14147',
                                                                                                               'fiber_length': 15554,
                                                                                                               'poller_hash': 'QURUTjI0MzE0Nzc5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679},
                                                                                                       {       'connected_time': '13987',
                                                                                                               'fiber_length': 16045,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RTdBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3679}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/24',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/25',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/25',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/25',
                                                                                      'id': 'xgspon '
                                                                                            '0/25',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/25',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/26',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:30:90'],
                                                                                       'in_nucast_pkts': '471',
                                                                                       'in_octets': '19007153',
                                                                                       'in_ucast_pkts': '108015',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '186652850',
                                                                                       'out_ucast_pkts': '147346',
                                                                                       'poller_hash': 'QURUTjI0MzE0MUI4IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/26',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNg==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/26',
                                                                                      'id': 'xgspon '
                                                                                            '0/26',
                                                                                      'links': [       {       'connected_time': '13673',
                                                                                                               'fiber_length': 16257,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MUI4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2221}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/26',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/27',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:a2:40'],
                                                                                       'in_nucast_pkts': '460',
                                                                                       'in_octets': '352430119',
                                                                                       'in_ucast_pkts': '444982',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '711308221',
                                                                                       'out_ucast_pkts': '635235',
                                                                                       'poller_hash': 'QURUTjIzMTI0MDg1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:47:40'],
                                                                                       'in_nucast_pkts': '382',
                                                                                       'in_octets': '30309881',
                                                                                       'in_ucast_pkts': '273373',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '598818952',
                                                                                       'out_ucast_pkts': '413672',
                                                                                       'poller_hash': 'QURUTjI0MzYzQkM2IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/27',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNw==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/27',
                                                                                      'id': 'xgspon '
                                                                                            '0/27',
                                                                                      'links': [       {       'connected_time': '13465',
                                                                                                               'fiber_length': 14837,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MDg1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3789},
                                                                                                       {       'connected_time': '10735',
                                                                                                               'fiber_length': 15222,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQkM2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3789}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/27',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNw==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/28',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:21:30'],
                                                                                       'in_nucast_pkts': '356',
                                                                                       'in_octets': '2465302042',
                                                                                       'in_ucast_pkts': '2433687',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '2049310938',
                                                                                       'out_ucast_pkts': '2813611',
                                                                                       'poller_hash': 'QURUTjI0MzE0MjYxIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/28',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yOA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/28',
                                                                                      'id': 'xgspon '
                                                                                            '0/28',
                                                                                      'links': [       {       'connected_time': '10242',
                                                                                                               'fiber_length': 12952,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MjYxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5813}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/28',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yOA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/29',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:51:60'],
                                                                                       'in_nucast_pkts': '336',
                                                                                       'in_octets': '4003849175',
                                                                                       'in_ucast_pkts': '3155807',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '385831299',
                                                                                       'out_ucast_pkts': '1781974',
                                                                                       'poller_hash': 'QURUTjIzMTI0OEFEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:24:20'],
                                                                                       'in_nucast_pkts': '319',
                                                                                       'in_octets': '39294729',
                                                                                       'in_ucast_pkts': '75878',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '53313507',
                                                                                       'out_ucast_pkts': '85994',
                                                                                       'poller_hash': 'QURUTjI0Mzg5NzYwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:19:50'],
                                                                                       'in_nucast_pkts': '2640',
                                                                                       'in_octets': '12409528845',
                                                                                       'in_ucast_pkts': '16209157',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1',
                                                                                       'out_octets': '66591314532',
                                                                                       'out_ucast_pkts': '46703384',
                                                                                       'poller_hash': 'QURUTjI0MzE0MjIyIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/29',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yOQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/29',
                                                                                      'id': 'xgspon '
                                                                                            '0/29',
                                                                                      'links': [       {       'connected_time': '9595',
                                                                                                               'fiber_length': 13766,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OEFEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2925},
                                                                                                       {       'connected_time': '9145',
                                                                                                               'fiber_length': 13569,
                                                                                                               'poller_hash': 'QURUTjI0Mzg5NzYwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2925},
                                                                                                       {       'connected_time': '78917',
                                                                                                               'fiber_length': 13857,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MjIyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2925}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/29',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yOQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/3',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:4f:e0'],
                                                                                       'in_nucast_pkts': '417',
                                                                                       'in_octets': '189644299',
                                                                                       'in_ucast_pkts': '2141521',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '5117268931',
                                                                                       'out_ucast_pkts': '3469122',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUU3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:18:60'],
                                                                                       'in_nucast_pkts': '399',
                                                                                       'in_octets': '316592568',
                                                                                       'in_ucast_pkts': '451821',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '47409844',
                                                                                       'out_ucast_pkts': '363406',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjJGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:13:00'],
                                                                                       'in_nucast_pkts': '214',
                                                                                       'in_octets': '328858804',
                                                                                       'in_ucast_pkts': '314716',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '430002187',
                                                                                       'out_ucast_pkts': '396950',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTQ4IHBlZXJpbnRm'},
                                                                               {       'in_nucast_pkts': '0',
                                                                                       'in_octets': '0',
                                                                                       'in_ucast_pkts': '0',
                                                                                       'oper_up': False,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '0',
                                                                                       'out_ucast_pkts': '0',
                                                                                       'poller_hash': 'QURUTjIzMTI0QTJGIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/3',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8z',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/3',
                                                                                      'id': 'xgspon '
                                                                                            '0/3',
                                                                                      'links': [       {       'connected_time': '12084',
                                                                                                               'fiber_length': 12881,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUU3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.459},
                                                                                                       {       'connected_time': '11745',
                                                                                                               'fiber_length': 13443,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjJGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.459},
                                                                                                       {       'connected_time': '6284',
                                                                                                               'fiber_length': 12846,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTQ4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.459},
                                                                                                       {       'connected_time': '1534',
                                                                                                               'fiber_length': 12844,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QTJGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.459}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/3',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/30',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:dc:a9:90'],
                                                                                       'in_nucast_pkts': '360',
                                                                                       'in_octets': '3047973644',
                                                                                       'in_ucast_pkts': '3207754',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '3865998534',
                                                                                       'out_ucast_pkts': '3411241',
                                                                                       'poller_hash': 'QURUTjIzMTI0RjNFIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/30',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/30',
                                                                                      'id': 'xgspon '
                                                                                            '0/30',
                                                                                      'links': [       {       'connected_time': '10735',
                                                                                                               'fiber_length': 13835,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RjNFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.514}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/30',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/31',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:a6:f0'],
                                                                                       'in_nucast_pkts': '354',
                                                                                       'in_octets': '7545875',
                                                                                       'in_ucast_pkts': '24250',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '23152246',
                                                                                       'out_ucast_pkts': '31353',
                                                                                       'poller_hash': 'QURUTjIzMTI0QjdDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:9d:e0'],
                                                                                       'in_nucast_pkts': '353',
                                                                                       'in_octets': '187970867',
                                                                                       'in_ucast_pkts': '482956',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1039638494',
                                                                                       'out_ucast_pkts': '979657',
                                                                                       'poller_hash': 'QURUTjI0MzYzQkNFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b4:50'],
                                                                                       'in_nucast_pkts': '335',
                                                                                       'in_octets': '111889214',
                                                                                       'in_ucast_pkts': '288282',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '798255098',
                                                                                       'out_ucast_pkts': '679095',
                                                                                       'poller_hash': 'QURUTjI0MzYzQ0FFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:b9:b0'],
                                                                                       'in_nucast_pkts': '324',
                                                                                       'in_octets': '372800126',
                                                                                       'in_ucast_pkts': '1249441',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1',
                                                                                       'out_octets': '2802365785',
                                                                                       'out_ucast_pkts': '2465921',
                                                                                       'poller_hash': 'QURUTjI0MzE0NTczIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/31',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/31',
                                                                                      'id': 'xgspon '
                                                                                            '0/31',
                                                                                      'links': [       {       'connected_time': '10396',
                                                                                                               'fiber_length': 15963,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QjdDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.325},
                                                                                                       {       'connected_time': '10236',
                                                                                                               'fiber_length': 15822,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQkNFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.325},
                                                                                                       {       'connected_time': '9616',
                                                                                                               'fiber_length': 15002,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQ0FFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.325},
                                                                                                       {       'connected_time': '9145',
                                                                                                               'fiber_length': 15712,
                                                                                                               'poller_hash': 'QURUTjI0MzE0NTczIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.325}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/31',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/32',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:50:27:60'],
                                                                                       'in_nucast_pkts': '410',
                                                                                       'in_octets': '113671917',
                                                                                       'in_ucast_pkts': '260446',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '481885682',
                                                                                       'out_ucast_pkts': '387448',
                                                                                       'poller_hash': 'QURUTjI0MzE0MjhDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ae:10'],
                                                                                       'in_nucast_pkts': '398',
                                                                                       'in_octets': '16801186',
                                                                                       'in_ucast_pkts': '60418',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '41618853',
                                                                                       'out_ucast_pkts': '58834',
                                                                                       'poller_hash': 'QURUTjI0MzYzRDZFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:50:00'],
                                                                                       'in_nucast_pkts': '384',
                                                                                       'in_octets': '7575294',
                                                                                       'in_ucast_pkts': '19653',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '13780860',
                                                                                       'out_ucast_pkts': '23372',
                                                                                       'poller_hash': 'QURUTjI0MzE0MUUwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:06:50'],
                                                                                       'in_nucast_pkts': '364',
                                                                                       'in_octets': '237885814',
                                                                                       'in_ucast_pkts': '2586699',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '10960583868',
                                                                                       'out_ucast_pkts': '7327483',
                                                                                       'poller_hash': 'QURUTjI0MzYzRDhGIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/32',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMg==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/32',
                                                                                      'id': 'xgspon '
                                                                                            '0/32',
                                                                                      'links': [       {       'connected_time': '12075',
                                                                                                               'fiber_length': 16428,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MjhDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3724},
                                                                                                       {       'connected_time': '11746',
                                                                                                               'fiber_length': 16238,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRDZFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -13.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3724},
                                                                                                       {       'connected_time': '11335',
                                                                                                               'fiber_length': 16214,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MUUwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3724},
                                                                                                       {       'connected_time': '10735',
                                                                                                               'fiber_length': 16282,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRDhGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3724}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/32',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/33',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/33',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMw==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/33',
                                                                                      'id': 'xgspon '
                                                                                            '0/33',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/33',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMw==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/34',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:c0'],
                                                                                       'in_nucast_pkts': '354',
                                                                                       'in_octets': '147713761',
                                                                                       'in_ucast_pkts': '346252',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '242285559',
                                                                                       'out_ucast_pkts': '356659',
                                                                                       'poller_hash': 'QURUTjIzMTI0MzEzIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/34',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/34',
                                                                                      'id': 'xgspon '
                                                                                            '0/34',
                                                                                      'links': [       {       'connected_time': '10401',
                                                                                                               'fiber_length': 18524,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MzEzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3184}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/34',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/35',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:a5:80'],
                                                                                       'in_nucast_pkts': '348',
                                                                                       'in_octets': '518603969',
                                                                                       'in_ucast_pkts': '823041',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1922800238',
                                                                                       'out_ucast_pkts': '1867021',
                                                                                       'poller_hash': 'QURUTjIzMTI0ODAyIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/35',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/35',
                                                                                      'id': 'xgspon '
                                                                                            '0/35',
                                                                                      'links': [       {       'connected_time': '10225',
                                                                                                               'fiber_length': 19286,
                                                                                                               'poller_hash': 'QURUTjIzMTI0ODAyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3199}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/35',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/36',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:80'],
                                                                                       'in_nucast_pkts': '335',
                                                                                       'in_octets': '131697222',
                                                                                       'in_ucast_pkts': '667878',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1550766700',
                                                                                       'out_ucast_pkts': '1241353',
                                                                                       'poller_hash': 'QURUTjIzMTI0RTA1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:ff:d0'],
                                                                                       'in_nucast_pkts': '466',
                                                                                       'in_octets': '4475804',
                                                                                       'in_ucast_pkts': '17296',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '29654500',
                                                                                       'out_ucast_pkts': '31303',
                                                                                       'poller_hash': 'QURUTjI0Mzg5QUQ3IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/36',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNg==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/36',
                                                                                      'id': 'xgspon '
                                                                                            '0/36',
                                                                                      'links': [       {       'connected_time': '9599',
                                                                                                               'fiber_length': 19392,
                                                                                                               'poller_hash': 'QURUTjIzMTI0RTA1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3194},
                                                                                                       {       'connected_time': '13830',
                                                                                                               'fiber_length': 19484,
                                                                                                               'poller_hash': 'QURUTjI0Mzg5QUQ3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3194}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/36',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/37',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:4c:08:c0'],
                                                                                       'in_nucast_pkts': '404',
                                                                                       'in_octets': '114352657',
                                                                                       'in_ucast_pkts': '280310',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1068000591',
                                                                                       'out_ucast_pkts': '862659',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzc3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:d6:3e:f0'],
                                                                                       'in_nucast_pkts': '318',
                                                                                       'in_octets': '1501185',
                                                                                       'in_ucast_pkts': '3878',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4',
                                                                                       'out_octets': '618004',
                                                                                       'out_ucast_pkts': '3352',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTk4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:51:80'],
                                                                                       'in_nucast_pkts': '292',
                                                                                       'in_octets': '319697907',
                                                                                       'in_ucast_pkts': '280722',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '336229569',
                                                                                       'out_ucast_pkts': '313730',
                                                                                       'poller_hash': 'QURUTjI0MzYzRDNCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:55:60'],
                                                                                       'in_nucast_pkts': '214',
                                                                                       'in_octets': '1189257',
                                                                                       'in_ucast_pkts': '5539',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '4245637',
                                                                                       'out_ucast_pkts': '6073',
                                                                                       'poller_hash': 'QURUTjI0MzYzRTBFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:60:c0'],
                                                                                       'in_nucast_pkts': '283',
                                                                                       'in_octets': '298891633',
                                                                                       'in_ucast_pkts': '298118',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '38244010',
                                                                                       'out_ucast_pkts': '59371',
                                                                                       'poller_hash': 'QURUTjI0MzE0MTU4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:61:b0'],
                                                                                       'in_nucast_pkts': '285',
                                                                                       'in_octets': '4258696',
                                                                                       'in_ucast_pkts': '16053',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '14346416',
                                                                                       'out_ucast_pkts': '19999',
                                                                                       'poller_hash': 'QURUTjI0MzE0MUM3IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/37',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNw==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/37',
                                                                                      'id': 'xgspon '
                                                                                            '0/37',
                                                                                      'links': [       {       'connected_time': '11845',
                                                                                                               'fiber_length': 20659,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzc3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3127},
                                                                                                       {       'connected_time': '8506',
                                                                                                               'fiber_length': 20359,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTk4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3127},
                                                                                                       {       'connected_time': '8336',
                                                                                                               'fiber_length': 21195,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRDNCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3127},
                                                                                                       {       'connected_time': '5786',
                                                                                                               'fiber_length': 21401,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRTBFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3127},
                                                                                                       {       'connected_time': '8055',
                                                                                                               'fiber_length': 20357,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MTU4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3127},
                                                                                                       {       'connected_time': '7885',
                                                                                                               'fiber_length': 20931,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MUM3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3127}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/37',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNw==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/38',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:03:40'],
                                                                                       'in_nucast_pkts': '273',
                                                                                       'in_octets': '70068336',
                                                                                       'in_ucast_pkts': '226855',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '1855721505',
                                                                                       'out_ucast_pkts': '1277516',
                                                                                       'poller_hash': 'QURUTjIzMTI1MENFIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ac:50'],
                                                                                       'in_nucast_pkts': '371',
                                                                                       'in_octets': '186838357',
                                                                                       'in_ucast_pkts': '1955695',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '9387461240',
                                                                                       'out_ucast_pkts': '6596709',
                                                                                       'poller_hash': 'QURUTjI0MzYzQ0REIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:8b:e0'],
                                                                                       'in_nucast_pkts': '368',
                                                                                       'in_octets': '402636359',
                                                                                       'in_ucast_pkts': '1914817',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '3756709932',
                                                                                       'out_ucast_pkts': '3144430',
                                                                                       'poller_hash': 'QURUTjI0Mzg5ODVBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:42:00'],
                                                                                       'in_nucast_pkts': '265',
                                                                                       'in_octets': '32432320',
                                                                                       'in_ucast_pkts': '109097',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1',
                                                                                       'out_octets': '100418286',
                                                                                       'out_ucast_pkts': '132715',
                                                                                       'poller_hash': 'QURUTjI0MzE0MUY3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dd:69:f0'],
                                                                                       'in_nucast_pkts': '455',
                                                                                       'in_octets': '3392834535',
                                                                                       'in_ucast_pkts': '4119641',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '3908400695',
                                                                                       'out_ucast_pkts': '4099334',
                                                                                       'poller_hash': 'QURUTjI0MzE0MjkzIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/38',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/38',
                                                                                      'id': 'xgspon '
                                                                                            '0/38',
                                                                                      'links': [       {       'connected_time': '7755',
                                                                                                               'fiber_length': 22608,
                                                                                                               'poller_hash': 'QURUTjIzMTI1MENFIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.658},
                                                                                                       {       'connected_time': '10926',
                                                                                                               'fiber_length': 22540,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQ0REIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.658},
                                                                                                       {       'connected_time': '10955',
                                                                                                               'fiber_length': 22280,
                                                                                                               'poller_hash': 'QURUTjI0Mzg5ODVBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.658},
                                                                                                       {       'connected_time': '7615',
                                                                                                               'fiber_length': 23438,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MUY3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.658},
                                                                                                       {       'connected_time': '13045',
                                                                                                               'fiber_length': 23309,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MjkzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.658}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/38',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/39',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:50:6e:c0'],
                                                                                       'in_nucast_pkts': '423',
                                                                                       'in_octets': '439947871',
                                                                                       'in_ucast_pkts': '2498522',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '9821310954',
                                                                                       'out_ucast_pkts': '6965154',
                                                                                       'poller_hash': 'QURUTjIzMTI0QkI4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:82:20'],
                                                                                       'in_nucast_pkts': '264',
                                                                                       'in_octets': '32350400',
                                                                                       'in_ucast_pkts': '53031',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '27088190',
                                                                                       'out_ucast_pkts': '37677',
                                                                                       'poller_hash': 'QURUTjIzMTI0QzQ3IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a8:d0'],
                                                                                       'in_nucast_pkts': '346',
                                                                                       'in_octets': '280013737',
                                                                                       'in_ucast_pkts': '511129',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4',
                                                                                       'out_octets': '769522058',
                                                                                       'out_ucast_pkts': '752311',
                                                                                       'poller_hash': 'QURUTjIzMTI0OTFCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:18:20'],
                                                                                       'in_nucast_pkts': '266',
                                                                                       'in_octets': '3165788',
                                                                                       'in_ucast_pkts': '13376',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3',
                                                                                       'out_octets': '2704308',
                                                                                       'out_ucast_pkts': '11696',
                                                                                       'poller_hash': 'QURUTjI0MzYzQ0UyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:5d:20'],
                                                                                       'in_nucast_pkts': '261',
                                                                                       'in_octets': '80334767',
                                                                                       'in_ucast_pkts': '525650',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '1',
                                                                                       'out_octets': '2663386215',
                                                                                       'out_ucast_pkts': '1861618',
                                                                                       'poller_hash': 'QURUTjI0MzYzQ0UwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ae:00'],
                                                                                       'in_nucast_pkts': '256',
                                                                                       'in_octets': '24310565',
                                                                                       'in_ucast_pkts': '60306',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '52819271',
                                                                                       'out_ucast_pkts': '67413',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzVDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:af:10'],
                                                                                       'in_nucast_pkts': '252',
                                                                                       'in_octets': '132752847',
                                                                                       'in_ucast_pkts': '281880',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '678981586',
                                                                                       'out_ucast_pkts': '569982',
                                                                                       'poller_hash': 'QURUTjI0MzYzRTY1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:20:e0'],
                                                                                       'in_nucast_pkts': '247',
                                                                                       'in_octets': '108395500',
                                                                                       'in_ucast_pkts': '122156',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '458423462',
                                                                                       'out_ucast_pkts': '353037',
                                                                                       'poller_hash': 'QURUTjI0MzE0MjQyIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:cc:60'],
                                                                                       'in_nucast_pkts': '232',
                                                                                       'in_octets': '185068829',
                                                                                       'in_ucast_pkts': '1034480',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '3948536680',
                                                                                       'out_ucast_pkts': '2762039',
                                                                                       'poller_hash': 'QURUTjI0MzE0MTg5IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/39',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/39',
                                                                                      'id': 'xgspon '
                                                                                            '0/39',
                                                                                      'links': [       {       'connected_time': '12195',
                                                                                                               'fiber_length': 21025,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QkI4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.401},
                                                                                                       {       'connected_time': '7505',
                                                                                                               'fiber_length': 20938,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QzQ3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.401},
                                                                                                       {       'connected_time': '9235',
                                                                                                               'fiber_length': 21128,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OTFCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.401},
                                                                                                       {       'connected_time': '7374',
                                                                                                               'fiber_length': 21189,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQ0UyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.401},
                                                                                                       {       'connected_time': '7305',
                                                                                                               'fiber_length': 21144,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQ0UwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.401},
                                                                                                       {       'connected_time': '7235',
                                                                                                               'fiber_length': 21170,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzVDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.401},
                                                                                                       {       'connected_time': '7115',
                                                                                                               'fiber_length': 21105,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRTY1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.401},
                                                                                                       {       'connected_time': '7045',
                                                                                                               'fiber_length': 20899,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MjQyIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.401},
                                                                                                       {       'connected_time': '6786',
                                                                                                               'fiber_length': 21098,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MTg5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.401}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/39',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/4',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:50:bb:00'],
                                                                                       'in_nucast_pkts': '444',
                                                                                       'in_octets': '148301056',
                                                                                       'in_ucast_pkts': '1064875',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '4749087359',
                                                                                       'out_ucast_pkts': '3324152',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUZBIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:25:10'],
                                                                                       'in_nucast_pkts': '274',
                                                                                       'in_octets': '142695515',
                                                                                       'in_ucast_pkts': '136615',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '77023990',
                                                                                       'out_ucast_pkts': '86514',
                                                                                       'poller_hash': 'QURUTjIzMTQ0OUJCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ad:20'],
                                                                                       'in_nucast_pkts': '493',
                                                                                       'in_octets': '603626029',
                                                                                       'in_ucast_pkts': '4061961',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '30525045979',
                                                                                       'out_ucast_pkts': '20467478',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QjM4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:a9:10'],
                                                                                       'in_nucast_pkts': '462',
                                                                                       'in_octets': '131974012',
                                                                                       'in_ucast_pkts': '138806',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '153872527',
                                                                                       'out_ucast_pkts': '143120',
                                                                                       'poller_hash': 'QURUTjIzMTQ0Q0Q5IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/4',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/4',
                                                                                      'id': 'xgspon '
                                                                                            '0/4',
                                                                                      'links': [       {       'connected_time': '13211',
                                                                                                               'fiber_length': 13634,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZBIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3734},
                                                                                                       {       'connected_time': '7801',
                                                                                                               'fiber_length': 13835,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0OUJCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3734},
                                                                                                       {       'connected_time': '13992',
                                                                                                               'fiber_length': 14557,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QjM4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3734},
                                                                                                       {       'connected_time': '13821',
                                                                                                               'fiber_length': 14541,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0Q5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3734}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/4',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/40',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:4c:30'],
                                                                                       'in_nucast_pkts': '483',
                                                                                       'in_octets': '3842103',
                                                                                       'in_ucast_pkts': '23681',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '63473472',
                                                                                       'out_ucast_pkts': '54463',
                                                                                       'poller_hash': 'QURUTjIzMTI0QTE5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:51:d0'],
                                                                                       'in_nucast_pkts': '453',
                                                                                       'in_octets': '277993925',
                                                                                       'in_ucast_pkts': '793672',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '3176166591',
                                                                                       'out_ucast_pkts': '2364957',
                                                                                       'poller_hash': 'QURUTjI0MzYzRTM5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:0a:40'],
                                                                                       'in_nucast_pkts': '453',
                                                                                       'in_octets': '34072316',
                                                                                       'in_ucast_pkts': '151624',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '612346539',
                                                                                       'out_ucast_pkts': '476736',
                                                                                       'poller_hash': 'QURUTjI0MzYzQjA3IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/40',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80MA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/40',
                                                                                      'id': 'xgspon '
                                                                                            '0/40',
                                                                                      'links': [       {       'connected_time': '13676',
                                                                                                               'fiber_length': 22184,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QTE5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4285},
                                                                                                       {       'connected_time': '13476',
                                                                                                               'fiber_length': 22643,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRTM5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4285},
                                                                                                       {       'connected_time': '13215',
                                                                                                               'fiber_length': 22536,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQjA3IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4285}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/40',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80MA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/41',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:51:51:b0'],
                                                                                       'in_nucast_pkts': '483',
                                                                                       'in_octets': '190529128',
                                                                                       'in_ucast_pkts': '499404',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '844189715',
                                                                                       'out_ucast_pkts': '704096',
                                                                                       'poller_hash': 'QURUTjIzMTI0MThGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ad:e0'],
                                                                                       'in_nucast_pkts': '481',
                                                                                       'in_octets': '596418325',
                                                                                       'in_ucast_pkts': '1150844',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '2713399495',
                                                                                       'out_ucast_pkts': '2079163',
                                                                                       'poller_hash': 'QURUTjIzMTI0Q0IzIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/41',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80MQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/41',
                                                                                      'id': 'xgspon '
                                                                                            '0/41',
                                                                                      'links': [       {       'connected_time': '14056',
                                                                                                               'fiber_length': 22879,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MThGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -19.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4551},
                                                                                                       {       'connected_time': '13987',
                                                                                                               'fiber_length': 22969,
                                                                                                               'poller_hash': 'QURUTjIzMTI0Q0IzIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.4551}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/41',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80MQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/42',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:4c:00:e0'],
                                                                                       'in_nucast_pkts': '476',
                                                                                       'in_octets': '151518526',
                                                                                       'in_ucast_pkts': '308639',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '643545462',
                                                                                       'out_ucast_pkts': '493210',
                                                                                       'poller_hash': 'QURUTjI0Mzg5Nzk0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:64:70'],
                                                                                       'in_nucast_pkts': '462',
                                                                                       'in_octets': '25881222',
                                                                                       'in_ucast_pkts': '104574',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '401822315',
                                                                                       'out_ucast_pkts': '303416',
                                                                                       'poller_hash': 'QURUTjI0MzE0NTAwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ad:80'],
                                                                                       'in_nucast_pkts': '373',
                                                                                       'in_octets': '134813114',
                                                                                       'in_ucast_pkts': '436485',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1024438683',
                                                                                       'out_ucast_pkts': '886825',
                                                                                       'poller_hash': 'QURUTjI0MzE0MjYwIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:01:40'],
                                                                                       'in_nucast_pkts': '362',
                                                                                       'in_octets': '201705501',
                                                                                       'in_ucast_pkts': '258577',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '123740882',
                                                                                       'out_ucast_pkts': '162937',
                                                                                       'poller_hash': 'QURUTjI0MzYzRTA1IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/42',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80Mg==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/42',
                                                                                      'id': 'xgspon '
                                                                                            '0/42',
                                                                                      'links': [       {       'connected_time': '13831',
                                                                                                               'fiber_length': 23714,
                                                                                                               'poller_hash': 'QURUTjI0Mzg5Nzk0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5986},
                                                                                                       {       'connected_time': '13681',
                                                                                                               'fiber_length': 23269,
                                                                                                               'poller_hash': 'QURUTjI0MzE0NTAwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5986},
                                                                                                       {       'connected_time': '10741',
                                                                                                               'fiber_length': 23407,
                                                                                                               'poller_hash': 'QURUTjI0MzE0MjYwIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5986},
                                                                                                       {       'connected_time': '10401',
                                                                                                               'fiber_length': 23187,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRTA1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -23.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.5986}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/42',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80Mg==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/43',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:a9:60'],
                                                                                       'in_nucast_pkts': '326',
                                                                                       'in_octets': '130676550',
                                                                                       'in_ucast_pkts': '131633',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '136433948',
                                                                                       'out_ucast_pkts': '133965',
                                                                                       'poller_hash': 'QURUTjIzMTI0NDFCIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:59:20'],
                                                                                       'in_nucast_pkts': '316',
                                                                                       'in_octets': '355790001',
                                                                                       'in_ucast_pkts': '448505',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '905402177',
                                                                                       'out_ucast_pkts': '768021',
                                                                                       'poller_hash': 'QURUTjI0MzYzRTVDIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:40:f0'],
                                                                                       'in_nucast_pkts': '344',
                                                                                       'in_octets': '102863677',
                                                                                       'in_ucast_pkts': '287941',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '774207983',
                                                                                       'out_ucast_pkts': '648073',
                                                                                       'poller_hash': 'QURUTjIzMTI0MjYxIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/43',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80Mw==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/43',
                                                                                      'id': 'xgspon '
                                                                                            '0/43',
                                                                                      'links': [       {       'connected_time': '9596',
                                                                                                               'fiber_length': 22218,
                                                                                                               'poller_hash': 'QURUTjIzMTI0NDFCIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.571},
                                                                                                       {       'connected_time': '9166',
                                                                                                               'fiber_length': 21948,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRTVDIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.571},
                                                                                                       {       'connected_time': '10226',
                                                                                                               'fiber_length': 21910,
                                                                                                               'poller_hash': 'QURUTjIzMTI0MjYxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.571}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/43',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80Mw==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/44',
                                                              'links': [       {       'bridged_mac_addresses': [       '8c:30:66:a0:51:91'],
                                                                                       'in_nucast_pkts': '12',
                                                                                       'in_octets': '25330521',
                                                                                       'in_ucast_pkts': '132223',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '4',
                                                                                       'out_octets': '1030177346',
                                                                                       'out_ucast_pkts': '837689',
                                                                                       'poller_hash': 'QURUTjIzMTI0OUREIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a9:20'],
                                                                                       'in_nucast_pkts': '239',
                                                                                       'in_octets': '41551028',
                                                                                       'in_ucast_pkts': '146435',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '647296471',
                                                                                       'out_ucast_pkts': '473841',
                                                                                       'poller_hash': 'QURUTjIzMTI0QTk0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4c:01:b0'],
                                                                                       'in_nucast_pkts': '234',
                                                                                       'in_octets': '7584355',
                                                                                       'in_ucast_pkts': '48306',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '106024528',
                                                                                       'out_ucast_pkts': '80583',
                                                                                       'poller_hash': 'QURUTjI0MzYzQzY1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ab:e0'],
                                                                                       'in_nucast_pkts': '232',
                                                                                       'in_octets': '1711866',
                                                                                       'in_ucast_pkts': '7762',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '4735143',
                                                                                       'out_ucast_pkts': '7073',
                                                                                       'poller_hash': 'QURUTjI0MzYzQkJGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:2a:00'],
                                                                                       'in_nucast_pkts': '222',
                                                                                       'in_octets': '83135370',
                                                                                       'in_ucast_pkts': '278430',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1620080306',
                                                                                       'out_ucast_pkts': '1179322',
                                                                                       'poller_hash': 'QURUTjI0MzE0NEY0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:ae:60'],
                                                                                       'in_nucast_pkts': '220',
                                                                                       'in_octets': '131014610',
                                                                                       'in_ucast_pkts': '136231',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '148345529',
                                                                                       'out_ucast_pkts': '138301',
                                                                                       'poller_hash': 'QURUTjI0MzYzRDcxIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:ff:a0'],
                                                                                       'in_nucast_pkts': '357',
                                                                                       'in_octets': '207164952',
                                                                                       'in_ucast_pkts': '2834583',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '11383810480',
                                                                                       'out_ucast_pkts': '7582616',
                                                                                       'poller_hash': 'QURUTjIzMTQ1MTVEIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/44',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80NA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/44',
                                                                                      'id': 'xgspon '
                                                                                            '0/44',
                                                                                      'links': [       {       'connected_time': '6761',
                                                                                                               'fiber_length': 23888,
                                                                                                               'poller_hash': 'QURUTjIzMTI0OUREIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.5,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2763},
                                                                                                       {       'connected_time': '6680',
                                                                                                               'fiber_length': 23035,
                                                                                                               'poller_hash': 'QURUTjIzMTI0QTk0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -22.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2763},
                                                                                                       {       'connected_time': '6600',
                                                                                                               'fiber_length': 24073,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQzY1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2763},
                                                                                                       {       'connected_time': '6530',
                                                                                                               'fiber_length': 22898,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQkJGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2763},
                                                                                                       {       'connected_time': '6369',
                                                                                                               'fiber_length': 22541,
                                                                                                               'poller_hash': 'QURUTjI0MzE0NEY0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2763},
                                                                                                       {       'connected_time': '6181',
                                                                                                               'fiber_length': 23123,
                                                                                                               'poller_hash': 'QURUTjI0MzYzRDcxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2763},
                                                                                                       {       'connected_time': '10261',
                                                                                                               'fiber_length': 23316,
                                                                                                               'poller_hash': 'QURUTjIzMTQ1MTVEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -21.0,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.2763}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/44',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80NA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/45',
                                                              'links': [       {       'bridged_mac_addresses': [       '38:f8:f6:52:b1:90'],
                                                                                       'in_nucast_pkts': '384',
                                                                                       'in_octets': '1719981',
                                                                                       'in_ucast_pkts': '6253',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '2845501',
                                                                                       'out_ucast_pkts': '5928',
                                                                                       'poller_hash': 'QURUTjI0MzE0NTNGIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/45',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80NQ==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/45',
                                                                                      'id': 'xgspon '
                                                                                            '0/45',
                                                                                      'links': [       {       'connected_time': '11336',
                                                                                                               'fiber_length': 17838,
                                                                                                               'poller_hash': 'QURUTjI0MzE0NTNGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -14.8,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3145}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/45',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80NQ==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/46',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/46',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80Ng==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/46',
                                                                                      'id': 'xgspon '
                                                                                            '0/46',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/46',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80Ng==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/47',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/47',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80Nw==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/47',
                                                                                      'id': 'xgspon '
                                                                                            '0/47',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/47',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80Nw==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/48',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/48',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80OA==',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/48',
                                                                                      'id': 'xgspon '
                                                                                            '0/48',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/48',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80OA==',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/5',
                                                              'links': [       {       'bridged_mac_addresses': [       'e8:9f:80:eb:9e:5b'],
                                                                                       'in_nucast_pkts': '605',
                                                                                       'in_octets': '198887852',
                                                                                       'in_ucast_pkts': '786066',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '3796663195',
                                                                                       'out_ucast_pkts': '2843479',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RUQ1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:a7:00'],
                                                                                       'in_nucast_pkts': '477',
                                                                                       'in_octets': '13771424',
                                                                                       'in_ucast_pkts': '28341',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '5013813',
                                                                                       'out_ucast_pkts': '24003',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTI0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:4b:fa:a0'],
                                                                                       'in_nucast_pkts': '482',
                                                                                       'in_octets': '311809807',
                                                                                       'in_ucast_pkts': '2158143',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '13305901754',
                                                                                       'out_ucast_pkts': '9269573',
                                                                                       'poller_hash': 'QURUTjIzMTQ1MUU5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:51:2d:a0'],
                                                                                       'in_nucast_pkts': '470',
                                                                                       'in_octets': '527087727',
                                                                                       'in_ucast_pkts': '727775',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '815753307',
                                                                                       'out_ucast_pkts': '904700',
                                                                                       'poller_hash': 'QURUTjIzMTQ0RjFGIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:dc:5d:20'],
                                                                                       'in_nucast_pkts': '470',
                                                                                       'in_octets': '440597491',
                                                                                       'in_ucast_pkts': '1305189',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '5075483606',
                                                                                       'out_ucast_pkts': '4186022',
                                                                                       'poller_hash': 'QURUTjIzMTQ0Q0I2IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:28:80'],
                                                                                       'in_nucast_pkts': '455',
                                                                                       'in_octets': '91561499',
                                                                                       'in_ucast_pkts': '875864',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '7726807898',
                                                                                       'out_ucast_pkts': '5156831',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QkE0IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:a1:b0'],
                                                                                       'in_nucast_pkts': '483',
                                                                                       'in_octets': '148162483',
                                                                                       'in_ucast_pkts': '210464',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '296898628',
                                                                                       'out_ucast_pkts': '276284',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTE4IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:a1:e0'],
                                                                                       'in_nucast_pkts': '470',
                                                                                       'in_octets': '179910866',
                                                                                       'in_ucast_pkts': '644858',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '0',
                                                                                       'out_octets': '1958509744',
                                                                                       'out_ucast_pkts': '1550636',
                                                                                       'poller_hash': 'QURUTjIzMTIzRUQxIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/5',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC81',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/5',
                                                                                      'id': 'xgspon '
                                                                                            '0/5',
                                                                                      'links': [       {       'connected_time': '10397',
                                                                                                               'fiber_length': 15354,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RUQ1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3339},
                                                                                                       {       'connected_time': '13827',
                                                                                                               'fiber_length': 15338,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTI0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -18.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3339},
                                                                                                       {       'connected_time': '13987',
                                                                                                               'fiber_length': 15473,
                                                                                                               'poller_hash': 'QURUTjIzMTQ1MUU5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3339},
                                                                                                       {       'connected_time': '13667',
                                                                                                               'fiber_length': 15664,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0RjFGIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -20.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3339},
                                                                                                       {       'connected_time': '13467',
                                                                                                               'fiber_length': 15534,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0I2IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.2,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3339},
                                                                                                       {       'connected_time': '13217',
                                                                                                               'fiber_length': 15534,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QkE0IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3339},
                                                                                                       {       'connected_time': '14137',
                                                                                                               'fiber_length': 15304,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTE4IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3339},
                                                                                                       {       'connected_time': '13987',
                                                                                                               'fiber_length': 15444,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRUQxIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.7,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3339}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/5',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/6',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:50:5c:10'],
                                                                                       'in_nucast_pkts': '482',
                                                                                       'in_octets': '100750444',
                                                                                       'in_ucast_pkts': '820066',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '2510666219',
                                                                                       'out_ucast_pkts': '1762190',
                                                                                       'poller_hash': 'QURUTjIzMTQ1MkZEIHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       '38:f8:f6:52:b2:30'],
                                                                                       'in_nucast_pkts': '475',
                                                                                       'in_octets': '92700805',
                                                                                       'in_ucast_pkts': '389024',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '3026149453',
                                                                                       'out_ucast_pkts': '2343828',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTI5IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:6e:20'],
                                                                                       'in_nucast_pkts': '143639',
                                                                                       'in_octets': '289424390272',
                                                                                       'in_ucast_pkts': '953720358',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '3299',
                                                                                       'out_octets': '3271451684392',
                                                                                       'out_ucast_pkts': '2357486524',
                                                                                       'poller_hash': 'QURUTjI0MzYzQkY5IHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/6',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC82',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/6',
                                                                                      'id': 'xgspon '
                                                                                            '0/6',
                                                                                      'links': [       {       'connected_time': '13676',
                                                                                                               'fiber_length': 14519,
                                                                                                               'poller_hash': 'QURUTjIzMTQ1MkZEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.9,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.301},
                                                                                                       {       'connected_time': '13466',
                                                                                                               'fiber_length': 14572,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTI5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -15.6,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.301},
                                                                                                       {       'connected_time': '4308364',
                                                                                                               'fiber_length': 14403,
                                                                                                               'poller_hash': 'QURUTjI0MzYzQkY5IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.3,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.301}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/6',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/7',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/7',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC83',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/7',
                                                                                      'id': 'xgspon '
                                                                                            '0/7',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/7',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC83',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/8',
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/8',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC84',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/8',
                                                                                      'id': 'xgspon '
                                                                                            '0/8',
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/8',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC84',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': True,
                                                              'id': '0/9',
                                                              'links': [       {       'bridged_mac_addresses': [       'cc:66:18:50:59:e0'],
                                                                                       'in_nucast_pkts': '487',
                                                                                       'in_octets': '227411280',
                                                                                       'in_ucast_pkts': '1600506',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '8356626066',
                                                                                       'out_ucast_pkts': '5772513',
                                                                                       'poller_hash': 'QURUTjIzMTQ0QTg1IHBlZXJpbnRm'},
                                                                               {       'bridged_mac_addresses': [       'cc:66:18:50:fe:90'],
                                                                                       'in_nucast_pkts': '480',
                                                                                       'in_octets': '443776936',
                                                                                       'in_ucast_pkts': '5950972',
                                                                                       'oper_up': True,
                                                                                       'out_nucast_pkts': '2',
                                                                                       'out_octets': '14021882176',
                                                                                       'out_ucast_pkts': '9510069',
                                                                                       'poller_hash': 'QURUTjIzMTIzRjBEIHBlZXJpbnRm'}],
                                                              'manufacturer': 'Adtran',
                                                              'model': '1442544F2',
                                                              'name': '0/9',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC85',
                                                              'pons': [       {       'description': 'xgspon '
                                                                                                     '0/9',
                                                                                      'id': 'xgspon '
                                                                                            '0/9',
                                                                                      'links': [       {       'connected_time': '14133',
                                                                                                               'fiber_length': 17009,
                                                                                                               'poller_hash': 'QURUTjIzMTQ0QTg1IHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -17.1,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3089},
                                                                                                       {       'connected_time': '13992',
                                                                                                               'fiber_length': 16868,
                                                                                                               'poller_hash': 'QURUTjIzMTIzRjBEIHBlZXJwb24=',
                                                                                                               'rx_link_rate': '10000000000',
                                                                                                               'rx_power': -16.4,
                                                                                                               'tx_link_rate': '10000000000',
                                                                                                               'tx_power': 5.3089}],
                                                                                      'max_distance': 20000,
                                                                                      'maximum_loss': -7.0,
                                                                                      'minimum_loss': -28.0,
                                                                                      'name': 'xgspon '
                                                                                              '0/9',
                                                                                      'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                      'pon_technology': 'XGS_PON',
                                                                                      'rx_link_rate': '10000000000',
                                                                                      'split_ratio': 128,
                                                                                      'tx_link_rate': '10000000000'}]},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/1',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '54:22:e0:77:ed:20',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/1',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8x',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/1:2',
                                                              'mac_address': '54:22:e0:77:ed:21',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/1:2',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8xOjI=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/1:3',
                                                              'mac_address': '54:22:e0:77:ed:22',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/1:3',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8xOjM=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/1:4',
                                                              'mac_address': '54:22:e0:77:ed:23',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/1:4',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8xOjQ=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/1:5',
                                                              'mac_address': '54:22:e0:77:ed:24',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/1:5',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8xOjU=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/1:6',
                                                              'mac_address': '54:22:e0:77:ed:25',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/1:6',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8xOjY=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/1:7',
                                                              'mac_address': '54:22:e0:77:ed:26',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/1:7',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8xOjc=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/1:8',
                                                              'mac_address': '54:22:e0:77:ed:27',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/1:8',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8xOjg=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/2',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '54:22:e0:77:ed:28',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/2',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8y',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/2:2',
                                                              'mac_address': '54:22:e0:77:ed:29',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/2:2',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8yOjI=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/2:3',
                                                              'mac_address': '54:22:e0:77:ed:2a',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/2:3',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8yOjM=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/2:4',
                                                              'mac_address': '54:22:e0:77:ed:2b',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/2:4',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8yOjQ=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/2:5',
                                                              'mac_address': '54:22:e0:77:ed:2c',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/2:5',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8yOjU=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/2:6',
                                                              'mac_address': '54:22:e0:77:ed:2d',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/2:6',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8yOjY=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/2:7',
                                                              'mac_address': '54:22:e0:77:ed:2e',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/2:7',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8yOjc=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/2:8',
                                                              'mac_address': '54:22:e0:77:ed:2f',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/2:8',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8yOjg=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/3',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '54:22:e0:77:ed:18',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/3',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8z',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/3:2',
                                                              'mac_address': '54:22:e0:77:ed:19',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/3:2',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8zOjI=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/3:3',
                                                              'mac_address': '54:22:e0:77:ed:1a',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/3:3',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8zOjM=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/3:4',
                                                              'mac_address': '54:22:e0:77:ed:1b',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/3:4',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC8zOjQ=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/4',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '54:22:e0:77:ed:1c',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/4',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC80',
                                                              'speed': '100000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/4:2',
                                                              'mac_address': '54:22:e0:77:ed:1d',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/4:2',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC80OjI=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/4:3',
                                                              'mac_address': '54:22:e0:77:ed:1e',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/4:3',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC80OjM=',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/4:4',
                                                              'mac_address': '54:22:e0:77:ed:1f',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/4:4',
                                                              'oper_up': False,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC80OjQ=',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'ethernet 0/5',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '242929560',
                                                              'in_octets': '517510055542061',
                                                              'in_ucast_pkts': '385101306961',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '54:22:e0:77:ed:16',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/5',
                                                              'oper_up': True,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '50095813',
                                                              'out_octets': '52472419604218',
                                                              'out_ucast_pkts': '154601281473',
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC81',
                                                              'speed': '10000000000',
                                                              'type': 6},
                                                      {       'admin_up': False,
                                                              'id': 'ethernet 0/6',
                                                              'in_discards': '0',
                                                              'in_errors': '0',
                                                              'in_nucast_pkts': '0',
                                                              'in_octets': '0',
                                                              'in_ucast_pkts': '0',
                                                              'in_unknown_protos': '0',
                                                              'mac_address': '54:22:e0:77:ed:17',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'ethernet 0/6',
                                                              'oper_up': False,
                                                              'out_discards': '0',
                                                              'out_errors': '0',
                                                              'out_nucast_pkts': '0',
                                                              'out_octets': '0',
                                                              'out_ucast_pkts': '0',
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgZXRoZXJuZXQgMC82',
                                                              'speed': '25000000000',
                                                              'type': 6},
                                                      {       'admin_up': True,
                                                              'id': 'management 0/1',
                                                              'mac_address': '54:22:e0:77:ed:14',
                                                              'manufacturer': 'Adtran',
                                                              'name': 'management 0/1',
                                                              'oper_up': True,
                                                              'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgbWFuYWdlbWVudCAwLzE=',
                                                              'type': 6}],
                                'manufacturer': 'Adtran',
                                'mem': 47.5,
                                'model': 'SDX 6330-48',
                                'name': 'Maurice-x1',
                                'poller_hash': 'b2x0IExCQURUTjI0MjBBQTIzNw==',
                                'serial_number': 'LBADTN2420AA237',
                                'sw_version': 'sdx6330-RL-23.5-30',
                                'temperature': 63.0,
                                'uptime': '9237181'},
                'peers': [       {       'interfaces': [       {       'id': 'onu-subscr-if-47.1.0.0.eth.1.phy',
                                                                       'name': 'Maurice-x48-ONU0_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDgtT05VMF9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0RUYyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUYyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x48-ONU0',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '47.1.0"(ADTN23144EF2) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/48.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-10-02T11:06:05Z',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVGMg==',
                                         'serial_number': 'ADTN23144EF2',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '605',
                                                                       'in_octets': '198887852',
                                                                       'in_ucast_pkts': '786066',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x5-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '3796663195',
                                                                       'out_ucast_pkts': '2843479',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NS1PTlUwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '3796663195',
                                                                       'in_ucast_pkts': '2843479',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '3796663195',
                                                                                                'in_ucast_pkts': '2843479',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '605',
                                                                                                'out_octets': '198887852',
                                                                                                'out_ucast_pkts': '786066',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '605',
                                                                       'out_octets': '198887852',
                                                                       'out_ucast_pkts': '786066',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUQ1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10397',
                                                                                                                        'fiber_length': 15354,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUQ1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x5-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVENQ==',
                                         'serial_number': 'ADTN23144ED5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10397'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '477',
                                                                       'in_octets': '13771424',
                                                                       'in_ucast_pkts': '28341',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x5-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '5013813',
                                                                       'out_ucast_pkts': '24003',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NS1PTlUyX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '5013813',
                                                                       'in_ucast_pkts': '24003',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '5013813',
                                                                                                'in_ucast_pkts': '24003',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '477',
                                                                                                'out_octets': '13771424',
                                                                                                'out_ucast_pkts': '28341',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '477',
                                                                       'out_octets': '13771424',
                                                                       'out_ucast_pkts': '28341',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTI0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13827',
                                                                                                                        'fiber_length': 15338,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTI0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x5-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEEyNA==',
                                         'serial_number': 'ADTN23144A24',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13827'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '444',
                                                                       'in_octets': '148301056',
                                                                       'in_ucast_pkts': '1064875',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x4-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '4749087359',
                                                                       'out_ucast_pkts': '3324152',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NC1PTlUwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '4749087359',
                                                                       'in_ucast_pkts': '3324152',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '4749087359',
                                                                                                'in_ucast_pkts': '3324152',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '444',
                                                                                                'out_octets': '148301056',
                                                                                                'out_ucast_pkts': '1064875',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '444',
                                                                       'out_octets': '148301056',
                                                                       'out_ucast_pkts': '1064875',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUZBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13211',
                                                                                                                        'fiber_length': 13634,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUZBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x4-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVGQQ==',
                                         'serial_number': 'ADTN23144EFA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13211'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '417',
                                                                       'in_octets': '189644299',
                                                                       'in_ucast_pkts': '2141521',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x3-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '5117268931',
                                                                       'out_ucast_pkts': '3469122',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14My1PTlUwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '5117268931',
                                                                       'in_ucast_pkts': '3469122',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '5117268931',
                                                                                                'in_ucast_pkts': '3469122',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '417',
                                                                                                'out_octets': '189644299',
                                                                                                'out_ucast_pkts': '2141521',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '417',
                                                                       'out_octets': '189644299',
                                                                       'out_ucast_pkts': '2141521',
                                                                       'poller_hash': 'QURUTjIzMTQ0RUU3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '12084',
                                                                                                                        'fiber_length': 12881,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RUU3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x3-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEVFNw==',
                                         'serial_number': 'ADTN23144EE7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '12084'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '482',
                                                                       'in_octets': '311809807',
                                                                       'in_ucast_pkts': '2158143',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x5-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '13305901754',
                                                                       'out_ucast_pkts': '9269573',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NS1PTlUxX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '13305901754',
                                                                       'in_ucast_pkts': '9269573',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '13305901754',
                                                                                                'in_ucast_pkts': '9269573',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '482',
                                                                                                'out_octets': '311809807',
                                                                                                'out_ucast_pkts': '2158143',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '482',
                                                                       'out_octets': '311809807',
                                                                       'out_ucast_pkts': '2158143',
                                                                       'poller_hash': 'QURUTjIzMTQ1MUU5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13987',
                                                                                                                        'fiber_length': 15473,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ1MUU5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x5-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzE0NTFFOQ==',
                                         'serial_number': 'ADTN231451E9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13987'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '274',
                                                                       'in_octets': '142695515',
                                                                       'in_ucast_pkts': '136615',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x4-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '77023990',
                                                                       'out_ucast_pkts': '86514',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NC1PTlUxX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '77023990',
                                                                       'in_ucast_pkts': '86514',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '77023990',
                                                                                                'in_ucast_pkts': '86514',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '274',
                                                                                                'out_octets': '142695515',
                                                                                                'out_ucast_pkts': '136615',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '274',
                                                                       'out_octets': '142695515',
                                                                       'out_ucast_pkts': '136615',
                                                                       'poller_hash': 'QURUTjIzMTQ0OUJCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '7801',
                                                                                                                        'fiber_length': 13835,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0OUJCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x4-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzE0NDlCQg==',
                                         'serial_number': 'ADTN231449BB',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '7801'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '470',
                                                                       'in_octets': '527087727',
                                                                       'in_ucast_pkts': '727775',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x5-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '815753307',
                                                                       'out_ucast_pkts': '904700',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NS1PTlUzX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '815753307',
                                                                       'in_ucast_pkts': '904700',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '815753307',
                                                                                                'in_ucast_pkts': '904700',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '470',
                                                                                                'out_octets': '527087727',
                                                                                                'out_ucast_pkts': '727775',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '470',
                                                                       'out_octets': '527087727',
                                                                       'out_ucast_pkts': '727775',
                                                                       'poller_hash': 'QURUTjIzMTQ0RjFGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13667',
                                                                                                                        'fiber_length': 15664,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RjFGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x5-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEYxRg==',
                                         'serial_number': 'ADTN23144F1F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13667'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '493',
                                                                       'in_octets': '603626029',
                                                                       'in_ucast_pkts': '4061961',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x4-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '30525045979',
                                                                       'out_ucast_pkts': '20467478',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NC1PTlUyX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '30525045979',
                                                                       'in_ucast_pkts': '20467478',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '30525045979',
                                                                                                'in_ucast_pkts': '20467478',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '493',
                                                                                                'out_octets': '603626029',
                                                                                                'out_ucast_pkts': '4061961',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '493',
                                                                       'out_octets': '603626029',
                                                                       'out_ucast_pkts': '4061961',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjM4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13992',
                                                                                                                        'fiber_length': 14557,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjM4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x4-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIzOA==',
                                         'serial_number': 'ADTN23144B38',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13992'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-3.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '462',
                                                                       'in_octets': '131974012',
                                                                       'in_ucast_pkts': '138806',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x4-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '153872527',
                                                                       'out_ucast_pkts': '143120',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NC1PTlUzX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '153872527',
                                                                       'in_ucast_pkts': '143120',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '153872527',
                                                                                                'in_ucast_pkts': '143120',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '462',
                                                                                                'out_octets': '131974012',
                                                                                                'out_ucast_pkts': '138806',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '462',
                                                                       'out_octets': '131974012',
                                                                       'out_ucast_pkts': '138806',
                                                                       'poller_hash': 'QURUTjIzMTQ0Q0Q5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13821',
                                                                                                                        'fiber_length': 14541,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0Q5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x4-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzE0NENEOQ==',
                                         'serial_number': 'ADTN23144CD9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13821'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '482',
                                                                       'in_octets': '100750444',
                                                                       'in_ucast_pkts': '820066',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x6-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '2510666219',
                                                                       'out_ucast_pkts': '1762190',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14Ni1PTlUwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '2510666219',
                                                                       'in_ucast_pkts': '1762190',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '2510666219',
                                                                                                'in_ucast_pkts': '1762190',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '482',
                                                                                                'out_octets': '100750444',
                                                                                                'out_ucast_pkts': '820066',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '482',
                                                                       'out_octets': '100750444',
                                                                       'out_ucast_pkts': '820066',
                                                                       'poller_hash': 'QURUTjIzMTQ1MkZEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13676',
                                                                                                                        'fiber_length': 14519,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ1MkZEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x6-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzE0NTJGRA==',
                                         'serial_number': 'ADTN231452FD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13676'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '399',
                                                                       'in_octets': '316592568',
                                                                       'in_ucast_pkts': '451821',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x3-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '47409844',
                                                                       'out_ucast_pkts': '363406',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14My1PTlUxX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '47409844',
                                                                       'in_ucast_pkts': '363406',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '47409844',
                                                                                                'in_ucast_pkts': '363406',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '399',
                                                                                                'out_octets': '316592568',
                                                                                                'out_ucast_pkts': '451821',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '399',
                                                                       'out_octets': '316592568',
                                                                       'out_ucast_pkts': '451821',
                                                                       'poller_hash': 'QURUTjIzMTQ0QjJGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11745',
                                                                                                                        'fiber_length': 13443,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 6.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QjJGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x3-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEIyRg==',
                                         'serial_number': 'ADTN23144B2F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '11745'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-1.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '491',
                                                                       'in_octets': '33125671',
                                                                       'in_ucast_pkts': '99597',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x2-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '326285482',
                                                                       'out_ucast_pkts': '264921',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14Mi1PTlUwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '326285482',
                                                                       'in_ucast_pkts': '264921',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '326285482',
                                                                                                'in_ucast_pkts': '264921',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '491',
                                                                                                'out_octets': '33125671',
                                                                                                'out_ucast_pkts': '99597',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8y'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '491',
                                                                       'out_octets': '33125671',
                                                                       'out_ucast_pkts': '99597',
                                                                       'poller_hash': 'QURUTjIzMTQ1MjMyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13992',
                                                                                                                        'fiber_length': 11584,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8y',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ1MjMyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x2-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzE0NTIzMg==',
                                         'serial_number': 'ADTN23145232',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13992'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '475',
                                                                       'in_octets': '92700805',
                                                                       'in_ucast_pkts': '389024',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x6-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '3026149453',
                                                                       'out_ucast_pkts': '2343828',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14Ni1PTlUxX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '3026149453',
                                                                       'in_ucast_pkts': '2343828',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '3026149453',
                                                                                                'in_ucast_pkts': '2343828',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '475',
                                                                                                'out_octets': '92700805',
                                                                                                'out_ucast_pkts': '389024',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '475',
                                                                       'out_octets': '92700805',
                                                                       'out_ucast_pkts': '389024',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTI5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13466',
                                                                                                                        'fiber_length': 14572,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTI5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x6-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEEyOQ==',
                                         'serial_number': 'ADTN23144A29',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13466'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-0.1.0.0.eth.1.phy',
                                                                       'name': 'Maurice-x1-ONU0_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MS1PTlUwX0VUSF8x'},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTQ0QTlEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTlEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x1-ONU0',
                                         'offline_reason': 'OFFLINE_REASON_POWER_OFF',
                                         'offline_reason_detail': 'A local power '
                                                                  'outage has occurred '
                                                                  'for ONU "onu '
                                                                  '0.1.0"(ADTN23144A9D).',
                                         'offline_time': '2025-08-07T12:52:26Z',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE5RA==',
                                         'serial_number': 'ADTN23144A9D',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '214',
                                                                       'in_octets': '328858804',
                                                                       'in_ucast_pkts': '314716',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x3-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '430002187',
                                                                       'out_ucast_pkts': '396950',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14My1PTlUyX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '430002187',
                                                                       'in_ucast_pkts': '396950',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '430002187',
                                                                                                'in_ucast_pkts': '396950',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '214',
                                                                                                'out_octets': '328858804',
                                                                                                'out_ucast_pkts': '314716',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '214',
                                                                       'out_octets': '328858804',
                                                                       'out_ucast_pkts': '314716',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTQ4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '6284',
                                                                                                                        'fiber_length': 12846,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTQ4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x3-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE0OA==',
                                         'serial_number': 'ADTN23144A48',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '6284'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '470',
                                                                       'in_octets': '440597491',
                                                                       'in_ucast_pkts': '1305189',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x5-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '5075483606',
                                                                       'out_ucast_pkts': '4186022',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NS1PTlU0X0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '5075483606',
                                                                       'in_ucast_pkts': '4186022',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '5075483606',
                                                                                                'in_ucast_pkts': '4186022',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '470',
                                                                                                'out_octets': '440597491',
                                                                                                'out_ucast_pkts': '1305189',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '470',
                                                                       'out_octets': '440597491',
                                                                       'out_ucast_pkts': '1305189',
                                                                       'poller_hash': 'QURUTjIzMTQ0Q0I2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13467',
                                                                                                                        'fiber_length': 15534,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0Q0I2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x5-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzE0NENCNg==',
                                         'serial_number': 'ADTN23144CB6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13467'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '455',
                                                                       'in_octets': '91561499',
                                                                       'in_ucast_pkts': '875864',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x5-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '7726807898',
                                                                       'out_ucast_pkts': '5156831',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NS1PTlU1X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '7726807898',
                                                                       'in_ucast_pkts': '5156831',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '7726807898',
                                                                                                'in_ucast_pkts': '5156831',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '455',
                                                                                                'out_octets': '91561499',
                                                                                                'out_ucast_pkts': '875864',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '455',
                                                                       'out_octets': '91561499',
                                                                       'out_ucast_pkts': '875864',
                                                                       'poller_hash': 'QURUTjIzMTQ0QkE0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13217',
                                                                                                                        'fiber_length': 15534,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QkE0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x5-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEJBNA==',
                                         'serial_number': 'ADTN23144BA4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13217'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '487',
                                                                       'in_octets': '227411280',
                                                                       'in_ucast_pkts': '1600506',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x9-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '8356626066',
                                                                       'out_ucast_pkts': '5772513',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14OS1PTlUwX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '8356626066',
                                                                       'in_ucast_pkts': '5772513',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '8356626066',
                                                                                                'in_ucast_pkts': '5772513',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '487',
                                                                                                'out_octets': '227411280',
                                                                                                'out_ucast_pkts': '1600506',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '487',
                                                                       'out_octets': '227411280',
                                                                       'out_ucast_pkts': '1600506',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTg1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14133',
                                                                                                                        'fiber_length': 17009,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTg1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x9-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE4NQ==',
                                         'serial_number': 'ADTN23144A85',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14133'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '483',
                                                                       'in_octets': '148162483',
                                                                       'in_ucast_pkts': '210464',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x5-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '296898628',
                                                                       'out_ucast_pkts': '276284',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NS1PTlU2X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '296898628',
                                                                       'in_ucast_pkts': '276284',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '296898628',
                                                                                                'in_ucast_pkts': '276284',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '483',
                                                                                                'out_octets': '148162483',
                                                                                                'out_ucast_pkts': '210464',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '483',
                                                                       'out_octets': '148162483',
                                                                       'out_ucast_pkts': '210464',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTE4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14137',
                                                                                                                        'fiber_length': 15304,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTE4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x5-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEExOA==',
                                         'serial_number': 'ADTN23144A18',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14137'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '465',
                                                                       'in_octets': '856146',
                                                                       'in_ucast_pkts': '3319',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x11-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '249954',
                                                                       'out_ucast_pkts': '2410',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTEtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '249954',
                                                                       'in_ucast_pkts': '2410',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '249954',
                                                                                                'in_ucast_pkts': '2410',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '465',
                                                                                                'out_octets': '856146',
                                                                                                'out_ucast_pkts': '3319',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '465',
                                                                       'out_octets': '856146',
                                                                       'out_ucast_pkts': '3319',
                                                                       'poller_hash': 'QURUTjIzMTQ0Qjc5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13470',
                                                                                                                        'fiber_length': 18588,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0Qjc5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x11-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEI3OQ==',
                                         'serial_number': 'ADTN23144B79',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13470'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-0.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '451',
                                                                       'in_octets': '242864315',
                                                                       'in_ucast_pkts': '448335',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x1-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '961946008',
                                                                       'out_ucast_pkts': '914137',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MS1PTlUxX0VUSF8x',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '961946008',
                                                                       'in_ucast_pkts': '914137',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '961946008',
                                                                                                'in_ucast_pkts': '914137',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '451',
                                                                                                'out_octets': '242864315',
                                                                                                'out_ucast_pkts': '448335',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8x'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '451',
                                                                       'out_octets': '242864315',
                                                                       'out_ucast_pkts': '448335',
                                                                       'poller_hash': 'QURUTjIzMTQ0RDQ3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13217',
                                                                                                                        'fiber_length': 11625,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8x',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RDQ3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x1-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEQ0Nw==',
                                         'serial_number': 'ADTN23144D47',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13217'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '499',
                                                                       'in_octets': '97705072',
                                                                       'in_ucast_pkts': '551175',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x12-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1170351240',
                                                                       'out_ucast_pkts': '974025',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTItT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1170351240',
                                                                       'in_ucast_pkts': '974025',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1170351240',
                                                                                                'in_ucast_pkts': '974025',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '499',
                                                                                                'out_octets': '97705072',
                                                                                                'out_ucast_pkts': '551175',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '499',
                                                                       'out_octets': '97705072',
                                                                       'out_ucast_pkts': '551175',
                                                                       'poller_hash': 'QURUTjIzMTI0RTBBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14137',
                                                                                                                        'fiber_length': 11276,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTBBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x12-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUwQQ==',
                                         'serial_number': 'ADTN23124E0A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14137'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '351',
                                                                       'in_octets': '135656681',
                                                                       'in_ucast_pkts': '145060',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '142610193',
                                                                       'out_ucast_pkts': '140424',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '142610193',
                                                                       'in_ucast_pkts': '140424',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '142610193',
                                                                                                'in_ucast_pkts': '140424',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '351',
                                                                                                'out_octets': '135656681',
                                                                                                'out_ucast_pkts': '145060',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '351',
                                                                       'out_octets': '135656681',
                                                                       'out_ucast_pkts': '145060',
                                                                       'poller_hash': 'QURUTjIzMTI0N0Q2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10224',
                                                                                                                        'fiber_length': 10077,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0N0Q2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDdENg==',
                                         'serial_number': 'ADTN231247D6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10224'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '342',
                                                                       'in_octets': '129347451',
                                                                       'in_ucast_pkts': '125597',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '138429097',
                                                                       'out_ucast_pkts': '127664',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '138429097',
                                                                       'in_ucast_pkts': '127664',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '138429097',
                                                                                                'in_ucast_pkts': '127664',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '342',
                                                                                                'out_octets': '129347451',
                                                                                                'out_ucast_pkts': '125597',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '342',
                                                                       'out_octets': '129347451',
                                                                       'out_ucast_pkts': '125597',
                                                                       'poller_hash': 'QURUTjIzMTI0QjZEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9593',
                                                                                                                        'fiber_length': 10108,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QjZEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEI2RA==',
                                         'serial_number': 'ADTN23124B6D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9593'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '391',
                                                                       'in_octets': '16872461',
                                                                       'in_ucast_pkts': '62795',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '208640346',
                                                                       'out_ucast_pkts': '186120',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VMl9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '208640346',
                                                                       'in_ucast_pkts': '186120',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '208640346',
                                                                                                'in_ucast_pkts': '186120',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '391',
                                                                                                'out_octets': '16872461',
                                                                                                'out_ucast_pkts': '62795',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '391',
                                                                       'out_octets': '16872461',
                                                                       'out_ucast_pkts': '62795',
                                                                       'poller_hash': 'QURUTjIzMTI0RENDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11334',
                                                                                                                        'fiber_length': 10168,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RENDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERDQw==',
                                         'serial_number': 'ADTN23124DCC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '11334'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-38.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '423',
                                                                       'in_octets': '439947871',
                                                                       'in_ucast_pkts': '2498522',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x39-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '9821310954',
                                                                       'out_ucast_pkts': '6965154',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzktT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '9821310954',
                                                                       'in_ucast_pkts': '6965154',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '9821310954',
                                                                                                'in_ucast_pkts': '6965154',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '423',
                                                                                                'out_octets': '439947871',
                                                                                                'out_ucast_pkts': '2498522',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '423',
                                                                       'out_octets': '439947871',
                                                                       'out_ucast_pkts': '2498522',
                                                                       'poller_hash': 'QURUTjIzMTI0QkI4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '12195',
                                                                                                                        'fiber_length': 21025,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QkI4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x39-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEJCOA==',
                                         'serial_number': 'ADTN23124BB8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '12195'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-40.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '483',
                                                                       'in_octets': '190529128',
                                                                       'in_ucast_pkts': '499404',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x41-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '844189715',
                                                                       'out_ucast_pkts': '704096',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDEtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '844189715',
                                                                       'in_ucast_pkts': '704096',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '844189715',
                                                                                                'in_ucast_pkts': '704096',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '483',
                                                                                                'out_octets': '190529128',
                                                                                                'out_ucast_pkts': '499404',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80MQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '483',
                                                                       'out_octets': '190529128',
                                                                       'out_ucast_pkts': '499404',
                                                                       'poller_hash': 'QURUTjIzMTI0MThGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14056',
                                                                                                                        'fiber_length': 22879,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80MQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MThGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x41-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDE4Rg==',
                                         'serial_number': 'ADTN2312418F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14056'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '372',
                                                                       'in_octets': '96698803',
                                                                       'in_ucast_pkts': '1106465',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '6589630494',
                                                                       'out_ucast_pkts': '4465065',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '6589630494',
                                                                       'in_ucast_pkts': '4465065',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '6589630494',
                                                                                                'in_ucast_pkts': '4465065',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '372',
                                                                                                'out_octets': '96698803',
                                                                                                'out_ucast_pkts': '1106465',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '372',
                                                                       'out_octets': '96698803',
                                                                       'out_ucast_pkts': '1106465',
                                                                       'poller_hash': 'QURUTjIzMTI0Nzg1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10734',
                                                                                                                        'fiber_length': 10205,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0Nzg1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDc4NQ==',
                                         'serial_number': 'ADTN23124785',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10734'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-43.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '12',
                                                                       'in_octets': '25330521',
                                                                       'in_ucast_pkts': '132223',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x44-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4',
                                                                       'out_octets': '1030177346',
                                                                       'out_ucast_pkts': '837689',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDQtT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4',
                                                                       'in_octets': '1030177346',
                                                                       'in_ucast_pkts': '837689',
                                                                       'links': [       {       'in_nucast_pkts': '4',
                                                                                                'in_octets': '1030177346',
                                                                                                'in_ucast_pkts': '837689',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '12',
                                                                                                'out_octets': '25330521',
                                                                                                'out_ucast_pkts': '132223',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80NA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '12',
                                                                       'out_octets': '25330521',
                                                                       'out_ucast_pkts': '132223',
                                                                       'poller_hash': 'QURUTjIzMTI0OUREIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '6761',
                                                                                                                        'fiber_length': 23888,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80NA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OUREIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x44-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDlERA==',
                                         'serial_number': 'ADTN231249DD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '6761'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-38.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '264',
                                                                       'in_octets': '32350400',
                                                                       'in_ucast_pkts': '53031',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x39-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '27088190',
                                                                       'out_ucast_pkts': '37677',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzktT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '27088190',
                                                                       'in_ucast_pkts': '37677',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '27088190',
                                                                                                'in_ucast_pkts': '37677',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '264',
                                                                                                'out_octets': '32350400',
                                                                                                'out_ucast_pkts': '53031',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '264',
                                                                       'out_octets': '32350400',
                                                                       'out_ucast_pkts': '53031',
                                                                       'poller_hash': 'QURUTjIzMTI0QzQ3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '7505',
                                                                                                                        'fiber_length': 20938,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QzQ3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x39-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEM0Nw==',
                                         'serial_number': 'ADTN23124C47',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '7505'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-43.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '239',
                                                                       'in_octets': '41551028',
                                                                       'in_ucast_pkts': '146435',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x44-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '647296471',
                                                                       'out_ucast_pkts': '473841',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDQtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '647296471',
                                                                       'in_ucast_pkts': '473841',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '647296471',
                                                                                                'in_ucast_pkts': '473841',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '239',
                                                                                                'out_octets': '41551028',
                                                                                                'out_ucast_pkts': '146435',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80NA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '239',
                                                                       'out_octets': '41551028',
                                                                       'out_ucast_pkts': '146435',
                                                                       'poller_hash': 'QURUTjIzMTI0QTk0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '6680',
                                                                                                                        'fiber_length': 23035,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80NA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QTk0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x44-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEE5NA==',
                                         'serial_number': 'ADTN23124A94',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '6680'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-39.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '483',
                                                                       'in_octets': '3842103',
                                                                       'in_ucast_pkts': '23681',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x40-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '63473472',
                                                                       'out_ucast_pkts': '54463',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDAtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '63473472',
                                                                       'in_ucast_pkts': '54463',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '63473472',
                                                                                                'in_ucast_pkts': '54463',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '483',
                                                                                                'out_octets': '3842103',
                                                                                                'out_ucast_pkts': '23681',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80MA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '483',
                                                                       'out_octets': '3842103',
                                                                       'out_ucast_pkts': '23681',
                                                                       'poller_hash': 'QURUTjIzMTI0QTE5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13676',
                                                                                                                        'fiber_length': 22184,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80MA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QTE5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x40-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEExOQ==',
                                         'serial_number': 'ADTN23124A19',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13676'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '361',
                                                                       'in_octets': '166981861',
                                                                       'in_ucast_pkts': '256927',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '618995496',
                                                                       'out_ucast_pkts': '498480',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VNF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '618995496',
                                                                       'in_ucast_pkts': '498480',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '618995496',
                                                                                                'in_ucast_pkts': '498480',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '361',
                                                                                                'out_octets': '166981861',
                                                                                                'out_ucast_pkts': '256927',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '361',
                                                                       'out_octets': '166981861',
                                                                       'out_ucast_pkts': '256927',
                                                                       'poller_hash': 'QURUTjIzMTI0MDQ4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10394',
                                                                                                                        'fiber_length': 10416,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MDQ4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDA0OA==',
                                         'serial_number': 'ADTN23124048',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10394'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '341',
                                                                       'in_octets': '88559950',
                                                                       'in_ucast_pkts': '533666',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '3581600402',
                                                                       'out_ucast_pkts': '2622981',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VNV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '3581600402',
                                                                       'in_ucast_pkts': '2622981',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '3581600402',
                                                                                                'in_ucast_pkts': '2622981',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '341',
                                                                                                'out_octets': '88559950',
                                                                                                'out_ucast_pkts': '533666',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '341',
                                                                       'out_octets': '88559950',
                                                                       'out_ucast_pkts': '533666',
                                                                       'poller_hash': 'QURUTjIzMTI1MDMwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10234',
                                                                                                                        'fiber_length': 10562,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MDMwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU5',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTAzMA==',
                                         'serial_number': 'ADTN23125030',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10234'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-43.1.2.0.eth.1.phy',
                                                                       'name': 'Maurice-x44-ONU2_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDQtT05VMl9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0QTBCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QTBCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x44-ONU2',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '43.1.2"(ADTN23124A0B) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/44.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-07-17T13:53:31Z',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEEwQg==',
                                         'serial_number': 'ADTN23124A0B',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-40.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '481',
                                                                       'in_octets': '596418325',
                                                                       'in_ucast_pkts': '1150844',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x41-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '2713399495',
                                                                       'out_ucast_pkts': '2079163',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDEtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '2713399495',
                                                                       'in_ucast_pkts': '2079163',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '2713399495',
                                                                                                'in_ucast_pkts': '2079163',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '481',
                                                                                                'out_octets': '596418325',
                                                                                                'out_ucast_pkts': '1150844',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80MQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '481',
                                                                       'out_octets': '596418325',
                                                                       'out_ucast_pkts': '1150844',
                                                                       'poller_hash': 'QURUTjIzMTI0Q0IzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13987',
                                                                                                                        'fiber_length': 22969,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80MQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0Q0IzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x41-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNENCMw==',
                                         'serial_number': 'ADTN23124CB3',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13987'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-33.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '354',
                                                                       'in_octets': '147713761',
                                                                       'in_ucast_pkts': '346252',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x34-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '242285559',
                                                                       'out_ucast_pkts': '356659',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzQtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '242285559',
                                                                       'in_ucast_pkts': '356659',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '242285559',
                                                                                                'in_ucast_pkts': '356659',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '354',
                                                                                                'out_octets': '147713761',
                                                                                                'out_ucast_pkts': '346252',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '354',
                                                                       'out_octets': '147713761',
                                                                       'out_ucast_pkts': '346252',
                                                                       'poller_hash': 'QURUTjIzMTI0MzEzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10401',
                                                                                                                        'fiber_length': 18524,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MzEzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x34-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDMxMw==',
                                         'serial_number': 'ADTN23124313',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10401'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-42.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '326',
                                                                       'in_octets': '130676550',
                                                                       'in_ucast_pkts': '131633',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x43-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '136433948',
                                                                       'out_ucast_pkts': '133965',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDMtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '136433948',
                                                                       'in_ucast_pkts': '133965',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '136433948',
                                                                                                'in_ucast_pkts': '133965',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '326',
                                                                                                'out_octets': '130676550',
                                                                                                'out_ucast_pkts': '131633',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80Mw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '326',
                                                                       'out_octets': '130676550',
                                                                       'out_ucast_pkts': '131633',
                                                                       'poller_hash': 'QURUTjIzMTI0NDFCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9596',
                                                                                                                        'fiber_length': 22218,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80Mw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NDFCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x43-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDQxQg==',
                                         'serial_number': 'ADTN2312441B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9596'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-4.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '470',
                                                                       'in_octets': '179910866',
                                                                       'in_ucast_pkts': '644858',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x5-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1958509744',
                                                                       'out_ucast_pkts': '1550636',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NS1PTlU3X0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1958509744',
                                                                       'in_ucast_pkts': '1550636',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1958509744',
                                                                                                'in_ucast_pkts': '1550636',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '470',
                                                                                                'out_octets': '179910866',
                                                                                                'out_ucast_pkts': '644858',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC81'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '470',
                                                                       'out_octets': '179910866',
                                                                       'out_ucast_pkts': '644858',
                                                                       'poller_hash': 'QURUTjIzMTIzRUQxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13987',
                                                                                                                        'fiber_length': 15444,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC81',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRUQxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x5-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0VEMQ==',
                                         'serial_number': 'ADTN23123ED1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13987'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '334',
                                                                       'in_octets': '58899795',
                                                                       'in_ucast_pkts': '204814',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1128714277',
                                                                       'out_ucast_pkts': '889018',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VNl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1128714277',
                                                                       'in_ucast_pkts': '889018',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1128714277',
                                                                                                'in_ucast_pkts': '889018',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '334',
                                                                                                'out_octets': '58899795',
                                                                                                'out_ucast_pkts': '204814',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '334',
                                                                       'out_octets': '58899795',
                                                                       'out_ucast_pkts': '204814',
                                                                       'poller_hash': 'QURUTjIzMTI0QTFEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9593',
                                                                                                                        'fiber_length': 10214,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QTFEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEExRA==',
                                         'serial_number': 'ADTN23124A1D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9593'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '481',
                                                                       'in_octets': '410242455',
                                                                       'in_ucast_pkts': '551868',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x13-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1042134894',
                                                                       'out_ucast_pkts': '924503',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTMtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1042134894',
                                                                       'in_ucast_pkts': '924503',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1042134894',
                                                                                                'in_ucast_pkts': '924503',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '481',
                                                                                                'out_octets': '410242455',
                                                                                                'out_ucast_pkts': '551868',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '481',
                                                                       'out_octets': '410242455',
                                                                       'out_ucast_pkts': '551868',
                                                                       'poller_hash': 'QURUTjIzMTI0QUNBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13674',
                                                                                                                        'fiber_length': 11798,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QUNBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x13-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEFDQQ==',
                                         'serial_number': 'ADTN23124ACA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13674'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '319',
                                                                       'in_octets': '78198887',
                                                                       'in_ucast_pkts': '303965',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1119873683',
                                                                       'out_ucast_pkts': '976914',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1119873683',
                                                                       'in_ucast_pkts': '976914',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1119873683',
                                                                                                'in_ucast_pkts': '976914',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '319',
                                                                                                'out_octets': '78198887',
                                                                                                'out_ucast_pkts': '303965',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '319',
                                                                       'out_octets': '78198887',
                                                                       'out_ucast_pkts': '303965',
                                                                       'poller_hash': 'QURUTjIzMTIzRThFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9144',
                                                                                                                        'fiber_length': 10440,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRThFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0U4RQ==',
                                         'serial_number': 'ADTN23123E8E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9144'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-16.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '384',
                                                                       'in_octets': '130081128',
                                                                       'in_ucast_pkts': '127560',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x17-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '137853785',
                                                                       'out_ucast_pkts': '131574',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '137853785',
                                                                       'in_ucast_pkts': '131574',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '137853785',
                                                                                                'in_ucast_pkts': '131574',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '384',
                                                                                                'out_octets': '130081128',
                                                                                                'out_ucast_pkts': '127560',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '384',
                                                                       'out_octets': '130081128',
                                                                       'out_ucast_pkts': '127560',
                                                                       'poller_hash': 'QURUTjIzMTI0Q0U1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11338',
                                                                                                                        'fiber_length': 11030,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0Q0U1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x17-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNENFNQ==',
                                         'serial_number': 'ADTN23124CE5',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '11338'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '413',
                                                                       'in_octets': '93745932',
                                                                       'in_ucast_pkts': '705884',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1',
                                                                       'out_octets': '2192693191',
                                                                       'out_ucast_pkts': '1588392',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1',
                                                                       'in_octets': '2192693191',
                                                                       'in_ucast_pkts': '1588392',
                                                                       'links': [       {       'in_nucast_pkts': '1',
                                                                                                'in_octets': '2192693191',
                                                                                                'in_ucast_pkts': '1588392',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '413',
                                                                                                'out_octets': '93745932',
                                                                                                'out_ucast_pkts': '705884',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '413',
                                                                       'out_octets': '93745932',
                                                                       'out_ucast_pkts': '705884',
                                                                       'poller_hash': 'QURUTjIzMTI0NzQ2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '12084',
                                                                                                                        'fiber_length': 10759,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NzQ2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU8',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDc0Ng==',
                                         'serial_number': 'ADTN23124746',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '12084'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-38.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '346',
                                                                       'in_octets': '280013737',
                                                                       'in_ucast_pkts': '511129',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x39-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4',
                                                                       'out_octets': '769522058',
                                                                       'out_ucast_pkts': '752311',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzktT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4',
                                                                       'in_octets': '769522058',
                                                                       'in_ucast_pkts': '752311',
                                                                       'links': [       {       'in_nucast_pkts': '4',
                                                                                                'in_octets': '769522058',
                                                                                                'in_ucast_pkts': '752311',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '346',
                                                                                                'out_octets': '280013737',
                                                                                                'out_ucast_pkts': '511129',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '346',
                                                                       'out_octets': '280013737',
                                                                       'out_ucast_pkts': '511129',
                                                                       'poller_hash': 'QURUTjIzMTI0OTFCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9235',
                                                                                                                        'fiber_length': 21128,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OTFCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x39-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDkxQg==',
                                         'serial_number': 'ADTN2312491B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9235'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-8.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '480',
                                                                       'in_octets': '443776936',
                                                                       'in_ucast_pkts': '5950972',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x9-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '14021882176',
                                                                       'out_ucast_pkts': '9510069',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14OS1PTlUxX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '14021882176',
                                                                       'in_ucast_pkts': '9510069',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '14021882176',
                                                                                                'in_ucast_pkts': '9510069',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '480',
                                                                                                'out_octets': '443776936',
                                                                                                'out_ucast_pkts': '5950972',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC85'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '480',
                                                                       'out_octets': '443776936',
                                                                       'out_ucast_pkts': '5950972',
                                                                       'poller_hash': 'QURUTjIzMTIzRjBEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13992',
                                                                                                                        'fiber_length': 16868,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC85',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTIzRjBEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x9-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyM0YwRA==',
                                         'serial_number': 'ADTN23123F0D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13992'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '446',
                                                                       'in_octets': '11778979',
                                                                       'in_ucast_pkts': '63827',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x14-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '238381558',
                                                                       'out_ucast_pkts': '195348',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTQtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '238381558',
                                                                       'in_ucast_pkts': '195348',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '238381558',
                                                                                                'in_ucast_pkts': '195348',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '446',
                                                                                                'out_octets': '11778979',
                                                                                                'out_ucast_pkts': '63827',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '446',
                                                                       'out_octets': '11778979',
                                                                       'out_ucast_pkts': '63827',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTY2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13214',
                                                                                                                        'fiber_length': 13407,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTY2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x14-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE2Ng==',
                                         'serial_number': 'ADTN23144A66',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13214'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-38.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '266',
                                                                       'in_octets': '3165788',
                                                                       'in_ucast_pkts': '13376',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x39-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3',
                                                                       'out_octets': '2704308',
                                                                       'out_ucast_pkts': '11696',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzktT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3',
                                                                       'in_octets': '2704308',
                                                                       'in_ucast_pkts': '11696',
                                                                       'links': [       {       'in_nucast_pkts': '3',
                                                                                                'in_octets': '2704308',
                                                                                                'in_ucast_pkts': '11696',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '266',
                                                                                                'out_octets': '3165788',
                                                                                                'out_ucast_pkts': '13376',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '266',
                                                                       'out_octets': '3165788',
                                                                       'out_ucast_pkts': '13376',
                                                                       'poller_hash': 'QURUTjI0MzYzQ0UyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '7374',
                                                                                                                        'fiber_length': 21189,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQ0UyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x39-ONU3',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0NFMg==',
                                         'serial_number': 'ADTN24363CE2',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '7374'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-16.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '372',
                                                                       'in_octets': '509766003',
                                                                       'in_ucast_pkts': '691910',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x17-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1712492583',
                                                                       'out_ucast_pkts': '1528806',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1712492583',
                                                                       'in_ucast_pkts': '1528806',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1712492583',
                                                                                                'in_ucast_pkts': '1528806',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '372',
                                                                                                'out_octets': '509766003',
                                                                                                'out_ucast_pkts': '691910',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '372',
                                                                       'out_octets': '509766003',
                                                                       'out_ucast_pkts': '691910',
                                                                       'poller_hash': 'QURUTjI0MzYzQzY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10739',
                                                                                                                        'fiber_length': 10813,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x17-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M2Nw==',
                                         'serial_number': 'ADTN24363C67',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10739'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-36.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '404',
                                                                       'in_octets': '114352657',
                                                                       'in_ucast_pkts': '280310',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x37-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1068000591',
                                                                       'out_ucast_pkts': '862659',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzctT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1068000591',
                                                                       'in_ucast_pkts': '862659',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1068000591',
                                                                                                'in_ucast_pkts': '862659',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '404',
                                                                                                'out_octets': '114352657',
                                                                                                'out_ucast_pkts': '280310',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '404',
                                                                       'out_octets': '114352657',
                                                                       'out_ucast_pkts': '280310',
                                                                       'poller_hash': 'QURUTjI0MzYzQzc3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11845',
                                                                                                                        'fiber_length': 20659,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzc3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x37-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M3Nw==',
                                         'serial_number': 'ADTN24363C77',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '11845'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-37.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '273',
                                                                       'in_octets': '70068336',
                                                                       'in_ucast_pkts': '226855',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x38-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1855721505',
                                                                       'out_ucast_pkts': '1277516',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzgtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1855721505',
                                                                       'in_ucast_pkts': '1277516',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1855721505',
                                                                                                'in_ucast_pkts': '1277516',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '273',
                                                                                                'out_octets': '70068336',
                                                                                                'out_ucast_pkts': '226855',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '273',
                                                                       'out_octets': '70068336',
                                                                       'out_ucast_pkts': '226855',
                                                                       'poller_hash': 'QURUTjIzMTI1MENFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '7755',
                                                                                                                        'fiber_length': 22608,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MENFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x38-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTBDRQ==',
                                         'serial_number': 'ADTN231250CE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '7755'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-39.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '453',
                                                                       'in_octets': '277993925',
                                                                       'in_ucast_pkts': '793672',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x40-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '3176166591',
                                                                       'out_ucast_pkts': '2364957',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDAtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '3176166591',
                                                                       'in_ucast_pkts': '2364957',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '3176166591',
                                                                                                'in_ucast_pkts': '2364957',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '453',
                                                                                                'out_octets': '277993925',
                                                                                                'out_ucast_pkts': '793672',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80MA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '453',
                                                                       'out_octets': '277993925',
                                                                       'out_ucast_pkts': '793672',
                                                                       'poller_hash': 'QURUTjI0MzYzRTM5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13476',
                                                                                                                        'fiber_length': 22643,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80MA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRTM5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x40-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0UzOQ==',
                                         'serial_number': 'ADTN24363E39',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13476'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-29.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '360',
                                                                       'in_octets': '3047973644',
                                                                       'in_ucast_pkts': '3207754',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x30-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '3865998534',
                                                                       'out_ucast_pkts': '3411241',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzAtT05VMF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '3865998534',
                                                                       'in_ucast_pkts': '3411241',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '3865998534',
                                                                                                'in_ucast_pkts': '3411241',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '360',
                                                                                                'out_octets': '3047973644',
                                                                                                'out_ucast_pkts': '3207754',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '360',
                                                                       'out_octets': '3047973644',
                                                                       'out_ucast_pkts': '3207754',
                                                                       'poller_hash': 'QURUTjIzMTI0RjNFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10735',
                                                                                                                        'fiber_length': 13835,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -11.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjNFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x30-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYzRQ==',
                                         'serial_number': 'ADTN23124F3E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10735'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '472',
                                                                       'in_octets': '326530641',
                                                                       'in_ucast_pkts': '310190',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '388114042',
                                                                       'out_ucast_pkts': '363110',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '388114042',
                                                                       'in_ucast_pkts': '363110',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '388114042',
                                                                                                'in_ucast_pkts': '363110',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '472',
                                                                                                'out_octets': '326530641',
                                                                                                'out_ucast_pkts': '310190',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '472',
                                                                       'out_octets': '326530641',
                                                                       'out_ucast_pkts': '310190',
                                                                       'poller_hash': 'QURUTjI0MzYzQzk0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13997',
                                                                                                                        'fiber_length': 15866,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzk0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M5NA==',
                                         'serial_number': 'ADTN24363C94',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13997'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '470',
                                                                       'in_octets': '1026782676',
                                                                       'in_ucast_pkts': '1405990',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x12-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1870686990',
                                                                       'out_ucast_pkts': '1866863',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTItT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1870686990',
                                                                       'in_ucast_pkts': '1866863',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1870686990',
                                                                                                'in_ucast_pkts': '1866863',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '470',
                                                                                                'out_octets': '1026782676',
                                                                                                'out_ucast_pkts': '1405990',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '470',
                                                                       'out_octets': '1026782676',
                                                                       'out_ucast_pkts': '1405990',
                                                                       'poller_hash': 'QURUTjI0MzYzRDA3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13996',
                                                                                                                        'fiber_length': 11960,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRDA3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x12-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0QwNw==',
                                         'serial_number': 'ADTN24363D07',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13996'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-12.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '464',
                                                                       'in_octets': '13566675',
                                                                       'in_ucast_pkts': '89916',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x13-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '97973196',
                                                                       'out_ucast_pkts': '101853',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTMtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '97973196',
                                                                       'in_ucast_pkts': '101853',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '97973196',
                                                                                                'in_ucast_pkts': '101853',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '464',
                                                                                                'out_octets': '13566675',
                                                                                                'out_ucast_pkts': '89916',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '464',
                                                                       'out_octets': '13566675',
                                                                       'out_ucast_pkts': '89916',
                                                                       'poller_hash': 'QURUTjI0MzYzQzRGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13474',
                                                                                                                        'fiber_length': 11903,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzRGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x13-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M0Rg==',
                                         'serial_number': 'ADTN24363C4F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13474'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-16.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '334',
                                                                       'in_octets': '24651498',
                                                                       'in_ucast_pkts': '146394',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x17-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1252514528',
                                                                       'out_ucast_pkts': '993141',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1252514528',
                                                                       'in_ucast_pkts': '993141',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1252514528',
                                                                                                'in_ucast_pkts': '993141',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '334',
                                                                                                'out_octets': '24651498',
                                                                                                'out_ucast_pkts': '146394',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '334',
                                                                       'out_octets': '24651498',
                                                                       'out_ucast_pkts': '146394',
                                                                       'poller_hash': 'QURUTjI0MzYzQ0ZGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9609',
                                                                                                                        'fiber_length': 11213,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQ0ZGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x17-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0NGRg==',
                                         'serial_number': 'ADTN24363CFF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9609'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '142525263',
                                                                       'in_ucast_pkts': '536733',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '3748978229',
                                                                       'out_ucast_pkts': '2730483',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '3748978229',
                                                                       'in_ucast_pkts': '2730483',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '3748978229',
                                                                                                'in_ucast_pkts': '2730483',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '476',
                                                                                                'out_octets': '142525263',
                                                                                                'out_ucast_pkts': '536733',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '142525263',
                                                                       'out_ucast_pkts': '536733',
                                                                       'poller_hash': 'QURUTjIzMTI0QjI1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13827',
                                                                                                                        'fiber_length': 16307,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QjI1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEIyNQ==',
                                         'serial_number': 'ADTN23124B25',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13827'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '466',
                                                                       'in_octets': '9302477',
                                                                       'in_ucast_pkts': '18880',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '12413789',
                                                                       'out_ucast_pkts': '22434',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '12413789',
                                                                       'in_ucast_pkts': '22434',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '12413789',
                                                                                                'in_ucast_pkts': '22434',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '466',
                                                                                                'out_octets': '9302477',
                                                                                                'out_ucast_pkts': '18880',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '466',
                                                                       'out_octets': '9302477',
                                                                       'out_ucast_pkts': '18880',
                                                                       'poller_hash': 'QURUTjIzMTI1MTM5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13826',
                                                                                                                        'fiber_length': 15592,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTM5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTEzOQ==',
                                         'serial_number': 'ADTN23125139',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13826'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-16.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '327',
                                                                       'in_octets': '543857883',
                                                                       'in_ucast_pkts': '1468691',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x17-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '6250712841',
                                                                       'out_ucast_pkts': '4357999',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '6250712841',
                                                                       'in_ucast_pkts': '4357999',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '6250712841',
                                                                                                'in_ucast_pkts': '4357999',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '327',
                                                                                                'out_octets': '543857883',
                                                                                                'out_ucast_pkts': '1468691',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '327',
                                                                       'out_octets': '543857883',
                                                                       'out_ucast_pkts': '1468691',
                                                                       'poller_hash': 'QURUTjIzMTI0N0IwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9149',
                                                                                                                        'fiber_length': 11290,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0N0IwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x17-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDdCMA==',
                                         'serial_number': 'ADTN231247B0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9149'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-16.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '409',
                                                                       'in_octets': '1074153',
                                                                       'in_ucast_pkts': '4234',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x17-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1032682',
                                                                       'out_ucast_pkts': '3572',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VNF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1032682',
                                                                       'in_ucast_pkts': '3572',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1032682',
                                                                                                'in_ucast_pkts': '3572',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '409',
                                                                                                'out_octets': '1074153',
                                                                                                'out_ucast_pkts': '4234',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '409',
                                                                       'out_octets': '1074153',
                                                                       'out_ucast_pkts': '4234',
                                                                       'poller_hash': 'QURUTjIzMTI0REFDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '12079',
                                                                                                                        'fiber_length': 11290,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REFDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x17-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERBQw==',
                                         'serial_number': 'ADTN23124DAC',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '12079'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-35.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '335',
                                                                       'in_octets': '131697222',
                                                                       'in_ucast_pkts': '667878',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x36-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1550766700',
                                                                       'out_ucast_pkts': '1241353',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzYtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1550766700',
                                                                       'in_ucast_pkts': '1241353',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1550766700',
                                                                                                'in_ucast_pkts': '1241353',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '335',
                                                                                                'out_octets': '131697222',
                                                                                                'out_ucast_pkts': '667878',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '335',
                                                                       'out_octets': '131697222',
                                                                       'out_ucast_pkts': '667878',
                                                                       'poller_hash': 'QURUTjIzMTI0RTA1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9599',
                                                                                                                        'fiber_length': 19392,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTA1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x36-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUwNQ==',
                                         'serial_number': 'ADTN23124E05',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9599'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-19.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '464',
                                                                       'in_octets': '170233309',
                                                                       'in_ucast_pkts': '209683',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x20-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '103661230',
                                                                       'out_ucast_pkts': '208638',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjAtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '103661230',
                                                                       'in_ucast_pkts': '208638',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '103661230',
                                                                                                'in_ucast_pkts': '208638',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '464',
                                                                                                'out_octets': '170233309',
                                                                                                'out_ucast_pkts': '209683',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '464',
                                                                       'out_octets': '170233309',
                                                                       'out_ucast_pkts': '209683',
                                                                       'poller_hash': 'QURUTjIzMTI0RTYzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13827',
                                                                                                                        'fiber_length': 14841,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTYzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x20-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEU2Mw==',
                                         'serial_number': 'ADTN23124E63',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13827'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '481',
                                                                       'in_octets': '331645176',
                                                                       'in_ucast_pkts': '717159',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x15-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1016931757',
                                                                       'out_ucast_pkts': '1123194',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTUtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1016931757',
                                                                       'in_ucast_pkts': '1123194',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1016931757',
                                                                                                'in_ucast_pkts': '1123194',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '481',
                                                                                                'out_octets': '331645176',
                                                                                                'out_ucast_pkts': '717159',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '481',
                                                                       'out_octets': '331645176',
                                                                       'out_ucast_pkts': '717159',
                                                                       'poller_hash': 'QURUTjIzMTI0QkZEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13674',
                                                                                                                        'fiber_length': 14227,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QkZEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x15-ONU2',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEJGRA==',
                                         'serial_number': 'ADTN23124BFD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13674'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '466',
                                                                       'in_octets': '168223928',
                                                                       'in_ucast_pkts': '324871',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x15-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '896806012',
                                                                       'out_ucast_pkts': '718117',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTUtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '896806012',
                                                                       'in_ucast_pkts': '718117',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '896806012',
                                                                                                'in_ucast_pkts': '718117',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '466',
                                                                                                'out_octets': '168223928',
                                                                                                'out_ucast_pkts': '324871',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '466',
                                                                       'out_octets': '168223928',
                                                                       'out_ucast_pkts': '324871',
                                                                       'poller_hash': 'QURUTjIzMTI0ODM4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13824',
                                                                                                                        'fiber_length': 14418,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0ODM4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x15-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDgzOA==',
                                         'serial_number': 'ADTN23124838',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13824'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-16.1.5.0.eth.1.phy',
                                                                       'name': 'Maurice-x17-ONU5_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VNV9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0QkVFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QkVFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x17-ONU5',
                                         'offline_reason': 'OFFLINE_REASON_POWER_OFF',
                                         'offline_reason_detail': 'A local power '
                                                                  'outage has occurred '
                                                                  'for ONU "onu '
                                                                  '16.1.5"(ADTN23124BEE).',
                                         'offline_time': '2025-08-16T10:50:02Z',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEJFRQ==',
                                         'serial_number': 'ADTN23124BEE',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-36.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '318',
                                                                       'in_octets': '1501185',
                                                                       'in_ucast_pkts': '3878',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x37-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '4',
                                                                       'out_octets': '618004',
                                                                       'out_ucast_pkts': '3352',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzctT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '4',
                                                                       'in_octets': '618004',
                                                                       'in_ucast_pkts': '3352',
                                                                       'links': [       {       'in_nucast_pkts': '4',
                                                                                                'in_octets': '618004',
                                                                                                'in_ucast_pkts': '3352',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '318',
                                                                                                'out_octets': '1501185',
                                                                                                'out_ucast_pkts': '3878',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '318',
                                                                       'out_octets': '1501185',
                                                                       'out_ucast_pkts': '3878',
                                                                       'poller_hash': 'QURUTjIzMTQ0QTk4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '8506',
                                                                                                                        'fiber_length': 20359,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0QTk4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x37-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEE5OA==',
                                         'serial_number': 'ADTN23144A98',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '8506'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-16.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '470',
                                                                       'in_octets': '226059110',
                                                                       'in_ucast_pkts': '401961',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x17-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1335899184',
                                                                       'out_ucast_pkts': '1128684',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VNl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1335899184',
                                                                       'in_ucast_pkts': '1128684',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1335899184',
                                                                                                'in_ucast_pkts': '1128684',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '470',
                                                                                                'out_octets': '226059110',
                                                                                                'out_ucast_pkts': '401961',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '470',
                                                                       'out_octets': '226059110',
                                                                       'out_ucast_pkts': '401961',
                                                                       'poller_hash': 'QURUTjIzMTI0Qjc1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13669',
                                                                                                                        'fiber_length': 10837,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0Qjc1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x17-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEI3NQ==',
                                         'serial_number': 'ADTN23124B75',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13669'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-28.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '336',
                                                                       'in_octets': '4003849175',
                                                                       'in_ucast_pkts': '3155807',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x29-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '385831299',
                                                                       'out_ucast_pkts': '1781974',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjktT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '385831299',
                                                                       'in_ucast_pkts': '1781974',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '385831299',
                                                                                                'in_ucast_pkts': '1781974',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '336',
                                                                                                'out_octets': '4003849175',
                                                                                                'out_ucast_pkts': '3155807',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '336',
                                                                       'out_octets': '4003849175',
                                                                       'out_ucast_pkts': '3155807',
                                                                       'poller_hash': 'QURUTjIzMTI0OEFEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9595',
                                                                                                                        'fiber_length': 13766,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OEFEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x29-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDhBRA==',
                                         'serial_number': 'ADTN231248AD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9595'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-30.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '354',
                                                                       'in_octets': '7545875',
                                                                       'in_ucast_pkts': '24250',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x31-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '23152246',
                                                                       'out_ucast_pkts': '31353',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzEtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '23152246',
                                                                       'in_ucast_pkts': '31353',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '23152246',
                                                                                                'in_ucast_pkts': '31353',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '354',
                                                                                                'out_octets': '7545875',
                                                                                                'out_ucast_pkts': '24250',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '354',
                                                                       'out_octets': '7545875',
                                                                       'out_ucast_pkts': '24250',
                                                                       'poller_hash': 'QURUTjIzMTI0QjdDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10396',
                                                                                                                        'fiber_length': 15963,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QjdDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x31-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEI3Qw==',
                                         'serial_number': 'ADTN23124B7C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10396'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-26.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '460',
                                                                       'in_octets': '352430119',
                                                                       'in_ucast_pkts': '444982',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x27-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '711308221',
                                                                       'out_ucast_pkts': '635235',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjctT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '711308221',
                                                                       'in_ucast_pkts': '635235',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '711308221',
                                                                                                'in_ucast_pkts': '635235',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '460',
                                                                                                'out_octets': '352430119',
                                                                                                'out_ucast_pkts': '444982',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '460',
                                                                       'out_octets': '352430119',
                                                                       'out_ucast_pkts': '444982',
                                                                       'poller_hash': 'QURUTjIzMTI0MDg1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13465',
                                                                                                                        'fiber_length': 14837,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MDg1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x27-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDA4NQ==',
                                         'serial_number': 'ADTN23124085',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13465'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-16.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '452',
                                                                       'in_octets': '541486255',
                                                                       'in_ucast_pkts': '1784498',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x17-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '3202701440',
                                                                       'out_ucast_pkts': '3004730',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '3202701440',
                                                                       'in_ucast_pkts': '3004730',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '3202701440',
                                                                                                'in_ucast_pkts': '3004730',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '452',
                                                                                                'out_octets': '541486255',
                                                                                                'out_ucast_pkts': '1784498',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '452',
                                                                       'out_octets': '541486255',
                                                                       'out_ucast_pkts': '1784498',
                                                                       'poller_hash': 'QURUTjIzMTI1MTQ0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13468',
                                                                                                                        'fiber_length': 10680,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI1MTQ0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x17-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzEyNTE0NA==',
                                         'serial_number': 'ADTN23125144',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13468'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '461',
                                                                       'in_octets': '5741096',
                                                                       'in_ucast_pkts': '21426',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '38726554',
                                                                       'out_ucast_pkts': '40615',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '38726554',
                                                                       'in_ucast_pkts': '40615',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '38726554',
                                                                                                'in_ucast_pkts': '40615',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '461',
                                                                                                'out_octets': '5741096',
                                                                                                'out_ucast_pkts': '21426',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '461',
                                                                       'out_octets': '5741096',
                                                                       'out_ucast_pkts': '21426',
                                                                       'poller_hash': 'QURUTjIzMTI0MTE1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13677',
                                                                                                                        'fiber_length': 16513,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MTE1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDExNQ==',
                                         'serial_number': 'ADTN23124115',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13677'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '486',
                                                                       'in_octets': '448932237',
                                                                       'in_ucast_pkts': '454935',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x14-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1090082492',
                                                                       'out_ucast_pkts': '961874',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTQtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1090082492',
                                                                       'in_ucast_pkts': '961874',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1090082492',
                                                                                                'in_ucast_pkts': '961874',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '486',
                                                                                                'out_octets': '448932237',
                                                                                                'out_ucast_pkts': '454935',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '486',
                                                                       'out_octets': '448932237',
                                                                       'out_ucast_pkts': '454935',
                                                                       'poller_hash': 'QURUTjIzMTI0ODZEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14143',
                                                                                                                        'fiber_length': 12827,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0ODZEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x14-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDg2RA==',
                                         'serial_number': 'ADTN2312486D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14143'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '460',
                                                                       'in_octets': '39321449',
                                                                       'in_ucast_pkts': '53652',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1',
                                                                       'out_octets': '42406679',
                                                                       'out_ucast_pkts': '51054',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VNF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1',
                                                                       'in_octets': '42406679',
                                                                       'in_ucast_pkts': '51054',
                                                                       'links': [       {       'in_nucast_pkts': '1',
                                                                                                'in_octets': '42406679',
                                                                                                'in_ucast_pkts': '51054',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '460',
                                                                                                'out_octets': '39321449',
                                                                                                'out_ucast_pkts': '53652',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '460',
                                                                       'out_octets': '39321449',
                                                                       'out_ucast_pkts': '53652',
                                                                       'poller_hash': 'QURUTjIzMTI0NjZCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13467',
                                                                                                                        'fiber_length': 16138,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0NjZCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDY2Qg==',
                                         'serial_number': 'ADTN2312466B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13467'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '481',
                                                                       'in_octets': '965316008',
                                                                       'in_ucast_pkts': '1586410',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x14-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '6978071567',
                                                                       'out_ucast_pkts': '5410288',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTQtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '6978071567',
                                                                       'in_ucast_pkts': '5410288',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '6978071567',
                                                                                                'in_ucast_pkts': '5410288',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '481',
                                                                                                'out_octets': '965316008',
                                                                                                'out_ucast_pkts': '1586410',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '481',
                                                                       'out_octets': '965316008',
                                                                       'out_ucast_pkts': '1586410',
                                                                       'poller_hash': 'QURUTjIzMTI0OThBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13985',
                                                                                                                        'fiber_length': 13210,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OThBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x14-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDk4QQ==',
                                         'serial_number': 'ADTN2312498A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13985'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-34.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '348',
                                                                       'in_octets': '518603969',
                                                                       'in_ucast_pkts': '823041',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x35-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1922800238',
                                                                       'out_ucast_pkts': '1867021',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzUtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1922800238',
                                                                       'in_ucast_pkts': '1867021',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1922800238',
                                                                                                'in_ucast_pkts': '1867021',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '348',
                                                                                                'out_octets': '518603969',
                                                                                                'out_ucast_pkts': '823041',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '348',
                                                                       'out_octets': '518603969',
                                                                       'out_ucast_pkts': '823041',
                                                                       'poller_hash': 'QURUTjIzMTI0ODAyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10225',
                                                                                                                        'fiber_length': 19286,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0ODAyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x35-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDgwMg==',
                                         'serial_number': 'ADTN23124802',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10225'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '473',
                                                                       'in_octets': '160826096',
                                                                       'in_ucast_pkts': '1877214',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x14-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '6590958232',
                                                                       'out_ucast_pkts': '4558090',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTQtT05VNF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '6590958232',
                                                                       'in_ucast_pkts': '4558090',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '6590958232',
                                                                                                'in_ucast_pkts': '4558090',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '473',
                                                                                                'out_octets': '160826096',
                                                                                                'out_ucast_pkts': '1877214',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '473',
                                                                       'out_octets': '160826096',
                                                                       'out_ucast_pkts': '1877214',
                                                                       'poller_hash': 'QURUTjIzMTI0OEQ3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13824',
                                                                                                                        'fiber_length': 13796,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OEQ3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x14-ONU4',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDhENw==',
                                         'serial_number': 'ADTN231248D7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13824'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '466',
                                                                       'in_octets': '17555200',
                                                                       'in_ucast_pkts': '121207',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '276319426',
                                                                       'out_ucast_pkts': '215064',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VNl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '276319426',
                                                                       'in_ucast_pkts': '215064',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '276319426',
                                                                                                'in_ucast_pkts': '215064',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '466',
                                                                                                'out_octets': '17555200',
                                                                                                'out_ucast_pkts': '121207',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '466',
                                                                       'out_octets': '17555200',
                                                                       'out_ucast_pkts': '121207',
                                                                       'poller_hash': 'QURUTjIzMTI0ODkxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13206',
                                                                                                                        'fiber_length': 16559,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0ODkxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU6',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDg5MQ==',
                                         'serial_number': 'ADTN23124891',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13206'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-17.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '483',
                                                                       'in_octets': '1026813',
                                                                       'in_ucast_pkts': '4111',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x18-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '376306',
                                                                       'out_ucast_pkts': '3108',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTgtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '376306',
                                                                       'in_ucast_pkts': '3108',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '376306',
                                                                                                'in_ucast_pkts': '3108',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '483',
                                                                                                'out_octets': '1026813',
                                                                                                'out_ucast_pkts': '4111',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xOA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '483',
                                                                       'out_octets': '1026813',
                                                                       'out_ucast_pkts': '4111',
                                                                       'poller_hash': 'QURUTjIzMTI0RjExIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13991',
                                                                                                                        'fiber_length': 13282,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xOA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RjExIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x18-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEYxMQ==',
                                         'serial_number': 'ADTN23124F11',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13991'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-17.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '486',
                                                                       'in_octets': '891543894',
                                                                       'in_ucast_pkts': '778638',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x18-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '417433523',
                                                                       'out_ucast_pkts': '452066',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTgtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '417433523',
                                                                       'in_ucast_pkts': '452066',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '417433523',
                                                                                                'in_ucast_pkts': '452066',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '486',
                                                                                                'out_octets': '891543894',
                                                                                                'out_ucast_pkts': '778638',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xOA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '486',
                                                                       'out_octets': '891543894',
                                                                       'out_ucast_pkts': '778638',
                                                                       'poller_hash': 'QURUTjIzMTI0MUE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13821',
                                                                                                                        'fiber_length': 13833,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xOA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MUE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x18-ONU1',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDFBNg==',
                                         'serial_number': 'ADTN231241A6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13821'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-38.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '261',
                                                                       'in_octets': '80334767',
                                                                       'in_ucast_pkts': '525650',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x39-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1',
                                                                       'out_octets': '2663386215',
                                                                       'out_ucast_pkts': '1861618',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzktT05VNF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1',
                                                                       'in_octets': '2663386215',
                                                                       'in_ucast_pkts': '1861618',
                                                                       'links': [       {       'in_nucast_pkts': '1',
                                                                                                'in_octets': '2663386215',
                                                                                                'in_ucast_pkts': '1861618',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '261',
                                                                                                'out_octets': '80334767',
                                                                                                'out_ucast_pkts': '525650',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '261',
                                                                       'out_octets': '80334767',
                                                                       'out_ucast_pkts': '525650',
                                                                       'poller_hash': 'QURUTjI0MzYzQ0UwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '7305',
                                                                                                                        'fiber_length': 21144,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQ0UwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x39-ONU4',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0NFMA==',
                                         'serial_number': 'ADTN24363CE0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '7305'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '475',
                                                                       'in_octets': '143127976',
                                                                       'in_ucast_pkts': '223240',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x12-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '452710226',
                                                                       'out_ucast_pkts': '375162',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTItT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '452710226',
                                                                       'in_ucast_pkts': '375162',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '452710226',
                                                                                                'in_ucast_pkts': '375162',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '475',
                                                                                                'out_octets': '143127976',
                                                                                                'out_ucast_pkts': '223240',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '475',
                                                                       'out_octets': '143127976',
                                                                       'out_ucast_pkts': '223240',
                                                                       'poller_hash': 'QURUTjI0MzYzQUVBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13827',
                                                                                                                        'fiber_length': 12184,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQUVBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x12-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0FFQQ==',
                                         'serial_number': 'ADTN24363AEA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13827'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-11.1.3.0.eth.1.phy',
                                                                       'name': 'Maurice-x12-ONU3_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTItT05VM19FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjI0MzYzRDdEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRDdEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x12-ONU3',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '11.1.3"(ADTN24363D7D) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/12.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-10-05T13:18:07Z',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0Q3RA==',
                                         'serial_number': 'ADTN24363D7D',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-37.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '371',
                                                                       'in_octets': '186838357',
                                                                       'in_ucast_pkts': '1955695',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x38-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '9387461240',
                                                                       'out_ucast_pkts': '6596709',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzgtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '9387461240',
                                                                       'in_ucast_pkts': '6596709',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '9387461240',
                                                                                                'in_ucast_pkts': '6596709',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '371',
                                                                                                'out_octets': '186838357',
                                                                                                'out_ucast_pkts': '1955695',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '371',
                                                                       'out_octets': '186838357',
                                                                       'out_ucast_pkts': '1955695',
                                                                       'poller_hash': 'QURUTjI0MzYzQ0REIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10926',
                                                                                                                        'fiber_length': 22540,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQ0REIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x38-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0NERA==',
                                         'serial_number': 'ADTN24363CDD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10926'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '402',
                                                                       'in_octets': '195815673',
                                                                       'in_ucast_pkts': '1253370',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1',
                                                                       'out_octets': '2858079183',
                                                                       'out_ucast_pkts': '2091469',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VOV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1',
                                                                       'in_octets': '2858079183',
                                                                       'in_ucast_pkts': '2091469',
                                                                       'links': [       {       'in_nucast_pkts': '1',
                                                                                                'in_octets': '2858079183',
                                                                                                'in_ucast_pkts': '2091469',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '402',
                                                                                                'out_octets': '195815673',
                                                                                                'out_ucast_pkts': '1253370',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '402',
                                                                       'out_octets': '195815673',
                                                                       'out_ucast_pkts': '1253370',
                                                                       'poller_hash': 'QURUTjI0MzYzQzczIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11754',
                                                                                                                        'fiber_length': 10470,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzczIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU9',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M3Mw==',
                                         'serial_number': 'ADTN24363C73',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '11754'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-16.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '456',
                                                                       'in_octets': '45022957',
                                                                       'in_ucast_pkts': '274008',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x17-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '122106102',
                                                                       'out_ucast_pkts': '316102',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '122106102',
                                                                       'in_ucast_pkts': '316102',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '122106102',
                                                                                                'in_ucast_pkts': '316102',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '456',
                                                                                                'out_octets': '45022957',
                                                                                                'out_ucast_pkts': '274008',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '456',
                                                                       'out_octets': '45022957',
                                                                       'out_ucast_pkts': '274008',
                                                                       'poller_hash': 'QURUTjI0MzYzQjExIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13218',
                                                                                                                        'fiber_length': 10874,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQjExIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x17-ONU8',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0IxMQ==',
                                         'serial_number': 'ADTN24363B11',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13218'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-36.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '292',
                                                                       'in_octets': '319697907',
                                                                       'in_ucast_pkts': '280722',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x37-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '336229569',
                                                                       'out_ucast_pkts': '313730',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzctT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '336229569',
                                                                       'in_ucast_pkts': '313730',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '336229569',
                                                                                                'in_ucast_pkts': '313730',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '292',
                                                                                                'out_octets': '319697907',
                                                                                                'out_ucast_pkts': '280722',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '292',
                                                                       'out_octets': '319697907',
                                                                       'out_ucast_pkts': '280722',
                                                                       'poller_hash': 'QURUTjI0MzYzRDNCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '8336',
                                                                                                                        'fiber_length': 21195,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRDNCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x37-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0QzQg==',
                                         'serial_number': 'ADTN24363D3B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '8336'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-30.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '353',
                                                                       'in_octets': '187970867',
                                                                       'in_ucast_pkts': '482956',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x31-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1039638494',
                                                                       'out_ucast_pkts': '979657',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzEtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1039638494',
                                                                       'in_ucast_pkts': '979657',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1039638494',
                                                                                                'in_ucast_pkts': '979657',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '353',
                                                                                                'out_octets': '187970867',
                                                                                                'out_ucast_pkts': '482956',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '353',
                                                                       'out_octets': '187970867',
                                                                       'out_ucast_pkts': '482956',
                                                                       'poller_hash': 'QURUTjI0MzYzQkNFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10236',
                                                                                                                        'fiber_length': 15822,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQkNFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x31-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0JDRQ==',
                                         'serial_number': 'ADTN24363BCE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10236'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '463',
                                                                       'in_octets': '32646246',
                                                                       'in_ucast_pkts': '164103',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x12-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '442333405',
                                                                       'out_ucast_pkts': '363467',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTItT05VNF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '442333405',
                                                                       'in_ucast_pkts': '363467',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '442333405',
                                                                                                'in_ucast_pkts': '363467',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '463',
                                                                                                'out_octets': '32646246',
                                                                                                'out_ucast_pkts': '164103',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '463',
                                                                       'out_octets': '32646246',
                                                                       'out_ucast_pkts': '164103',
                                                                       'poller_hash': 'QURUTjI0MzYzQzc4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13476',
                                                                                                                        'fiber_length': 11077,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzc4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x12-ONU4',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M3OA==',
                                         'serial_number': 'ADTN24363C78',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13476'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-13.1.6.0.eth.1.phy',
                                                                       'name': 'Maurice-x14-ONU6_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTQtT05VNl9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0OEUyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OEUyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x14-ONU6',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDhFMg==',
                                         'serial_number': 'ADTN231248E2',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '471',
                                                                       'in_octets': '1774270060',
                                                                       'in_ucast_pkts': '2666821',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x14-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '7485507904',
                                                                       'out_ucast_pkts': '5484464',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTQtT05VNV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '7485507904',
                                                                       'in_ucast_pkts': '5484464',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '7485507904',
                                                                                                'in_ucast_pkts': '5484464',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '471',
                                                                                                'out_octets': '1774270060',
                                                                                                'out_ucast_pkts': '2666821',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '471',
                                                                       'out_octets': '1774270060',
                                                                       'out_ucast_pkts': '2666821',
                                                                       'poller_hash': 'QURUTjI0MzYzQ0U2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13684',
                                                                                                                        'fiber_length': 13958,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQ0U2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x14-ONU5',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0NFNg==',
                                         'serial_number': 'ADTN24363CE6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13684'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-38.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '256',
                                                                       'in_octets': '24310565',
                                                                       'in_ucast_pkts': '60306',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x39-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '52819271',
                                                                       'out_ucast_pkts': '67413',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzktT05VNV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '52819271',
                                                                       'in_ucast_pkts': '67413',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '52819271',
                                                                                                'in_ucast_pkts': '67413',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '256',
                                                                                                'out_octets': '24310565',
                                                                                                'out_ucast_pkts': '60306',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '256',
                                                                       'out_octets': '24310565',
                                                                       'out_ucast_pkts': '60306',
                                                                       'poller_hash': 'QURUTjI0MzYzQzVDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '7235',
                                                                                                                        'fiber_length': 21170,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzVDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x39-ONU5',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M1Qw==',
                                         'serial_number': 'ADTN24363C5C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '7235'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '453',
                                                                       'in_octets': '33980506',
                                                                       'in_ucast_pkts': '378028',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x12-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '2024115893',
                                                                       'out_ucast_pkts': '1360137',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTItT05VNV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '2024115893',
                                                                       'in_ucast_pkts': '1360137',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '2024115893',
                                                                                                'in_ucast_pkts': '1360137',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '453',
                                                                                                'out_octets': '33980506',
                                                                                                'out_ucast_pkts': '378028',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '453',
                                                                       'out_octets': '33980506',
                                                                       'out_ucast_pkts': '378028',
                                                                       'poller_hash': 'QURUTjI0MzYzQkM4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13217',
                                                                                                                        'fiber_length': 11983,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQkM4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x12-ONU5',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0JDOA==',
                                         'serial_number': 'ADTN24363BC8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13217'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '477',
                                                                       'in_octets': '74726912',
                                                                       'in_ucast_pkts': '246695',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x12-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '628628742',
                                                                       'out_ucast_pkts': '546779',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTItT05VNl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '628628742',
                                                                       'in_ucast_pkts': '546779',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '628628742',
                                                                                                'in_ucast_pkts': '546779',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '477',
                                                                                                'out_octets': '74726912',
                                                                                                'out_ucast_pkts': '246695',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '477',
                                                                       'out_octets': '74726912',
                                                                       'out_ucast_pkts': '246695',
                                                                       'poller_hash': 'QURUTjI0MzYzREIxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14148',
                                                                                                                        'fiber_length': 11015,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzREIxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x12-ONU6',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0RCMQ==',
                                         'serial_number': 'ADTN24363DB1',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14148'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-42.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '316',
                                                                       'in_octets': '355790001',
                                                                       'in_ucast_pkts': '448505',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x43-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '905402177',
                                                                       'out_ucast_pkts': '768021',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDMtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '905402177',
                                                                       'in_ucast_pkts': '768021',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '905402177',
                                                                                                'in_ucast_pkts': '768021',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '316',
                                                                                                'out_octets': '355790001',
                                                                                                'out_ucast_pkts': '448505',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80Mw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '316',
                                                                       'out_octets': '355790001',
                                                                       'out_ucast_pkts': '448505',
                                                                       'poller_hash': 'QURUTjI0MzYzRTVDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9166',
                                                                                                                        'fiber_length': 21948,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80Mw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRTVDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x43-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0U1Qw==',
                                         'serial_number': 'ADTN24363E5C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9166'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '477',
                                                                       'in_octets': '74649393',
                                                                       'in_ucast_pkts': '124008',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '52665276',
                                                                       'out_ucast_pkts': '112193',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '52665276',
                                                                       'in_ucast_pkts': '112193',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '52665276',
                                                                                                'in_ucast_pkts': '112193',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '477',
                                                                                                'out_octets': '74649393',
                                                                                                'out_ucast_pkts': '124008',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '477',
                                                                       'out_octets': '74649393',
                                                                       'out_ucast_pkts': '124008',
                                                                       'poller_hash': 'QURUTjI0MzYzQzZGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14146',
                                                                                                                        'fiber_length': 15994,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzZGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU7',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M2Rg==',
                                         'serial_number': 'ADTN24363C6F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14146'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '455',
                                                                       'in_octets': '10875783',
                                                                       'in_ucast_pkts': '43047',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x14-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '24822738',
                                                                       'out_ucast_pkts': '42142',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTQtT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '24822738',
                                                                       'in_ucast_pkts': '42142',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '24822738',
                                                                                                'in_ucast_pkts': '42142',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '455',
                                                                                                'out_octets': '10875783',
                                                                                                'out_ucast_pkts': '43047',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '455',
                                                                       'out_octets': '10875783',
                                                                       'out_ucast_pkts': '43047',
                                                                       'poller_hash': 'QURUTjI0MzYzQUJEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13223',
                                                                                                                        'fiber_length': 12895,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQUJEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x14-ONU7',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0FCRA==',
                                         'serial_number': 'ADTN24363ABD',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13223'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-39.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '453',
                                                                       'in_octets': '34072316',
                                                                       'in_ucast_pkts': '151624',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x40-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '612346539',
                                                                       'out_ucast_pkts': '476736',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDAtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '612346539',
                                                                       'in_ucast_pkts': '476736',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '612346539',
                                                                                                'in_ucast_pkts': '476736',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '453',
                                                                                                'out_octets': '34072316',
                                                                                                'out_ucast_pkts': '151624',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80MA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '453',
                                                                       'out_octets': '34072316',
                                                                       'out_ucast_pkts': '151624',
                                                                       'poller_hash': 'QURUTjI0MzYzQjA3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13215',
                                                                                                                        'fiber_length': 22536,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80MA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQjA3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x40-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0IwNw==',
                                         'serial_number': 'ADTN24363B07',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13215'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-16.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '498',
                                                                       'in_octets': '235958907',
                                                                       'in_ucast_pkts': '2297650',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x17-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '12723574439',
                                                                       'out_ucast_pkts': '8782231',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VOV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '12723574439',
                                                                       'in_ucast_pkts': '8782231',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '12723574439',
                                                                                                'in_ucast_pkts': '8782231',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '498',
                                                                                                'out_octets': '235958907',
                                                                                                'out_ucast_pkts': '2297650',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '498',
                                                                       'out_octets': '235958907',
                                                                       'out_ucast_pkts': '2297650',
                                                                       'poller_hash': 'QURUTjI0MzYzQjVBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14148',
                                                                                                                        'fiber_length': 11228,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQjVBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x17-ONU9',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0I1QQ==',
                                         'serial_number': 'ADTN24363B5A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14148'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-5.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '143639',
                                                                       'in_octets': '289424390272',
                                                                       'in_ucast_pkts': '953720358',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x6-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '3299',
                                                                       'out_octets': '3271451684392',
                                                                       'out_ucast_pkts': '2357486524',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14Ni1PTlUyX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '3299',
                                                                       'in_octets': '3271451684392',
                                                                       'in_ucast_pkts': '2357486524',
                                                                       'links': [       {       'in_nucast_pkts': '3299',
                                                                                                'in_octets': '3271451684392',
                                                                                                'in_ucast_pkts': '2357486524',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '143639',
                                                                                                'out_octets': '289424390272',
                                                                                                'out_ucast_pkts': '953720358',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC82'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '143639',
                                                                       'out_octets': '289424390272',
                                                                       'out_ucast_pkts': '953720358',
                                                                       'poller_hash': 'QURUTjI0MzYzQkY5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '4308364',
                                                                                                                        'fiber_length': 14403,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC82',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQkY5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x6-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0JGOQ==',
                                         'serial_number': 'ADTN24363BF9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_5_2-28-b',
                                         'uptime': '4308364'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '472',
                                                                       'in_octets': '34363364',
                                                                       'in_ucast_pkts': '197604',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '253902422',
                                                                       'out_ucast_pkts': '302249',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '253902422',
                                                                       'in_ucast_pkts': '302249',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '253902422',
                                                                                                'in_ucast_pkts': '302249',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '472',
                                                                                                'out_octets': '34363364',
                                                                                                'out_ucast_pkts': '197604',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '472',
                                                                       'out_octets': '34363364',
                                                                       'out_ucast_pkts': '197604',
                                                                       'poller_hash': 'QURUTjI0MzYzRDlBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13996',
                                                                                                                        'fiber_length': 16064,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRDlBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU8',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0Q5QQ==',
                                         'serial_number': 'ADTN24363D9A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13996'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.9.0.eth.1.phy',
                                                                       'in_nucast_pkts': '482',
                                                                       'in_octets': '1698528',
                                                                       'in_ucast_pkts': '5694',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU9_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '2263753',
                                                                       'out_ucast_pkts': '5032',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VOV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '2263753',
                                                                       'in_ucast_pkts': '5032',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '2263753',
                                                                                                'in_ucast_pkts': '5032',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '482',
                                                                                                'out_octets': '1698528',
                                                                                                'out_ucast_pkts': '5694',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '482',
                                                                       'out_octets': '1698528',
                                                                       'out_ucast_pkts': '5694',
                                                                       'poller_hash': 'QURUTjI0MzYzQzEwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13826',
                                                                                                                        'fiber_length': 16032,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzEwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU9',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0MxMA==',
                                         'serial_number': 'ADTN24363C10',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13826'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-43.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '234',
                                                                       'in_octets': '7584355',
                                                                       'in_ucast_pkts': '48306',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x44-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '106024528',
                                                                       'out_ucast_pkts': '80583',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDQtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '106024528',
                                                                       'in_ucast_pkts': '80583',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '106024528',
                                                                                                'in_ucast_pkts': '80583',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '234',
                                                                                                'out_octets': '7584355',
                                                                                                'out_ucast_pkts': '48306',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80NA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '234',
                                                                       'out_octets': '7584355',
                                                                       'out_ucast_pkts': '48306',
                                                                       'poller_hash': 'QURUTjI0MzYzQzY1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '6600',
                                                                                                                        'fiber_length': 24073,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80NA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzY1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x44-ONU3',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M2NQ==',
                                         'serial_number': 'ADTN24363C65',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '6600'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-30.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '335',
                                                                       'in_octets': '111889214',
                                                                       'in_ucast_pkts': '288282',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x31-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '798255098',
                                                                       'out_ucast_pkts': '679095',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzEtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '798255098',
                                                                       'in_ucast_pkts': '679095',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '798255098',
                                                                                                'in_ucast_pkts': '679095',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '335',
                                                                                                'out_octets': '111889214',
                                                                                                'out_ucast_pkts': '288282',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '335',
                                                                       'out_octets': '111889214',
                                                                       'out_ucast_pkts': '288282',
                                                                       'poller_hash': 'QURUTjI0MzYzQ0FFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9616',
                                                                                                                        'fiber_length': 15002,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQ0FFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x31-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0NBRQ==',
                                         'serial_number': 'ADTN24363CAE',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9616'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-43.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '232',
                                                                       'in_octets': '1711866',
                                                                       'in_ucast_pkts': '7762',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x44-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '4735143',
                                                                       'out_ucast_pkts': '7073',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDQtT05VNF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '4735143',
                                                                       'in_ucast_pkts': '7073',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '4735143',
                                                                                                'in_ucast_pkts': '7073',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '232',
                                                                                                'out_octets': '1711866',
                                                                                                'out_ucast_pkts': '7762',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80NA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '232',
                                                                       'out_octets': '1711866',
                                                                       'out_ucast_pkts': '7762',
                                                                       'poller_hash': 'QURUTjI0MzYzQkJGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '6530',
                                                                                                                        'fiber_length': 22898,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80NA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -25.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQkJGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x44-ONU4',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0JCRg==',
                                         'serial_number': 'ADTN24363BBF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '6530'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-42.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '344',
                                                                       'in_octets': '102863677',
                                                                       'in_ucast_pkts': '287941',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x43-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '774207983',
                                                                       'out_ucast_pkts': '648073',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDMtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '774207983',
                                                                       'in_ucast_pkts': '648073',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '774207983',
                                                                                                'in_ucast_pkts': '648073',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '344',
                                                                                                'out_octets': '102863677',
                                                                                                'out_ucast_pkts': '287941',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80Mw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '344',
                                                                       'out_octets': '102863677',
                                                                       'out_ucast_pkts': '287941',
                                                                       'poller_hash': 'QURUTjIzMTI0MjYxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10226',
                                                                                                                        'fiber_length': 21910,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80Mw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0MjYxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x43-ONU0',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDI2MQ==',
                                         'serial_number': 'ADTN23124261',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10226'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '460',
                                                                       'in_octets': '55807624',
                                                                       'in_ucast_pkts': '182498',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '2662806574',
                                                                       'out_ucast_pkts': '1801793',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VMTBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '2662806574',
                                                                       'in_ucast_pkts': '1801793',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '2662806574',
                                                                                                'in_ucast_pkts': '1801793',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '460',
                                                                                                'out_octets': '55807624',
                                                                                                'out_ucast_pkts': '182498',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '460',
                                                                       'out_octets': '55807624',
                                                                       'out_ucast_pkts': '182498',
                                                                       'poller_hash': 'QURUTjI0MzYzRTNEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13687',
                                                                                                                        'fiber_length': 15731,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRTNEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU10',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0UzRA==',
                                         'serial_number': 'ADTN24363E3D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13687'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '482',
                                                                       'in_octets': '147895232',
                                                                       'in_ucast_pkts': '1084132',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x12-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '11084945193',
                                                                       'out_ucast_pkts': '8436789',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTItT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '11084945193',
                                                                       'in_ucast_pkts': '8436789',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '11084945193',
                                                                                                'in_ucast_pkts': '8436789',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '482',
                                                                                                'out_octets': '147895232',
                                                                                                'out_ucast_pkts': '1084132',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '482',
                                                                       'out_octets': '147895232',
                                                                       'out_ucast_pkts': '1084132',
                                                                       'poller_hash': 'QURUTjI0MzYzQjc4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13997',
                                                                                                                        'fiber_length': 10680,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQjc4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x12-ONU7',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0I3OA==',
                                         'serial_number': 'ADTN24363B78',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13997'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-42.1.4.0.eth.1.phy',
                                                                       'name': 'Maurice-x43-ONU4_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDMtT05VNF9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0REY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0REY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x43-ONU4',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNERGNw==',
                                         'serial_number': 'ADTN23124DF7',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-42.1.5.0.eth.1.phy',
                                                                       'name': 'Maurice-x43-ONU5_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDMtT05VNV9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0RTE2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTE2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x43-ONU5',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUxNg==',
                                         'serial_number': 'ADTN23124E16',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-42.1.6.0.eth.1.phy',
                                                                       'name': 'Maurice-x43-ONU6_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDMtT05VNl9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0RTBEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTBEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x43-ONU6',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUwRA==',
                                         'serial_number': 'ADTN23124E0D',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-42.1.7.0.eth.1.phy',
                                                                       'name': 'Maurice-x43-ONU7_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDMtT05VN19FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0RTI2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0RTI2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x43-ONU7',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEUyNg==',
                                         'serial_number': 'ADTN23124E26',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-9.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '50590282',
                                                                       'in_ucast_pkts': '185582',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x10-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '147152550',
                                                                       'out_ucast_pkts': '202288',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTAtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '147152550',
                                                                       'in_ucast_pkts': '202288',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '147152550',
                                                                                                'in_ucast_pkts': '202288',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '476',
                                                                                                'out_octets': '50590282',
                                                                                                'out_ucast_pkts': '185582',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '50590282',
                                                                       'out_ucast_pkts': '185582',
                                                                       'poller_hash': 'QURUTjI0MzYzRERBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13827',
                                                                                                                        'fiber_length': 17649,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRERBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x10-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0REQQ==',
                                         'serial_number': 'ADTN24363DDA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13827'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-36.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '214',
                                                                       'in_octets': '1189257',
                                                                       'in_ucast_pkts': '5539',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x37-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '4245637',
                                                                       'out_ucast_pkts': '6073',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzctT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '4245637',
                                                                       'in_ucast_pkts': '6073',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '4245637',
                                                                                                'in_ucast_pkts': '6073',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '214',
                                                                                                'out_octets': '1189257',
                                                                                                'out_ucast_pkts': '5539',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '214',
                                                                       'out_octets': '1189257',
                                                                       'out_ucast_pkts': '5539',
                                                                       'poller_hash': 'QURUTjI0MzYzRTBFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '5786',
                                                                                                                        'fiber_length': 21401,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRTBFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x37-ONU3',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0UwRQ==',
                                         'serial_number': 'ADTN24363E0E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '5786'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-11.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '475',
                                                                       'in_octets': '180046930',
                                                                       'in_ucast_pkts': '395488',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x12-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '1547014878',
                                                                       'out_ucast_pkts': '1222738',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTItT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '1547014878',
                                                                       'in_ucast_pkts': '1222738',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '1547014878',
                                                                                                'in_ucast_pkts': '1222738',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '475',
                                                                                                'out_octets': '180046930',
                                                                                                'out_ucast_pkts': '395488',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '475',
                                                                       'out_octets': '180046930',
                                                                       'out_ucast_pkts': '395488',
                                                                       'poller_hash': 'QURUTjI0MzYzQkY0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13826',
                                                                                                                        'fiber_length': 10619,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQkY0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x12-ONU8',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0JGNA==',
                                         'serial_number': 'ADTN24363BF4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13826'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-38.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '252',
                                                                       'in_octets': '132752847',
                                                                       'in_ucast_pkts': '281880',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x39-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '678981586',
                                                                       'out_ucast_pkts': '569982',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzktT05VNl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '678981586',
                                                                       'in_ucast_pkts': '569982',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '678981586',
                                                                                                'in_ucast_pkts': '569982',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '252',
                                                                                                'out_octets': '132752847',
                                                                                                'out_ucast_pkts': '281880',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '252',
                                                                       'out_octets': '132752847',
                                                                       'out_ucast_pkts': '281880',
                                                                       'poller_hash': 'QURUTjI0MzYzRTY1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '7115',
                                                                                                                        'fiber_length': 21105,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRTY1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x39-ONU6',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0U2NQ==',
                                         'serial_number': 'ADTN24363E65',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '7115'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '452',
                                                                       'in_octets': '6100443',
                                                                       'in_ucast_pkts': '18941',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '12040094',
                                                                       'out_ucast_pkts': '18524',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VMTFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '12040094',
                                                                       'in_ucast_pkts': '18524',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '12040094',
                                                                                                'in_ucast_pkts': '18524',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '452',
                                                                                                'out_octets': '6100443',
                                                                                                'out_ucast_pkts': '18941',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '452',
                                                                       'out_octets': '6100443',
                                                                       'out_ucast_pkts': '18941',
                                                                       'poller_hash': 'QURUTjI0Mzg5OUQ5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13476',
                                                                                                                        'fiber_length': 16307,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0Mzg5OUQ5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU11',
                                         'poller_hash': 'b250IEFEVE4yNDM4OTlEOQ==',
                                         'serial_number': 'ADTN243899D9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13476'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-41.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '476',
                                                                       'in_octets': '151518526',
                                                                       'in_ucast_pkts': '308639',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x42-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '643545462',
                                                                       'out_ucast_pkts': '493210',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDItT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '643545462',
                                                                       'in_ucast_pkts': '493210',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '643545462',
                                                                                                'in_ucast_pkts': '493210',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '476',
                                                                                                'out_octets': '151518526',
                                                                                                'out_ucast_pkts': '308639',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80Mg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '476',
                                                                       'out_octets': '151518526',
                                                                       'out_ucast_pkts': '308639',
                                                                       'poller_hash': 'QURUTjI0Mzg5Nzk0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13831',
                                                                                                                        'fiber_length': 23714,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80Mg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0Mzg5Nzk0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x42-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDM4OTc5NA==',
                                         'serial_number': 'ADTN24389794',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13831'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-35.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '466',
                                                                       'in_octets': '4475804',
                                                                       'in_ucast_pkts': '17296',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x36-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '29654500',
                                                                       'out_ucast_pkts': '31303',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzYtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '29654500',
                                                                       'in_ucast_pkts': '31303',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '29654500',
                                                                                                'in_ucast_pkts': '31303',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '466',
                                                                                                'out_octets': '4475804',
                                                                                                'out_ucast_pkts': '17296',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '466',
                                                                       'out_octets': '4475804',
                                                                       'out_ucast_pkts': '17296',
                                                                       'poller_hash': 'QURUTjI0Mzg5QUQ3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13830',
                                                                                                                        'fiber_length': 19484,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0Mzg5QUQ3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x36-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDM4OUFENw==',
                                         'serial_number': 'ADTN24389AD7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13830'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-10.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '465',
                                                                       'in_octets': '718482852',
                                                                       'in_ucast_pkts': '3061983',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x11-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '9215264617',
                                                                       'out_ucast_pkts': '6426261',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTEtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '9215264617',
                                                                       'in_ucast_pkts': '6426261',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '9215264617',
                                                                                                'in_ucast_pkts': '6426261',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '465',
                                                                                                'out_octets': '718482852',
                                                                                                'out_ucast_pkts': '3061983',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '465',
                                                                       'out_octets': '718482852',
                                                                       'out_ucast_pkts': '3061983',
                                                                       'poller_hash': 'QURUTjI0MzYzQzlFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13221',
                                                                                                                        'fiber_length': 18740,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzlFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x11-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M5RQ==',
                                         'serial_number': 'ADTN24363C9E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13221'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-13.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '485',
                                                                       'in_octets': '190135208',
                                                                       'in_ucast_pkts': '1406309',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x14-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '3792845478',
                                                                       'out_ucast_pkts': '2666815',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTQtT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '3792845478',
                                                                       'in_ucast_pkts': '2666815',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '3792845478',
                                                                                                'in_ucast_pkts': '2666815',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '485',
                                                                                                'out_octets': '190135208',
                                                                                                'out_ucast_pkts': '1406309',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '485',
                                                                       'out_octets': '190135208',
                                                                       'out_ucast_pkts': '1406309',
                                                                       'poller_hash': 'QURUTjI0MzYzQzVEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14143',
                                                                                                                        'fiber_length': 13024,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQzVEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x14-ONU8',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0M1RA==',
                                         'serial_number': 'ADTN24363C5D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14143'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-30.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '324',
                                                                       'in_octets': '372800126',
                                                                       'in_ucast_pkts': '1249441',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x31-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1',
                                                                       'out_octets': '2802365785',
                                                                       'out_ucast_pkts': '2465921',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzEtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1',
                                                                       'in_octets': '2802365785',
                                                                       'in_ucast_pkts': '2465921',
                                                                       'links': [       {       'in_nucast_pkts': '1',
                                                                                                'in_octets': '2802365785',
                                                                                                'in_ucast_pkts': '2465921',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '324',
                                                                                                'out_octets': '372800126',
                                                                                                'out_ucast_pkts': '1249441',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '324',
                                                                       'out_octets': '372800126',
                                                                       'out_ucast_pkts': '1249441',
                                                                       'poller_hash': 'QURUTjI0MzE0NTczIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9145',
                                                                                                                        'fiber_length': 15712,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0NTczIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x31-ONU3',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDU3Mw==',
                                         'serial_number': 'ADTN24314573',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9145'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-37.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '368',
                                                                       'in_octets': '402636359',
                                                                       'in_ucast_pkts': '1914817',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x38-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '3756709932',
                                                                       'out_ucast_pkts': '3144430',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzgtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '3756709932',
                                                                       'in_ucast_pkts': '3144430',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '3756709932',
                                                                                                'in_ucast_pkts': '3144430',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '368',
                                                                                                'out_octets': '402636359',
                                                                                                'out_ucast_pkts': '1914817',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '368',
                                                                       'out_octets': '402636359',
                                                                       'out_ucast_pkts': '1914817',
                                                                       'poller_hash': 'QURUTjI0Mzg5ODVBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10955',
                                                                                                                        'fiber_length': 22280,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0Mzg5ODVBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x38-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDM4OTg1QQ==',
                                         'serial_number': 'ADTN2438985A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10955'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-17.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '459',
                                                                       'in_octets': '7256001',
                                                                       'in_ucast_pkts': '52116',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x18-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '107197188',
                                                                       'out_ucast_pkts': '84441',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTgtT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '107197188',
                                                                       'in_ucast_pkts': '84441',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '107197188',
                                                                                                'in_ucast_pkts': '84441',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '459',
                                                                                                'out_octets': '7256001',
                                                                                                'out_ucast_pkts': '52116',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xOA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '459',
                                                                       'out_octets': '7256001',
                                                                       'out_ucast_pkts': '52116',
                                                                       'poller_hash': 'QURUTjI0MzE0MjNFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13671',
                                                                                                                        'fiber_length': 13806,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xOA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MjNFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x18-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDIzRQ==',
                                         'serial_number': 'ADTN2431423E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13671'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-38.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '247',
                                                                       'in_octets': '108395500',
                                                                       'in_ucast_pkts': '122156',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x39-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '458423462',
                                                                       'out_ucast_pkts': '353037',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzktT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '458423462',
                                                                       'in_ucast_pkts': '353037',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '458423462',
                                                                                                'in_ucast_pkts': '353037',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '247',
                                                                                                'out_octets': '108395500',
                                                                                                'out_ucast_pkts': '122156',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '247',
                                                                       'out_octets': '108395500',
                                                                       'out_ucast_pkts': '122156',
                                                                       'poller_hash': 'QURUTjI0MzE0MjQyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '7045',
                                                                                                                        'fiber_length': 20899,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MjQyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x39-ONU7',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDI0Mg==',
                                         'serial_number': 'ADTN24314242',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '7045'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-0.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '477',
                                                                       'in_octets': '386990823',
                                                                       'in_ucast_pkts': '759975',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x1-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1656358492',
                                                                       'out_ucast_pkts': '1417813',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MS1PTlUyX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1656358492',
                                                                       'in_ucast_pkts': '1417813',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1656358492',
                                                                                                'in_ucast_pkts': '1417813',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '477',
                                                                                                'out_octets': '386990823',
                                                                                                'out_ucast_pkts': '759975',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8x'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '477',
                                                                       'out_octets': '386990823',
                                                                       'out_ucast_pkts': '759975',
                                                                       'poller_hash': 'QURUTjI0MzE0NzcxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14147',
                                                                                                                        'fiber_length': 11724,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8x',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0NzcxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x1-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDc3MQ==',
                                         'serial_number': 'ADTN24314771',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14147'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-36.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '283',
                                                                       'in_octets': '298891633',
                                                                       'in_ucast_pkts': '298118',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x37-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '38244010',
                                                                       'out_ucast_pkts': '59371',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzctT05VNF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '38244010',
                                                                       'in_ucast_pkts': '59371',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '38244010',
                                                                                                'in_ucast_pkts': '59371',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '283',
                                                                                                'out_octets': '298891633',
                                                                                                'out_ucast_pkts': '298118',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '283',
                                                                       'out_octets': '298891633',
                                                                       'out_ucast_pkts': '298118',
                                                                       'poller_hash': 'QURUTjI0MzE0MTU4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '8055',
                                                                                                                        'fiber_length': 20357,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.1}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MTU4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x37-ONU4',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDE1OA==',
                                         'serial_number': 'ADTN24314158',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '8055'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-41.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '462',
                                                                       'in_octets': '25881222',
                                                                       'in_ucast_pkts': '104574',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x42-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '401822315',
                                                                       'out_ucast_pkts': '303416',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDItT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '401822315',
                                                                       'in_ucast_pkts': '303416',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '401822315',
                                                                                                'in_ucast_pkts': '303416',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '462',
                                                                                                'out_octets': '25881222',
                                                                                                'out_ucast_pkts': '104574',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80Mg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '462',
                                                                       'out_octets': '25881222',
                                                                       'out_ucast_pkts': '104574',
                                                                       'poller_hash': 'QURUTjI0MzE0NTAwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13681',
                                                                                                                        'fiber_length': 23269,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80Mg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0NTAwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x42-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDUwMA==',
                                         'serial_number': 'ADTN24314500',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13681'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '455',
                                                                       'in_octets': '76744010',
                                                                       'in_ucast_pkts': '414343',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '472202097',
                                                                       'out_ucast_pkts': '575679',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VMTJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '472202097',
                                                                       'in_ucast_pkts': '575679',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '472202097',
                                                                                                'in_ucast_pkts': '575679',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '455',
                                                                                                'out_octets': '76744010',
                                                                                                'out_ucast_pkts': '414343',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '455',
                                                                       'out_octets': '76744010',
                                                                       'out_ucast_pkts': '414343',
                                                                       'poller_hash': 'QURUTjI0MzE0MUI5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13217',
                                                                                                                        'fiber_length': 15966,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MUI5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU12',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDFCOQ==',
                                         'serial_number': 'ADTN243141B9',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13217'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-31.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '410',
                                                                       'in_octets': '113671917',
                                                                       'in_ucast_pkts': '260446',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x32-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '481885682',
                                                                       'out_ucast_pkts': '387448',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzItT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '481885682',
                                                                       'in_ucast_pkts': '387448',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '481885682',
                                                                                                'in_ucast_pkts': '387448',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '410',
                                                                                                'out_octets': '113671917',
                                                                                                'out_ucast_pkts': '260446',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '410',
                                                                       'out_octets': '113671917',
                                                                       'out_ucast_pkts': '260446',
                                                                       'poller_hash': 'QURUTjI0MzE0MjhDIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '12075',
                                                                                                                        'fiber_length': 16428,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MjhDIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x32-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDI4Qw==',
                                         'serial_number': 'ADTN2431428C',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '12075'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-37.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '265',
                                                                       'in_octets': '32432320',
                                                                       'in_ucast_pkts': '109097',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x38-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1',
                                                                       'out_octets': '100418286',
                                                                       'out_ucast_pkts': '132715',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzgtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1',
                                                                       'in_octets': '100418286',
                                                                       'in_ucast_pkts': '132715',
                                                                       'links': [       {       'in_nucast_pkts': '1',
                                                                                                'in_octets': '100418286',
                                                                                                'in_ucast_pkts': '132715',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '265',
                                                                                                'out_octets': '32432320',
                                                                                                'out_ucast_pkts': '109097',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '265',
                                                                       'out_octets': '32432320',
                                                                       'out_ucast_pkts': '109097',
                                                                       'poller_hash': 'QURUTjI0MzE0MUY3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '7615',
                                                                                                                        'fiber_length': 23438,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -24.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MUY3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x38-ONU3',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDFGNw==',
                                         'serial_number': 'ADTN243141F7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '7615'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-37.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '455',
                                                                       'in_octets': '3392834535',
                                                                       'in_ucast_pkts': '4119641',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x38-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '3908400695',
                                                                       'out_ucast_pkts': '4099334',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzgtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '3908400695',
                                                                       'in_ucast_pkts': '4099334',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '3908400695',
                                                                                                'in_ucast_pkts': '4099334',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '455',
                                                                                                'out_octets': '3392834535',
                                                                                                'out_ucast_pkts': '4119641',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '455',
                                                                       'out_octets': '3392834535',
                                                                       'out_ucast_pkts': '4119641',
                                                                       'poller_hash': 'QURUTjI0MzE0MjkzIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13045',
                                                                                                                        'fiber_length': 23309,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MjkzIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x38-ONU4',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDI5Mw==',
                                         'serial_number': 'ADTN24314293',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13045'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-36.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '285',
                                                                       'in_octets': '4258696',
                                                                       'in_ucast_pkts': '16053',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x37-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '14346416',
                                                                       'out_ucast_pkts': '19999',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzctT05VNV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '14346416',
                                                                       'in_ucast_pkts': '19999',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '14346416',
                                                                                                'in_ucast_pkts': '19999',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '285',
                                                                                                'out_octets': '4258696',
                                                                                                'out_ucast_pkts': '16053',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '285',
                                                                       'out_octets': '4258696',
                                                                       'out_ucast_pkts': '16053',
                                                                       'poller_hash': 'QURUTjI0MzE0MUM3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '7885',
                                                                                                                        'fiber_length': 20931,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.4}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MUM3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x37-ONU5',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDFDNw==',
                                         'serial_number': 'ADTN243141C7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '7885'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-43.1.5.0.eth.1.phy',
                                                                       'in_nucast_pkts': '222',
                                                                       'in_octets': '83135370',
                                                                       'in_ucast_pkts': '278430',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x44-ONU5_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1620080306',
                                                                       'out_ucast_pkts': '1179322',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDQtT05VNV9FVEhfMQ==',
                                                                       'speed': '100000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1620080306',
                                                                       'in_ucast_pkts': '1179322',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1620080306',
                                                                                                'in_ucast_pkts': '1179322',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '222',
                                                                                                'out_octets': '83135370',
                                                                                                'out_ucast_pkts': '278430',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80NA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '222',
                                                                       'out_octets': '83135370',
                                                                       'out_ucast_pkts': '278430',
                                                                       'poller_hash': 'QURUTjI0MzE0NEY0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '6369',
                                                                                                                        'fiber_length': 22541,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80NA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -23.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0NEY0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x44-ONU5',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDRGNA==',
                                         'serial_number': 'ADTN243144F4',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '6369'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-27.1.0.0.eth.1.phy',
                                                                       'name': 'Maurice-x28-ONU0_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjgtT05VMF9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjI0MzE0MjM2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MjM2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x28-ONU0',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '27.1.0"(ADTN24314236) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/28.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-08-12T09:13:02Z',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDIzNg==',
                                         'serial_number': 'ADTN24314236',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-41.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '373',
                                                                       'in_octets': '134813114',
                                                                       'in_ucast_pkts': '436485',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x42-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1024438683',
                                                                       'out_ucast_pkts': '886825',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDItT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1024438683',
                                                                       'in_ucast_pkts': '886825',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1024438683',
                                                                                                'in_ucast_pkts': '886825',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '373',
                                                                                                'out_octets': '134813114',
                                                                                                'out_ucast_pkts': '436485',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80Mg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '373',
                                                                       'out_octets': '134813114',
                                                                       'out_ucast_pkts': '436485',
                                                                       'poller_hash': 'QURUTjI0MzE0MjYwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10741',
                                                                                                                        'fiber_length': 23407,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80Mg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MjYwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x42-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDI2MA==',
                                         'serial_number': 'ADTN24314260',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10741'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-44.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '384',
                                                                       'in_octets': '1719981',
                                                                       'in_ucast_pkts': '6253',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x45-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '2845501',
                                                                       'out_ucast_pkts': '5928',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDUtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '2845501',
                                                                       'in_ucast_pkts': '5928',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '2845501',
                                                                                                'in_ucast_pkts': '5928',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '384',
                                                                                                'out_octets': '1719981',
                                                                                                'out_ucast_pkts': '6253',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80NQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '384',
                                                                       'out_octets': '1719981',
                                                                       'out_ucast_pkts': '6253',
                                                                       'poller_hash': 'QURUTjI0MzE0NTNGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11336',
                                                                                                                        'fiber_length': 17838,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80NQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0NTNGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x45-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDUzRg==',
                                         'serial_number': 'ADTN2431453F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '11336'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-16.1.10.0.eth.1.phy',
                                                                       'in_nucast_pkts': '363',
                                                                       'in_octets': '144139860',
                                                                       'in_ucast_pkts': '1057494',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x17-ONU10_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '4670288854',
                                                                       'out_ucast_pkts': '3250941',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VMTBfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '4670288854',
                                                                       'in_ucast_pkts': '3250941',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '4670288854',
                                                                                                'in_ucast_pkts': '3250941',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '363',
                                                                                                'out_octets': '144139860',
                                                                                                'out_ucast_pkts': '1057494',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '363',
                                                                       'out_octets': '144139860',
                                                                       'out_ucast_pkts': '1057494',
                                                                       'poller_hash': 'QURUTjI0Mzg5NzI5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10408',
                                                                                                                        'fiber_length': 11321,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.3,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0Mzg5NzI5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x17-ONU10',
                                         'poller_hash': 'b250IEFEVE4yNDM4OTcyOQ==',
                                         'serial_number': 'ADTN24389729',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10408'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-31.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '398',
                                                                       'in_octets': '16801186',
                                                                       'in_ucast_pkts': '60418',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x32-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '41618853',
                                                                       'out_ucast_pkts': '58834',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzItT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '41618853',
                                                                       'in_ucast_pkts': '58834',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '41618853',
                                                                                                'in_ucast_pkts': '58834',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '398',
                                                                                                'out_octets': '16801186',
                                                                                                'out_ucast_pkts': '60418',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '398',
                                                                       'out_octets': '16801186',
                                                                       'out_ucast_pkts': '60418',
                                                                       'poller_hash': 'QURUTjI0MzYzRDZFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11746',
                                                                                                                        'fiber_length': 16238,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRDZFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x32-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0Q2RQ==',
                                         'serial_number': 'ADTN24363D6E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '11746'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-31.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '384',
                                                                       'in_octets': '7575294',
                                                                       'in_ucast_pkts': '19653',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x32-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '13780860',
                                                                       'out_ucast_pkts': '23372',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzItT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '13780860',
                                                                       'in_ucast_pkts': '23372',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '13780860',
                                                                                                'in_ucast_pkts': '23372',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '384',
                                                                                                'out_octets': '7575294',
                                                                                                'out_ucast_pkts': '19653',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '384',
                                                                       'out_octets': '7575294',
                                                                       'out_ucast_pkts': '19653',
                                                                       'poller_hash': 'QURUTjI0MzE0MUUwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11335',
                                                                                                                        'fiber_length': 16214,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MUUwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x32-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDFFMA==',
                                         'serial_number': 'ADTN243141E0',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '11335'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-17.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '454',
                                                                       'in_octets': '16054503',
                                                                       'in_ucast_pkts': '76659',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x18-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '114169609',
                                                                       'out_ucast_pkts': '98571',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTgtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '114169609',
                                                                       'in_ucast_pkts': '98571',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '114169609',
                                                                                                'in_ucast_pkts': '98571',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '454',
                                                                                                'out_octets': '16054503',
                                                                                                'out_ucast_pkts': '76659',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xOA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '454',
                                                                       'out_octets': '16054503',
                                                                       'out_ucast_pkts': '76659',
                                                                       'poller_hash': 'QURUTjI0MzE0MzBFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13471',
                                                                                                                        'fiber_length': 12282,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xOA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -11.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MzBFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x18-ONU3',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDMwRQ==',
                                         'serial_number': 'ADTN2431430E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13471'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-25.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '471',
                                                                       'in_octets': '19007153',
                                                                       'in_ucast_pkts': '108015',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x26-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '186652850',
                                                                       'out_ucast_pkts': '147346',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjYtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '186652850',
                                                                       'in_ucast_pkts': '147346',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '186652850',
                                                                                                'in_ucast_pkts': '147346',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '471',
                                                                                                'out_octets': '19007153',
                                                                                                'out_ucast_pkts': '108015',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '471',
                                                                       'out_octets': '19007153',
                                                                       'out_ucast_pkts': '108015',
                                                                       'poller_hash': 'QURUTjI0MzE0MUI4IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13673',
                                                                                                                        'fiber_length': 16257,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MUI4IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x26-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDFCOA==',
                                         'serial_number': 'ADTN243141B8',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13673'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-15.1.10.0.eth.1.phy',
                                                                       'name': 'Maurice-x16-ONU10_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VMTBfRVRIXzE='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjI0Mzg5Nzk2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0Mzg5Nzk2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x16-ONU10',
                                         'offline_reason': 'OFFLINE_REASON_OTHER',
                                         'offline_reason_detail': 'ONT is in '
                                                                  '"discovering" '
                                                                  'state: it was '
                                                                  'detected by the OLT '
                                                                  'but is not yet '
                                                                  'fully registered or '
                                                                  'activated.',
                                         'poller_hash': 'b250IEFEVE4yNDM4OTc5Ng==',
                                         'serial_number': 'ADTN24389796',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '472',
                                                                       'in_octets': '1020052778',
                                                                       'in_ucast_pkts': '1498303',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x15-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '3821608507',
                                                                       'out_ucast_pkts': '3181992',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTUtT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '3821608507',
                                                                       'in_ucast_pkts': '3181992',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '3821608507',
                                                                                                'in_ucast_pkts': '3181992',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '472',
                                                                                                'out_octets': '1020052778',
                                                                                                'out_ucast_pkts': '1498303',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '472',
                                                                       'out_octets': '1020052778',
                                                                       'out_ucast_pkts': '1498303',
                                                                       'poller_hash': 'QURUTjI0Mzg5NzIyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13994',
                                                                                                                        'fiber_length': 14621,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.0}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0Mzg5NzIyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x15-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDM4OTcyMg==',
                                         'serial_number': 'ADTN24389722',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13994'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-28.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '319',
                                                                       'in_octets': '39294729',
                                                                       'in_ucast_pkts': '75878',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x29-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '53313507',
                                                                       'out_ucast_pkts': '85994',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjktT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '53313507',
                                                                       'in_ucast_pkts': '85994',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '53313507',
                                                                                                'in_ucast_pkts': '85994',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '319',
                                                                                                'out_octets': '39294729',
                                                                                                'out_ucast_pkts': '75878',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '319',
                                                                       'out_octets': '39294729',
                                                                       'out_ucast_pkts': '75878',
                                                                       'poller_hash': 'QURUTjI0Mzg5NzYwIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '9145',
                                                                                                                        'fiber_length': 13569,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0Mzg5NzYwIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x29-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDM4OTc2MA==',
                                         'serial_number': 'ADTN24389760',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '9145'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '454',
                                                                       'in_octets': '44051786',
                                                                       'in_ucast_pkts': '202316',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x15-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '488754397',
                                                                       'out_ucast_pkts': '389744',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTUtT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '488754397',
                                                                       'in_ucast_pkts': '389744',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '488754397',
                                                                                                'in_ucast_pkts': '389744',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '454',
                                                                                                'out_octets': '44051786',
                                                                                                'out_ucast_pkts': '202316',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '454',
                                                                       'out_octets': '44051786',
                                                                       'out_ucast_pkts': '202316',
                                                                       'poller_hash': 'QURUTjI0Mzg5NzU0IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13474',
                                                                                                                        'fiber_length': 14296,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0Mzg5NzU0IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x15-ONU3',
                                         'poller_hash': 'b250IEFEVE4yNDM4OTc1NA==',
                                         'serial_number': 'ADTN24389754',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13474'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-9.1.1.0.eth.1.phy',
                                                                       'name': 'Maurice-x10-ONU1_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTAtT05VMV9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjI0MzE0MjYyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MjYyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x10-ONU1',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '9.1.1"(ADTN24314262) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/10.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-09-02T17:11:59Z',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDI2Mg==',
                                         'serial_number': 'ADTN24314262',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-21.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '414',
                                                                       'in_octets': '672959934',
                                                                       'in_ucast_pkts': '1315820',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x22-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '4342032313',
                                                                       'out_ucast_pkts': '3372938',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjItT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '4342032313',
                                                                       'in_ucast_pkts': '3372938',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '4342032313',
                                                                                                'in_ucast_pkts': '3372938',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '414',
                                                                                                'out_octets': '672959934',
                                                                                                'out_ucast_pkts': '1315820',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '414',
                                                                       'out_octets': '672959934',
                                                                       'out_ucast_pkts': '1315820',
                                                                       'poller_hash': 'QURUTjI0MzE0MUVGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11897',
                                                                                                                        'fiber_length': 14374,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -16.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MUVGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x22-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDFFRg==',
                                         'serial_number': 'ADTN243141EF',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '11897'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-28.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '2640',
                                                                       'in_octets': '12409528845',
                                                                       'in_ucast_pkts': '16209157',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x29-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '1',
                                                                       'out_octets': '66591314532',
                                                                       'out_ucast_pkts': '46703384',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjktT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '1',
                                                                       'in_octets': '66591314532',
                                                                       'in_ucast_pkts': '46703384',
                                                                       'links': [       {       'in_nucast_pkts': '1',
                                                                                                'in_octets': '66591314532',
                                                                                                'in_ucast_pkts': '46703384',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '2640',
                                                                                                'out_octets': '12409528845',
                                                                                                'out_ucast_pkts': '16209157',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2640',
                                                                       'out_octets': '12409528845',
                                                                       'out_ucast_pkts': '16209157',
                                                                       'poller_hash': 'QURUTjI0MzE0MjIyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '78917',
                                                                                                                        'fiber_length': 13857,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MjIyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x29-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDIyMg==',
                                         'serial_number': 'ADTN24314222',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '78917'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-21.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '444',
                                                                       'in_octets': '1254326334',
                                                                       'in_ucast_pkts': '995741',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x22-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1131627148',
                                                                       'out_ucast_pkts': '1072018',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjItT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1131627148',
                                                                       'in_ucast_pkts': '1072018',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1131627148',
                                                                                                'in_ucast_pkts': '1072018',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '444',
                                                                                                'out_octets': '1254326334',
                                                                                                'out_ucast_pkts': '995741',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '444',
                                                                       'out_octets': '1254326334',
                                                                       'out_ucast_pkts': '995741',
                                                                       'poller_hash': 'QURUTjI0MzE0MUFBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13216',
                                                                                                                        'fiber_length': 14505,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MUFBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x22-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDFBQQ==',
                                         'serial_number': 'ADTN243141AA',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13216'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-4.1.8.0.eth.1.phy',
                                                                       'name': 'Maurice-x5-ONU8_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NS1PTlU4X0VUSF8x'},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjI0MzE0NjZFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0NjZFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x5-ONU8',
                                         'offline_reason': 'OFFLINE_REASON_POWER_OFF',
                                         'offline_reason_detail': 'A local power '
                                                                  'outage has occurred '
                                                                  'for ONU "onu '
                                                                  '4.1.8"(ADTN2431466E).',
                                         'offline_time': '2025-07-20T18:25:17Z',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDY2RQ==',
                                         'serial_number': 'ADTN2431466E',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-27.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '356',
                                                                       'in_octets': '2465302042',
                                                                       'in_ucast_pkts': '2433687',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x28-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '2049310938',
                                                                       'out_ucast_pkts': '2813611',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjgtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '2049310938',
                                                                       'in_ucast_pkts': '2813611',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '2049310938',
                                                                                                'in_ucast_pkts': '2813611',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '356',
                                                                                                'out_octets': '2465302042',
                                                                                                'out_ucast_pkts': '2433687',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yOA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '356',
                                                                       'out_octets': '2465302042',
                                                                       'out_ucast_pkts': '2433687',
                                                                       'poller_hash': 'QURUTjI0MzE0MjYxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10242',
                                                                                                                        'fiber_length': 12952,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yOA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MjYxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x28-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDI2MQ==',
                                         'serial_number': 'ADTN24314261',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10242'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '416',
                                                                       'in_octets': '370352738',
                                                                       'in_ucast_pkts': '2517837',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '10119353774',
                                                                       'out_ucast_pkts': '7166713',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VMTFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '10119353774',
                                                                       'in_ucast_pkts': '7166713',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '10119353774',
                                                                                                'in_ucast_pkts': '7166713',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '416',
                                                                                                'out_octets': '370352738',
                                                                                                'out_ucast_pkts': '2517837',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '416',
                                                                       'out_octets': '370352738',
                                                                       'out_ucast_pkts': '2517837',
                                                                       'poller_hash': 'QURUTjI0MzE0MTZFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '12083',
                                                                                                                        'fiber_length': 10093,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -11.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.9}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MTZFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU11',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDE2RQ==',
                                         'serial_number': 'ADTN2431416E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '12083'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-21.1.2.0.eth.1.phy',
                                                                       'in_nucast_pkts': '86',
                                                                       'in_octets': '23044891',
                                                                       'in_ucast_pkts': '133642',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x22-ONU2_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '887837606',
                                                                       'out_ucast_pkts': '609426',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjItT05VMl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '887837606',
                                                                       'in_ucast_pkts': '609426',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '887837606',
                                                                                                'in_ucast_pkts': '609426',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '86',
                                                                                                'out_octets': '23044891',
                                                                                                'out_ucast_pkts': '133642',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '86',
                                                                       'out_octets': '23044891',
                                                                       'out_ucast_pkts': '133642',
                                                                       'poller_hash': 'QURUTjI0MzE0Mjk1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '2167',
                                                                                                                        'fiber_length': 14572,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0Mjk1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x22-ONU2',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDI5NQ==',
                                         'serial_number': 'ADTN24314295',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '2167'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-19.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '461',
                                                                       'in_octets': '4873684',
                                                                       'in_ucast_pkts': '39084',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x20-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '73121332',
                                                                       'out_ucast_pkts': '62546',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjAtT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '73121332',
                                                                       'in_ucast_pkts': '62546',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '73121332',
                                                                                                'in_ucast_pkts': '62546',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '461',
                                                                                                'out_octets': '4873684',
                                                                                                'out_ucast_pkts': '39084',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yMA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '461',
                                                                       'out_octets': '4873684',
                                                                       'out_ucast_pkts': '39084',
                                                                       'poller_hash': 'QURUTjI0MzE0MTlCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13677',
                                                                                                                        'fiber_length': 14803,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yMA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MTlCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x20-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDE5Qg==',
                                         'serial_number': 'ADTN2431419B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13677'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-18.1.0.0.eth.1.phy',
                                                                       'in_nucast_pkts': '446',
                                                                       'in_octets': '121809105',
                                                                       'in_ucast_pkts': '351198',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x19-ONU0_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '1022799880',
                                                                       'out_ucast_pkts': '852967',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTktT05VMF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '1022799880',
                                                                       'in_ucast_pkts': '852967',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '1022799880',
                                                                                                'in_ucast_pkts': '852967',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '446',
                                                                                                'out_octets': '121809105',
                                                                                                'out_ucast_pkts': '351198',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '446',
                                                                       'out_octets': '121809105',
                                                                       'out_ucast_pkts': '351198',
                                                                       'poller_hash': 'QURUTjI0MzYzREQ3IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13219',
                                                                                                                        'fiber_length': 13158,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.4,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzREQ3IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x19-ONU0',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0RENw==',
                                         'serial_number': 'ADTN24363DD7',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13219'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-31.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '364',
                                                                       'in_octets': '237885814',
                                                                       'in_ucast_pkts': '2586699',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x32-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '10960583868',
                                                                       'out_ucast_pkts': '7327483',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzItT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '10960583868',
                                                                       'in_ucast_pkts': '7327483',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '10960583868',
                                                                                                'in_ucast_pkts': '7327483',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '364',
                                                                                                'out_octets': '237885814',
                                                                                                'out_ucast_pkts': '2586699',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zMg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '364',
                                                                       'out_octets': '237885814',
                                                                       'out_ucast_pkts': '2586699',
                                                                       'poller_hash': 'QURUTjI0MzYzRDhGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10735',
                                                                                                                        'fiber_length': 16282,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zMg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRDhGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x32-ONU3',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0Q4Rg==',
                                         'serial_number': 'ADTN24363D8F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10735'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.13.0.eth.1.phy',
                                                                       'in_nucast_pkts': '486',
                                                                       'in_octets': '90624732',
                                                                       'in_ucast_pkts': '959739',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU13_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '5417896669',
                                                                       'out_ucast_pkts': '3938892',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VMTNfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '5417896669',
                                                                       'in_ucast_pkts': '3938892',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '5417896669',
                                                                                                'in_ucast_pkts': '3938892',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '486',
                                                                                                'out_octets': '90624732',
                                                                                                'out_ucast_pkts': '959739',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '486',
                                                                       'out_octets': '90624732',
                                                                       'out_ucast_pkts': '959739',
                                                                       'poller_hash': 'QURUTjI0MzE0Nzc5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14147',
                                                                                                                        'fiber_length': 15554,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -14.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0Nzc5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU13',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDc3OQ==',
                                         'serial_number': 'ADTN24314779',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14147'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-18.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '481',
                                                                       'in_octets': '121064401',
                                                                       'in_ucast_pkts': '1607299',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x19-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '4878114535',
                                                                       'out_ucast_pkts': '3323816',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTktT05VMV9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '4878114535',
                                                                       'in_ucast_pkts': '3323816',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '4878114535',
                                                                                                'in_ucast_pkts': '3323816',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '481',
                                                                                                'out_octets': '121064401',
                                                                                                'out_ucast_pkts': '1607299',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '481',
                                                                       'out_octets': '121064401',
                                                                       'out_ucast_pkts': '1607299',
                                                                       'poller_hash': 'QURUTjI0MzE0MjJFIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '14138',
                                                                                                                        'fiber_length': 13053,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.8}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MjJFIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x19-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDIyRQ==',
                                         'serial_number': 'ADTN2431422E',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '14138'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-41.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '362',
                                                                       'in_octets': '201705501',
                                                                       'in_ucast_pkts': '258577',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x42-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '123740882',
                                                                       'out_ucast_pkts': '162937',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDItT05VM19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '123740882',
                                                                       'in_ucast_pkts': '162937',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '123740882',
                                                                                                'in_ucast_pkts': '162937',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '362',
                                                                                                'out_octets': '201705501',
                                                                                                'out_ucast_pkts': '258577',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80Mg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '362',
                                                                       'out_octets': '201705501',
                                                                       'out_ucast_pkts': '258577',
                                                                       'poller_hash': 'QURUTjI0MzYzRTA1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10401',
                                                                                                                        'fiber_length': 23187,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80Mg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -20.2,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRTA1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x42-ONU3',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0UwNQ==',
                                         'serial_number': 'ADTN24363E05',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10401'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-43.1.6.0.eth.1.phy',
                                                                       'in_nucast_pkts': '220',
                                                                       'in_octets': '131014610',
                                                                       'in_ucast_pkts': '136231',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x44-ONU6_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '148345529',
                                                                       'out_ucast_pkts': '138301',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDQtT05VNl9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '148345529',
                                                                       'in_ucast_pkts': '138301',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '148345529',
                                                                                                'in_ucast_pkts': '138301',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '220',
                                                                                                'out_octets': '131014610',
                                                                                                'out_ucast_pkts': '136231',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80NA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '220',
                                                                       'out_octets': '131014610',
                                                                       'out_ucast_pkts': '136231',
                                                                       'poller_hash': 'QURUTjI0MzYzRDcxIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '6181',
                                                                                                                        'fiber_length': 23123,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80NA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -22.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRDcxIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x44-ONU6',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0Q3MQ==',
                                         'serial_number': 'ADTN24363D71',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '6181'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-38.1.8.0.eth.1.phy',
                                                                       'in_nucast_pkts': '232',
                                                                       'in_octets': '185068829',
                                                                       'in_ucast_pkts': '1034480',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x39-ONU8_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '3948536680',
                                                                       'out_ucast_pkts': '2762039',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MzktT05VOF9FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '3948536680',
                                                                       'in_ucast_pkts': '2762039',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '3948536680',
                                                                                                'in_ucast_pkts': '2762039',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '232',
                                                                                                'out_octets': '185068829',
                                                                                                'out_ucast_pkts': '1034480',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8zOQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '232',
                                                                       'out_octets': '185068829',
                                                                       'out_ucast_pkts': '1034480',
                                                                       'poller_hash': 'QURUTjI0MzE0MTg5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '6786',
                                                                                                                        'fiber_length': 21098,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8zOQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -18.9,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MTg5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x39-ONU8',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDE4OQ==',
                                         'serial_number': 'ADTN24314189',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '6786'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-15.1.12.0.eth.1.phy',
                                                                       'in_nucast_pkts': '399',
                                                                       'in_octets': '340370588',
                                                                       'in_ucast_pkts': '2029064',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x16-ONU12_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '6082153947',
                                                                       'out_ucast_pkts': '4547465',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTYtT05VMTJfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '6082153947',
                                                                       'in_ucast_pkts': '4547465',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '6082153947',
                                                                                                'in_ucast_pkts': '4547465',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '399',
                                                                                                'out_octets': '340370588',
                                                                                                'out_ucast_pkts': '2029064',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNg=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '399',
                                                                       'out_octets': '340370588',
                                                                       'out_ucast_pkts': '2029064',
                                                                       'poller_hash': 'QURUTjI0MzYzRTIyIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '11754',
                                                                                                                        'fiber_length': 10578,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNg==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -12.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzRTIyIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x16-ONU12',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0UyMg==',
                                         'serial_number': 'ADTN24363E22',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '11754'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-14.1.4.0.eth.1.phy',
                                                                       'in_nucast_pkts': '449',
                                                                       'in_octets': '20061858',
                                                                       'in_ucast_pkts': '92108',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x15-ONU4_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '56360098',
                                                                       'out_ucast_pkts': '91493',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTUtT05VNF9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '56360098',
                                                                       'in_ucast_pkts': '91493',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '56360098',
                                                                                                'in_ucast_pkts': '91493',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '449',
                                                                                                'out_octets': '20061858',
                                                                                                'out_ucast_pkts': '92108',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNQ=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '449',
                                                                       'out_octets': '20061858',
                                                                       'out_ucast_pkts': '92108',
                                                                       'poller_hash': 'QURUTjI0MzE0MjM5IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13213',
                                                                                                                        'fiber_length': 13780,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNQ==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.8,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzE0MjM5IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x15-ONU4',
                                         'poller_hash': 'b250IEFEVE4yNDMxNDIzOQ==',
                                         'serial_number': 'ADTN24314239',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13213'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-26.1.1.0.eth.1.phy',
                                                                       'in_nucast_pkts': '382',
                                                                       'in_octets': '30309881',
                                                                       'in_ucast_pkts': '273373',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x27-ONU1_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '598818952',
                                                                       'out_ucast_pkts': '413672',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjctT05VMV9FVEhfMQ==',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '598818952',
                                                                       'in_ucast_pkts': '413672',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '598818952',
                                                                                                'in_ucast_pkts': '413672',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '382',
                                                                                                'out_octets': '30309881',
                                                                                                'out_ucast_pkts': '273373',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '382',
                                                                       'out_octets': '30309881',
                                                                       'out_ucast_pkts': '273373',
                                                                       'poller_hash': 'QURUTjI0MzYzQkM2IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10735',
                                                                                                                        'fiber_length': 15222,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.0,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.7}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjI0MzYzQkM2IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x27-ONU1',
                                         'poller_hash': 'b250IEFEVE4yNDM2M0JDNg==',
                                         'serial_number': 'ADTN24363BC6',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10735'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-43.1.7.0.eth.1.phy',
                                                                       'in_nucast_pkts': '357',
                                                                       'in_octets': '207164952',
                                                                       'in_ucast_pkts': '2834583',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x44-ONU7_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '11383810480',
                                                                       'out_ucast_pkts': '7582616',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDQtT05VN19FVEhfMQ==',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '11383810480',
                                                                       'in_ucast_pkts': '7582616',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '11383810480',
                                                                                                'in_ucast_pkts': '7582616',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '357',
                                                                                                'out_octets': '207164952',
                                                                                                'out_ucast_pkts': '2834583',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC80NA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '357',
                                                                       'out_octets': '207164952',
                                                                       'out_ucast_pkts': '2834583',
                                                                       'poller_hash': 'QURUTjIzMTQ1MTVEIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10261',
                                                                                                                        'fiber_length': 23316,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC80NA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -21.6,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ1MTVEIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x44-ONU7',
                                         'poller_hash': 'b250IEFEVE4yMzE0NTE1RA==',
                                         'serial_number': 'ADTN2314515D',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10261'},
                                 {       'interfaces': [       {       'id': 'onu-subscr-if-44.1.0.0.eth.1.phy',
                                                                       'name': 'Maurice-x45-ONU0_ETH_1',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14NDUtT05VMF9FVEhfMQ=='},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'poller_hash': 'QURUTjIzMTI0OURGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0OURGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON'}]}],
                                         'manufacturer': 'Adtran',
                                         'name': 'Maurice-x45-ONU0',
                                         'offline_reason': 'OFFLINE_REASON_LOSS_OF_SIGNAL',
                                         'offline_reason_detail': 'The ONU "onu '
                                                                  '44.1.0"(ADTN231249DF) '
                                                                  'on channel '
                                                                  'termination '
                                                                  '"channel-termination '
                                                                  '0/45.xgs-pon" is in '
                                                                  'LOS.',
                                         'offline_time': '2025-09-26T09:57:07Z',
                                         'poller_hash': 'b250IEFEVE4yMzEyNDlERg==',
                                         'serial_number': 'ADTN231249DF',
                                         'status': 'ELEMENT_STATUS_OFFLINE'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-16.1.11.0.eth.1.phy',
                                                                       'in_nucast_pkts': '355',
                                                                       'in_octets': '614206',
                                                                       'in_ucast_pkts': '2446',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x17-ONU11_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '180776',
                                                                       'out_ucast_pkts': '1782',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MTctT05VMTFfRVRIXzE=',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '180776',
                                                                       'in_ucast_pkts': '1782',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '180776',
                                                                                                'in_ucast_pkts': '1782',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '355',
                                                                                                'out_octets': '614206',
                                                                                                'out_ucast_pkts': '2446',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8xNw=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '355',
                                                                       'out_octets': '614206',
                                                                       'out_ucast_pkts': '2446',
                                                                       'poller_hash': 'QURUTjIzMTQ0RTdCIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '10228',
                                                                                                                        'fiber_length': 10732,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8xNw==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -13.7,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.2}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RTdCIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x17-ONU11',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEU3Qg==',
                                         'serial_number': 'ADTN23144E7B',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '10228'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-23.1.14.0.eth.1.phy',
                                                                       'in_nucast_pkts': '487',
                                                                       'in_octets': '854654976',
                                                                       'in_ucast_pkts': '3209662',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x24-ONU14_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '5691213118',
                                                                       'out_ucast_pkts': '5581369',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MjQtT05VMTRfRVRIXzE=',
                                                                       'speed': '2500000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '5691213118',
                                                                       'in_ucast_pkts': '5581369',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '5691213118',
                                                                                                'in_ucast_pkts': '5581369',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '487',
                                                                                                'out_octets': '854654976',
                                                                                                'out_ucast_pkts': '3209662',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8yNA=='}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '487',
                                                                       'out_octets': '854654976',
                                                                       'out_ucast_pkts': '3209662',
                                                                       'poller_hash': 'QURUTjIzMTQ0RTdBIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13987',
                                                                                                                        'fiber_length': 16045,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8yNA==',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -15.5,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.5}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RTdBIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x24-ONU14',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEU3QQ==',
                                         'serial_number': 'ADTN23144E7A',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13987'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-0.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '833',
                                                                       'in_octets': '90243343',
                                                                       'in_ucast_pkts': '319978',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 9216,
                                                                       'name': 'Maurice-x1-ONU3_ETH_1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '2',
                                                                       'out_octets': '196904150',
                                                                       'out_ucast_pkts': '369594',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14MS1PTlUzX0VUSF8x',
                                                                       'speed': '1000000000',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '2',
                                                                       'in_octets': '196904150',
                                                                       'in_ucast_pkts': '369594',
                                                                       'links': [       {       'in_nucast_pkts': '2',
                                                                                                'in_octets': '196904150',
                                                                                                'in_ucast_pkts': '369594',
                                                                                                'oper_up': True,
                                                                                                'out_nucast_pkts': '833',
                                                                                                'out_octets': '90243343',
                                                                                                'out_ucast_pkts': '319978',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8x'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': True,
                                                                       'out_nucast_pkts': '833',
                                                                       'out_octets': '90243343',
                                                                       'out_ucast_pkts': '319978',
                                                                       'poller_hash': 'QURUTjIzMTQ0RTk1IHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '13986',
                                                                                                                        'fiber_length': 10921,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8x',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -19.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.6}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTQ0RTk1IHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x1-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzE0NEU5NQ==',
                                         'serial_number': 'ADTN23144E95',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '25_2-14-b',
                                         'uptime': '13986'},
                                 {       'interfaces': [       {       'admin_up': True,
                                                                       'id': 'onu-subscr-if-2.1.3.0.eth.1.phy',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'manufacturer': 'Adtran',
                                                                       'mtu': 0,
                                                                       'name': 'Maurice-x3-ONU3_ETH_1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgTWF1cmljZS14My1PTlUzX0VUSF8x',
                                                                       'type': 6},
                                                               {       'admin_up': True,
                                                                       'id': 'pon1',
                                                                       'in_nucast_pkts': '0',
                                                                       'in_octets': '0',
                                                                       'in_ucast_pkts': '0',
                                                                       'links': [       {       'in_nucast_pkts': '0',
                                                                                                'in_octets': '0',
                                                                                                'in_ucast_pkts': '0',
                                                                                                'oper_up': False,
                                                                                                'out_nucast_pkts': '0',
                                                                                                'out_octets': '0',
                                                                                                'out_ucast_pkts': '0',
                                                                                                'poller_hash': 'TEJBRFROMjQyMEFBMjM3IGludGYgMC8z'}],
                                                                       'name': 'pon1',
                                                                       'oper_up': False,
                                                                       'out_nucast_pkts': '0',
                                                                       'out_octets': '0',
                                                                       'out_ucast_pkts': '0',
                                                                       'poller_hash': 'QURUTjIzMTI0QTJGIHBlZXJpbnRm',
                                                                       'pons': [       {       'description': 'pon1',
                                                                                               'id': 'pon1',
                                                                                               'links': [       {       'connected_time': '1534',
                                                                                                                        'fiber_length': 12844,
                                                                                                                        'poller_hash': 'TEJBRFROMjQyMEFBMjM3IG9sdHBvbiB4Z3Nwb24gMC8z',
                                                                                                                        'rx_link_rate': '10000000000',
                                                                                                                        'rx_power': -17.1,
                                                                                                                        'tx_link_rate': '10000000000',
                                                                                                                        'tx_power': 5.3}],
                                                                                               'max_distance': 20000,
                                                                                               'maximum_loss': -9.0,
                                                                                               'minimum_loss': -28.0,
                                                                                               'name': 'pon1',
                                                                                               'poller_hash': 'QURUTjIzMTI0QTJGIHBlZXJwb24=',
                                                                                               'pon_technology': 'XGS_PON',
                                                                                               'rx_link_rate': '10000000000',
                                                                                               'split_ratio': 128,
                                                                                               'tx_link_rate': '10000000000'}]}],
                                         'manufacturer': 'Adtran',
                                         'model': 'SDX 631q',
                                         'name': 'Maurice-x3-ONU3',
                                         'poller_hash': 'b250IEFEVE4yMzEyNEEyRg==',
                                         'serial_number': 'ADTN23124A2F',
                                         'status': 'ELEMENT_STATUS_ONLINE',
                                         'sw_version': '12_1-31',
                                         'uptime': '1534'}]
        }
        id_field = 'mac_address' if not obj.stations else ('mac_address' if hasattr(obj.stations[0], 'mac_address') and getattr(obj.stations[0], 'mac_address') else ('imsi' if hasattr(obj.stations[0], 'imsi') else 'serial'))
        cur_info = {x: getattr(obj, x, None) for (x, data_type) in self.apdd.get_metric_fields('info', 'ap', obj.model) + [('vendor', 'str'), ('model', 'str'), ('sw_version', 'str')]}
        if 'rf_freq' in cur_info and cur_info.get('rf_freq') and (cur_info['rf_freq'] <= 0 or int.bit_length(cur_info['rf_freq']) >= 32):
            cur_info.pop('rf_freq')
        cur_stats = {x: getattr(obj, x, None) for (x, data_type) in self.apdd.get_metric_fields('stats', 'ap', obj.model)}
        cur_sta_stats = {getattr(sta, id_field): {x: getattr(sta, x, None) for (x, data_type) in sorted(self.apdd.get_metric_fields('stats', 'cpe', obj.model)) + [(id_field, 'str'), ('ip_address', 'str'), ('name', 'str'), ('dev_macs', 'str')]} for sta in sorted(obj.stations)}
        cur_ne_msg = getattr(obj, 'network_element_update_msg', None)
        cur_ne_msg = cur_ne_msg.pb if cur_ne_msg else None
        self.assertEqual(cur_info, exp_info)
        self.assertEqual(cur_stats, exp_stats)
        self.assertEqual(cur_sta_stats, exp_sta_stats)
        self.assertTrue(self.compare_pb_dict(cur_ne_msg, exp_ne_msg))
